/*
 *      callback.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXCCALLBACKS_H
#define DXCCALLBACKS_H

#include <sigc++/sigc++.h>
#include <purplemm.h>

class dxcCallbacks : public PURPLE::Callbacks
{
public:
    dxcCallbacks(const std::string& path);
    virtual ~dxcCallbacks();

    sigc::signal<void, std::string> signalError;
    sigc::signal<void, PURPLE::Account> signalAccountAdded;
    sigc::signal<void, std::string, std::string> signalAccountRemoved;
    sigc::signal<void, PURPLE::Account, std::string> signalAccountConnecting;
    sigc::signal<void, PURPLE::Account> signalAccountConnected;
    sigc::signal<void, PURPLE::Account, std::string> signalAccountDisconnected;
    sigc::signal<void, PURPLE::Account> signalAccountStatusChanged;
    sigc::signal<void, PURPLE::Account> signalAccountSignedOn;
    sigc::signal<void, PURPLE::Conversation*> signalNewIM;
    sigc::signal<void, PURPLE::Conversation*> signalNewChat;
    sigc::signal<void, PURPLE::Conversation*> signalCloseConvTab;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string, bool, std::string> signalNewMyMessage;
    sigc::signal<void, PURPLE::Conversation*, std::string> signalMessageRecived;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string, std::string, bool, bool> signalNewIMMessage;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string, std::string, bool, bool, bool> signalNewChatMessage;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string> signalNewConvSystemMessage;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string> signalNewConvErrorMessage;
    sigc::signal<void, PURPLE::RoomList*> signalRoomlistCreated;
    sigc::signal<void, PURPLE::RoomList*, std::vector<std::string> > signalRoomlistColumns;
    sigc::signal<void, PURPLE::RoomList*, std::vector<std::string> > signalRoomlistAddRoom;
    sigc::signal<void, PURPLE::RoomList*> signalRoomlistFinished;
    sigc::signal<void, PURPLE::ActionRequest*> signalActionRequest;
    sigc::signal<void, PURPLE::ChoiceRequest*> signalChoiceRequest;
    sigc::signal<void, PURPLE::FieldsRequest*> signalFieldsRequest;
    sigc::signal<void, PURPLE::FileRequest*> signalFileRequest;
    sigc::signal<void, PURPLE::InputRequest*> signalInputRequest;
    sigc::signal<void, PURPLE::Conversation*, std::string> signalChatBuddyJoined;
    sigc::signal<void, PURPLE::Conversation*, std::string> signalChatBuddyUpdated;
    sigc::signal<void, PURPLE::Conversation*, std::string> signalChatBuddyLeft;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string> signalChatBuddyRenamed;
    sigc::signal<void, PURPLE::Conversation*, std::string, std::string> signalChatTopic;
    sigc::signal<void, PURPLE::Conversation*, bool> signalBuddyTyping;
    sigc::signal<void, PURPLE::Buddy> signalBuddySignedOn;
    sigc::signal<void, PURPLE::Buddy> signalBuddySignedOff;
    sigc::signal<void, PURPLE::Buddy, PurpleStatusPrimitive> signalBuddyStatusChanged;
    sigc::signal<void, PURPLE::Buddy> signalBuddyIconChanged;
    sigc::signal<void, PURPLE::Buddy> signalBuddyAdded;
    sigc::signal<void, PURPLE::Account, std::string> signalBuddyRemoved;
    sigc::signal<void, PURPLE::Chat> signalChatAdded;
    sigc::signal<void, PURPLE::Account, std::string> signalChatRemoved;
    sigc::signal<void, PurpleNotifyMsgType, std::string, std::string> signalNotifyMessage;
    sigc::signal<void, std::string, std::string> signalNotifyEmail;
    sigc::signal<void, PURPLE::Account, std::string, std::string> signalNotifyUserinfo;
    sigc::signal<void, std::string> signalNotifyUri;
    sigc::signal<void, PURPLE::FileTransfer*> signalNewFileTransfer;
    sigc::signal<void, PURPLE::FileTransfer*> signalFileTransferFinished;
    sigc::signal<void, PURPLE::FileTransfer*> signalFileTransferStarted;
    sigc::signal<void, PURPLE::FileTransfer*> signalFileTransferProgress;
    sigc::signal<void, PURPLE::FileTransfer*> signalFileTransferCanceled;

    PURPLE::Purple *getPurplemm();
private:
    PURPLE::Purple *m_purple;

    /** Allow your application to be notified about errors on library layer */
    void showError(std::string msg);
    /** Allow your application to be notified about libpurple debug messages */
    void showDebug(std::string debug);
    /** Buddy list loaded */
    void buddylistLoaded() {}
    /** New account added */
    void accountAdded(PURPLE::Account account);
    /** Account removed */
    void accountRemoved(std::string account, std::string protocolID);
    /** Account connecting */
    void accountConnecting(PURPLE::Account account, std::string msg);
    /** Acount connected*/
    void accountConnected(PURPLE::Account account);
    /** Acount disconnected*/
    void accountDisconnected(PURPLE::Account account, std::string reason);
    /** Account status changed*/
    void accountStatusChanged(PURPLE::Account account);
    /** Account signed on, so show buddy list */
    void accountSignedOn(PURPLE::Account account);
    /** Buddy signed on */
    void buddySignedOn(PURPLE::Buddy buddy);
    /** Buddy signed off */
    void buddySignedOff(PURPLE::Buddy buddy);
    /** Buddy status changed */
    void buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus);
    /** Buddy icon changed */
    void buddyIconChanged(PURPLE::Buddy buddy);
    /** Buddy added */
    void buddyAdded(PURPLE::Buddy buddy);
    /** Buddy removed */
    void buddyRemoved(PURPLE::Account account, std::string buddy);
    /** Chat added */
    void chatAdded(PURPLE::Chat chat);
    /** Chat removed */
    void chatRemoved(PURPLE::Account account, std::string chat);
    /** New IM created */
    void newIM(PURPLE::Conversation *conv);
    /** New Chat created */
    void newChat(PURPLE::Conversation *conv);
    /** IM closed */
    void closedIM(PURPLE::Conversation *conv);
    /** Chat closed */
    void closedChat(PURPLE::Conversation *conv);
    /** New my message
     * @param conv          The conversation.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param action        Is message in 3rd person (/me)?
     * @param messageID     message ID, usefull for XEP-0184
     */
    void newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID);
    /** Message recived, usefull for XEP-0184
     * @param conv          The conversation.
     * @param messageID     message ID, usefull for XEP-0184
     */
    void messageRecived(PURPLE::Conversation *conv, std::string messageID);
    /** New IM message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action);
    /** New Chat message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action);
    /** New system mesage for conversation
     *
     * @param conv  The conversation
     * @param msg  The message text
     * @param time  The time the message was sent
     */
    void newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time);
    /** New error mesage for conversation
     *
     * @param conv      conversation
     * @param msg       message text
     * @param time      time the message was sent
     */
    void newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time);
    /** ChatBuddy joined chat */
    void chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy);
    /** ChatBuddy updated */
    void chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy);
    /** ChatBuddy left chat */
    void chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy);
    /** ChatBuddy renamed
     * @param who       buddy which was renamed
     * @param nbuddy    new buddy established by rename
     */
    void chatBuddyRenamed(PURPLE::Conversation *conv, std::string who, PURPLE::ChatBuddy nbuddy);
    /** Chat topic changed
     * @param who  topic changer
     * @param topic  new topic
     */
    void chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic);
    /** Buddy typing on IM
     * @param typing =true typing, =false stoped typing
     */
    void buddyTyping(PURPLE::Conversation *conv, bool typing);
    /** New FileTransfer */
    void newFileTransfer(PURPLE::FileTransfer *ft);
    /** FileTransfer finished */
    void fileTransferFinished(PURPLE::FileTransfer *ft);
    /** FileTransfer started */
    void fileTransferStarted(PURPLE::FileTransfer *ft);
    /** FileTransfer progress */
    void fileTransferProgress(PURPLE::FileTransfer *ft, double percent);
    /** FileTransfer canceled */
    void fileTransferCanceled(PURPLE::FileTransfer *ft);
    /** A new list was created */
    void roomlistCreated(PURPLE::RoomList *rlist);
    /** Sets the columns of RoomList */
    void roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns);
    /** Add a room to the list */
    void roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields);
    /** Listing RoomList finished */
    void roomlistFinished(PURPLE::RoomList *rlist);
    /** Notification message to the user
     * @param type  notification type
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
    */
    void notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary);
    /** Email notification to the user
     * @param subject  subject of the email
     * @param from  from address
     * @param to  estination address
     * @param url  URL where the message can be read
    */
    void notifyEmail(std::string subject, std::string from, std::string to, std::string url);
    /** Notification with formatted text
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
     * @param text  formatted text
    */
    void notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated);
    /** Notification information about buddy
     * @param account   requested account
     * @param buddy     Buddy which about
     * @param info      vector with UserInfo values
    */
    void notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info);
    /** Opens a URI or somehow presents it to the user
     * @param uri    The URI to display or go to.
    */
    void notifyURI(std::string uri);
    /** Play sound for given event */
    void playSound(PurpleSoundEventID event){}
    /** New InputRequest */
    void newInputRequest(PURPLE::InputRequest *request);
    /** New ChoiceRequest */
    void newChoiceRequest(PURPLE::ChoiceRequest *request);
    /** New ActionRequest */
    void newActionRequest(PURPLE::ActionRequest *request);
    /** New FileRequest */
    void newFileRequest(PURPLE::FileRequest *request);
    /** New FieldsRequest */
    void newFieldsRequest(PURPLE::FieldsRequest *request);
    /** Close given request (ex. dialog) */
    void closeRequest(PURPLE::BasicRequest *request){}
};

#endif  /* DXCCALLBACKS_H */

