/*
 *      dxim.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXIM_H
#define DXIM_H

#ifdef __GNUC__
#define _attribute(x) __attribute__(x)
#else
#define _attribute(x)
#endif

#include <cppconsui/CoreManager.h>
#include "callback/callback.h"
#include "ui/filetransferdialog.h"
#include <vector>

#define DXIM (dxim::Instance())

class dxim
: public CppConsUI::InputProcessor
{
public:
    enum ScreenArea {
        ROSTER_AREA,
        CHAT_AREA,
        FOOTER_AREA,
        LOG_AREA,
        WHOLE_AREA,
        AREAS_NUM
    };

    static dxim *Instance();
    // InputProcessor
    virtual bool ProcessInput(const TermKeyKey& key);
    int Run(const std::string& path, bool offline);
    void Quit();
    // returns a position and size of a selected area
    CppConsUI::Rect GetScreenArea(ScreenArea area);
    CppConsUI::Rect GetScreenAreaCentered(ScreenArea area);
    bool LoadColorSchemeConfig();
    bool LoadKeyConfig();
    bool GetExpandedConversations() const { return m_convs_expanded; }
    PURPLE::Purple *GetPurplemm();
    dxcCallbacks *GetCallback() { return m_cb; }
    void Resize();
private:
    dxim();
    dxim(const dxim&);
    dxim& operator=(const dxim&);
    virtual ~dxim() {}

    dxcCallbacks *m_cb;
    CppConsUI::CoreManager *m_mngr;
    sigc::connection m_resize_conn;
    sigc::connection m_top_window_change_conn;
    CppConsUI::Rect m_areas[AREAS_NUM];
    static const char *m_named_colors[];
    // flag to indicate if the conversation full-screen mode is activated
    bool m_convs_expanded;
    FileTransferDialog *m_ftdialog;

    void OnScreenResized();
    void OnTopWindowChanged();
    void LoadDefaultColorSchemeConfig();
    bool SaveColorSchemeConfig();
    char *ColorToString(int color);
    bool StringToColor(const char *str, int *color);
    char *ColorAttributesToString(int attrs);
    bool StringToColorAttributes(const char *str, int *attrs);
    void LoadDefaultKeyConfig();
    bool SaveKeyConfig();
    void DeclareBindables();
    void ActionRosterActive();
    void ActionTabitemActive();
    void ActionStatusMenu();
    void ActionMainMenu();
    void ActionRosterToggleOffline();
    void ActionTabbookExpand();
    void ActionTabitemPrev();
    void ActionTabitemNext();
    void ActionFocusTabitem(int i);
    void ActionCloseCurrentTabitem();
    void ActionNewImMenu();
    void ActionJoinChatMenu();
    //purplemm slots
    void OnUserInfo(PURPLE::Account account, std::string buddy, std::string text);
    void OnPurpleError(std::string error);
    void OnNotifyMessage(PurpleNotifyMsgType type, std::string title, std::string text);
    void OnNotifyEmail(std::string text, std::string url);
    void OnNotifyUri(std::string uri);
    void OnNewInputRequest(PURPLE::InputRequest *request);
    void OnNewActionRequest(PURPLE::ActionRequest *request);
    void OnNewChoiceRequest(PURPLE::ChoiceRequest *request);
    void OnNewFieldsRequest(PURPLE::FieldsRequest *request);
    void OnNewFileRequest(PURPLE::FileRequest *request);
    void OnNewFileTransfer(PURPLE::FileTransfer *ft);
};

#endif  /* DXIM_H */

