/*
 *      dxutils.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXUTILS_H
#define DXUTILS_H

#include <purplemm.h>

#define FOREACH(t, v, it) \
    for(t::iterator it = v.begin(); it < v.end(); it++)

class dxutils {
public:
    dxutils();
    virtual ~dxutils();

    static dxutils &instance();
    static std::string getPresenceString(PurpleStatusPrimitive presence);
    static std::string getPresenceScheme(std::string base, PurpleStatusPrimitive presence);
    static std::string getChatRoleString(PurpleConvChatBuddyFlags role);
    static guint hash(const std::string &str);
    static time_t now();
    static std::string time(const char *format, time_t time=0);
    static std::vector<std::string> stringSplit(const std::string &text, char splitter);
    static std::string stringSection(const std::string &text, char splitter, int section, bool toEnd=false);
    static bool stringContains(std::string text, std::string sub, bool ignoreCase=true);
    /**
     *
     * @param label         string to normalize
     * @param normalize     char for remove, usually '_'
     * @return              Normalized std::string
     */
    static std::string normalizeLabelString(std::string label, char normalize);
    static std::string getFileSize(size_t size);
    static std::string getRemaining(size_t size, size_t speed);
    static std::string typingToString(std::string nick, bool typing);
    static std::string sformat(const char *fmt, ...);
    bool haveNotify();
    void showNotify(std::string text);
    static bool isRightLogFile(const std::string &path, const std::string &name);
private:
    bool m_haveNotify;
};

#endif  /* DXUTILS_H */

