/*
 *      preferences.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "preferences.h"
#include "dxim.h"
#include "dxutils.h"

Preferences::Preferences()
{
    //General settings
    m_useAway = true;
    m_awayTimeout = 5;
    m_logging = true;
#ifdef _WIN32
    m_logPath = g_build_filename(g_getenv("AppData"), "dxim", "logs", NULL);
#else
    m_logPath = g_build_filename(g_get_home_dir(), ".dxim", "logs", NULL);
#endif
    m_logLoadCurrent = false;
    m_receiptsMark = true;
    //Notification settings
    m_notify = false;
    m_notifyAccountConnect = false;
    m_notifyAccountDisconnect = false;
    m_notifyBuddyConnect = false;
    m_notifyBuddyDisconnect = false;
    m_notifyMessage = false;
    m_notifyFiletransfer = false;
    //Other settings
    m_hideOffline = true;
    m_showStatusmsg = true;
    m_showRosterTree = true;
    m_rosterWidth = 35;
    m_logHeight = 20;

    DXIM->GetPurplemm()->addNonePref(CONF_PREFIX);
    DXIM->GetPurplemm()->addNonePref(CONF_PREFIX "/general");
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/general/useAway", m_useAway);
    DXIM->GetPurplemm()->addIntPref(CONF_PREFIX "/general/awayTimeout", m_awayTimeout);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/general/logging", m_logging);
    DXIM->GetPurplemm()->addStringPref(CONF_PREFIX "/general/logPath", m_logPath);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/general/logLoadCurrent", m_logLoadCurrent);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/general/receiptsMark", m_receiptsMark);
    DXIM->GetPurplemm()->addNonePref(CONF_PREFIX "/notification");
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notify", m_notify);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notifyAccountConnect", m_notifyAccountConnect);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notifyAccountDisconnect", m_notifyAccountDisconnect);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notifyBuddyConnect", m_notifyBuddyConnect);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notifyMessage", m_notifyMessage);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/notification/notifyFiletransfer", m_notifyFiletransfer);
    DXIM->GetPurplemm()->addNonePref(CONF_PREFIX "/other");
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/other/hideOffline", m_hideOffline);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/other/showStatusmsg", m_showStatusmsg);
    DXIM->GetPurplemm()->addBoolPref(CONF_PREFIX "/other/showRosterTree", m_showRosterTree);
    DXIM->GetPurplemm()->addIntPref(CONF_PREFIX "/other/rosterWidth", m_rosterWidth);
    DXIM->GetPurplemm()->addIntPref(CONF_PREFIX "/other/logHeight", m_logHeight);
}

Preferences &Preferences::instance()
{
    static Preferences p;
    return p;
}

void Preferences::load()
{
    //General settings
    m_useAway = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/general/useAway", m_useAway);
    m_awayTimeout = DXIM->GetPurplemm()->getIntPref(CONF_PREFIX "/general/awayTimeout", m_awayTimeout);
    m_logging = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/general/logging", m_logging);
    m_logPath = DXIM->GetPurplemm()->getStringPref(CONF_PREFIX "/general/logPath", m_logPath);
    m_logLoadCurrent = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/general/logLoadCurrent", m_logLoadCurrent);
    m_receiptsMark = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/general/receiptsMark", m_receiptsMark);
    //Notification settings
    m_notify = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notify", m_notify);
    m_notifyAccountConnect = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notifyAccountConnect", m_notifyAccountConnect);
    m_notifyAccountDisconnect = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notifyAccountDisconnect", m_notifyAccountDisconnect);
    m_notifyBuddyConnect = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notifyBuddyConnect", m_notifyBuddyConnect);
    m_notifyBuddyDisconnect = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    m_notifyMessage = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notifyMessage", m_notifyMessage);
    m_notifyFiletransfer = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/notification/notifyFiletransfer", m_notifyFiletransfer);
    if(!dxutils::instance().haveNotify())
    {
        m_notify = m_notifyAccountConnect = m_notifyAccountDisconnect = m_notifyBuddyConnect = m_notifyBuddyDisconnect = m_notifyMessage = m_notifyFiletransfer = false;
    }
    //Other settings
    m_hideOffline = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/other/hideOffline", m_hideOffline);
    m_showStatusmsg = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/other/showStatusmsg", m_showStatusmsg);
    m_showRosterTree = DXIM->GetPurplemm()->getBoolPref(CONF_PREFIX "/other/showRosterTree", m_showRosterTree);
    m_rosterWidth = DXIM->GetPurplemm()->getIntPref(CONF_PREFIX "/other/rosterWidth", m_rosterWidth);
    m_logHeight = DXIM->GetPurplemm()->getIntPref(CONF_PREFIX "/other/logHeight", m_logHeight);
}

void Preferences::save()
{
    //General settings
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/general/useAway", m_useAway);
    DXIM->GetPurplemm()->setIntPref(CONF_PREFIX "/general/awayTimeout", m_awayTimeout);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/general/logging", m_logging);
    DXIM->GetPurplemm()->setStringPref(CONF_PREFIX "/general/logPath", m_logPath);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/general/logLoadCurrent", m_logLoadCurrent);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/general/receiptsMark", m_receiptsMark);
    //Notification settings
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notify", m_notify);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notifyAccountConnect", m_notifyAccountConnect);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notifyAccountDisconnect", m_notifyAccountDisconnect);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notifyBuddyConnect", m_notifyBuddyConnect);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notifyMessage", m_notifyMessage);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/notification/notifyFiletransfer", m_notifyFiletransfer);
    //Other settings value
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/other/hideOffline", m_hideOffline);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/other/showStatusmsg", m_showStatusmsg);
    DXIM->GetPurplemm()->setBoolPref(CONF_PREFIX "/other/showRosterTree", m_showRosterTree);
    DXIM->GetPurplemm()->setIntPref(CONF_PREFIX "/other/rosterWidth", m_rosterWidth);
    DXIM->GetPurplemm()->setIntPref(CONF_PREFIX "/other/logHeight", m_logHeight);
}
