/*
 *      accountdialog.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ACCOUNTDIALOG_H
#define ACCOUNTDIALOG_H

#include <cppconsui/Button.h>
#include <cppconsui/CheckBox.h>
#include <cppconsui/ComboBox.h>
#include <cppconsui/InputDialog.h>
#include <cppconsui/TreeView.h>
#include <cppconsui/SplitDialog.h>
#include <cppconsui/MessageDialog.h>
#include <purplemm.h>

class AccountDialog
: public CppConsUI::SplitDialog
{
public:
    /* Constructor for modifying account */
    AccountDialog(PURPLE::Account account);
    /* Constructor for creating new account */
    AccountDialog(PURPLE::Protocol protocol);
    virtual ~AccountDialog() {}

    // FreeWindow
    virtual void OnScreenResized();
protected:
    class BooleanOption
    : public CppConsUI::CheckBox
    {
    friend class AccountDialog;
    public:
        BooleanOption(const char *text, bool value);
        virtual ~BooleanOption();
    protected:
        bool m_value;

        void OnToggle(CppConsUI::CheckBox& activator, bool new_state);
    private:
        BooleanOption(const BooleanOption&);
        BooleanOption& operator=(const BooleanOption&);
    };

    class StringOption
    : public CppConsUI::Button
    {
    friend class AccountDialog;
    public:
        StringOption(const char *text, const char *value, bool editable=true);
        virtual ~StringOption();
    protected:
        std::string m_value;

        void OnActivate(CppConsUI::Button& activator);
        void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    private:
        StringOption(const StringOption&);
        StringOption& operator=(const StringOption&);
    };

    class IntegerOption
    : public CppConsUI::Button
    {
    friend class AccountDialog;
    public:
        IntegerOption(const char *text, int value);
        virtual ~IntegerOption();
    protected:
        int m_value;

        void OnActivate(CppConsUI::Button& activator);
        void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    private:
        IntegerOption(const IntegerOption&);
        IntegerOption& operator=(const IntegerOption&);
    };

    class ChoiceOption
    : public CppConsUI::ComboBox
    {
    friend class AccountDialog;
    public:
        ChoiceOption(const char *text);
        virtual ~ChoiceOption();

        void AddOption(const char *title, const char *value, const char *defaultValue);
    protected:
        std::string m_value;

        void OnSelectionChanged(CppConsUI::ComboBox& activator, int new_entry, const char *title, intptr_t data);
    private:
        ChoiceOption(const ChoiceOption&);
        ChoiceOption& operator=(const ChoiceOption&);
    };
private:
    AccountDialog(const AccountDialog&);
    AccountDialog& operator=(const AccountDialog&);

    PURPLE::Protocol m_protocol; //use for creating
    PURPLE::Account m_account; //use for modifying
    PURPLE::Protocol::Options m_options;
    bool m_editing;
    std::vector<StringOption*> m_usernameFields;
    std::vector<CppConsUI::Widget*> m_optionsFields;
    StringOption *m_password, *m_alias;
    BooleanOption *m_registerAccount, *m_checkEmail, *m_rememberPassword, *m_enabled;

    void CreateWidgets();
    std::string JoinUsername();
    void FillProtocolOptions();
    void OnOk(CppConsUI::Button& activator);
    void OnDelete(CppConsUI::Button& activator);
    void OnDeleteResponseHandler(CppConsUI::MessageDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
};

#endif // ACCOUNTDIALOG_H

