/*
 *      accountsdialog.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ACCOUNTSDIALOG_H
#define ACCOUNTSDIALOG_H

#include <cppconsui/Button.h>
#include <cppconsui/ListBox.h>
#include <cppconsui/SplitDialog.h>
#include <cppconsui/AbstractDialog.h>
#include <purplemm.h>

class ProtocolDialog;

class AccountsDialog
: public CppConsUI::SplitDialog
{
public:
    AccountsDialog();
    virtual ~AccountsDialog() {}

    // FreeWindow
    virtual void OnScreenResized();
protected:
    class AccountItem
    : public CppConsUI::Button
    {
    public:
        AccountItem(PURPLE::Account account, CppConsUI::SplitDialog *parent);
        virtual ~AccountItem();
        
        PURPLE::Account GetAccount();
    protected:
        PURPLE::Account m_account;
        CppConsUI::SplitDialog *m_parent;

        void OnActivate(CppConsUI::Button& activator);
    private:
        AccountItem(const AccountItem&);
        AccountItem& operator=(const AccountItem&);
    };

    void AddAccount(PURPLE::Account account);
private:
    AccountsDialog(const AccountsDialog&);
    AccountsDialog& operator=(const AccountsDialog&);

    CppConsUI::ListBox *m_box;
    std::vector<AccountItem*> m_items;

    void OnAdd(CppConsUI::Button& activator);
    void OnProtocol(ProtocolDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    void OnAccountAdded(PURPLE::Account account);
    void OnAccountDeleted(std::string account, std::string protocolID);
};

#endif // ACCOUNTSDIALOG_H

