/*
 *      actionrequestdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "gettext.h"
#include "dxim.h"
#include "actionrequestdialog.h"
#include "log.h"
#include "dxutils.h"

ActionRequestDialog::ActionRequestDialog(PURPLE::ActionRequest *request)
: SplitDialog(0, 0, 80, 24, request->getTitle().empty()?_("New request"):request->getTitle().c_str()), m_request(request)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getPrimary().c_str());
    box->AppendWidget(*b);
    if(!m_request->getSecondary().empty())
    {
        b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getSecondary().c_str());
        box->AppendWidget(*b);
    }
    std::map<std::string, PurpleRequestActionCb> callbacks = m_request->getCallbacks();
    std::map<std::string,PurpleRequestActionCb>::iterator it;
    size_t i;
    for(it=callbacks.begin(),i=0; it != callbacks.end(); it++,i++)
    {
        b = buttons->AppendItem(dxutils::normalizeLabelString((*it).first, '_').c_str(), sigc::bind(sigc::mem_fun(this, &ActionRequestDialog::OnCallback), (*it).second));
        if(i < callbacks.size()-2) buttons->AppendSeparator();
        if(i == (size_t)m_request->getDefaultAction()) buttons->SetFocusChild(*b);
    }
    LOG->Debug("ActionRequestDialog::ActionRequestDialog(): this=%p", static_cast<void*>(this));
}

void ActionRequestDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void ActionRequestDialog::OnCallback(CppConsUI::Button& /*activator*/, PurpleRequestActionCb cb)
{
    m_request->runCallback(cb);
    Close();
}




