/*
 *      addbuddyndialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "addbuddydialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"
#include <utilsmm.h>

AddBuddyDialog::AddBuddyDialog(PURPLE::Account account)
: SplitDialog(0, 0, 80, 24, _("Add friend")), m_account(account)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Account"),
                                                 dxutils::sformat("%s [%s]", account.getUsername().c_str(), account.getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    m_id = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("ID"));
    m_id->signal_activate.connect(sigc::mem_fun(this, &AddBuddyDialog::OnId));
    box->AppendWidget(*m_id);
    m_nick = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Nick"));
    m_nick->signal_activate.connect(sigc::mem_fun(this, &AddBuddyDialog::OnNick));
    box->AppendWidget(*m_nick);
    m_groups = new CppConsUI::ComboBox(AUTOSIZE, 1, _("Group"));
    std::vector<std::string> v = m_account.getAllGroups();
    bool friends = false;
    int i = 0;
    FOREACH(std::vector<std::string>, v, it)
    {
        if(*it == _("Friends"))
        {
            friends = true;
            i = m_groups->AddOption((*it).c_str());
        }
        else
        {
            m_groups->AddOption((*it).c_str());
        }
    }
    if(!friends) i = m_groups->AddOption(_("Friends"));
    m_groups->SetSelected(i);
    box->AppendWidget(*m_groups);
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &AddBuddyDialog::OnOk));
}

void AddBuddyDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void AddBuddyDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    m_account.addBuddy(PURPLE::p2s(m_id->GetValue()), m_groups->GetSelectedTitle(), _("I want to add you in buddies"), PURPLE::p2s(m_nick->GetValue()));
    Close();
}

void AddBuddyDialog::OnId(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("ID"), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &AddBuddyDialog::OnIdResponseHandler));
    dialog->Show();
}

void AddBuddyDialog::OnIdResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_id->SetValue(activator.GetText());
    }
}

void AddBuddyDialog::OnNick(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("Nick"), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &AddBuddyDialog::OnNickResponseHandler));
    dialog->Show();
}

void AddBuddyDialog::OnNickResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_nick->SetValue(activator.GetText());
    }
}
