/*
 *      bindingsdialog.cpp
 *
 *      Copyright 2017 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "bindingsdialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"

BindingsDialog::BindingsDialog()
: SplitDialog(0, 0, 80, 24, _("Key bindings"))
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    m_text = new CppConsUI::TextView(1, 1, false, true);
    box->AppendWidget(*m_text);
    m_text->Append("dxim");
    m_text->Append("----");
    m_text->Append(dxutils::sformat(_("%s - Quit"), KEYCONFIG->GetKeyBind("dxim", "quit", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Main menu"), KEYCONFIG->GetKeyBind("dxim", "mainmenu", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Buddy list"), KEYCONFIG->GetKeyBind("dxim", "roster-active", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Active tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-active", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Status menu"), KEYCONFIG->GetKeyBind("dxim", "roster-active", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - New IM"), KEYCONFIG->GetKeyBind("dxim", "newimmenu", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Join chat"), KEYCONFIG->GetKeyBind("dxim", "joinchatmenu", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Hide/Show offline friends"), KEYCONFIG->GetKeyBind("dxim", "roster-toggle-offline", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Expand tabs"), KEYCONFIG->GetKeyBind("dxim", "tabbook-expand", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Close current tab"), KEYCONFIG->GetKeyBind("dxim", "tabbook-closecurrent", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Previous tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-prev", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Next tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-next", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 1. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number1", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 2. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number2", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 3. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number3", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 4. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number4", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 5. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number5", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 6. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number6", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 7. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number7", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 8. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number8", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Go to 9. tab"), KEYCONFIG->GetKeyBind("dxim", "tabitem-number9", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(" ");
    m_text->Append(_("Conversation"));
    m_text->Append("------------");
    m_text->Append(dxutils::sformat(_("%s - Send text"), KEYCONFIG->GetKeyBind("tabitem", "send", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(dxutils::sformat(_("%s - Clear text"), KEYCONFIG->GetKeyBind("tabitem", "clear", TERMKEY_FORMAT_LONGMOD)).c_str());
    m_text->Append(" ");
    m_text->Append(_("Buddy list"));
    m_text->Append("----------");
    m_text->Append(dxutils::sformat(_("%s - Context menu"), KEYCONFIG->GetKeyBind("roster", "contextmenu", TERMKEY_FORMAT_LONGMOD)).c_str());
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::hide(sigc::mem_fun(this, &BindingsDialog::Close)));
    m_text->Resize(DXIM->GetScreenArea(dxim::CHAT_AREA).width, height-buttons->GetHeight()-1);
}

void BindingsDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
    m_text->Resize(DXIM->GetScreenArea(dxim::CHAT_AREA).width, height-buttons->GetHeight()-1);
}

