/*
 *      chatdialog.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CHATDIALOG_H
#define CHATDIALOG_H

#include <cppconsui/SplitDialog.h>
#include <cppconsui/InputDialog.h>
#include <cppconsui/Button.h>
#include <cppconsui/ComboBox.h>
#include <cppconsui/CheckBox.h>
#include <purplemm.h>

class ChatDialog
: public CppConsUI::SplitDialog
{
    public:
        //for create new PURPLE::Chat
        ChatDialog(PURPLE::Account account);
        //for modify PURPLE::Chat
        ChatDialog(PURPLE::Chat);
        virtual ~ChatDialog() {}

        // FreeWindow
        virtual void OnScreenResized();
    protected:
        class IntegerOption
        : public CppConsUI::Button
        {
        public:
            IntegerOption(const char *text, const char *value, int min, int max);
            virtual ~IntegerOption();
        protected:
            int m_min, m_max;

            void OnActivate(CppConsUI::Button& activator);
            void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
        private:
            IntegerOption(const IntegerOption&);
            IntegerOption& operator=(const IntegerOption&);
        };
        
        class StringOption
        : public CppConsUI::Button
        {
        public:
            StringOption(const char *text, const char *value);
            virtual ~StringOption();
        protected:
            void OnActivate(CppConsUI::Button& activator);
            void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
        private:
            StringOption(const StringOption&);
            StringOption& operator=(const StringOption&);
        };
    private:
        ChatDialog(const ChatDialog&);
        ChatDialog& operator=(const ChatDialog&);

        PURPLE::Account m_account;
        PURPLE::Chat m_chat;
        std::vector<CppConsUI::Button*> m_fields;
        CppConsUI::CheckBox *m_autoJoin;
        StringOption *m_alias;
        CppConsUI::ComboBox *m_groups;
        bool m_modifing;

        void CreateWidgets();
        std::vector<PURPLE::Account::ChatEntry> FillChatEntryFields();
        void OnOk(CppConsUI::Button& activator);
};

#endif // CHATDIALOG_H
