/*
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef CHATTABITEM_H
#define CHATTABITEM_H

#include "log.h"
#include "tabitem.h"
#include <cppconsui/AbstractLine.h>
#include <cppconsui/HorizontalLine.h>
#include <cppconsui/Window.h>
#include <cppconsui/Label.h>
#include "callback/callback.h"
#include "../logger/logger.h"

class ChatTabItem
: public TabItem
{
public:
    ChatTabItem(PURPLE::Conversation *conv);
    virtual ~ChatTabItem();

    // InputProcessor
    virtual bool ProcessInput(const TermKeyKey& key);
    // Widget
    virtual void MoveResize(int newx, int newy, int neww, int newh);
protected:
    class ChatTabItemLine
    : public CppConsUI::AbstractLine
    {
    public:
        ChatTabItemLine(const char *text_);
        virtual ~ChatTabItemLine();
        // Widget
        virtual void Draw();
    protected:
        char *m_text;
        size_t m_text_width;
    private:
        ChatTabItemLine(const ChatTabItemLine&);
        ChatTabItemLine& operator=(const ChatTabItemLine&);
    };
    CppConsUI::Label *m_topic;
    CppConsUI::HorizontalLine *m_topicline;
    ChatTabItemLine *m_line;
    std::vector<std::string> m_buddies;

    PURPLE::ChatBuddy GetChatBuddy(std::string name);
private:
    ChatTabItem();
    ChatTabItem(const ChatTabItem&);
    ChatTabItem& operator=(const ChatTabItem&);

    bool ProcessCompletion();
    std::string GetCompletion(std::string line, size_t cursorPos);
    //purplemm slots
    void OnChatBuddies(PURPLE::Conversation *conv, std::vector<std::string> buddies);
    void OnChatBuddyJoined(PURPLE::Conversation *conv, std::string buddy);
    void OnChatBuddyLeft(PURPLE::Conversation *conv, std::string buddy);
    void OnChatBuddyRenamed(PURPLE::Conversation *conv, std::string buddy, std::string nbuddy);
    void OnChatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy);
    void OnChatTopic(PURPLE::Conversation *conv, std::string who, std::string topic);
};

#endif // CHATTABITEM_H


