/*
 *      choicerequestdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "gettext.h"
#include "dxim.h"
#include "choicerequestdialog.h"
#include "log.h"
#include "dxutils.h"

ChoiceRequestDialog::ChoiceRequestDialog(PURPLE::ChoiceRequest *request)
: SplitDialog(0, 0, 80, 24, request->getTitle().empty()?_("New request"):request->getTitle().c_str()), m_request(request)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getPrimary().c_str());
    box->AppendWidget(*b);
    if(!m_request->getSecondary().empty())
    {
        b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getSecondary().c_str());
        box->AppendWidget(*b);
    }
    m_choice = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    ChoiceOption *ch;
    std::map<std::string, int> choices = m_request->getChoices();
    std::map<std::string, int>::iterator it;
    for(it=choices.begin(); it != choices.end(); it++)
    {
        ch = new ChoiceOption((*it).first.c_str(), (*it).second);
        m_choice->AppendWidget(*ch);
    }
    box->AppendWidget(*m_choice);
    buttons->AppendItem(dxutils::normalizeLabelString(request->getOkText(), '_').c_str(), sigc::bind(sigc::mem_fun(this, &ChoiceRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_OK));
    buttons->AppendSeparator();
    buttons->AppendItem(CANCEL_BUTTON_TEXT, sigc::bind(sigc::mem_fun(this, &ChoiceRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_CANCEL));
    LOG->Debug("ChoiceRequestDialog::ChoiceRequestDialog(): this=%p", static_cast<void*>(this));
}

void ChoiceRequestDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void ChoiceRequestDialog::OnResponse(CppConsUI::Button& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
        m_request->sendChoice(static_cast<ChoiceOption*>(m_choice->GetFocusChild())->GetID());
    else
        m_request->dennyRequest();
    Close();
}

ChoiceRequestDialog::ChoiceOption::ChoiceOption(const char* text, int id)
: Button(AUTOSIZE, 1, text), m_id(id)
{
    g_assert(text);
}

ChoiceRequestDialog::ChoiceOption::~ChoiceOption()
{}




