/*
 *      fieldsrequestdialog.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef FIELDSREQUESTDIALOG_H
#define FIELDSREQUESTDIALOG_H

#include <cppconsui/SplitDialog.h>
#include <cppconsui/InputDialog.h>
#include <cppconsui/Button.h>
#include <cppconsui/CheckBox.h>
#include <cppconsui/ComboBox.h>
#include <cppconsui/Label.h>
#include <purplemm.h>

class FieldsRequestDialog
: public CppConsUI::SplitDialog
{
    public:
        FieldsRequestDialog(PURPLE::FieldsRequest *request);
        virtual ~FieldsRequestDialog() {}

        // FreeWindow
        virtual void OnScreenResized();
        void OnResponse(CppConsUI::Button& activator, CppConsUI::AbstractDialog::ResponseType response);
    protected:
        class BooleanField
        : public CppConsUI::CheckBox
        {
        friend class FieldsRequestDialog;
        public:
            BooleanField(const char *text, bool value);
            virtual ~BooleanField();
        protected:
            bool m_value;

            void OnToggle(CppConsUI::CheckBox& activator, bool new_state);
        private:
            BooleanField(const BooleanField&);
            BooleanField& operator=(const BooleanField&);
        };

        class StringField
        : public CppConsUI::Button
        {
        friend class FieldsRequestDialog;
        public:
            StringField(const char *text, const char *value, bool editable=true);
            virtual ~StringField();
        protected:
            std::string m_value;

            void OnActivate(CppConsUI::Button& activator);
            void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
        private:
            StringField(const StringField&);
            StringField& operator=(const StringField&);
        };

        class IntegerField
        : public CppConsUI::Button
        {
        friend class FieldsRequestDialog;
        public:
            IntegerField(const char *text, int value);
            virtual ~IntegerField();
        protected:
            int m_value;

            void OnActivate(CppConsUI::Button& activator);
            void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
        private:
            IntegerField(const IntegerField&);
            IntegerField& operator=(const IntegerField&);
        };

        class ChoiceField
        : public CppConsUI::ComboBox
        {
        friend class FieldsRequestDialog;
        public:
            ChoiceField(const char *text);
            virtual ~ChoiceField();
        protected:
            int m_value;

            void OnSelectionChanged(CppConsUI::ComboBox& activator, int new_entry, const char *title, intptr_t data);
        private:
            ChoiceField(const ChoiceField&);
            ChoiceField& operator=(const ChoiceField&);
        };

        class ListField
        : public CppConsUI::HorizontalListBox
        {
        friend class FieldsRequestDialog;
        public:
            ListField(const char *text);
            virtual ~ListField() {}
        protected:
            class ListFieldItem
            : public CppConsUI::CheckBox
            {
            friend class FieldsRequestDialog;
            public:
                ListFieldItem(const char *text, bool selected);
                virtual ~ListFieldItem() {}
            protected:
                bool m_value;

                void OnToggle(CppConsUI::CheckBox& activator, bool new_state);
            private:
                ListFieldItem(const ListFieldItem&);
                ListFieldItem& operator=(const ListFieldItem&);
          };

          std::vector<ListFieldItem*> m_fields;
          
          void AppendItem(ListFieldItem *item);
        private:
          ListField(const ListField&);
          ListField& operator=(const ListField&);
          
          CppConsUI::ListBox *m_list;
        };
        
        class SingleField
        : public CppConsUI::ComboBox
        {
        friend class FieldsRequestDialog;
        public:
            SingleField(const char *text);
            virtual ~SingleField();
            
            void AddField(const char *title, const char *selected);
        protected:
            std::string m_value;

            void OnSelectionChanged(CppConsUI::ComboBox& activator, int new_entry, const char *title, intptr_t data);
        private:
            SingleField(const SingleField&);
            SingleField& operator=(const SingleField&);
        };
    private:
        FieldsRequestDialog(const FieldsRequestDialog&);
        FieldsRequestDialog& operator=(const FieldsRequestDialog&);

        PURPLE::FieldsRequest *m_request;
        std::vector<CppConsUI::Widget*> m_fields;

        void CreateFields(CppConsUI::ListBox *box);
        void FillFields();
};

#endif // FIELDSREQUESTDIALOG_H
