/*
 *      filerequestdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "gettext.h"
#include "dxim.h"
#include "filerequestdialog.h"
#include "log.h"
#include "dxutils.h"
#include <utilsmm.h>

FileRequestDialog::FileRequestDialog(PURPLE::FileRequest *request)
: SplitDialog(0, 0, 80, 24, request->getTitle().empty()?(request->getSaveDialog()?_("Save file"):_("Select file")):request->getTitle().c_str()), m_request(request)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    m_folder = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Folder"), g_get_home_dir());
    m_folder->signal_activate.connect(sigc::mem_fun(this, &FileRequestDialog::OnFolderActivate));
    box->AppendWidget(*m_folder);
    m_file = new StringOption(_("File name"), request->getFilename().c_str());
    box->AppendWidget(*m_file);
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::bind(sigc::mem_fun(this, &FileRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_OK));
    buttons->AppendSeparator();
    buttons->AppendItem(CANCEL_BUTTON_TEXT, sigc::bind(sigc::mem_fun(this, &FileRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_CANCEL));
    LOG->Debug("FileRequestDialog::FileRequestDialog(): this=%p", static_cast<void*>(this));
}

void FileRequestDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void FileRequestDialog::OnResponse(CppConsUI::Button& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        if(m_request->getSaveDialog())
        {
            if(g_file_test(m_folder->GetValue(), G_FILE_TEST_IS_DIR) && !PURPLE::p2s(m_file->GetValue()).empty())
                m_request->setFilename(g_build_filename(m_folder->GetValue(), m_file->GetValue(), NULL));
            else
                m_request->dennyRequest();
        }
        else
        {
            char *file = g_build_filename(m_folder->GetValue(), m_file->GetValue(), NULL);
            if(g_file_test(m_folder->GetValue(), G_FILE_TEST_IS_DIR) &&
                g_file_test(file, G_FILE_TEST_EXISTS) && !g_file_test(file, G_FILE_TEST_IS_DIR))
                m_request->setFilename(file);
            else
                m_request->dennyRequest();
        }
    }
    else
        m_request->dennyRequest();
    Close();
}

void FileRequestDialog::OnFolderActivate(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(m_folder->GetText(), m_folder->GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &FileRequestDialog::OnFolderResponseHandler));
    dialog->Show();
}

void FileRequestDialog::OnFolderResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      m_folder->SetValue(activator.GetText());
      break;
    default:
      break;
    }
}

FileRequestDialog::StringOption::StringOption(const char *text, const char *value)
: Button(AUTOSIZE, 1, FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &StringOption::OnActivate));
}

FileRequestDialog::StringOption::~StringOption()
{}

void FileRequestDialog::StringOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringOption::ResponseHandler));
    dialog->Show();
}

void FileRequestDialog::StringOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    switch (response) {
      case CppConsUI::AbstractDialog::RESPONSE_OK:
        SetValue(activator.GetText());
        break;
      default:
        break;
    }
}


