/*
 * Copyright (C) 2011-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "footer.h"
#include <cppconsui/KeyConfig.h>
#include <string.h> // strstr, strncpy, strlen

Footer *Footer::m_instance = NULL;

Footer *Footer::Instance()
{
    return m_instance;
}

void Footer::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::FOOTER_AREA));
}

void Footer::SetText(const char *fmt, ...)
{
    m_values.clear();
    if(!fmt)
    {
        UpdateText();
        return;
    }
    m_values.push_back(std::string(fmt));
    va_list args;
    va_start(args, fmt);
    while(*fmt)
    {
        if(*fmt == '%')
        {
            if(*(fmt + 1) == '%')
                fmt++;
            else if(*(fmt + 1) == 's')
            {
                const char *v = va_arg(args, const char*);
                m_values.push_back(std::string(v));
                fmt++;
            }
        }
        fmt++;
    }
    va_end(args);
    UpdateText();
}

Footer::Footer()
: FreeWindow(0, 24, 80, 1, TYPE_NON_FOCUSABLE)
{
    SetColorScheme("footer");
    m_label = new CppConsUI::Label;
    AddWidget(*m_label, 0, 0);
}

void Footer::Init()
{
    g_assert(!m_instance);
    m_instance = new Footer;
    m_instance->Show();
}

void Footer::Finalize()
{
    g_assert(m_instance);
    delete m_instance;
    m_instance = NULL;
}

void Footer::UpdateText()
{
    if(m_values.empty())
    {
        m_label->SetText(NULL);
        return;
    }
    Values::iterator i = m_values.begin();
    const char *fmt = i->c_str();
    i++;
    char out[1024];
    char *cur_out = out;
    while(*fmt && cur_out < out + sizeof(out) - 1)
    {
        if(*fmt == '%')
        {
            if(*(fmt + 1) == '%')
                fmt++;
            else if (*(fmt + 1) == 's')
            {
                char *con = g_strdup(i->c_str());
                i++;
                char *act = strstr(con, "|");
                g_assert(act);
                *act++ = '\0';
                const char *key = KEYCONFIG->GetKeyBind(con, act);
                g_free(con);

                strncpy(cur_out, key, out + sizeof(out) - 1 - cur_out);
                cur_out += strlen(key);

                fmt += 2;
                continue;
            }
        }
        *cur_out++ = *fmt++;
    }
    *cur_out = '\0';
    m_label->SetText(out);
}

