/*
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef IMTABITEM_H
#define IMTABITEM_H

#include "log.h"
#include "tabitem.h"
#include <cppconsui/AbstractLine.h>
#include <cppconsui/Window.h>
#include "callback/callback.h"
#include "../logger/logger.h"

class IMTabItem
: public TabItem
{
public:
    IMTabItem(PURPLE::Conversation *conv);
    virtual ~IMTabItem();

    // InputProcessor
    virtual bool ProcessInput(const TermKeyKey& key);
    // Widget
    virtual void MoveResize(int newx, int newy, int neww, int newh);
    void LoadCurrentLog();
protected:
    class IMTabItemLine
    : public CppConsUI::AbstractLine
    {
    public:
        IMTabItemLine(const char *text);
        virtual ~IMTabItemLine();
        // Widget
        virtual void Draw();
        void SetText(const char* text);
    protected:
        char *m_text;
        size_t m_text_width;
    private:
        IMTabItemLine(const IMTabItemLine&);
        IMTabItemLine& operator=(const IMTabItemLine&);
    };
    IMTabItemLine *m_line;
private:
    IMTabItem();
    IMTabItem(const IMTabItem&);
    IMTabItem& operator=(const IMTabItem&);
    
    bool ProcessCompletion();
    bool OnTimeout();
    void OnInputTextChange(CppConsUI::TextEdit& activator);
    //purplemm slots
    void OnBuddySignedOn(PURPLE::Buddy buddy);
    void OnBuddySignedOff(PURPLE::Buddy buddy);
    void OnBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive status);
};

#endif // IMTABITEM_H


