/*
 *      inputrequestdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "gettext.h"
#include "dxim.h"
#include "inputrequestdialog.h"
#include "log.h"
#include "dxutils.h"
#include <utilsmm.h>

InputRequestDialog::InputRequestDialog(PURPLE::InputRequest *request)
: SplitDialog(0, 0, 80, 24, request->getTitle().empty()?_("New request"):request->getTitle().c_str()), m_request(request)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getPrimary().c_str());
    box->AppendWidget(*b);
    if(!m_request->getSecondary().empty())
    {
        b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getSecondary().c_str());
        box->AppendWidget(*b);
    }
    m_input = new StringOption(_("Request"), m_request->getDefaultValue().c_str());
    box->AppendWidget(*m_input);
    buttons->AppendItem(dxutils::normalizeLabelString(request->getOkText(), '_').c_str(), sigc::bind(sigc::mem_fun(this, &InputRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_OK));
    buttons->AppendSeparator();
    buttons->AppendItem(CANCEL_BUTTON_TEXT, sigc::bind(sigc::mem_fun(this, &InputRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_CANCEL));
    LOG->Debug("InputRequestDialog::InputRequestDialog(): this=%p", static_cast<void*>(this));
}

void InputRequestDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void InputRequestDialog::OnResponse(CppConsUI::Button& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
        m_request->sendInput(PURPLE::p2s(m_input->GetValue()));
    else
        m_request->dennyRequest();
    Close();
}

InputRequestDialog::StringOption::StringOption(const char *text, const char *value)
: Button(AUTOSIZE, 1, FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &StringOption::OnActivate));
}

InputRequestDialog::StringOption::~StringOption()
{}

void InputRequestDialog::StringOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringOption::ResponseHandler));
    dialog->Show();
}

void InputRequestDialog::StringOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
  switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      SetValue(activator.GetText());
      break;
    default:
      break;
  }
}


