/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef LOG_H
#define LOG_H

#include "dxim.h"
#include <cppconsui/TextView.h>
#include <cppconsui/Window.h>

#define LOG (Log::Instance())

class Log
: public CppConsUI::Window
{
public:
    enum Level {
        LEVEL_NONE,
        LEVEL_ERROR,
        LEVEL_WARNING,
        LEVEL_MESSAGE,
        LEVEL_DEBUG
    };
    static Log *Instance();
    // FreeWindow
    virtual void OnScreenResized();
    void Error(const char *fmt, ...) _attribute((format(printf, 2, 3)));
    void Warning(const char *fmt, ...) _attribute((format(printf, 2, 3)));
    void Message(const char *fmt, ...) _attribute((format(printf, 2, 3)));
    void Debug(const char *fmt, ...) _attribute((format(printf, 2, 3)));
private:
    friend class dxim;
    CppConsUI::TextView *m_textview;
    static Log *m_instance;
    guint m_default_handler;
    guint m_glib_handler;
    guint m_gmodule_handler;
    guint m_glib_gobject_handler;
    guint m_gthread_handler;
    guint m_cppconsui_handler;

    Log();
    Log(const Log&);
    Log& operator=(const Log&);
    virtual ~Log();

    static void Init();
    static void Finalize();
    void ShortenWindowText();
    void Write(const char *text, int color = 0);
    // to catch default messages
    static void default_log_handler(const char *domain, GLogLevelFlags flags,  const char *msg, gpointer user_data)
    {
        reinterpret_cast<Log*>(user_data)->logHandler(domain, msg);
    }
    // to catch glib's messages
    static void glib_log_handler(const char *domain, GLogLevelFlags flags, const char *msg, gpointer user_data)
    {
        reinterpret_cast<Log*>(user_data)->logHandler(domain, msg);
    }
    // to catch cppconsui messages
    static void cppconsui_log_handler(const char *domain, GLogLevelFlags flags, const char *msg, gpointer user_data)
    {
        reinterpret_cast<Log*>(user_data)->logHandler(domain, msg);
    }
    void logHandler(const char *domain, const char *msg);
};

#endif // LOG_H

