/*
 * mainmenu.cpp
 *
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "mainmenu.h"
#include "gettext.h"
#include "dxim.h"
#include "statusmenu.h"
#include "newimmenu.h"
#include "joinchatmenu.h"
#include "optiondialog.h"
#include "tabbook.h"
#include "accountsdialog.h"
#include "aboutdialog.h"
#include "pluginsdialog.h"
#include "newbuddymenu.h"
#include "newchatmenu.h"
#include "newgroupmenu.h"
#include "bindingsdialog.h"

MainMenu::MainMenu()
: MenuWindow(0, 0, AUTOSIZE, AUTOSIZE)
{
    SetColorScheme("generalmenu");
    AppendItem(_("Manage accounts"), sigc::mem_fun(this, &MainMenu::OpenAccountsDialog));
    AppendSeparator();
    if(DXIM->GetPurplemm()->isSomeAccountConnected())
    {
        AppendItem(_("Add buddy"), sigc::mem_fun(this, &MainMenu::OpenNewBuddyMenu));
        AppendItem(_("Add chat"), sigc::mem_fun(this, &MainMenu::OpenNewChatMenu));
        AppendItem(_("Add group"), sigc::mem_fun(this, &MainMenu::OpenNewGroupMenu));
        AppendSeparator();
        AppendItem(_("New IM"), sigc::mem_fun(this, &MainMenu::OpenNewImMenu));
        AppendItem(_("Join chat"), sigc::mem_fun(this, &MainMenu::OpenJoinChatMenu));
        AppendSeparator();
    }
    if(TABBOOK->GetNumTabItems())
    {
        AppendItem(_("Clear window"), sigc::mem_fun(this, &MainMenu::ClearCurrent));
        AppendItem(_("Clear all windows"), sigc::mem_fun(this, &MainMenu::ClearAll));
        AppendSeparator();
    }
    AppendItem(_("Plugins"), sigc::mem_fun(this, &MainMenu::OpenPluginsDialog));
    AppendItem(_("Preferences"), sigc::mem_fun(this, &MainMenu::OpenOptionDialog));
    AppendSeparator();
    AppendItem(_("Change status"), sigc::mem_fun(this, &MainMenu::OpenStatusMenu));
    AppendSeparator();
    AppendItem(_("Key bindings"), sigc::mem_fun(this, &MainMenu::Keybindings));
    AppendSeparator();
    AppendItem(_("About"), sigc::mem_fun(this, &MainMenu::About));
    AppendSeparator();
    AppendItem(_("Quit"), sigc::hide(sigc::mem_fun(DXIM, &dxim::Quit)));
}

void MainMenu::OnScreenResized()
{
    CppConsUI::Rect chat = DXIM->GetScreenArea(dxim::CHAT_AREA);
    Move(chat.x, chat.y);
}

void MainMenu::OpenAccountsDialog(CppConsUI::Button& /*activator*/)
{
    AccountsDialog *dialog = new AccountsDialog();
    dialog->Show();
    Close();
}

void MainMenu::OpenNewBuddyMenu(CppConsUI::Button& /*activator*/)
{
    NewBuddyMenu *menu = new NewBuddyMenu();
    menu->Show();
    Close();
}

void MainMenu::OpenNewChatMenu(CppConsUI::Button& /*activator*/)
{
    NewChatMenu *menu = new NewChatMenu();
    menu->Show();
    Close();
}

void MainMenu::OpenNewGroupMenu(CppConsUI::Button& /*activator*/)
{
    NewGroupMenu *menu = new NewGroupMenu();
    menu->Show();
    Close();
}

void MainMenu::OpenStatusMenu(CppConsUI::Button& /*activator*/)
{
    StatusMenu *menu = new StatusMenu();
    menu->Show();
    Close();
}

void MainMenu::OpenNewImMenu(CppConsUI::Button& /*activator*/)
{
    NewImMenu *menu = new NewImMenu();
    menu->Show();
    Close();
}

void MainMenu::OpenJoinChatMenu(CppConsUI::Button& /*activator*/)
{
    JoinChatMenu *menu = new JoinChatMenu();
    menu->Show();
    Close();
}

void MainMenu::OpenPluginsDialog(CppConsUI::Button& /*activator*/)
{
    PluginsDialog *dialog = new PluginsDialog();
    dialog->Show();
    Close();
}

void MainMenu::OpenOptionDialog(CppConsUI::Button& /*activator*/)
{
    OptionDialog *dialog = new OptionDialog();
    dialog->Show();
    Close();
}

void MainMenu::ClearCurrent(CppConsUI::Button& /*activator*/)
{
    TABBOOK->ClearCurrentTabItem();
    Close();
}

void MainMenu::ClearAll(CppConsUI::Button& /*activator*/)
{
    TABBOOK->ClearAllTabItems();
    Close();
}

void MainMenu::About(CppConsUI::Button& /*activator*/)
{
    AboutDialog *dialog = new AboutDialog();
    dialog->Show();
    Close();
}

void MainMenu::Keybindings(CppConsUI::Button & /*activator*/)
{
    BindingsDialog *dialog = new BindingsDialog();
    dialog->Show();
    Close();
}

