/*
 * newgroupmenu.cpp
 *
 * Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "newgroupmenu.h"
#include "gettext.h"
#include "dxim.h"
#include "dxutils.h"
#include "utilsmm.h"

NewGroupMenu::NewGroupMenu()
: MenuWindow(0, 0, AUTOSIZE, AUTOSIZE)
{
    SetTitle(_("Select account"));
    SetColorScheme("generalmenu");
    std::vector<PURPLE::Account> accounts = DXIM->GetPurplemm()->getConnectedAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        if(accounts[i].supportsChats()) AppendItem(dxutils::sformat("%s [%s]", accounts[i].getUsername().c_str(), accounts[i].getProtocol().getName().c_str()).c_str(),
                   sigc::bind(sigc::mem_fun(this, &NewGroupMenu::OpenPopup), accounts[i]));
    }
}

void NewGroupMenu::OnScreenResized()
{
    CppConsUI::Rect chat = DXIM->GetScreenArea(dxim::CHAT_AREA);
    Move(chat.x, chat.y);
}

void NewGroupMenu::OpenPopup(CppConsUI::Button& /*activator*/, PURPLE::Account account)
{
    m_account = account;
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("Add group"), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &NewGroupMenu::OnAddGroupResponseHandler));
    dialog->Show();
}

void NewGroupMenu::OnAddGroupResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_account.addGroup(PURPLE::p2s(activator.GetText()));
    }
    Close();
}


