/*
 * newimmenu.cpp
 *
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "newimmenu.h"
#include "gettext.h"
#include "dxim.h"
#include "dxutils.h"
#include <utilsmm.h>

NewImMenu::NewImMenu()
: MenuWindow(0, 0, AUTOSIZE, AUTOSIZE)
{
    SetTitle(_("Select account"));
    SetColorScheme("generalmenu");
    std::vector<PURPLE::Account> accounts = DXIM->GetPurplemm()->getConnectedAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        AppendItem(dxutils::sformat("%s [%s]", accounts[i].getUsername().c_str(), accounts[i].getProtocol().getName().c_str()).c_str(),
                   sigc::bind(sigc::mem_fun(this, &NewImMenu::OpenPopup), accounts[i]));
    }
}

void NewImMenu::OnScreenResized()
{
    CppConsUI::Rect chat = DXIM->GetScreenArea(dxim::CHAT_AREA);
    Move(chat.x, chat.y);
}

void NewImMenu::OpenPopup(CppConsUI::Button& /*activator*/, PURPLE::Account account)
{
    PopupDialog *dialog = new PopupDialog(account);
    dialog->Show();
    Close();
}

NewImMenu::PopupDialog::PopupDialog(PURPLE::Account account)
: SplitDialog(5, 5, 50, 6, _("New IM")), m_account(account)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Account"),
                                                 dxutils::sformat("%s [%s]", account.getUsername().c_str(), account.getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    m_with = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("With"));
    m_with->signal_activate.connect(sigc::mem_fun(this, &PopupDialog::OnWith));
    box->AppendWidget(*m_with);
    buttons->AppendItem(_("IM"), sigc::mem_fun(this, &PopupDialog::OnIM));
}

void NewImMenu::PopupDialog::OnWith(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(m_with->GetText(), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &PopupDialog::OnWithResponseHandler));
    dialog->Show();
}

void NewImMenu::PopupDialog::OnWithResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_with->SetValue(activator.GetText());
    }
}

void NewImMenu::PopupDialog::OnIM(CppConsUI::Button& /*activator*/)
{
    std::string with = PURPLE::p2s(m_with->GetValue());
    if(with.empty())
    {
        Close();
        return;
    }
    DXIM->GetPurplemm()->createConversation(m_account, with, false);
    Close();
}

