/*
 *      optiondialog.h
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef OPTIONDIALOG_H
#define OPTIONDIALOG_H

#include <cppconsui/Button.h>
#include <cppconsui/CheckBox.h>
#include <cppconsui/ComboBox.h>
#include <cppconsui/InputDialog.h>
#include <cppconsui/SplitDialog.h>

class OptionDialog
: public CppConsUI::SplitDialog
{
public:
    OptionDialog();
    virtual ~OptionDialog() {}

    // FreeWindow
    virtual void OnScreenResized();
protected:
    class BooleanOption
    : public CppConsUI::CheckBox
    {
    friend class OptionDialog;
    public:
        BooleanOption(const char *text, bool value);
        virtual ~BooleanOption();
    protected:
        bool m_value;

        void OnToggle(CppConsUI::CheckBox& activator, bool new_state);
    private:
        BooleanOption(const BooleanOption&);
        BooleanOption& operator=(const BooleanOption&);
    };

    class StringOption
    : public CppConsUI::Button
    {
    friend class OptionDialog;
    public:
        StringOption(const char *text, const char *value);
        virtual ~StringOption();
    protected:
        std::string m_value;

        void OnActivate(CppConsUI::Button& activator);
        void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    private:
        StringOption(const StringOption&);
        StringOption& operator=(const StringOption&);
    };

    class IntegerOption
    : public CppConsUI::Button
    {
    friend class OptionDialog;
    public:
        IntegerOption(const char *text, int value);
        virtual ~IntegerOption();
    protected:
        int m_value;

        void OnActivate(CppConsUI::Button& activator);
        void ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    private:
        IntegerOption(const IntegerOption&);
        IntegerOption& operator=(const IntegerOption&);
    };
private:
    OptionDialog(const OptionDialog&);
    OptionDialog& operator=(const OptionDialog&);

    //General settings
    BooleanOption *m_useAway;
    IntegerOption *m_awayTimeout; //in minutes
    BooleanOption *m_logging;
    StringOption *m_logPath;
    BooleanOption *m_logLoadCurrent;
    BooleanOption *m_receiptsMark;
    //Notification settings
    BooleanOption *m_notify;
    BooleanOption *m_notifyAccountConnect;
    BooleanOption *m_notifyAccountDisconnect;
    BooleanOption *m_notifyBuddyConnect;
    BooleanOption *m_notifyBuddyDisconnect;
    BooleanOption *m_notifyMessage;
    BooleanOption *m_notifyFiletransfer;
    //Other settings value
    BooleanOption *m_hideOffline;
    BooleanOption *m_showStatusmsg;
    BooleanOption *m_showRosterTree;
    IntegerOption *m_rosterWidth;
    IntegerOption *m_logHeight;

    void OnDone(CppConsUI::Button& activator);
};

#endif // OPTIONDIALOG_H

