/*
 *      pluginsdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <cppconsui/ComboBox.h>
#include "pluginsdialog.h"
#include "plugindialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"

PluginsDialog::PluginsDialog()
: SplitDialog(0, 0, 80, 24, _("Plugins list"))
{
    SetColorScheme("generaldialog");
    m_box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*m_box);
    std::vector<PURPLE::Plugin> v = DXIM->GetPurplemm()->getPluginsList();
    FOREACH(std::vector<PURPLE::Plugin>, v, it)
    {
        AddPlugin(*it);
    }
    buttons->AppendItem(_("Done"), sigc::mem_fun(this, &PluginsDialog::OnDone));
}

void PluginsDialog::OnDone(CppConsUI::Button& /*activator*/)
{
    DXIM->GetPurplemm()->saveLoadedPlugins("/dxim/plugins/loaded");
    Close();
}

void PluginsDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void PluginsDialog::AddPlugin(PURPLE::Plugin plugin)
{
    PluginItem *item = new PluginItem(plugin, this);
    m_box->AppendWidget(*item);
}

PluginsDialog::PluginItem::PluginItem(PURPLE::Plugin plugin, SplitDialog *parent)
: Button(AUTOSIZE, 1, plugin.getName().c_str()),
  m_plugin(plugin), m_parent(parent)
{
    if(plugin.isLoaded())
        SetColorScheme("rosteraccount_online");
    else
        SetColorScheme("rosteraccount_offline");
    signal_activate.connect(sigc::mem_fun(this, &PluginItem::OnActivate));
}

PluginsDialog::PluginItem::~PluginItem()
{}

void PluginsDialog::PluginItem::OnActivate(CppConsUI::Button& /*activator*/)
{
    PluginDialog *dialog = new PluginDialog(m_plugin);
    dialog->Show();
    if(m_plugin.isLoaded())
        SetColorScheme("rosteraccount_online");
    else
        SetColorScheme("rosteraccount_offline");
}


