/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ROSTER_H
#define ROSTER_H

#include <vector>
#include <cppconsui/TreeView.h>
#include <cppconsui/Window.h>
#include "callback/callback.h"
#include "rosteritem.h"

#define ROSTER (Roster::Instance())

class Roster
: public CppConsUI::Window
{
public:
    static Roster *Instance();
    // Widget
    virtual bool RestoreFocus();
    virtual void UngrabFocus();
    // FreeWindow
    virtual void OnScreenResized();
    void FillRoster();
    void ToggleHideOffline();
    void ToggleShowTree();
    bool ProcessInput(const TermKeyKey& key);
private:
    friend class dxim;
    CppConsUI::TreeView *m_treeview;
    static Roster *m_instance;
    std::vector<RosterItem*> m_items;

    Roster();
    Roster(const Roster&);
    Roster& operator=(const Roster&);
    virtual ~Roster();

    static void Init();
    static void Finalize();
    RosterAccountItem *AddAccount(PURPLE::Account account);
    RosterGroupItem *AddGroup(RosterAccountItem *account, std::string name);
    RosterBuddyItem *AddBuddy(RosterGroupItem *group, PURPLE::Buddy buddy);
    RosterChatItem *AddChat(RosterGroupItem *group, PURPLE::Chat chat);
    RosterAccountItem *FindAccount(std::string account, std::string protocolID);
    RosterGroupItem *FindGroup(PURPLE::Account account, std::string group);
    RosterBuddyItem *FindBuddy(PURPLE::Account account, PURPLE::Buddy buddy);
    RosterChatItem *FindChat(PURPLE::Account account, PURPLE::Chat chat);
    void Sort();

    //purplemm slots
    void OnAccountAdded(PURPLE::Account account);
    void OnAccountRemoved(std::string account, std::string protocolID);
    void OnAccountSignedOn(PURPLE::Account account);
    void OnBuddyAdded(PURPLE::Buddy buddy);
    void OnBuddyRemoved(PURPLE::Account account, std::string buddy);
    void OnChatAdded(PURPLE::Chat chat);
    void OnChatRemoved(PURPLE::Account account, std::string chat);
};

#endif // ROSTER_H

