/*
 * rosteritem.h
 *
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ROSTERITEM_H
#define	ROSTERITEM_H

#include <cppconsui/Button.h>
#include <cppconsui/CheckBox.h>
#include <cppconsui/TreeView.h>
#include <cppconsui/ComboBox.h>
#include <cppconsui/MenuWindow.h>
#include <cppconsui/AbstractDialog.h>
#include <cppconsui/InputDialog.h>
#include <cppconsui/MessageDialog.h>
#include <cppconsui/SplitDialog.h>
#include <purplemm.h>

class RosterItem
: public CppConsUI::Button
{
public:
    enum RosterItemType {
        Account = 0,
        Group,
        Buddy,
        Chat
    };
    RosterItem(PURPLE::Account account, CppConsUI::TreeView *tree, RosterItemType type);
    virtual ~RosterItem();

    virtual void Draw();
    virtual void SetText(const char *new_text);
    virtual void SetStatus(std::string new_status);
    virtual const char *GetStatus() const { return m_status; }
    virtual void Update() {}
    void SetRefNode(CppConsUI::TreeView::NodeReference n) { m_ref = n; }
    CppConsUI::TreeView::NodeReference GetRefNode() const { return m_ref; }
    PURPLE::Account GetAccount() const { return m_account; }
    RosterItemType GetType() const { return m_type; }
    virtual std::string GetName() const { return ""; }
protected:
    char *m_status;
    PURPLE::Account m_account;
    CppConsUI::TreeView *m_tree;
    CppConsUI::TreeView::NodeReference m_ref;
    RosterItemType m_type;

    virtual void OpenContextMenu() = 0;
private:
    RosterItem(const RosterItem&);
    RosterItem& operator=(const RosterItem&);

    void ActionOpenContextMenu();
    void DeclareBindables();
    void UpdateWidthHeight();
};

class RosterAccountItem
: public RosterItem
{
public:
    RosterAccountItem(PURPLE::Account account, CppConsUI::TreeView *tree);
    virtual ~RosterAccountItem() {}

    virtual void OnActivate(CppConsUI::Button& activator);
    void Update();
    std::string GetName() const { return m_account.getUsername(); }
private:
    class PopupDialog
    : public CppConsUI::SplitDialog
    {
        public:
            PopupDialog(PURPLE::Account account);
            virtual ~PopupDialog() {}
        private:
            PURPLE::Account m_account;
            CppConsUI::Button *m_with;

            PopupDialog(const PopupDialog&);
            PopupDialog& operator=(const PopupDialog&);

            void OnWith(CppConsUI::Button& activator);
            void OnWithResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
            void OnIM(CppConsUI::Button& activator);
    };

    RosterAccountItem(const RosterAccountItem&);
    RosterAccountItem& operator=(const RosterAccountItem&);

    CppConsUI::MenuWindow *m_popup;

    void UpdateColorScheme();
    void OpenContextMenu();
    void OnStatus(CppConsUI::Button& activator);
    void OnReconnect(CppConsUI::Button& activator);
    void OnEnable(CppConsUI::Button& activator);
    void OnAlias(CppConsUI::Button& activator);
    void OnAliasResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    void OnAddBuddy(CppConsUI::Button& activator);
    void OnAddChat(CppConsUI::Button& activator);
    void OnAddGroup(CppConsUI::Button& activator);
    void OnAddGroupResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    void OnAccountCb(CppConsUI::Button& activator, std::string cmd);
    void OnIM(CppConsUI::Button& activator);
    void OnJoinChat(CppConsUI::Button& activator);
    void OnConfig(CppConsUI::Button& activator);
    //purplemm slots
    void OnAccountConnecting(PURPLE::Account account, std::string msg);
    void OnAccountConnected(PURPLE::Account account);
    void OnAccountDisconnected(PURPLE::Account account, std::string reason);
    void OnAccountStatusChanged(PURPLE::Account account);
};

class RosterGroupItem
: public RosterItem
{
public:
    RosterGroupItem(PURPLE::Account account, CppConsUI::TreeView *tree, std::string name);
    virtual ~RosterGroupItem() {}

    virtual void OnActivate(CppConsUI::Button& activator);
    void Update();
    std::string GetName() const { return m_name; }
private:
    RosterGroupItem(const RosterGroupItem&);
    RosterGroupItem& operator=(const RosterGroupItem&);

    std::string m_name;

    void OpenContextMenu();
    //purplemm slots
    void OnAccountConnected(PURPLE::Account account);
    void OnAccountDisconnected(PURPLE::Account account, std::string reason);
    void OnBuddySignedOn(PURPLE::Buddy buddy);
    void OnBuddySignedOff(PURPLE::Buddy buddy);
};

class RosterBuddyItem
: public RosterItem
{
public:
    RosterBuddyItem(PURPLE::Buddy buddy, CppConsUI::TreeView *tree);
    virtual ~RosterBuddyItem() {}

    virtual void OnActivate(CppConsUI::Button& activator);
    void Update();
    std::string GetName() const { return m_buddy.getAlias(); }
    void SetRefNode(CppConsUI::TreeView::NodeReference n) { m_ref = n; }
    CppConsUI::TreeView::NodeReference GetRefNode() const { return m_ref; }
    PURPLE::Buddy GetBuddy() const { return m_buddy; }
protected:
    class AliasDialog
    : public CppConsUI::SplitDialog
    {
        public:
            AliasDialog(PURPLE::Buddy buddy, RosterBuddyItem *parent);
            virtual ~AliasDialog() {}
            
            // FreeWindow
            virtual void OnScreenResized();
        private:
            AliasDialog(const AliasDialog&);
            AliasDialog& operator=(const AliasDialog&);
            
            RosterBuddyItem *m_parent;
            PURPLE::Buddy m_buddy;
            CppConsUI::Button *m_alias;
            CppConsUI::CheckBox *m_serverAlias;
            
            void OnOk(CppConsUI::Button& activator);
            void OnAlias(CppConsUI::Button& activator);
            void OnAliasResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    };
    class MoveDialog
    : public CppConsUI::SplitDialog
    {
        public:
            MoveDialog(PURPLE::Buddy buddy);
            virtual ~MoveDialog() {}
            
            // FreeWindow
            virtual void OnScreenResized();
        private:
            MoveDialog(const MoveDialog&);
            MoveDialog& operator=(const MoveDialog&);
            
            PURPLE::Buddy m_buddy;
            CppConsUI::ComboBox *m_groups;
            
            void OnOk(CppConsUI::Button& activator);
    };
private:
    RosterBuddyItem(const RosterBuddyItem&);
    RosterBuddyItem& operator=(const RosterBuddyItem&);

    PURPLE::Buddy m_buddy;
    time_t m_lastChange;
    CppConsUI::MenuWindow *m_popup;

    void UpdateColorScheme();
    void OpenContextMenu();
    void OnIM(CppConsUI::Button& activator);
    void OnInfo(CppConsUI::Button& activator);
    void OnSendFile(CppConsUI::Button& activator);
    void OnRename(CppConsUI::Button& activator);
    void OnMove(CppConsUI::Button& activator);
    void OnRemove(CppConsUI::Button& activator);
    void OnRemoveResponseHandler(CppConsUI::MessageDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
    void OnBuddyMenuItem(CppConsUI::Button& activator, PURPLE::Buddy::BuddyMenuItem item);
    //purplemm slots
    void OnAccountConnected(PURPLE::Account account);
    void OnBuddySignedOn(PURPLE::Buddy buddy);
    void OnBuddySignedOff(PURPLE::Buddy buddy);
    void OnBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive status);
};

class RosterChatItem
: public RosterItem
{
public:
    RosterChatItem(PURPLE::Chat chat, CppConsUI::TreeView *tree);
    virtual ~RosterChatItem() {}

    virtual void OnActivate(CppConsUI::Button& activator);
    PURPLE::Chat GetChat() const { return m_chat; }
    std::string GetName() const { return m_chat.getAlias(); }
private:
    RosterChatItem(const RosterChatItem&);
    RosterChatItem& operator=(const RosterChatItem&);

    PURPLE::Chat m_chat;
    CppConsUI::MenuWindow *m_popup;

    void OpenContextMenu();
    void OnJoin(CppConsUI::Button& activator);
    void OnModify(CppConsUI::Button& activator);
    void OnRemove(CppConsUI::Button& activator);
    void OnRemoveResponseHandler(CppConsUI::MessageDialog& activator, CppConsUI::AbstractDialog::ResponseType response);
};

#endif	/* ROSTERITEM_H */

