/*
 *      statusdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "statusdialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"
#include <utilsmm.h>

StatusDialog::StatusDialog(PURPLE::Account account)
: SplitDialog(0, 0, 80, 24, _("Set status")), m_account(account)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Account"),
                                                 dxutils::sformat("%s [%s]", account.getUsername().c_str(), account.getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    m_statuses = new CppConsUI::ComboBox(AUTOSIZE, 1, _("Status"));
    std::vector<PURPLE::Account::AccountStatusItem> v = m_account.getStatusMenuItems();
    FOREACH(std::vector<PURPLE::Account::AccountStatusItem>, v, it)
    {
        m_statuses->AddOption((*it).getName().c_str());
    }
    box->AppendWidget(*m_statuses);
    m_msg = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Status message"));
    m_msg->signal_activate.connect(sigc::mem_fun(this, &StatusDialog::OnMsg));
    if(!m_account.getStatusMessage().empty())
        m_msg->SetValue(m_account.getStatusMessage().c_str());
    box->AppendWidget(*m_msg);
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &StatusDialog::OnOk));
}

void StatusDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void StatusDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    std::vector<PURPLE::Account::AccountStatusItem> v = m_account.getStatusMenuItems();
    m_account.setStatus(v[m_statuses->GetSelected()].getID());
    if(m_account.isConnected())
        m_account.setStatusMessage(PURPLE::p2s(m_msg->GetValue()));
    Close();
}

void StatusDialog::OnMsg(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("Status message"), m_msg->GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StatusDialog::OnMsgResponseHandler));
    dialog->Show();
}

void StatusDialog::OnMsgResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_msg->SetValue(activator.GetText());
    }
}
