/*
 * statusmenu.cpp
 *
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "statusmenu.h"
#include "gettext.h"
#include "log.h"
#include "dxim.h"
#include <utilsmm.h>

StatusMenu::StatusMenu()
: MenuWindow(0, 0, AUTOSIZE, AUTOSIZE)
{
    SetColorScheme("generalmenu");
    AppendItem( _("Available"), sigc::bind(sigc::mem_fun(this, &StatusMenu::SetStatus), PURPLE_STATUS_AVAILABLE));
    AppendItem( _("Away"), sigc::bind(sigc::mem_fun(this, &StatusMenu::SetStatus), PURPLE_STATUS_AWAY));
    AppendItem( _("Busy"), sigc::bind(sigc::mem_fun(this, &StatusMenu::SetStatus), PURPLE_STATUS_UNAVAILABLE));
    AppendItem( _("Invisible"), sigc::bind(sigc::mem_fun(this, &StatusMenu::SetStatus), PURPLE_STATUS_INVISIBLE));
    AppendSeparator();
    AppendItem(_("Set status message"), sigc::mem_fun(this, &StatusMenu::SetStatusMsg));
    AppendSeparator();
    AppendItem( _("Offline"), sigc::bind(sigc::mem_fun(this, &StatusMenu::SetStatus), PURPLE_STATUS_OFFLINE));
    AppendSeparator();
    AppendItem(_("Reconnect"), sigc::mem_fun(this, &StatusMenu::Reconnect));
}

void StatusMenu::OnScreenResized()
{
    CppConsUI::Rect chat = DXIM->GetScreenArea(dxim::CHAT_AREA);
    Move(chat.x, chat.y);
}

void StatusMenu::Reconnect(CppConsUI::Button& /*activator*/)
{
    std::vector<PURPLE::Account> accounts = DXIM->GetPurplemm()->getEnabledAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        if(accounts[i].isConnecting())
            continue;
        if(accounts[i].isConnected())
        {
            accounts[i].disconnect();
        }
        accounts[i].connect();
    }
}

void StatusMenu::SetStatus(CppConsUI::Button& /*activator*/, PurpleStatusPrimitive status)
{
    std::vector<PURPLE::Account> accounts = DXIM->GetPurplemm()->getEnabledAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        accounts[i].setStatus(status);
    }
    Close();
}

void StatusMenu::SetStatusMsg(CppConsUI::Button& /*activator*/)
{
    if(!DXIM->GetPurplemm()->isSomeAccountConnected())
    {
        LOG->Warning("[%s] %s", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."));
        return;
    }
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("Change status message"), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StatusMenu::StatusMsgResponseHandler));
    dialog->Show();
}

void StatusMenu::StatusMsgResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        std::vector<PURPLE::Account> accounts = DXIM->GetPurplemm()->getEnabledAccountsList();
        for(guint i=0; i<accounts.size(); i++)
        {
            accounts[i].setStatusMessage(PURPLE::p2s(activator.GetText()));
        }
    }
    Close();
}


