/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef TABBOOK_H
#define TABBOOK_H

#include "tabitem.h"
#include <cppconsui/FreeWindow.h>
#include <cppconsui/HorizontalListBox.h>
#include <cppconsui/Label.h>
#include <cppconsui/Spacer.h>
#include <callback/callback.h>
#include <vector>

#define TABBOOK (TabBook::Instance())

class TabBook
: public CppConsUI::FreeWindow
{
public:
    static TabBook *Instance();
    // FreeWindow
    virtual void OnScreenResized();
    void FocusActiveTabitem();
    void FocusTabitem(int i);
    void FocusPrevTabitem();
    void FocusNextTabItem();
    void SetExpandedTabBook(bool expanded);
    void CloseCurrentTabitem();
    void StopLogging();
    void ClearCurrentTabItem();
    void ClearAllTabItems();
    size_t GetNumTabItems() { return m_items.size(); }
    // tabitem by ID
    int FindTabItem(const std::string &id);
    TabItem *CreateNewItem(const std::string &id);
    void AppendText(const std::string &id, const std::string &text, bool clear = true);
private:
    friend class dxim;
    struct ItemChild
    {
        PURPLE::Conversation *conv;
        TabItem *item;
        CppConsUI::Label *label;
        char typing_status;
    };
    typedef std::vector<ItemChild> TabBookVector;
    TabBookVector m_items;
    // active tabitem, -1 if none
    int m_active;
    CppConsUI::HorizontalListBox *m_outer_list;
    CppConsUI::Spacer *m_left_spacer;
    CppConsUI::Spacer *m_right_spacer;
    CppConsUI::HorizontalListBox *m_item_list;
    static TabBook *m_instance;

    TabBook();
    TabBook(const TabBook&);
    TabBook& operator=(const TabBook&);
    virtual ~TabBook();

    static void Init();
    static void Finalize();
    // find PURPLE::Conversation in tabitem
    int FindConversation(PURPLE::Conversation *conv);
    int PrevActiveTabItem(int current);
    int NextActiveTabItem(int current);
    void ActivateTabitem(int i);
    // update a single tabitem label
    void UpdateLabel(int i);
    // update all tabitem labels
    void UpdateLabels();
    // purplemm slots
    void OnNewTab(PURPLE::Conversation *conv);
    void OnNewMyMsg(PURPLE::Conversation *conv, std::string time, std::string msg, bool action, std::string messageID);
    void OnMessageRecived(PURPLE::Conversation *conv, std::string messageID);
    void OnNewIMMsg(PURPLE::Conversation *conv, std::string time, std::string sender, std::string msg, bool highlight, bool action);
    void OnNewChatMsg(PURPLE::Conversation *conv, std::string time, std::string sender, std::string msg, bool delayed, bool highlight, bool action);
    void OnNewConvSystemMsg(PURPLE::Conversation *conv, std::string time, std::string msg);
    void OnNewConvErrorMsg(PURPLE::Conversation *conv, std::string time, std::string msg);
    void OnBuddyTyping(PURPLE::Conversation *conv, bool typing);
    void OnCloseConvTab(PURPLE::Conversation *conv);
    void OnAccountDisconnected(PURPLE::Account account, std::string);
    void OnAccountConnected(PURPLE::Account account);
};

#endif // TABBOOK_H

