/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef TABITEM_H
#define TABITEM_H

#include "log.h"
#include <cppconsui/AbstractLine.h>
#include <cppconsui/TextEdit.h>
#include <cppconsui/TextView.h>
#include <cppconsui/Window.h>
#include "callback/callback.h"
#include "../logger/logger.h"

enum IMStateType {
    STARTED=0, //started typing
    PAUSED, //paused typing
    STOPPED, //stopped typing
    ACTIVE, //imtabitem is opended or text sent
    LEFTIM, //buddy left IM
};

class TabItem
: public CppConsUI::Window
{
public:
    TabItem(PURPLE::Conversation *conv);
    TabItem(const std::string &id);
    virtual ~TabItem();

    // InputProcessor
    virtual bool ProcessInput(const TermKeyKey& key);
    // Widget
    virtual bool RestoreFocus();
    virtual void UngrabFocus();
    virtual void MoveResize(int newx, int newy, int neww, int newh);
    // FreeWindow
    virtual void Show();
    virtual void Close();
    virtual void OnScreenResized();
    PURPLE::Conversation *GetPurpleConversation() const { return m_conv; }
    std::string GetID() const { return m_id; }
    // For insert msgs
    void AppendMyMsg(std::string time, std::string msg, bool action, std::string messageID);
    void MessageRecived(std::string messageID);
    void AppendMsg(std::string time, std::string sender, std::string msg, bool action, bool highlight, bool delayed);
    void AppendSystemMsg(std::string time, std::string msg, bool log = true);
    void AppendErrorMsg(std::string time, std::string msg);
    // For insert texts
    void AppendText(const std::string &text);
    // Stop logging for preferences change
    void StopLogging();
    void ClearText();
protected:
    CppConsUI::TextView *m_view;
    CppConsUI::TextEdit *m_input;
    PURPLE::Conversation *m_conv;
    Logger m_logger;
    std::vector<std::string> m_commands;
    time_t m_lastKeyPress;
    IMStateType m_imState;
    std::string m_id;

    void StartLogging();
    std::string EscapeText(std::string text);
    void ProcessCommandReturn(int ret, const std::string &command);
    void ActionSend();
    void ActionClear();
    void DeclareBindables();
    void ShowInternalHelp(const std::string &command);
    void CompleteHelpCommand();
    virtual PURPLE::ChatBuddy GetChatBuddy(std::string name) { return PURPLE::ChatBuddy(); }
private:
    TabItem();
    TabItem(const TabItem&);
    TabItem& operator=(const TabItem&);
};

#endif // TABITEM_H


