/*
 *      FXTrayApp.cpp
 *
 *      Copyright (c) 2008, Hendrik Rittich
 *      Copyright (C) 2006-2011 by Sander Jansen
 *      Copyright 2012 David Vachulka <david@konstrukce-cad.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <xincs.h>
#include "FXTrayApp.h"
#include "FXTrayIcon.h"

namespace FX {

FXIMPLEMENT(FXTrayApp, FXApp, NULL, 0);

FXTrayApp::FXTrayApp(const FXString& name, const FXString& vendor) :
    FXApp(name, vendor)
{
#ifndef WIN32
    mXembed = 0;
#endif
}

void FXTrayApp::create()
{
    FXApp::create();
#ifdef WIN32
    mTaskbarRestartId = RegisterWindowMessage(TEXT("TaskbarCreated"));
#else
    FXString systemtray = FXStringFormat("_NET_SYSTEM_TRAY_S%d",DefaultScreen((Display*)getDisplay()));
    mXembed = (FXID)XInternAtom((Display*)getDisplay(),"_XEMBED",False);
    mXmanager = (FXID)XInternAtom((Display*)getDisplay(),"MANAGER",True);
    mXsystemtray = (FXID)XInternAtom((Display*)getDisplay(),systemtray.text(),True);
    XSelectInput((Display*)getDisplay(),getRootWindow()->id(),KeyPressMask|KeyReleaseMask|StructureNotifyMask);
#endif
}

#ifdef WIN32
long FXTrayApp::dispatchEvent(FXID hwnd,unsigned int iMsg,unsigned int wParam,long lParam)
{
    // return TRUE; // event dispached successful
    if (iMsg == mTaskbarRestartId)
    {
        FXWindow* wnd = findWindowWithId(hwnd);
        if (wnd && wnd->isMemberOf(&FXTrayIcon::metaClass) ) {
            static_cast<FXTrayIcon*>(wnd)->mapToManager();
        }

        return TRUE;
    }

    return FXApp::dispatchEvent(hwnd, iMsg, wParam, lParam);
}
#else
bool FXTrayApp::dispatchEvent(FXRawEvent& ev)
{
    if (ev.xany.window == getRootWindow()->id() && ev.xany.type == ClientMessage)
    {
        if ((Atom) ev.xclient.message_type == (Atom) mXmanager && (Atom) ev.xclient.data.l[1] == (Atom) mXsystemtray)
        {
            if(mIcon)
                mIcon->dock();
            return true;
        }
    }

    FXWindow * window = findWindowWithId(ev.xany.window);
    if (window && ev.xany.type == ClientMessage && ev.xclient.message_type == mXembed)
    {
        switch (ev.xclient.data.l[1])
        {
        case XEMBED_EMBEDDED_NOTIFY: window->tryHandle(this, FXSEL(SEL_EMBED_NOTIFY, 0), (void*) (FXival) ev.xclient.data.l[3]);
            break;
        case XEMBED_MODALITY_ON: window->tryHandle(this, FXSEL(SEL_EMBED_MODAL_ON, 0), NULL);
            break;
        case XEMBED_MODALITY_OFF: window->tryHandle(this, FXSEL(SEL_EMBED_MODAL_OFF, 0), NULL);
            break;
        default: /*fxmessage("Missed a message %d\n",ev.xclient.data.l[1]);*/ break;
        }
        return true;
    }
    return FXApp::dispatchEvent(ev);
}

void FXTrayApp::registerIcon(FXTrayIcon* ic)
{
    mIcon = ic;   
}

void FXTrayApp::unregisterIcon()
{
    mIcon = NULL;
}
#endif
}
