/*
 *      FXTrayIcon.h
 *
 *      Copyright (c) 2008, Hendrik Rittich
 *      Copyright (C) 2006-2011 by Sander Jansen
 *      Copyright 2012 David Vachulka <david@konstrukce-cad.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef FX_TRAY_ICON_H
#define FX_TRAY_ICON_H

#include "fx.h"
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>
#endif

namespace FX {

enum {
    XEMBED_EMBEDDED_NOTIFY = 0,
    XEMBED_MODALITY_ON = 10,
    XEMBED_MODALITY_OFF = 11,
    XEMBED_REQUEST_FOCUS = 3
};

enum {
    TRAY_MENU_ON_LEFT = 0x0001,
    TRAY_MENU_ON_RIGHT = 0x0002,
    TRAY_CMD_ON_LEFT = 0x0004,
    TRAY_CMD_ON_RIGHT = 0x0008
};

#ifdef WIN32
class FXTrayIcon : public FXTopWindow
{
    FXDECLARE(FXTrayIcon)
public:
    enum {
        ID_POPTIMEOUT = FXTopWindow::ID_LAST,
        ID_LAST
    };
    FXTrayIcon(FXApp* app, const FXString& text, FXIcon* icon,
        FXPopup* pup = NULL, FXObject* target = 0, FXSelector sel = 0,
        FXuint opts = TRAY_MENU_ON_LEFT);
    ~FXTrayIcon();

    void create();

    void mapToManager();

    long onEvent(FXObject*, FXSelector, void* ptr);
    long onTimeout(FXObject*, FXSelector, void*);

    void setMenu(FXPopup* pup) { m_popup = pup; }
    FXPopup* getMenu() const { return m_popup; }

    void setIcon(FXIcon* icon);
    inline FXIcon* getIcon() const { return mIcon; }

    void setText(const FXString& text);
    inline const FXString& getText() const { return mTooltip; }

    static HICON createMswIcon(FXIcon* icon);

    FXbool getOpaque() const { return false; }

    void setTrayColor(FXColor) {}
private:
    HICON mWIcon;
    FXIcon* mIcon;
    DWORD mTrayID;

    FXPopup* m_popup;
    FXuint m_opts;
    FXString mTooltip;

    static DWORD sTrayIconCount;

    FXTrayIcon()
    {}

    void setupNotifyData(NOTIFYICONDATA* data);
};
#else
class FXTrayIcon : public FXTopWindow
{
    FXDECLARE(FXTrayIcon)
private:
    FXTrayIcon(const FXTrayIcon*);
    FXTrayIcon& operator=(const FXTrayIcon&);

    void popup(FXint x, FXint y);
protected:
    FXID m_xtraywindow;
    FXID m_xtrayopcode;
    FXID m_xtrayorientation;
    FXID m_xtrayxfceorientation;
    FXID m_xtrayvisual;
    FXID m_socket;
    FXIcon *m_icon;
    FXbool m_opaque;
    FXString m_tip;
    FXPopup *m_popup;
    FXuint m_opts;
    FXColor m_traycolor;

    FXTrayIcon();

    FXbool findSystemTray();
    void requestDock();
    FXuint getTrayOrientation();
    FXuint getTrayVisual();
    virtual bool doesOverrideRedirect() const;
public:
    enum {
        ID_POPTIMEOUT = FXTopWindow::ID_LAST,
        ID_LAST
    };
    FXTrayIcon(FXApp* app, const FXString& text, FXIcon* icon,
            FXPopup* popup = NULL, FXObject* target = 0, FXSelector sel = 0,
            FXuint opts = TRAY_MENU_ON_RIGHT);
    virtual ~FXTrayIcon();

    long onPaint(FXObject*, FXSelector, void*);
    long onConfigure(FXObject*, FXSelector, void*);
    long onLeftBtnPress(FXObject*, FXSelector, void*);
    long onRightBtnRelease(FXObject*, FXSelector, void*);
    long onQueryTip(FXObject*, FXSelector, void*);
    long onEmbedded(FXObject*, FXSelector, void*);
    long onTimeout(FXObject*, FXSelector, void*);

    virtual void create();
    FXbool dock();
    virtual void setFocus();
    /** Change the tool tip text. */
    void setText(const FXString & t) {
        m_tip = t;
    }
    /** Set the popup menu. */
    void setMenu(FXPopup *p) {
        m_popup = p;
    }
    /** Change the icon in the tray. */
    void setIcon(FXIcon *ic);
    /** Get the icon. */
    FXIcon* getIcon() const;
    /** Is windows opaque? */
    FXbool getOpaque() const { return m_opaque; }
    /** Set tray color, usefull for opaque window */
    void setTrayColor(FXColor color);
};
#endif
}

#endif
