/*
 *      callback.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXFCALLBACKS_H
#define DXFCALLBACKS_H

#include <fx.h>
#include <purplemm.h>
#include "dxmessagechannel.h"

struct AccountEvent {
    PURPLE::Account account;
    FXString msg;
};

struct AccountRemoved {
    FXString account;
    FXString protocolID;
};

struct NewMsg {
    PURPLE::Conversation *conv;
    FXString sender;
    FXString msg;
    FXTime time; //event time
    bool history; //event from log or stored on server
    bool highlight; //msg contains my nick
    bool action; //msg starts with /me
    FXString messageID; //message ID, usefull for XEP-0184
};

struct NewSystemMsg {
    PURPLE::Conversation *conv;
    FXString msg;
    FXTime time; //event time
};

struct BuddyStatus {
    PURPLE::Buddy buddy;
    PurpleStatusPrimitive oldStatus;
    PurpleStatusPrimitive newStatus;
};

struct BuddyTyping {
    PURPLE::Conversation *conv;
    bool typing;
};

struct ChatTopic {
    PURPLE::Conversation *conv;
    FXString who;
    FXString topic;
};

struct RoomlistColumns {
    PURPLE::RoomList *list;
    std::vector<std::string> columns;
};

struct Room {
    PURPLE::RoomList *list;
    std::vector<std::string> fields;
};

struct UserInfo {
    PURPLE::Account account;
    FXString buddy;
    std::vector<PURPLE::Notify::UserInfo> info;
};

struct ChatBuddies {
    PURPLE::Conversation conv;
    std::vector<PURPLE::ChatBuddy> buddies;
};

enum ChatBuddyAction {
    CBJOINED = 0,
    CBUPDATED,
    CBLEFT,
    CBRENAMED
};

struct ChatBuddyEvent {
    PURPLE::Conversation *conv;
    PURPLE::ChatBuddy buddy;
    ChatBuddyAction action;
    FXString who; //for CBLEFT and CBRENAMED
};

struct NodeRemoved {
    PURPLE::Account account;
    FXString who;
};

enum ProtocolEventType {
    // Buddy list loaded
    DX_BUDDYLOADED = FX::SEL_LAST+13,
    //Account added
    //data = PURPLE::Account&
    DX_NEWACCOUNT,
    //Account removed
    //data = AccountRemoved
    DX_ACCOUNTREMOVED,
    //Connecting to server
    //data = AccountEvent
    DX_CONNECTING,
    //Connected to server
    //data = PURPLE::Account&
    DX_CONNECTED,
    //Disconneted from server
    //data = AccountEvent
    DX_DISCONNECTED,
    //Signed on, show buddies
    //data = PURPLE::Account&
    DX_SIGNEDON,
    //Account status changed
    //data = PURPLE::Account&
    DX_ACCOUNTPRESENCE,
    //Buddy signed on
    //data = PURPLE::Buddy&
    DX_BUDDYSIGNEDON,
    //Buddy signed off
    //data = PURPLE::Buddy&
    DX_BUDDYSIGNEDOFF,
    //buddy change presence
    //data = BuddyStatus
    DX_BUDDYPRESENCE,
    //buddy change icon
    //data = PURPLE::Buddy&
    DX_BUDDYICON,
    //Buddy added
    //data = PURPLE::Buddy&
    DX_NEWBUDDY,
    //Buddy removed
    //data = NodeRemoved
    DX_BUDDYREMOVED,
    //Chat added
    //data = PURPLE::Chat&
    DX_NEWCHAT,
    //Chat removed
    //data = NodeRemoved
    DX_CHATREMOVED,
    //New im or chat
    //data = PURPLE::Conversation&
    DX_NEWTAB,
    //Close im or chat
    //data = PURPLE::Conversation&
    DX_CLOSETAB,
    //New message arrived
    //data = NewMsg*
    DX_NEWMSG,
    //New my message sent
    //data = NewMsg*
    DX_NEWMYMSG,
    //Message recived
    //data = NewMsg*
    DX_MESSAGERECIVED,
    //New system conv message arrived
    //data = NewSystemMsg*
    DX_NEWSYSTEMMSG,
    //New error conv message arrived
    //data = NewSystemMsg*
    DX_NEWERRORMSG,
    //Chat topic changed
    //data = ChatTopic*
    DX_CHATTOPIC,
    //New roomlist
    //data = PURPLE::RoomList&
    DX_NEWROOMLIST,
    //RoomList columns
    //data = RoomlistColumns
    DX_ROOMLISTCOLUMNS,
    //Room of RoomList
    //data = Room
    DX_ROOMLISTROOM,
    //RoomList finished
    //data = PURPLE::RoomList&
    DX_ROOMLISTFINISHED,
    //New InputRequest
    //data = PURPLE::InputRequest
    DX_INPUTREQUEST,
    //New ChoiceRequest
    //data = PURPLE::ChoiceRequest
    DX_CHOICEREQUEST,
    //New ActionRequest
    //data = PURPLE::ActionRequest
    DX_ACTIONREQUEST,
    //New FileRequest
    //data = PURPLE::FileRequest
    DX_FILEREQUEST,
    //New FieldsRequest
    //data = PURPLE::FieldsRequest
    DX_FIELDSREQUEST,
    //New file transfer
    //data = PURPLE::FileTransfer
    DX_NEWFT,
    //file transfer started
    //data = PURPLE::FileTransfer
    DX_STARTEDFT,
    //file transfer finished
    //data = PURPLE::FileTransfer
    DX_FINISHEDFT,
    //file transfer canceled
    //data = PURPLE::FileTransfer
    DX_CANCELEDFT,
    //file transfer progress
    //data = PURPLE::FileTransfer
    DX_PROGRESSFT,
    //New notify message, new email etc.
    //data = MsgInfo
    DX_NEWNOTIFYMSG,
    //New ChatBuddyEvent
    //data = ChatBuddyEvent
    DX_CHATBUDDY,
    //UserInfo
    //data = UserInfo
    DX_USERINFO,
    //Buddy typing
    //data = BuddyTyping
    DX_BUDDYTYPING
};

class dxfCallbacks: public PURPLE::Callbacks
{
    friend gpointer purpleMainEventLoop(gpointer data);
public:
    dxfCallbacks(FXApp *app, FXObject *target, bool offline=false);
    virtual ~dxfCallbacks();
    FXMessageChannel m_feedback;

    PURPLE::Purple *getPurplemm();
    /** Allow your application to be notified about errors on library layer */
    void showError(std::string msg);
    /** Allow your application to be notified about libpurple debug messages */
    void showDebug(std::string debug);
    /** Buddy list loaded */
    void buddylistLoaded();
    /** New account added */
    void accountAdded(PURPLE::Account account);
    /** Account removed */
    void accountRemoved(std::string account, std::string protocolID);
    /** Account connecting */
    void accountConnecting(PURPLE::Account account, std::string msg);
    /** Acount connected*/
    void accountConnected(PURPLE::Account account);
    /** Acount disconnected*/
    void accountDisconnected(PURPLE::Account account, std::string reason);
    /** Account status changed*/
    void accountStatusChanged(PURPLE::Account account);
    /** Account signed on, so show buddy list */
    void accountSignedOn(PURPLE::Account account);
    /** Buddy signed on */
    void buddySignedOn(PURPLE::Buddy buddy);
    /** Buddy signed off */
    void buddySignedOff(PURPLE::Buddy buddy);
    /** Buddy status changed */
    void buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus);
    /** Buddy icon changed */
    void buddyIconChanged(PURPLE::Buddy buddy);
    /** Buddy added */
    void buddyAdded(PURPLE::Buddy buddy);
    /** Buddy removed */
    void buddyRemoved(PURPLE::Account account, std::string buddy);
    /** Chat added */
    void chatAdded(PURPLE::Chat chat);
    /** Chat removed */
    void chatRemoved(PURPLE::Account account, std::string chat);
    /** New IM created */
    void newIM(PURPLE::Conversation *conv);
    /** New Chat created */
    void newChat(PURPLE::Conversation *conv);
    /** IM closed */
    void closedIM(PURPLE::Conversation *conv);
    /** Chat closed */
    void closedChat(PURPLE::Conversation *conv);
    /** New my message
     * @param conv          The conversation.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param action        Is message in 3rd person (/me)?
     * @param messageID     message ID, usefull for XEP-0184
     */
    void newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID);
    /** Message recived, usefull for XEP-0184
     * @param conv          The conversation.
     * @param messageID     message ID, usefull for XEP-0184
     */
    void messageRecived(PURPLE::Conversation *conv, std::string messageID);
    /** New IM message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action);
    /** New Chat message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action);
    /** New system mesage for conversation
     *
     * @param conv  The conversation
     * @param msg  The message text
     * @param time  The time the message was sent
     */
    void newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time);
    /** New error mesage for conversation
     *
     * @param conv      conversation
     * @param msg       error text
     * @param time      time the error was sent
     */
    void newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time);
    /** ChatBuddy list fired after chat joined*/
    void chatBuddies(PURPLE::Conversation conv, std::vector<PURPLE::ChatBuddy> buddies);
    /** ChatBuddy joined chat */
    void chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy);
    /** ChatBuddy updated */
    void chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy);
    /** ChatBuddy left chat */
    void chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy);
    /** ChatBuddy renamed
     * @param who       old buddy which was renamed
     * @param nbuddy    new buddy established by rename
     */
    void chatBuddyRenamed(PURPLE::Conversation *conv, std::string who, PURPLE::ChatBuddy nbuddy);
    /** Chat topic changed
     * @param who  topic changer
     * @param topic  new topic
     */
    void chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic);
    /** Buddy typing on IM
     * @param typing =true typing, =false stoped typing
     */
    void buddyTyping(PURPLE::Conversation *conv, bool typing);
    /** New FileTransfer */
    void newFileTransfer(PURPLE::FileTransfer *ft);
    /** FileTransfer finished */
    void fileTransferFinished(PURPLE::FileTransfer *ft);
    /** FileTransfer started */
    void fileTransferStarted(PURPLE::FileTransfer *ft);
    /** FileTransfer progress */
    void fileTransferProgress(PURPLE::FileTransfer *ft, double percent);
    /** FileTransfer canceled */
    void fileTransferCanceled(PURPLE::FileTransfer *ft);
    /** A new list was created */
    void roomlistCreated(PURPLE::RoomList *rlist);
    /** Sets the columns of RoomList */
    void roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns);
    /** Add a room to the list */
    void roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields);
    /** Listing RoomList finished */
    void roomlistFinished(PURPLE::RoomList *rlist);
    /** Notification message to the user
     * @param type  notification type
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
    */
    void notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary);
    /** Email notification to the user
     * @param subject  subject of the email
     * @param from  from address
     * @param to  estination address
     * @param url  URL where the message can be read
    */
    void notifyEmail(std::string subject, std::string from, std::string to, std::string url);
    /** Notification with formatted text
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
     * @param text  formatted text
    */
    void notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated);
    /** Notification information about buddy
     * @param account   requested account
     * @param buddy     buddy which about
     * @param info      vector with UserInfo values
    */
    void notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info);
    /** Opens a URI or somehow presents it to the user
     * @param uri    The URI to display or go to.
    */
    void notifyURI(std::string uri);
    /** Play sound for given event */
    void playSound(PurpleSoundEventID event);
    /** New InputRequest */
    void newInputRequest(PURPLE::InputRequest *request);
    /** New ChoiceRequest */
    void newChoiceRequest(PURPLE::ChoiceRequest *request);
    /** New ActionRequest */
    void newActionRequest(PURPLE::ActionRequest *request);
    /** New FileRequest */
    void newFileRequest(PURPLE::FileRequest *request);
    /** New FieldsRequest */
    void newFieldsRequest(PURPLE::FieldsRequest *request);
    /** Close given request (ex. dialog) */
    void closeRequest(PURPLE::BasicRequest *request);
private:
    PURPLE::Purple *m_purple;
    FXObject *m_target;

    void emitEvent(FXObject *tgt, FXuint type, FXuint id, const void* data, FXint size);
};

#endif // DXFCALLBACKS_H

