/*
 *      accountdialog.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "accountdialog.h"
#include "config.h"
#include "i18n.h"
#include "accounteditor.h"
#include "dxutils.h"
#include "icons.h"
#include "preferences.h"

FXDEFMAP(AccountDialog) AccountDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND,          AccountDialog_ADD,       AccountDialog::onAdd),
    FXMAPFUNC(SEL_COMMAND,          AccountDialog_MODIFY,    AccountDialog::onModify),
    FXMAPFUNC(SEL_COMMAND,          AccountDialog_DELETE,    AccountDialog::onDelete),
    FXMAPFUNC(SEL_COMMAND,          AccountDialog_ENABLE,    AccountDialog::onEnable),
    FXMAPFUNC(SEL_KEYPRESS,         0,                       AccountDialog::onKeyPress),
    FXMAPFUNC(SEL_CHANGED,          AccountDialog_TABLE,     AccountDialog::onTableChanged)
};

FXIMPLEMENT(AccountDialog, FXDialogBox, AccountDialogMap, ARRAYNUMBER(AccountDialogMap))

AccountDialog::AccountDialog(FXMainWindow *owner, PURPLE::Purple *purple)
    : FXDialogBox(owner, _("Accounts list"), DECOR_ALL, 0,0,0,0, 0,0,0,0, 0,0), m_purple(purple)
{
    FXVerticalFrame *contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);

    FXVerticalFrame *tableframe = new FXVerticalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK);
    m_table = new FXTable(tableframe, this, AccountDialog_TABLE, TABLE_COL_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 2,2,2,2);
    m_table->setVisibleRows(5);
    m_table->setVisibleColumns(3);
    m_table->setTableSize(purple->getAccountsList().size(), 3);
    m_table->setColumnText(0, _("User name"));
    m_table->setColumnText(1, _("Protocol"));
    m_table->setColumnText(2, _("Enabled"));
    m_table->setEditable(FALSE);
    m_table->showHorzGrid(FALSE);
    m_table->showVertGrid(FALSE);
    m_table->setRowHeaderWidth(0);

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X);
    m_buttonClose = new dxEXButton(buttonframe, _("&Close"), NULL, this, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonDelete = new dxEXButton(buttonframe, _("&Delete"), NULL, this, AccountDialog_DELETE, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonDelete->disable();
    m_buttonModify = new dxEXButton(buttonframe, _("&Modify"), NULL, this, AccountDialog_MODIFY, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonModify->disable();
    m_buttonAdd = new dxEXButton(buttonframe, _("&Add"), NULL, this, AccountDialog_ADD, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonEnable = new dxEXButton(buttonframe, _("&Enable"), NULL, this, AccountDialog_ENABLE, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonEnable->disable();

    updateTable();
}

AccountDialog::~AccountDialog()
{
}

long AccountDialog::onAdd(FXObject*,FXSelector,void*)
{
    FXDialogBox protocolDialog(this, _("Select protocol"), DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    FXVerticalFrame *contents = new FXVerticalFrame(&protocolDialog, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    FXHorizontalFrame *listframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new FXLabel(listframe, _("Protocol:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_X);
    FXListBox *protocols = new FXListBox(listframe, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    std::vector<PURPLE::Protocol> v = m_purple->getProtocolsList();
    FOREACH(std::vector<PURPLE::Protocol>, v, it)
    {
        protocols->appendItem(dxutils::fromStdString(it->getName()), dxutils::getProtocolIcon(dxutils::fromStdString(it->getPurpleID())));
    }
    protocols->setSortFunc(FXList::ascendingCase);
    protocols->sortItems();
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &protocolDialog, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonframe, _("&OK"), NULL, &protocolDialog, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    if(protocolDialog.execute(PLACEMENT_OWNER))
    {
        FOREACH(std::vector<PURPLE::Protocol>, v, it)
        {
            if(!it->getName().compare(protocols->getItemText(protocols->getCurrentItem()).text()))
            {
                AccountEditor editor(this, *it);
                if(editor.execute(PLACEMENT_OWNER))
                {
                    NewAccount *account = new NewAccount;
                    editor.fillAccount(account);
                    PURPLE::Account acc = m_purple->addAccount(account->protocol, account->username, account->password, account->rememberPassword, account->options, account->registerAccount);
                    acc.setCheckMail(account->checkEmail);
                    acc.setAlias(account->alias);
                    if(!account->iconPath.empty()) acc.setBuddyIcon(account->iconPath);
                    updateTable();
                }
                return 1;
            }
        }
        if(protocols->getItemText(protocols->getCurrentItem()) == "Google Talk")
        {
            AccountEditor editor(this, m_purple->getProtocolById("prpl-jabber"), true);
            if(editor.execute(PLACEMENT_OWNER))
            {
                NewAccount *account = new NewAccount;
                editor.fillAccount(account);
                PURPLE::Account acc = m_purple->addAccount(account->protocol, account->username, account->password, account->rememberPassword, account->options, account->registerAccount);
                acc.setCheckMail(account->checkEmail);
                acc.setAlias(account->alias);
                if(!account->iconPath.empty()) acc.setBuddyIcon(account->iconPath);
                updateTable();
            }
            return 1;
        }
    }
    return 1;
}

long AccountDialog::onModify(FXObject*,FXSelector,void*)
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        if(i == (FXuint)m_table->getCurrentRow())
        {
            AccountEditor editor(this, accounts[i]);
            if(editor.execute(PLACEMENT_OWNER))
            {
                NewAccount *account = new NewAccount;
                editor.fillAccount(account);
                accounts[i].setOptions(account->options);
                accounts[i].setPassword(account->password, account->rememberPassword);
                accounts[i].setCheckMail(account->checkEmail);
                accounts[i].setAlias(account->alias);
                account->iconPath.empty()?accounts[i].removeBuddyIcon():accounts[i].setBuddyIcon(account->iconPath);
                updateTable();
            }
            return 1;
        }
    }
    return 1;
}

long AccountDialog::onDelete(FXObject*,FXSelector,void*)
{
    if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Remove account"), _("Do you want really remove %s?"), m_table->getItemText(m_table->getCurrentRow(), 0).text()) == 1)
    {
        std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
        for(FXuint i=0; i<accounts.size(); i++)
        {
            if(i == (FXuint)m_table->getCurrentRow())
            {
                m_purple->deleteAccount(accounts[i]);
            }
        }
        updateTable();
    }
    return 1;
}

long AccountDialog::onKeyPress(FXObject *sender,FXSelector sel,void *ptr)
{
    if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),NULL);
        return 1;
    }
    return 0;
}

long AccountDialog::onTableChanged(FXObject*, FXSelector, void *ptr)
{
    FXTablePos *tp = (FXTablePos*)ptr;
    if(m_table->getItemText(tp->row, 2)==_("Yes"))
        m_buttonEnable->setText(_("Di&sable"));
    else
        m_buttonEnable->setText(_("&Enable"));
    return 1;
}

long AccountDialog::onEnable(FXObject*,FXSelector,void*)
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        if(i == (FXuint)m_table->getCurrentRow())
        {
            if(m_table->getItemText(m_table->getCurrentRow(), 2)==_("Yes"))
            {
                accounts[i].disconnect();
                m_buttonEnable->setText(_("&Enable"));
            }
            else
            {
                accounts[i].connect();
                m_buttonEnable->setText(_("Di&sable"));
            }
        }
    }
    updateTable();
    return 1;
}

FXbool AccountDialog::accountExist(const FXString &id, const FXString &protocol)
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        if(!compare(id, accounts[i].getUsername().c_str())
                &&!compare(protocol, accounts[i].getProtocol().getPurpleID().c_str()))
            return TRUE;
    }
    return FALSE;
}

void AccountDialog::updateTable()
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    if(accounts.size())
    {
        if((FXuint)m_table->getNumRows() < accounts.size())
            m_table->insertRows(0, accounts.size()-m_table->getNumRows());
        if((FXuint)m_table->getNumRows() > accounts.size())
            m_table->removeRows(0, m_table->getNumRows()-accounts.size());
        for(FXuint i=0; i<accounts.size(); i++)
        {
            m_table->setItemIconPosition(i, 0, FXTableItem::BEFORE);
            m_table->setItemJustify(i, 0, FXTableItem::LEFT);
            m_table->setItemIcon(i, 0, makeScaledIcon(getApp(), dxutils::fromStdString(accounts[i].getBuddyIcon()), 16, DXPREFERENCES.m_appTheme.back));
            m_table->setItemText(i, 0, dxutils::fromStdString(accounts[i].getUsername()));
            m_table->setItemIconPosition(i, 1, FXTableItem::BEFORE);
            m_table->setItemJustify(i, 1, FXTableItem::LEFT);
            m_table->setItemIcon(i, 1, dxutils::getProtocolIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID())));
            m_table->setItemText(i, 1, dxutils::fromStdString(accounts[i].getProtocol().getName()));
            m_table->setItemJustify(i, 2, FXTableItem::LEFT);
            m_table->setItemText(i, 2, accounts[i].isEnabled()? _("Yes") : _("No"));
        }
        m_buttonDelete->enable();
        m_buttonModify->enable();
        m_buttonEnable->enable();
        m_table->selectRow(0);
        m_table->setCurrentItem(0,0,true);
    }
    else
    {
        m_buttonDelete->disable();
        m_buttonModify->disable();
        m_buttonEnable->disable();
    }
}


