/*
 *      accounteditor.h
 *
 *      Copyright 2010 David Schwarz <david.schwarz@live.com>
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ACCOUNT_EDITOR_H
#define ACCOUNT_EDITOR_H

#include "defs.h"
#include "fxext.h"
#include "tabitems/dxtabbook.h"

typedef FXArray<FXTextField*> dxUsernameArray;
typedef FXArray<FXWindow*> dxOptionsArray;

struct NewAccount {
    PURPLE::Protocol protocol;
    PURPLE::Protocol::Options options;
    std::string username;
    std::string password;
    std::string iconPath;
    bool registerAccount;
    bool checkEmail;
    bool rememberPassword;
    std::string alias;
};

class AccountEditor : public FXDialogBox
{
    FXDECLARE(AccountEditor);
private:
    PURPLE::Protocol m_protocol; //use for creating
    PURPLE::Account m_account; //use for modifying
    PURPLE::Protocol::Options m_options;
    dxTabBook *m_tabbook;
    dxEXButton *m_buttonOK, *m_buttonCancel, *m_buddyIcon, *m_removeBuddyIcon;
    FXTextField *m_password, *m_alias;
    FXCheckButton *m_registerAccount, *m_checkEmail, *m_rememberPassword;
    bool m_editing;
    bool m_gtalk;
    dxUsernameArray m_usernameFields;
    dxOptionsArray m_optionsFields;
    FXString m_buddyIconPath;

    AccountEditor() { }
    AccountEditor(const AccountEditor &);
    void initializeEditor();
    void createAccountTab();
    void createProtocolTab();
    std::string getStrlistValue(std::map<std::string, std::string> map, std::string key);
    void fillProtocolOptions();
    std::string joinUsername();
public:
    /* Constructor for modifying account */
    AccountEditor(FXWindow *, PURPLE::Account);
    /* Constructor for creating new account */
    AccountEditor(FXWindow *, PURPLE::Protocol, bool gtalk=false);
    virtual ~AccountEditor();

    /* Handles modifying & creating account */
    long onSave(FXObject *, FXSelector, void *);
    /* Handles cancelation of modifying or creating account */
    long onCancel(FXObject *, FXSelector, void *);
    /* Handles keypress actions */
    long onKeyPress(FXObject *, FXSelector, void *);
    /* Handles mouse wheel for tabbook */
    long onMouseWheel(FXObject *, FXSelector, void *);
    /* Handles click on m_buddyIcon */
    long onBuddyIcon(FXObject *, FXSelector, void *);
    /* Handles click on m_removeBuddyIcon */
    long onRemoveBuddyIcon(FXObject *, FXSelector, void *);

    /* Fills new Account */
    void fillAccount(NewAccount *account);
};

#endif
