/*
 *      actionrequestdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "actionrequestdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"

FXDEFMAP(ActionRequestDialog) ActionRequestDialogMap[] = {
    FXMAPFUNCS(SEL_COMMAND, ActionRequestDialog_BUTTON, ActionRequestDialog_BUTTON+10, ActionRequestDialog::onButtons)
};

FXIMPLEMENT(ActionRequestDialog, FXDialogBox, ActionRequestDialogMap, ARRAYNUMBER(ActionRequestDialogMap))

ActionRequestDialog::ActionRequestDialog(FXMainWindow *owner, PURPLE::ActionRequest *request)
: FXDialogBox(owner, request->getTitle().empty() ? _("New request") : dxutils::fromStdString(request->getTitle()),
        DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0), m_request(request)
{
    FXVerticalFrame *contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    new FXLabel(contents, dxutils::fromStdString(m_request->getPrimary()), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    if(!m_request->getSecondary().empty()) new FXLabel(contents, dxutils::fromStdString(m_request->getSecondary()), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXHorizontalFrame *buttons = new FXHorizontalFrame(contents, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    std::map<std::string, PurpleRequestActionCb> callbacks = m_request->getCallbacks();
    std::map<std::string, PurpleRequestActionCb>::iterator it;
    int i;
    for(it=callbacks.begin(),i=0; it != callbacks.end(); it++,i++)
    {
        if(i == m_request->getDefaultAction())
        {
            dxEXButton *button = new dxEXButton(buttons, dxutils::fromStdString((*it).first).substitute('_', '&'), 0, this, ActionRequestDialog_BUTTON+i, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X, 0,0,0,0, 10,10,2,2);
            button->setFocus();
        }
        else
            new dxEXButton(buttons, dxutils::fromStdString((*it).first).substitute('_', '&'), 0, this, ActionRequestDialog_BUTTON+i, FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X, 0,0,0,0, 10,10,2,2);
    }
}

ActionRequestDialog::~ActionRequestDialog() {
}

long ActionRequestDialog::onButtons(FXObject*, FXSelector sel, void*) {
    FXuint which = FXSELID(sel)-ActionRequestDialog_BUTTON;
    std::map<std::string, PurpleRequestActionCb> callbacks = m_request->getCallbacks();
    std::map<std::string, PurpleRequestActionCb>::iterator it;
    FXuint i;
    for(it=callbacks.begin(),i=0; it != callbacks.end(); it++,i++)
    {
        if(i == which)
        {
            m_request->runCallback((*it).second);
        }
    }
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

