/*
 *      chatdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "chatdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include <sstream>

template<typename T>
/** String from someone*/
std::string t2s( const T & Value )
{
    std::ostringstream oss;
    oss << Value;
    return oss.str();
}

FXDEFMAP(ChatDialog) ChatDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, ChatDialog_OK, ChatDialog::onOK)
};

FXIMPLEMENT(ChatDialog, FXDialogBox, ChatDialogMap, ARRAYNUMBER(ChatDialogMap))

ChatDialog::ChatDialog(FXMainWindow* owner, PURPLE::Account account)
: FXDialogBox(owner, _("Add chat"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_account = account;
    m_chat = PURPLE::Chat();
    m_modifing = false;
    createWidgets();
}

ChatDialog::ChatDialog(FXMainWindow* owner, PURPLE::Chat chat)
: FXDialogBox(owner, _("Modify chat"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_account = chat.getAccount();
    m_chat = chat;
    m_modifing = true;
    createWidgets();
}

ChatDialog::~ChatDialog()
{
}

void ChatDialog::createWidgets()
{
    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    m_matrix = new FXMatrix(m_contents, 2, MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
    new FXLabel(m_matrix, _("Account:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    new FXLabel(m_matrix, dxutils::fromStdString(m_account.getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(m_account.getProtocol().getPurpleID())), ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    new FXLabel(m_matrix, _("Group:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_groups = new FXComboBox(m_matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    new FXLabel(m_matrix, _("Alias:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_alias = new FXTextField(m_matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    if(m_modifing) m_alias->setText(dxutils::fromStdString(m_chat.getAlias()));
    std::vector<PURPLE::Account::ChatEntry> fields;
    if(m_modifing) fields = m_chat.getChatParameters();
    else fields = m_account.getChatParameters();
    FXTextField *text;
    FXSpinner *spinner;
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        new FXLabel(m_matrix, dxutils::normalizeLabelString(entry.getLabel(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        if(entry.getIsInt())
        {
            spinner = new FXSpinner(m_matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            spinner->setRange(entry.getMin(), entry.getMax());
            m_chatEntryFields.append(spinner);
        }
        else
        {
            if(entry.getSecret()) text = new FXTextField(m_matrix, 30, NULL, 0, TEXTFIELD_PASSWD|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            else text = new FXTextField(m_matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            text->setText(dxutils::fromStdString(entry.getValue()));
            m_chatEntryFields.append(text);
        }
    }
    m_autoJoin = new FXCheckButton(m_matrix, _("Autojoin chat"));
    if(m_modifing) m_autoJoin->setCheck(m_chat.getAutojoin());
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, _("&OK"), NULL, this, ChatDialog_OK, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    FXString groups = FXString::null;
    std::vector<std::string> v = m_account.getAllGroups();
    FOREACH(std::vector<std::string>, v, it)
    {
        groups.append(dxutils::fromStdString((*it))+"\n");
    }
    if(!groups.contains(_("Chats")))
        groups.prepend(FXStringFormat("%s\n", _("Chats")));
    m_groups->fillItems(groups);
}

long ChatDialog::onOK(FXObject*, FXSelector, void*)
{
    std::string alias;
    if(m_alias->getText().empty()) alias = dxutils::toStdString(((FXTextField*)m_chatEntryFields[0])->getText());
    else alias = dxutils::toStdString(m_alias->getText());
    //chat name empty => return
    if(alias.empty()) return 1;
    std::string group = dxutils::toStdString(m_groups->getText());
    if(group.empty()) group = _("Chats");
    if(m_modifing)
    {
        m_chat.setChatParameters(fillChatEntryFields());
        m_chat.setAlias(alias);
        m_chat.setAutojoin(m_autoJoin->getCheck());
        if(m_chat.getGroupName().compare(group))
        {
            m_account.moveChat(m_chat, group);
        }
    }
    else
    {
        m_account.addChat(fillChatEntryFields(), alias, group, m_autoJoin->getCheck());
    }
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

std::vector<PURPLE::Account::ChatEntry> ChatDialog::fillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields = m_account.getChatParameters();
    std::vector<PURPLE::Account::ChatEntry>::iterator it;
    FXuint i;
    for(it=fields.begin(), i=0; it!=fields.end(); it++,i++)
    {
        PURPLE::Account::ChatEntry& entry = *it;
        if(entry.getIsInt())
        {
            entry.setValue(t2s(static_cast<FXSpinner*>(m_chatEntryFields[i])->getValue()));
        }
        else
        {
            entry.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_chatEntryFields[i])->getText()));
        }
    }
    return fields;
}

