/*
 *      chatjoindialog.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "chatjoindialog.h"
#include "roomlistdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include <sstream>

#define VSPACE 4

template<typename T>
/** String from someone*/
std::string t2s( const T & Value )
{
    std::ostringstream oss;
    oss << Value;
    return oss.str();
}

FXDEFMAP(ChatJoinDialog) ChatJoinDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, ChatJoinDialog_LIST, ChatJoinDialog::onRoomList),
    FXMAPFUNC(SEL_COMMAND, ChatJoinDialog_JOIN, ChatJoinDialog::onJoinRoom),
    FXMAPFUNC(SEL_COMMAND, ChatJoinDialog_ID, ChatJoinDialog::onAccount)
};

FXIMPLEMENT(ChatJoinDialog, FXDialogBox, ChatJoinDialogMap, ARRAYNUMBER(ChatJoinDialogMap))

ChatJoinDialog::ChatJoinDialog(FXMainWindow* owner, PURPLE::Purple *purple)
        : FXDialogBox(owner, _("Connect chat"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_accounts = purple->getConnectedAccountsList();
    //remove accounts !supportChats
    for(FXuint i=0; i<m_accounts.size(); i++)
    {
        if(!m_accounts[i].supportsChats())
        {
            m_accounts.erase(m_accounts.begin()+i);
        }
    }
    createControls();
}

ChatJoinDialog::ChatJoinDialog(FXMainWindow *owner, PURPLE::Account account)
        : FXDialogBox(owner, _("Connect chat"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_accounts.push_back(account);
    createControls();
}

ChatJoinDialog::~ChatJoinDialog()
{
}

long ChatJoinDialog::onRoomList(FXObject*, FXSelector, void*)
{
    m_accounts[m_ids->getCurrentItem()].displayRoomList();
    return 1;
}

long ChatJoinDialog::onJoinRoom(FXObject*, FXSelector, void*)
{
    m_accounts[m_ids->getCurrentItem()].joinChat(fillChatEntryFields());
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long ChatJoinDialog::onAccount(FXObject*, FXSelector, void *ptr)
{
    FXint index = (FXint)(FXival)ptr;
    createChatEntryFields(m_accounts[index].getChatParameters(), true);
    if(m_accounts[index].supportRoomlist()) m_getRoomList->enable();
    else m_getRoomList->disable();
    return 1;
}

void ChatJoinDialog::newRoomList(PURPLE::RoomList *rlist)
{
    RoomListDialog dialog(this, rlist);
    dialog.execute();
}

void ChatJoinDialog::setRoom(FXString room)
{
    //Terrible hack
    if(m_chatEntryFields.no())
        ((FXTextField*)m_chatEntryFields[0])->setText(room);
}

void ChatJoinDialog::createControls()
{
    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    m_matrix = new FXMatrix(m_contents, 2, MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

    new FXLabel(m_matrix, _("Account:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_ids = new FXListBox(m_matrix, this, ChatJoinDialog_ID, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    for(FXuint i=0; i<m_accounts.size(); i++)
    {
        m_ids->appendItem(dxutils::fromStdString(m_accounts[i].getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(m_accounts[i].getProtocol().getPurpleID())));
    }

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, _("&Join"), NULL, this, ChatJoinDialog_JOIN, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_getRoomList = new dxEXButton(buttonframe, _("&Get chats list"), NULL, this, ChatJoinDialog_LIST, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    if(!m_accounts[0].supportRoomlist()) m_getRoomList->disable();

    createChatEntryFields(m_accounts[0].getChatParameters());
}

void ChatJoinDialog::createChatEntryFields(std::vector<PURPLE::Account::ChatEntry> fields, bool create)
{
    FXint h = this->getDefaultHeight();
    FXint n = m_chatEntryFields.no();
    FXLabel *label;
    FXTextField *text;
    FXSpinner *spinner;
    for(FXint i=0; i<m_labels.no(); i++)
    {
        delete m_labels[i];
    }
    m_labels.clear();
    for(FXint i=0; i<m_chatEntryFields.no(); i++)
    {
        delete m_chatEntryFields[i];
    }
    m_chatEntryFields.clear();
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        label = new FXLabel(m_matrix, createLabel(entry.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        if(create) label->create();
        m_labels.append(label);
        if(entry.getIsInt())
        {
            spinner = new FXSpinner(m_matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            spinner->setRange(entry.getMin(), entry.getMax());
            if(create) spinner->create();
            m_chatEntryFields.append(spinner);
        }
        else
        {
            if(entry.getSecret()) text = new FXTextField(m_matrix, 30, NULL, 0, TEXTFIELD_PASSWD|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            else text = new FXTextField(m_matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            text->setText(dxutils::fromStdString(entry.getValue()));
            if(create) text->create();
            m_chatEntryFields.append(text);
        }
    }
    if(create)
    {
        h = h - (n - m_chatEntryFields.no())*m_chatEntryFields[0]->getDefaultHeight()
            - (n - m_chatEntryFields.no())*VSPACE;
        this->resize(this->getWidth(), h);
    }
}

FXString ChatJoinDialog::createLabel(std::string label)
{
    FXString text = "";
    FOREACH(std::string, label, it)
    {
        if((*it)=='_') continue;
        text.append((*it));
    }
    return text;
}

std::vector<PURPLE::Account::ChatEntry> ChatJoinDialog::fillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields = m_accounts[m_ids->getCurrentItem()].getChatParameters();
    std::vector<PURPLE::Account::ChatEntry>::iterator it;
    FXuint i;
    for(it=fields.begin(), i=0; it!=fields.end(); it++,i++)
    {
        PURPLE::Account::ChatEntry& entry = *it;
        if(entry.getIsInt())
        {
            entry.setValue(t2s(static_cast<FXSpinner*>(m_chatEntryFields[i])->getValue()));
        }
        else
        {
            entry.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_chatEntryFields[i])->getText()));
        }
    }
    return fields;
}

