/*
 *      choicerequestdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "choicerequestdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"

class ChoiceListItem : public FXListItem
{
public:
    ChoiceListItem(const FXString& text, int id) : FXListItem(text), m_id(id) {}

    int getID() const { return m_id; }
private:
    int m_id;
};

FXDEFMAP(ChoiceRequestDialog) ChoiceRequestDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, ChoiceRequestDialog_CANCEL, ChoiceRequestDialog::onCancel),
    FXMAPFUNC(SEL_COMMAND, ChoiceRequestDialog_OK, ChoiceRequestDialog::onOK)
};

FXIMPLEMENT(ChoiceRequestDialog, FXDialogBox, ChoiceRequestDialogMap, ARRAYNUMBER(ChoiceRequestDialogMap))

ChoiceRequestDialog::ChoiceRequestDialog(FXMainWindow *owner, PURPLE::ChoiceRequest *request)
: FXDialogBox(owner, request->getTitle().empty() ? _("New request") : dxutils::fromStdString(request->getTitle()),
        DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0), m_request(request)
{
    FXVerticalFrame *contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    new FXLabel(contents, dxutils::fromStdString(m_request->getPrimary()), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    if(!m_request->getSecondary().empty()) new FXLabel(contents, dxutils::fromStdString(m_request->getSecondary()), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXVerticalFrame *listframe = new FXVerticalFrame(contents, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_choices = new FXList(listframe, NULL, 0, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    std::map<std::string, int> choices = m_request->getChoices();
    std::map<std::string, int>::iterator it;
    for(it=choices.begin(); it != choices.end(); it++)
    {
        ChoiceListItem *item = new ChoiceListItem(dxutils::normalizeLabelString((*it).first, '_'), (*it).second);
        m_choices->appendItem(item);
        if((*it).second == m_request->getDefaultValue())
            m_choices->setCurrentItem(m_choices->getNumItems()-1);
    }
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, ChoiceRequestDialog_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, dxutils::fromStdString(request->getOkText()).substitute('_', '&'), NULL, this, ChoiceRequestDialog_OK, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
}

ChoiceRequestDialog::~ChoiceRequestDialog() {
}

//handle press okbutton
long ChoiceRequestDialog::onOK(FXObject*, FXSelector, void*)
{
    m_request->sendChoice(static_cast<ChoiceListItem*>(m_choices->getItem(m_choices->getCurrentItem()))->getID());
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

//handle press cancelbutton
long ChoiceRequestDialog::onCancel(FXObject*, FXSelector, void*)
{
    m_request->dennyRequest();
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}

