/*
 *      fieldsrequestdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "fieldsrequestdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"

#define MAX_ICON_SIZE 256

static bool vectorContains(std::vector<std::string> v, const std::string &value)
{
    FOREACH(std::vector<std::string>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

static FXIcon *createIconFromName(FXApp *app, const FXString& path)
{
    FXIconSource source(app);
    FXIcon *icon = NULL;
    if(!FXStat::exists(path))
        return NULL;
    icon = source.loadIconFile(path, "png");
    if(icon)
    {
        FXuint w = icon->getWidth();
        FXuint h = icon->getHeight();
        if(w>MAX_ICON_SIZE || h>MAX_ICON_SIZE)
        {
            if(w>h)
                icon->scale(MAX_ICON_SIZE, (MAX_ICON_SIZE*h)/w, 1);
            else
                icon->scale((MAX_ICON_SIZE*w)/h, MAX_ICON_SIZE, 1);
        }
        icon->blend(app->getBaseColor());
        icon->create();
        return icon;
    }
    else
    {
        icon = source.loadIconFile(path, "jpg");
        if(icon)
        {
            FXuint w = icon->getWidth();
            FXuint h = icon->getHeight();
            if(w>MAX_ICON_SIZE || h>MAX_ICON_SIZE)
            {
                if(w>h)
                    icon->scale(MAX_ICON_SIZE, (MAX_ICON_SIZE*h)/w, 1);
                else
                    icon->scale((MAX_ICON_SIZE*w)/h, MAX_ICON_SIZE, 1);
            }
            icon->blend(app->getBaseColor());
            icon->create();
            return icon;
        }
        else
        {
            icon = source.loadIconFile(path, "gif");
            if(icon)
            {
                FXuint w = icon->getWidth();
                FXuint h = icon->getHeight();
                if(w>MAX_ICON_SIZE || h>MAX_ICON_SIZE)
                {
                    if(w>h)
                        icon->scale(MAX_ICON_SIZE, (MAX_ICON_SIZE*h)/w, 1);
                    else
                        icon->scale((MAX_ICON_SIZE*w)/h, MAX_ICON_SIZE, 1);
                }
                icon->blend(app->getBaseColor());
                icon->create();
                return icon;
            }
            else
                return NULL;
        }
    }
}

FXDEFMAP(FieldsRequestDialog) FieldsRequestDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, FieldsRequestDialog_OK, FieldsRequestDialog::onOk),
    FXMAPFUNC(SEL_COMMAND, FieldsRequestDialog_CANCEL, FieldsRequestDialog::onCancel)
};

FXIMPLEMENT(FieldsRequestDialog, FXDialogBox, FieldsRequestDialogMap, ARRAYNUMBER(FieldsRequestDialogMap))

FieldsRequestDialog::FieldsRequestDialog(FXMainWindow *owner, PURPLE::FieldsRequest *request)
: FXDialogBox(owner, request->getTitle().empty() ? _("New request") : dxutils::fromStdString(request->getTitle()),
        DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0), m_request(request)
{
    FXVerticalFrame *contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    new FXLabel(contents, dxutils::fromStdString(m_request->getPrimary()), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    if(!m_request->getSecondary().empty()) new FXLabel(contents, dxutils::fromStdString(m_request->getSecondary()), 0, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    createFields(contents);
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, dxutils::fromStdString(request->getCancelText()).substitute('_', '&'), NULL, this, FieldsRequestDialog_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, dxutils::fromStdString(request->getOkText()).substitute('_', '&'), NULL, this, FieldsRequestDialog_OK, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
}

FieldsRequestDialog::~FieldsRequestDialog() {
}

long FieldsRequestDialog::onOk(FXObject*, FXSelector, void*) {
    fillFields();
    m_request->acceptRequest();
    getApp()->stopModal(this, TRUE);
    hide();
    return 1;
}

long FieldsRequestDialog::onCancel(FXObject*, FXSelector, void*) {
    m_request->dennyRequest();
    getApp()->stopModal(this, TRUE);
    hide();
    return 1;
}

void FieldsRequestDialog::createFields(FXVerticalFrame *contents)
{
    std::vector<PURPLE::FieldsRequest::Field> v = m_request->getFields();
    FXString group = "";
    FXMatrix *matrix = new FXMatrix(contents, 2, LAYOUT_FILL_X|MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
    FXTextField *tfield;
    FXText *text;
    FXListBox *combo;
    FXList *list;
    FXCheckButton *check;
    FOREACH(std::vector<PURPLE::FieldsRequest::Field>, v, it)
    {
        PURPLE::FieldsRequest::Field field = *it;
        if(group != dxutils::fromStdString(field.getGroup()))
        {
            group = dxutils::fromStdString(field.getGroup());
            if(!group.empty()) new FXLabel(contents, group, NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            matrix = new FXMatrix(contents, 2, LAYOUT_FILL_X|MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
        }
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            new FXLabel(matrix, dxutils::fromStdString(field.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            if(field.getMultiline())
            {
                FXVerticalFrame *frame = new FXVerticalFrame(matrix, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                text = new FXText(frame, NULL, 0, TEXT_WORDWRAP|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH, 0, 0, 400);
                text->setVisibleRows(5);
                text->setText(dxutils::fromStdString(field.getStringValue()));
                text->setEditable(field.getEditable());
                m_fields.append(text);
            }
            else
            {
                tfield = new FXTextField(matrix, 30, NULL, 0, field.getMasked()?TEXTFIELD_PASSWD|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X:FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                tfield->setText(dxutils::fromStdString(field.getStringValue()));
                tfield->setEditable(field.getEditable());
                m_fields.append(tfield);
            }
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            new FXLabel(matrix, dxutils::fromStdString(field.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            tfield = new FXTextField(matrix, 30, NULL, 0, TEXTFIELD_INTEGER|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            tfield->setText(dxutils::fromStdString(field.getStringValue()));
            m_fields.append(tfield);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            new FXLabel(matrix, dxutils::fromStdString(field.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            check = new FXCheckButton(matrix, "", NULL, 0);
            check->setCheck(field.getBoolValue());
            m_fields.append(check);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            new FXLabel(matrix, dxutils::fromStdString(field.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            combo = new FXListBox(matrix, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            std::vector<std::string> s = field.getLabels();
            FOREACH(std::vector<std::string>, s, sit)
            {
                combo->appendItem(dxutils::fromStdString(*sit));
            }
            m_fields.append(combo);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            new FXLabel(matrix, dxutils::fromStdString(field.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            FXVerticalFrame *frame = new FXVerticalFrame(matrix, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            if(field.getMultiple()) list = new FXList(frame, NULL, 0, LIST_BROWSESELECT|LIST_MULTIPLESELECT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            else list = new FXList(frame, NULL, 0, LIST_BROWSESELECT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            std::vector<std::string> s = field.getLabels();
            std::vector<std::string> sel = field.getSelected();
            FOREACH(std::vector<std::string>, s, sit)
            {
                FXint id = list->appendItem(dxutils::fromStdString(*sit));
                if(vectorContains(sel, *sit)) list->selectItem(id);
            }
            m_fields.append(list);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FIMAGE)
        {
            new FXLabel(matrix, dxutils::fromStdString(field.getLabel()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            new FXLabel(matrix, "", createIconFromName(getApp(), dxutils::fromStdString(field.getStringValue())), JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        }
    }
}

void FieldsRequestDialog::fillFields()
{
    PURPLE::FieldsRequest::Fields fields = m_request->getFields();
    PURPLE::FieldsRequest::Fields::iterator it;
    FXuint i = 0;
    for(it=fields.begin(); it!=fields.end(); it++)
    {
        PURPLE::FieldsRequest::Field& field = *it;
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            if(field.getMultiline())
            {
                field.setValue(dxutils::toStdString(static_cast<FXText*>(m_fields[i])->getText()));
            }
            else
            {
                field.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_fields[i])->getText()));
            }
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            field.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_fields[i])->getText()));
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            field.setValue(static_cast<FXCheckButton*>(m_fields[i])->getCheck());
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            field.setValue(static_cast<FXListBox*>(m_fields[i])->getCurrentItem());
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            FXList *list = static_cast<FXList*>(m_fields[i]);
            std::vector<std::string> v;
            for(FXint j=0; j<list->getNumItems(); j++)
            {
                if(list->getItem(j)->isSelected())
                    v.push_back(dxutils::toStdString(list->getItemText(j)));
            }
            field.setSelected(v);
            i++;
        }
        else
            continue;
    }
    m_request->setFields(fields);
}

