/*
 *      ftdialog.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "ftdialog.h"
#include "icons.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "dxim.h"

FXDEFMAP(FTDialog) FTDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND,              FTDialog_CLOSE,        FTDialog::onClose),
    FXMAPFUNC(SEL_CLOSE,                0,                     FTDialog::onClose),
    FXMAPFUNC(SEL_KEYPRESS,             0,                     FTDialog::onKeyPress),
    FXMAPFUNC(SEL_COMMAND,              FTDialog_CLEAR,        FTDialog::onClear),
    FXMAPFUNC(SEL_COMMAND,              FTDialog_CANCELED,     FTDialog::onCanceled),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,   FTDialog_TABLE,        FTDialog::onRightClick),
    FXMAPFUNC(SEL_CHANGED,              FTDialog_TABLE,        FTDialog::onTableChanged),
    FXMAPFUNC(SEL_COMMAND,              FTDialog_POPUPCANCEL,  FTDialog::onPopupCancel)
};

FXIMPLEMENT(FTDialog, FXTopWindow, FTDialogMap, ARRAYNUMBER(FTDialogMap))

FTDialog::FTDialog(FXApp *app)
    : FXTopWindow(app, _("File transfers"), NULL, NULL, DECOR_ALL, 0,0,0,0, 0,0,0,0, 0,0)
{
    setIcon(ICO_BIGICON);
    setMiniIcon(ICO_SMALLICON);

    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_tableframe = new FXVerticalFrame(m_contents, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);
    m_table = new FXTable(m_tableframe, this, FTDialog_TABLE, TABLE_COL_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 2,2,2,2);
    m_table->setVisibleRows(5);
    m_table->setVisibleColumns(6);
    m_table->setTableSize(0, 6);
    m_table->setColumnText(0, _("File"));
    m_table->setColumnText(1, _("Size"));
    m_table->setColumnText(2, _("Position"));
    m_table->setColumnText(3, "%");
    m_table->setColumnText(4, _("Speed"));
    m_table->setColumnText(5, _("Remaining time"));
    m_table->setEditable(FALSE);
    m_table->showHorzGrid(FALSE);
    m_table->showVertGrid(FALSE);
    m_table->setRowHeaderWidth(0);

    m_group = new FXGroupBox(m_contents, _("Details"), FRAME_GROOVE|LAYOUT_LEFT|LAYOUT_FILL_X, 0,0,0,0, 4,4,4,4, 4,4);
    m_matrix = new FXMatrix(m_group, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(m_matrix, _("Path:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_path = new FXLabel(m_matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);
    new FXLabel(m_matrix, _("Account:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_account = new FXLabel(m_matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);
    new FXLabel(m_matrix, _("Protocol:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_protocol = new FXLabel(m_matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);
    m_protocol->setIconPosition(ICON_BEFORE_TEXT);
    m_direction = new FXLabel(m_matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_buddy = new FXLabel(m_matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);

    m_buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    m_buttonClose = new dxEXButton(m_buttonframe, _("C&lose"), NULL, this, FTDialog_CLOSE, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonClear = new dxEXButton(m_buttonframe, _("Clear &finished"), NULL, this, FTDialog_CLEAR, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonCanceled = new dxEXButton(m_buttonframe, _("Clear &canceled"), NULL, this, FTDialog_CANCELED, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
}

FTDialog::~FTDialog()
{}

void FTDialog::create()
{
    FXTopWindow::create();
    updateTable();
}

long FTDialog::onClose(FXObject*,FXSelector,void*)
{
    hide();
    return 1;
}

long FTDialog::onKeyPress(FXObject *sender,FXSelector sel,void *ptr)
{
    FXEvent *event = (FXEvent*)ptr;
    if(event->code == KEY_Escape)
    {
        //close FTDialog
        hide();
        return 1;
    }
    else
    {
        if(FXTopWindow::onKeyPress(sender,sel,ptr))
            return 1;
    }
    return 0;
}

long FTDialog::onClear(FXObject*, FXSelector, void*)
{
    if(!m_transfers.no())
        return 1;
    for(FXint i=m_transfers.no()-1; i>-1; i--)
    {
        if(m_transfers[i]->isFinished())
        {
            m_transfers.erase(i);
        }
    }
    updateTable();
    return 1;
}

long FTDialog::onCanceled(FXObject*, FXSelector, void*)
{
    if(!m_transfers.no())
        return 1;
    for(FXint i=m_transfers.no()-1; i>-1; i--)
    {
        if(m_transfers[i]->isCanceled())
            m_transfers.erase(i);
    }
    updateTable();
    return 1;
}

long FTDialog::onRightClick(FXObject*, FXSelector, void *ptr)
{
    FXEvent* event = (FXEvent*)ptr;
    if(event->moved) return 1;
    m_indexOnRight = m_table->rowAtY(event->win_y);
    if(m_indexOnRight >= m_transfers.no())
        return 1;
    if(m_transfers[m_indexOnRight]->isCanceled() || m_transfers[m_indexOnRight]->isFinished())
        return 1;
    FXMenuPane popup(this);
    new FXMenuCommand(&popup, _("&Cancel"), NULL, this, FTDialog_POPUPCANCEL);
    popup.create();
    popup.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&popup);
    return 1;
}

long FTDialog::onPopupCancel(FXObject*, FXSelector, void*)
{
    m_transfers[m_indexOnRight]->cancelFT();
    return 1;
}

long FTDialog::onTableChanged(FXObject*, FXSelector, void *ptr)
{
    FXTablePos *tp = (FXTablePos*)ptr;
    m_contents->recalc();
    m_path->setText(dxutils::fromStdString(m_transfers[tp->row]->getLocalFileName()));
    m_account->setText(dxutils::fromStdString(m_transfers[tp->row]->getAccount().getUsername()));
    m_protocol->setText(dxutils::fromStdString(m_transfers[tp->row]->getAccount().getProtocol().getName()));
    m_buddy->setText(dxutils::fromStdString(m_transfers[tp->row]->getRemoteUser()));
    return 1;
}

void FTDialog::updateTable()
{
    bool canceled = false;
    bool finished = false;
    if(!m_transfers.no())
    {
        m_buttonClear->disable();
        m_buttonCanceled->disable();
        m_table->removeRows(0, m_table->getNumRows());
        m_group->hide();
        m_contents->recalc();
        return;
    }
    if(!shown())
        return;
    if(m_table->getNumRows() < m_transfers.no())
        m_table->insertRows(0, m_transfers.no()-m_table->getNumRows());
    if(m_table->getNumRows() > m_transfers.no())
        m_table->removeRows(0, m_table->getNumRows()-m_transfers.no());
    for(FXint i=0; i<m_transfers.no(); i++)
    {
        if(m_transfers[i]->isFinished())
        {
            if(m_transfers[i]->isCanceled())
            {
                m_table->setItemIcon(i, 0, ICO_CANCEL);
                canceled = true;
            }
            else
            {
                m_table->setItemIcon(i, 0, ICO_FINISH);
                finished = true;
            }
            m_table->setItemIconPosition(i, 0, FXTableItem::BEFORE);
            m_table->setItemJustify(i, 0, FXTableItem::LEFT);
        }
        else
        {
            if(m_transfers[i]->isCanceled())
            {
                m_table->setItemIcon(i, 0, ICO_CANCEL);
                canceled = true;
            }
            else m_table->setItemIcon(i, 0, m_transfers[i]->isReceiving()?ICO_DOWN:ICO_UP);
            m_table->setItemIconPosition(i, 0, FXTableItem::BEFORE);
            m_table->setItemJustify(i, 0, FXTableItem::LEFT);
        }
        m_table->setItemText(i, 0, dxutils::fromStdString(m_transfers[i]->getFileName()));
        m_table->setItemText(i, 1, dxutils::getFileSize(m_transfers[i]->getFileSize()));
        if(!m_transfers[i]->isFinished())
        {
            m_table->setItemText(i, 2, dxutils::getFileSize(m_transfers[i]->getBytesSent()));
            m_table->setItemText(i, 3, dxutils::getPercentPosition(m_transfers[i]->getFileSize(), m_transfers[i]->getBytesSent()));
        }
        else
        {
            m_table->setItemText(i, 2, dxutils::getFileSize(m_transfers[i]->getFileSize()));
            m_table->setItemText(i, 3, "100%");
        }
        if(m_transfers[i]->isFinished() || m_transfers[i]->isCanceled())
        {
            m_table->setItemText(i, 4, "");
            m_table->setItemText(i, 5, m_transfers[i]->isFinished()?_("Finished"):_("Canceled"));
        }
        else
        {
            if(!m_transfers[i]->getBytesSent())
            {
                m_table->setItemText(i, 4, "?");
                m_table->setItemText(i, 5, m_transfers[i]->isStarted()?dxutils::getRemaining(0,0):_("Waiting"));
            }
            else
            {
                m_table->setItemText(i, 4, dxutils::getSpeed(m_transfers[i]->getSpeed()));
                m_table->setItemText(i, 5, dxutils::getRemaining(m_transfers[i]->getFileSize()-m_transfers[i]->getBytesSent(), m_transfers[i]->getSpeed()));
            }
        }
    }
    if(finished) m_buttonClear->enable();
    else m_buttonClear->disable();
    if(canceled) m_buttonCanceled->enable();
    else m_buttonCanceled->disable();
    m_path->setText(dxutils::fromStdString(m_transfers[m_table->getCurrentRow()]->getLocalFileName()));
    m_account->setText(dxutils::fromStdString(m_transfers[m_table->getCurrentRow()]->getAccount().getUsername()));
    m_protocol->setText(dxutils::fromStdString(m_transfers[m_table->getCurrentRow()]->getAccount().getProtocol().getName()));
    m_protocol->setIcon(dxutils::getProtocolIcon(dxutils::fromStdString(m_transfers[m_table->getCurrentRow()]->getAccount().getProtocol().getPurpleID())));
    m_direction->setText(m_transfers[m_table->getCurrentRow()]->isReceiving()?_("From:"):_("To:"));
    m_buddy->setText(dxutils::fromStdString(m_transfers[m_table->getCurrentRow()]->getRemoteUser()));
    m_group->show();
    m_contents->recalc();
}

void FTDialog::newFT(PURPLE::FileTransfer *ft)
{
    m_transfers.append(ft);
    updateTable();
}

void FTDialog::updateFT(PURPLE::FileTransfer *ft)
{
    updateTable();
}

