/*
 *      imdialog.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "imdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"

FXDEFMAP(IMDialog) IMDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, IMDialog_ID, IMDialog::onIDs),
    FXMAPFUNC(SEL_CHANGED, IMDialog_WITH, IMDialog::onKeyPress),
    FXMAPFUNC(SEL_COMMAND, IMDialog_BUDDIES, IMDialog::onBuddies)
};

FXIMPLEMENT(IMDialog, FXDialogBox, IMDialogMap, ARRAYNUMBER(IMDialogMap))

IMDialog::IMDialog(FXMainWindow* owner, PURPLE::Purple *purple)
        : FXDialogBox(owner, _("New IM"), DECOR_STRETCHABLE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_accounts = purple->getConnectedAccountsList();
    if(m_accounts.size()) m_buddies = m_accounts[0].getBuddies();
    else close();

    getAccelTable()->addAccel(KEY_Return, this, FXDialogBox::ID_ACCEPT);
    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    m_matrix = new FXMatrix(m_contents, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X | LAYOUT_FILL_Y);

    new FXLabel(m_matrix, _("Account:"), NULL, JUSTIFY_LEFT);
    m_ids = new FXListBox(m_matrix, this, IMDialog_ID, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    for(FXuint i=0; i<m_accounts.size(); i++)
    {
        m_ids->appendItem(dxutils::fromStdString(m_accounts[i].getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(m_accounts[i].getProtocol().getPurpleID())));
    }

    new FXLabel(m_matrix, _("With:"), NULL, JUSTIFY_LEFT);
    m_with = new FXTextField(m_matrix, 25, this, IMDialog_WITH, TEXTFIELD_ENTER_ONLY | FRAME_THICK | FRAME_SUNKEN | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXSpring(m_matrix);
    m_buddiesList = new FXList(m_matrix, this, IMDialog_BUDDIES, LAYOUT_FILL_COLUMN | LAYOUT_FILL_ROW | LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_buddiesList->setSortFunc(FXList::ascendingCase);
    m_buddiesList->setNumVisible(5);
    m_buddiesList->hide();

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&IM"), NULL, this, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
}

IMDialog::IMDialog(FXMainWindow* owner, PURPLE::Account account)
        : FXDialogBox(owner, _("New IM"), DECOR_STRETCHABLE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_accounts.push_back(account);
    if(m_accounts.size()) m_buddies = m_accounts[0].getBuddies();
    else close();

    getAccelTable()->addAccel(KEY_Return, this, FXDialogBox::ID_ACCEPT);
    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    m_matrix = new FXMatrix(m_contents, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X | LAYOUT_FILL_Y);

    new FXLabel(m_matrix, _("Account:"), NULL, JUSTIFY_LEFT);
    new FXLabel(m_matrix, dxutils::fromStdString(account.getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(account.getProtocol().getPurpleID())), ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);

    new FXLabel(m_matrix, _("With:"), NULL, JUSTIFY_LEFT);
    m_with = new FXTextField(m_matrix, 25, this, IMDialog_WITH, TEXTFIELD_ENTER_ONLY | FRAME_THICK | FRAME_SUNKEN | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXSpring(m_matrix);
    m_buddiesList = new FXList(m_matrix, this, IMDialog_BUDDIES, LAYOUT_FILL_COLUMN | LAYOUT_FILL_ROW | LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_buddiesList->setSortFunc(FXList::ascendingCase);
    m_buddiesList->setNumVisible(5);
    m_buddiesList->hide();

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&IM"), NULL, this, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
}

IMDialog::~IMDialog()
{
}

void IMDialog::close()
{
    hide();
}

//handle keypress
long IMDialog::onKeyPress(FXObject*, FXSelector, void*)
{
    m_buddiesList->clearItems();
    if(m_with->getText().empty())
    {
        m_buddiesList->hide();
        FXDialogBox::layout();
        update();
        return 1;
    }
    m_buddiesList->show();
    FXString text = m_with->getText();
    FXString name, id;
    for(FXuint i=0 ; i < m_buddies.size(); i++)
    {
        name = dxutils::fromStdString(m_buddies[i].getAlias());
        id = dxutils::fromStdString(m_buddies[i].getUsername());
        if(comparecase(text,name.left(text.length()))==0)
        {
            m_buddiesList->appendItem(FXStringFormat("%s (%s)", name.text(), id.text()));
            continue;
        }
        if(comparecase(text,id.left(text.length()))==0)
        {
            m_buddiesList->appendItem(FXStringFormat("%s (%s)", name.text(), id.text()));
        }
    }
    if(!m_buddiesList->getNumItems())
        m_buddiesList->hide();
    FXDialogBox::layout();
    update();
    return 1;
}

//handle buddies selection
long IMDialog::onBuddies(FXObject*, FXSelector, void *ptr)
{
    FXint i = (FXint)(FXival)ptr;
    m_with->setText(m_buddiesList->getItemText(i).rafter('(').before(')'));
    return 1;
}

//handle m_ids change
long IMDialog::onIDs(FXObject*, FXSelector, void *ptr)
{
    FXint i = (FXint)(FXival)ptr;
    m_buddies = m_accounts[i].getBuddies();
    m_with->setText("");
    m_buddiesList->clearItems();
    return 1;
}

PURPLE::Account IMDialog::getAccount()
{
    if(m_accounts.size() > 1)
    {
        return m_accounts[m_ids->getCurrentItem()];
    }
    return m_accounts[0];
}
