/*
 *      inputrequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "inputrequestdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "dxutils.h"

FXDEFMAP(InputRequestDialog) InputRequestDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, InputRequestDialog_CANCEL, InputRequestDialog::onCancel),
    FXMAPFUNC(SEL_COMMAND, InputRequestDialog_OK, InputRequestDialog::onOK)
};

FXIMPLEMENT(InputRequestDialog, FXDialogBox, InputRequestDialogMap, ARRAYNUMBER(InputRequestDialogMap))

InputRequestDialog::InputRequestDialog(FXMainWindow* owner, PURPLE::InputRequest *request)
        : FXDialogBox(owner, request->getTitle().empty() ? _("New request") : dxutils::fromStdString(request->getTitle()),
                    DECOR_STRETCHABLE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0), m_request(request)
{
    getAccelTable()->addAccel(KEY_Return, this, InputRequestDialog_OK);
    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    new FXLabel(m_contents, dxutils::fromStdString(request->getPrimary()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    if(!request->getSecondary().empty()) new FXLabel(m_contents, dxutils::fromStdString(request->getSecondary()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    if(request->getMasked()) m_input = new FXTextField(m_contents, 25, NULL, 0, TEXTFIELD_ENTER_ONLY | TEXTFIELD_PASSWD | FRAME_THICK | FRAME_SUNKEN | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    else m_input = new FXTextField(m_contents, 25, NULL, 0, TEXTFIELD_ENTER_ONLY | FRAME_THICK | FRAME_SUNKEN | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_input->setText(dxutils::fromStdString(request->getDefaultValue()));
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, InputRequestDialog_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, dxutils::fromStdString(request->getOkText()).substitute('_', '&'), NULL, this, InputRequestDialog_OK, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
}

InputRequestDialog::~InputRequestDialog()
{
}

//handle press okbutton
long InputRequestDialog::onOK(FXObject*, FXSelector, void*)
{
    m_request->sendInput(dxutils::toStdString(m_input->getText()));
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

//handle press cancelbutton
long InputRequestDialog::onCancel(FXObject*, FXSelector, void*)
{
    m_request->dennyRequest();
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}
