/*
 *      plugindialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "plugindialog.h"
#include "plugineditor.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

FXDEFMAP(PluginDialog) PluginDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND,          PluginDialog_MODIFY,    PluginDialog::onModify),
    FXMAPFUNC(SEL_COMMAND,          PluginDialog_ENABLE,    PluginDialog::onEnable),
    FXMAPFUNC(SEL_KEYPRESS,         0,                      PluginDialog::onKeyPress),
    FXMAPFUNC(SEL_CHANGED,          PluginDialog_TABLE,     PluginDialog::onTableChanged),
    FXMAPFUNC(SEL_COMMAND,          PluginDialog_HOMEPAGE,  PluginDialog::onLink),
    FXMAPFUNC(SEL_MOTION,           PluginDialog_HOMEPAGE,  PluginDialog::onLinkMotion)
};

FXIMPLEMENT(PluginDialog, FXDialogBox, PluginDialogMap, ARRAYNUMBER(PluginDialogMap))

PluginDialog::PluginDialog(FXMainWindow *owner, std::vector<PURPLE::Plugin> plugins)
    : FXDialogBox(owner, _("Plugins list"), DECOR_ALL, 0,0,0,0, 0,0,0,0, 0,0), m_plugins(plugins)
{
    m_contents = new FXVerticalFrame(this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);

    FXVerticalFrame *tableframe = new FXVerticalFrame(m_contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK);
    m_table = new FXTable(tableframe, this, PluginDialog_TABLE, TABLE_COL_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 2,2,2,2);
    m_table->setVisibleRows(5);
    m_table->setVisibleColumns(2);
    m_table->setTableSize(m_plugins.size(), 2);
    m_table->setColumnText(0, _("Plugin"));
    m_table->setColumnText(1, _("Enabled"));
    m_table->setEditable(FALSE);
    m_table->showHorzGrid(FALSE);
    m_table->showVertGrid(FALSE);
    m_table->setRowHeaderWidth(0);

    m_group = new FXGroupBox(m_contents, _("Details"), FRAME_GROOVE|LAYOUT_LEFT|LAYOUT_FILL_X, 0,0,0,0, 4,4,4,4, 4,4);
    FXMatrix *matrix = new FXMatrix(m_group, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(matrix, _("Summary:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_summary =  new FXText(matrix, NULL, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY|TEXT_WORDWRAP);
    m_summary->setVisibleRows(2);
    m_summary->setBackColor(getApp()->getBaseColor());
    new FXLabel(matrix, _("Version:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_version =  new FXLabel(matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);
    new FXLabel(matrix, _("Description:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_description =  new FXText(matrix, NULL, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY|TEXT_WORDWRAP);
    m_description->setVisibleRows(2);
    m_description->setBackColor(getApp()->getBaseColor());
    new FXLabel(matrix, _("Author:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_author =  new FXLabel(matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);
    new FXLabel(matrix, _("Path:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_path =  new FXLabel(matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN);
    new FXLabel(matrix, _("Homepage:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_homepage = new FXButton(matrix, "", 0, this, PluginDialog_HOMEPAGE, LAYOUT_FILL_COLUMN);
    m_homepage->setTextColor(FXRGB(142,196,210));

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    new dxEXButton(buttonframe, _("&Save&&Close"), NULL, this, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, this, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonModify = new dxEXButton(buttonframe, _("&Modify"), NULL, this, PluginDialog_MODIFY, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonModify->disable();
    m_buttonEnable = new dxEXButton(buttonframe, _("&Enable"), NULL, this, PluginDialog_ENABLE, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonEnable->disable();

    updateTable();
}

PluginDialog::~PluginDialog()
{
}

long PluginDialog::onModify(FXObject*,FXSelector,void*)
{
    for(FXuint i=0; i<m_plugins.size(); i++)
    {
        if(i == (FXuint)m_table->getCurrentRow())
        {
            PluginEditor editor(this, m_plugins[i]);
            editor.execute(PLACEMENT_OWNER);
            return 1;
        }
    }
    return 1;
}

long PluginDialog::onKeyPress(FXObject *sender,FXSelector sel,void *ptr)
{
    if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        getApp()->stopModal(this,FALSE);
        hide();
        return 1;
    }
    return 0;
}

long PluginDialog::onTableChanged(FXObject*, FXSelector, void*)
{
    updateButtons();
    updateDetails();
    return 1;
}

long PluginDialog::onEnable(FXObject*,FXSelector,void*)
{
    for(FXuint i=0; i<m_plugins.size(); i++)
    {
        if(i == (FXuint)m_table->getCurrentRow())
        {
            if(!m_plugins[i].isLoadable())
                return 1;
            if(m_table->getItemText(m_table->getCurrentRow(), 1)==_("Yes"))
            {
                m_plugins[i].unload();
                m_buttonEnable->setText(_("&Enable"));
            }
            else
            {
                m_plugins[i].load();
                m_buttonEnable->setText(_("Di&sable"));
            }
        }
    }
    updateTable();
    updateButtons();
    return 1;
}

void PluginDialog::updateTable()
{
    if(m_plugins.size())
    {
        if((FXuint)m_table->getNumRows() < m_plugins.size())
            m_table->insertRows(0, m_plugins.size()-m_table->getNumRows());
        if((FXuint)m_table->getNumRows() > m_plugins.size())
            m_table->removeRows(0, m_table->getNumRows()-m_plugins.size());
        for(FXuint i=0; i<m_plugins.size(); i++)
        {
            m_table->setItemJustify(i, 0, FXTableItem::LEFT);
            m_table->setItemText(i, 0, dxutils::fromStdString(m_plugins[i].getName()));
            m_table->setItemJustify(i, 1, FXTableItem::LEFT);
            m_table->setItemText(i, 1, m_plugins[i].isLoaded() ? _("Yes") : _("No"));
        }
        m_table->selectRow(0);
        m_table->setCurrentItem(0,0,true);
    }
    else
    {
        m_buttonModify->disable();
        m_buttonEnable->disable();
    }
}

void PluginDialog::updateDetails()
{
    FXuint i = (FXuint)m_table->getCurrentRow();
    if(i < m_plugins.size())
    {
        m_description->setText(dxutils::fromStdString(m_plugins[i].getDescription()));
        m_version->setText(dxutils::fromStdString(m_plugins[i].getVersion()));
        m_summary->setText(dxutils::fromStdString(m_plugins[i].getSummary()));
        m_author->setText(dxutils::fromStdString(m_plugins[i].getAuthor()));
        m_homepage->setText(dxutils::fromStdString(m_plugins[i].getHomepage()));
        m_path->setText(dxutils::fromStdString(m_plugins[i].getPath()));
    }
    else
    {
        m_description->setText("");
        m_version->setText("");
        m_summary->setText("");
        m_author->setText("");
        m_homepage->setText("");
        m_path->setText("");
    }
    m_group->show();
    m_contents->recalc();
}

void PluginDialog::updateButtons()
{
    FXuint row = (FXuint)m_table->getCurrentRow();
    if(m_table->getItemText(row, 1)==_("Yes"))
    {
        m_buttonEnable->setText(_("Di&sable"));
    }
    else
    {
        m_buttonEnable->setText(_("&Enable"));
    }
    if(row < m_plugins.size())
    {
        if(m_plugins[row].isLoaded())
        {
            m_buttonEnable->enable();
            if(m_plugins[row].hasOptions()) m_buttonModify->enable();
            else m_buttonModify->disable();
        }
        else
        {
            if(m_plugins[row].isLoadable())
            {
                m_buttonEnable->enable();
            }
            else
            {
                m_buttonEnable->disable();
            }
            m_buttonModify->disable();
        }
    }
    else
    {
        m_buttonModify->disable();
        m_buttonEnable->disable();
    }
}

long PluginDialog::onLink(FXObject *sender, FXSelector, void*)
{
    dxutils::launchLink(((FXButton*)sender)->getText());
    return 1;
}

long PluginDialog::onLinkMotion(FXObject *sender, FXSelector, void*)
{
    ((FXButton*)sender)->setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
    return 1;
}


