/*
 *      accounteditor.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "plugineditor.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "dxutils.h"

//{ FOX method mapping

FXDEFMAP(PluginEditor) PluginEditorMap[] =
{
    FXMAPFUNC(FX::SEL_COMMAND,          PluginEditor_SAVE,         PluginEditor::onSave),
    FXMAPFUNC(FX::SEL_COMMAND,          PluginEditor_CANCEL,       PluginEditor::onCancel),
    FXMAPFUNC(FX::SEL_CLOSE,            0,                         PluginEditor::onCancel),
    FXMAPFUNC(FX::SEL_KEYPRESS,         0,                         PluginEditor::onKeyPress)
};

FXIMPLEMENT(PluginEditor, FXDialogBox, PluginEditorMap, ARRAYNUMBER(PluginEditorMap))

//}

//{ Constructor & Destructor

/* Constructor - Modify existing plugin */
PluginEditor::PluginEditor(FXDialogBox *owner, PURPLE::Plugin plugin)
: FXDialogBox(owner, _("Modify plugin"), DECOR_RESIZE|DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
{
    this->m_plugin = plugin;
    this->m_options = plugin.getOptions();
    this->initializeEditor();
}

/* Destructor */
PluginEditor::~PluginEditor()
{
}

//}

//{ Methods

/* Initializies editor */
void PluginEditor::initializeEditor()
{
    FXHorizontalFrame *buttonFrame = new FXHorizontalFrame(this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonFrame, _("&Cancel"), NULL, this, PluginEditor_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonFrame, _("&OK"), NULL, this, PluginEditor_SAVE, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);

    FXVerticalFrame *contents = new FXVerticalFrame(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(contents, dxutils::fromStdString(m_plugin.getName()), NULL, JUSTIFY_LEFT);
    FXMatrix *matrix = new FXMatrix(contents, 2, MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
    FXTextField *text;
    FXListBox *combo;
    FXCheckButton *check;
    FXSpinner *spinner;
    FOREACH(PURPLE::Plugin::Options, m_options, it)
    {
        PURPLE::Plugin::Option& option = *it;
        std::vector<std::string> svalues = option.getStringList();
        std::map<std::string, int> ivalues = option.getIntList();
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                new FXLabel(matrix, dxutils::normalizeLabelString(option.getText(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                text = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                text->setText(dxutils::fromStdString(option.getStringValue()));
                m_optionsFields.append(text);
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                new FXLabel(matrix, dxutils::normalizeLabelString(option.getText(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                combo = new FXListBox(matrix, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                std::vector<std::string>::iterator value = svalues.begin();
                FXint i=0;
                for(; value!=svalues.end(); ++value, i++)
                {
                    combo->appendItem(dxutils::fromStdString(*value));
                    if(!option.getStringValue().compare(*value))
                        combo->setCurrentItem(i);
                }
                m_optionsFields.append(combo);
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                new FXLabel(matrix, dxutils::normalizeLabelString(option.getText(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                spinner = new FXSpinner(matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                spinner->setRange(option.getIntMin(), option.getIntMax());
                spinner->setValue(option.getIntValue());
                m_optionsFields.append(spinner);
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                new FXLabel(matrix, dxutils::normalizeLabelString(option.getText(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                combo = new FXListBox(matrix, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                std::map<std::string, int>::iterator value = ivalues.begin();
                FXint i=0;
                for(; value!=ivalues.end(); ++value, i++)
                {
                    combo->appendItem(dxutils::fromStdString((*value).first));
                    if(option.getIntValue() == (*value).second)
                        combo->setCurrentItem(i);
                }
                m_optionsFields.append(combo);
                break;
            }
            case PURPLE::Plugin::Option::BOOL:
            {
                new FXLabel(matrix, dxutils::normalizeLabelString(option.getText(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                check = new FXCheckButton(matrix, "", NULL, 0);
                check->setCheck(option.getBoolValue());
                m_optionsFields.append(check);
                break;
            }
            case PURPLE::Plugin::Option::NONE:
                new FXLabel(matrix, dxutils::normalizeLabelString(option.getText(), '_'), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                new FXLabel(matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                m_optionsFields.append(NULL);
                break;
        }
    }
}

int PluginEditor::getIntListValue(std::map<std::string,int> map, std::string key)
{
    std::map<std::string, int>::iterator it = map.find(key);
    if(it!=map.end())
        return (*it).second;
    return -1;
}

void PluginEditor::saveOptions()
{
    PURPLE::Plugin::Options::iterator it;
    FXuint i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Plugin::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                option.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_optionsFields[i])->getText()));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                FXListBox *combo = static_cast<FXListBox*>(m_optionsFields[i]);
                option.setValue(dxutils::toStdString(combo->getItemText(combo->getCurrentItem())));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                option.setValue(dxutils::toStdString(FXStringVal(static_cast<FXSpinner*>(m_optionsFields[i])->getValue())));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                FXListBox *combo = static_cast<FXListBox*>(m_optionsFields[i]);
                option.setValue(dxutils::toStdString(FXStringVal(getIntListValue(option.getIntList(), dxutils::toStdString(combo->getItemText(combo->getCurrentItem()))))));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::BOOL:
            {
                option.setValue(static_cast<FXCheckButton*>(m_optionsFields[i])->getCheck() ? "true" : "false");
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::NONE:
                /* noting to save. */
                break;
        }
    }
}

/* Handles cancelation of modifying plugin */
long PluginEditor::onCancel(FXObject *, FXSelector, void *)
{
    // Nothing to do..
    getApp()->stopModal(this, FALSE);
    this->hide();
    return 1;
}

/* Handles modifying plugin */
long PluginEditor::onSave(FXObject *, FXSelector, void *)
{
    saveOptions();
    getApp()->stopModal(this, TRUE);
    this->hide();
    return 1;
}

/* Handles users key pressing */
long PluginEditor::onKeyPress(FXObject *sender, FXSelector selector, void *ptr)
{
    if(FXTopWindow::onKeyPress(sender, selector, ptr))
    {
        return 1;
    }

    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        handle(this, FXSEL(SEL_COMMAND, PluginEditor_CANCEL), NULL);
        return 1;
    }

    return 0;
}


//}
