/*
 *      roomlistdialog.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "roomlistdialog.h"
#include "chatjoindialog.h"
#include "config.h"
#include "i18n.h"
#include "callback/callback.h"

FXDEFMAP(RoomListDialog) RoomListDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, RoomListDialog_JOIN, RoomListDialog::onJoin),
    FXMAPFUNC(SEL_COMMAND, RoomListDialog_CANCEL, RoomListDialog::onCancel),
    FXMAPFUNC(SEL_CLOSE, 0, RoomListDialog::onCancel),
    FXMAPFUNC(SEL_KEYPRESS, 0, RoomListDialog::onKeyPress),
    FXMAPFUNC(SEL_COMMAND, RoomListDialog_TABLE, RoomListDialog::onTable),
    FXMAPFUNC(SEL_CHANGED, RoomListDialog_SEARCH, RoomListDialog::onSearch)
};

FXIMPLEMENT(RoomListDialog, FXDialogBox, RoomListDialogMap, ARRAYNUMBER(RoomListDialogMap))

RoomListDialog::RoomListDialog(ChatJoinDialog *owner, PURPLE::RoomList *list)
        : FXDialogBox(owner, _("Chats list"), DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE, 0,0,0,0, 0,0,0,0, 0,0), m_owner(owner), m_list(list)
{
    m_contents = new FXVerticalFrame(this, LAYOUT_FILL_X|LAYOUT_FILL_Y);

    new FXLabel(m_contents, dxutils::fromStdString(m_list->getAccount().getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(m_list->getAccount().getProtocol().getPurpleID())), ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);

    m_searchframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    new FXLabel(m_searchframe, _("Search"));
    m_search = new FXTextField(m_searchframe, 25, this, RoomListDialog_SEARCH, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X);
    m_search->disable();

    m_tableframe = new FXVerticalFrame(m_contents, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK);
    m_table = new FXIconList(m_tableframe, this, RoomListDialog_TABLE, ICONLIST_AUTOSIZE|ICONLIST_DETAILED|ICONLIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_table->appendHeader(_("Chat"), NULL, 150);
    m_table->setSortFunc(FXIconList::ascendingCase);

    m_buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    m_buttonCancel = new dxEXButton(m_buttonframe, _("&Cancel"), NULL, this, RoomListDialog_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonJoin = new dxEXButton(m_buttonframe, _("&Select"), NULL, this, RoomListDialog_JOIN, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,2);
    m_buttonJoin->disable();
}

RoomListDialog::~RoomListDialog()
{
}

long RoomListDialog::onJoin(FXObject*, FXSelector, void*)
{
    m_owner->setRoom(m_table->getItemText(m_table->getCurrentItem()).before('\t'));
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long RoomListDialog::onCancel(FXObject*, FXSelector, void*)
{
    m_list->cancelListing();
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}

long RoomListDialog::onKeyPress(FXObject *sender, FXSelector sel, void *ptr)
{
    if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),NULL);
        return 1;
    }
    return 0;
}

long RoomListDialog::onSearch(FXObject *, FXSelector, void *)
{
    m_table->clearItems();
    if(m_search->getText().empty())
    {
        for(int i=0; i<m_rooms.no(); i++)
        {
            m_table->appendItem(m_rooms[i]);
        }
    }
    else
    {
        for(FXint i=0; i<m_rooms.no(); i++)
        {
            if(dxutils::getParam(m_rooms[i], 1, FALSE, '\t').find(m_search->getText()) != -1)
            {
                m_table->appendItem(m_rooms[i]);
            }
        }
    }
    m_table->sortItems();
    return 1;
}

long RoomListDialog::onTable(FXObject*, FXSelector, void *)
{
    m_buttonJoin->enable();
    return 1;
}

void RoomListDialog::createColumns(std::vector<std::string> columns)
{
    for(size_t i=0; i<columns.size(); i++)
    {
        m_table->appendHeader(dxutils::fromStdString(columns[i]), NULL, 150);
    }
    FXDialogBox::create();
    resize(170+150*columns.size(), 250);
    update();
}

void RoomListDialog::addRoom(std::vector<std::string> values)
{
    FXString item = "";
    for(size_t i=0; i<values.size(); i++)
    {
        item.append(dxutils::fromStdString(values[i]));
        item.append(TAB);
    }
    m_rooms.append(item);
    m_table->appendItem(item);
}

void RoomListDialog::listFinished()
{
    if(m_table->getNumItems())
    {
        m_buttonJoin->enable();
        m_search->enable();
    }
    m_table->sortItems();
}
