/*      dxim.cpp
*
*      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 3 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
*      MA 02110-1301, USA.
*/

#include "config.h"
#ifdef _WIN32
  #define _WIN32_LEAN_AND_MEAN
  #include <windows.h>
#else
  #include <sys/file.h>
  #include <errno.h>
  #include <signal.h>
  #include <unistd.h>
#ifdef HAVE_X11
  #include <X11/Xlib.h>
#endif //HAVE_X11
#ifdef USE_SCREENSAVER
  #include <X11/Xutil.h>
  #include <X11/extensions/scrnsaver.h>
#endif //USE_SCREENSAVER
#endif //_WIN32
#include "defs.h"
#include "callback/callback.h"
#include "dxutils.h"
#include "FXTrayApp.h"
#include "tabitems/dxtabbook.h"
#include "dialogs/logviewer.h"
#include "i18n.h"
#include "icons.h"
#include "dialogs/aboutdialog.h"
#include "roster/roster.h"
#include "dialogs/configdialog.h"
#include "tabitems/dxtabitem.h"
#include "tabitems/imtabitem.h"
#include "tabitems/msgtabitem.h"
#include "tabitems/chattabitem.h"
#include "tabitems/texttabitem.h"
#include "dialogs/chatdialog.h"
#include "dialogs/chatjoindialog.h"
#include "dialogs/imdialog.h"
#include "dialogs/accountdialog.h"
#include "dialogs/ftdialog.h"
#include "dialogs/roomlistdialog.h"
#include "dialogs/actionrequestdialog.h"
#include "dialogs/inputrequestdialog.h"
#include "dialogs/fieldsrequestdialog.h"
#include "dialogs/choicerequestdialog.h"
#include "dialogs/plugindialog.h"
#include "dialogs/accounteditor.h"
#include "dxim.h"
#include <utilsmm.h>

#define DISPLAY(app) ((Display*)((app)->getDisplay()))

int argn; //argc for restart
char** args; //argv for restart

#ifndef _WIN32
void dxunlink(void)
{
    unlink(FXString(FXSystem::getTempDirectory()+PATHSEPSTRING+"dxim-"+FXSystem::currentUserName()).text());
}
#endif

// For handling Single Instance of Application
FXbool isInstanceRunningAlready()
{
#ifdef _WIN32
    HANDLE hSingleInstance = NULL;
    hSingleInstance = CreateMutex(NULL, FALSE,TEXT("dxim-fox.exe"));
    DWORD dwLastError = GetLastError();
    if(dwLastError == ERROR_ALREADY_EXISTS)
    {
        MessageBox(NULL, TEXT("dxim already running"), TEXT("dxim"), MB_OK | MB_ICONINFORMATION);
        CloseHandle(hSingleInstance);
        return TRUE;
    }
#else
    FXString path = FXSystem::getTempDirectory()+PATHSEPSTRING+"dxim-fox-"+FXSystem::currentUserName();
    struct flock fl;
    int fd;
    fd = open(path.text(), O_RDWR | O_CREAT, 0600);
    if(fd < 0)
    {
        perror(path.text());
        _exit(1);
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    if(fcntl(fd, F_SETLK, &fl) < 0)
    {
        FXApp application("dxim");
        application.init(argn, args);
        FXMainWindow *main = new FXMainWindow(&application, "dxim", NULL, NULL, DECOR_TITLE|DECOR_CLOSE);
        new FXLabel(main, "dxim already running");
        new FXButton(main, "C&lose", NULL, &application, FXApp::ID_QUIT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X);
        application.create();
        main->show(PLACEMENT_SCREEN);
        application.run();
        _exit(1);
    }
    atexit(dxunlink);
#endif
    return FALSE;
}

/*from Goggles Music Manager
thanks Sander Jansen */
#ifdef ENABLE_NLS
#include <FXTranslator.h>
#include <fxdefs.h>

class dxTranslator : public FXTranslator
{
    FXDECLARE(dxTranslator)
private:
    dxTranslator(const dxTranslator&);
    dxTranslator & operator=(const dxTranslator&);
protected:
    dxTranslator() {
    }
public:
    /// Construct translator

    dxTranslator(FXApp* a) : FXTranslator(a) {
#ifndef _WIN32
        setlocale(LC_MESSAGES, "");
        setlocale(LC_NUMERIC, "C");
#endif //_WIN32
        bindtextdomain("dxim", LOCALEDIR);
        bind_textdomain_codeset("dxim", "UTF-8");
        bind_textdomain_codeset("pidgin", "UTF-8");
        textdomain("dxim");
        dxutils::debugLine(FXStringFormat("localedir: %s", LOCALEDIR));
    }

    virtual const FXchar* tr(const FXchar* context, const FXchar* message, const FXchar* hint = NULL) const;

    ~dxTranslator() {}
};

FXIMPLEMENT(dxTranslator, FXTranslator, NULL, 0)

const FXchar* dxTranslator::tr(const FXchar*, const FXchar* message, const FXchar*) const {
    return gettext(message);
}
#endif //ENABLE_NLS

// Message Map for the dxim class
FXDEFMAP(dxim) dximMap[] = {
    //________Message_Type_____________ID_____________Message_Handler_______
    FXMAPFUNC(SEL_SIGNAL, dxim_QUIT, dxim::onCmdQuit),
    FXMAPFUNC(SEL_COMMAND, dxim_QUIT, dxim::onCmdQuit),
    FXMAPFUNC(SEL_COMMAND, dxim_RESTART, dxim::onCmdRestart),
    FXMAPFUNC(SEL_COMMAND, dxim_ABOUT, dxim::onCmdAbout),
    FXMAPFUNC(SEL_COMMAND, dxim_ACCOUNTS, dxim::onAccounts),
    FXMAPFUNC(SEL_COMMAND, dxim_ADDBUDDY, dxim::onCmdAddBuddy),
    FXMAPFUNC(SEL_COMMAND, dxim_ADDCHAT, dxim::onCmdAddChat),
    FXMAPFUNC(SEL_COMMAND, dxim_ADDGROUP, dxim::onCmdAddGroup),
    FXMAPFUNC(SEL_COMMAND, dxim_CHAT, dxim::onCmdJoinChat),
    FXMAPFUNC(SEL_COMMAND, dxim_TRAY_CLICKED, dxim::onTrayRestore),
    FXMAPFUNC(SEL_CLOSE, 0, dxim::onClose),
    FXMAPFUNC(SEL_TIMEOUT, dxim_AWAYTIMEOUT, dxim::onAwayTimeout),
    FXMAPFUNC(SEL_COMMAND, dxim_CLEAR, dxim::onCmdClear),
    FXMAPFUNC(SEL_COMMAND, dxim_CLEARALL, dxim::onCmdClearAll),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, dxim_ROSTER, dxim::onRightRelease),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPIM, dxim::onPopupIM),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPINFO, dxim::onPopupInfo),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPCHAT, dxim::onPopupChat),
    FXMAPFUNC(SEL_COMMAND, dxim_CLOSETAB, dxim::onCmdCloseTab),
    FXMAPFUNC(SEL_COMMAND, dxim_TABS, dxim::onTabBook),
    FXMAPFUNC(SEL_MOUSEWHEEL, dxim_TABS, dxim::onMouseWheel),
    FXMAPFUNC(SEL_COMMAND, dxim_NEXTTAB, dxim::onCmdNextTab),
    FXMAPFUNC(SEL_COMMAND, dxim_NEXTUNREAD, dxim::onCmdNextUnread),
    FXMAPFUNC(SEL_COMMAND, dxim_RECONNECT, dxim::onCmdReconnect),
    FXMAPFUNCS(SEL_COMMAND, dxim_AVAILABLE, dxim_OFFLINE, dxim::onCmdStatus),
    FXMAPFUNC(SEL_COMMAND, dxim_STATUSMSG, dxim::onCmdStatusMsg),
    FXMAPFUNC(SEL_DOUBLECLICKED, dxim_ROSTER, dxim::onLeftDoubleClick),
    FXMAPFUNC(SEL_COMMAND, dxim_PLUGINS, dxim::onCmdPlugins),
    FXMAPFUNC(SEL_COMMAND, dxim_OPTIONS, dxim::onCmdOptions),
    FXMAPFUNC(SEL_COMMAND, dxim_LOG, dxim::onCmdLog),
    FXMAPFUNC(SEL_COMMAND, dxim_HIDEROSTER, dxim::onCmdHideroster),
    FXMAPFUNC(SEL_COMMAND, dxim_HIDECHATROSTER, dxim::onCmdChatHideroster),
    FXMAPFUNC(SEL_COMMAND, dxim_SHOWAVATARS, dxim::onCmdShowAvatars),
    FXMAPFUNC(SEL_COMMAND, dxim_HIDEOFFLINE, dxim::onCmdHideOffline),
    FXMAPFUNC(SEL_COMMAND, dxim_SHOWSTATUSMSG, dxim::onCmdShowStatusmsg),
    FXMAPFUNC(SEL_COMMAND, dxim_SELECTTAB, dxim::onCmdSelectTab),
    FXMAPFUNC(SEL_COMMAND, dxim_IM, dxim::onCmdIM),
    FXMAPFUNC(SEL_COMMAND, dxim_TRANSFERS, dxim::onCmdTransfers),
    FXMAPFUNC(SEL_COMMAND, ImTabItem_HIGHLIGHT, dxim::onHighlight),
    FXMAPFUNC(SEL_COMMAND, ChatTabItem_HIGHLIGHT, dxim::onHighlight),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPSENDFILE, dxim::onPopupSendFile),
    FXMAPFUNC(SEL_TIMEOUT, dxim_NOTIFYHIDE, dxim::onNotifyHide),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPACCOUNTSTATUS, dxim::onPopupAccountStatus),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPBUDDYCB, dxim::onPopupBuddyCb),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPACCOUNTCB, dxim::onPopupAccountCb),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPADDBUDDY, dxim::onPopupAddBuddy),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPADDCHAT, dxim::onPopupAddChat),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPADDGROUP, dxim::onPopupAddGroup),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPRENAMEBUDDY, dxim::onPopupRenameBuddy),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPMOVEBUDDY, dxim::onPopupMoveBuddy),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPREMOVEBUDDY, dxim::onPopupRemoveBuddy),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPUPDATECHAT, dxim::onPopupUpdateChat),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPREMOVECHAT, dxim::onPopupRemoveChat),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPENABLEACCOUNT, dxim::onPopupEnableAccount),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPCONFIGACCOUNT, dxim::onPopupConfigAccount),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPALIASACCOUNT, dxim::onPopupAliasAccount),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPRECONNECTACCOUNT, dxim::onPopupReconnectAccount),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPACCOUNTIM, dxim::onPopupAccountIM),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPACCOUNTCHAT, dxim::onPopupAccountChat),
    FXMAPFUNC(SEL_COMMAND, dxim_POPUPHISTORY, dxim::onPopupHistory),
    FXMAPFUNC(SEL_COMMAND, dxTabItem_SHOWLOG, dxim::onShowLog),
    //Protocol events
    FXMAPFUNC(DX_BUDDYLOADED, Purple_EVENT, dxim::onBuddylistLoaded),
    FXMAPFUNC(DX_NEWACCOUNT, Purple_EVENT, dxim::onAccountAdded),
    FXMAPFUNC(DX_ACCOUNTREMOVED, Purple_EVENT, dxim::onAccountRemoved),
    FXMAPFUNC(DX_ACCOUNTPRESENCE, Purple_EVENT, dxim::onAccountPresence),
    FXMAPFUNC(DX_CONNECTING, Purple_EVENT, dxim::onConnecting),
    FXMAPFUNC(DX_CONNECTED, Purple_EVENT, dxim::onConnected),
    FXMAPFUNC(DX_DISCONNECTED, Purple_EVENT, dxim::onDisconnected),
    FXMAPFUNC(DX_SIGNEDON, Purple_EVENT, dxim::onSignedOn),
    FXMAPFUNC(DX_NEWTAB, Purple_EVENT, dxim::onNewTab),
    FXMAPFUNC(DX_CLOSETAB, Purple_EVENT, dxim::onCloseTab),
    FXMAPFUNC(DX_NEWMYMSG, Purple_EVENT, dxim::onNewMyMsg),
    FXMAPFUNC(DX_MESSAGERECIVED, Purple_EVENT, dxim::onMessageRecived),
    FXMAPFUNC(DX_NEWMSG, Purple_EVENT, dxim::onNewMsg),
    FXMAPFUNC(DX_NEWSYSTEMMSG, Purple_EVENT, dxim::onNewSystemMsg),
    FXMAPFUNC(DX_NEWERRORMSG, Purple_EVENT, dxim::onNewErrorMsg),
    FXMAPFUNC(DX_BUDDYSIGNEDON, Purple_EVENT, dxim::onBuddySignedOn),
    FXMAPFUNC(DX_BUDDYSIGNEDOFF, Purple_EVENT, dxim::onBuddySignedOff),
    FXMAPFUNC(DX_BUDDYPRESENCE, Purple_EVENT, dxim::onBuddyPresence),
    FXMAPFUNC(DX_BUDDYICON, Purple_EVENT, dxim::onBuddyIcon),
    FXMAPFUNC(DX_CHATTOPIC, Purple_EVENT, dxim::onChatTopic),
    FXMAPFUNC(DX_NEWROOMLIST, Purple_EVENT, dxim::onNewRoomList),
    FXMAPFUNC(DX_ROOMLISTCOLUMNS, Purple_EVENT, dxim::onRoomListColumns),
    FXMAPFUNC(DX_ROOMLISTROOM, Purple_EVENT, dxim::onRoomListRoom),
    FXMAPFUNC(DX_ROOMLISTFINISHED, Purple_EVENT, dxim::onRoomListFinished),
    FXMAPFUNC(DX_CHOICEREQUEST, Purple_EVENT, dxim::onNewChoiceRequest),
    FXMAPFUNC(DX_ACTIONREQUEST, Purple_EVENT, dxim::onNewActionRequest),
    FXMAPFUNC(DX_INPUTREQUEST, Purple_EVENT, dxim::onNewInputRequest),
    FXMAPFUNC(DX_FILEREQUEST, Purple_EVENT, dxim::onNewFileRequest),
    FXMAPFUNC(DX_FIELDSREQUEST, Purple_EVENT, dxim::onNewFieldsRequest),
    FXMAPFUNC(DX_NEWFT, Purple_EVENT, dxim::onNewFileTransfer),
    FXMAPFUNC(DX_STARTEDFT, Purple_EVENT, dxim::onStartedFileTransfer),
    FXMAPFUNC(DX_FINISHEDFT, Purple_EVENT, dxim::onFinishedFileTransfer),
    FXMAPFUNC(DX_CANCELEDFT, Purple_EVENT, dxim::onCanceledFileTransfer),
    FXMAPFUNC(DX_PROGRESSFT, Purple_EVENT, dxim::onProgressFileTransfer),
    FXMAPFUNC(DX_NEWNOTIFYMSG, Purple_EVENT, dxim::onNewNotifyMsg),
    FXMAPFUNC(DX_CHATBUDDY, Purple_EVENT, dxim::onChatBuddyEvent),
    FXMAPFUNC(DX_USERINFO, Purple_EVENT, dxim::onUserInfo),
    FXMAPFUNC(DX_NEWBUDDY, Purple_EVENT, dxim::onNewBuddy),
    FXMAPFUNC(DX_BUDDYREMOVED, Purple_EVENT, dxim::onBuddyRemoved),
    FXMAPFUNC(DX_NEWCHAT, Purple_EVENT, dxim::onNewChat),
    FXMAPFUNC(DX_CHATREMOVED, Purple_EVENT, dxim::onChatRemoved),
    FXMAPFUNC(DX_BUDDYTYPING, Purple_EVENT, dxim::onBuddyTyping)
};

//Macro for the dxim class hierarchy implementation
FXIMPLEMENT(dxim, FXMainWindow, dximMap, ARRAYNUMBER(dximMap))

dxim *dxim::_pThis = NULL;

//dxim constructor
dxim::dxim(FXApp *ap, bool offline)
: FXMainWindow(ap, "dxim", 0, 0, DECOR_ALL, 0, 0, 800, 600), m_app(ap)
{
    _pThis = this;

    setIcon(ICO_BIGICON);
    setMiniIcon(ICO_SMALLICON);

    m_itemOnRight = NULL;
    m_imFont = NULL;
    m_trayMenu = NULL;
    m_trayStatusMenu = NULL;
    m_viewer = NULL;
    m_presence = PURPLE_STATUS_OFFLINE;
    m_firstNotify = false;
    m_notifiesHeight = 0;
    m_chatDialog = NULL;
    m_ftDialog = NULL;
    m_newMsg = false;

    DXPREFERENCES.load();
    if(!DXPREFERENCES.m_imFont.empty())
    {
        m_imFont = new FXFont(m_app, DXPREFERENCES.m_imFont);
        m_imFont->create();
    }
    else
    {
        m_app->getNormalFont()->create();
        FXFontDesc fontdescription;
        m_app->getNormalFont()->getFontDesc(fontdescription);
        m_imFont = new FXFont(m_app,fontdescription);
        m_imFont->create();
    }

    m_menubar = new FXMenuBar(this, FRAME_RAISED | LAYOUT_SIDE_TOP | LAYOUT_FILL_X);

    m_mainframe = new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_splitter = new FXSplitter(m_mainframe, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y | SPLITTER_REVERSED | SPLITTER_TRACKING);
    m_tabframe = new FXVerticalFrame(m_splitter, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_tabbook = new dxTabBook(m_tabframe, this, dxim_TABS, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_rosterframe = new FXVerticalFrame(m_splitter, FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_WIDTH);
    m_rosterList = new dxRosterList(m_rosterframe, this, dxim_ROSTER, FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_rosterList->setSortFunc(FXTreeList::ascendingCase);
    if(!DXPREFERENCES.m_rosterShown)
    {
        m_rosterframe->hide();
        m_rosterList->hide();
        m_splitter->setSplit(1,0);
    }

    setTempKeybindings(DXPREFERENCES.m_keybindings);
    m_filemenu = new FXMenuPane(this);
    new FXMenuCommand(m_filemenu, _("Manage accounts")+TAB+DXPREFERENCES.getBinding("manageAccounts"), NULL, this, dxim_ACCOUNTS);
    new FXMenuSeparator(m_filemenu);
    new FXMenuCommand(m_filemenu, _("Add buddy"), NULL, this, dxim_ADDBUDDY);
    new FXMenuCommand(m_filemenu, _("Add chat"), NULL, this, dxim_ADDCHAT);
    new FXMenuCommand(m_filemenu, _("Add group"), NULL, this, dxim_ADDGROUP);
    new FXMenuSeparator(m_filemenu);
    new FXMenuCommand(m_filemenu, _("New IM")+TAB+DXPREFERENCES.getBinding("newIM"), NULL, this, dxim_IM);
    new FXMenuCommand(m_filemenu, _("Join chat")+TAB+DXPREFERENCES.getBinding("joinChat"), NULL, this, dxim_CHAT);
    new FXMenuSeparator(m_filemenu);
    new FXMenuCommand(m_filemenu, _("File &transfers")+TAB+DXPREFERENCES.getBinding("fileTransfers"), ICO_TRANSFER, this, dxim_TRANSFERS);
    new FXMenuSeparator(m_filemenu);
    m_logviewer = new FXMenuCommand(m_filemenu, _("&Log viewer")+TAB+DXPREFERENCES.getBinding("logViewer"), ICO_LOGS, this, dxim_LOG);
    if(!DXPREFERENCES.m_logging) m_logviewer->disable();
    new FXMenuSeparator(m_filemenu);
    new FXMenuCommand(m_filemenu, _("&Quit")+TAB+DXPREFERENCES.getBinding("quit"), NULL, this, dxim_QUIT);
    new FXMenuTitle(m_menubar, "&dxim", NULL, m_filemenu);

    m_editmenu = new FXMenuPane(this);
    m_closeTab = new FXMenuCommand(m_editmenu, _("Close current tab")+TAB+DXPREFERENCES.getBinding("closeCurrentTab"), NULL, this, dxim_CLOSETAB);
    m_closeTab->disable();
    new FXMenuSeparator(m_editmenu);
    m_clearTab = new FXMenuCommand(m_editmenu, _("Clear window")+TAB+DXPREFERENCES.getBinding("clearWindow"), NULL, this, dxim_CLEAR);
    m_clearTabs = new FXMenuCommand(m_editmenu, _("Clear all windows")+TAB+DXPREFERENCES.getBinding("clearAllWindows"), NULL, this, dxim_CLEARALL);
    new FXMenuSeparator(m_editmenu);
    (new FXMenuCheck(m_editmenu, _("Friends list")+TAB+DXPREFERENCES.getBinding("friendsList")+TAB+_("Show/Hide friends list"), this, dxim_HIDEROSTER))->setCheck(DXPREFERENCES.m_rosterShown);
    (new FXMenuCheck(m_editmenu, _("Friends list in chats\t\tShow/Hide friends list in chats"), this, dxim_HIDECHATROSTER))->setCheck(DXPREFERENCES.m_chatRosterShown);
    (new FXMenuCheck(m_editmenu, _("Show avatars\t\tShow/Hide avatars"), this, dxim_SHOWAVATARS))->setCheck(DXPREFERENCES.m_showAvatars);
    (new FXMenuCheck(m_editmenu, _("Hide offline friends\t\tShow/Hide offline friends"), this, dxim_HIDEOFFLINE))->setCheck(DXPREFERENCES.m_hideOffline);
    (new FXMenuCheck(m_editmenu, _("Show friends status message\t\tShow/Hide friends status message"), this, dxim_SHOWSTATUSMSG))->setCheck(DXPREFERENCES.m_showStatusmsg);
    new FXMenuSeparator(m_editmenu);
    new FXMenuCommand(m_editmenu, _("Plugins"), NULL, this, dxim_PLUGINS);
    new FXMenuCommand(m_editmenu, _("&Preferences"), NULL, this, dxim_OPTIONS);
    new FXMenuTitle(m_menubar, _("&Edit"), NULL, m_editmenu);

    m_statusmenu = new FXMenuPane(this);
    new FXMenuCommand(m_statusmenu, _("Available"), ICO_ONLINE, this, dxim_AVAILABLE);
    new FXMenuCommand(m_statusmenu, _("Away"), ICO_AWAY, this, dxim_AWAY);
    new FXMenuCommand(m_statusmenu, _("Busy"), ICO_DND, this, dxim_BUSY);
    new FXMenuCommand(m_statusmenu, _("Invisible"), ICO_INVISIBLE, this, dxim_INVISIBLE);
    new FXMenuSeparator(m_statusmenu);
    new FXMenuCommand(m_statusmenu, _("Set status message"), NULL, this, dxim_STATUSMSG);
    new FXMenuSeparator(m_statusmenu);
    new FXMenuCommand(m_statusmenu, _("Offline"), ICO_OFFLINE, this, dxim_OFFLINE);
    new FXMenuSeparator(m_statusmenu);
    new FXMenuCommand(m_statusmenu, _("Reconnect"), NULL, this, dxim_RECONNECT);
    new FXMenuTitle(m_menubar, _("&Status"), NULL, m_statusmenu);

    m_helpmenu = new FXMenuPane(this);
    new FXMenuCommand(m_helpmenu, _("&About..."), NULL, this, dxim_ABOUT);
    new FXMenuTitle(m_menubar, _("&Help"), NULL, m_helpmenu);

#ifdef HAVE_TRAY
    if(DXPREFERENCES.m_useTray)
    {
        m_trayIcon = new FXTrayIcon(m_app, "dxim", ICO_OFFLINE, 0, this, dxim_TRAY_CLICKED, TRAY_CMD_ON_LEFT|TRAY_MENU_ON_RIGHT);
        if(m_trayMenu==NULL)
            m_trayMenu = new FXPopup(m_trayIcon);
        if(m_trayStatusMenu==NULL)
            m_trayStatusMenu = new FXPopup(m_trayIcon);
        new FXMenuCommand(m_trayStatusMenu, _("Available"), ICO_ONLINE, this, dxim_AVAILABLE);
        new FXMenuCommand(m_trayStatusMenu, _("Away"), ICO_AWAY, this, dxim_AWAY);
        new FXMenuCommand(m_trayStatusMenu, _("Busy"), ICO_DND, this, dxim_BUSY);
        new FXMenuCommand(m_trayStatusMenu, _("Invisible"), ICO_INVISIBLE, this, dxim_INVISIBLE);
        new FXMenuSeparator(m_trayStatusMenu);
        new FXMenuCommand(m_trayStatusMenu, _("Set status message"), NULL, this, dxim_STATUSMSG);
        m_statusCascade = new FXMenuCascade(m_trayMenu, _("Status"), NULL, m_trayStatusMenu);
        new FXMenuSeparator(m_trayMenu);
        new FXMenuCommand(m_trayMenu, _("New IM"), NULL, this, dxim_IM);
        new FXMenuCommand(m_trayMenu, _("Join chat"), NULL, this, dxim_CHAT);
        new FXMenuSeparator(m_trayMenu);
        new FXMenuCommand(m_trayMenu, _("&Quit"), NULL, this, dxim_QUIT);
        m_trayIcon->setMenu(m_trayMenu);
    }
#endif

    new FXToolTip(m_app,TOOLTIP_VARIABLE);

    m_cb = new dxfCallbacks(m_app, this, offline);

    //set mainwindow size and position
    setX(FXMAX(0,DXPREFERENCES.m_x));
    setY(FXMAX(0,DXPREFERENCES.m_y));
    setWidth(DXPREFERENCES.m_w);
    setHeight(DXPREFERENCES.m_h);

    updateMenu();
    updateTheme();
    updateFont();
    updateTabs(DXPREFERENCES.m_useSmileys && (FXint)DXPREFERENCES.m_smileysMap.size());
    updateTabPosition();

    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("switchTab")), this, FXSEL(SEL_COMMAND, dxim_NEXTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("unreadTab")), this, FXSEL(SEL_COMMAND, dxim_NEXTUNREAD));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("firstTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("secondTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("thirdTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("fourthTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("fifthTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("sixthTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("seventhTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("eighthTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
    getAccelTable()->addAccel(dxutils::parseAccel(DXPREFERENCES.getBinding("ninethTab")), this, FXSEL(SEL_COMMAND, dxim_SELECTTAB));
}

//dxim destructor
dxim::~dxim()
{
    delete ICO_BIGICON;
    delete ICO_SMALLICON;
    delete ICO_ABOUTICON;
    delete ICO_NEWM;
    delete ICO_AWAY;
    delete ICO_INVISIBLE;
    delete ICO_DND;
    delete ICO_OFFLINE;
    delete ICO_ONLINE;
    delete ICO_IDLE;
    delete ICO_PLAY;
    delete ICO_CLOSEFOLDER;
    delete ICO_OPENFOLDER;
    delete ICO_FILE;
    delete ICO_IM;
    delete ICO_IMNEWMSG;
    delete ICO_IMCOMPOSE;
    delete ICO_IMOFF;
    delete ICO_CHAT;
    delete ICO_CHATNEWMSG;
    delete ICO_ROSTERCHAT;
    delete ICO_SSL;
    delete ICO_BIGINFORMATION;
    delete ICO_BIGWARNING;
    delete ICO_BIGERROR;
    delete ICO_BIGQUESTION;
    delete ICO_CLEAR;
    delete ICO_SELECTALL;
    delete ICO_COPY;
    delete ICO_FIND;
    delete ICO_XMPP;
    delete ICO_WLM;
    delete ICO_DOWN;
    delete ICO_UP;
    delete ICO_CANCEL;
    delete ICO_FINISH;
    delete ICO_TRANSFER;
    delete ICO_KEYBINDINGS;
    delete ICO_REFRESH;
    delete ICO_INFORMATION;
    delete ICO_WARNING;
    delete ICO_QUESTION;
    delete ICO_EMAIL;
    delete ICO_ROSTEROPENED;
    delete ICO_ROSTERCLOSED;
    delete ICO_LOGS;
    delete ICO_ROSTERAWAY;
    delete ICO_ROSTERDND;
    delete ICO_ROSTERIDLE;
    delete ICO_ROSTERINVISIBLE;
    delete ICO_ROSTEROFFLINE;
    delete ICO_ROSTERONLINE;
    delete ICO_ROSTERCONNECTING;
    delete ICO_IRC;
    delete ICO_ICQ;
    delete ICO_AIM;
    delete ICO_BONJOUR;
    delete ICO_GG;
    delete ICO_NOVELL;
    delete ICO_MYSPACE;
    delete ICO_SILC;
    delete ICO_MEANWHILE;
    delete ICO_SIMPLE;
    delete ICO_YAHOO;
    delete ICO_SKYPE;
    delete ICO_MXIT;
    delete ICO_ZEPHYR;
    delete ICO_TELEGRAM;
    delete ICO_SLACK;
    delete ICO_CHRFOUNDER;
    delete ICO_CHRHALFOP;
    delete ICO_CHRNORMAL;
    delete ICO_CHROP;
    delete ICO_CHRVOICE;
    // Delete all menu panes
    delete m_filemenu;
    delete m_helpmenu;
    delete m_editmenu;
    delete m_statusmenu;
#ifdef HAVE_TRAY
    if(m_trayMenu)
    {
        delete m_trayMenu;
    }
    if(m_trayStatusMenu)
    {
        delete m_trayStatusMenu;
    }
#endif
    delete m_imFont;
    getApp()->removeTimeout(this, dxim_AWAYTIMEOUT);
    _pThis = NULL;
}

dxim* dxim::instance()
{
    return _pThis;
}

//Create dxim and initialize
void dxim::create()
{
    FXMainWindow::create();
#ifndef _WIN32
#ifdef HAVE_TRAY
    m_trayIcon->create();
    if(!m_trayIcon->dock())
    {
        delete m_trayIcon;
        m_trayIcon=NULL;
    }
#endif //HAVE_TRAY
#endif
    if(DXPREFERENCES.m_maximized) maximize(TRUE);
    //Checking for screen resolution and correction size, position
    else
    {
        FXint maxWidth = getRoot()->getWidth();
        FXint maxHeight = getRoot()->getHeight();
        if(getX()+getWidth() > maxWidth || getX()<0)
        {
            setX(0);
            setWidth(maxWidth);
        }
        if(getY()+getHeight() > maxHeight || getY()<0)
        {
            setY(0);
            setHeight(maxHeight);
        }
    }
    m_splitter->setSplit(1, DXPREFERENCES.m_rosterWidth);
#ifdef HAVE_TRAY
    if(DXPREFERENCES.m_useTray && DXPREFERENCES.m_minToTray && m_trayIcon)
        hide();
    else
        show();
    updateTrayIconColor();
#else
    show();
#endif //HAVE_TRAY
}

// Flash the window to get user's attention
// Taken from fox development version 1.7
void dxim::flash(FXbool yes)
{
    if(xid)
    {
#ifdef _WIN32
        FlashWindow((HWND)xid, true);
#else
#ifdef HAVE_X11
        XEvent se;
        se.xclient.type=ClientMessage;
        se.xclient.display=DISPLAY(m_app);
        se.xclient.message_type=XInternAtom(DISPLAY(m_app), "_NET_WM_STATE", 0);;
        se.xclient.format=32;
        se.xclient.window=xid;
        se.xclient.data.l[0]=yes;   // 0=_NET_WM_STATE_REMOVE, 1=_NET_WM_STATE_ADD, 2=_NET_WM_STATE_TOGGLE
        se.xclient.data.l[1]=XInternAtom(DISPLAY(m_app), "_NET_WM_STATE_DEMANDS_ATTENTION", 0);
        se.xclient.data.l[2]=0;
        se.xclient.data.l[3]=0;
        se.xclient.data.l[4]=0;
        XSendEvent(DISPLAY(m_app),XDefaultRootWindow(DISPLAY(m_app)),False,SubstructureRedirectMask|SubstructureNotifyMask,&se);
#endif
#endif
    }
}

//show notify
void dxim::showNotify(const FXString& notify, FXint pos)
{
    if(dxutils::instance().haveNotify())
    {
        FXString exec;
        exec.format("notify-send -i %s/icons/dxim.png dxim '%s'",DXIM_DATADIR,notify.text());
        system(exec.text());
    }
    else
    {
        if(!m_firstNotify)
        {
            m_firstNotify = true;
            m_notifiesHeight = 0;
        }
        dxEXNotify *wnotify = new dxEXNotify(m_app, ICO_BIGICON, "dxim", notify, m_notifiesHeight, 5000, DXPREFERENCES.m_appTheme.notifyfore, DXPREFERENCES.m_appTheme.notifyback);
        wnotify->notify(getX(), getY(), pos);
        getApp()->addTimeout(this, dxim_NOTIFYHIDE, 5000);
        m_notifiesHeight += wnotify->getDefaultHeight()+4;
    }
}

void dxim::setTempKeybindings(dxStringMap bindings)
{
    m_tempKeybindings = bindings;
}

// Quit command called
long dxim::onCmdQuit(FXObject *, FXSelector, void *)
{
    DXPREFERENCES.m_x = getX();
    DXPREFERENCES.m_y = getY();
    DXPREFERENCES.m_w = getWidth();
    DXPREFERENCES.m_h = getHeight();
    DXPREFERENCES.m_maximized = isMaximized();
    DXPREFERENCES.m_rosterWidth = m_splitter->getSplit(1);
    DXPREFERENCES.save();
    getApp()->exit(0);
    return 1;
}

// About command called
long dxim::onCmdAbout(FXObject *, FXSelector, void *)
{
    AboutDialog about(this, m_cb->getPurplemm());
    about.execute(PLACEMENT_OWNER);
    return 1;
}

//accounts command handle
long dxim::onAccounts(FXObject *, FXSelector, void *)
{
    AccountDialog dialog(this, m_cb->getPurplemm());
    dialog.execute(PLACEMENT_OWNER);
    return 1;
}

//add buddy handle
long dxim::onCmdAddBuddy(FXObject *, FXSelector, void *)
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgInfo msg;
        msg.type = MSGWARNING;
        msg.title = _("Warning");
        msg.text = _("Any account isn't connected.\nConnect someone and try again.");
        msg.okText = _("&OK");
        msg.time = FXSystem::now();
        appendMsgInfo(msg);
        return 1;
    }

    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getConnectedAccountsList();
    PURPLE::Account account;
    FXDialogBox selectAccount(this, _("Select account"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    selectAccount.getAccelTable()->addAccel(KEY_Return, this, FXDialogBox::ID_ACCEPT);
    FXVerticalFrame *contents = new FXVerticalFrame(&selectAccount, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    FXMatrix *matrix = new FXMatrix(contents, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X | LAYOUT_FILL_Y);
    new FXLabel(matrix, _("Account:"), NULL, JUSTIFY_LEFT);
    FXListBox *box = new FXListBox(matrix, this, IMDialog_ID, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    for(FXuint i=0; i<accounts.size(); i++)
    {
        box->appendItem(dxutils::fromStdString(accounts[i].getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID())));
    }
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &selectAccount, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&OK"), NULL, &selectAccount, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    if(selectAccount.execute(PLACEMENT_OWNER))
    {
        account = findAccount(box->getItem(box->getCurrentItem()), dxutils::fromStdString(accounts[box->getCurrentItem()].getProtocol().getPurpleID()));
        if(!account.isValid()) return 1;
    }
    else
    {
        return 1;
    }

    FXDialogBox addBuddy(this, _("Add friend"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    addBuddy.getAccelTable()->addAccel(KEY_Return, &addBuddy, FXDialogBox::ID_ACCEPT);
    contents = new FXVerticalFrame(&addBuddy, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(matrix, _("Account:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    new FXLabel(matrix, dxutils::fromStdString(account.getUsername()),dxutils::getProtocolIcon(dxutils::fromStdString(account.getProtocol().getPurpleID())),ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    new FXLabel(matrix, _("ID:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *id = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    new FXLabel(matrix, _("Nick:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *nick = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    new FXLabel(matrix, _("Group:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXComboBox *group = new FXComboBox(matrix, 25,NULL, 0, COMBOBOX_INSERT_FIRST|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    FXString groups = getAllGroupList(account);
    if(!groups.contains(_("Friends")))
        groups.prepend(FXStringFormat("%s\n", _("Friends")));
    group->fillItems(groups);
    buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &addBuddy, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&OK"), NULL, &addBuddy, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    if(addBuddy.execute(PLACEMENT_OWNER) && !id->getText().empty())
    {
        account.addBuddy(dxutils::toStdString(id->getText()),
                         group->getText().empty()?_("Friends"):dxutils::toStdString(group->getText()),
                         _("I want to add you in buddies"),
                         dxutils::toStdString(nick->getText()));
    }
    return 1;
}

//add chat handle
long dxim::onCmdAddChat(FXObject *, FXSelector, void *)
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgInfo msg;
        msg.type = MSGWARNING;
        msg.title = _("Warning");
        msg.text = _("Any account isn't connected.\nConnect someone and try again.");
        msg.okText = _("&OK");
        msg.time = FXSystem::now();
        appendMsgInfo(msg);
        return 1;
    }

    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getConnectedAccountsList();
    PURPLE::Account account;
    FXDialogBox selectAccount(this, _("Select account"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    selectAccount.getAccelTable()->addAccel(KEY_Return, this, FXDialogBox::ID_ACCEPT);
    FXVerticalFrame *contents = new FXVerticalFrame(&selectAccount, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    FXMatrix *matrix = new FXMatrix(contents, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X | LAYOUT_FILL_Y);
    new FXLabel(matrix, _("Account:"), NULL, JUSTIFY_LEFT);
    FXListBox *box = new FXListBox(matrix, this, IMDialog_ID, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    for(FXuint i=0; i<accounts.size(); i++)
    {
        box->appendItem(dxutils::fromStdString(accounts[i].getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID())));
    }
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &selectAccount, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&OK"), NULL, &selectAccount, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    if(selectAccount.execute(PLACEMENT_OWNER))
    {
        account = findAccount(box->getItem(box->getCurrentItem()), dxutils::fromStdString(accounts[box->getCurrentItem()].getProtocol().getPurpleID()));
        if(!account.isValid()) return 1;
    }
    else
    {
        return 1;
    }

    ChatDialog dialog(this, account);
    dialog.execute();

    return 1;
}

//add group handle
long dxim::onCmdAddGroup(FXObject *, FXSelector, void *)
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgInfo msg;
        msg.type = MSGWARNING;
        msg.title = _("Warning");
        msg.text = _("Any account isn't connected.\nConnect someone and try again.");
        msg.okText = _("&OK");
        msg.time = FXSystem::now();
        appendMsgInfo(msg);
        return 1;
    }

    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getConnectedAccountsList();
    PURPLE::Account account;
    FXDialogBox selectAccount(this, _("Select account"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    selectAccount.getAccelTable()->addAccel(KEY_Return, this, FXDialogBox::ID_ACCEPT);
    FXVerticalFrame *contents = new FXVerticalFrame(&selectAccount, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10, 0, 0);
    FXMatrix *matrix = new FXMatrix(contents, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X | LAYOUT_FILL_Y);
    new FXLabel(matrix, _("Account:"), NULL, JUSTIFY_LEFT);
    FXListBox *box = new FXListBox(matrix, this, IMDialog_ID, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    for(FXuint i=0; i<accounts.size(); i++)
    {
        box->appendItem(dxutils::fromStdString(accounts[i].getUsername()), dxutils::getProtocolIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID())));
    }
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &selectAccount, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&OK"), NULL, &selectAccount, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    if(selectAccount.execute(PLACEMENT_OWNER))
    {
        account = findAccount(box->getItem(box->getCurrentItem()), dxutils::fromStdString(accounts[box->getCurrentItem()].getProtocol().getPurpleID()));
        if(!account.isValid()) return 1;
    }
    else
    {
        return 1;
    }

    FXString group;
    if(dxEXInputDialog::getString(group, this, _("Add group"), _("New group:")))
    {
        account.addGroup(dxutils::toStdString(group));
    }

    return 1;
}

//join chat handle
long dxim::onCmdJoinChat(FXObject*, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(!this->shown())
    {
        this->show();
        updateTrayIconPresence();
    }
#endif
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgInfo msg;
        msg.type = MSGWARNING;
        msg.title = _("Warning");
        msg.text = _("Any account isn't connected.\nConnect someone and try again.");
        msg.okText = _("&OK");
        msg.time = FXSystem::now();
        appendMsgInfo(msg);
        return 1;
    }
    if(m_chatDialog) m_chatDialog = NULL;
    m_chatDialog = new ChatJoinDialog(this, m_cb->getPurplemm());
    m_chatDialog->execute(PLACEMENT_OWNER);
    return 1;
}

long dxim::onTrayRestore(FXObject*, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(this->shown())
    {
        this->hide();
    }
    else
    {
        this->show();
        updateTrayIconPresence();
    }
#endif
    return 1;
}

long dxim::onCmdRestart(FXObject*, FXSelector, void*)
{
    DXPREFERENCES.m_x = getX();
    DXPREFERENCES.m_y = getY();
    DXPREFERENCES.m_w = getWidth();
    DXPREFERENCES.m_h = getHeight();
    DXPREFERENCES.m_maximized = isMaximized();
    DXPREFERENCES.m_keybindings = m_tempKeybindings;
    DXPREFERENCES.save();
#ifdef _WIN32
    getApp()->exit();
    FXString str ;
    for(FXint i=0; i < argn; i++)
    {
        str += args[i];
        str += (i != argn - 1? " " : "");
    }
    ShellExecuteA(NULL, "open", FXPath::enquote(args[0]).text(), argn>1?str.text():NULL, NULL, SW_SHOWNORMAL);
#else
    if(fork() == 0)
        execvp(args[0], args);
    else
        _exit(0);
#endif //_WIN32
    return 1;
}

long dxim::onClose(FXObject*, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(DXPREFERENCES.m_closeToTray && m_trayIcon)
        hide();
    else
        onCmdQuit(NULL, 0, NULL);
#else
    onCmdQuit(NULL, 0, NULL);
#endif
    return 1;
}

//check inactivity for autoaway
long dxim::onAwayTimeout(FXObject*, FXSelector, void*)
{
    if(!DXPREFERENCES.m_useAway)
    {
        m_cb->getPurplemm()->idleTouch();
        getApp()->addTimeout(this, dxim_AWAYTIMEOUT, 1000);
        return 1;
    }
    FXTime idleTime = 0;
    bool away = false;
#ifdef _WIN32
    LASTINPUTINFO lif;
    lif.cbSize = sizeof(LASTINPUTINFO);
    DWORD tickCount;
    GetLastInputInfo(&lif);
    tickCount = GetTickCount();
    idleTime = (tickCount-lif.dwTime)/1000;
#else
 #ifdef USE_SCREENSAVER
    /* Query xscreensaver */
    static XScreenSaverInfo *mit_info = NULL;
    static int has_extension = -1;
    int event_base, error_base;
    if(has_extension == -1)
        has_extension = XScreenSaverQueryExtension(DISPLAY(m_app), &event_base, &error_base);
    if(has_extension)
    {
        if (mit_info == NULL)
            mit_info = XScreenSaverAllocInfo();
        XScreenSaverQueryInfo(DISPLAY(m_app), xid, mit_info);
        idleTime = (mit_info->idle)/1000;
    }
    else
        idleTime = 0;
 #endif
#endif
    if(idleTime>DXPREFERENCES.m_awayTimeout*60)
    {
        away = true;
    }
    if(!away && !idleTime)
    {
        m_cb->getPurplemm()->idleTouch();
    }
    getApp()->addTimeout(this, dxim_AWAYTIMEOUT, 1000);
    return 1;
}

long dxim::onCmdClear(FXObject*, FXSelector, void*)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        dxTabItem *currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(index));
        if(currenttab->getEnableClear()) currenttab->clearChat();
    }
    return 1;
}

long dxim::onCmdClearAll(FXObject*, FXSelector, void*)
{
    dxTabItem *currenttab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(currenttab->getEnableClear()) currenttab->clearChat();
        if(m_app->getForeColor() != currenttab->getTextColor()) currenttab->setTextColor(m_app->getForeColor());
        if(currenttab->getIcon()==ICO_CHATNEWMSG)
            currenttab->setIcon(ICO_CHAT);
        if(currenttab->getIcon()==ICO_IMNEWMSG)
        {
            currenttab->setIcon(currenttab->getConversation()->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?ICO_IMOFF:ICO_IM);
        }
    }
    updateTrayIconPresence();
    return 1;
}

//right click on roster
long dxim::onRightRelease(FXObject*, FXSelector, void *ptr)
{
    FXEvent* event = (FXEvent*)ptr;
    if(event->moved) return 1;
    dxRosterTreeItem *item = static_cast<dxRosterTreeItem*>(m_rosterList->getItemAt(event->win_x,event->win_y));
    if(item)
    {
        if(item->isFriend()) //friends
        {
            m_itemOnRight = item;
            bool online = item->getBuddy().isOnline();
            FXMenuPane *popup = new FXMenuPane(this);
            if(online) new FXMenuCommand(popup, _("IM"), NULL, this, dxim_POPUPIM);
            new FXMenuCommand(popup, _("Info"), NULL, this, dxim_POPUPINFO);
            if(online && item->getBuddy().getAccount().supportsSendFile()) new FXMenuCommand(popup, _("Send file"), NULL, this, dxim_POPUPSENDFILE);
            if(DXPREFERENCES.m_logging) new FXMenuCommand(popup, _("Show logs"), NULL, this, dxim_POPUPHISTORY);
            new FXMenuSeparator(popup);
            new FXMenuCommand(popup, _("Rename"), NULL, this, dxim_POPUPRENAMEBUDDY);
            new FXMenuCommand(popup, _("Move"), NULL, this, dxim_POPUPMOVEBUDDY);
            new FXMenuCommand(popup, _("Remove"), NULL, this, dxim_POPUPREMOVEBUDDY);
            std::vector<PURPLE::Buddy::BuddyMenuItem> v = item->getBuddy().getMenuItems();
            if(v.size()) new FXMenuSeparator(popup);
            FOREACH(std::vector<PURPLE::Buddy::BuddyMenuItem>, v, it)
            {
                new dxEXBuddyMenuCommand(popup, dxutils::fromStdString((*it).getLabel()).substitute('_','&'), (*it), NULL, this, dxim_POPUPBUDDYCB);
            }
            popup->create();
            popup->popup(NULL,event->root_x,event->root_y);
            getApp()->runModalWhileShown(popup);
            return 1;
        }
        if(item->isChat()) //bookmarked Chat
        {
            m_itemOnRight = item;
            FXMenuPane *popup = new FXMenuPane(this);
            new FXMenuCommand(popup, _("Join"), NULL, this, dxim_POPUPCHAT);
            if(DXPREFERENCES.m_logging) new FXMenuCommand(popup, _("Show logs"), NULL, this, dxim_POPUPHISTORY);
            new FXMenuSeparator(popup);
            new FXMenuCommand(popup, _("Modify"), NULL, this, dxim_POPUPUPDATECHAT);
            new FXMenuCommand(popup, _("Remove"), NULL, this, dxim_POPUPREMOVECHAT);
            popup->create();
            popup->popup(NULL,event->root_x,event->root_y);
            getApp()->runModalWhileShown(popup);
            return 1;
        }
        if(item->isAccount()) //account item
        {
            m_itemOnRight = item;
            PURPLE::Account account = item->getAccount();
            if(account.isValid())
            {
                FXMenuPane *popup = new FXMenuPane(this);
                if(account.isEnabled())
                {
                    new FXMenuCommand(popup, _("Status"), NULL, this, dxim_POPUPACCOUNTSTATUS);
                    new FXMenuSeparator(popup);
                    new FXMenuCommand(popup, _("Reconnect"), NULL, this, dxim_POPUPRECONNECTACCOUNT);
                    new FXMenuSeparator(popup);
                    new FXMenuCommand(popup, _("Disable"), NULL, this, dxim_POPUPENABLEACCOUNT);
                }
                else
                {
                    new FXMenuCommand(popup, _("Enable"), NULL, this, dxim_POPUPENABLEACCOUNT);
                }
                new FXMenuSeparator(popup);
                new FXMenuCommand(popup, _("Account settings"), NULL, this, dxim_POPUPCONFIGACCOUNT);
                new FXMenuCommand(popup, _("Alias"), NULL, this, dxim_POPUPALIASACCOUNT);
                if(DXPREFERENCES.m_logging) new FXMenuCommand(popup, _("Show logs"), NULL, this, dxim_POPUPHISTORY);
                if(account.isConnected())
                {
                    new FXMenuSeparator(popup);
                    new FXMenuCommand(popup, _("New IM"), NULL, this, dxim_POPUPACCOUNTIM);
                    if(account.supportsChats())
                    {
                        new FXMenuCommand(popup, _("Join chat"), NULL, this, dxim_POPUPACCOUNTCHAT);
                    }
                    new FXMenuSeparator(popup);
                    new FXMenuCommand(popup, _("Add buddy"), NULL, this, dxim_POPUPADDBUDDY);
                    if(account.supportsChats()) new FXMenuCommand(popup, _("Add chat"), NULL, this, dxim_POPUPADDCHAT);
                    new FXMenuCommand(popup, _("Add group"), NULL, this, dxim_POPUPADDGROUP);
                    std::vector<std::string> v = account.getCommandsList();
                    if(v.size())
                         new FXMenuSeparator(popup);
                    FOREACH(std::vector<std::string>, v, it)
                    {
                        if((*it).compare("----")) new FXMenuCommand(popup, dxutils::fromStdString((*it)), NULL, this, dxim_POPUPACCOUNTCB);
                        else new FXMenuSeparator(popup);
                    }
                }
                popup->create();
                popup->popup(NULL,event->root_x,event->root_y);
                getApp()->runModalWhileShown(popup);
            }
            return 1;
        }
    }
    return 1;
}

//handle double click in roster
long dxim::onLeftDoubleClick(FXObject*, FXSelector, void *ptr)
{
    dxRosterTreeItem *item = static_cast<dxRosterTreeItem*>(ptr);
    if(item)
    {
        if(item->isFriend())
        {
            m_cb->getPurplemm()->createConversation(item->getBuddy().getAccount(), item->getBuddy());
        }
        else if(item->isChat())
        {
            item->getChat().join();
        }
        else
        {
            if(m_rosterList->isItemExpanded(item)) m_rosterList->collapseTree(item,TRUE);
            else m_rosterList->expandTree(item,TRUE);
        }
    }
    return 1;
}

//start IM session for righ-click popup
long dxim::onPopupIM(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isFriend())
    {
        m_cb->getPurplemm()->createConversation(m_itemOnRight->getBuddy().getAccount(), m_itemOnRight->getBuddy());
    }
    return 1;
}

//start IM sessoin for righ-click popup account menu item
long dxim::onPopupAccountIM(FXObject *, FXSelector, void *)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        IMDialog newIM(this, m_itemOnRight->getAccount());
        if(newIM.execute(PLACEMENT_OWNER))
        {
            FXString with = newIM.getIM();
            if(with.empty()) return 1;
            m_cb->getPurplemm()->createConversation(m_itemOnRight->getAccount(), dxutils::toStdString(with), false);
        }
    }
    return 1;
}

//get buddy info
long dxim::onPopupInfo(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight)
    {
        if(m_itemOnRight->isFriend())
        {
            m_itemOnRight->getBuddy().retrieveInfo();
            return 1;
        }
    }
    return 1;
}

//show logs for buddy or chat
long dxim::onPopupHistory(FXObject *, FXSelector, void *)
{
    if(m_itemOnRight)
    {
        if(m_itemOnRight->isChat())
        {
            if(dxutils::hasLog(dxutils::fromStdString(m_itemOnRight->getAccount().getUsername()).before('/'), dxutils::fromStdString(m_itemOnRight->getChat().getChatName()), false))
            {
                LogViewer *viewer = new LogViewer(m_app, m_imFont, m_itemOnRight->getChat());
                viewer->create();
                viewer->show(PLACEMENT_CURSOR);
            }
            else
            {
                dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Any logs for chat %s"), m_itemOnRight->getChat().getChatName().c_str());
            }
            return 1;
        }
        if(m_itemOnRight->isFriend())
        {
            if(dxutils::hasLog(dxutils::fromStdString(m_itemOnRight->getAccount().getUsername()).before('/'), dxutils::fromStdString(m_itemOnRight->getBuddy().getUsername()), false))
            {
                LogViewer *viewer = new LogViewer(m_app, m_imFont, m_itemOnRight->getBuddy());
                viewer->create();
                viewer->show(PLACEMENT_CURSOR);
            }
            else
            {
                dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Any logs for buddy %s"), m_itemOnRight->getBuddy().getUsername().c_str());
            }
            return 1;
        }
        if(m_itemOnRight->isAccount())
        {
            if(dxutils::hasLog(dxutils::fromStdString(m_itemOnRight->getAccount().getUsername()).before('/'), "", true))
            {
                LogViewer *viewer = new LogViewer(m_app, m_imFont, m_itemOnRight->getAccount());
                viewer->create();
                viewer->show(PLACEMENT_CURSOR);
            }
            else
            {
                dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Any logs for account %s"),
                                            dxutils::fromStdString(m_itemOnRight->getAccount().getUsername()).before('/').text());
            }
            return 1;
        }
    }
    return 1;
}

//join chat for right-click popup
long dxim::onPopupChat(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight)
    {
        if(!m_itemOnRight->isChat())
            return 1;
        else
            m_itemOnRight->getChat().join();
    }
    return 1;
}

//join chat for right-click popup account menu item
long dxim::onPopupAccountChat(FXObject *, FXSelector, void *)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        if(m_chatDialog) m_chatDialog = NULL;
        m_chatDialog = new ChatJoinDialog(this, m_itemOnRight->getAccount());
        m_chatDialog->execute(PLACEMENT_OWNER);
    }
    return 1;
}

//command CloseTab called
long dxim::onCmdCloseTab(FXObject*, FXSelector, void*)
{
    if(m_tabbook->numChildren())
    {
        FXint index = m_tabbook->getCurrent()*2;
        if(m_tabbook->childAtIndex(index)->getMetaClass()==&ImTabItem::metaClass
                || m_tabbook->childAtIndex(index)->getMetaClass()==&ChatTabItem::metaClass)
        {
            dxTabItem *currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(index));
            currenttab->getConversation()->leave();
        }
        delete m_tabbook->childAtIndex(index);
        delete m_tabbook->childAtIndex(index);
        m_tabbook->recalc();
        if(m_tabbook->numChildren())
        {
            m_tabbook->setCurrent(FXMAX(0,index/2-1), TRUE);
        }
        updateMenu();
    }
    return 1;
}

//handle tab change
long dxim::onTabBook(FXObject *, FXSelector, void *ptr)
{
    FXint index = (FXint)(FXival)ptr*2;
    dxutils::debugLine(FXStringFormat("onTabBook(%d), Class: %s", index, m_tabbook->childAtIndex(index)->getClassName()));
    dxTabItem *currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(index));
    if(m_app->getForeColor() != currenttab->getTextColor()) currenttab->setTextColor(m_app->getForeColor());
    if(currenttab->getIcon()==ICO_CHATNEWMSG)
    {
        currenttab->setIcon(ICO_CHAT);
    }
    else if(currenttab->getIcon()==ICO_IMNEWMSG)
    {
        currenttab->setIcon(currenttab->getConversation()->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?ICO_IMOFF:ICO_IM);
    }
    if(m_newMsg) updateTrayIconPresence();
    currenttab->setFocus();
    currenttab->setCommandFocus();
    currenttab->makeLastRowVisible();
    return 1;
}

//Handle mousewheel for change currenttab
long dxim::onMouseWheel(FXObject *, FXSelector, void *ptr)
{
    FXEvent *event = (FXEvent*)ptr;
    FXint index = m_tabbook->getCurrent();
    if(event->code > 0) //positive movement
    {
        if(m_tabbook->numChildren())
        {
            if((index-1) >= 0) m_tabbook->setCurrent(index-1, TRUE);
            else m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
        }
    }
    else
    {
        if(m_tabbook->numChildren())
        {
            if((index+1)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index+1, m_tabbook->numChildren() > index*2 ? TRUE : FALSE);
            else m_tabbook->setCurrent(0, TRUE);
        }
    }
    return 1;
}

//change current tab
long dxim::onCmdNextTab(FXObject *, FXSelector, void *)
{
    FXint index = m_tabbook->getCurrent();
    if(m_tabbook->numChildren())
    {
        if((index+1)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index+1, m_tabbook->numChildren() > index*2 ? TRUE : FALSE);
        else m_tabbook->setCurrent(0, TRUE);
    }
    return 1;
}

//change tab on next unread
long dxim::onCmdNextUnread(FXObject*, FXSelector, void*)
{
    if(m_tabbook->numChildren())
    {
        for(FXint i = m_tabbook->getCurrent()*2; i<m_tabbook->numChildren(); i+=2)
        {
            if(DXPREFERENCES.m_appTheme.fore != static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getTextColor())
            {
                m_tabbook->setCurrent(i/2, TRUE);
                return 1;
            }
        }
        for(FXint i = m_tabbook->getCurrent()*2; i>-1; i-=2)
        {
            if(DXPREFERENCES.m_appTheme.fore != static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->getTextColor())
            {
                m_tabbook->setCurrent(i/2, TRUE);
                return 1;
            }
        }
    }
    return 1;
}

//handle reconnect command
long dxim::onCmdReconnect(FXObject*, FXSelector, void*)
{
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getEnabledAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        if(accounts[i].isConnecting())
            continue;
        if(accounts[i].isConnected())
        {
            accounts[i].disconnect();
        }
        accounts[i].connect();
    }
    return 1;
}

//handle request for status change command
long dxim::onCmdStatus(FXObject*, FXSelector sel, void*)
{
    switch(FXSELID(sel)){
        case dxim_AVAILABLE: m_presence = PURPLE_STATUS_AVAILABLE; break;
        case dxim_INVISIBLE: m_presence = PURPLE_STATUS_INVISIBLE; break;
        case dxim_AWAY: m_presence = PURPLE_STATUS_AWAY; break;
        case dxim_BUSY: m_presence = PURPLE_STATUS_UNAVAILABLE; break;
        case dxim_OFFLINE: m_presence = PURPLE_STATUS_OFFLINE; break;
    }
    if(shown()) updateTrayIconPresence();
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getEnabledAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        accounts[i].setStatus(m_presence);
    }
    return 1;
}

//handle status msg command
long dxim::onCmdStatusMsg(FXObject*, FXSelector, void*)
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgInfo msg;
        msg.type = MSGWARNING;
        msg.title = _("Warning");
        msg.text = _("Any account isn't connected.\nConnect someone and try again.");
        msg.okText = _("&OK");
        msg.time = FXSystem::now();
        appendMsgInfo(msg);
        return 1;
    }
    FXDialogBox msgEdit(this, _("Change status message"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
    msgEdit.getAccelTable()->addAccel(KEY_Return, &msgEdit, FXDialogBox::ID_ACCEPT);
    FXVerticalFrame *contents = new FXVerticalFrame(&msgEdit, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
    FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    new FXLabel(matrix, _("Status message:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXTextField *msg = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonframe, _("&Cancel"), NULL, &msgEdit, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    new dxEXButton(buttonframe, _("&OK"), NULL, &msgEdit, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    if(msgEdit.execute(PLACEMENT_OWNER))
    {
        std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getEnabledAccountsList();
        for(FXuint i=0; i<accounts.size(); i++)
            accounts[i].setStatusMessage(dxutils::toStdString(msg->getText()));
    }
    return 1;
}

// Plugins command called
long dxim::onCmdPlugins(FXObject*, FXSelector, void*)
{
    PluginDialog dialog(this, m_cb->getPurplemm()->getPluginsList());
    if(dialog.execute(PLACEMENT_OWNER))
    {
        m_cb->getPurplemm()->saveLoadedPlugins("/dxim/plugins/loaded");
    }
    return 1;
}

// Preferences command called
long dxim::onCmdOptions(FXObject*, FXSelector, void*)
{
    dxStringMap oldsmileysMap = DXPREFERENCES.m_smileysMap;
    FXbool oldUseSmileys = DXPREFERENCES.m_useSmileys;
    FXString oldThemeStatusPath = DXPREFERENCES.m_themeStatusPath;
    FXString oldThemeRolePath = DXPREFERENCES.m_themeRolePath;
    FXColor oldImTextBackColor = DXPREFERENCES.m_colors.back;
    FXbool oldShowRosterTree = DXPREFERENCES.m_showRosterTree;
    ConfigDialog dialog(this);
    if(dialog.execute(PLACEMENT_CURSOR))
    {
        if(oldShowRosterTree!=DXPREFERENCES.m_showRosterTree)
        {
            m_rosterList->clearItems();
            fillRoster();
            updateHideOffline();
        }
        FXbool recreateSmileys = oldUseSmileys!=DXPREFERENCES.m_useSmileys ? TRUE : FALSE;
        if(DXPREFERENCES.m_logging) m_logviewer->enable();
        else m_logviewer->disable();
        if(m_viewer)
        {
            m_viewer->setLogPath(DXPREFERENCES.m_logPath);
            m_viewer->preferencesUpdated();
        }
        if(m_imFont->getFont()!=DXPREFERENCES.m_imFont)
        {
            recreateSmileys = TRUE;
            if(!DXPREFERENCES.m_imFont.empty())
            {
                m_imFont = new FXFont(m_app, DXPREFERENCES.m_imFont);
                m_imFont->create();
            }
            else
            {
                m_app->getNormalFont()->create();
                FXFontDesc fontdescription;
                m_app->getNormalFont()->getFontDesc(fontdescription);
                m_imFont = new FXFont(m_app,fontdescription);
                m_imFont->create();
            }
        }
        if(oldsmileysMap!=DXPREFERENCES.m_smileysMap) recreateSmileys = TRUE;
        updateTheme();
        updateFont();
        updateTabs(recreateSmileys);
        updateTabPosition();
        updateTrayIconColor();
        if(oldThemeStatusPath!=DXPREFERENCES.m_themeStatusPath || oldImTextBackColor!=DXPREFERENCES.m_colors.back) updateBuddyIcons();
        if(oldThemeRolePath!=DXPREFERENCES.m_themeRolePath || oldImTextBackColor!=DXPREFERENCES.m_colors.back) updateChatBuddyIcons();
        recalc();
        m_cb->getPurplemm()->setIdle(DXPREFERENCES.m_useAway, DXPREFERENCES.m_awayTimeout);
    }
    return 1;
}

//logviewer called
long dxim::onCmdLog(FXObject*, FXSelector, void*)
{
    if(m_viewer == NULL)
    {
        m_viewer = new LogViewer(m_app, DXPREFERENCES.m_logPath, m_imFont);
        m_viewer->create();
    }
    m_viewer->show(PLACEMENT_CURSOR);
    return 1;
}

//ftdialog called
long dxim::onCmdTransfers(FXObject*, FXSelector, void*)
{
    if(m_ftDialog == NULL)
    {
        m_ftDialog = new FTDialog(m_app);
        m_ftDialog->create();
    }
    m_ftDialog->show(PLACEMENT_CURSOR);
    return 1;
}

//handle hide roster
long dxim::onCmdHideroster(FXObject*, FXSelector, void*)
{
    DXPREFERENCES.m_rosterShown = !DXPREFERENCES.m_rosterShown;
    if(DXPREFERENCES.m_rosterShown)
    {
        m_splitter->setSplit(1, DXPREFERENCES.m_rosterWidth);
        m_rosterframe->show();
        m_rosterList->show();
    }
    else
    {
        DXPREFERENCES.m_rosterWidth = m_splitter->getSplit(1);
        m_rosterframe->hide();
        m_rosterList->hide();
        m_splitter->setSplit(1,0);
    }
    return 1;
}

//handle hide roster in chattabitem
long dxim::onCmdChatHideroster(FXObject*, FXSelector, void*)
{
    DXPREFERENCES.m_chatRosterShown = !DXPREFERENCES.m_chatRosterShown;
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(DXPREFERENCES.m_chatRosterShown) tab->showUsers();
        else tab->hideUsers();
    }
    return 1;
}

//handle show avatars
long dxim::onCmdShowAvatars(FXObject*, FXSelector, void*)
{
    DXPREFERENCES.m_showAvatars = !DXPREFERENCES.m_showAvatars;
    dxRosterTreeItem *item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
    m_app->beginWaitCursor();
    while(item)
    {
        item->updateShowAvatars();
        // Go deeper
        if(item->first)
        {
            item=(dxRosterTreeItem*)item->first;
            continue;
        }
        // Go up
        while(!item->next && item->parent)
        {
            item=(dxRosterTreeItem*)item->parent;
        }
        // Go to next
        item=(dxRosterTreeItem*)item->next;
    }
    m_app->endWaitCursor();
    m_rosterList->recalc();
    return 1;
}

//handle hideoffline
long dxim::onCmdHideOffline(FXObject*, FXSelector, void*)
{
    DXPREFERENCES.m_hideOffline = !DXPREFERENCES.m_hideOffline;
    updateHideOffline();
    return 1;
}

//handle show friends status message
long dxim::onCmdShowStatusmsg(FXObject*, FXSelector, void*)
{
    DXPREFERENCES.m_showStatusmsg = !DXPREFERENCES.m_showStatusmsg;
    m_rosterList->recalc();
    return 1;
}

//change current tab fot alt-number
long dxim::onCmdSelectTab(FXObject*, FXSelector, void *ptr)
{
    FXint index = 0;
    FXEvent* event = (FXEvent*)ptr;
    switch(event->code){
        case KEY_1:
        {
            index = 0;
            break;
        }
        case KEY_2:
        {
            index = 1;
            break;
        }
        case KEY_3:
        {
            index = 2;
            break;
        }
        case KEY_4:
        {
            index = 3;
            break;
        }
        case KEY_5:
        {
            index = 4;
            break;
        }
        case KEY_6:
        {
            index = 5;
            break;
        }
        case KEY_7:
        {
            index = 6;
            break;
        }
        case KEY_8:
        {
            index = 7;
            break;
        }
        case KEY_9:
        {
            index = 8;
            break;
        }
    }
    if((index)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index, TRUE);
    return 1;
}

//handle new IM
long dxim::onCmdIM(FXObject*, FXSelector, void*)
{
#ifdef HAVE_TRAY
    if(!this->shown())
    {
        this->show();
        updateTrayIconPresence();
    }
#endif
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgInfo msg;
        msg.type = MSGWARNING;
        msg.title = _("Warning");
        msg.text = _("Any account isn't connected.\nConnect someone and try again.");
        msg.okText = _("&OK");
        msg.time = FXSystem::now();
        appendMsgInfo(msg);
        return 1;
    }
    IMDialog newIM(this, m_cb->getPurplemm());
    if(newIM.execute(PLACEMENT_OWNER))
    {
        PURPLE::Account account = newIM.getAccount();
        if(!account.isValid()) return 1;
        FXString with = newIM.getIM();
        if(with.empty()) return 1;
        m_cb->getPurplemm()->createConversation(account, dxutils::toStdString(with), false);
    }
    return 1;
}



//create ImTabItem or ChatTabItem
dxTabItem* dxim::createTab(PURPLE::Conversation *conv)
{
    if(hasTabItem(conv))
    {
        m_tabbook->setCurrent(indexTabItem(conv), TRUE);
        return NULL;
    }
    dxTabItem *item = NULL;
    if(conv->getType()==PURPLE_CONV_TYPE_IM)
    {
        ImTabItem *tabitem = new ImTabItem(m_tabbook, dxutils::fromStdString(conv->getTitle()), conv, conv->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?ICO_IMOFF:ICO_IM, TAB_TOP);
        tabitem->create();
        tabitem->createGeom();
        tabitem->setInfolabelIcon(dxutils::getPresenceIcon(conv->getIMBuddyStatus()));
        tabitem->setSmileys(DXPREFERENCES.m_useSmileys, m_smileys);
        tabitem->preferencesUpdated();
        item = tabitem;
    }
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        ChatTabItem *tabitem = new ChatTabItem(m_tabbook, dxutils::fromStdString(conv->getTitle()), conv, ICO_CHAT, TAB_TOP);
        tabitem->create();
        tabitem->createGeom();
        tabitem->setSmileys(DXPREFERENCES.m_useSmileys, m_smileys);
        tabitem->preferencesUpdated();
        item = tabitem;
#ifdef HAVE_TRAY
        if(DXPREFERENCES.m_useTray && DXPREFERENCES.m_raiseWindow)
            this->show();
#endif
    }
    m_tabbook->recalc();
//    m_tabbook->setCurrent(FXMAX(0,m_tabbook->numTabs()-1), TRUE);
    updateTabPosition();
    updateMenu();
    return item;
}

FXbool dxim::hasTabItem(PURPLE::Conversation *conv)
{
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(conv==tab->getConversation())
        {
            return TRUE;
        }
    }
    return FALSE;
}

FXint dxim::indexTabItem(PURPLE::Conversation *conv)
{
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(conv==tab->getConversation())
        {
            return FXMAX(0,i/2);
        }
    }
    return -1;
}

//send file to buddy
long dxim::onPopupSendFile(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isFriend())
    {
        m_itemOnRight->getBuddy().sendFile();
    }
    return 1;
}

//rename buddy on blist and server (if supported)
long dxim::onPopupRenameBuddy(FXObject *obj, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isFriend())
    {
        FXDialogBox renameBuddy(this, _("Rename friend"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
        renameBuddy.getAccelTable()->addAccel(KEY_Return, &renameBuddy, FXDialogBox::ID_ACCEPT);
        FXVerticalFrame *contents = new FXVerticalFrame(&renameBuddy, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
        FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
        new FXLabel(matrix, _("ID:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        new FXLabel(matrix, dxutils::fromStdString(m_itemOnRight->getBuddy().getUsername()),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        new FXLabel(matrix, _("Alias:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXTextField *alias = new FXTextField(matrix, 25, NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        alias->setText(dxutils::fromStdString(m_itemOnRight->getBuddy().getAlias()));
        FXCheckButton *server = new FXCheckButton(contents, _("Set on server too"));
        if(m_itemOnRight->getBuddy().getAccount().hasServerAliases())
        {
            server->setCheck(true);
        }
        else
        {
            server->setCheck(false);
            server->hide();
        }
        FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
        new dxEXButton(buttonframe, _("&Cancel"), NULL, &renameBuddy, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        new dxEXButton(buttonframe, _("&OK"), NULL, &renameBuddy, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        if(renameBuddy.execute(PLACEMENT_OWNER) && !alias->getText().empty()
            && m_itemOnRight->getBuddy().getAlias().compare(dxutils::toStdString(alias->getText())))
        {
            m_itemOnRight->getBuddy().setAlias(dxutils::toStdString(alias->getText()), server->getCheck());
            m_rosterList->setItemText(m_itemOnRight, alias->getText());
            m_rosterList->sortItems();
        }
    }
    return 1;
}

//move buddy to another group
long dxim::onPopupMoveBuddy(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isFriend())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        std::string oldgroup = m_itemOnRight->getBuddy().getGroupName();
        FXDialogBox moveBuddy(this, _("Move friend"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
        moveBuddy.getAccelTable()->addAccel(KEY_Return, &moveBuddy, FXDialogBox::ID_ACCEPT);
        FXVerticalFrame *contents = new FXVerticalFrame(&moveBuddy, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
        FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
        new FXLabel(matrix, _("ID:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        new FXLabel(matrix, dxutils::fromStdString(m_itemOnRight->getBuddy().getUsername()),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        new FXLabel(matrix, _("Group:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXComboBox *group = new FXComboBox(matrix, 25,NULL, 0, COMBOBOX_INSERT_FIRST|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        FXString groups = getAllGroupList(account);
        if(oldgroup != _("Friends") && !groups.contains(_("Friends")))
            groups.prepend(FXStringFormat("%s\n", _("Friends")));
        group->fillItems(groups);
        FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
        new dxEXButton(buttonframe, _("&Cancel"), NULL, &moveBuddy, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        new dxEXButton(buttonframe, _("&OK"), NULL, &moveBuddy, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        if(moveBuddy.execute(PLACEMENT_OWNER))
        {
            std::string newgroup = group->getText().empty()?_("Friends"):dxutils::toStdString(group->getText());
            if(oldgroup != newgroup) account.moveBuddy(m_itemOnRight->getBuddy(), newgroup);
        }
    }
    return 1;
}

//remove buddy
long dxim::onPopupRemoveBuddy(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isFriend())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Remove friend"), _("Do you want really remove %s?"), m_itemOnRight->getBuddy().getUsername().c_str()) == 1)
        {
            account.removeBuddy(m_itemOnRight->getBuddy());
        }
    }
    return 1;
}

//handle PurpleCallback menu item for friend
long dxim::onPopupBuddyCb(FXObject *obj, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isFriend())
    {
        dxEXBuddyMenuCommand *item = static_cast<dxEXBuddyMenuCommand*>(obj);
        item->getMenuItem().runCallback();
    }
    return 1;
}

//handle PurpleCallback menu item for account
long dxim::onPopupAccountCb(FXObject *obj, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        FXMenuCommand *item = static_cast<FXMenuCommand*>(obj);
        m_itemOnRight->getAccount().callCommand(dxutils::toStdString(item->getText()));
    }
    return 1;
}

//handle popup add buddy
long dxim::onPopupAddBuddy(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        if(!account.isConnected()) return 1;
        FXDialogBox addBuddy(this, _("Add friend"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
        addBuddy.getAccelTable()->addAccel(KEY_Return, &addBuddy, FXDialogBox::ID_ACCEPT);
        FXVerticalFrame *contents = new FXVerticalFrame(&addBuddy, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
        FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
        new FXLabel(matrix, _("Account:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        new FXLabel(matrix, m_itemOnRight->getText(),dxutils::getProtocolIcon(dxutils::fromStdString(account.getProtocol().getPurpleID())),ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        new FXLabel(matrix, _("ID:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXTextField *id = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        new FXLabel(matrix, _("Nick:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXTextField *nick = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        new FXLabel(matrix, _("Group:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXComboBox *group = new FXComboBox(matrix, 25,NULL, 0, COMBOBOX_INSERT_FIRST|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        FXString groups = getAllGroupList(account);
        if(!groups.contains(_("Friends")))
            groups.prepend(FXStringFormat("%s\n", _("Friends")));
        group->fillItems(groups);
        FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
        new dxEXButton(buttonframe, _("&Cancel"), NULL, &addBuddy, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        new dxEXButton(buttonframe, _("&OK"), NULL, &addBuddy, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        if(addBuddy.execute(PLACEMENT_OWNER) && !id->getText().empty())
        {
            account.addBuddy(dxutils::toStdString(id->getText()),
                             group->getText().empty()?_("Friends"):dxutils::toStdString(group->getText()),
                             _("I want to add you in buddies"),
                             dxutils::toStdString(nick->getText()));
        }
    }
    return 1;
}

//handle popup add chat
long dxim::onPopupAddChat(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        ChatDialog dialog(this, m_itemOnRight->getAccount());
        dialog.execute();
    }
    return 1;
}

//handle popup add group
long dxim::onPopupAddGroup(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        FXString group;
        if(dxEXInputDialog::getString(group, this, _("Add group"), _("New group:")))
        {
            m_itemOnRight->getAccount().addGroup(dxutils::toStdString(group));
        }
    }
    return 1;
}

//handle popup request for account status change command
long dxim::onPopupAccountStatus(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        if(!account.isValid()) return 1;
        FXDialogBox setStatus(this, _("Set status"), DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0);
        setStatus.getAccelTable()->addAccel(KEY_Return, &setStatus, FXDialogBox::ID_ACCEPT);
        FXVerticalFrame *contents = new FXVerticalFrame(&setStatus, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 10,10,10,10, 0,0);
        FXMatrix *matrix = new FXMatrix(contents,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
        new FXLabel(matrix, _("Account:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        new FXLabel(matrix, m_itemOnRight->getText(),dxutils::getProtocolIcon(dxutils::fromStdString(account.getProtocol().getPurpleID())),ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        new FXLabel(matrix, _("Status:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXListBox *statuses = new FXListBox(matrix,NULL,0,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK);
        std::vector<PURPLE::Account::AccountStatusItem> v = account.getStatusMenuItems();
        FOREACH(std::vector<PURPLE::Account::AccountStatusItem>, v, it)
        {
            statuses->appendItem(dxutils::fromStdString((*it).getName()));
        }
        new FXLabel(matrix, _("Status message:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        FXTextField *msg = new FXTextField(matrix, 25,NULL, 0, TEXTFIELD_ENTER_ONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        msg->setText(dxutils::fromStdString(account.getStatusMessage()));
        FXHorizontalFrame *buttonframe = new FXHorizontalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
        new dxEXButton(buttonframe, _("&Cancel"), NULL, &setStatus, FXDialogBox::ID_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        new dxEXButton(buttonframe, _("&OK"), NULL, &setStatus, FXDialogBox::ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
        if(setStatus.execute(PLACEMENT_OWNER))
        {
            account.setStatus(v[statuses->getCurrentItem()].getID());
            if(account.isConnected())
                account.setStatusMessage(dxutils::toStdString(msg->getText()));
        }
    }
    return 1;
}

//update alias, chat parameters
long dxim::onPopupUpdateChat(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isChat())
    {
        ChatDialog dialog(this, m_itemOnRight->getChat());
        if(dialog.execute())
        {
            PURPLE::Chat chat = m_itemOnRight->getChat();
            m_rosterList->removeItem(m_itemOnRight);
            createChat(chat);
            m_rosterList->sortItems();
        }
    }
    return 1;
}

//remove chat
long dxim::onPopupRemoveChat(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isChat())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Remove chat"), _("Do you want really remove %s?"), m_itemOnRight->getChat().getAlias().c_str()) == 1)
        {
            account.removeChat(m_itemOnRight->getChat());
        }
    }
    return 1;
}

long dxim::onPopupEnableAccount(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        if(account.isEnabled())
            account.disconnect();
        else
            account.connect();
    }
    return 1;
}

long dxim::onPopupConfigAccount(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        AccountEditor editor(this, account);
        if(editor.execute())
        {
            NewAccount *naccount = new NewAccount;
            editor.fillAccount(naccount);
            account.setOptions(naccount->options);
            account.setPassword(naccount->password, naccount->rememberPassword);
            account.setCheckMail(naccount->checkEmail);
            account.setAlias(naccount->alias);
            naccount->iconPath.empty()?account.removeBuddyIcon():account.setBuddyIcon(naccount->iconPath);
        }
    }
    return 1;
}

long dxim::onPopupAliasAccount(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount())
    {
        PURPLE::Account account = m_itemOnRight->getAccount();
        FXString alias = dxutils::fromStdString(account.getAlias());
        if(dxEXInputDialog::getString(alias, this, _("Set alias"), _("Alias")) && !alias.empty())
        {
            account.setAlias(dxutils::toStdString(alias));
        }
    }
    return 1;
}

long dxim::onPopupReconnectAccount(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight && m_itemOnRight->isAccount() && !m_itemOnRight->getAccount().isConnecting())
    {
        if(m_itemOnRight->getAccount().isConnected())
            m_itemOnRight->getAccount().disconnect();
        m_itemOnRight->getAccount().connect();
    }
    return 1;
}

//notify hides
long dxim::onNotifyHide(FXObject*, FXSelector, void*)
{
    if(m_firstNotify)
        m_firstNotify = !m_firstNotify;
    return 1;
}

//handle DX_BUDDYLOADED
long dxim::onBuddylistLoaded(FXObject*, FXSelector, void*)
{
    fillRoster();
    getApp()->addTimeout(this, dxim_AWAYTIMEOUT, 1000);
    return 1;
}

void dxim::fillRoster()
{
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        dxRosterTreeItem *aitem = NULL;
        if(DXPREFERENCES.m_showRosterTree) aitem = createAccount(accounts[i]);
        dxutils::debugLine(FXStringFormat("Account: %s",  accounts[i].getUsername().c_str()));
        std::vector<std::string> g = accounts[i].getGroups();
        std::vector<PURPLE::Buddy> b = accounts[i].getBuddies();
        std::vector<PURPLE::Chat> ch = accounts[i].getChats();
        for(FXuint j=0; j<g.size(); j++)
        {
            dxutils::debugLine(FXStringFormat("\tGroup: %s",  g[j].c_str()));
            if(DXPREFERENCES.m_showRosterTree) createGroup(accounts[i], dxutils::fromStdString(g[j]));
            for(FXuint k=0; k<b.size(); k++)
            {
                if(g[j] == b[k].getGroupName())
                {
                    dxutils::debugLine(FXStringFormat("\t\tBuddy: %s",  b[k].getUsername().c_str()));
                    createBuddy(b[k]);
                }
            }
            for(FXuint m=0; m<ch.size(); m++)
            {
                if(g[j] == ch[m].getGroupName())
                {
                    dxutils::debugLine(FXStringFormat("\t\tChat: %s",  ch[m].getAlias().c_str()));
                    createChat(ch[m]);
                }
            }
        }
        if(DXPREFERENCES.m_showRosterTree)
        {
            dxRosterTreeItem *item = (dxRosterTreeItem*)aitem->getFirst(); //Group item
            while(item!=NULL && item->getParent()==aitem)
            {
                if(!accounts[i].isConnected())
                    item->state |= dxRosterTreeItem::HIDDEN;
                else
                {
                    if(DXPREFERENCES.m_hideOffline && !accounts[i].isGroupOnline(dxutils::toStdString(item->getText())))
                    {
                        item->state |= dxRosterTreeItem::HIDDEN;
                    }
                }
                dxRosterTreeItem *chitem = (dxRosterTreeItem*)item->getFirst(); //child group item
                while(chitem!=NULL && chitem->getParent()==item)
                {
                    if(!accounts[i].isConnected())
                        chitem->state |= dxRosterTreeItem::HIDDEN;
                    else
                    {
                        if(DXPREFERENCES.m_hideOffline && item->isFriend()
                                && !item->isOnline())
                        {
                            chitem->state |= dxRosterTreeItem::HIDDEN;
                        }
                    }
                    chitem = (dxRosterTreeItem*)chitem->getNext();
                }
                item = (dxRosterTreeItem*)item->getNext();
            }
        }
        else
        {
            dxRosterTreeItem *item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
            while(item!=NULL)
            {
                if(!accounts[i].isConnected())
                    item->state |= dxRosterTreeItem::HIDDEN;
                else
                {
                    if(DXPREFERENCES.m_hideOffline && item->isFriend()
                            && !item->isOnline())
                    {
                        item->state |= dxRosterTreeItem::HIDDEN;
                    }
                }
                item = (dxRosterTreeItem*)item->getNext();
            }
        }
        m_rosterList->sortItems();
        m_rosterList->recalc();
    }
}

//handle DX_NEWACCOUNT
long dxim::onAccountAdded(FXObject*, FXSelector, void *data)
{
    PURPLE::Account *account = (PURPLE::Account*)data;
    if(account)
    {
        createAccount(*account);
    }
    return 1;
}

//handle DX_ACCOUNTREMOVED
long dxim::onAccountRemoved(FXObject*, FXSelector, void *data)
{
    AccountRemoved *ev = *((AccountRemoved**)data);
    if(ev)
    {
        closeAccount(ev->account, ev->protocolID);
        m_rosterList->removeItem(findAccountItem(ev->account, ev->protocolID));
    }
    return 1;
}

//handle DX_ACCOUNTPRESENCE
long dxim::onAccountPresence(FXObject*, FXSelector, void *data)
{
    PURPLE::Account *account = (PURPLE::Account*)data;
    if(account)
    {
        dxRosterTreeItem *item = findAccountItem(dxutils::fromStdString(account->getUsername()), dxutils::fromStdString(account->getProtocol().getPurpleID()));
        if(item)
        {
            m_rosterList->setItemOpenIcon(item, dxutils::getPresenceIcon(account->getStatus()));
            m_rosterList->setItemClosedIcon(item, dxutils::getPresenceIcon(account->getStatus()));
            m_rosterList->recalc();
        }
    }
    return 1;
}

//handle DX_CONNECTING
long dxim::onConnecting(FXObject*, FXSelector, void *data)
{
    AccountEvent *ev = *((AccountEvent**)data);
    if(ev)
    {
        dxutils::debugLine(FXStringFormat(_("Connecting %s-%s\n%s\n"), ev->account.getProtocol().getName().c_str(), ev->account.getUsername().c_str(), ev->msg.text()));
        dxRosterTreeItem *item = findAccountItem(dxutils::fromStdString(ev->account.getUsername()), dxutils::fromStdString(ev->account.getProtocol().getPurpleID()));
        if(item)
        {
            m_rosterList->setItemOpenIcon(item, ICO_ROSTERCONNECTING);
            m_rosterList->setItemClosedIcon(item, ICO_ROSTERCONNECTING);
        }
    }
    return 1;
}

//handle DX_CONNECTED
long dxim::onConnected(FXObject*, FXSelector, void *data)
{
    PURPLE::Account *account = (PURPLE::Account*)data;
    if(account)
    {
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountConnect) showNotify(FXStringFormat(_("Connected %s-%s"), account->getProtocol().getName().c_str(), account->getUsername().c_str()));
        if(m_presence == PURPLE_STATUS_OFFLINE)
        {
            m_presence = PURPLE_STATUS_AVAILABLE;
            updateTrayIconPresence();
        }
        //hack for disable idle after connected
        account->setStatus(m_presence);
        dxRosterTreeItem *aitem = findAccountItem(dxutils::fromStdString(account->getUsername()), dxutils::fromStdString(account->getProtocol().getPurpleID()));
        if(aitem)
        {
            m_rosterList->setItemOpenIcon(aitem, dxutils::getPresenceIcon(account->getStatus()));
            m_rosterList->setItemClosedIcon(aitem, dxutils::getPresenceIcon(account->getStatus()));
            dxRosterTreeItem *gitem = (dxRosterTreeItem*)aitem->getFirst(); //is Group item
            while(gitem!=NULL && gitem->getParent()==aitem)
            {
                if(gitem->isGroup() && account->isGroupOnline(dxutils::toStdString(gitem->getText()))
                    && gitem->isHidden())
                {
                    gitem->state &= ~dxRosterTreeItem::HIDDEN;
                    dxRosterTreeItem *chitem = (dxRosterTreeItem*)gitem->getFirst(); //child group item
                    while(chitem!=NULL && chitem->getParent()==gitem)
                    {
                        if(chitem->isFriend() && !DXPREFERENCES.m_hideOffline && chitem->isHidden())
                            chitem->state &= ~dxRosterTreeItem::HIDDEN;
                        if(chitem->isChat() && chitem->isHidden())
                            chitem->state &= ~dxRosterTreeItem::HIDDEN;
                        chitem = (dxRosterTreeItem*)chitem->getNext();
                    }
                }
                gitem = (dxRosterTreeItem*)gitem->getNext();
            }
            m_rosterList->sortItems();
            m_rosterList->recalc();
        }
        else //showRosterTree == FALSE
        {
            dxRosterTreeItem *chitem = (dxRosterTreeItem*)m_rosterList->getFirstItem();
            while(chitem!=NULL)
            {
                if(chitem->isFriend() && chitem->isAccountOnline()
                        && !DXPREFERENCES.m_hideOffline && chitem->isHidden())
                    chitem->state &= ~dxRosterTreeItem::HIDDEN;
                if(chitem->isChat() && chitem->isHidden())
                    chitem->state &= ~dxRosterTreeItem::HIDDEN;
                chitem = (dxRosterTreeItem*)chitem->getNext();
            }
            m_rosterList->sortItems();
            m_rosterList->recalc();
        }
        dxTabItem *currenttab = NULL;
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
            if(currenttab->getConversation()->isValid() && currenttab->getConversation()->getAccount() == *account)
            {
                currenttab->appendLinkText(FXStringFormat(_("%s [%s]: Connected"), account->getUsername().c_str(), account->getProtocol().getName().c_str()), 5, FXSystem::now());
            }
        }
    }
    return 1;
}

//handle DX_DISCONNECTED
long dxim::onDisconnected(FXObject*, FXSelector, void *data)
{
    AccountEvent *ev = *((AccountEvent**)data);
    if(ev)
    {
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountDisconnect)
        {
            if(ev->msg.empty()) showNotify(FXStringFormat(_("Disconnected %s-%s"), ev->account.getProtocol().getName().c_str(), ev->account.getUsername().c_str()));
            else showNotify(FXStringFormat(_("Disconnected %s-%s\n%s"), ev->account.getProtocol().getName().c_str(), ev->account.getUsername().c_str(), ev->msg.text()));
        }
        closeAccount(dxutils::fromStdString(ev->account.getUsername()), dxutils::fromStdString(ev->account.getProtocol().getPurpleID()));
        m_rosterList->update();
        dxTabItem *currenttab = NULL;
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
            if(currenttab->getConversation()->isValid() && currenttab->getConversation()->getAccount() == ev->account)
            {
                currenttab->appendLinkText(FXStringFormat(_("%s [%s]: Disconnected"), ev->account.getUsername().c_str(), ev->account.getProtocol().getName().c_str()), 5, FXSystem::now());
            }
        }
    }
    return 1;
}

//handle DX_SIGNEDON
long dxim::onSignedOn(FXObject*, FXSelector, void *data)
{
    PURPLE::Account *account = (PURPLE::Account*)data;
    if(account)
    {
        dxRosterTreeItem *aitem = findAccountItem(dxutils::fromStdString(account->getUsername()), dxutils::fromStdString(account->getProtocol().getPurpleID()));
        if(aitem)
        {
            std::vector<std::string> g = account->getGroups();
            std::vector<PURPLE::Buddy> b = account->getBuddies();
            std::vector<PURPLE::Chat> ch = account->getChats();
            for(FXuint j=0; j<g.size(); j++)
            {
                dxRosterTreeItem *gitem = createGroup(*account, dxutils::fromStdString(g[j]));
                for(FXuint k=0; k<b.size(); k++)
                {
                    if(g[j] == b[k].getGroupName())
                    {
                        createBuddy(b[k]);
                    }
                }
                for(FXuint m=0; m<ch.size(); m++)
                {
                    if(g[j] == ch[m].getGroupName())
                    {
                        dxRosterTreeItem *chitem = createChat(ch[m]);
                        if(chitem && chitem->isHidden()) chitem->state &= ~dxRosterTreeItem::HIDDEN;
                    }
                }
                if(DXPREFERENCES.m_hideOffline && !account->isGroupOnline(g[j])
                    && !gitem->isHidden())
                    gitem->state |= dxRosterTreeItem::HIDDEN;
            }
        }
        else // showRosterTree == FALSE
        {
            std::vector<PURPLE::Buddy> b = account->getBuddies();
            std::vector<PURPLE::Chat> ch = account->getChats();
            for(FXuint i=0; i<b.size(); i++)
            {
                createBuddy(b[i]);
            }
            for(FXuint i=0; i<ch.size(); i++)
            {
                dxRosterTreeItem *chitem = createChat(ch[i]);
                if(chitem && chitem->isHidden()) chitem->state &= ~dxRosterTreeItem::HIDDEN;
            }
        }
        m_rosterList->sortItems();
        m_rosterList->recalc();
    }
    return 1;
}

//handle DX_NEWTAB
long dxim::onNewTab(FXObject*, FXSelector, void *data)
{
    PURPLE::Conversation *conv = *((PURPLE::Conversation**)data);
    if(conv)
        createTab(conv);
    return 1;
}

//handle DX_CLOSETAB
long dxim::onCloseTab(FXObject*, FXSelector, void *data)
{
    PURPLE::Conversation *conv = *((PURPLE::Conversation**)data);
    if(conv)
    {
        dxTabItem *currenttab = NULL;
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&ImTabItem::metaClass
                || m_tabbook->childAtIndex(i)->getMetaClass()==&ChatTabItem::metaClass)
            {
                currenttab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
                if(*currenttab->getConversation() == *conv)
                {
                    delete m_tabbook->childAtIndex(i);
                    delete m_tabbook->childAtIndex(i);
                    m_tabbook->recalc();
                    if(m_tabbook->numChildren())
                    {
                        m_tabbook->setCurrent(FXMAX(0,i/2-1), TRUE);
                    }
                    updateMenu();
                    return 1;
                }
            }
        }
    }
    return 1;
}

//handle DX_NEWMYMSG
long dxim::onNewMyMsg(FXObject*, FXSelector, void *data)
{
    NewMsg *msg = *((NewMsg**)data);
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(msg->conv==tab->getConversation())
        {
            tab->appendNewMyMsg(msg);
            return 1;
        }
    }
    //we haven't tab
    tab = createTab(msg->conv);
    tab->appendNewMyMsg(msg);
    return 1;
}

//handle DX_MESSAGERECIVED
long dxim::onMessageRecived(FXObject *, FXSelector, void *data)
{
    NewMsg *msg = *((NewMsg**)data);
    if(msg->messageID.empty()) return 1;
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(msg->conv==tab->getConversation())
        {
            tab->messageRecived(msg);
            return 1;
        }
    }
    return 1;
}

//handle DX_NEWMSG
long dxim::onNewMsg(FXObject*, FXSelector, void *data)
{
    NewMsg *msg = *((NewMsg**)data);
 #ifdef HAVE_TRAY
    if(!msg->history && DXPREFERENCES.m_notifyMessageTray && !m_newMsg && !shown())
    {
        updateTrayIconNewmsg();
    }
#endif
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(msg->conv==tab->getConversation())
        {
            tab->appendNewMsg(msg);
            return 1;
        }
    }
    //we haven't tab
    tab = createTab(msg->conv);
    tab->appendNewMsg(msg);
    return 1;
}

//handle DX_NEWSYSTEMMSG
long dxim::onNewSystemMsg(FXObject*, FXSelector, void *data)
{
    NewSystemMsg *msg = *((NewSystemMsg**)data);
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(msg->conv==tab->getConversation())
        {
            tab->appendNewSystemMsg(msg);
            return 1;
        }
    }
    //we haven't tab
    tab = createTab(msg->conv);
    tab->appendNewSystemMsg(msg);
    return 1;
}

//handle DX_NEWERRORMSG
long dxim::onNewErrorMsg(FXObject*, FXSelector, void *data)
{
    NewSystemMsg *msg = *((NewSystemMsg**)data);
    dxTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        tab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        if(msg->conv==tab->getConversation())
        {
            tab->appendNewErrorMsg(msg);
            return 1;
        }
    }
    //we haven't tab
    tab = createTab(msg->conv);
    tab->appendNewErrorMsg(msg);
    return 1;
}

//handle DX_BUDDYSIGNEDON
long dxim::onBuddySignedOn(FXObject*, FXSelector, void *data)
{
    PURPLE::Buddy *buddy = (PURPLE::Buddy*)data;
    if(buddy)
    {
        dxRosterTreeItem *id = findAccountItem(dxutils::fromStdString(buddy->getAccount().getUsername()), dxutils::fromStdString(buddy->getAccount().getProtocol().getPurpleID()));
        if(DXPREFERENCES.m_showRosterTree && id==NULL)
            return 1;
        dxRosterTreeItem *item = findBuddyByID(buddy->getAccount(), dxutils::fromStdString(buddy->getUsername()));
        if(item)
        {
            item->setText(dxutils::fromStdString(buddy->getAlias()));
            FXbool notify = false;
            FXbool play = DXPREFERENCES.m_sounds && (!shown() || isMinimized());
            if(DXPREFERENCES.m_showRosterTree)
            {
                dxRosterTreeItem *pitem = (dxRosterTreeItem*)item->parent; //friend item has parent
                if(pitem->isHidden()) pitem->state &= ~dxRosterTreeItem::HIDDEN;
                m_rosterList->expandTree(pitem);
            }
            notify = FXSystem::now()-item->getLastStatusChange()>2;
            m_rosterList->setItemOpenIcon(item, dxutils::getPresenceIcon(buddy->getStatusPrimitive()));
            m_rosterList->setItemClosedIcon(item, dxutils::getPresenceIcon(buddy->getStatusPrimitive()));
            item->setAvatarPath(dxutils::fromStdString(buddy->getIconPath()));
            item->setStatusChange();
            if(item->isHidden()) item->state &= ~dxRosterTreeItem::HIDDEN;
            if(notify && play && DXPREFERENCES.m_soundConnect) dxutils::instance().playFile(DXPREFERENCES.m_pathConnect);
            if(notify && DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyConnect) showNotify(FXStringFormat(_("%s\n(%s)\nis online"), dxutils::fromStdString(buddy->getAlias()).text(), dxutils::fromStdString(buddy->getUsername()).text()));
            m_rosterList->expandTree(m_rosterList->getFirstItem());
            m_rosterList->sortItems();
            m_rosterList->recalc();
        }
        ImTabItem *tab = NULL;
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&ImTabItem::metaClass)
            {
                tab = static_cast<ImTabItem*>(m_tabbook->childAtIndex(i));
                if(dxutils::fromStdString(tab->getConversation()->getName())==dxutils::fromStdString(buddy->getUsername()))
                {
                    tab->setInfolabelIcon(dxutils::getPresenceIcon(buddy->getStatusPrimitive()));
                    tab->setIcon(ICO_IM);
                }
            }
        }
    }
    return 1;
}

//handle DX_BUDDYSIGNEDOFF
long dxim::onBuddySignedOff(FXObject*, FXSelector, void *data)
{
    PURPLE::Buddy *buddy = (PURPLE::Buddy*)data;
    if(buddy)
    {
        dxRosterTreeItem *id = findAccountItem(dxutils::fromStdString(buddy->getAccount().getUsername()), dxutils::fromStdString(buddy->getAccount().getProtocol().getPurpleID()));
        if(id==NULL)
            return 1;
        dxRosterTreeItem *item = findBuddyByID(buddy->getAccount(), dxutils::fromStdString(buddy->getUsername()));
        if(item)
        {
            FXbool notify = false;
            FXbool play = DXPREFERENCES.m_sounds && (!shown() || isMinimized());
            dxRosterTreeItem *pitem = (dxRosterTreeItem*)item->parent; //friend item has parent
            notify = FXSystem::now()-item->getLastStatusChange()>2;
            m_rosterList->setItemOpenIcon(item, dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE));
            m_rosterList->setItemClosedIcon(item, dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE));
            item->setStatusChange();
            if(DXPREFERENCES.m_hideOffline)
            {
                item->state |= dxRosterTreeItem::HIDDEN;
                if(pitem->getNumChildren()==pitem->getNumHiddenChildren())
                    pitem->state |= dxRosterTreeItem::HIDDEN;
            }
            m_rosterList->expandTree(pitem);
            if(notify && play && DXPREFERENCES.m_soundDisconnect) dxutils::instance().playFile(DXPREFERENCES.m_pathDisconnect);
            if(notify && DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyDisconnect) showNotify(FXStringFormat(_("%s\n(%s)\nis offline"), dxutils::fromStdString(buddy->getAlias()).text(), dxutils::fromStdString(buddy->getUsername()).text()));
            m_rosterList->expandTree(m_rosterList->getFirstItem());
            m_rosterList->sortItems();
            m_rosterList->recalc();
        }
        ImTabItem *tab = NULL;
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&ImTabItem::metaClass)
            {
                tab = static_cast<ImTabItem*>(m_tabbook->childAtIndex(i));
                if(dxutils::fromStdString(tab->getConversation()->getName())==dxutils::fromStdString(buddy->getUsername()))
                {
                    tab->setInfolabelIcon(dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE));
                    tab->setIcon(ICO_IMOFF);
                }
            }
        }
    }
    return 1;
}

//handle DX_BUDDYPRESENCE
long dxim::onBuddyPresence(FXObject*, FXSelector, void *data)
{
    BuddyStatus *event = *((BuddyStatus**)data);
    if(event && event->buddy.isValid())
    {
        dxRosterTreeItem *id = findAccountItem(dxutils::fromStdString(event->buddy.getAccount().getUsername()), dxutils::fromStdString(event->buddy.getAccount().getProtocol().getPurpleID()));
        if(id==NULL)
            return 1;
        dxRosterTreeItem *item = findBuddyByID(event->buddy.getAccount(), dxutils::fromStdString(event->buddy.getUsername()));
        if(item)
        {
            m_rosterList->setItemOpenIcon(item, dxutils::getPresenceIcon(event->newStatus));
            m_rosterList->setItemClosedIcon(item, dxutils::getPresenceIcon(event->newStatus));
            item->setStatusChange();
            m_rosterList->expandTree(m_rosterList->getFirstItem());
            m_rosterList->sortItems();
            m_rosterList->recalc();
        }
        ImTabItem *tab = NULL;
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            if(m_tabbook->childAtIndex(i)->getMetaClass()==&ImTabItem::metaClass)
            {
                tab = static_cast<ImTabItem*>(m_tabbook->childAtIndex(i));
                if(dxutils::fromStdString(tab->getConversation()->getName())==dxutils::fromStdString(event->buddy.getUsername()))
                    tab->setInfolabelIcon(dxutils::getPresenceIcon(event->newStatus));
            }
        }
    }
    return 1;
}

//handle DX_BUDDYICON
long dxim::onBuddyIcon(FXObject*, FXSelector, void *data)
{
    PURPLE::Buddy *buddy = (PURPLE::Buddy*)data;
    if(buddy)
    {
        FXString protocol = dxutils::fromStdString(buddy->getAccount().getProtocol().getPurpleID());
        dxRosterTreeItem *id = findAccountItem(dxutils::fromStdString(buddy->getAccount().getUsername()), protocol);
        if(id==NULL)
            return 1;
        dxRosterTreeItem *item = findBuddyByID(buddy->getAccount(), dxutils::fromStdString(buddy->getUsername()));
        if(item)
        {
            item->setAvatarPath(dxutils::fromStdString(buddy->getIconPath()));
        }
    }
    return 1;
}

//handle DX_NEWBUDDY
long dxim::onNewBuddy(FXObject*, FXSelector, void *data)
{
    PURPLE::Buddy *buddy = (PURPLE::Buddy*)data;
    if(buddy)
    {
        createBuddy(PURPLE::Buddy(buddy->getPurpleBuddy()));
        m_rosterList->sortItems();
    }
    return 1;
}

//handle DX_BUDDYREMOVED
long dxim::onBuddyRemoved(FXObject*, FXSelector, void *data)
{
    NodeRemoved *event = *((NodeRemoved**)data);
    if(event)
    {
        dxRosterTreeItem *id = findAccountItem(dxutils::fromStdString(event->account.getUsername()),
                dxutils::fromStdString(event->account.getProtocol().getPurpleID()));
        if(id==NULL)
            return 1;
        m_rosterList->expandTree(id);
        dxRosterTreeItem *item = findBuddyByID(event->account, event->who);
        if(item)
        {
            FXTreeItem *pitem = item->parent; //friend item has parent
            m_rosterList->removeItem(item);
            if(!pitem->getNumChildren())
                m_rosterList->removeItem(pitem);
        }
    }
    return 1;
}

//handle DX_NEWCHAT
long dxim::onNewChat(FXObject*, FXSelector, void *data)
{
    PURPLE::Chat *chat = (PURPLE::Chat*)data;
    if(chat)
    {
        createChat(*chat);
        m_rosterList->sortItems();
    }
    return 1;
}

//handle DX_CHATREMOVED
long dxim::onChatRemoved(FXObject*, FXSelector, void *data)
{
    NodeRemoved *event = *((NodeRemoved**)data);
    if(event)
    {
        dxRosterTreeItem *id = findAccountItem(dxutils::fromStdString(event->account.getUsername()),
                dxutils::fromStdString(event->account.getProtocol().getPurpleID()));
        if(id==NULL)
            return 1;
        m_rosterList->expandTree(id);
        dxRosterTreeItem *item = findChatByID(event->account, event->who);
        if(item)
        {
            FXTreeItem *pitem = item->parent; //chat item has parent
            m_rosterList->removeItem(item);
            if(!pitem->getNumChildren())
                m_rosterList->removeItem(pitem);
        }
    }
    return 1;
}

//handle DX_CHATTOPIC
long dxim::onChatTopic(FXObject*, FXSelector, void *data)
{
    ChatTopic *topic = *((ChatTopic**)data);
    ChatTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&ChatTabItem::metaClass)
        {
            tab = static_cast<ChatTabItem*>(m_tabbook->childAtIndex(i));
            if(topic->conv==tab->getConversation())
            {
                tab->chatTopic(topic);
                return 1;
            }
        }
    }
    //we haven't tab
    tab = (ChatTabItem*)createTab(topic->conv);
    tab->chatTopic(topic);
    return 1;
}

//handle DX_BUDDYTYPING
long dxim::onBuddyTyping(FXObject*, FXSelector, void *data)
{
    BuddyTyping *event = *((BuddyTyping**)data);
    ImTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&ImTabItem::metaClass)
        {
            tab = static_cast<ImTabItem*>(m_tabbook->childAtIndex(i));
            if(event->conv==tab->getConversation())
            {
                tab->buddyTyping(event->typing);
                return 1;
            }
        }
    }
    //we haven't tab
    tab = (ImTabItem*)createTab(event->conv);
    tab->buddyTyping(event->typing);
    return 1;
}

//handle DX_NEWROOMLIST
long dxim::onNewRoomList(FXObject*, FXSelector, void *data)
{
    PURPLE::RoomList *rlist = *((PURPLE::RoomList**)data);
    if(rlist->isValid() && m_chatDialog)
    {
        m_chatDialog->newRoomList(rlist);
    }
    return 1;
}

//handle DX_ROOMLISTCOLUMNS
long dxim::onRoomListColumns(FXObject*, FXSelector, void *data)
{
    RoomlistColumns *event = *((RoomlistColumns**)data);
    if(event->list->isValid())
    {
        RoomListDialog *dialog = getRoomListDialog(event->list);
        if(dialog)
        {
            dialog->createColumns(event->columns);
        }
    }
    return 1;
}

//handle DX_ROOMLISTROOM
long dxim::onRoomListRoom(FXObject*, FXSelector, void *data)
{
    Room *event = *((Room**)data);
    if(event->list->isValid())
    {
        RoomListDialog *dialog = getRoomListDialog(event->list);
        if(dialog)
        {
            dialog->addRoom(event->fields);
        }
    }
    return 1;
}

//handle DX_ROOMLISTFINISHED
long dxim::onRoomListFinished(FXObject*, FXSelector, void *data)
{
    PURPLE::RoomList *rlist = *((PURPLE::RoomList**)data);
    if(rlist->isValid())
    {
        RoomListDialog *dialog = getRoomListDialog(rlist);
        if(dialog)
        {
            dialog->listFinished();
        }
    }
    return 1;
}

//handle DX_CHOICEREQUEST
long dxim::onNewChoiceRequest(FXObject*, FXSelector, void *data)
{
    ChoiceRequestDialog dialog(this, *((PURPLE::ChoiceRequest**)data));
    dialog.execute();
    return 1;
}

//handle DX_ACTIONREQUEST
long dxim::onNewActionRequest(FXObject*, FXSelector, void *data)
{
    ActionRequestDialog dialog(this, *((PURPLE::ActionRequest**)data));
    dialog.execute();
    return 1;
}

//handle DX_INPUTREQUEST
long dxim::onNewInputRequest(FXObject*, FXSelector, void *data)
{
    InputRequestDialog dialog(this, *((PURPLE::InputRequest**)data));
    dialog.execute();
    return 1;
}

//handle DX_FILEREQUEST
long dxim::onNewFileRequest(FXObject*, FXSelector, void *data)
{
    PURPLE::FileRequest *request = *((PURPLE::FileRequest**)data);
    if(request->getSaveDialog())
    {
        dxEXFileDialog dialog(this, dxutils::fromStdString(request->getTitle()));
        dialog.setFilename(dxutils::fromStdString(request->getFilename()));
        if(dialog.execute())
            request->setFilename(dxutils::toStdString(dialog.getFilename()));
        else
            request->dennyRequest();
    }
    else
    {
        dxEXFileDialog dialog(this, dxutils::fromStdString(request->getTitle()));
        dialog.setFilename(dxutils::fromStdString(request->getFilename()));
        if(dialog.execute())
            request->setFilename(dxutils::toStdString(dialog.getFilename()));
        else
            request->dennyRequest();
    }
    return 1;
}

//handle DX_FIELDSREQUEST
long dxim::onNewFieldsRequest(FXObject*, FXSelector, void *data)
{
    FieldsRequestDialog dialog(this, *((PURPLE::FieldsRequest**)data));
    dialog.execute();
    return 1;
}

//handle DX_NEWFT
long dxim::onNewFileTransfer(FXObject*, FXSelector, void *ptr)
{
    if(m_ftDialog == NULL)
    {
        m_ftDialog = new FTDialog(m_app);
        m_ftDialog->create();
    }
    m_ftDialog->show(PLACEMENT_CURSOR);
    m_ftDialog->newFT(*(PURPLE::FileTransfer**)ptr);
    return 1;
}

//handle DX_STARTEDFT
long dxim::onStartedFileTransfer(FXObject*, FXSelector, void *ptr)
{
    m_ftDialog->updateFT(*(PURPLE::FileTransfer**)ptr);
    return 1;
}

//handle DX_FINISHEDFT
long dxim::onFinishedFileTransfer(FXObject*, FXSelector, void *ptr)
{
    PURPLE::FileTransfer *ft = *(PURPLE::FileTransfer**)ptr;
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) showNotify(FXStringFormat(_("Transfer of %s finished"), ft->getFileName().c_str()));
    m_ftDialog->updateFT(ft);
    return 1;
}

//handle DX_CANCELEDFT
long dxim::onCanceledFileTransfer(FXObject*, FXSelector, void *ptr)
{
    PURPLE::FileTransfer *ft = *(PURPLE::FileTransfer**)ptr;
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) showNotify(FXStringFormat(_("Transfer of %s canceled"), ft->getFileName().c_str()));
    m_ftDialog->updateFT(ft);
    return 1;
}

//handle DX_PROGRESSDFT
long dxim::onProgressFileTransfer(FXObject*, FXSelector, void *ptr)
{
    m_ftDialog->updateFT(*(PURPLE::FileTransfer**)ptr);
    return 1;
}

//handle DX_NEWNOTIFYMSG
long dxim::onNewNotifyMsg(FXObject*, FXSelector, void *ptr)
{
    MsgInfo event = **((MsgInfo**)ptr);
    appendMsgInfo(event);
    return 1;
}

//handle DX_CHATBUDDY
long dxim::onChatBuddyEvent(FXObject*, FXSelector, void *data)
{
    ChatBuddyEvent *ev = *((ChatBuddyEvent**)data);
    ChatTabItem *tab = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&ChatTabItem::metaClass)
        {
            tab = static_cast<ChatTabItem*>(m_tabbook->childAtIndex(i));
            if(ev->conv==tab->getConversation())
            {
                tab->processChatBuddy(ev);
                return 1;
            }
        }
    }
    //we haven't tab
    tab = (ChatTabItem*)createTab(ev->conv);
    tab->processChatBuddy(ev);
    return 1;
}

//handle DX_USERINFO
long dxim::onUserInfo(FXObject*, FXSelector, void *data)
{
    UserInfo *info = *((UserInfo**)data);
    FXString id = FXStringFormat("%s-%s-%s", info->account.getUsername().c_str(), info->account.getProtocol().getName().c_str(), info->buddy.text());
    TextTabItem *tabitem = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&TextTabItem::metaClass && id == static_cast<TextTabItem*>(m_tabbook->childAtIndex(i))->getID())
        {
            tabitem = static_cast<TextTabItem*>(m_tabbook->childAtIndex(i));
            break;
        }
    }
    if(!tabitem)
    {
        dxSmileyArray smileys;
        tabitem = new TextTabItem(m_tabbook, id, info->buddy, NULL, TAB_TOP);
        tabitem->create();
        tabitem->createGeom();
        tabitem->setSmileys(true, smileys);
        tabitem->preferencesUpdated();
        m_tabbook->recalc();
        m_tabbook->setCurrent(FXMAX(0,m_tabbook->numTabs()-1), TRUE);
        updateTabPosition();
        updateMenu();
    }
    else
    {
        tabitem->clearText();
    }
    tabitem->appendText(FXStringFormat(_("Information about %s"), info->buddy.text()));
    tabitem->appendText("");
    FOREACH(std::vector<PURPLE::Notify::UserInfo>, info->info, it)
    {
        FXString value = dxutils::fromStdString((*it).getValue());
        if(value[0]=='\023')
        {
            FXString path = value.after(':');
            FXIcon *icon = makeScaledIcon(m_app, path, 128, DXPREFERENCES.m_colors.back);
            if(!icon)
            {
                value = _("Image couldn't be loaded");
            }
            else
            {
                dxSmiley smiley;
                smiley.text = value.after('\023').before(':');
                smiley.path = path;
                smiley.icon = icon;
                tabitem->appendSmiley(smiley);
                value = smiley.text;
            }
        }
        if(value.empty()) tabitem->appendText(dxutils::parseHtml(FXStringFormat("\002%s\002:", (*it).getLabel().c_str())));
        else tabitem->appendText(dxutils::parseHtml(FXStringFormat("\002%s\002: %s", (*it).getLabel().c_str(), value.text())));
    }
    return 1;
}

//handle ImTabITem_HIGHLIGHT, ChatTabITem_HIGHLIGHT
long dxim::onHighlight(FXObject *obj, FXSelector sel, void*)
{
    dxTabItem *tab = static_cast<dxTabItem*>(obj);
    if(!shown() && DXPREFERENCES.m_raiseWindow)
    {
        this->show();
    }
    if(!m_newMsg && (!shown() || m_tabbook->childAtIndex(m_tabbook->getCurrent()*2) != tab))
    {
        updateTrayIconNewmsg();
    }
    if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage && (!shown() || isMinimized() || m_tabbook->childAtIndex(m_tabbook->getCurrent()*2) != tab))
        dxutils::instance().playFile(DXPREFERENCES.m_pathMessage);
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage)
    {
        if(FXSELID(sel) == ImTabItem_HIGHLIGHT)
            showNotify(FXStringFormat(_("New message on IM with %s"), tab->getText().text()));
        else
            showNotify(FXStringFormat(_("New highlighted message on %s"), tab->getText().text()));
    }
    flash(TRUE);
    return 1;
}

//handle dxTabItem_SHOWLOG
long dxim::onShowLog(FXObject *obj, FXSelector, void *data)
{
    dxTabItem *tab = static_cast<dxTabItem*>(obj);
    FXString id = FXStringFormat("Log-%s-%s-%s", tab->getConversation()->getAccount().getUsername().c_str(),
                        tab->getConversation()->getAccount().getProtocol().getName().c_str(),
                        dxutils::fromStdString(tab->getConversation()->getName()).before('/').text());
    TextTabItem *tabitem = NULL;
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        if(m_tabbook->childAtIndex(i)->getMetaClass()==&TextTabItem::metaClass && id == static_cast<TextTabItem*>(m_tabbook->childAtIndex(i))->getID())
        {
            tabitem = static_cast<TextTabItem*>(m_tabbook->childAtIndex(i));
            m_tabbook->setCurrent(i/2, TRUE);
            break;
        }
    }
    if(!tabitem)
    {
        tabitem = new TextTabItem(m_tabbook, id, dxutils::fromStdString(tab->getConversation()->getName()).before('/'), ICO_LOGS, TAB_TOP);
        tabitem->create();
        tabitem->createGeom();
        tabitem->setSmileys(DXPREFERENCES.m_useSmileys, m_smileys);
        tabitem->preferencesUpdated();
        m_tabbook->recalc();
        m_tabbook->setCurrent(FXMAX(0,m_tabbook->numTabs()-1), TRUE);
        updateTabPosition();
        updateMenu();
    }
    tabitem->showLog(static_cast<FXString*>(data)->text());
    return 1;
}

//create or recreate smileys
void dxim::createSmileys()
{
    while(m_smileys.no())
    {
        delete m_smileys[0].icon;
        m_smileys[0].icon = NULL;
        m_smileys.erase(0);
    }
    if((FXint)DXPREFERENCES.m_smileysMap.size() && m_imFont)
    {
        dxStringIt it;
        for(it=DXPREFERENCES.m_smileysMap.begin(); it!=DXPREFERENCES.m_smileysMap.end(); it++)
        {
            dxSmiley smiley;
            smiley.text = (*it).first;
            smiley.path = (*it).second;
            smiley.icon = makeIcon(m_app, smiley.path, m_imFont->getFontHeight(), DXPREFERENCES.m_colors.back);
            m_smileys.append(smiley);
        }
    }
}

//update fox toolkit theme
void dxim::updateTheme()
{
    FXWindow *w = FXApp::instance()->getRootWindow();

    FX7Segment * sevensegment;
    FXTextField * textfield;
    FXIconList * iconlist;
    FXList * list;
    FXListBox * listbox;
    FXTreeList * treelist;
    FXComboBox * combobox;
    FXArrowButton * arrowbuton;
    dxEXButton * button;
    FXFrame * frame;
    FXLabel * label;
    FXPopup * popup;
    FXMenuTitle * menutitle;
    FXMenuCheck * menucheck;
    FXMenuRadio * menuradio;
    FXMenuCaption * menucaption;
    FXMenuSeparator * menuseparator;
    FXText * text;
    dxText * dtext;
    FXFoldingList * foldinglist;
    FXMDIChild * mdichild;
    FXTable * table;
    FXDockTitle * docktitle;
    FXPacker * packer;
    FXHeader * header;
    FXGroupBox * groupbox;
    FXScrollBar * scrollbar;
    FXSlider * slider;
    FXStatusLine * statusline;
    FXDragCorner * dragcorner;
    FXRadioButton * radiobutton;
    FXCheckButton * checkbutton;
    FXToolTip * tooltip;
    FXImageFrame * imageframe;

    FXbool update = FALSE;
    FXColor oldForeColor = m_app->getForeColor();
    if(m_app->getBaseColor() != DXPREFERENCES.m_appTheme.base)
    {
        update = TRUE;
        m_app->setBaseColor(DXPREFERENCES.m_appTheme.base);
    }
    if(m_app->getBackColor() != DXPREFERENCES.m_appTheme.back)
    {
        update = TRUE;
        m_app->setBackColor(DXPREFERENCES.m_appTheme.back);
    }
    if(m_app->getBorderColor() != DXPREFERENCES.m_appTheme.border)
    {
        update = TRUE;
        m_app->setBorderColor(DXPREFERENCES.m_appTheme.border);
    }
    if(m_app->getForeColor() != DXPREFERENCES.m_appTheme.fore)
    {
        update = TRUE;
        m_app->setForeColor(DXPREFERENCES.m_appTheme.fore);
    }
    if(m_app->getSelMenuBackColor() != DXPREFERENCES.m_appTheme.menuback)
    {
        update = TRUE;
        m_app->setSelMenuBackColor(DXPREFERENCES.m_appTheme.menuback);
    }
    if(m_app->getSelMenuTextColor() != DXPREFERENCES.m_appTheme.menufore)
    {
        update = TRUE;
        m_app->setSelMenuTextColor(DXPREFERENCES.m_appTheme.menufore);
    }
    if(m_app->getSelbackColor() != DXPREFERENCES.m_appTheme.selback)
    {
        update = TRUE;
        m_app->setSelbackColor(DXPREFERENCES.m_appTheme.selback);
    }
    if(m_app->getSelforeColor() != DXPREFERENCES.m_appTheme.selfore)
    {
        update = TRUE;
        m_app->setSelforeColor(DXPREFERENCES.m_appTheme.selfore);
    }
    if(m_app->getTipbackColor() != DXPREFERENCES.m_appTheme.tipback)
    {
        update = TRUE;
        m_app->setTipbackColor(DXPREFERENCES.m_appTheme.tipback);
    }
    if(m_app->getTipforeColor() != DXPREFERENCES.m_appTheme.tipfore)
    {
        update = TRUE;
        m_app->setTipforeColor(DXPREFERENCES.m_appTheme.tipfore);
    }
    if(m_app->getHiliteColor() != DXPREFERENCES.m_appTheme.hilite)
    {
        update = TRUE;
        m_app->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
    }
    if(m_app->getShadowColor() != DXPREFERENCES.m_appTheme.shadow)
    {
        update = TRUE;
        m_app->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
    }
    if(!update)
        return;

    while (w)
    {
        w->setBackColor(DXPREFERENCES.m_appTheme.base);
        if ((frame = dynamic_cast<FXFrame*> (w)))
        {
            frame->setBaseColor(DXPREFERENCES.m_appTheme.base);
            frame->setBackColor(DXPREFERENCES.m_appTheme.base);
            frame->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            frame->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
            frame->setBorderColor(DXPREFERENCES.m_appTheme.border);
            if ((label = dynamic_cast<FXLabel*> (w)))
            {
                if(label->getTextColor() == oldForeColor) label->setTextColor(DXPREFERENCES.m_appTheme.fore);
                if ((button = dynamic_cast<dxEXButton*> (w)))
                {
                    if (dynamic_cast<FXListBox*> (button->getParent()))
                    {
                        w->setBackColor(DXPREFERENCES.m_appTheme.back);
                    }
                    else
                    {
                        w->setBackColor(DXPREFERENCES.m_appTheme.base);
                    }
                }
                else if ((checkbutton = dynamic_cast<FXCheckButton*> (w)))
                {
                    checkbutton->setCheckColor(DXPREFERENCES.m_appTheme.fore);
                    checkbutton->setBoxColor(DXPREFERENCES.m_appTheme.back);
                }
                else if ((radiobutton = dynamic_cast<FXRadioButton*> (w)))
                {
                    radiobutton->setRadioColor(DXPREFERENCES.m_appTheme.fore);
                    radiobutton->setDiskColor(DXPREFERENCES.m_appTheme.back);
                }
            }
            else if ((arrowbuton = dynamic_cast<FXArrowButton*> (w)))
            {
                arrowbuton->setArrowColor(DXPREFERENCES.m_appTheme.fore);
            }
            else if ((textfield = dynamic_cast<FXTextField*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                textfield->setTextColor(DXPREFERENCES.m_appTheme.fore);
                textfield->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                textfield->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((docktitle = dynamic_cast<FXDockTitle*> (w)))
            {
                docktitle->setCaptionColor(DXPREFERENCES.m_appTheme.selfore);
                docktitle->setBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((header = dynamic_cast<FXHeader*> (w)))
            {
                header->setTextColor(DXPREFERENCES.m_appTheme.fore);
            }
            else if ((statusline = dynamic_cast<FXStatusLine*> (w)))
            {
                statusline->setTextColor(DXPREFERENCES.m_appTheme.fore);
            }
            else if ((sevensegment = dynamic_cast<FX7Segment*> (w)))
            {
                sevensegment->setTextColor(DXPREFERENCES.m_appTheme.fore);
            }
            else if ((slider = dynamic_cast<FXSlider*> (w)))
            {
                slider->setSlotColor(DXPREFERENCES.m_appTheme.back);
            }
            else if ((imageframe = dynamic_cast<FXImageFrame*> (w)))
            {
                imageframe->setBackColor(DXPREFERENCES.m_appTheme.back); /// fixme, only for coverframe in mainwindow
            }
        }
        else if ((packer = dynamic_cast<FXPacker*> (w)))
        {
            packer->setBaseColor(DXPREFERENCES.m_appTheme.base);
            packer->setBackColor(DXPREFERENCES.m_appTheme.base);
            packer->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            packer->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
            packer->setBorderColor(DXPREFERENCES.m_appTheme.border);
            if ((combobox = dynamic_cast<FXComboBox*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
            }
            else if ((listbox = dynamic_cast<FXListBox*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
            }
            else if ((groupbox = dynamic_cast<FXGroupBox*> (w)))
            {
                groupbox->setTextColor(DXPREFERENCES.m_appTheme.fore);
            }
        }
        else if ((popup = dynamic_cast<FXPopup*> (w)))
        {
            popup->setBaseColor(DXPREFERENCES.m_appTheme.base);
            popup->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            popup->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
            popup->setBorderColor(DXPREFERENCES.m_appTheme.border);
        }
        else if ((menucaption = dynamic_cast<FXMenuCaption*> (w)))
        {
            w->setBackColor(DXPREFERENCES.m_appTheme.base);
            menucaption->setTextColor(DXPREFERENCES.m_appTheme.fore);
            menucaption->setSelTextColor(DXPREFERENCES.m_appTheme.menufore);
            menucaption->setSelBackColor(DXPREFERENCES.m_appTheme.menuback);
            menucaption->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            menucaption->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);

            if ((menucheck = dynamic_cast<FXMenuCheck*> (w)))
            {
                menucheck->setBoxColor(DXPREFERENCES.m_appTheme.back);
            }
            else if ((menuradio = dynamic_cast<FXMenuRadio*> (w)))
            {
                menuradio->setRadioColor(DXPREFERENCES.m_appTheme.back);
            }
            else if ((menutitle = dynamic_cast<FXMenuTitle*> (w)))
            {
                menutitle->setTextColor(DXPREFERENCES.m_appTheme.fore);
                menutitle->setSelTextColor(DXPREFERENCES.m_appTheme.fore);
                menutitle->setSelBackColor(DXPREFERENCES.m_appTheme.base);
            }
        }
        else if ((menuseparator = dynamic_cast<FXMenuSeparator*> (w)))
        {
            menuseparator->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            menuseparator->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
        }
        else if ((scrollbar = dynamic_cast<FXScrollBar*> (w)))
        {
            scrollbar->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            scrollbar->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
            scrollbar->setBorderColor(DXPREFERENCES.m_appTheme.border);
            scrollbar->setArrowColor(DXPREFERENCES.m_appTheme.fore);
        }
        else if ((dragcorner = dynamic_cast<FXDragCorner*> (w)))
        {
            dragcorner->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            dragcorner->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
        }
        else if (dynamic_cast<FXScrollArea*> (w))
        {
            if ((text = dynamic_cast<FXText*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                text->setTextColor(DXPREFERENCES.m_appTheme.fore);
                text->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                text->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((dtext = dynamic_cast<dxText*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                dtext->setTextColor(DXPREFERENCES.m_appTheme.fore);
                dtext->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                dtext->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((list = dynamic_cast<FXList*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                list->setTextColor(DXPREFERENCES.m_appTheme.fore);
                list->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                list->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((treelist = dynamic_cast<FXTreeList*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                treelist->setTextColor(DXPREFERENCES.m_appTheme.fore);
                treelist->setLineColor(DXPREFERENCES.m_appTheme.shadow);
                treelist->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                treelist->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((iconlist = dynamic_cast<FXIconList*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                iconlist->setTextColor(DXPREFERENCES.m_appTheme.fore);
                iconlist->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                iconlist->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
            else if ((foldinglist = dynamic_cast<FXFoldingList*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                foldinglist->setTextColor(DXPREFERENCES.m_appTheme.fore);
                foldinglist->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                foldinglist->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
                foldinglist->setLineColor(DXPREFERENCES.m_appTheme.shadow);
            }
            else if ((table = dynamic_cast<FXTable*> (w)))
            {
                w->setBackColor(DXPREFERENCES.m_appTheme.back);
                table->setTextColor(DXPREFERENCES.m_appTheme.fore);
                table->setSelTextColor(DXPREFERENCES.m_appTheme.selfore);
                table->setSelBackColor(DXPREFERENCES.m_appTheme.selback);
            }
        }
        else if ((mdichild = dynamic_cast<FXMDIChild*> (w)))
        {
            mdichild->setBackColor(DXPREFERENCES.m_appTheme.base);
            mdichild->setBaseColor(DXPREFERENCES.m_appTheme.base);
            mdichild->setShadowColor(DXPREFERENCES.m_appTheme.shadow);
            mdichild->setHiliteColor(DXPREFERENCES.m_appTheme.hilite);
            mdichild->setBorderColor(DXPREFERENCES.m_appTheme.border);
            mdichild->setTitleColor(DXPREFERENCES.m_appTheme.selfore);
            mdichild->setTitleBackColor(DXPREFERENCES.m_appTheme.selback);
        }
        else if ((tooltip = dynamic_cast<FXToolTip*> (w)))
        {
            tooltip->setTextColor(DXPREFERENCES.m_appTheme.tipfore);
            tooltip->setBackColor(DXPREFERENCES.m_appTheme.tipback);
        }

        w->update();
        if (w->getFirst())
        {
            w = w->getFirst();
            continue;
        }
        while (!w->getNext() && w->getParent())
        {
            w = w->getParent();
        }
        w = w->getNext();
    }
}

//update application font
void dxim::updateFont()
{
    m_app->getNormalFont()->destroy();
    m_app->getNormalFont()->setFont(DXPREFERENCES.m_font);
    m_app->getNormalFont()->create();
    FXWindow *w = this;
    while(w)
    {
        w->recalc();
        if(w->getFirst())
        {
            w = w->getFirst();
            continue;
        }
        while(!w->getNext() && w->getParent())
        {
            w = w->getParent();
        }
        w = w->getNext();
    }
}

//update roster for hideoffline
void dxim::updateHideOffline()
{
    if(m_rosterList->getNumItems())
    {
        if(DXPREFERENCES.m_showRosterTree)
        {
            dxRosterTreeItem *accountItem = (dxRosterTreeItem*)m_rosterList->getFirstItem();
            dxRosterTreeItem *groupItem = NULL;
            dxRosterTreeItem *item = NULL;
            while(accountItem!=NULL)
            {
                bool online = accountItem->isAccountOnline();
                groupItem = (dxRosterTreeItem*)accountItem->getFirst();
                while(groupItem!=NULL)
                {
                    item = (dxRosterTreeItem*)groupItem->getFirst();
                    while(item!=NULL)
                    {
                        if(item->isFriend())
                        {
                            if(online)
                            {
                                if(DXPREFERENCES.m_hideOffline)
                                {
                                    if(!item->getBuddy().isOnline()) item->state |= dxRosterTreeItem::HIDDEN;
                                    else item->state &= ~dxRosterTreeItem::HIDDEN;
                                }
                                else
                                {
                                    if(item->isHidden()) item->state &= ~dxRosterTreeItem::HIDDEN;
                                }
                            }
                            else
                            {
                                item->state |= dxRosterTreeItem::HIDDEN;
                            }
                        }
                        else //chatitem :)
                        {
                            if(online)
                            {
                                if(item->isHidden()) item->state &= ~dxRosterTreeItem::HIDDEN;
                            }
                            else
                            {
                                item->state |= dxRosterTreeItem::HIDDEN;
                            }
                        }
                        item = (dxRosterTreeItem*)item->getNext();
                    }
                    if(online)
                    {
                        if(DXPREFERENCES.m_hideOffline)
                        {
                            if(groupItem->getNumChildren()==groupItem->getNumHiddenChildren()) groupItem->state |= dxRosterTreeItem::HIDDEN;
                        }
                        else
                        {
                            if(groupItem->isHidden()) groupItem->state &= ~dxRosterTreeItem::HIDDEN;
                        }
                    }
                    else
                    {
                        groupItem->state |= dxRosterTreeItem::HIDDEN;
                    }
                    groupItem = (dxRosterTreeItem*)groupItem->getNext();
                }
                accountItem=(dxRosterTreeItem*)accountItem->getNext();
            }
        }
        else
        {
            dxRosterTreeItem *item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
            while(item != NULL)
            {
                if(item->isFriend())
                {
                    if(item->isAccountOnline())
                    {
                        if(DXPREFERENCES.m_hideOffline)
                        {
                            if(!item->getBuddy().isOnline()) item->state |= dxRosterTreeItem::HIDDEN;
                            else item->state &= ~dxRosterTreeItem::HIDDEN;
                        }
                        else
                        {
                            if(item->isHidden()) item->state &= ~dxRosterTreeItem::HIDDEN;
                        }
                    }
                    else
                    {
                        item->state |= dxRosterTreeItem::HIDDEN;
                    }
                }
                else //chatitem :)
                {
                    if(item->isAccountOnline())
                    {
                        if(item->isHidden()) item->state &= ~dxRosterTreeItem::HIDDEN;
                    }
                    else
                    {
                        item->state |= dxRosterTreeItem::HIDDEN;
                    }
                }
                item = (dxRosterTreeItem*)item->getNext();
            }
        }
        m_rosterList->expandTree(m_rosterList->getFirstItem());
        m_rosterList->sortItems();
        m_rosterList->recalc();
    }
    else
        return;
}

//update menu accordning connection
void dxim::updateMenu()
{
    if(m_tabbook->numChildren())
    {
        m_closeTab->enable();
        m_clearTab->enable();
        m_clearTabs->enable();
    }
    else
    {
        m_closeTab->disable();
        m_clearTab->disable();
        m_clearTabs->disable();
    }
}

//set tabs position
void dxim::updateTabPosition()
{
    switch(DXPREFERENCES.m_tabPosition) {
        case 0: //bottom
            {
                m_tabbook->setTabStyle(TABBOOK_BOTTOMTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_BOTTOM);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        case 1: //left
            {
                m_tabbook->setTabStyle(TABBOOK_LEFTTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_LEFT);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing |= PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        case 2: //top
            {
                m_tabbook->setTabStyle(TABBOOK_TOPTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_TOP);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        case 3: //right
            {
                m_tabbook->setTabStyle(TABBOOK_RIGHTTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_RIGHT);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing |= PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }break;
        default:
            {
                m_tabbook->setTabStyle(TABBOOK_BOTTOMTABS);
                for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabbook->childAtIndex(i))->setTabOrientation(TAB_BOTTOM);
                }
                FXuint packing = m_tabbook->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabbook->setPackingHints(packing);
            }
    }
}

void dxim::updateTabs(FXbool recreateSmileys)
{
    if(recreateSmileys)
    {
        for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
        {
            static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->removeSmileys();
        }
        createSmileys();
    }
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        dxTabItem *imtab = static_cast<dxTabItem*>(m_tabbook->childAtIndex(i));
        imtab->preferencesUpdated();
        imtab->updateImFont();
        imtab->setSmileys(DXPREFERENCES.m_useSmileys, m_smileys);
    }
    m_rosterList->setTextColor(DXPREFERENCES.m_colors.text);
    m_rosterList->setBackColor(DXPREFERENCES.m_colors.back);
    if(DXPREFERENCES.m_sameList) m_rosterList->setFont(m_imFont);
    else m_rosterList->setFont(getApp()->getNormalFont());
    //update font in LogViewer too. Both must be same
    if(m_viewer) m_viewer->setFont(m_imFont);
}

//update trayicon for presence
void dxim::updateTrayIconPresence(PurpleStatusPrimitive presence)
{
#ifdef HAVE_TRAY
    if(DXPREFERENCES.m_useTray && m_trayIcon)
    {
        if(presence == PURPLE_STATUS_UNSET) presence = m_presence;
        switch(presence){
            case PURPLE_STATUS_AVAILABLE: m_trayIcon->setIcon(ICO_ONLINE); break;
            case PURPLE_STATUS_INVISIBLE: m_trayIcon->setIcon(ICO_INVISIBLE); break;
            case PURPLE_STATUS_AWAY: m_trayIcon->setIcon(ICO_AWAY); break;
            case PURPLE_STATUS_UNAVAILABLE: m_trayIcon->setIcon(ICO_DND); break;
            case PURPLE_STATUS_OFFLINE: m_trayIcon->setIcon(ICO_OFFLINE); break;
            case PURPLE_STATUS_EXTENDED_AWAY: m_trayIcon->setIcon(ICO_IDLE); break;
            //some presence mising
            default: m_trayIcon->setIcon(ICO_ONLINE); break;
        }
    }
    m_newMsg = false;
#endif
}

void dxim::updateTrayIconNewmsg()
{
#ifdef HAVE_TRAY
    if(DXPREFERENCES.m_useTray && m_trayIcon)
    {
        m_trayIcon->setIcon(ICO_NEWM);
    }
    m_newMsg = true;
#endif
}

//update trayicon color
void dxim::updateTrayIconColor()
{
#ifdef HAVE_TRAY
#ifndef _WIN32
    if(DXPREFERENCES.m_useTray && m_trayIcon)
    {
        m_trayIcon->setTrayColor(DXPREFERENCES.m_trayColor);
    }
#endif
#endif
}

//update icons for roster
void dxim::updateBuddyIcons()
{
    dxRosterTreeItem *item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
    while(item)
    {
        if(item->isAccount() || item->isFriend())
        {
            m_rosterList->setItemOpenIcon(item, NULL);
            m_rosterList->setItemClosedIcon(item, NULL);
        }
        // Go deeper
        if(item->first)
        {
            item=(dxRosterTreeItem*)item->first;
            continue;
        }
        // Go up
        while(!item->next && item->parent)
        {
            item=(dxRosterTreeItem*)item->parent;
        }
        // Go to next
        item=(dxRosterTreeItem*)item->next;
    }
    recreateStatusIcons(m_app, DXPREFERENCES.m_themeStatusPath, DXPREFERENCES.m_colors.back);
    item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
    while(item)
    {
        if(item->isAccount())
        {
            m_rosterList->setItemOpenIcon(item, dxutils::getPresenceIcon(item->getAccount().getStatus()));
            m_rosterList->setItemClosedIcon(item, dxutils::getPresenceIcon(item->getAccount().getStatus()));
        }
        else if(item->isFriend())
        {
            m_rosterList->setItemOpenIcon(item, dxutils::getPresenceIcon(item->getBuddy().getStatusPrimitive()));
            m_rosterList->setItemClosedIcon(item, dxutils::getPresenceIcon(item->getBuddy().getStatusPrimitive()));
        }
        // Go deeper
        if(item->first)
        {
            item=(dxRosterTreeItem*)item->first;
            continue;
        }
        // Go up
        while(!item->next && item->parent)
        {
            item=(dxRosterTreeItem*)item->parent;
        }
        // Go to next
        item=(dxRosterTreeItem*)item->next;
    }
}

//update icons for chat
void dxim::updateChatBuddyIcons()
{
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->clearChatBuddyIcons();
    }
    recreateRoleIcons(m_app, DXPREFERENCES.m_themeRolePath, DXPREFERENCES.m_colors.back);
    for(FXint i = 0; i<m_tabbook->numChildren(); i+=2)
    {
        static_cast<dxTabItem*>(m_tabbook->childAtIndex(i))->recreateChatBuddyIcons();
    }
}

//check if parent has child with text
FXbool dxim::hasItem(FXTreeItem* parent, FXString text)
{
    FXTreeItem *item;
    item = parent->getFirst();
    while(item!=NULL && item->getParent()==parent)
    {
        if(item->getText()==text) return TRUE;
        item=item->getNext();
    }
    return FALSE;
}

//close tabitems etc. for given account
void dxim::closeAccount(FXString ID, FXString protocol)
{
    dxRosterTreeItem *accountitem = findAccountItem(ID, protocol);
    if(accountitem)
    {
        m_rosterList->setItemOpenIcon(accountitem, dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE));
        m_rosterList->setItemClosedIcon(accountitem, dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE));
        dxRosterTreeItem *item = (dxRosterTreeItem*)accountitem->getFirst();
        while(item!=NULL && item->getParent()==accountitem)
        {
            item->state |= dxRosterTreeItem::HIDDEN;
            item = (dxRosterTreeItem*)item->getNext();
        }
        m_rosterList->sortItems();
        m_rosterList->recalc();
    }
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        m_presence = PURPLE_STATUS_OFFLINE;
        if(!m_newMsg) updateTrayIconPresence();
        updateMenu();
    }
}

void dxim::appendMsgInfo(MsgInfo msg)
{
    FXIcon *icon = NULL;
    switch(msg.type) {
        case MSGINFORMATION: icon = ICO_INFORMATION; break;
        case MSGWARNING: icon = ICO_WARNING; break;
        case MSGERROR: icon = ICO_CANCEL; break;
        case MSGNEWMAIL: icon = ICO_EMAIL; break;
    }
    MsgTabItem *item = new MsgTabItem(m_tabbook, msg.title, msg, icon, TAB_TOP);
    item->create();
    item->createGeom();
    item->preferencesUpdated();
    m_tabbook->recalc();
    updateTabPosition();
    updateMenu();
    m_tabbook->setCurrent(m_tabbook->numTabs()-1, true);
    if(!m_newMsg && !shown())
    {
        updateTrayIconNewmsg();
    }
    if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage && (!shown() || isMinimized()))
        dxutils::instance().playFile(DXPREFERENCES.m_pathMessage);
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage)
        showNotify(FXStringFormat(_("New info %s"), msg.title.text()));
    flash(TRUE);
}

dxRosterTreeItem *dxim::findAccountItem(FXString ID, FXString protocol)
{
    dxRosterTreeItem *item;
    item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
    while(item!=NULL)
    {
        if(item->getText()==ID && item->getProtocol()==protocol)
            return item;
        item = (dxRosterTreeItem*)item->getNext();
    }
    return NULL;
}

PURPLE::Account dxim::findAccount(FXString ID, FXString protocol)
{
    PURPLE::Account account = PURPLE::Account();
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getAccountsList();
    for(FXuint i=0; i<accounts.size(); i++)
    {
        if(ID == dxutils::fromStdString(accounts[i].getUsername())
                && protocol == dxutils::fromStdString(accounts[i].getProtocol().getPurpleID()))
        {
            account = accounts[i];
            break;
        }
    }
    return account;
}

//find friend item by ID
dxRosterTreeItem *dxim::findBuddyByID(PURPLE::Account account, FXString id)
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        dxRosterTreeItem *aitem = findAccountItem(dxutils::fromStdString(account.getUsername()), dxutils::fromStdString(account.getProtocol().getPurpleID()));
        if(aitem)
        {
            dxRosterTreeItem *item;
            item = (dxRosterTreeItem*)aitem->getFirst();
            while(item!=NULL)
            {
                if(item->isAccount())
                    break;
                if(item->isFriend() && item->getID()==id)
                {
                    return item;
                }
                item=(dxRosterTreeItem*)item->getBelow();
            }
        }
    }
    else
    {
        dxRosterTreeItem *item;
        item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
        while(item!=NULL)
        {
            if(item->isFriend() && item->getID()==id
                    && item->getAccount()==account)
            {
                return item;
            }
            item=(dxRosterTreeItem*)item->getNext();
        }
    }
    return NULL;
}

//find chat item by ID
dxRosterTreeItem *dxim::findChatByID(PURPLE::Account account, FXString id)
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        dxRosterTreeItem *aitem = findAccountItem(dxutils::fromStdString(account.getUsername()), dxutils::fromStdString(account.getProtocol().getPurpleID()));
        if(aitem)
        {
            dxRosterTreeItem *item;
            item = (dxRosterTreeItem*)aitem->getFirst();
            while(item!=NULL)
            {
                if(item->isAccount())
                    break;
                if(item->isChat() && item->getID()==id)
                {
                    return item;
                }
                item=(dxRosterTreeItem*)item->getBelow();
            }
        }
    }
    else
    {
        dxRosterTreeItem *item;
        item = (dxRosterTreeItem*)m_rosterList->getFirstItem();
        while(item!=NULL)
        {
            if(item->isChat() && item->getID()==id
                    && item->getAccount()==account)
            {
                return item;
            }
            item=(dxRosterTreeItem*)item->getNext();
        }
    }
    return NULL;
}

//create roster item account
dxRosterTreeItem* dxim::createAccount(PURPLE::Account account)
{
    dxRosterTreeItem *item = findAccountItem(dxutils::fromStdString(account.getUsername()),
            dxutils::fromStdString(account.getProtocol().getPurpleID()));
    if(item==NULL)
    {
        item = new dxRosterTreeItem(dxutils::fromStdString(account.getUsername()),
                dxutils::fromStdString(account.getUsername()), account, PURPLE::Buddy(),
                dxutils::getPresenceIcon(account.getStatus()),
                dxutils::getPresenceIcon(account.getStatus()),
                NULL,
                dxutils::getProtocolIcon(dxutils::fromStdString(account.getProtocol().getPurpleID())));
        m_rosterList->appendItem(NULL, item);
        item->setExpanded(TRUE);
        item->state |= dxRosterTreeItem::ACCOUNT;
    }
    return item;
}

dxRosterTreeItem* dxim::createGroup(PURPLE::Account account, FXString name)
{
    dxRosterTreeItem *acc = findAccountItem(dxutils::fromStdString(account.getUsername()),
            dxutils::fromStdString(account.getProtocol().getPurpleID()));
    if(acc==NULL)
        return NULL;
    dxRosterTreeItem *group = NULL;
    if(!hasItem(acc, name))
    {
        group = new dxRosterTreeItem(name, name, account);
        m_rosterList->appendItem(acc, group);
        group->state |= dxRosterTreeItem::GROUP;
    }
    else
    {
        group = (dxRosterTreeItem*)acc->getFirst();
        while(group!=NULL)
        {
            if(group->getText()==name)
                break;
            group = (dxRosterTreeItem*)group->getNext();
        }
    }
    return group;
}

//create roster item buddy
dxRosterTreeItem* dxim::createBuddy(PURPLE::Buddy buddy)
{
    dxRosterTreeItem *account = findAccountItem(dxutils::fromStdString(buddy.getAccount().getUsername()),
            dxutils::fromStdString(buddy.getAccount().getProtocol().getPurpleID()));
    if(DXPREFERENCES.m_showRosterTree)
    {
        if(account==NULL)
            return NULL;
        m_rosterList->expandTree(account);
    }
    dxRosterTreeItem *group = createGroup(buddy.getAccount(), dxutils::fromStdString(buddy.getGroupName()));
    if(DXPREFERENCES.m_showRosterTree)
    {
        if(group==NULL)
            return NULL;
    }
    dxRosterTreeItem *bitem = findBuddyByID(buddy.getAccount(), dxutils::fromStdString(buddy.getUsername()));
    if(!bitem)
    {
        bitem = new dxRosterTreeItem(dxutils::fromStdString(buddy.getAlias()),
            dxutils::fromStdString(buddy.getUsername()), buddy.getAccount(), buddy,
            dxutils::getPresenceIcon(buddy.getStatusPrimitive()),
            dxutils::getPresenceIcon(buddy.getStatusPrimitive()));
        m_rosterList->appendItem(DXPREFERENCES.m_showRosterTree?group:NULL, bitem);
        bitem->state |= dxRosterTreeItem::FRIEND;
        if(DXPREFERENCES.m_hideOffline && !buddy.isOnline())
            bitem->state |= dxRosterTreeItem::HIDDEN;
        bitem->setAvatarPath(dxutils::fromStdString(buddy.getIconPath()));
        if(DXPREFERENCES.m_showRosterTree) m_rosterList->expandTree(group);
    }
    else
    {
        bitem->setStatusChange();
    }
    return bitem;
}

//create roster item chat
dxRosterTreeItem* dxim::createChat(PURPLE::Chat chat)
{
    dxRosterTreeItem *account = findAccountItem(dxutils::fromStdString(chat.getAccount().getUsername()),
            dxutils::fromStdString(chat.getAccount().getProtocol().getPurpleID()));
    if(DXPREFERENCES.m_showRosterTree)
    {
        if(account==NULL)
            return NULL;
        m_rosterList->expandTree(account);
    }
    dxRosterTreeItem *group = createGroup(chat.getAccount(), dxutils::fromStdString(chat.getGroupName()));
    if(DXPREFERENCES.m_showRosterTree)
    {
        if(group==NULL)
            return NULL;
    }
    dxRosterTreeItem *chitem = findChatByID(chat.getAccount(), dxutils::fromStdString(chat.getAlias()));
    if(!chitem)
    {
        chitem = new dxRosterTreeItem(dxutils::fromStdString(chat.getAlias()),
            dxutils::fromStdString(chat.getAlias()),
            chat.getAccount(), PURPLE::Buddy(), ICO_ROSTERCHAT, ICO_ROSTERCHAT);
        m_rosterList->appendItem(DXPREFERENCES.m_showRosterTree?group:NULL, chitem);
        chitem->state |= dxRosterTreeItem::CHAT;
        if(DXPREFERENCES.m_hideOffline && !chat.getAccount().isConnected())
        {
            chitem->state |= dxRosterTreeItem::HIDDEN;
        }
        chitem->setChat(chat);
        if(DXPREFERENCES.m_showRosterTree) m_rosterList->expandTree(group);
    }
    return chitem;
}

//return grouplist for account item
FXString dxim::getGroupList(PURPLE::Account account, std::string notInclude)
{
    FXString groups = FXString::null;
    std::vector<std::string> v = account.getGroups();
    FOREACH(std::vector<std::string>, v, it)
    {
        if(*it != notInclude) groups.append(dxutils::fromStdString((*it))+"\n");
    }
    return groups;
}

FXString dxim::getAllGroupList(PURPLE::Account account, std::string notInclude)
{
    FXString groups = FXString::null;
    std::vector<std::string> v = account.getAllGroups();
    FOREACH(std::vector<std::string>, v, it)
    {
        if(*it != notInclude) groups.append(dxutils::fromStdString((*it))+"\n");
    }
    return groups;
}

//return RoomListDialog for given PURPLE::RoomList
RoomListDialog *dxim::getRoomListDialog(PURPLE::RoomList *rlist)
{
    FXWindow *w = FXApp::instance()->getRootWindow();
    RoomListDialog *dialog = NULL;
    while(w)
    {
        if((dialog = dynamic_cast<RoomListDialog*> (w)))
        {
            if(dialog->getRoomList() == rlist)
                return dialog;
        }

        w->update();
        if (w->getFirst())
        {
            w = w->getFirst();
            continue;
        }
        while (!w->getNext() && w->getParent())
        {
            w = w->getParent();
        }
        w = w->getNext();
    }
    return NULL;
}

#define USAGE_MSG _("\
\nUsage: dxim [options] \n\
\n\
    [options] can be any of the following:\n\
\n\
        -h, --help         Print (this) help screen and exit.\n\
        -v, --version      Print version information and exit.\n\
        -l [PATH]          Use PATH for configuration.\n\
        -o, --offline      Start with all accounts set offline.\n\
\n")

int main(int argc, char **argv)
{
#ifndef _WIN32
    signal(SIGPIPE, SIG_IGN);
#endif

    argn = argc;
    args = argv;

    FXString datadir = DXIM_DATADIR;

    bool offline = false;

    for(FXint i=0; i<argc; ++i)
    {
        if(compare(argv[i],"-v")==0 || compare(argv[i],"--version")==0)
        {
            fprintf(stdout, "%s %s\n", PACKAGE, VERSION);
            exit(0);
        }
        if(compare(argv[i],"-h")==0 || compare(argv[i],"--help")==0)
        {
            fprintf(stdout, USAGE_MSG);
            exit(0);
        }
        if(compare(argv[i],"-l")==0)
        {
            dxutils::instance().setIniPath(argv[i+1]);
        }
        if(compare(argv[i],"-o")==0 || compare(argv[i],"--offline")==0)
        {
            offline = true;
        }
    }

    FXbool oneinstance = dxutils::instance().getBoolIniEntry("GENERAL", "oneinstance", FALSE);
    if(oneinstance && isInstanceRunningAlready())
        return 0;

#ifdef HAVE_TRAY
    FXTrayApp app(PACKAGE, FXString::null);
#else
    FXApp app(PACKAGE, FXString::null);
#endif
    app.reg().setAsciiMode(TRUE);
    app.init(argc, argv);
#ifdef ENABLE_NLS
    app.setTranslator(new dxTranslator(&app));
#endif

    makeAllIcons(&app, dxutils::instance().getIniPath(), datadir);
    new dxim(&app, offline);
    app.create();
    return app.run();
}

