/*      dxim.h
*
*      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 3 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
*      MA 02110-1301, USA.
*/


#ifndef DXIM_H
#define DXIM_H

#ifndef FX_TRAY_ICON_H
#include "FXTrayIcon.h"
#endif

class dxRosterList;
class dxTabBook;
class dxTabItem;
class dxRosterTreeItem;
class LogViewer;
class FTDialog;
class dxfCallbacks;
class RoomListDialog;
class ChatJoinDialog;

class dxim: public FXMainWindow
{
    FXDECLARE(dxim)
protected:
    dxim() {}
public:
    dxim(FXApp *app, bool offline=false);
    ~dxim();

    virtual void create();
    void flash(FXbool yes);
    void showNotify(const FXString& notify, FXint pos=-1);
    FXFont* getImFont() const { return m_imFont; }
    static dxim* instance();
    dxStringMap getTempKeybindings() { return m_tempKeybindings; }
    void setTempKeybindings(dxStringMap bindings);

    //Actions, events etc. handlers
    long onCmdQuit(FXObject *, FXSelector, void *);
    long onCmdRestart(FXObject *, FXSelector, void *);
    long onCmdAbout(FXObject *, FXSelector, void *);
    long onAccounts(FXObject *, FXSelector, void *);
    long onCmdAddBuddy(FXObject *, FXSelector, void *);
    long onCmdAddChat(FXObject *, FXSelector, void *);
    long onCmdAddGroup(FXObject *, FXSelector, void *);
    long onCmdJoinChat(FXObject *, FXSelector, void *);
    long onTrayRestore(FXObject *, FXSelector, void *);
    long onClose(FXObject *, FXSelector, void *);
    long onAwayTimeout(FXObject *, FXSelector, void *);
    long onCmdClear(FXObject *, FXSelector, void *);
    long onCmdClearAll(FXObject *, FXSelector, void *);
    long onRightRelease(FXObject *, FXSelector, void *);
    long onLeftDoubleClick(FXObject *, FXSelector, void *);
    long onPopupIM(FXObject *, FXSelector, void *);
    long onPopupAccountIM(FXObject *, FXSelector, void *);
    long onPopupChat(FXObject *, FXSelector, void *);
    long onPopupAccountChat(FXObject *, FXSelector, void *);
    long onPopupInfo(FXObject *, FXSelector, void *);
    long onPopupHistory(FXObject *, FXSelector, void *);
    long onCmdCloseTab(FXObject *, FXSelector, void *);
    long onTabBook(FXObject *, FXSelector, void *);
    long onMouseWheel(FXObject*, FXSelector, void*);
    long onCmdNextTab(FXObject *, FXSelector, void *);
    long onCmdNextUnread(FXObject *, FXSelector, void *);
    long onCmdReconnect(FXObject *, FXSelector, void *);
    long onCmdStatus(FXObject *, FXSelector, void *);
    long onCmdStatusMsg(FXObject *, FXSelector, void *);
    long onCmdPlugins(FXObject *, FXSelector, void *);
    long onCmdOptions(FXObject *, FXSelector, void *);
    long onCmdLog(FXObject *, FXSelector, void *);
    long onCmdHideroster(FXObject *, FXSelector, void *);
    long onCmdChatHideroster(FXObject *, FXSelector, void *);
    long onCmdShowAvatars(FXObject *, FXSelector, void *);
    long onCmdShowStatusmsg(FXObject *, FXSelector, void *);
    long onCmdHideOffline(FXObject *, FXSelector, void *);
    long onCmdSelectTab(FXObject *, FXSelector, void *);
    long onCmdIM(FXObject *, FXSelector, void *);
    long onCmdTransfers(FXObject *, FXSelector, void *);
    long onHighlight(FXObject *, FXSelector, void *);
    long onPopupSendFile(FXObject *, FXSelector, void *);
    long onPopupBuddyCb(FXObject *, FXSelector, void *);
    long onPopupAccountCb(FXObject *, FXSelector, void *);
    long onPopupAccountStatus(FXObject *, FXSelector, void *);
    long onNotifyHide(FXObject*, FXSelector, void *);
    long onPopupAddBuddy(FXObject*, FXSelector, void *);
    long onPopupAddChat(FXObject*, FXSelector, void *);
    long onPopupAddGroup(FXObject*, FXSelector, void *);
    long onPopupRenameBuddy(FXObject*, FXSelector, void *);
    long onPopupMoveBuddy(FXObject*, FXSelector, void *);
    long onPopupRemoveBuddy(FXObject*, FXSelector, void *);
    long onPopupUpdateChat(FXObject*, FXSelector, void *);
    long onPopupRemoveChat(FXObject*, FXSelector, void *);
    long onPopupEnableAccount(FXObject*, FXSelector, void *);
    long onPopupConfigAccount(FXObject*, FXSelector, void *);
    long onPopupAliasAccount(FXObject*, FXSelector, void *);
    long onPopupReconnectAccount(FXObject*, FXSelector, void *);
    long onShowLog(FXObject*, FXSelector, void *);
    //protocol events
    long onBuddylistLoaded(FXObject*, FXSelector, void *);
    long onAccountAdded(FXObject*, FXSelector, void *);
    long onAccountRemoved(FXObject*, FXSelector, void *);
    long onAccountPresence(FXObject*, FXSelector, void *);
    long onConnecting(FXObject*, FXSelector, void *);
    long onConnected(FXObject*, FXSelector, void *);
    long onDisconnected(FXObject*, FXSelector, void *);
    long onSignedOn(FXObject*, FXSelector, void *);
    long onNewTab(FXObject*, FXSelector, void *);
    long onCloseTab(FXObject*, FXSelector, void *);
    long onNewMyMsg(FXObject*, FXSelector, void *);
    long onMessageRecived(FXObject*, FXSelector, void *);
    long onNewMsg(FXObject*, FXSelector, void *);
    long onNewSystemMsg(FXObject*, FXSelector, void *);
    long onNewErrorMsg(FXObject*, FXSelector, void *);
    long onBuddySignedOn(FXObject*, FXSelector, void *);
    long onBuddySignedOff(FXObject*, FXSelector, void *);
    long onBuddyPresence(FXObject*, FXSelector, void *);
    long onBuddyIcon(FXObject*, FXSelector, void *);
    long onChatTopic(FXObject*, FXSelector, void *);
    long onNewRoomList(FXObject*, FXSelector, void *);
    long onRoomListColumns(FXObject*,FXSelector,void*);
    long onRoomListRoom(FXObject*,FXSelector,void*);
    long onRoomListFinished(FXObject*,FXSelector,void*);
    long onNewChoiceRequest(FXObject*,FXSelector,void*);
    long onNewActionRequest(FXObject*,FXSelector,void*);
    long onNewInputRequest(FXObject*,FXSelector,void*);
    long onNewFileRequest(FXObject*,FXSelector,void*);
    long onNewFieldsRequest(FXObject*,FXSelector,void*);
    long onNewFileTransfer(FXObject*,FXSelector,void*);
    long onStartedFileTransfer(FXObject*,FXSelector,void*);
    long onFinishedFileTransfer(FXObject*,FXSelector,void*);
    long onCanceledFileTransfer(FXObject*,FXSelector,void*);
    long onProgressFileTransfer(FXObject*,FXSelector,void*);
    long onNewNotifyMsg(FXObject*,FXSelector,void*);
    long onChatBuddyEvent(FXObject*,FXSelector,void*);
    long onUserInfo(FXObject*,FXSelector,void*);
    long onNewBuddy(FXObject*,FXSelector,void*);
    long onBuddyRemoved(FXObject*,FXSelector,void*);
    long onNewChat(FXObject*,FXSelector,void*);
    long onChatRemoved(FXObject*,FXSelector,void*);
    long onBuddyTyping(FXObject*,FXSelector,void*);
private:
    static dxim *_pThis;
    FXApp *m_app;
    FXVerticalFrame *m_mainframe, *m_tabframe, *m_rosterframe;
    dxRosterList *m_rosterList;
    FXSplitter *m_splitter;
    dxTabBook *m_tabbook;
    FXMenuBar *m_menubar;
    FXMenuPane *m_filemenu, *m_helpmenu, *m_editmenu, *m_statusmenu;
    FXMenuCommand *m_closeTab, *m_clearTab, *m_clearTabs, *m_logviewer;
    FXPopup *m_trayMenu, *m_trayStatusMenu;
    FXMenuCascade *m_statusCascade;
    FXTrayIcon *m_trayIcon;
    dxRosterTreeItem *m_itemOnRight;
    LogViewer *m_viewer;
    FXFont *m_imFont;
    dxSmileyArray m_smileys;
    PurpleStatusPrimitive m_presence;
    bool m_firstNotify;
    bool m_newMsg;
    FXint m_notifiesHeight;
    dxfCallbacks *m_cb;
    ChatJoinDialog *m_chatDialog;
    FTDialog *m_ftDialog;
    dxStringMap m_tempKeybindings;

    void updateTheme();
    void updateFont();
    void updateHideOffline();
    void updateMenu();
    void updateTabPosition();
    void updateTabs(FXbool recreateSmileys);
    void updateTrayIconPresence(PurpleStatusPrimitive presence=PURPLE_STATUS_UNSET);
    void updateTrayIconNewmsg();
    void updateTrayIconColor();
    void updateBuddyIcons();
    void updateChatBuddyIcons();
    void createSmileys();
    void fillRoster();
    dxTabItem* createTab(PURPLE::Conversation *conv);
    FXbool hasTabItem(PURPLE::Conversation *conv);
    FXint indexTabItem(PURPLE::Conversation *conv);
    FXbool hasItem(FXTreeItem* parent, FXString text);
    dxRosterTreeItem *findAccountItem(FXString ID, FXString protocol);
    PURPLE::Account findAccount(FXString ID, FXString protocol);
    dxRosterTreeItem *findBuddyByID(PURPLE::Account account, FXString id);
    dxRosterTreeItem *findChatByID(PURPLE::Account account, FXString id);
    dxRosterTreeItem* createAccount(PURPLE::Account account);
    dxRosterTreeItem* createGroup(PURPLE::Account account, FXString name);
    dxRosterTreeItem* createBuddy(PURPLE::Buddy buddy);
    dxRosterTreeItem* createChat(PURPLE::Chat chat);
    FXString getGroupList(PURPLE::Account account, std::string notInclude="");
    FXString getAllGroupList(PURPLE::Account account, std::string notInclude="");
    RoomListDialog *getRoomListDialog(PURPLE::RoomList *rlist);
    void closeAccount(FXString ID, FXString protocol);
    void appendMsgInfo(MsgInfo msg);
};

#endif /* DXIM_H */
