/*
 *      dxutils.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <list>
#include <utilsmm.h>
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "preferences.h"

//This's from Xfe, thanks
int streq(const FXchar *a, const FXchar *b)
{
    if (a == NULL || b == NULL)
        return 0;
    return (strcmp(a, b) == 0);
}

dxutils::dxutils()
{
    m_lcodec = NULL;
    m_iniPath = FXString::null;
    m_haveNotify = false;
#ifndef _WIN32
    if(!FXPath::search(FXSystem::getExecPath(), "notify-send").empty())
        m_haveNotify = true;
#endif // _WIN32
}

dxutils::~dxutils()
{
}

dxutils &dxutils::instance()
{
    static dxutils iutils;
    return iutils;
}

//get system locale
FXTextCodec* dxutils::getCodec()
{
    if(m_lcodec != NULL)
        return m_lcodec;
    else
    {
#ifdef _WIN32
        UINT codepage = ::GetACP();
        switch(codepage)
        {
            case 1250:
            {
                m_lcodec = new FXCP1250Codec();
                return m_lcodec;
            }
            case 1251:
            {
                m_lcodec = new FXCP1251Codec();
                return m_lcodec;
            }
            case 1252:
            {
                m_lcodec = new FXCP1252Codec();
                return m_lcodec;
            }
            case 1253:
            {
                m_lcodec = new FXCP1253Codec();
                return m_lcodec;
            }
            case 1254:
            {
                m_lcodec = new FXCP1254Codec();
                return m_lcodec;
            }
            case 1255:
            {
                m_lcodec = new FXCP1255Codec();
                return m_lcodec;
            }
            case 1256:
            {
                m_lcodec = new FXCP1256Codec();
                return m_lcodec;
            }
            case 1257:
            {
                m_lcodec = new FXCP1257Codec();
                return m_lcodec;
            }
            case 1258:
            {
                m_lcodec = new FXCP1258Codec();
                return m_lcodec;
            }
            case 437:
            {
                m_lcodec = new FXCP437Codec();
                return m_lcodec;
            }
            case 850:
            {
                m_lcodec = new FXCP850Codec();
                return m_lcodec;
            }
            case 852:
            {
                m_lcodec = new FXCP852Codec();
                return m_lcodec;
            }
            case 855:
            {
                m_lcodec = new FXCP855Codec();
                return m_lcodec;
            }
            case 856:
            {
                m_lcodec = new FXCP856Codec();
                return m_lcodec;
            }
            case 857:
            {
                m_lcodec = new FXCP857Codec();
                return m_lcodec;
            }
            case 860:
            {
                m_lcodec = new FXCP860Codec();
                return m_lcodec;
            }
            case 861:
            {
                m_lcodec = new FXCP861Codec();
                return m_lcodec;
            }
            case 862:
            {
                m_lcodec = new FXCP862Codec();
                return m_lcodec;
            }
            case 863:
            {
                m_lcodec = new FXCP863Codec();
                return m_lcodec;
            }
            case 864:
            {
                m_lcodec = new FXCP864Codec();
                return m_lcodec;
            }
            case 865:
            {
                m_lcodec = new FXCP865Codec();
                return m_lcodec;
            }
            case 866:
            {
                m_lcodec = new FXCP866Codec();
                return m_lcodec;
            }
            case 869:
            {
                m_lcodec = new FXCP869Codec();
                return m_lcodec;
            }
            case 874:
            {
                m_lcodec = new FXCP874Codec();
                return m_lcodec;
            }
            case 20866:
            {
                m_lcodec = new FXKOI8RCodec();
                return m_lcodec;
            }
            default:
            {
                m_lcodec = new FXCP1252Codec();
                return m_lcodec;
            }
        }
#else
        FXString locale, language, territory, codeset, modifier;
        if(!(FXSystem::getEnvironment("LANG")).empty()) locale = FXSystem::getEnvironment("LANG");
        else if(!(FXSystem::getEnvironment("LC_ALL")).empty()) locale = FXSystem::getEnvironment("LC_ALL");
        else if(!(FXSystem::getEnvironment("LC_MESSAGES")).empty()) locale = FXSystem::getEnvironment("LC_MESSAGES");
        else locale = "en_US";
        if(locale == "C" || locale == "POSIX")
        {
            locale = "en_US";
        }
        locale.lower();
        if(locale.contains("utf")) locale = locale.before('.');
        language = locale.before('_');
        territory = locale.after('_').before('.').before('@');
        codeset = locale.after('.');
        modifier = locale.after('@');

        if(language == "af")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ar")
        {
            m_lcodec = new FX88596Codec();
            return m_lcodec;
        }
        if(language == "az")
        {
            m_lcodec = new FX88599Codec();
            return m_lcodec;
        }
        if(language == "be")
        {
            m_lcodec = new FXCP1251Codec();
            return m_lcodec;
        }
        if(language == "bg")
        {
            if(codeset=="iso88595") m_lcodec = new FX88595Codec();
            else if(codeset=="koi8r") m_lcodec = new FXKOI8RCodec();
            else m_lcodec = new FXCP1251Codec();
            return m_lcodec;
        }
        if(language == "br")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else if(codeset=="iso885914") m_lcodec = new FX885914Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ca")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "cs")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "cy")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else if(codeset=="iso885914") m_lcodec = new FX885914Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "da")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "de")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ee")
        {
            m_lcodec = new FX88594Codec();
            return m_lcodec;
        }
        if(language == "el")
        {
            if(modifier=="euro") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88597Codec();
            return m_lcodec;
        }
        if(language == "en")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "eo")
        {
            m_lcodec = new FX88593Codec();
            return m_lcodec;
        }
        if(language == "es")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "et")
        {
            if(codeset=="iso88591") m_lcodec = new FX88591Codec();
            else if(codeset=="iso885913") m_lcodec = new FX885913Codec();
            else if(codeset=="iso88594") m_lcodec = new FX88594Codec();
            else m_lcodec = new FX885915Codec();
            return m_lcodec;
        }
        if(language == "eu")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "fi")
        {
            if(codeset=="88591" || codeset=="iso88591") m_lcodec = new FX88591Codec();
            else m_lcodec = new FX885915Codec();
            return m_lcodec;
        }
        if(language == "fo")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "fr")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "fre")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ga")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else if(codeset=="iso885914") m_lcodec = new FX885914Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "gd")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else if(codeset=="iso885914") m_lcodec = new FX885914Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ger")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "gl")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "gv")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else if(codeset=="iso885914") m_lcodec = new FX885914Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "he")
        {
            if(codeset=="cp1255" || codeset=="microsoftcp1255") m_lcodec = new FXCP1255Codec();
            else m_lcodec = new FX88598Codec();
            return m_lcodec;
        }
        if(language == "hr")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "hu")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "id")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "in")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "is")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "it")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "iw")
        {
            m_lcodec = new FX88598Codec();
            return m_lcodec;
        }
        if(language == "kl")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "kw")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else if(codeset=="iso885914") m_lcodec = new FX885914Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "lt")
        {
            if(codeset=="iso88594") m_lcodec = new FX88594Codec();
            else m_lcodec = new FX885913Codec();
            return m_lcodec;
        }
        if(language == "lv")
        {
            if(codeset=="iso88594") m_lcodec = new FX88594Codec();
            else m_lcodec = new FX885913Codec();
            return m_lcodec;
        }
        if(language == "mi")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "mk")
        {
            if(codeset=="cp1251" || codeset=="microsoftc1251") m_lcodec = new FXCP1251Codec();
            else m_lcodec = new FX88595Codec();
            return m_lcodec;
        }
        if(language == "ms")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "mt")
        {
            m_lcodec = new FX88593Codec();
            return m_lcodec;
        }
        if(language == "nb")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "nl")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "nn")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "no")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ny")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "oc")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "pd")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ph")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "pl")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "pp")
        {
            m_lcodec = new FX88591Codec;
            return m_lcodec;
        }
        if(language == "pt")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "ro")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "ru")
        {
            if(codeset=="koi8r" || territory=="ua") m_lcodec = new FXKOI8RCodec();
            else if(codeset=="cp1251" || codeset=="microsoftcp1251") m_lcodec = new FXCP1251Codec();
            else m_lcodec = new FX88595Codec();
            return m_lcodec;
        }
        if(language == "sh")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "sk")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "sl")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "sp")
        {
            m_lcodec = new FX88595Codec();
            return m_lcodec;
        }
        if(language == "sq")
        {
            m_lcodec = new FX88592Codec();
            return m_lcodec;
        }
        if(language == "sr")
        {
            if(codeset=="iso88592" || territory=="sp") m_lcodec = new FX88592Codec();
            else if(codeset=="cp1251" || codeset=="microsoftcp1251") m_lcodec = new FXCP1251Codec();
            else m_lcodec = new FX88595Codec();
            return m_lcodec;
        }
        if(language == "sv")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "th")
        {
            m_lcodec = new FX885911Codec();
            return m_lcodec;
        }
        if(language == "tl")
        {
            m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "tr")
        {
            m_lcodec = new FX88599Codec();
            return m_lcodec;
        }
        if(language == "tt")
        {
            m_lcodec = new FXKOI8RCodec();
            return m_lcodec;
        }
        if(language == "uk")
        {
            if(codeset=="cp1251" || codeset=="microsoftcp1251") m_lcodec = new FXCP1251Codec();
            else if(codeset=="koi8u") new FX885915Codec(); // FXKOI8UCodec doesn't exist
            else m_lcodec = new FXKOI8RCodec();
            return m_lcodec;
        }
        if(language == "ur")
        {
            m_lcodec = new FXCP1256Codec();
            return m_lcodec;
        }
        if(language == "wa")
        {
            if(modifier=="euro" || codeset=="iso885915") m_lcodec = new FX885915Codec();
            else m_lcodec = new FX88591Codec();
            return m_lcodec;
        }
        if(language == "yi")
        {
            m_lcodec = new FXCP1255Codec();
            return m_lcodec;
        }
        m_lcodec = new FX885915Codec();
        return m_lcodec;
#endif
    }
    return m_lcodec;
}

//text in locale encoding to utf8
FXString dxutils::localeToUtf8(const FXchar *buffer)
{
    return getCodec()->mb2utf(buffer);
}

//text in locale encoding to utf8
FXString dxutils::localeToUtf8(const FXString& text)
{
    return getCodec()->mb2utf(text);
}

//std::string to FXString
FXString dxutils::fromStdString(const std::string& str)
{
    if(str.empty()) return "";
    return FXString(str.c_str());
}

//FXString to std::string
std::string dxutils::toStdString(const FXString& str)
{
    if(str.empty()) return "";
    return std::string(str.text());
}

//setting used inifile, usefull for -l argv
void dxutils::setIniPath(const FXString &path)
{
    if(path.empty())
    {
        m_iniPath = g_build_filename(g_get_user_config_dir(), "dxim", NULL);
        if(!(g_file_test(m_iniPath.text(), G_FILE_TEST_IS_DIR)))
        {
            g_mkdir_with_parents(m_iniPath.text(), 0755);
        }
        return;
    }
    m_iniPath = path;
    if(!g_file_test(m_iniPath.text(), G_FILE_TEST_IS_DIR))
    {
        g_mkdir_with_parents(m_iniPath.text(), 0755);
    }
}

//get inifile for another OS
FXString dxutils::getIniPath()
{
    if(m_iniPath.empty())
        m_iniPath = g_build_filename(g_get_user_config_dir(), "dxim", NULL);
    if(!g_file_test(m_iniPath.text(), G_FILE_TEST_IS_DIR))
    {
        g_mkdir_with_parents(m_iniPath.text(), 0755);
    }
    return m_iniPath;
}

//write debugline
void dxutils::debugLine(const FXString &line)
{
#ifdef DEBUG
    if(line.right(1)=="\n")
        fxmessage("[%s] %s", FXSystem::time("%H:%M:%S", FXSystem::now()).text(), line.text());
    else
        fxmessage("[%s] %s\n", FXSystem::time("%H:%M:%S", FXSystem::now()).text(), line.text());
#endif
}

//check path for status icons
FXString dxutils::checkThemeStatusPath(const FXString &path)
{
    const char *themeDefaultPath = DXIM_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "statuses";
    if(!FXPath::search(path, "online.png").empty()) return path;
    return FXString(themeDefaultPath);
}

//check path list for status icons
FXString dxutils::checkThemesStatusList(const FXString &list)
{
    const char *themeDefaultPath = DXIM_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "statuses;";
    FXString themes;
    for(FXint i=0; i<list.contains(';'); i++)
    {
        if(!FXPath::search(list.before(';', i+1).rafter(';'), "online.png").empty()) themes.append(list.before(';', i+1).rafter(';')+";");
    }
    if(!themes.empty()) return themes;
    return FXString(themeDefaultPath);
}

//check path for role icons
FXString dxutils::checkThemeRolePath(const FXString &path)
{
    const char *themeDefaultPath = DXIM_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "roles";
    if(!FXPath::search(path, "normal.png").empty()) return path;
    return FXString(themeDefaultPath);
}

//check path list for role icons
FXString dxutils::checkThemesRoleList(const FXString &list)
{
    const char *themeDefaultPath = DXIM_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "roles;";
    FXString themes;
    for(FXint i=0; i<list.contains(';'); i++)
    {
        if(!FXPath::search(list.before(';', i+1).rafter(';'), "normal.png").empty()) themes.append(list.before(';', i+1).rafter(';')+";");
    }
    if(!themes.empty()) return themes;
    return FXString(themeDefaultPath);
}

//play event sound
void dxutils::playFile(const FXString &file)
{
    if(!FXStat::exists(file))
        return;
#ifdef _WIN32
    PlaySoundA((LPCSTR)file.text(), NULL, SND_FILENAME | SND_ASYNC);
#else
    static const char * players[]={"aplay","play","esdplay","artsplay","ossplay",NULL};
    FXString path = FXSystem::getExecPath();
    FXString exec;
    for(int i=0; players[i]!=NULL; i++)
    {
        exec = FXPath::search(path, players[i]);
        if(!exec.empty()) break;
    }
    if(exec.empty()) return;
    if(exec.contains("aplay")) exec += " -q";
    exec += " "+FXPath::enquote(file)+" &";
    system(exec.text());
#endif
}

//This's from Xfe, thanks
FXbool dxutils::isUtf8(const FXchar* string, FXuint length)
{
    FXchar s[4];
    const FXchar BOM[] = { (FXchar)0xEF, (FXchar)0xBB, (FXchar)0xBF, '\0' };

    // Keep only length left bytes
    FXString str=string;
    str=str.left(length);

    // Convert forth and back to UTF8
    FXUTF8Codec utf8;
    FXString utf8str=utf8.mb2utf(utf8.utf2mb(str));

    // Strings are equal => UTF8
    if (str==utf8str)
        return TRUE;

    // Strings not equal => test if BOM is present
    else
    {
        // String too small to contain BOM
        if (length<=2)
            return FALSE;

        // Test if string contains BOM
        else
        {
            s[0]=string[0];
            s[1]=string[1];
            s[2]=string[2];
            s[3]='\0';

            // String contains BOM => UTF8
            if (streq(s,BOM))
                return TRUE;

            // String don't contain BOM
            else
                return FALSE;
        }
    }
}

//return bool entry in inifile
FXbool dxutils::getBoolIniEntry(const FXchar* section, const FXchar* key, FXbool def)
{
    FXSettings set;
    set.parseFile(m_iniPath.empty()?getIniPath()+PATHSEPSTRING+"dxim-fox":m_iniPath+PATHSEPSTRING+"dxim-fox", TRUE);
    return set.readBoolEntry(section, key, def);
}

//return integer entry in inifile
FXint dxutils::getIntIniEntry(const FXchar* section, const FXchar* key, FXint def)
{
    FXSettings set;
    set.parseFile(m_iniPath.empty()?getIniPath()+PATHSEPSTRING+"dxim-fox":m_iniPath+PATHSEPSTRING+"dxim-fox", TRUE);
    return set.readIntEntry(section, key, def);
}

//split FXString to dxStringArray by separator
dxStringArray dxutils::splitString(const FXString& text, FXchar separator)
{
    dxStringArray array;
    array.no(0);
    FXint num = text.contains(separator);
    if(num)
    {
        for(FXint i=0; i<=num; i++)
        {
            array.append(text.section(separator,i));
        }
    }
    return array;
}

//launch link in browser
FXint dxutils::launchLink(const FXString& link)
{
#ifdef _WIN32
    return ((FXlong)ShellExecuteA(NULL,"open",link.text(),NULL,NULL,SW_SHOWNORMAL)) > 32;
#else
    static const char * browsers[]={"xdg-open","firefox","iceweasel","konqueror","opera","netscape","dillo","open",NULL};
    FXString path = FXSystem::getExecPath();
    FXString exec;
    for(int i=0; browsers[i]!=NULL; i++)
    {
        exec = FXPath::search(path, browsers[i]);
        if(!exec.empty()) break;
    }
    if(exec.empty()) return 0;
    exec += " "+FXPath::enquote(link)+" &";
    return system(exec.text());
#endif
}

//Return file size in human readable form
FXString dxutils::getFileSize(size_t size)
{
    FXfloat fsize = 0.0;
    if(size > 1000000000)
    {
        fsize = size/1073741824.0;
        return FXStringFormat("%.2f %s", fsize, _("GB"));
    }
    if(size > 100000)
    {
        fsize = size/1048576.0;
        return FXStringFormat("%.2f %s", fsize, _("MB"));
    }
    if(size > 1000)
    {
        fsize = size/1024.0;
        return FXStringFormat("%.2f %s", fsize, _("KB"));
    }
    return FXStringFormat("%s %s", FXStringVal(size).text(), _("bytes"));
}

//Return file size in human readable form
FXString dxutils::getFileSize(const FXString &ssize)
{
    FXlong size = FXLongVal(ssize);
    FXfloat fsize = 0.0;
    if(size > 1000000000)
    {
        fsize = size/1073741824.0;
        return FXStringFormat("%.2f %s", fsize, _("GB"));
    }
    if(size > 100000)
    {
        fsize = size/1048576.0;
        return FXStringFormat("%.2f %s", fsize, _("MB"));
    }
    if(size > 1000)
    {
        fsize = size/1024.0;
        return FXStringFormat("%.2f %s", fsize, _("KB"));
    }
    return FXStringFormat("%s %s", ssize.text(), _("bytes"));
}

/*Return download/send speed in human readable form
 * speed = difference between two position during 1 second
 */
FXString dxutils::getSpeed(size_t speed)
{
    FXfloat fspeed = 0.0;
    if(speed > 100000)
    {
        fspeed = speed/(1048576.0);
        return FXStringFormat("%.2f %s", fspeed, _("MB/s"));
    }
    fspeed = speed/(1024.0);
    return FXStringFormat("%.2f %s", fspeed, _("KB/s"));
}

//Return remaining time of download/send
FXString dxutils::getRemaining(size_t size, size_t speed)
{
    if(!speed) return "?";
    size_t remainingTime = size/speed;
    if(remainingTime <= 0 || remainingTime > 54000) return "?";
    else
    {
        unsigned int secs = (unsigned int)remainingTime;
        unsigned int hours = remainingTime/3600;
        secs -= hours*3600;
        unsigned int mins = secs/60;
        secs -= mins*60;
        return FXStringFormat("%u:%02u:%02u", hours, mins, secs);
    }
}

//Return position in file in percent string
FXString dxutils::getPercentPosition(size_t size, size_t position)
{
    if(!size) return "0%";
    return FXStringFormat("%lu%%", (unsigned long)(100*position)/size);
}

//get month string
FXString dxutils::getMonth(FXint month)
{
    switch(month) {
        case 1: return _("January");
        case 2: return _("February");
        case 3: return _("March");
        case 4: return _("April");
        case 5: return _("May");
        case 6: return _("June");
        case 7: return _("July");
        case 8: return _("August");
        case 9: return _("September");
        case 10: return _("October");
        case 11: return _("November");
        case 12: return _("December");
        default: return "Rok ma jen 12 mesicu:)";
    }
}

//get month string-number for name
FXString dxutils::getMonthNumber(const FXString& month)
{
    if(month==_("January")) return "01";
    else if(month==_("February")) return "02";
    else if(month==_("March")) return "03";
    else if(month==_("April")) return "04";
    else if(month==_("May")) return "05";
    else if(month==_("June")) return "06";
    else if(month==_("July")) return "07";
    else if(month==_("August")) return "08";
    else if(month==_("September")) return "09";
    else if(month==_("October")) return "10";
    else if(month==_("November")) return "11";
    else if(month==_("December")) return "12";
    return "00";
}

FXString dxutils::getParam(FXString toParse, FXint n, FXbool toEnd)
{
    if (toEnd)
    {
        return toParse.after(' ', n-1);
    }
    else
    {
        return toParse.before(' ', n).rafter(' ');
    }
}

FXString dxutils::getParam(FXString toParse, FXint n, FXbool toEnd, const FXchar &separator)
{
    if (toEnd)
    {
        return toParse.after(separator, n-1);
    }
    else
    {
        return toParse.before(separator, n).rafter(separator);
    }
}

//time stamp to human readable string
FXString dxutils::stamp2fxstring(FXString datestr, FXString timestr)
{
    FXString year = datestr.section('-',0);
    FXString month = datestr.section('-',1);
    FXString day = datestr.section('-',2);
    struct tm timeinfo;
    timeinfo.tm_year = FXIntVal(year) - 1900;
    timeinfo.tm_mon = FXIntVal(month) - 1;
    timeinfo.tm_mday = FXIntVal(day);
    timeinfo.tm_hour = 0;
    timeinfo.tm_min = 0;
    timeinfo.tm_sec = 0;
    timeinfo.tm_isdst = 0;
    time_t local = mktime(&timeinfo);
    return FXSystem::time("[%x ", local)+timestr+"]";
}

//normalize log line
FXString dxutils::normalizeLogLine(FXString datestr, std::string line)
{
    FXString nline = line.c_str();
    FXString date = stamp2fxstring(datestr, nline.before(']').after('['));
    nline = date+nline.after(']');
    return nline;
}

//is file with name on path right dxim log file?
bool dxutils::isRightLogFile(const FXString& path, const FXString& name)
{
    if(FXRex("^\\d\\d\\d\\d-\\d\\d-\\d\\d+$").match(name))
    {
        FXFile textfile(path,FXFile::Reading);
        FXint n;
        FXchar *txt;
        if(!textfile.isOpen()) return false;
        if(!FXMALLOC(&txt,FXchar,22)) return false;
        n = textfile.readBlock(txt,22);
        if(n<0)
        {
            FXFREE(&txt);
            return false;
        }
        if(FXRex("^\\[(.*)(\\d\\d:\\d\\d:\\d\\d\\])").match(txt,22))
        {
            FXFREE(&txt);
            return true;
        }
        else
        {
            FXFREE(&txt);
            return false;
        }
    }
    return false;
}

void dxutils::sortStringArray(dxStringArray &array)
{
    FXString v;
    FXint i,j,h;
    for(h=1; h<=array.no()/9; h=3*h+1);
    for(; h > 0; h /= 3)
    {
        for(i = h + 1; i <= array.no(); i++)
        {
            v = array[i - 1];
            j = i;
            while (j > h && comparecase(array[j - h - 1], v) > 0)
            {
                array[j - 1] = array[j - h - 1];
                j -= h;
            }
            array[j - 1] = v;
        }
    }
}

bool dxutils::containsStringArray(const dxStringArray &array, const FXString &value)
{
    for(FXint i=0; i<array.no(); i++)
    {
        if(!compare(array[i], value))
            return true;
    }
    return false;
}

FXIcon *dxutils::getProtocolIcon(const FXString& protocol)
{
    if(protocol.contains("jabber")) return ICO_XMPP;
    else if(protocol.contains("msn")) return ICO_WLM;
    else if(protocol.contains("irc")) return ICO_IRC;
    else if(protocol.contains("icq") || protocol.contains("icyque")) return ICO_ICQ;
    else if(protocol.contains("aim")) return ICO_AIM;
    else if(protocol.contains("bonjour")) return ICO_BONJOUR;
    else if(protocol.contains("gg")) return ICO_GG;
    else if(protocol.contains("novell")) return ICO_NOVELL;
    else if(protocol.contains("myspace")) return ICO_MYSPACE;
    else if(protocol.contains("silc")) return ICO_SILC;
    else if(protocol.contains("meanwhile")) return ICO_MEANWHILE;
    else if(protocol.contains("simple")) return ICO_SIMPLE;
    else if(protocol.contains("yahoo")) return ICO_YAHOO;
    else if(protocol.contains("skype")) return ICO_SKYPE;
    else if(protocol.contains("mxit")) return ICO_MXIT;
    else if(protocol.contains("zephyr")) return ICO_ZEPHYR;
    else if(protocol.contains("telegram")) return ICO_TELEGRAM;
    else if(protocol.contains("slack")) return ICO_SLACK;
    else return NULL;
}

//presence to FXIcon
FXIcon *dxutils::getPresenceIcon(PurpleStatusPrimitive presence)
{
    switch(presence) {
    case PURPLE_STATUS_UNSET:
    case PURPLE_STATUS_OFFLINE:
    {
        return ICO_ROSTEROFFLINE;
    }
    case PURPLE_STATUS_UNAVAILABLE:
    {
        return ICO_ROSTERDND;
    }
    case PURPLE_STATUS_INVISIBLE:
    {
        return ICO_ROSTERINVISIBLE;
    }
    case PURPLE_STATUS_AWAY:
    {
        return ICO_ROSTERAWAY;
    }
    case PURPLE_STATUS_EXTENDED_AWAY:
    {
        return ICO_ROSTERIDLE;
    }
    case PURPLE_STATUS_AVAILABLE:
    case PURPLE_STATUS_MOBILE:
    case PURPLE_STATUS_TUNE:
    default:
    {
        return ICO_ROSTERONLINE;
    }
    }
    return NULL;
}

//ChatRole to string
FXString dxutils::getChatRoleString(PurpleConvChatBuddyFlags role)
{
    if(role & PURPLE_CBFLAGS_FOUNDER)
        return _("Founder");
    else if(role & PURPLE_CBFLAGS_OP)
        return _("Moderator");
    else if(role & PURPLE_CBFLAGS_HALFOP)
        return _("Half-op");
    else if(role & PURPLE_CBFLAGS_VOICE)
        return _("Voice");
    else
        return _("Any role");
}

//ChatRole to FXIcon*
FXIcon *dxutils::getChatRoleIcon(PurpleConvChatBuddyFlags role)
{
    if(role & PURPLE_CBFLAGS_FOUNDER)
        return ICO_CHRFOUNDER;
    else if(role & PURPLE_CBFLAGS_OP)
        return ICO_CHROP;
    else if(role & PURPLE_CBFLAGS_HALFOP)
        return ICO_CHRHALFOP;
    else if(role & PURPLE_CBFLAGS_VOICE)
        return ICO_CHRVOICE;
    else
        return ICO_CHRNORMAL;
}

//Get offline color
FXColor dxutils::makeOfflineColor(FXColor clr)
{
    FXuint r,g,b;
    r=FXREDVAL(clr);
    g=FXGREENVAL(clr);
    b=FXBLUEVAL(clr);
    if(r==g && r==b && !r)
    {
        r=255;
        g=255;
        b=255;
    }
    r=(50*r)/100;
    g=(50*g)/100;
    b=(50*b)/100;
    return FXRGB(r,g,b);
}

//typing to human readable string
FXString dxutils::typingToString(FXString nick, bool typing)
{
    if(typing)
        return FXStringFormat(_("%s is composing."), nick.text());
    else
        return FXStringFormat(_("%s stoped compose."), nick.text());
}

// Get key binding string from user input
// Code adapted from FXAccelTable::unparseAccel() and modified to get strings like 'Ctrl-A' instead of 'ctrl+a'
FXString dxutils::getKeybinding(FXEvent* event)
{
    // Get modifiers and key
    FXint mods=event->state;
    FXint code=event->code;
    FXchar buffer[64];
    FXString s;
    // Handle modifier keys
    if(mods&CONTROLMASK)
        s+="Ctrl-";
    if(mods&ALTMASK)
        s+="Alt-";
    if(mods&SHIFTMASK)
        s+="Shift-";
    if(mods&METAMASK)
        s+="Meta-";
    // Handle some special keys
    switch(code)
    {
        case KEY_Home:
            s+="Home";
            break;
        case KEY_End:
            s+="End";
            break;
        case KEY_Page_Up:
            s+="PgUp";
            break;
        case KEY_Page_Down:
            s+="PgDn";
            break;
        case KEY_Left:
            s+="Left";
            break;
        case KEY_Right:
            s+="Right";
            break;
        case KEY_Up:
            s+="Up";
            break;
        case KEY_Down:
            s+="Down";
            break;
        case KEY_Insert:
            s+="Ins";
            break;
        case KEY_Delete:
            s+="Del";
            break;
        case KEY_Escape:
            s+="Esc";
            break;
        case KEY_Tab:
            s+="Tab";
            break;
        case KEY_Return:
            s+="Return";
            break;
        case KEY_BackSpace:
            s+="Back";
            break;
        case KEY_space:
            s+="Space";
            break;
        case KEY_F1:
        case KEY_F2:
        case KEY_F3:
        case KEY_F4:
        case KEY_F5:
        case KEY_F6:
        case KEY_F7:
        case KEY_F8:
        case KEY_F9:
        case KEY_F10:
        case KEY_F11:
        case KEY_F12:
        case KEY_F13:
        case KEY_F14:
        case KEY_F15:
        case KEY_F16:
        case KEY_F17:
        case KEY_F18:
        case KEY_F19:
        case KEY_F20:
        case KEY_F21:
        case KEY_F22:
        case KEY_F23:
        case KEY_F24:
        case KEY_F25:
        case KEY_F26:
        case KEY_F27:
        case KEY_F28:
        case KEY_F29:
        case KEY_F30:
        case KEY_F31:
        case KEY_F32:
        case KEY_F33:
        case KEY_F34:
        case KEY_F35:
            sprintf(buffer,"F%d",code-KEY_F1+1);
            s+=buffer;
            break;
        default:
        if(Ascii::isPrint(code))
            s+=Ascii::toUpper(code);
        else
            s="";  // Invalid case
        break;
    }
    return s;
}

// Parse accelerator from string
FXHotKey dxutils::parseAccel(const FXString& string)
{
    FXuint code=0,mods=0;
    FXint pos=0;
    // Parse leading space
    while(pos<string.length() && Ascii::isSpace(string[pos]))
        pos++;
    // Parse modifiers
    while(pos<string.length())
    {
        // Modifier
        if(comparecase(&string[pos],"ctl",3)==0)
        {
            mods|=CONTROLMASK;
            pos+=3;
        }
        else if(comparecase(&string[pos],"ctrl",4)==0)
        {
            mods|=CONTROLMASK;
            pos+=4;
        }
        else if(comparecase(&string[pos],"alt",3)==0)
        {
            mods|=ALTMASK;
            pos+=3;
        }
        else if(comparecase(&string[pos],"meta",4)==0)
        {
            mods|=METAMASK;
            pos+=4;
        }
        else if(comparecase(&string[pos],"shift",5)==0)
        {
            mods|=SHIFTMASK;
            pos+=5;
        }
        else
            break;
        // Separator
        if(string[pos]=='+' || string[pos]=='-' || Ascii::isSpace(string[pos]))
            pos++;
    }
    // Test for some special keys
    if(comparecase(&string[pos],"home",4)==0)
        code=KEY_Home;
    else if(comparecase(&string[pos],"end",3)==0)
        code=KEY_End;
    else if(comparecase(&string[pos],"pgup",4)==0)
        code=KEY_Page_Up;
    else if(comparecase(&string[pos],"pgdn",4)==0)
        code=KEY_Page_Down;
    else if(comparecase(&string[pos],"left",4)==0)
        code=KEY_Left;
    else if(comparecase(&string[pos],"right",5)==0)
        code=KEY_Right;
    else if(comparecase(&string[pos],"up",2)==0)
        code=KEY_Up;
    else if(comparecase(&string[pos],"down",4)==0)
        code=KEY_Down;
    else if(comparecase(&string[pos],"ins",3)==0)
        code=KEY_Insert;
    else if(comparecase(&string[pos],"del",3)==0)
        code=KEY_Delete;
    else if(comparecase(&string[pos],"esc",3)==0)
        code=KEY_Escape;
    else if(comparecase(&string[pos],"tab",3)==0)
        code=KEY_Tab;
    else if(comparecase(&string[pos],"return",6)==0)
        code=KEY_Return;
    else if(comparecase(&string[pos],"enter",5)==0)
        code=KEY_Return;
    else if(comparecase(&string[pos],"back",4)==0)
        code=KEY_BackSpace;
    else if(comparecase(&string[pos],"spc",3)==0)
        code=KEY_space;
    else if(comparecase(&string[pos],"space",5)==0)
        code=KEY_space;
    // Test for function keys
    else if(Ascii::toLower(string[pos])=='f' && Ascii::isDigit(string[pos+1]))
    {
        if(Ascii::isDigit(string[pos+2]))
        {
            code=KEY_F1+10*(string[pos+1]-'0')+(string[pos+2]-'0')-1;
        }
        else
            code=KEY_F1+string[pos+1]-'1';
    }
    // Test if hexadecimal code designator
    else if(string[pos]=='#')
        code=strtoul(&string[pos+1],NULL,16);
    // Test if its a single character accelerator
    else if(Ascii::isPrint(string[pos]))
    {
        if(mods&SHIFTMASK)
            code=Ascii::toUpper(string[pos])+KEY_space-' ';
        else
            code=Ascii::toLower(string[pos])+KEY_space-' ';
    }
    return MKUINT(code,mods);
}

FXString dxutils::normalizeLabelString(std::string label, char normalize)
{
    FXString text = "";
    FOREACH(std::string, label, it)
    {
        if((*it)==normalize) continue;
        text.append((*it));
    }
    return text;
}

FXString dxutils::parseHtml(FXString text)
{
    FXint i=0;
    FXint tagLength=0;
    FXint length=text.length();
    FXString tag="";
    std::list<FXchar> span;
    while(i<length)
    {
        if(text[i]=='<')
        {
            tagLength = 1;
            for(FXint j=i+1; j<length; j++)
            {
                if(text[j]=='>')
                {
                    tagLength++;
                    break;
                }
                else if(text[j]=='<')
                {
                    break;
                }
                tagLength++;
            }
            tag = text.mid(i,tagLength).lower();
            debugLine(FXStringFormat("tag: %s", tag.text()));
            if(!compare(tag.left(3),"<br")) //tag which will replaced by PURPLE::stripHtml
            {
                i += tagLength;
            }
            else
            {
                if(!compare(tag,"<b>") || !compare(tag,"</b>"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    text.insert(i,'\002');
                    i++;
                    length++;
                }
                else if(!compare(tag,"<u>") || !compare(tag,"</u>"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    text.insert(i,'\037');
                    i++;
                    length++;
                }
                else if(!compare(tag,"<i>") || !compare(tag,"</i>")
                    || !compare(tag,"<em>") || !compare(tag,"</em>"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    text.insert(i,'\035');
                    i++;
                    length++;
                }
                else if(!compare(tag.left(5),"<font"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    dxStringArray items = splitString(tag, '=');
                    for(FXint j=0; j<items.no(); j++)
                    {
                        if(!compare(items[j].right(5),"color"))
                        {
                            text.insert(i,'\003');
                            i++;
                            length++;
                            FXString color = items[j+1].erase(items[j+1].find(' '));
                            color = color.after('\"').before('\"');
                            text.insert(i, colorToHex(fxcolorfromname(color.text())));
                            i += 6;
                            length += 6;
                        }
                        else if(!compare(items[j].right(4),"back"))
                        {
                            text.insert(i,'\025');
                            i++;
                            length++;
                            FXString color = items[j+1].erase(items[j+1].find(' '));
                            color = color.after('\"').before('\"');
                            text.insert(i, colorToHex(fxcolorfromname(color.text())));
                            i += 6;
                            length += 6;
                        }
                    }
                }
                else if(!compare(tag,"</font>"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    text.insert(i,'\025');
                    text.insert(i,'\003');
                    i += 2;
                    length += 2;
                }
                else if(!compare(tag.left(5),"<span"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    dxStringArray items = splitString(tag, '=');
                    for(FXint j=0; j<items.no(); j++)
                    {
                        if(!compare(items[j].left(6),"'color"))
                        {
                            text.insert(i,'\003');
                            i++;
                            length++;
                            span.push_back('\003');
                            text.insert(i, colorToHex(fxcolorfromname(items[j].after(' ').before(';').text())));
                            i += 6;
                            length += 6;
                        }
                        else if(!compare(items[j].left(11),"'background"))
                        {
                            text.insert(i,'\025');
                            i++;
                            length++;
                            span.push_back('\025');
                            text.insert(i, colorToHex(fxcolorfromname(items[j].after(' ').before(';').text())));
                            i += 6;
                            length += 6;
                        }
                        else if(!compare(items[j].left(18),"'font-weight: bold"))
                        {
                            text.insert(i,'\002');
                            i++;
                            length++;
                            span.push_back('\002');
                        }
                        else if(!compare(items[j].left(27),"'text-decoration: underline"))
                        {
                            text.insert(i,'\037');
                            i++;
                            length++;
                            span.push_back('\037');
                        }
                        else
                        {
                            span.push_back('\005');
                        }
                    }
                }
                else if(!compare(tag,"</span>"))
                {
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    if(span.back()!='\005')
                    {
                        text.insert(i,span.back());
                        i++;
                        length++;
                    }
                    if(span.size()) span.pop_back();
                }
                else if(!compare(tag.left(2),"<a"))
                {
                    for(FXint j=i+tagLength; j<length; j++)
                    {
                        if(text[j]=='<' && text[j+1]=='/' && (text[j+2]=='a'||text[j+2]=='A'))
                        {
                            tagLength += 4;
                            break;
                        }
                        tagLength++;
                    }
                    text.erase(i,tagLength);
                    length = length-tagLength;
                    FXString link;
                    if(tag.find("=\"")!=-1)
                        link = tag.after('\"').before('\"');
                    else
                        link = tag.after('\'').before('\'');
                    text.insert(i,link);
                    length += link.length();
                    i += link.length();
                }
                else
                {
                    if(!compare(tag.left(5),"<html") || !compare(tag,"</html>")
                        || !compare(tag.left(5),"<body") || !compare(tag,"</body>")
                        || !compare(tag,"<p>") || !compare(tag,"</p>")
                        || !compare(tag,"</a>"))
                    {
                        text.erase(i,tagLength);
                        length = length-tagLength;
                        i++;
                    }
                    else
                    {
                        text.replace(i, 1, "&lt;");
                        i += tagLength+3;
                        length += 3;
                    }
                }
            }
        }
        else
            i++;
    }
    //parse rest, e.g &lt; etc.
    text = fromStdString(PURPLE::stripHtml(text.text()));
    return text;
}

FXString dxutils::colorToHex(FXColor color)
{
    return FXStringFormat("%02X%02X%02X", FXREDVAL(color), FXGREENVAL(color), FXBLUEVAL(color));
}

bool dxutils::haveNotify()
{
    return m_haveNotify;
}

bool dxutils::hasLog(const FXString &accountName, const FXString &conversation, bool account)
{
    FXString path = account?DXPREFERENCES.m_logPath+PATHSEPSTRING+accountName:DXPREFERENCES.m_logPath+PATHSEPSTRING+accountName+PATHSEPSTRING+conversation;
    FXDir dir(path);
    if(account)
    {
        if(dir.isOpen())
        {
            while(dir.next())
            {
                if(FXStat::isDirectory(path+PATHSEPSTRING+dir.name()))
                {
                    FXDir adir(path+PATHSEPSTRING+dir.name());
                    if(adir.isOpen())
                    {
                        while(adir.next())
                        {
                            if(FXStat::isFile(path+PATHSEPSTRING+dir.name()+PATHSEPSTRING+adir.name())
                                    && isRightLogFile(path+PATHSEPSTRING+dir.name()+PATHSEPSTRING+adir.name(), adir.name()))
                                return true;
                        }
                    }
                }
            }
        }
    }
    else
    {
        if(dir.isOpen())
        {
            while(dir.next())
            {
                if(FXStat::isFile(path+PATHSEPSTRING+dir.name())
                        && isRightLogFile(path+PATHSEPSTRING+dir.name(), dir.name()))
                    return true;
            }
        }
    }
    return false;
}

