/*
 *      dxutils.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXUTILS_H
#define DXUTILS_H

#ifdef _WIN32
 #define _WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <mmsystem.h>
 #include <ctime>
 #include <shellapi.h>
#endif
#include "defs.h"
#include <FX885910Codec.h>
#include <FX885911Codec.h>
#include <FX885913Codec.h>
#include <FX885914Codec.h>
#include <FX885915Codec.h>
#include <FX885916Codec.h>
#include <FX88591Codec.h>
#include <FX88592Codec.h>
#include <FX88593Codec.h>
#include <FX88594Codec.h>
#include <FX88595Codec.h>
#include <FX88596Codec.h>
#include <FX88597Codec.h>
#include <FX88598Codec.h>
#include <FX88599Codec.h>
#include <FXCP1250Codec.h>
#include <FXCP1251Codec.h>
#include <FXCP1252Codec.h>
#include <FXCP1253Codec.h>
#include <FXCP1254Codec.h>
#include <FXCP1255Codec.h>
#include <FXCP1256Codec.h>
#include <FXCP1257Codec.h>
#include <FXCP1258Codec.h>
#include <FXCP437Codec.h>
#include <FXCP850Codec.h>
#include <FXCP852Codec.h>
#include <FXCP855Codec.h>
#include <FXCP856Codec.h>
#include <FXCP857Codec.h>
#include <FXCP860Codec.h>
#include <FXCP861Codec.h>
#include <FXCP862Codec.h>
#include <FXCP863Codec.h>
#include <FXCP864Codec.h>
#include <FXCP865Codec.h>
#include <FXCP866Codec.h>
#include <FXCP869Codec.h>
#include <FXCP874Codec.h>
#include <FXKOI8RCodec.h>
#include <FXUTF8Codec.h>

class dxutils
{
private:
    dxutils();

    FXTextCodec *m_lcodec;
    FXString m_iniPath;
    bool m_haveNotify;

    FXTextCodec* getCodec();
public:
    virtual ~dxutils();

    static dxutils &instance();
    FXString localeToUtf8(const FXchar *buffer);
    FXString localeToUtf8(const FXString &text);
    static FXString fromStdString(const std::string &str);
    static std::string toStdString(const FXString &str);
    void setIniPath(const FXString &path);
    FXString getIniPath();
    static void debugLine(const FXString&);
    static FXString checkThemeStatusPath(const FXString&);
    static FXString checkThemesStatusList(const FXString&);
    static FXString checkThemeRolePath(const FXString&);
    static FXString checkThemesRoleList(const FXString&);
    static void playFile(const FXString &file);
    static FXbool isUtf8(const FXchar* string, FXuint length);
    FXbool getBoolIniEntry(const FXchar* section, const FXchar* key, FXbool def=FALSE);
    FXint getIntIniEntry(const FXchar* section, const FXchar* key, FXint def=0);
    static dxStringArray splitString(const FXString &text, FXchar separator);
    static FXint launchLink(const FXString &link);
    static FXString getFileSize(size_t size);
    static FXString getFileSize(const FXString &ssize);
    static FXString getSpeed(size_t speed);
    static FXString getRemaining(size_t size, size_t speed);
    static FXString getPercentPosition(size_t size, size_t position);
    static FXString getMonth(FXint month);
    static FXString getMonthNumber(const FXString &month);
    static FXString getParam(FXString, FXint, FXbool);
    static FXString getParam(FXString, FXint, FXbool, const FXchar&);
    static FXString stamp2fxstring(FXString datestr, FXString timestr);
    static FXString normalizeLogLine(FXString datestr, std::string line);
    static bool isRightLogFile(const FXString& path, const FXString& name);
    static void sortStringArray(dxStringArray &array);
    static bool containsStringArray(const dxStringArray &array, const FXString &value);
    static FXIcon *getProtocolIcon(const FXString &protocol);
    static FXIcon *getPresenceIcon(PurpleStatusPrimitive presence);
    static FXString getChatRoleString(PurpleConvChatBuddyFlags role);
    static FXIcon *getChatRoleIcon(PurpleConvChatBuddyFlags role);
    static FXColor makeOfflineColor(FXColor clr);
    static FXString typingToString(FXString nick, bool typing);
    static FXString getKeybinding(FXEvent *event);
    static FXHotKey parseAccel(const FXString& string);
    /**
     *
     * @param label         string to normalize
     * @param normalize     char for remove, usually '_'
     * @return              Normalized FXString
     */
    static FXString normalizeLabelString(std::string label, char normalize);
    static FXString parseHtml(FXString text);
    static FXString colorToHex(FXColor color);
    bool haveNotify();
    static bool hasLog(const FXString &accountName, const FXString &conversation, bool account);
};


#endif  /* DXUTILS_H */

