//      preferences.cpp
//
//      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.


#include "defs.h"
#include "config.h"
#include "dxutils.h"
#include "i18n.h"
#include "preferences.h"

Preferences::Preferences() :
    m_oneInstance(FALSE),
    m_useAway(TRUE),
    m_awayTimeout(5),
#ifdef HAVE_TRAY
    m_useTray(TRUE),
#else
    m_useTray(FALSE),
#endif //HAVE_TRAY
    m_closeToTray(m_useTray?TRUE:FALSE),
    m_minToTray(FALSE),
    m_logging(TRUE),
#ifdef _WIN32
    m_logPath(dxutils::instance().localeToUtf8(FXSystem::getEnvironment("AppData")+PATHSEPSTRING+"dxim"+PATHSEPSTRING+"logs")),
#else
    m_logPath(FXSystem::getHomeDirectory()+PATHSEPSTRING+".dxim"+PATHSEPSTRING+"logs"),
#endif
    m_logLoadCurrent(FALSE),
    m_showRosterTree(TRUE),
    m_receiptsMark(TRUE),
    m_themeStatusPath(DXIM_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "statuses"),
    m_themesStatusList(FXString(m_themeStatusPath+";")),
    m_themeRolePath(DXIM_DATADIR PATHSEPSTRING "icons" PATHSEPSTRING "roles"),
    m_themesRoleList(FXString(m_themeRolePath+";")),
    m_unreadColor(FXRGB(0,0,255)),
    m_highlightColor(FXRGB(255,0,0)),
    m_tabPosition(0),
    m_font("Helvetica,90,normal,regular"),
    m_imFont(""),
    m_nickCompletionChar(":"),
    m_sameCmd(FALSE),
    m_sameList(FALSE),
    m_coloredNick(TRUE),
    m_sounds(FALSE),
    m_soundConnect(FALSE),
    m_pathConnect(DXIM_DATADIR PATHSEPSTRING "sounds" PATHSEPSTRING "connected.wav"),
    m_soundDisconnect(FALSE),
    m_pathDisconnect(DXIM_DATADIR PATHSEPSTRING "sounds" PATHSEPSTRING "disconnected.wav"),
    m_soundMessage(FALSE),
    m_pathMessage(DXIM_DATADIR PATHSEPSTRING "sounds" PATHSEPSTRING "message.wav"),
    m_notify(FALSE),
    m_notifyPosition(3),
    m_notifyAccountConnect(FALSE),
    m_notifyAccountDisconnect(FALSE),
    m_notifyBuddyConnect(FALSE),
    m_notifyBuddyDisconnect(FALSE),
    m_notifyMessage(FALSE),
    m_notifyFiletransfer(FALSE),
    m_raiseWindow(FALSE),
    m_notifyMessageTray(FALSE),
    m_useSmileys(FALSE),
    m_hideOffline(TRUE),
    m_rosterShown(TRUE),
    m_chatRosterShown(TRUE),
    m_showStatusmsg(TRUE),
    m_x(50),
    m_y(50),
    m_w(800),
    m_h(600),
    m_maximized(FALSE),
    m_rosterWidth(180),
    m_showSpellCombo(TRUE),
    m_showAvatars(TRUE)
{
    m_appTheme.base = FXRGB(212,208,200);
    m_appTheme.back = FXRGB(255,255,255);
    m_appTheme.border = FXRGB(0,0,0);
    m_appTheme.fore = FXRGB(0,0,0);
    m_appTheme.menuback = FXRGB(10,36,106);
    m_appTheme.menufore = FXRGB(255,255,255);
    m_appTheme.selback = FXRGB(10,36,106);
    m_appTheme.selfore = FXRGB(255,255,255);
    m_appTheme.tipback = FXRGB(255,255,225);
    m_appTheme.tipfore = FXRGB(0,0,0);
    m_appTheme.notifyback = FXRGB(255,255,225);
    m_appTheme.notifyfore = FXRGB(0,0,0);
    m_appTheme.hilite = makeHiliteColor(m_appTheme.base);
    m_appTheme.shadow = makeShadowColor(m_appTheme.base);
    m_colors.text = FXRGB(0,0,0);
    m_colors.back = FXRGB(255,255,255);
    m_colors.user = FXRGB(43,57,35);
    m_colors.action = FXRGB(255,165,0);
    m_colors.error = FXRGB(255,0,0);
    m_colors.hilight = FXRGB(11,154,11);
    m_colors.link = FXRGB(142,196,210);
    m_colors.mymsg = FXRGB(130,150,253);
    m_colors.history = FXRGB(43,57,35);
    fillDefaultKeybindings();
}

Preferences &Preferences::instance()
{
    static Preferences p;
    return p;
}

void Preferences::load()
{
    FXSettings set;
    set.parseFile(dxutils::instance().getIniPath()+PATHSEPSTRING+"dxim-fox", TRUE);

    //General settings tab in Configdialog
    m_themeStatusPath = dxutils::checkThemeStatusPath(set.readStringEntry("GENERAL", "themeStatusPath", m_themeStatusPath.text()));
    m_themesStatusList = dxutils::checkThemesStatusList(set.readStringEntry("GENERAL", "themesStatusList", FXString(m_themeStatusPath+";").text()));
    if(!m_themesStatusList.contains(m_themeStatusPath)) m_themesStatusList.append(m_themeStatusPath+";");
    m_themeRolePath = dxutils::checkThemeRolePath(set.readStringEntry("GENERAL", "themeRolePath", m_themeRolePath.text()));
    m_themesRoleList = dxutils::checkThemesRoleList(set.readStringEntry("GENERAL", "themesRoleList", FXString(m_themeRolePath+";").text()));
    if(!m_themesRoleList.contains(m_themeRolePath)) m_themesRoleList.append(m_themeRolePath+";");
    m_oneInstance = set.readBoolEntry("GENERAL", "oneinstance", m_oneInstance);
    m_useAway = set.readBoolEntry("GENERAL", "useAway", m_useAway);
    m_awayTimeout = set.readIntEntry("GENERAL", "awayTimeout", m_awayTimeout);
    m_useTray = set.readBoolEntry("GENERAL", "tray", m_useTray);
    m_closeToTray = set.readBoolEntry("GENERAL", "closeToTray", m_closeToTray);
    m_minToTray = set.readBoolEntry("GENERAL", "minToTray", m_minToTray);
    m_logging = set.readBoolEntry("GENERAL", "logging", m_logging);
    m_logPath = set.readStringEntry("GENERAL", "logPath", m_logPath.text());
    m_logLoadCurrent = set.readBoolEntry("GENERAL", "logLoadCurrent", m_logLoadCurrent);
    if(!m_logging) m_logLoadCurrent = false;
    m_showRosterTree = set.readBoolEntry("GENERAL", "showRosterTree", m_showRosterTree);
    m_receiptsMark = set.readBoolEntry("GENERAL", "receiptsMark", m_receiptsMark);

    //Look settings tab in Configdialog
    m_appTheme.base = set.readColorEntry("LOOK", "basecolor", m_appTheme.base);
    m_appTheme.back = set.readColorEntry("LOOK", "backcolor", m_appTheme.back);
    m_appTheme.border = set.readColorEntry("LOOK", "bordercolor", m_appTheme.border);
    m_appTheme.fore = set.readColorEntry("LOOK", "forecolor", m_appTheme.fore);
    m_appTheme.menuback = set.readColorEntry("LOOK", "selmenubackcolor", m_appTheme.menuback);
    m_appTheme.menufore = set.readColorEntry("LOOK", "selmenutextcolor", m_appTheme.menufore);
    m_appTheme.selback = set.readColorEntry("LOOK", "selbackcolor", m_appTheme.selback);
    m_appTheme.selfore = set.readColorEntry("LOOK", "selforecolor", m_appTheme.selfore);
    m_appTheme.tipback = set.readColorEntry("LOOK", "tipbackcolor", m_appTheme.tipback);
    m_appTheme.tipfore = set.readColorEntry("LOOK", "tipforecolor", m_appTheme.tipfore);
    m_appTheme.notifyback = set.readColorEntry("LOOK", "notifybackcolor", m_appTheme.notifyback);
    m_appTheme.notifyfore = set.readColorEntry("LOOK", "notifyforecolor", m_appTheme.notifyfore);
    m_appTheme.hilite = set.readColorEntry("LOOK", "hilitecolor", m_appTheme.hilite);
    m_appTheme.shadow = set.readColorEntry("LOOK", "shadowcolor", m_appTheme.shadow);
    m_trayColor = set.readColorEntry("LOOK", "traycolor", m_appTheme.back);
    m_unreadColor = set.readColorEntry("LOOK", "unreadcolor", m_unreadColor);
    m_highlightColor = set.readColorEntry("LOOK", "highlightcolor", m_highlightColor);
    m_tabPosition = set.readIntEntry("LOOK", "tabPosition", m_tabPosition);
    m_font = set.readStringEntry("LOOK", "normalfont", m_font.text());

    //Im Text settings tab in Configdialog
    m_colors.text = set.readColorEntry("IMTEXT", "textColor", m_colors.text);
    m_colors.back = set.readColorEntry("IMTEXT", "textBackColor", m_colors.back);
    m_colors.user = set.readColorEntry("IMTEXT", "userColor", m_colors.user);
    m_colors.action = set.readColorEntry("IMTEXT", "actionsColor", m_colors.action);
    m_colors.error = set.readColorEntry("IMTEXT", "errorColor", m_colors.error);
    m_colors.hilight = set.readColorEntry("IMTEXT", "hilightColor", m_colors.hilight);
    m_colors.link = set.readColorEntry("IMTEXT", "linkColor", m_colors.link);
    m_colors.mymsg = set.readColorEntry("IMTEXT", "mymsgColor", m_colors.mymsg);
    m_colors.history = set.readColorEntry("IMTEXT", "historyColor", m_colors.history);
    m_imFont = set.readStringEntry("IMTEXT", "imFont", m_imFont.text());
    m_nickCompletionChar = FXString(set.readStringEntry("IMTEXT", "nickCompletionChar", m_nickCompletionChar.text())).left(1);
    m_sameCmd = set.readBoolEntry("IMTEXT", "sameCmd", m_sameCmd);
    m_sameList = set.readBoolEntry("IMTEXT", "sameList", m_sameList);
    m_coloredNick = set.readBoolEntry("IMTEXT", "coloredNick", m_coloredNick);

    //Notification settings tab in Configdialog
    m_sounds = set.readBoolEntry("NOTIFICATION", "sounds", m_sounds);
    m_soundConnect = set.readBoolEntry("NOTIFICATION", "soundConnect", m_soundConnect);
    m_pathConnect = set.readStringEntry("NOTIFICATION", "pathConnect", m_pathConnect.text());
    m_soundDisconnect = set.readBoolEntry("NOTIFICATION", "soundDisconnect", m_soundDisconnect);
    m_pathDisconnect = set.readStringEntry("NOTIFICATION", "pathDisconnect", m_pathDisconnect.text());
    m_soundMessage = set.readBoolEntry("NOTIFICATION", "soundMessage", m_soundMessage);
    m_pathMessage = set.readStringEntry("NOTIFICATION", "pathMessage", m_pathMessage.text());
    m_notify = set.readBoolEntry("NOTIFICATION", "notify", m_notify);
    m_notifyPosition = set.readIntEntry("NOTIFICATION", "notifyPosition", m_notifyPosition);
    m_notifyAccountConnect = set.readBoolEntry("NOTIFICATION", "notifyAccountConnect", m_notifyAccountConnect);
    m_notifyAccountDisconnect = set.readBoolEntry("NOTIFICATION", "notifyAccountDisconnect", m_notifyAccountDisconnect);
    m_notifyBuddyConnect = set.readBoolEntry("NOTIFICATION", "notifyBuddyConnect", m_notifyBuddyConnect);
    m_notifyBuddyDisconnect = set.readBoolEntry("NOTIFICATION", "notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    m_notifyMessage = set.readBoolEntry("NOTIFICATION", "notifyMessage", m_notifyMessage);
    m_notifyFiletransfer = set.readBoolEntry("NOTIFICATION", "notifyFiletransfer", m_notifyFiletransfer);
    m_raiseWindow = set.readBoolEntry("NOTIFICATION", "raiseWindow", m_raiseWindow);
    m_notifyMessageTray = set.readBoolEntry("NOTIFICATION", "notifyMessageTray", m_notifyMessageTray);
    if(m_raiseWindow) m_notifyMessageTray = FALSE;

    //Smileys settings tab in Configdialog
    m_useSmileys = set.readBoolEntry("SMILEYS", "useSmileys", m_useSmileys);
    FXint smileysNum = set.readIntEntry("SMILEYS", "number", 0);
    m_smileysMap.clear();
    if(smileysNum)
    {
        for(FXint i=0; i<smileysNum; i++)
        {
            FXString key, value;
            key = set.readStringEntry("SMILEYS", FXStringFormat("smiley%d", i).text(), "");
            value = set.readStringEntry("SMILEYS", FXStringFormat("path%d", i).text(), "");
            if(!key.empty())
                m_smileysMap.insert(dxStringPair(key, value));
        }
    }

    //Other settings value
    m_hideOffline = set.readBoolEntry("OTHER", "hideOffline", m_hideOffline);
    m_rosterShown = set.readBoolEntry("OTHER", "rosterShown", m_rosterShown);
    m_chatRosterShown = set.readBoolEntry("OTHER", "chatRosterShown", m_chatRosterShown);
    m_showStatusmsg = set.readBoolEntry("OTHER", "showStatusmsg", m_showStatusmsg);
    m_x=set.readIntEntry("OTHER","x", m_x);
    m_y=set.readIntEntry("OTHER","y", m_y);
    m_w=set.readIntEntry("OTHER","w", m_w);
    m_h=set.readIntEntry("OTHER","h", m_h);
    m_maximized = set.readBoolEntry("OTHER", "maximized", m_maximized);
    m_rosterWidth = set.readIntEntry("OTHER", "rosterWidth", m_rosterWidth);
    m_showSpellCombo = set.readBoolEntry("OTHER", "showSpellCombo", m_showSpellCombo);
    m_showAvatars = set.readBoolEntry("OTHER", "showAvatars", m_showAvatars);
    if((FXint)m_defaultkeybindings.size())
    {
        dxStringIt it;
        for(it=m_defaultkeybindings.begin(); it!=m_defaultkeybindings.end(); it++)
            m_keybindings.insert(dxStringPair((*it).first, set.readStringEntry("KEYBINDINGS", (*it).first.text(), (*it).second.text())));
    }
}

void Preferences::save()
{
    FXSettings set;

    //General settings tab in Configdialog
    set.writeStringEntry("GENERAL", "themeStatusPath", m_themeStatusPath.text());
    set.writeStringEntry("GENERAL", "themesStatusList", m_themesStatusList.text());
    set.writeStringEntry("GENERAL", "themeRolePath", m_themeRolePath.text());
    set.writeStringEntry("GENERAL", "themesRoleList", m_themesRoleList.text());
    set.writeBoolEntry("GENERAL", "oneinstance", m_oneInstance);
    set.writeBoolEntry("GENERAL", "useAway", m_useAway);
    set.writeIntEntry("GENERAL", "awayTimeout", m_awayTimeout);
    set.writeBoolEntry("GENERAL", "tray", m_useTray);
    set.writeBoolEntry("GENERAL", "closeToTray", m_closeToTray);
    set.writeBoolEntry("GENERAL", "minToTray", m_minToTray);
    set.writeBoolEntry("GENERAL", "logging", m_logging);
    set.writeStringEntry("GENERAL", "logPath", m_logPath.text());
    set.writeBoolEntry("GENERAL", "logLoadCurrent", m_logLoadCurrent);
    set.writeBoolEntry("GENERAL", "showRosterTree", m_showRosterTree);
    set.writeBoolEntry("GENERAL", "receiptsMark", m_receiptsMark);

    //Look settings tab in Configdialog
    set.writeColorEntry("LOOK", "basecolor", m_appTheme.base);
    set.writeColorEntry("LOOK", "backcolor", m_appTheme.back);
    set.writeColorEntry("LOOK", "bordercolor", m_appTheme.border);
    set.writeColorEntry("LOOK", "forecolor", m_appTheme.fore);
    set.writeColorEntry("LOOK", "selmenubackcolor", m_appTheme.menuback);
    set.writeColorEntry("LOOK", "selmenutextcolor", m_appTheme.menufore);
    set.writeColorEntry("LOOK", "selbackcolor", m_appTheme.selback);
    set.writeColorEntry("LOOK", "selforecolor", m_appTheme.selfore);
    set.writeColorEntry("LOOK", "tipbackcolor", m_appTheme.tipback);
    set.writeColorEntry("LOOK", "tipforecolor", m_appTheme.tipfore);
    set.writeColorEntry("LOOK", "notifybackcolor", m_appTheme.notifyback);
    set.writeColorEntry("LOOK", "notifyforecolor", m_appTheme.notifyfore);
    set.writeColorEntry("LOOK", "hilitecolor", m_appTheme.hilite);
    set.writeColorEntry("LOOK", "shadowcolor", m_appTheme.shadow);
    set.writeColorEntry("LOOK", "traycolor", m_trayColor);
    set.writeColorEntry("LOOK", "unreadcolor", m_unreadColor);
    set.writeColorEntry("LOOK", "highlightcolor", m_highlightColor);
    set.writeIntEntry("LOOK", "tabPosition", m_tabPosition);
    set.writeStringEntry("LOOK", "normalfont", m_font.text());

    //Im Text settings tab in Configdialog
    set.writeColorEntry("IMTEXT", "textColor", m_colors.text);
    set.writeColorEntry("IMTEXT", "textBackColor", m_colors.back);
    set.writeColorEntry("IMTEXT", "userColor", m_colors.user);
    set.writeColorEntry("IMTEXT", "actionsColor", m_colors.action);
    set.writeColorEntry("IMTEXT", "errorColor", m_colors.error);
    set.writeColorEntry("IMTEXT", "hilightColor", m_colors.hilight);
    set.writeColorEntry("IMTEXT", "linkColor", m_colors.link);
    set.writeColorEntry("IMTEXT", "mymsgColor", m_colors.mymsg);
    set.writeColorEntry("IMTEXT", "historyColor", m_colors.history);
    set.writeStringEntry("IMTEXT", "imFont", m_imFont.text());
    set.writeStringEntry("IMTEXT", "nickCompletionChar", m_nickCompletionChar.left(1).text());
    set.writeBoolEntry("IMTEXT", "sameCmd", m_sameCmd);
    set.writeBoolEntry("IMTEXT", "sameList", m_sameList);
    set.writeBoolEntry("IMTEXT", "coloredNick", m_coloredNick);

    //Notification settings tab in Configdialog
    set.writeBoolEntry("NOTIFICATION", "sounds", m_sounds);
    set.writeBoolEntry("NOTIFICATION", "soundConnect", m_soundConnect);
    set.writeStringEntry("NOTIFICATION", "pathConnect", m_pathConnect.text());
    set.writeBoolEntry("NOTIFICATION", "soundDisconnect", m_soundDisconnect);
    set.writeStringEntry("NOTIFICATION", "pathDisconnect", m_pathDisconnect.text());
    set.writeBoolEntry("NOTIFICATION", "soundMessage", m_soundMessage);
    set.writeStringEntry("NOTIFICATION", "pathMessage", m_pathMessage.text());
    set.writeBoolEntry("NOTIFICATION", "notify", m_notify);
    set.writeIntEntry("NOTIFICATION", "notifyPosition", m_notifyPosition);
    set.writeBoolEntry("NOTIFICATION", "notifyAccountConnect", m_notifyAccountConnect);
    set.writeBoolEntry("NOTIFICATION", "notifyAccountDisconnect", m_notifyAccountDisconnect);
    set.writeBoolEntry("NOTIFICATION", "notifyBuddyConnect", m_notifyBuddyConnect);
    set.writeBoolEntry("NOTIFICATION", "notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    set.writeBoolEntry("NOTIFICATION", "notifyMessage", m_notifyMessage);
    set.writeBoolEntry("NOTIFICATION", "notifyFiletransfer", m_notifyFiletransfer);
    set.writeBoolEntry("NOTIFICATION", "raiseWindow", m_raiseWindow);
    set.writeBoolEntry("NOTIFICATION", "notifyMessageTray", m_notifyMessageTray);

    //Smileys settings tab in Configdialog
    set.writeBoolEntry("SMILEYS", "useSmileys", m_useSmileys);
    set.writeIntEntry("SMILEYS", "number", (FXint)m_smileysMap.size());
    if((FXint)m_smileysMap.size())
    {
        dxStringIt it;
        FXint i;
        for(i=0, it=m_smileysMap.begin(); it!=m_smileysMap.end(); it++,i++)
        {
            set.writeStringEntry("SMILEYS", FXStringFormat("smiley%d", i).text(), (*it).first.text());
            set.writeStringEntry("SMILEYS", FXStringFormat("path%d", i).text(), (*it).second.text());
        }
    }

    //Other settings value
    set.writeBoolEntry("OTHER", "hideOffline", m_hideOffline);
    set.writeBoolEntry("OTHER", "rosterShown", m_rosterShown);
    set.writeBoolEntry("OTHER", "chatRosterShown", m_chatRosterShown);
    set.writeBoolEntry("OTHER", "showStatusmsg", m_showStatusmsg);
    set.writeIntEntry("OTHER","x", m_x);
    set.writeIntEntry("OTHER","y", m_y);
    set.writeIntEntry("OTHER","w", m_w);
    set.writeIntEntry("OTHER","h", m_h);
    set.writeBoolEntry("OTHER", "maximized", m_maximized);
    set.writeIntEntry("OTHER", "rosterWidth", m_rosterWidth);
    set.writeBoolEntry("OTHER", "showSpellCombo", m_showSpellCombo);
    set.writeBoolEntry("OTHER", "showAvatars", m_showAvatars);
    if((FXint)m_keybindings.size())
    {
        dxStringIt it;
        for(it=m_keybindings.begin(); it!=m_keybindings.end(); it++)
            set.writeStringEntry("KEYBINDINGS", (*it).first.text(), (*it).second.text());
    }

    set.setModified();
    set.unparseFile(dxutils::instance().getIniPath()+PATHSEPSTRING+"dxim-fox");
}

void Preferences::fillDefaultKeybindings()
{
    m_defaultkeybindings.clear();
    m_namekeybindings.clear();
    m_defaultkeybindings.insert(dxStringPair("manageAccounts", "Ctrl-A"));
    m_namekeybindings.insert(dxStringPair("manageAccounts", _("Manage accounts")));
    m_defaultkeybindings.insert(dxStringPair("newIM", "Ctrl-M"));
    m_namekeybindings.insert(dxStringPair("newIM", _("New IM")));
    m_defaultkeybindings.insert(dxStringPair("joinChat", "Ctrl-T"));
    m_namekeybindings.insert(dxStringPair("joinChat", _("Join chat")));
    m_defaultkeybindings.insert(dxStringPair("fileTransfers", "Ctrl-F"));
    m_namekeybindings.insert(dxStringPair("fileTransfers", _("File transfers")));
    m_defaultkeybindings.insert(dxStringPair("logViewer", "Ctrl-G"));
    m_namekeybindings.insert(dxStringPair("logViewer", _("Log viewer")));
#ifdef _WIN32
    m_defaultkeybindings.insert(dxStringPair("quit", "Alt-F4"));
#else
    m_defaultkeybindings.insert(dxStringPair("quit", "Ctrl-Q"));
#endif //_WIN32
    m_namekeybindings.insert(dxStringPair("quit", _("Quit")));
    m_defaultkeybindings.insert(dxStringPair("closeCurrentTab", "Ctrl-W"));
    m_namekeybindings.insert(dxStringPair("closeCurrentTab", _("Close current tab")));
    m_defaultkeybindings.insert(dxStringPair("clearWindow", "Ctrl-L"));
    m_namekeybindings.insert(dxStringPair("clearWindow", _("Clear window")));
    m_defaultkeybindings.insert(dxStringPair("clearAllWindows", "Ctrl-Shift-L"));
    m_namekeybindings.insert(dxStringPair("clearAllWindows", _("Clear all windows")));
    m_defaultkeybindings.insert(dxStringPair("friendsList", "Ctrl-U"));
    m_namekeybindings.insert(dxStringPair("friendsList", _("Friends list")));
    m_defaultkeybindings.insert(dxStringPair("switchTab", "Ctrl-Tab"));
    m_namekeybindings.insert(dxStringPair("switchTab", _("Switch between tabs")));
    m_defaultkeybindings.insert(dxStringPair("unreadTab", "Ctrl-N"));
    m_namekeybindings.insert(dxStringPair("unreadTab", _("Switch between unreaded tabs")));
    m_defaultkeybindings.insert(dxStringPair("firstTab", "Alt-1"));
    m_namekeybindings.insert(dxStringPair("firstTab", _("Go to 1. tab")));
    m_defaultkeybindings.insert(dxStringPair("secondTab", "Alt-2"));
    m_namekeybindings.insert(dxStringPair("secondTab", _("Go to 2. tab")));
    m_defaultkeybindings.insert(dxStringPair("thirdTab", "Alt-3"));
    m_namekeybindings.insert(dxStringPair("thirdTab", _("Go to 3. tab")));
    m_defaultkeybindings.insert(dxStringPair("fourthTab", "Alt-4"));
    m_namekeybindings.insert(dxStringPair("fourthTab", _("Go to 4. tab")));
    m_defaultkeybindings.insert(dxStringPair("fifthTab", "Alt-5"));
    m_namekeybindings.insert(dxStringPair("fifthTab", _("Go to 5. tab")));
    m_defaultkeybindings.insert(dxStringPair("sixthTab", "Alt-6"));
    m_namekeybindings.insert(dxStringPair("sixthTab", _("Go to 6. tab")));
    m_defaultkeybindings.insert(dxStringPair("seventhTab", "Alt-7"));
    m_namekeybindings.insert(dxStringPair("seventhTab", _("Go to 7. tab")));
    m_defaultkeybindings.insert(dxStringPair("eighthTab", "Alt-8"));
    m_namekeybindings.insert(dxStringPair("eighthTab", _("Go to 8. tab")));
    m_defaultkeybindings.insert(dxStringPair("ninethTab", "Alt-9"));
    m_namekeybindings.insert(dxStringPair("ninethTab", _("Go to 9. tab")));
}

FXString Preferences::getBinding(const FXString& key)
{
    if(m_keybindings.find(key)==m_keybindings.end())
        return "";
    else
        return m_keybindings.find(key)->second;
}

FXString Preferences::getBindingName(const FXString& key)
{
    if(m_namekeybindings.find(key)==m_namekeybindings.end())
        return "";
    else
        return m_namekeybindings.find(key)->second;
}


