/*
*      preferences.h
*
*      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 3 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
*      MA 02110-1301, USA.
*/


#ifndef PREFERENCES_H
#define PREFERENCES_H

#define DXPREFERENCES Preferences::instance()

class Preferences
{
public:
    Preferences();
    Preferences(const Preferences &p);

    //Save to FXSettings
    void save();
    //Load from FXSettings
    void load();
    static Preferences &instance();

    //General settings tab in Configdialog
    FXbool m_oneInstance;
    FXbool m_useAway;
    FXint m_awayTimeout; //in minutes
    FXbool m_useTray;
    FXbool m_closeToTray;
    FXbool m_minToTray;
    FXbool m_logging;
    FXString m_logPath;
    FXbool m_logLoadCurrent;
    FXbool m_showRosterTree;
    FXbool m_receiptsMark;

    //Icons settings tab in Configdialog
    FXString m_themeStatusPath;
    FXString m_themesStatusList;
    FXString m_themeRolePath;
    FXString m_themesRoleList;

    //Look settings tab in Configdialog
    ColorTheme m_appTheme;
    FXColor m_trayColor;
    FXColor m_unreadColor;
    FXColor m_highlightColor;
    FXint m_tabPosition;
    FXString m_font;

    //Im Text settings tab in Configdialog
    ImColor m_colors;
    FXString m_imFont;
    FXString m_nickCompletionChar;
    FXbool m_sameCmd;
    FXbool m_sameList;
    FXbool m_coloredNick;

    //Notification settings tab in Configdialog
    FXbool m_sounds;
    FXbool m_soundConnect;
    FXString m_pathConnect;
    FXbool m_soundDisconnect;
    FXString m_pathDisconnect;
    FXbool m_soundMessage;
    FXString m_pathMessage;
    FXbool m_notify;
    FXint m_notifyPosition;
    FXbool m_notifyAccountConnect;
    FXbool m_notifyAccountDisconnect;
    FXbool m_notifyBuddyConnect;
    FXbool m_notifyBuddyDisconnect;
    FXbool m_notifyMessage;
    FXbool m_notifyFiletransfer;
    FXbool m_raiseWindow;
    FXbool m_notifyMessageTray; //true = all messages change tray icon

    //Smileys settings tab in Configdialog
    FXbool m_useSmileys;
    dxStringMap m_smileysMap;

    //Other settings value
    FXbool m_hideOffline;
    FXbool m_rosterShown;
    FXbool m_chatRosterShown;
    FXbool m_showStatusmsg;
    FXint m_x;
    FXint m_y;
    FXint m_w;
    FXint m_h;
    FXbool m_maximized;
    FXint m_rosterWidth;
    FXbool m_showSpellCombo;
    FXbool m_showAvatars;
    dxStringMap m_keybindings;

    FXString getBinding(const FXString &key);
    FXString getBindingName(const FXString &key);
    dxStringMap getDefaultKeybindings() { return m_defaultkeybindings; }
private:
    dxStringMap m_defaultkeybindings;
    dxStringMap m_namekeybindings;

    void fillDefaultKeybindings();
};

#endif /* PREFERENCES_H */
