/*
 *      roster.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ROSTER_H
#define ROSTER_H

#include "defs.h"

class dxim;
class dxRosterTreeItem;

class dxRosterList : public FXTreeList
{
    FXDECLARE(dxRosterList)
    friend class dxim;
public:
    dxRosterList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=TREELIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
    long onQueryTip(FXObject*,FXSelector,void*);
    void setItemSecondIcon(dxRosterTreeItem* item,FXIcon* icon,FXbool owned=FALSE);
    FXIcon* getItemSecondIcon(const dxRosterTreeItem* item) const;
protected:
    dxRosterList() {}
private:
    dxRosterList(const dxRosterList&);
    dxRosterList& operator=(const dxRosterList&);
};

class dxRosterTreeItem : public FXTreeItem
{
    FXDECLARE(dxRosterTreeItem)
    friend class dxRosterList;
    friend class dxim;
public:
    dxRosterTreeItem(const FXString& text, const FXString& ID, PURPLE::Account account, PURPLE::Buddy buddy = PURPLE::Buddy(), FXIcon* oi = NULL, FXIcon* ci = NULL, void* ptr = NULL, FXIcon* ai = NULL)
            : FXTreeItem(text, oi, ci, ptr), m_secondIcon(ai), m_ID(ID),
                    m_lastStatusChange(FXSystem::now()), m_buddy(buddy), m_account(account) {}
    enum {
        SECONDICONOWNED = 512, //second icon owned by item
        ACCOUNT = 1024, //Account item
        GROUP = 2048, //Group item
        FRIEND = 4096, //Frien item
        CHAT = 8192, //Bookmarked MUC
        HIDDEN = 16384 //Hidden item
    };

    void setSecondIcon(FXIcon *icn, FXbool owned=FALSE);
    FXIcon* getSecondIcon() const { return m_secondIcon; }
    FXint getWidth(const FXTreeList* list) const;
    FXint getHeight(const FXTreeList* list) const;
    FXbool isAccount() const { return (state&ACCOUNT)!=0; }
    FXbool isGroup() const { return (state&GROUP)!=0; }
    FXbool isFriend() const { return (state&FRIEND)!=0; }
    FXbool isChat() const { return (state&CHAT)!=0; }
    FXbool isHidden() const { return (state&HIDDEN)!=0; }
    FXint getNumHiddenChildren() const;
    FXString getID() const { return m_ID; }
    void setID(const FXString &ID);
    PURPLE::Account getAccount() const { return m_account; }
    PURPLE::Buddy getBuddy() const { return m_buddy; }
    PURPLE::Chat getChat() const { return m_chat; }
    void setChat(PURPLE::Chat chat);
    void setAvatarPath(FXString avatarPath);
    const FXString& getAvatarPath() const { return m_avatarPath; }
    void updateShowAvatars();
    FXString getProtocol();
    void setStatusChange();
    FXTime getLastStatusChange() const { return m_lastStatusChange; }
    FXString getStatusMsg() const;
    FXbool isAccountOnline() const { return m_account.isConnected(); }
protected:
    dxRosterTreeItem() : m_secondIcon(NULL), m_lastStatusChange(FXSystem::now()), m_buddy(PURPLE::Buddy()), m_account(PURPLE::Account()) {}
    FXIcon *m_secondIcon; //second icon ssl or avatar

    void draw(const FXTreeList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
private:
    dxRosterTreeItem(const dxRosterTreeItem&);
    dxRosterTreeItem & operator=(const dxRosterTreeItem&);

    FXString m_ID;
    FXTime m_lastStatusChange;
    PURPLE::Buddy m_buddy;
    FXString m_avatarPath;
    PURPLE::Account m_account;
    PURPLE::Chat m_chat;

    FXbool isOnline() const;
    FXString getWrappedStatusMsg(FXFont *font, FXint w) const;
    FXint charWidth(FXFont *font, FXwchar ch, FXint col) const;
};

#endif // ROSTER_H

