/*
 *      chattabitem.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

/*
 * DON'T USE ChatTabItem::appendLinkText
 * m_text->appendText and
 * m_text->appendStyledText DIRECTLY
 */

#include "chattabitem.h"
#include "icons.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"
#include "dxim.h"
#include <utilsmm.h>

//Map
FXDEFMAP(dxChatList) dxChatListMap [] = {
    FXMAPFUNC(SEL_QUERY_TIP, 0, dxChatList::onQueryTip)
};

FXIMPLEMENT(dxChatList, FXList, dxChatListMap, ARRAYNUMBER(dxChatListMap))

dxChatList::dxChatList(FXComposite* p, FXObject* tgt, FXSelector sel, FXuint opts, FXint x, FXint y, FXint w, FXint h)
: FXList(p, tgt, sel, opts, x, y, w, h)
{

}

long dxChatList::onQueryTip(FXObject *sender, FXSelector sel, void *ptr)
{
    if(FXWindow::onQueryTip(sender,sel,ptr)) return 1;
    if((flags&FLAG_TIP) && !(options&LIST_AUTOSELECT) && (0<=cursor)) // No tip when autoselect!
    {
        dxChatItem *item = (dxChatItem*)items[cursor];
        FXString string = item->getText();
        string.append('\n');
        if(!item->getRealName().empty()) string.append(FXStringFormat(_("ID: %s\n"), item->getRealName().text()));
        string.append(FXStringFormat(_("Role: %s"), dxutils::getChatRoleString(item->getRole()).text()));
        sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&string);
        return 1;
    }
  return 0;
}

FXIMPLEMENT(dxChatItem,  FXObject, NULL, 0)

FXDEFMAP(ChatTabItem) ChatTabItemMap[] = {
    FXMAPFUNC(SEL_COMMAND, ChatTabItem_COMMANDLINE, ChatTabItem::onCommandline),
    FXMAPFUNC(SEL_LINK, ChatTabItem_TOPIC, dxTabItem::onTextLink),
    FXMAPFUNC(SEL_KEYPRESS, ChatTabItem_COMMANDLINE, ChatTabItem::onKeyPress),
    FXMAPFUNC(SEL_COMMAND, ChatTabItem_TOPIC, ChatTabItem::onTopicline),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, ChatTabItem_USERS, ChatTabItem::onRightRelease),
    FXMAPFUNC(SEL_COMMAND, ChatTabItem_POPUPIM, ChatTabItem::onPopupIM),
    FXMAPFUNC(SEL_COMMAND, ChatTabItem_POPUPINFO, ChatTabItem::onPopupInfo),
    FXMAPFUNC(SEL_COMMAND, ChatTabItem_POPUPIGNORE, ChatTabItem::onPopupIgnore),
    FXMAPFUNC(SEL_COMMAND, LogThread_LIST, ChatTabItem::onListLogThread)
};

FXIMPLEMENT(ChatTabItem, dxTabItem, ChatTabItemMap, ARRAYNUMBER(ChatTabItemMap))

ChatTabItem::ChatTabItem(dxTabBook *tab, const FXString &tabtext, PURPLE::Conversation *conv, FXIcon *ic, FXuint opts)
: dxTabItem(tab, tabtext, conv, ic, opts), m_parent(tab)
{
    m_mainframe = new FXVerticalFrame(m_parent, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_splitter = new FXSplitter(m_mainframe, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_REVERSED|SPLITTER_TRACKING);

    m_textframe = new FXVerticalFrame(m_splitter, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_info = new FXLabel(m_textframe, dxutils::fromStdString(m_conv->getName())+" ("+dxutils::fromStdString(m_conv->getAccount().getUsername())+")", dxutils::getProtocolIcon(dxutils::fromStdString(m_conv->getAccount().getProtocol().getPurpleID())), ICON_BEFORE_TEXT);
    m_topicline = new dxTextField(m_textframe, 50, this, ChatTabItem_TOPIC, FRAME_SUNKEN|TEXTFIELD_ENTER_ONLY|JUSTIFY_LEFT|LAYOUT_FILL_X);
    m_topicline->setLinkColor(FXRGB(142,196,210));
    m_topicline->setUseLink(TRUE);
    m_topicline->setTopicline(TRUE);
    m_topicline->setFont(dxim::instance()->getImFont());
    if(!m_conv->hasTopic()) m_topicline->hide();
    m_text->reparent(m_textframe);
    m_text->setSelTextColor(getApp()->getSelforeColor());
    m_text->setSelBackColor(getApp()->getSelbackColor());
    m_text->setFont(dxim::instance()->getImFont());

    m_usersframe = new FXVerticalFrame(m_splitter, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH);
    m_users = new dxChatList(m_usersframe, this, ChatTabItem_USERS, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_users->setSortFunc(FXList::ascendingCase);
    if(DXPREFERENCES.m_sameList)
        m_users->setFont(dxim::instance()->getImFont());
    //m_users->setScrollStyle(HSCROLLING_OFF);
    if(!DXPREFERENCES.m_chatRosterShown)
    {
        m_usersframe->hide();
        m_users->hide();
        m_splitter->setSplit(1,0);
    }

    m_commandline = new FXTextField(m_mainframe, 25, this, ChatTabItem_COMMANDLINE, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_BOTTOM, 0, 0, 0, 0, 1, 1, 1, 1);
    if(DXPREFERENCES.m_sameCmd)
        m_commandline->setFont(dxim::instance()->getImFont());

    m_nick = dxutils::fromStdString(m_conv->getMyNick());
    m_currentPosition = 0;
    m_historyMax = 25;
    m_itemOnRight = NULL;

#ifndef _WIN32
    m_commands.append("exec");
#endif
    m_commands.append("help");
    m_commands.append("say");
    m_commands.append("ignored");
    m_commands.append("log");
    std::vector<std::string> v = m_conv->getCommandsList();
    FOREACH(std::vector<std::string>, v, it)
        m_commands.append(dxutils::fromStdString(*it));
    if(!dxutils::containsStringArray(m_commands, "me"))
        m_commands.append("me");
    dxutils::sortStringArray(m_commands);

    if(m_conv->isValid()) m_logThread = new LogThread(this, getApp(), "");
}

ChatTabItem::~ChatTabItem()
{}

void ChatTabItem::appendNewMyMsg(NewMsg *msg)
{
    appendMyMsg(msg->msg, FXSystem::time("[%H:%M:%S] ", msg->time), msg->action);
}

void ChatTabItem::appendNewMsg(NewMsg *msg)
{
    FXString time = msg->history?FXSystem::time("[%x %H:%M:%S] ", msg->time):FXSystem::time("[%H:%M:%S] ", msg->time);
    if(msg->history)
    {
        appendHistoryMsg(msg->sender, msg->msg, time);
        return;
    }
    if(msg->sender == dxutils::fromStdString(m_conv->getMyNick()))
    {
        if(msg->action) appendStyledText(msg->sender+" "+msg->msg+"\n", 3, time);
        appendMyMsg(msg->msg, time);
        return;
    }
    if(msg->sender.empty()) msg->sender = m_info->getText();
    if(msg->highlight)
    {
        appendStyledText(msg->sender+(msg->action?" ":DXPREFERENCES.m_nickCompletionChar+" ")+msg->msg+"\n", 2, time);
        m_parent->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, ChatTabItem_HIGHLIGHT), NULL);
    }
    else
    {
        if(msg->action) appendStyledText(msg->sender+" "+msg->msg+"\n", 3, time);
        else appendNickText(msg->sender, msg->msg, DXPREFERENCES.m_coloredNick?getNickColor(msg->sender):6, time);
    }
    if(!msg->history && DXPREFERENCES.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
    {
        if(msg->highlight)
        {
            this->setTextColor(DXPREFERENCES.m_highlightColor);
            setIcon(ICO_CHATNEWMSG);
        }
        else this->setTextColor(DXPREFERENCES.m_unreadColor);
    }
}

void ChatTabItem::appendNewSystemMsg(NewSystemMsg* msg)
{
    appendStyledText(msg->msg, 5, FXSystem::time("[%H:%M:%S] ", msg->time));
}

void ChatTabItem::appendNewErrorMsg(NewSystemMsg *msg)
{
    appendStyledText(msg->msg, 4, FXSystem::time("[%H:%M:%S] ", msg->time));
}

void ChatTabItem::chatTopic(ChatTopic* topic)
{
    m_topicline->setText(topic->topic);
    m_topicline->setCursorPos(0);
    m_topicline->makePositionVisible(0);
    makeLastRowVisible();
}

void ChatTabItem::processChatBuddies(std::vector<PURPLE::ChatBuddy> buddies)
{
    FXString name;
    FXint i;
    FOREACH(std::vector<PURPLE::ChatBuddy>, buddies, b)
    {
        name = dxutils::fromStdString((*b).getName());
        i = m_users->findItem(name);
        if(i == -1)
        {
            dxChatItem *item = new dxChatItem(name, dxutils::fromStdString((*b).getRealName()), (*b).getChatRole(), dxutils::getChatRoleIcon((*b).getChatRole()));
            m_users->appendItem(item);
        }
    }
    m_users->sortItems();
}

void ChatTabItem::processChatBuddy(ChatBuddyEvent* event)
{
    switch(event->action) {
        case CBJOINED:
        {
            if(!event->buddy.isValid()) return;
            FXString name = dxutils::fromStdString(event->buddy.getName());
            FXint i = m_users->findItem(name);
            if(i == -1)
            {
                dxChatItem *item = new dxChatItem(name, dxutils::fromStdString(event->buddy.getRealName()), event->buddy.getChatRole(), dxutils::getChatRoleIcon(event->buddy.getChatRole()));
                m_users->appendItem(item);
            }
            break;
        }
        case CBUPDATED:
        {
            if(!m_conv->hasChatBuddy(dxutils::toStdString(event->who))) return;
            FXint i = m_users->findItem(event->who);
            if(i != -1)
            {
                PURPLE::ChatBuddy buddy = m_conv->findChatBuddy(dxutils::toStdString(event->who));
                if(buddy.isValid())
                {
                    dxChatItem *item = (dxChatItem*)m_users->getItem(i);
                    item->setRealName(dxutils::fromStdString(buddy.getRealName()));
                    item->setRole(buddy.getChatRole());
                    m_users->setItemIcon(i, dxutils::getChatRoleIcon(buddy.getChatRole()));
                }
            }
            break;
        }
        case CBRENAMED:
        {
            if(!event->buddy.isValid()) return;
            FXint i = m_users->findItem(event->who);
            if(i != -1)
            {
                m_users->setItemText(i, dxutils::fromStdString(event->buddy.getName()));
            }
            break;
        }
        case CBLEFT:
        {
            removeNick(event->who);
            break;
        }
    }
    m_users->sortItems();
}

//handle commadline enter
long ChatTabItem::onCommandline(FXObject*, FXSelector, void*)
{
    if(!m_conv->getAccount().isConnected())
        return 1;
    FXString command = m_commandline->getText();
    if(command.empty())
        return 1;
    m_commandsHistory.append(command);
    if (m_commandsHistory.no() > m_historyMax)
        m_commandsHistory.erase(0);
    m_currentPosition = m_commandsHistory.no()-1;
    m_commandline->setText("");
    processCommand(command);
    return 1;
}

//handle keypress in commandline
long ChatTabItem::onKeyPress(FXObject *, FXSelector, void *ptr)
{
    if(m_commandline->hasFocus())
    {
        FXString line = m_commandline->getText();
        FXString key = dxutils::getKeybinding((FXEvent*)ptr);
        if(comparecase(key, "Tab")==0) //tab completion
        {
            if(comparecase(line.before(' '), "/help") == 0 && !line.after(' ').empty())
            {
                completeHelpCommand();
                return 1;
            }
            if(line[0] == '/' && line.after(' ').empty())
            {
                for(FXint i = 0; i < m_commands.no(); i++)
                {
                    if(comparecase(line.after('/').before(' '), m_commands[i]) == 0)
                    {
                        if((i+1) < m_commands.no()) m_commandline->setText("/"+m_commands[++i]+" ");
                        else m_commandline->setText("/"+m_commands[0]+" ");
                        break;
                    }
                    else if(comparecase(line.after('/'), m_commands[i].left(line.after('/').length())) == 0)
                    {
                        m_commandline->setText("/"+m_commands[i]+" ");
                        break;
                    }
                }
                return 1;
            }
            if(line[0] != '/' && line.after(' ').empty())
            {
                if(line.empty())
                {
                    m_commandline->setText(getNick(0)+DXPREFERENCES.m_nickCompletionChar+" ");
                    return 1;
                }
                for(FXint j = 0; j < m_users->getNumItems(); j++)
                {
                    if(comparecase(line, getNick(j).left(line.length())) == 0)
                    {
                        m_commandline->setText(getNick(j)+DXPREFERENCES.m_nickCompletionChar+" ");
                        break;
                    }
                    else if(comparecase(line.section(DXPREFERENCES.m_nickCompletionChar, 0, 1), getNick(j)) == 0)
                    {
                        if((j+1) < m_users->getNumItems()) m_commandline->setText(getNick(++j)+DXPREFERENCES.m_nickCompletionChar+" ");
                        else m_commandline->setText(getNick(0)+DXPREFERENCES.m_nickCompletionChar+" ");
                        break;
                    }
                }
                return 1;
            }
            if(line.find(' ') != -1)
            {
                FXint curpos;
                line[m_commandline->getCursorPos()] == ' ' ? curpos = m_commandline->getCursorPos()-1 : curpos = m_commandline->getCursorPos();
                FXint pos = line.rfind(' ', curpos)+1;
                FXint n = line.find(' ', curpos)>0 ? line.find(' ', curpos)-pos : line.length()-pos;
                FXString toCompletion = line.mid(pos, n);
                for(FXint j = 0; j < m_users->getNumItems(); j++)
                {
                    if(comparecase(toCompletion, getNick(j)) == 0)
                    {
                        if((j+1) < m_users->getNumItems())
                        {
                            m_commandline->setText(line.replace(pos, n, getNick(j+1)));
                            m_commandline->setCursorPos(pos+getNick(j+1).length());
                        }
                        else
                        {
                            m_commandline->setText(line.replace(pos, n, getNick(0)));
                            m_commandline->setCursorPos(pos+getNick(0).length());
                        }
                        break;
                    }
                    else if(comparecase(toCompletion, getNick(j).left(toCompletion.length())) == 0)
                    {
                        m_commandline->setText(line.replace(pos, n, getNick(j)));
                        m_commandline->setCursorPos(pos+getNick(j).length());
                        break;
                    }
                }
                return 1;
            }
            return 1;
        }
        else if(comparecase(key, "Up")==0) //append history command backward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition > 0 && !line.empty())
                    --m_currentPosition;
                m_commandline->setText(m_commandsHistory[m_currentPosition]);
            }
            return 1;
        }
        else if(comparecase(key, "Down")==0) //append history command forward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition < m_commandsHistory.no()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_commandsHistory[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
            return 1;
        }
    }
    return 0;
}

//handle enter in topic line
long ChatTabItem::onTopicline(FXObject*, FXSelector, void*)
{
    m_conv->setTopic(m_topicline->getText().text());
    return 1;
}

//create geom, needed for dynamically create tab
void ChatTabItem::createGeom()
{
    m_mainframe->create();
    m_usersWidth = FXMIN(140, m_parent->getWidth()/4);
    m_splitter->setSplit(1, m_usersWidth);
    m_commandline->setFocus();
}

void ChatTabItem::hideUsers()
{
    m_usersWidth = m_splitter->getSplit(1);
    m_usersframe->hide();
    m_users->hide();
    m_splitter->setSplit(1,0);
}

void ChatTabItem::showUsers()
{
    m_splitter->setSplit(1, m_usersWidth);
    m_usersframe->show();
    m_users->show();
}

//usefull for set tab current
void ChatTabItem::setCommandFocus()
{
    m_commandline->setFocus();
}

void ChatTabItem::appendText(FXString msg, FXbool highlight, FXbool log)
{
    FXString time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_text->appendText(time);
    appendLinkText(dxutils::parseHtml(msg), highlight?2:0);
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logLine(dxutils::toStdString(time), dxutils::toStdString(msg));
    }
}

//append text with bold or colored nick
void ChatTabItem::appendNickText(const FXString &nick, const FXString &msg, FXint style, FXString time, FXbool log)
{
    if(time.empty()) time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_text->appendText(time);
    m_text->appendStyledText(nick+DXPREFERENCES.m_nickCompletionChar+" ", style);
    appendLinkText(dxutils::parseHtml(msg));
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logNickLine(dxutils::toStdString(time), dxutils::toStdString(nick), dxutils::toStdString(msg));
    }
}

//append my text
void ChatTabItem::appendMyMsg(const FXString &msg, FXString time, FXbool action)
{
    if(time.empty()) time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_text->appendText(time);
    if(action)
    {
        m_text->appendStyledText(dxutils::fromStdString(m_conv->getMyNick())+" "+dxutils::parseHtml(msg)+"\n", 3);
        if(DXPREFERENCES.m_logging)
        {
            startLogging();
            m_logger.logStyledLine(dxutils::toStdString(time), "action", dxutils::toStdString(dxutils::fromStdString(m_conv->getMyNick())+" "+msg));
        }
    }
    else
    {
        m_text->appendStyledText(FXStringFormat("%s%s ", dxutils::fromStdString(m_conv->getMyNick()).text(), DXPREFERENCES.m_nickCompletionChar.text()), DXPREFERENCES.m_coloredNick?getNickColor(dxutils::fromStdString(m_conv->getMyNick())):6);
        appendLinkText(dxutils::parseHtml(msg), 7);
        if(DXPREFERENCES.m_logging)
        {
            startLogging();
            m_logger.logMyMsgLine(dxutils::toStdString(time), m_conv->getMyNick(), dxutils::toStdString(msg));
        }
    }
}

//append styled text
void ChatTabItem::appendStyledText(const FXString& msg, FXint style, FXString time, FXbool log)
{
    if(time.empty()) time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_text->appendText(time);
    appendLinkText(dxutils::parseHtml(msg), style);
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logStyledLine(dxutils::toStdString(time), styleToString(style), dxutils::toStdString(msg));
    }
}

//get nick in users list
FXString ChatTabItem::getNick(FXint index)
{
    if(m_users->getNumItems()) return m_users->getItemText(index);
    return "";
}

//set text forecolor
void ChatTabItem::setTextForeColor(FXColor clr)
{
    m_textStyleList[5].normalForeColor = clr;
    m_text->setTextColor(clr);
    m_commandline->setTextColor(clr);
    m_commandline->setCursorColor(clr);
    m_topicline->setTextColor(clr);
    m_topicline->setCursorColor(clr);
    m_users->setTextColor(clr);
}

//set text backcolor
void ChatTabItem::setTextBackColor(FXColor clr)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        m_textStyleList[i].normalBackColor = clr;
    }
    m_text->setBackColor(clr);
    m_commandline->setBackColor(clr);
    m_topicline->setBackColor(clr);
    m_users->setBackColor(clr);
}

//set color for link text
void ChatTabItem::setLinkColor(FXColor clr)
{
    m_textStyleList[0].normalForeColor = clr;
    m_topicline->setLinkColor(clr);
}

//set font
void ChatTabItem::updateImFont()
{
    m_text->recalc();
    m_text->setFont(dxim::instance()->getImFont());
    m_topicline->recalc();
    m_topicline->setFont(dxim::instance()->getImFont());
    m_commandline->recalc();
    if(DXPREFERENCES.m_sameCmd)
        m_commandline->setFont(dxim::instance()->getImFont());
    else
        m_commandline->setFont(getApp()->getNormalFont());
    m_users->recalc();
    if(DXPREFERENCES.m_sameList)
        m_users->setFont(dxim::instance()->getImFont());
    else
        m_users->setFont(getApp()->getNormalFont());
}

//start logging
void ChatTabItem::startLogging()
{
    if(m_logger.isReady()) return;
    FXString folder = dxutils::fromStdString(m_conv->getName());
    FXString username = dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/');
    m_logger.setPath(dxutils::toStdString(DXPREFERENCES.m_logPath+PATHSEPSTRING+username+PATHSEPSTRING+folder));
}

void ChatTabItem::stopLogging()
{
    if(!DXPREFERENCES.m_logging) m_logger.stopLogging();
}

void ChatTabItem::processCommand(FXString commandtext)
{
    if(commandtext[0]=='/')
    {
        FXString command = commandtext.after('/').before(' ');
        if(comparecase(command, "exec")==0)
        {
            FXString arg = commandtext.after(' ');
            if(arg.empty()) showInternalHelp("exec");
            else
            {
                if(!m_pipe) m_pipe = new dxPipe(getApp(), this);
                m_pipeStrings.clear();
                if(arg.before(' ').contains("-o"))
                {
                    m_sendPipe = TRUE;
                    appendStyledText(FXStringFormat(_("Command %s started"), arg.after(' ').text()), 2, "", false);
                    m_pipe->execCmd(arg.after(' '));
                }
                else if(arg.before(' ').contains("-c"))
                {
                    m_sendPipe = FALSE;
                    appendStyledText(FXStringFormat(_("Command %s stopped"), m_pipe->getCommand().text()), 2, "", false);
                    m_pipeStrings.clear();
                    m_pipe->stopCmd();
                }
                else
                {
                    m_sendPipe = FALSE;
                    appendStyledText(FXStringFormat(_("Command %s started"), arg.text()), 2, "", false);
                    m_pipe->execCmd(arg);
                }
            }
            return;
        }
        else if(comparecase(command, "help")==0)
        {
            FXString arg = commandtext.after(' ');
            if(arg.empty())
            {
                FXString commands = "";
                for(int i=0; i<m_commands.no(); i++)
                {
                    if(i+1!=m_commands.no())
                        commands.append(m_commands[i]+", ");
                    else
                        commands.append(m_commands[i]+".");
                }
                appendStyledText(FXStringFormat(_("Available commands: %s"), commands.text()), 2, "", false);
                return;
            }
            if(comparecase(arg,"exec")==0 || comparecase(arg,"help")==0 || comparecase(arg,"me")==0 || comparecase(arg,"say")==0
               || comparecase(arg,"ignored")==0 || comparecase(arg,"log")==0)
            {
                showInternalHelp(arg);
                return;
            }
            std::string response = m_conv->getCommandHelp(dxutils::toStdString(arg));
            if(response.empty())
                appendStyledText(FXStringFormat(_("Command '%s' doesn't exist or hasn't help"), arg.text()), 4, "", false);
            else
            {
                appendStyledText(FXStringFormat(_("Help for: %s"), response.c_str()), 2, "", false);
            }
        }
        else if(comparecase(command,"me")==0)
        {
            if(!commandtext.after(' ').empty()) m_conv->sendActionMessage(commandtext.after(' ').substitute("<","&lt;").substitute(">","&gt;").text());
        }
        else if(comparecase(command,"say")==0)
        {
            if(!commandtext.after(' ').empty()) m_conv->sendMessage(commandtext.after(' ').substitute("<","&lt;").substitute(">","&gt;").text());
        }
        else if(comparecase(command,"ignored")==0)
        {
            FXString ignored = "";
            std::vector<std::string> v = m_conv->getIgnored();
            for(size_t i=0; i<v.size(); i++)
            {
                if(i+1!=v.size())
                    ignored.append(dxutils::fromStdString(v[i])+", ");
                else
                    ignored.append(dxutils::fromStdString(v[i])+".");
            }
            if(v.size())
                appendStyledText(FXStringFormat(_("Ignored buddies: %s"), ignored.text()), 2, "", false);
            else
                appendStyledText(_("Nobody ignored"), 2, "", false);
        }
        else if(comparecase(command,"log")==0)
        {
            FXString logcommand = commandtext.after(' ').before(' ');
            FXString logfile = commandtext.after(' ').after(' ');
            if(comparecase(logcommand,"list")==0)
            {
                listLogFiles();
            }
            else if(comparecase(logcommand, "show")==0)
            {
                if(logfile.empty())
                    logfile = FXSystem::time("%Y-%m-%d", FXSystem::now());
                FXString logpath = DXPREFERENCES.m_logPath+PATHSEPSTRING+dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/')
                                    +PATHSEPSTRING+dxutils::fromStdString(m_conv->getName())+PATHSEPSTRING+logfile;
                if(!FXStat::exists(logpath))
                {
                    appendStyledText(FXStringFormat(_("Log file %s doesn't exist"), logfile.text()), 4, "", false);
                    return;
                }
                m_parent->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, dxTabItem_SHOWLOG), &logpath);
            }
            else
            {
                appendStyledText(FXStringFormat(_("'%s' isn't <list|show>"), logcommand.text()), 4, "", false);
            }
        }
        else
        {
            int ret = m_conv->sendCommand(commandtext.after('/').text());
            processCommandReturn(ret, command.text());
        }
    }
    else
    {
        m_conv->sendMessage(commandtext.substitute("<","&lt;").substitute(">","&gt;").text());
    }
}

//right click on users
long ChatTabItem::onRightRelease(FXObject*, FXSelector, void *ptr)
{
    FXEvent* event = (FXEvent*)ptr;
    if(event->moved) return 1;
    FXint index = m_users->getItemAt(event->win_x,event->win_y);
    if(index >= 0)
    {
        m_itemOnRight = (dxChatItem*)m_users->getItem(index);
        if(dxutils::fromStdString(m_conv->getMyNick()) != m_itemOnRight->getText())
        {
            PURPLE::ChatBuddy buddy = m_conv->findChatBuddy(dxutils::toStdString(m_itemOnRight->getText()));
            if(!buddy.isValid())
            {
                removeNick(m_itemOnRight->getText());
                return 1;
            }
            FXMenuPane popup(this);
            new FXMenuCommand(&popup, _("IM"), NULL, this, ChatTabItem_POPUPIM);
            new FXMenuCommand(&popup, _("Info"), NULL, this, ChatTabItem_POPUPINFO);
            new FXMenuSeparator(&popup);
            new FXMenuCommand(&popup, m_conv->isChatBuddyIgnored(dxutils::toStdString(m_itemOnRight->getText()))?_("Unignore"):_("Ignore"), NULL, this, ChatTabItem_POPUPIGNORE);
            popup.create();
            popup.popup(NULL,event->root_x,event->root_y);
            getApp()->runModalWhileShown(&popup);
        }
    }
    return 1;
}

//start im for righ-click popup
long ChatTabItem::onPopupIM(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight)
    {
        PURPLE::ChatBuddy buddy = m_conv->findChatBuddy(dxutils::toStdString(m_itemOnRight->getText()));
        if(!buddy.isValid())
        {
            removeNick(m_itemOnRight->getText());
            return 1;
        }
        m_conv->createIM(buddy);
    }
    return 1;
}

//retrieve info for righ-click popup
long ChatTabItem::onPopupInfo(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight)
    {
        PURPLE::ChatBuddy buddy = m_conv->findChatBuddy(dxutils::toStdString(m_itemOnRight->getText()));
        if(!buddy.isValid())
        {
            removeNick(m_itemOnRight->getText());
            return 1;
        }
        m_conv->retrieveChatBuddyInfo(buddy.getName());
    }
    return 1;
}

//ignore || unignore for righ-click popup
long ChatTabItem::onPopupIgnore(FXObject*, FXSelector, void*)
{
    if(m_itemOnRight)
    {
        PURPLE::ChatBuddy buddy = m_conv->findChatBuddy(dxutils::toStdString(m_itemOnRight->getText()));
        if(!buddy.isValid())
        {
            removeNick(m_itemOnRight->getText());
            return 1;
        }
        if(m_conv->isChatBuddyIgnored(buddy.getName()))
            m_conv->unignore(buddy.getName());
        else
            m_conv->ignore(buddy.getName());
    }
    return 1;
}

//remove user from CHAT
void ChatTabItem::removeNick(const FXString& nick)
{
    FXint index = m_users->findItem(nick);
    if(index != -1)
    {
        m_users->removeItem(index);
        m_users->sortItems();
    }
}

//process return m_conv->sendCommand
void ChatTabItem::processCommandReturn(int ret, const FXchar *command)
{
    switch(ret) {
        case PURPLE_CMD_STATUS_OK:
            break;
        case PURPLE_CMD_STATUS_NOT_FOUND:
            appendStyledText(FXStringFormat(_("Unknown command '%s'."), command), 4);
            break;
        case PURPLE_CMD_STATUS_WRONG_ARGS:
            appendStyledText(FXStringFormat(_("Syntax Error:  You typed the wrong number of arguments "
                                "to that command '%s'."), command), 4);
            break;
        case PURPLE_CMD_STATUS_FAILED:
            appendStyledText(FXStringFormat(_("Your command '%s' failed for an unknown reason."), command), 4);
            break;
        case PURPLE_CMD_STATUS_WRONG_TYPE:
            if(m_conv->getType() == PURPLE_CONV_TYPE_IM)
                appendStyledText(FXStringFormat(_("That command '%s' only works in chats, not IMs."), command), 4);
            else
                appendStyledText(FXStringFormat(_("That command '%s' only works in IMs, not chats."), command), 4);
            break;
        case PURPLE_CMD_STATUS_WRONG_PRPL:
            appendStyledText(FXStringFormat(_("That command '%s' doesn't work on this protocol."), command), 4);
            break;
    }
}

void ChatTabItem::showInternalHelp(const FXString &command)
{
    if(comparecase(command, "exec")==0)
    {
        appendStyledText(_("Help for: exec [-o|-c] <command>, executes command, -o sends output to chat, -c closes running command"), 2, "", false);
    }
    else if(comparecase(command, "help")==0)
    {
        appendStyledText(_("Help for: help [command]: lists available commands or show help for command"), 2, "", false);
    }
    else if(comparecase(command, "me")==0)
    {
        appendStyledText(_("Help for: me <text>: sends text in 3rd person"), 2, "", false);
    }
    else if(comparecase(command, "say")==0)
    {
        appendStyledText(_("Help for: say <text>: sends text"), 2, "", false);
    }
    else if(comparecase(command, "ignored")==0)
    {
        appendStyledText(_("Help for: ignored: show list ignored buddies"), 2, "", false);
    }
    else if(comparecase(command, "log")==0)
    {
        appendStyledText(_("Help for: log"), 2, "", false);
        appendStyledText(_("LOG list, shows list of log files for current im/chat"), 2, "", false);
        appendStyledText(_("LOG show [date], shows current log file for current im/chat, if date is empty, else file for given date"), 2, "", false);
        appendStyledText(_("date example: 2011-01-31"), 2, "", false);
    }
}

void ChatTabItem::completeHelpCommand()
{
    FXString line = m_commandline->getText();
    for(FXint i = 0; i < m_commands.no(); i++)
    {
        if(comparecase(line.after(' '), m_commands[i]) == 0)
        {
            if((i+1) < m_commands.no()) m_commandline->setText("/help "+m_commands[++i]);
            else m_commandline->setText("/help "+m_commands[0]);
            break;
        }
        else if(comparecase(line.after(' '), m_commands[i].left(line.after(' ').length())) == 0)
        {
            m_commandline->setText("/help "+m_commands[i]);
            break;
        }
    }
}

void ChatTabItem::clearChatBuddyIcons()
{
    for(FXint i = 0; i < m_users->getNumItems(); i++)
        m_users->getItem(i)->setIcon(NULL);
}

void ChatTabItem::recreateChatBuddyIcons()
{
    for(FXint i = 0; i < m_users->getNumItems(); i++)
    {
        m_users->setItemIcon(i, dxutils::getChatRoleIcon(static_cast<dxChatItem*>(m_users->getItem(i))->getRole()));
    }
}

//List log files for im
void ChatTabItem::listLogFiles()
{
    if(m_logThread->running())
    {
        appendStyledText(_("Thread running, try again after some seconds"), 2, "", false);
        return;
    }
    m_logThread->setDirectory(DXPREFERENCES.m_logPath+PATHSEPSTRING+dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/')
            +PATHSEPSTRING+dxutils::fromStdString(m_conv->getName()));
    m_logThread->start();
}

//handle LogThread_LIST
long ChatTabItem::onListLogThread(FXObject*, FXSelector, void*)
{
    dxStringArray files = m_logThread->getResult();
    if(!files.no())
    {
        appendStyledText(FXStringFormat(_("No log files for %s-%s."), dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/').text(),
                dxutils::fromStdString(m_conv->getName()).text()), 4, "", false);
        return 1;
    }
    appendStyledText(_("Log files:"), 6, "", false);
    FXString lastYear = dxutils::getParam(files[0], 1, FALSE, '-');
    FXint lastMonth = FXIntVal(dxutils::getParam(files[0], 2, FALSE, '-'));
    appendStyledText(FXStringFormat("%s:", lastYear.text()), 6, "", false);
    appendStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 6, "", false);
    FXString list = "";
    for(FXint i=0; i<files.no(); i++)
    {
        if(lastYear!=dxutils::getParam(files[i], 1, FALSE, '-'))
        {
            lastYear = dxutils::getParam(files[i], 1, FALSE, '-');
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendText(list.rbefore(','), false, false);
            appendStyledText(FXStringFormat("%s:", lastYear.text()), 6, "", false);
            appendStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 6, "", false);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else if(lastMonth!=FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-')))
        {
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendText(list.rbefore(','), false, false);
            appendStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 6, "", false);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
    }
    if(!list.empty()) appendText(list.rbefore(','), false, false);
    return 1;
}
