/*
 *      chattabitem.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CHATTABITEM_H
#define	CHATTABITEM_H

#include "dxtabitem.h"
#include "logthread.h"
#include "../logger/logger.h"

class ChatTabItem;

class dxChatList : public FXList
{
    FXDECLARE(dxChatList)
    friend class ChatTabItem;
public:
    dxChatList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=LIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
    long onQueryTip(FXObject*,FXSelector,void*);
protected:
    dxChatList() {}
private:
    dxChatList(const dxChatList&);
    dxChatList& operator=(const dxChatList&);
};

class dxChatItem : public FXListItem
{
    FXDECLARE(dxChatItem)
    friend class dxChatList;
    friend class ChatTabItem;
public:
    dxChatItem(const FXString& text, const FXString& realName, PurpleConvChatBuddyFlags role, FXIcon* ic = NULL, void* ptr = NULL)
        : FXListItem(text, ic, ptr), m_realName(realName), m_role(role) {}

    void setRealName(FXString realName) { m_realName = realName; }
    FXString getRealName() const { return m_realName; }
    void setRole(PurpleConvChatBuddyFlags role) { m_role = role; }
    PurpleConvChatBuddyFlags getRole() const { return m_role; }
protected:
    dxChatItem() {}
private:
    dxChatItem(const dxChatItem&);
    dxChatItem& operator=(const dxChatItem&);

    FXString m_realName;
    PurpleConvChatBuddyFlags m_role;
};

class ChatTabItem : public dxTabItem
{
    FXDECLARE(ChatTabItem)
public:
    ChatTabItem(dxTabBook *tab, const FXString &tabtext, PURPLE::Conversation *conv, FXIcon *ic=0, FXuint opts=TAB_TOP_NORMAL);
    virtual ~ChatTabItem();

    long onCommandline(FXObject *, FXSelector, void*);
    long onKeyPress(FXObject *, FXSelector, void*);
    long onTopicline(FXObject *, FXSelector, void*);
    long onRightRelease(FXObject*, FXSelector, void*);
    long onPopupIM(FXObject *, FXSelector, void*);
    long onPopupInfo(FXObject *, FXSelector, void*);
    long onPopupIgnore(FXObject *, FXSelector, void*);
    long onListLogThread(FXObject*, FXSelector, void*);

    void createGeom();
    void hideUsers();
    void showUsers();
    void setCommandFocus();
    void updateImFont();
    void appendNewMyMsg(NewMsg *msg);
    void appendNewMsg(NewMsg *msg);
    void appendNewSystemMsg(NewSystemMsg *msg);
    void appendNewErrorMsg(NewSystemMsg *msg);
    void chatTopic(ChatTopic *topic);
    void processChatBuddies(std::vector<PURPLE::ChatBuddy> buddies);
    void processChatBuddy(ChatBuddyEvent *event);
    void clearChatBuddyIcons();
    void recreateChatBuddyIcons();
private:
    ChatTabItem() {}

    dxTabBook *m_parent;
    FXVerticalFrame *m_textframe, *m_usersframe, *m_mainframe;
    FXLabel *m_info;
    FXSplitter *m_splitter;
    dxChatList *m_users;
    FXTextField *m_commandline;
    dxTextField *m_topicline;
    FXString m_nick; //my nick in chat
    Logger m_logger;
    FXint m_currentPosition, m_historyMax;
    dxStringArray m_commandsHistory;
    dxChatItem *m_itemOnRight;
    dxStringArray m_commands;
    FXint m_usersWidth;
    LogThread *m_logThread;

    void appendText(FXString msg, FXbool highlight, FXbool logLine=TRUE);
    void appendNickText(const FXString &nick, const FXString &msg, FXint style, FXString time="", FXbool log=TRUE);
    void appendStyledText(const FXString &msg, FXint style=0, FXString time="", FXbool log=TRUE);
    void appendMyMsg(const FXString &msg, FXString time="", FXbool action=FALSE);
    FXString getNick(FXint index);
    void setTextForeColor(FXColor);
    void setTextBackColor(FXColor);
    void setLinkColor(FXColor);
    void startLogging();
    void stopLogging();
    void processCommand(FXString commandtext);
    void removeNick(const FXString& nick);
    void processCommandReturn(int ret, const FXchar *command);
    void showInternalHelp(const FXString &command);
    void completeHelpCommand();
    void listLogFiles();
};

#endif	/* _ChatTabItem_H */

