/*
 *      dxtabitem.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef DXTABITEM_H
#define DXTABITEM_H

#include "defs.h"
#include "dxtabbook.h"
#include "dxtextfield.h"
#include "fxext.h"
#include "dxpipe.h"
#include "callback/callback.h"
#ifndef DXTEXT_H
#include "dxtext.h"
#endif

class dxTabItem: public dxEXTabItem
{
    FXDECLARE(dxTabItem)
public:
    dxTabItem(dxTabBook *tab, const FXString &tabtext, PURPLE::Conversation *conv, FXIcon *icon=0, FXuint opts=TAB_TOP_NORMAL, bool enableClear=true);
    virtual ~dxTabItem();

    virtual void createGeom() {}
    virtual void clearChat();
    virtual void hideUsers() {}
    virtual void showUsers() {}
    virtual void reparentTab() {}
    void preferencesUpdated();
    virtual void updateImFont();
    virtual void setCommandFocus() {}
    void setSmileys(FXbool smiley, dxSmileyArray nsmileys);
    virtual void showSpellComboUpdated() {}
    void removeSmileys();
    void makeLastRowVisible();
    virtual FXString getSpellLang() {return ""; }
    PURPLE::Conversation *getConversation() { return m_conv; }
    virtual void appendNewMyMsg(NewMsg *msg) {}
    virtual void messageRecived(NewMsg *msg) {}
    virtual void appendNewMsg(NewMsg *msg) {}
    virtual void appendNewSystemMsg(NewSystemMsg *msg) {}
    virtual void appendNewErrorMsg(NewSystemMsg *msg) {}
    FXString getAccountID();
    FXString getAccountProtocol();
    bool getEnableClear() { return m_enableClear; }
    virtual void clearChatBuddyIcons() {}
    virtual void recreateChatBuddyIcons() {}
    void appendLinkText(const FXString &txt, FXint stylenum=0, FXTime time=0);

    long onTextLink(FXObject *, FXSelector, void *);
    long onTextRightMouse(FXObject *, FXSelector, void *);
    long onTextMiddleMouse(FXObject *, FXSelector, void *);
    long onPipe(FXObject *, FXSelector, void *);
    long onPipeTimeout(FXObject *, FXSelector, void *);
protected:
    dxTabItem(){}

    dxText *m_text;
    PURPLE::Conversation *m_conv;
    dxPipe *m_pipe;
    dxHiliteArray m_textStyleList;
    FXbool m_sendPipe;
    dxStringArray m_pipeStrings;

    virtual void appendText(FXString msg, FXbool highlight, FXbool logLine=TRUE) {}
    virtual void appendStyledText(FXString text, FXint style, FXbool highlight, FXbool logLine=TRUE) {}
    void appendHistoryMsg(const FXString &nick, const FXString &msg, const FXString &time);
    FXint hiliteStyleExist(FXColor foreColor, FXColor backColor, FXuint style);
    void createHiliteStyle(FXColor foreColor, FXColor backColor, FXuint style);
    virtual void setTextForeColor(FXColor);
    virtual void setTextBackColor(FXColor);
    virtual void setUserColor(FXColor);
    virtual void setActionsColor(FXColor);
    virtual void setErrorColor(FXColor);
    virtual void setHilightColor(FXColor);
    virtual void setLinkColor(FXColor);
    virtual void setMymsgColor(FXColor);
    virtual void stopLogging() {}
    FXint getNickColor(const FXString &nick);
    std::string styleToString(FXint style);
private:
    bool m_enableClear;
};

#endif // DXTABITEM_H
