/*
 *      imtabitem.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

/*
 * DON'T USE ImTabItem::appendLinkText
 * m_text->appendText and
 * m_text->appendStyledText DIRECTLY
 */

#include "imtabitem.h"
#include "icons.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"
#include "dxim.h"
#include <utilsmm.h>

FXDEFMAP(ImTabItem) ImTabItemMap[] = {
    FXMAPFUNC(SEL_COMMAND, ImTabItem_COMMANDLINE, ImTabItem::onCommandline),
    FXMAPFUNC(SEL_KEYPRESS, ImTabItem_COMMANDLINE, ImTabItem::onKeyPress),
    FXMAPFUNC(SEL_TIMEOUT, ImTabItem_COMPOSETIMEOUT, ImTabItem::onComposeTimeout),
    FXMAPFUNC(SEL_COMMAND, LogThread_LIST, ImTabItem::onListLogThread)
};

FXIMPLEMENT(ImTabItem, dxTabItem, ImTabItemMap, ARRAYNUMBER(ImTabItemMap))

ImTabItem::ImTabItem(dxTabBook *tab, const FXString &tabtext, PURPLE::Conversation *conv, FXIcon *ic, FXuint opts)
: dxTabItem(tab, tabtext, conv, ic, opts), m_parent(tab)
{
    m_mainframe = new FXVerticalFrame(m_parent, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_textframe = new FXVerticalFrame(m_mainframe, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXHorizontalFrame *hframe = new FXHorizontalFrame(m_textframe, LAYOUT_FILL_X);
    m_info = new FXLabel(hframe, dxutils::fromStdString(m_conv->getName())+" ("+dxutils::fromStdString(m_conv->getAccount().getUsername())+")");
    m_protocol = new FXLabel(hframe, "", dxutils::getProtocolIcon(dxutils::fromStdString(m_conv->getAccount().getProtocol().getPurpleID())), LAYOUT_RIGHT);
    m_text->reparent(m_textframe);
    m_text->setSelTextColor(getApp()->getSelforeColor());
    m_text->setSelBackColor(getApp()->getSelbackColor());
    m_text->setFont(dxim::instance()->getImFont());

    m_commandline = new FXTextField(m_mainframe, 25, this, ImTabItem_COMMANDLINE, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_BOTTOM, 0, 0, 0, 0, 1, 1, 1, 1);
    if(DXPREFERENCES.m_sameCmd)
        m_commandline->setFont(dxim::instance()->getImFont());
    else
        m_commandline->setFont(getApp()->getNormalFont());

    m_currentPosition = 0;
    m_historyMax = 25;
    m_lastKeyPress = 0;
    m_removeLastLine = false;
    m_imState = ACTIVE;

    getApp()->addTimeout(this, ImTabItem_COMPOSETIMEOUT, 2000);

#ifndef _WIN32
    m_commands.append("exec");
#endif
    m_commands.append("help");
    m_commands.append("say");
    m_commands.append("log");
    std::vector<std::string> v = m_conv->getCommandsList();
    FOREACH(std::vector<std::string>, v, it)
        m_commands.append(dxutils::fromStdString(*it));
    if(!dxutils::containsStringArray(m_commands, "me"))
        m_commands.append("me");
    dxutils::sortStringArray(m_commands);

    if(m_conv->isValid()) m_logThread = new LogThread(this, getApp(), "");

    if(DXPREFERENCES.m_logLoadCurrent) loadCurrentLog();
}

ImTabItem::~ImTabItem()
{
    getApp()->removeTimeout(this, ImTabItem_COMPOSETIMEOUT);
}

//handle commadline enter
long ImTabItem::onCommandline(FXObject*, FXSelector, void*)
{
    if(!m_conv->getAccount().isConnected())
        return 1;
    FXString command = m_commandline->getText();
    if(command.empty())
        return 1;
    m_commandsHistory.append(command);
    if (m_commandsHistory.no() > m_historyMax)
        m_commandsHistory.erase(0);
    m_currentPosition = m_commandsHistory.no()-1;
    m_commandline->setText("");
    processCommand(command);
    return 1;
}

void ImTabItem::appendNewMyMsg(NewMsg *msg)
{
    if(m_removeLastLine)
    {
        m_text->removeLastLine();
        m_messageIDs.erase(m_messageIDs.no()-1);
    }
    m_removeLastLine = false;
    appendMyMsg(msg->msg, FXSystem::time("[%H:%M:%S] ", msg->time), msg->action, msg->messageID);
}

void ImTabItem::messageRecived(NewMsg *msg)
{
    for(FXint i=0; i<m_messageIDs.no(); i++)
    {
        if(m_messageIDs[i] == msg->messageID)
        {
            m_text->messageRecived(i);
        }
    }
}

void ImTabItem::appendNewMsg(NewMsg *msg)
{
    setIcon(m_conv->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?ICO_IMOFF:ICO_IM);
    FXString time = msg->history?FXSystem::time("[%x %H:%M:%S] ", msg->time):FXSystem::time("[%H:%M:%S] ", msg->time);
    if(msg->history)
    {
        appendHistoryMsg(msg->sender, msg->msg, time);
        return;
    }
    if(m_removeLastLine)
    {
        m_text->removeLastLine();
        m_messageIDs.erase(m_messageIDs.no()-1);
    }
    m_removeLastLine = false;
    if(this->getText()!=msg->sender)
        this->setText(msg->sender);
    if(msg->highlight)
    {
        appendStyledText(msg->sender+(msg->action?" ":DXPREFERENCES.m_nickCompletionChar+" ")+msg->msg+"\n", 2, time);
    }
    else
    {
        if(msg->action) appendStyledText(msg->sender+" "+msg->msg+"\n", 3, time);
        else appendNickText(msg->sender, msg->msg, DXPREFERENCES.m_coloredNick?getNickColor(msg->sender):6, time);
    }
    if(DXPREFERENCES.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
    {
        if(msg->highlight)
        {
            this->setTextColor(DXPREFERENCES.m_highlightColor);
        }
        else this->setTextColor(DXPREFERENCES.m_unreadColor);
        setIcon(ICO_IMNEWMSG);
    }
    m_parent->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, ImTabItem_HIGHLIGHT), NULL);
}

void ImTabItem::appendNewSystemMsg(NewSystemMsg *msg)
{
    if(m_removeLastLine)
    {
        m_text->removeLastLine();
        m_messageIDs.erase(m_messageIDs.no()-1);
    }
    m_removeLastLine = false;
    appendStyledText(msg->msg, 5, FXSystem::time("[%H:%M:%S] ", msg->time));
}

void ImTabItem::appendNewErrorMsg(NewSystemMsg *msg)
{
    if(m_removeLastLine)
    {
        m_text->removeLastLine();
        m_messageIDs.erase(m_messageIDs.no()-1);
    }
    m_removeLastLine = false;
    appendStyledText(msg->msg, 4, FXSystem::time("[%H:%M:%S] ", msg->time));
}

void ImTabItem::buddyTyping(bool typing)
{
    if(m_removeLastLine)
    {
        m_text->removeLastLine();
        m_messageIDs.erase(m_messageIDs.no()-1);
    }
    m_removeLastLine = true;
    appendStyledText(dxutils::typingToString(this->getText(), typing), 5, "", FALSE);
    if(getIcon()!=ICO_IMNEWMSG)
    {
        if(typing) setIcon(ICO_IMCOMPOSE);
        else setIcon(ICO_IM);
    }
}

//handle keypress in commandline
long ImTabItem::onKeyPress(FXObject *, FXSelector, void *ptr)
{
    if(m_commandline->hasFocus())
    {
        m_lastKeyPress = FXSystem::now();
        FXString line = m_commandline->getText();
        FXString key = dxutils::getKeybinding((FXEvent*)ptr);
        if(comparecase(key, "Tab")==0) //tab completion
        {
            if(comparecase(line.before(' '), "/help") == 0 && !line.after(' ').empty())
            {
                completeHelpCommand();
                return 1;
            }
            if(line[0] == '/' && line.after(' ').empty())
            {
                for(FXint i = 0; i < m_commands.no(); i++)
                {
                    if(comparecase(line.after('/').before(' '), m_commands[i]) == 0)
                    {
                        if((i+1) < m_commands.no()) m_commandline->setText("/"+m_commands[++i]+" ");
                        else m_commandline->setText("/"+m_commands[0]+" ");
                        break;
                    }
                    else if(comparecase(line.after('/'), m_commands[i].left(line.after('/').length())) == 0)
                    {
                        m_commandline->setText("/"+m_commands[i]+" ");
                        break;
                    }
                }
            }
            return 1;
        }
        else if(comparecase(key, "Up")==0) //append history command backward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition > 0 && !line.empty())
                    --m_currentPosition;
                m_commandline->setText(m_commandsHistory[m_currentPosition]);
            }
            return 1;
        }
        else if(comparecase(key, "Down")==0) //append history command forward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition < m_commandsHistory.no()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_commandsHistory[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
            return 1;
        }
    }
    return 0;
}

//handle composing timeout
long ImTabItem::onComposeTimeout(FXObject*, FXSelector, void*)
{
    getApp()->addTimeout(this, ImTabItem_COMPOSETIMEOUT, 2000);
    if(!m_lastKeyPress) return 1;
    if(FXSystem::now()-m_lastKeyPress>5)
    {
        if(m_imState==STARTED)
        {
            m_imState = PAUSED;
            m_conv->sendTyping(false);
        }
        return 1;
    }
    else
    {
        if(m_imState==ACTIVE || m_imState==PAUSED)
        {
            m_imState = STARTED;
            m_conv->sendTyping(true);
        }
        return 1;
    }
    return 1;
}

//create geom, needed for dynamically create tab
void ImTabItem::createGeom()
{
    m_mainframe->create();
    m_commandline->setFocus();
}

void ImTabItem::clearChat()
{
    m_textStyleList.no(16);
    m_text->setHiliteStyles(m_textStyleList.data());
    m_text->clearText();
    m_messageIDs.clear();
}

//change info text icon
void ImTabItem::setInfolabelIcon(FXIcon* icon)
{
    if(m_info)
    {
        m_info->setIcon(icon);
    }
}

//usefull for set tab current
void ImTabItem::setCommandFocus()
{
    m_commandline->setFocus();
}

void ImTabItem::appendText(FXString msg, FXbool highlight, FXbool log)
{
    FXString time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_messageIDs.append("");
    m_text->appendText(time);
    appendLinkText(dxutils::parseHtml(msg), highlight?2:0);
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logLine(dxutils::toStdString(time), dxutils::toStdString(msg));
    }
}

//append text with bold or colored nick
void ImTabItem::appendNickText(const FXString &nick, const FXString &msg, FXint style, FXString time, FXbool log)
{
    if(time.empty()) time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_messageIDs.append("");
    m_text->appendText(time);
    m_text->appendStyledText(nick+DXPREFERENCES.m_nickCompletionChar+" ", style);
    appendLinkText(dxutils::parseHtml(msg));
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logNickLine(dxutils::toStdString(time), dxutils::toStdString(nick), dxutils::toStdString(msg));
    }
}

//append my text
void ImTabItem::appendMyMsg(const FXString &msg, FXString time, FXbool action, const FXString &messageID)
{
    if(time.empty()) time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    dxutils::debugLine(messageID);
    m_messageIDs.append(messageID);
    m_text->appendText(time);
    if(action)
    {
        m_text->appendStyledText(dxutils::fromStdString(m_conv->getMyNick())+" "+dxutils::parseHtml(msg)+"\n", 3);
        if(DXPREFERENCES.m_logging)
        {
            startLogging();
            m_logger.logStyledLine(dxutils::toStdString(time), "action", dxutils::toStdString(dxutils::fromStdString(m_conv->getMyNick())+" "+msg));
        }
    }
    else
    {
        m_text->appendStyledText(FXStringFormat("%s%s ", dxutils::fromStdString(m_conv->getMyNick()).text(), DXPREFERENCES.m_nickCompletionChar.text()), DXPREFERENCES.m_coloredNick?getNickColor(dxutils::fromStdString(m_conv->getMyNick())):6);
        appendLinkText(dxutils::parseHtml(msg), 7);
        if(DXPREFERENCES.m_logging)
        {
            startLogging();
            m_logger.logMyMsgLine(dxutils::toStdString(time), m_conv->getMyNick(), dxutils::toStdString(msg));
        }
    }
}

//append styled text
void ImTabItem::appendStyledText(const FXString& msg, FXint style, FXString time, FXbool log)
{
    if(time.empty()) time = FXSystem::time("[%H:%M:%S] ", FXSystem::now());
    m_messageIDs.append("");
    m_text->appendText(time);
    appendLinkText(dxutils::parseHtml(msg), style);
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logStyledLine(dxutils::toStdString(time), styleToString(style), dxutils::toStdString(msg));
    }
}

//set text forecolor
void ImTabItem::setTextForeColor(FXColor clr)
{
    m_textStyleList[5].normalForeColor = clr;
    m_text->setTextColor(clr);
    m_commandline->setTextColor(clr);
    m_commandline->setCursorColor(clr);
}

//set text backcolor
void ImTabItem::setTextBackColor(FXColor clr)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        m_textStyleList[i].normalBackColor = clr;
    }
    m_text->setBackColor(clr);
    m_commandline->setBackColor(clr);
}

//set font
void ImTabItem::updateImFont()
{
    m_text->recalc();
    m_text->setFont(dxim::instance()->getImFont());
    m_commandline->recalc();
    if(DXPREFERENCES.m_sameCmd)
        m_commandline->setFont(dxim::instance()->getImFont());
    else
        m_commandline->setFont(getApp()->getNormalFont());
}

//start logging
void ImTabItem::startLogging()
{
    if(m_logger.isReady()) return;
    FXString folder = dxutils::fromStdString(m_conv->getName()).before('/');
    FXString username = dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/');
    m_logger.setPath(dxutils::toStdString(DXPREFERENCES.m_logPath+PATHSEPSTRING+username+PATHSEPSTRING+folder));
}

void ImTabItem::stopLogging()
{
    if(!DXPREFERENCES.m_logging) m_logger.stopLogging();
}

void ImTabItem::processCommand(FXString commandtext)
{
    if(commandtext[0]=='/')
    {
        FXString command = commandtext.after('/').before(' ');
        if(comparecase(command, "exec")==0)
        {
            FXString arg = commandtext.after(' ');
            if(arg.empty()) showInternalHelp("exec");
            else
            {
                if(!m_pipe) m_pipe = new dxPipe(getApp(), this);
                m_pipeStrings.clear();
                if(arg.before(' ').contains("-o"))
                {
                    m_sendPipe = TRUE;
                    appendStyledText(FXStringFormat(_("Command %s started"), arg.after(' ').text()), 2, "", false);
                    m_pipe->execCmd(arg.after(' '));
                }
                else if(arg.before(' ').contains("-c"))
                {
                    m_sendPipe = FALSE;
                    appendStyledText(FXStringFormat(_("Command %s stopped"), m_pipe->getCommand().text()), 2, "", false);
                    m_pipeStrings.clear();
                    m_pipe->stopCmd();
                }
                else
                {
                    m_sendPipe = FALSE;
                    appendStyledText(FXStringFormat(_("Command %s started"), arg.text()), 2, "", false);
                    m_pipe->execCmd(arg);
                }
            }
            return;
        }
        else if(comparecase(command, "help")==0)
        {
            FXString arg = commandtext.after(' ');
            if(arg.empty())
            {
                FXString commands = "";
                for(int i=0; i<m_commands.no(); i++)
                {
                    if(i+1!=m_commands.no())
                        commands.append(m_commands[i]+", ");
                    else
                        commands.append(m_commands[i]+".");
                }
                appendStyledText(FXStringFormat(_("Available commands: %s"), commands.text()), 2, "", false);
                return;
            }
            if(comparecase(arg,"exec")==0 || comparecase(arg,"help")==0 || comparecase(arg,"me")==0 || comparecase(arg,"say")==0
                    || comparecase(arg,"log")==0)
            {
                showInternalHelp(arg);
                return;
            }
            std::string response = m_conv->getCommandHelp(dxutils::toStdString(arg));
            if(response.empty())
                appendStyledText(FXStringFormat(_("Command '%s' doesn't exist or hasn't help"), arg.text()), 4, "", false);
            else
            {
                appendStyledText(FXStringFormat(_("Help for: %s"), response.c_str()), 2, "", false);
            }
        }
        else if(comparecase(command,"me")==0)
        {
            if(!commandtext.after(' ').empty()) m_conv->sendActionMessage(commandtext.after(' ').substitute("<","&lt;").substitute(">","&gt;").text());
        }
        else if(comparecase(command,"say")==0)
        {
            if(!commandtext.after(' ').empty()) m_conv->sendMessage(commandtext.after(' ').substitute("<","&lt;").substitute(">","&gt;").text());
        }
        else if(comparecase(command,"log")==0)
        {
            FXString logcommand = commandtext.after(' ').before(' ');
            FXString logfile = commandtext.after(' ').after(' ');
            if(comparecase(logcommand,"list")==0)
            {
                listLogFiles();
            }
            else if(comparecase(logcommand, "show")==0)
            {
                if(logfile.empty())
                    logfile = FXSystem::time("%Y-%m-%d", FXSystem::now());
                FXString logpath = DXPREFERENCES.m_logPath+PATHSEPSTRING+dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/')
                                    +PATHSEPSTRING+dxutils::fromStdString(m_conv->getName()).before('/')+PATHSEPSTRING+logfile;
                if(!FXStat::exists(logpath))
                {
                    appendStyledText(FXStringFormat(_("Log file %s doesn't exist"), logfile.text()), 4, "", false);
                    return;
                }
                m_parent->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, dxTabItem_SHOWLOG), &logpath);
            }
            else
            {
                appendStyledText(FXStringFormat(_("'%s' isn't <list|show>"), logcommand.text()), 4, "", false);
            }
        }
        else
        {
            int ret = m_conv->sendCommand(commandtext.after('/').text());
            processCommandReturn(ret, command.text());
        }
    }
    else
    {
        m_conv->sendMessage(commandtext.substitute("<","&lt;").substitute(">","&gt;").text());
    }
    m_lastKeyPress = 0;
    m_imState = ACTIVE;
}

//process return m_conv->sendCommand
void ImTabItem::processCommandReturn(int ret, const FXchar *command)
{
    switch(ret) {
        case PURPLE_CMD_STATUS_OK:
            break;
        case PURPLE_CMD_STATUS_NOT_FOUND:
            appendStyledText(FXStringFormat(_("Unknown command '%s'."), command), 4);
            break;
        case PURPLE_CMD_STATUS_WRONG_ARGS:
            appendStyledText(FXStringFormat(_("Syntax Error:  You typed the wrong number of arguments "
                                "to that command '%s'."), command), 4);
            break;
        case PURPLE_CMD_STATUS_FAILED:
            appendStyledText(FXStringFormat(_("Your command '%s' failed for an unknown reason."), command), 4);
            break;
        case PURPLE_CMD_STATUS_WRONG_TYPE:
            if(m_conv->getType() == PURPLE_CONV_TYPE_IM)
                appendStyledText(FXStringFormat(_("That command '%s' only works in chats, not IMs."), command), 4);
            else
                appendStyledText(FXStringFormat(_("That command '%s' only works in IMs, not chats."), command), 4);
            break;
        case PURPLE_CMD_STATUS_WRONG_PRPL:
            appendStyledText(FXStringFormat(_("That command '%s' doesn't work on this protocol."), command), 4);
            break;
    }
}

void ImTabItem::showInternalHelp(const FXString &command)
{
    if(comparecase(command, "exec")==0)
    {
        appendStyledText(_("Help for: exec [-o|-c] <command>, executes command, -o sends output to im, -c closes running command"), 2, "", false);
    }
    else if(comparecase(command, "help")==0)
    {
        appendStyledText(_("Help for: help [command]: lists available commands or show help for command"), 2, "", false);
    }
    else if(comparecase(command, "me")==0)
    {
        appendStyledText(_("Help for: me <text>: sends text in 3rd person"), 2, "", false);
    }
    else if(comparecase(command, "say")==0)
    {
        appendStyledText(_("Help for: say <text>: sends text"), 2, "", false);
    }
    else if(comparecase(command, "log")==0)
    {
        appendStyledText(_("Help for: log"), 2, "", false);
        appendStyledText(_("LOG list, shows list of log files for current im/chat"), 2, "", false);
        appendStyledText(_("LOG show [date], shows current log file for current im/chat, if date is empty, else file for given date"), 2, "", false);
        appendStyledText(_("date example: 2011-01-31"), 2, "", false);
    }
}

void ImTabItem::completeHelpCommand()
{
    FXString line = m_commandline->getText();
    for(FXint i = 0; i < m_commands.no(); i++)
    {
        if(comparecase(line.after(' '), m_commands[i]) == 0)
        {
            if((i+1) < m_commands.no()) m_commandline->setText("/help "+m_commands[++i]);
            else m_commandline->setText("/help "+m_commands[0]);
            break;
        }
        else if(comparecase(line.after(' '), m_commands[i].left(line.after(' ').length())) == 0)
        {
            m_commandline->setText("/help "+m_commands[i]);
            break;
        }
    }
}

//List log files for im
void ImTabItem::listLogFiles()
{
    if(m_logThread->running())
    {
        appendStyledText(_("Thread running, try again after some seconds"), 2, "", false);
        return;
    }
    m_logThread->setDirectory(DXPREFERENCES.m_logPath+PATHSEPSTRING+dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/')
            +PATHSEPSTRING+dxutils::fromStdString(m_conv->getName()).before('/'));
    m_logThread->start();
}

//handle LogThread_LIST
long ImTabItem::onListLogThread(FXObject*, FXSelector, void*)
{
    dxStringArray files = m_logThread->getResult();
    if(!files.no())
    {
        appendStyledText(FXStringFormat(_("No log files for %s-%s."), dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/').text(),
                dxutils::fromStdString(m_conv->getName()).before('/').text()), 4, "", false);
        return 1;
    }
    appendStyledText(_("Log files:"), 6, "", false);
    FXString lastYear = dxutils::getParam(files[0], 1, FALSE, '-');
    FXint lastMonth = FXIntVal(dxutils::getParam(files[0], 2, FALSE, '-'));
    appendStyledText(FXStringFormat("%s:", lastYear.text()), 6, "", false);
    appendStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 6, "", false);
    FXString list = "";
    for(FXint i=0; i<files.no(); i++)
    {
        if(lastYear!=dxutils::getParam(files[i], 1, FALSE, '-'))
        {
            lastYear = dxutils::getParam(files[i], 1, FALSE, '-');
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendText(list.rbefore(','), false, false);
            appendStyledText(FXStringFormat("%s:", lastYear.text()), 6, "", false);
            appendStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 6, "", false);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else if(lastMonth!=FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-')))
        {
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendText(list.rbefore(','), false, false);
            appendStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 6, "", false);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
    }
    if(!list.empty()) appendText(list.rbefore(','), false, false);
    return 1;
}

void ImTabItem::loadCurrentLog()
{
    FXString logpath = DXPREFERENCES.m_logPath+PATHSEPSTRING+dxutils::fromStdString(m_conv->getAccount().getUsername()).before('/')
                        +PATHSEPSTRING+dxutils::fromStdString(m_conv->getName()).before('/')+PATHSEPSTRING+FXSystem::time("%Y-%m-%d", FXSystem::now());
    if(FXStat::exists(logpath) && dxutils::isRightLogFile(logpath, FXSystem::time("%Y-%m-%d", FXSystem::now())))
    {
        std::ifstream file(logpath.text());
        std::string sline;
        FXString line;
        while(getline(file, sline))
        {
            m_messageIDs.append("");
            line = dxutils::fromStdString(sline);
            FXint pos = line.find(']')+2;
            if(pos<2) pos = 0;
            if(line.length()>pos+1 && line[pos]=='\014') //log style mark
            {
                FXint style = stringToStyle(line.after('\014').before('\014'));
                m_text->appendText(line.before('\014'));
                if(line.after('\014',2)[0]=='<') //has nick
                {
                    FXString nick = line.after('<').before('>');
                    FXString msg = line.after('>');
                    m_text->appendStyledText(nick+DXPREFERENCES.m_nickCompletionChar, DXPREFERENCES.m_coloredNick?getNickColor(nick):6);
                    appendLinkText(dxutils::parseHtml(msg), style);
                }
                else appendLinkText(dxutils::parseHtml(line.after('\014', 2)), style);
            }
            else
            {
                if(line.length()>pos+1 && line[pos]=='<') //line with nick
                {
                    m_text->appendText(line.before('<'));
                    FXString nick = line.after('<').before('>');
                    FXString msg = line.after('>').trimBegin();
                    m_text->appendStyledText(nick+DXPREFERENCES.m_nickCompletionChar+" ", DXPREFERENCES.m_coloredNick?getNickColor(nick):6);
                    appendLinkText(dxutils::parseHtml(msg), 0);
                }
                else appendLinkText(dxutils::parseHtml(line), 0);
            }
        }
        makeLastRowVisible();
    }
}

FXint ImTabItem::stringToStyle(FXString style)
{
    if(comparecase(style, "highlight")==0)
        return 2;
    if(comparecase(style, "action")==0)
        return 3;
    if(comparecase(style, "error")==0)
        return 4;
    if(comparecase(style, "user")==0)
        return 5;
    if(comparecase(style, "mymsg")==0)
        return 7;
    return 0;
}

