/*
 *      imtabitem.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef IMTABITEM_H
#define	IMTABITEM_H

#include "dxtabitem.h"
#include "logthread.h"
#include "../logger/logger.h"

enum IMStateType {
    STARTED=0, //started typing
    PAUSED, //paused typing
    STOPPED, //stopped typing
    ACTIVE, //imtabitem is opended or text sent
    LEFTIM, //buddy left IM
};

class ImTabItem : public dxTabItem
{
    FXDECLARE(ImTabItem)
public:
    ImTabItem(dxTabBook *tab, const FXString &tabtext, PURPLE::Conversation *conv, FXIcon *ic=0, FXuint opts=TAB_TOP_NORMAL);
    virtual ~ImTabItem();

    long onCommandline(FXObject *, FXSelector, void*);
    long onComposeTimeout(FXObject *, FXSelector, void*);
    long onKeyPress(FXObject *, FXSelector, void*);
    long onListLogThread(FXObject*, FXSelector, void*);

    void createGeom();
    void clearChat();
    void setInfolabelIcon(FXIcon *icon);
    void setCommandFocus();
    void updateImFont();
    void appendNewMyMsg(NewMsg *msg);
    void messageRecived(NewMsg *msg);
    void appendNewMsg(NewMsg *msg);
    void appendNewSystemMsg(NewSystemMsg *msg);
    void appendNewErrorMsg(NewSystemMsg *msg);
    void buddyTyping(bool typing);
private:
    ImTabItem() {}

    dxTabBook *m_parent;
    FXVerticalFrame *m_textframe, *m_mainframe;
    FXLabel *m_info, *m_protocol;
    FXTextField *m_commandline;
    Logger m_logger;
    FXint m_currentPosition, m_historyMax;
    dxStringArray m_commandsHistory;
    FXTime m_lastKeyPress;
    bool m_removeLastLine;
    IMStateType m_imState;
    dxStringArray m_commands;
    LogThread *m_logThread;
    dxStringArray m_messageIDs;

    void appendText(FXString msg, FXbool highlight, FXbool logLine=TRUE);
    void appendNickText(const FXString &nick, const FXString &msg, FXint style, FXString time="", FXbool log=TRUE);
    void appendStyledText(const FXString &msg, FXint style=0, FXString time="", FXbool log=TRUE);
    void appendMyMsg(const FXString &msg, FXString time="", FXbool action=FALSE, const FXString &messageID="");
    void setTextForeColor(FXColor);
    void setTextBackColor(FXColor);
    void startLogging();
    void stopLogging();
    void processCommand(FXString commandtext);
    void startChat(FXString nick);
    void processCommandReturn(int ret, const FXchar *command);
    void showInternalHelp(const FXString &command);
    void completeHelpCommand();
    void listLogFiles();
    void loadCurrentLog();
    FXint stringToStyle(FXString style);
};

#endif	/* IMTABITEM_H */

