/*
 *      msgtabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "msgtabitem.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "dxutils.h"

FXDEFMAP(MsgTabItem) MsgTabItemMap[] = {
    FXMAPFUNC(SEL_COMMAND, MsgTabItem_MSGOK, MsgTabItem::onMsgOk),
    FXMAPFUNC(SEL_COMMAND, MsgTabItem_MSGCANCEL, MsgTabItem::onMsgCancel)
};

FXIMPLEMENT(MsgTabItem, dxTabItem, MsgTabItemMap, ARRAYNUMBER(MsgTabItemMap))

MsgTabItem::MsgTabItem(dxTabBook *tab, const FXString &tabtext, MsgInfo msg, FXIcon *icon, FXuint opts)
: dxTabItem(tab, tabtext, new PURPLE::Conversation(), icon, opts, false), m_message(msg)
{
    m_contents = new FXVerticalFrame(tab, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXHorizontalFrame *labelframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    m_msgIcon = new FXLabel(labelframe, FXString::null, NULL, JUSTIFY_LEFT);
    FXVerticalFrame *textframe = new FXVerticalFrame(m_contents, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_text->reparent(textframe);
    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_contents, LAYOUT_FILL_X);
    m_msgCancelButton = new dxEXButton(buttonframe, _("&Cancel"), NULL, this, MsgTabItem_MSGCANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
    m_msgOkButton = new dxEXButton(buttonframe, _("&OK"), NULL, this, MsgTabItem_MSGOK, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
}

MsgTabItem::~MsgTabItem()
{
}

void MsgTabItem::createGeom()
{
    m_contents->create();
    showMsg();
}

//handle click on m_msgOkButton
long MsgTabItem::onMsgOk(FXObject*, FXSelector, void*)
{
    if(m_message.type==MSGNEWMAIL && !m_message.url.empty())
        dxutils::instance().launchLink(m_message.url);
    this->getParent()->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, dxim_CLOSETAB), NULL);
    return 1;
}

//handle click on m_msgCancelButton
long MsgTabItem::onMsgCancel(FXObject*, FXSelector, void*)
{
    this->getParent()->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, dxim_CLOSETAB), NULL);
    return 1;
}

//show message
void MsgTabItem::showMsg()
{
    switch(m_message.type) {
        case MSGINFORMATION: m_msgIcon->setIcon(ICO_BIGINFORMATION); m_msgCancelButton->hide(); break;
        case MSGWARNING: m_msgIcon->setIcon(ICO_BIGWARNING); m_msgCancelButton->hide(); break;
        case MSGERROR: m_msgIcon->setIcon(ICO_BIGERROR); m_msgCancelButton->hide(); break;
        case MSGNEWMAIL: m_msgIcon->setIcon(ICO_BIGINFORMATION); m_msgCancelButton->show(); break;
    }
    m_text->clearText();
    m_text->appendStyledText(FXSystem::time("%H:%M:%S\n\n", m_message.time), 6);
    m_text->appendText(dxutils::parseHtml(m_message.text));
    m_msgOkButton->setText(m_message.okText);
    m_msgCancelButton->setText(m_message.cancelText);
    m_contents->recalc();
}

