/*
 *      texttabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

/*
 * DON'T USE TextTabItem::appendLinkText
 * m_text->appendText and
 * m_text->appendStyledText DIRECTLY
 */

#include "texttabitem.h"
#include "icons.h"
#include "preferences.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "dxim.h"

FXDEFMAP(TextTabItem) TextTabItemMap[] = {
    FXMAPFUNC(SEL_COMMAND, TextTabItem_OK, TextTabItem::onOk)
};

FXIMPLEMENT(TextTabItem, dxTabItem, TextTabItemMap, ARRAYNUMBER(TextTabItemMap))

TextTabItem::TextTabItem(dxTabBook *tab, const FXString &id, const FXString &tabtext, FXIcon *ic, FXuint opts)
: dxTabItem(tab, tabtext, new PURPLE::Conversation(), ic, opts, false), m_id(id), m_parent(tab)
{
    m_mainframe = new FXVerticalFrame(m_parent, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_textframe = new FXVerticalFrame(m_mainframe, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_text->reparent(m_textframe);
    m_text->setSelTextColor(getApp()->getSelforeColor());
    m_text->setSelBackColor(getApp()->getSelbackColor());
    m_text->setFont(dxim::instance()->getImFont());

    FXHorizontalFrame *buttonframe = new FXHorizontalFrame(m_mainframe, LAYOUT_FILL_X);
    new dxEXButton(buttonframe, _("&OK"), NULL, this, TextTabItem_OK, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0,0,0,0, 10,10,2,5);
}

TextTabItem::~TextTabItem()
{
}

//handle click on OkButton
long TextTabItem::onOk(FXObject*, FXSelector, void*)
{
    this->getParent()->getParent()->getParent()->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, dxim_CLOSETAB), NULL);
    return 1;
}

//create geom, needed for dynamically create tab
void TextTabItem::createGeom()
{
    m_mainframe->create();
}

void TextTabItem::clearText()
{
    m_text->clearText();
}

//append text
void TextTabItem::appendText(const FXString& msg)
{
    appendLinkText(msg, 0);
}

//append styled text
void TextTabItem::appendStyledText(const FXString& text, FXint style, FXString time)
{
    if(!time.empty()) m_text->appendText(time);
    appendLinkText(dxutils::parseHtml(text), style);
}

//append smiley/icon
void TextTabItem::appendSmiley(dxSmiley smiley)
{
    m_text->appendSmiley(smiley);
}

void TextTabItem::showLog(const FXString& path)
{
    if(FXStat::exists(path) && dxutils::isRightLogFile(path, FXPath::name(path)))
    {
        FXString name = FXPath::name(path);
        clearChat();
        appendStyledText(FXStringFormat(_("Log from %s:"), name.text()), 6, "");
        std::ifstream file(path.text());
        std::string sline;
        FXString line;
        while(getline(file, sline))
        {
            line = dxutils::fromStdString(sline);
            FXint pos = line.find(']')+2;
            if(pos<2) pos = 0;
            if(line.length()>pos+1 && line[pos]=='\014') //log style mark
            {
                FXint style = stringToStyle(line.after('\014').before('\014'));
                m_text->appendText(line.before('\014'));
                if(line.after('\014',2)[0]=='<') //has nick
                {
                    FXString nick = line.after('<').before('>');
                    FXString msg = line.after('>');
                    m_text->appendStyledText(nick+":", getNickStyle(nick));
                    appendLinkText(dxutils::parseHtml(msg), style);
                }
                else appendLinkText(dxutils::parseHtml(line.after('\014', 2)), style);
            }
            else
            {
                if(line.length()>pos+1 && line[pos]=='<') //line with nick
                {
                    m_text->appendText(line.before('<'));
                    FXString nick = line.after('<').before('>');
                    FXString msg = line.after('>').trimBegin();
                    m_text->appendStyledText(nick+": ", getNickStyle(nick));
                    appendLinkText(dxutils::parseHtml(msg), 0);
                }
                else appendLinkText(dxutils::parseHtml(line), 0);
            }
        }
    }
    else
    {
        appendStyledText(FXStringFormat(_("'%s' path doesn't exist"), path.text()), 4, "");
    }
}

FXint TextTabItem::stringToStyle(FXString style)
{
    if(comparecase(style, "highlight")==0)
        return 2;
    if(comparecase(style, "action")==0)
        return 3;
    if(comparecase(style, "error")==0)
        return 4;
    if(comparecase(style, "user")==0)
        return 5;
    if(comparecase(style, "mymsg")==0)
        return 7;
    return 0;
}

FXint TextTabItem::getNickStyle(const FXString &nick)
{
    if(DXPREFERENCES.m_coloredNick) return 9+nick.hash()%8; //9 is first colored nick style
    return 6; //bold
}
