/*
 *      accounteditor.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ACCOUNTEDITOR_H
#define ACCOUNTEDITOR_H

#include <gtkmm.h>
#include <purplemm.h>
#include "widgets/imagelabel.h"

struct NewAccount {
    PURPLE::Protocol protocol;
    PURPLE::Protocol::Options options;
    std::string username;
    std::string password;
    std::string iconPath;
    bool registerAccount;
    bool checkEmail;
    bool rememberPassword;
    std::string alias;
};

class AccountEditor : public Gtk::Dialog {
public:
    /* Constructor for modifying account */
    AccountEditor(Gtk::Window& parent, PURPLE::Account account);
    AccountEditor(PURPLE::Account account);
    /* Constructor for creating new account */
    AccountEditor(Gtk::Window& parent, PURPLE::Protocol protocol, bool gtalk=false);
    virtual ~AccountEditor();

    /* Fills new Account */
    void fillAccount(NewAccount *account);
private:
    PURPLE::Protocol m_protocol; //use for creating
    PURPLE::Account m_account; //use for modifying
    PURPLE::Protocol::Options m_options;
    bool m_editing;
    bool m_gtalk;
    std::string m_iconPath;
    Gtk::Notebook m_tabbook;
    Gtk::Entry m_password, m_alias;
    Gtk::CheckButton m_registerAccount, m_checkEmail, m_rememberPassword;
    std::vector<Gtk::Entry*> m_usernameFields;
    std::vector<Gtk::Widget*> m_optionsFields;
    Gtk::VBox m_iconBox;
    Gtk::Button m_iconBtn, m_iconRemove;
    ImageLabel m_iconImage;

    void initializeEditor();
    void createAccountTab();
    void createProtocolTab();
    std::string getStrlistValue(std::map<std::string, std::string> map, std::string key);
    void fillProtocolOptions();
    std::string joinUsername();
    void onSetIcon();
    void onRemoveIcon();
};

#endif  /* ACCOUNTEDITOR_H */

