/*
 *      actionrequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "actionrequestdialog.h"
#include "config.h"
#include "i18n.h"

ActionRequestDialog::ActionRequestDialog(Gtk::Window& parent, PURPLE::ActionRequest* request)
: Gtk::Dialog(request->getTitle().empty()?_("New request"):request->getTitle(), parent), m_request(request)
{
    get_vbox()->pack_start(*Gtk::manage(new Gtk::Label(m_request->getPrimary(), Gtk::ALIGN_START)));
    if(!m_request->getSecondary().empty()) get_vbox()->pack_start(*Gtk::manage(new Gtk::Label(m_request->getSecondary(), Gtk::ALIGN_START)));
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    std::map<std::string, PurpleRequestActionCb> callbacks = m_request->getCallbacks();
    std::map<std::string,PurpleRequestActionCb>::iterator it;
    int i;
    for(it=callbacks.begin(),i=0; it != callbacks.end(); it++,i++)
    {
        Gtk::Button *button = Gtk::manage(new Gtk::Button((*it).first, true));
        button->signal_clicked().connect(sigc::bind<PurpleRequestActionCb>(sigc::mem_fun(this, &ActionRequestDialog::onCallback), (*it).second));
        if(i == m_request->getDefaultAction()) set_focus_child(*button);
        btnbox->pack_end(*button, Gtk::PACK_SHRINK);
    }
    get_vbox()->pack_start(*btnbox, false, false, 3);
    show_all();
}

ActionRequestDialog::~ActionRequestDialog()
{
}

void ActionRequestDialog::onCallback(PurpleRequestActionCb cb)
{
    m_request->runCallback(cb);
    hide();
}


