/*
 *      chatdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chatdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "widgets/imagelabel.h"

ChatDialog::ChatDialog(PURPLE::Account account)
: Gtk::Dialog(_("Add chat"))
{
    m_account = account;
    m_chat = PURPLE::Chat();
    m_modifing = false;
    createWidgets();
}

ChatDialog::ChatDialog(PURPLE::Chat chat)
: Gtk::Dialog(_("Modify chat"))
{
    m_account = chat.getAccount();
    m_chat = chat;
    m_modifing = true;
    createWidgets();
}

void ChatDialog::createWidgets()
{
    m_table = Gtk::manage(new Gtk::Table());
    m_table->attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    ImageLabel *label = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(m_account.getProtocol().getPurpleID()), m_account.getUsername()));
    m_table->attach(*label, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_table->attach(*Gtk::manage(new Gtk::Label(_("Group:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_groups = Gtk::manage(new Gtk::ComboBoxText(true));
    std::vector<std::string> v = m_account.getAllGroups();
    bool chats = false;
    FOREACH(std::vector<std::string>, v, it)
    {
        if((*it) == _("Chats"))
            chats = true;
        m_groups->append(*it);
    }
    if(!chats) m_groups->insert(0, _("Chats"));
    m_groups->set_active(0);
    m_table->attach(*m_groups, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_table->attach(*Gtk::manage(new Gtk::Label(_("Alias:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_alias = Gtk::manage(new Gtk::Entry());
    if(m_modifing) m_alias->set_text(m_chat.getAlias());
    m_table->attach(*m_alias, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    std::vector<PURPLE::Account::ChatEntry> fields;
    if(m_modifing) fields = m_chat.getChatParameters();
    else fields = m_account.getChatParameters();
    int row = 3;
    Gtk::SpinButton *spin;
    Gtk::Entry *text;
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        m_table->attach(*Gtk::manage(new Gtk::Label(createLabel(entry.getLabel()), Gtk::ALIGN_START)), 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        if(entry.getIsInt())
        {
            spin = Gtk::manage(new Gtk::SpinButton());
            spin->set_range(entry.getMin(), entry.getMax());
            spin->set_increments(1, 1);
            spin->set_text(entry.getValue());
            m_fields.push_back(spin);
            m_table->attach(*spin, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        }
        else
        {
            text = Gtk::manage(new Gtk::Entry());
            text->set_text(entry.getValue());
            if(entry.getSecret())
            {
                text->set_visibility(false);
                text->set_invisible_char('*');
            }
            m_fields.push_back(text);
            m_table->attach(*text, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        }
        row++;
    }
    m_autoJoin = Gtk::manage(new Gtk::CheckButton(_("Autojoin chat")));
    if(m_modifing) m_autoJoin->set_active(m_chat.getAutojoin());
    m_table->attach(*m_autoJoin, 0, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    get_vbox()->pack_start(*m_table, Gtk::PACK_SHRINK);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    Gtk::Button *okbtn = Gtk::manage(new Gtk::Button(Gtk::Stock::OK));
    okbtn->signal_clicked().connect(sigc::mem_fun(this, &ChatDialog::onOK));
    Gtk::Button *closebtn = Gtk::manage(new Gtk::Button(Gtk::Stock::CANCEL));
    closebtn->signal_clicked().connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*okbtn, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebtn, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    show_all();
}

ChatDialog::~ChatDialog()
{
}

Glib::ustring ChatDialog::createLabel(std::string label)
{
    Glib::ustring text = "";
    FOREACH(std::string, label, it)
    {
        if((*it)=='_') continue;
        text.append(1, (*it));
    }
    return text;
}

std::vector<PURPLE::Account::ChatEntry> ChatDialog::fillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields = m_account.getChatParameters();
    std::vector<PURPLE::Account::ChatEntry>::iterator it;
    guint i;
    for(it=fields.begin(), i=0; it!=fields.end(); it++, i++)
    {
        PURPLE::Account::ChatEntry& entry = *it;
        if(entry.getIsInt())
        {
            entry.setValue(static_cast<Gtk::SpinButton*>(m_fields[i])->get_text());
        }
        else
        {
            entry.setValue(static_cast<Gtk::Entry*>(m_fields[i])->get_text());
        }
    }
    return fields;
}

void ChatDialog::onOK()
{
    Glib::ustring alias = m_alias->get_text();
    if(alias.empty()) alias = static_cast<Gtk::Entry*>(m_fields[0])->get_text();
    //chat name empty => return
    if(alias.empty()) return;
    Glib::ustring group = m_groups->get_entry_text();
    if(group.empty()) group = _("Chats");
    if(m_modifing)
    {
        m_chat.setChatParameters(fillChatEntryFields());
        m_chat.setAlias(alias);
        m_chat.setAutojoin(m_autoJoin->get_active());
        if(m_chat.getGroupName().compare(group))
        {
            m_account.moveChat(m_chat, group);
        }
    }
    else
    {
        m_account.addChat(fillChatEntryFields(), alias, group, m_autoJoin->get_active());
    }
    response(Gtk::RESPONSE_OK);
    hide();
}


