/*
 *      chatjoindialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chatjoindialog.h"
#include "roomlistdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

ChatJoinDialog::ChatJoinDialog(Gtk::Window& parent, PURPLE::Purple* purple, dxgCallbacks* cb)
: Gtk::Dialog(_("Connect chat"), parent), m_cb(cb)
{
    std::vector<PURPLE::Account> accounts = purple->getConnectedAccountsList();
    //remove accounts !supportChats
    for(guint i=0; i<accounts.size(); i++)
    {
        if(!accounts[i].supportsChats())
        {
            accounts.erase(accounts.begin()+i);
        }
    }
    createControls(accounts);
}

ChatJoinDialog::ChatJoinDialog(PURPLE::Account account, dxgCallbacks* cb)
: Gtk::Dialog(_("Connect chat")), m_cb(cb)
{
    std::vector<PURPLE::Account> accounts;
    accounts.push_back(account);
    createControls(accounts);
}

ChatJoinDialog::~ChatJoinDialog()
{
}

void ChatJoinDialog::createChatEntryFields(std::vector<PURPLE::Account::ChatEntry> fields)
{
    while(m_fields.size())
    {
        Gtk::Widget *w = *(m_fields.begin());
        m_table->remove(*w);
        m_fields.erase(m_fields.begin());
        delete w;
    }
    m_fields.clear();
    int row = 1;
    Gtk::Label *label;
    Gtk::SpinButton *spin;
    Gtk::Entry *text;
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        label = Gtk::manage(new Gtk::Label(createLabel(entry.getLabel()), Gtk::ALIGN_START));
        m_table->attach(*label, 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        m_fields.push_back(label);
        if(entry.getIsInt())
        {
            spin = Gtk::manage(new Gtk::SpinButton());
            spin->set_range(entry.getMin(), entry.getMax());
            spin->set_increments(1, 1);
            spin->set_text(entry.getValue());
            m_fields.push_back(spin);
            m_table->attach(*spin, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        }
        else
        {
            text = Gtk::manage(new Gtk::Entry());
            text->set_text(entry.getValue());
            if(entry.getSecret())
            {
                text->set_visibility(false);
                text->set_invisible_char('*');
            }
            m_fields.push_back(text);
            m_table->attach(*text, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        }
        row++;
    }
    show_all();
    resize(1,1);
}

Glib::ustring ChatJoinDialog::createLabel(std::string label)
{
    Glib::ustring text = "";
    FOREACH(std::string, label, it)
    {
        if((*it)=='_') continue;
        text.append(1, (*it));
    }
    return text;
}

std::vector<PURPLE::Account::ChatEntry> ChatJoinDialog::fillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields;
    Gtk::TreeModel::iterator iter = m_ids.get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            PURPLE::Account account = row[m_idColumns.m_account];
            fields = account.getChatParameters();
            std::vector<PURPLE::Account::ChatEntry>::iterator it;
            guint i;
            for(it=fields.begin(), i=1; it!=fields.end(); it++,i=i+2)
            {
                PURPLE::Account::ChatEntry& entry = *it;
                if(entry.getIsInt())
                {
                    entry.setValue(static_cast<Gtk::SpinButton*>(m_fields[i])->get_text());
                }
                else
                {
                    entry.setValue(static_cast<Gtk::Entry*>(m_fields[i])->get_text());
                }
            }
        }
    }
    return fields;
}

void ChatJoinDialog::createControls(std::vector<PURPLE::Account> accounts)
{
    m_idModel = Gtk::TreeStore::create(m_idColumns);
    m_ids.set_model(m_idModel);
    for(guint i=0; i<accounts.size(); i++)
    {
        Gtk::TreeModel::Row row = *(m_idModel->append());
        row[m_idColumns.m_icon] = dxutils::getProtocolIcon(accounts[i].getProtocol().getPurpleID());
        row[m_idColumns.m_name] = accounts[i].getUsername();
        row[m_idColumns.m_account] = accounts[i];
    }
    m_ids.pack_start(m_idColumns.m_icon);
    m_ids.pack_start(m_idColumns.m_name);
    m_ids.set_active(0);
    m_ids.signal_changed().connect(sigc::mem_fun(this, &ChatJoinDialog::onAccount));
    m_table = Gtk::manage(new Gtk::Table());
    m_table->attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_table->attach(m_ids, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    get_vbox()->pack_start(*m_table, Gtk::PACK_SHRINK);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    m_chatButton = Gtk::manage(new Gtk::Button(_("_Get chats list"), true));
    m_chatButton->signal_clicked().connect(sigc::mem_fun(this, &ChatJoinDialog::onGetChats));
    if(!accounts[0].supportRoomlist()) m_chatButton->set_sensitive(false);
    Gtk::Button *joinbtn = Gtk::manage(new Gtk::Button(_("_Join"), true));
    joinbtn->signal_clicked().connect(sigc::mem_fun(this, &ChatJoinDialog::onJoin));
    Gtk::Button *closebtn = Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
    closebtn->signal_clicked().connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*m_chatButton, Gtk::PACK_SHRINK);
    btnbox->pack_end(*joinbtn, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebtn, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    createChatEntryFields(accounts[0].getChatParameters());

    m_cb->signalRoomlistCreated.connect(sigc::mem_fun(this, &ChatJoinDialog::onRoomlistCreated));
}

void ChatJoinDialog::onAccount()
{
    Gtk::TreeModel::iterator iter = m_ids.get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            PURPLE::Account account = row[m_idColumns.m_account];
            createChatEntryFields(account.getChatParameters());
            if(account.supportRoomlist()) m_chatButton->set_sensitive(true);
            else m_chatButton->set_sensitive(false);
        }
    }
}

void ChatJoinDialog::onGetChats()
{
    Gtk::TreeModel::iterator iter = m_ids.get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            PURPLE::Account account = row[m_idColumns.m_account];
            account.displayRoomList();
        }
    }
    m_chatButton->set_sensitive(false);
}

void ChatJoinDialog::onJoin()
{
    Gtk::TreeModel::iterator iter = m_ids.get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            PURPLE::Account account = row[m_idColumns.m_account];
            account.joinChat(fillChatEntryFields());
        }
    }
    hide();
}

void ChatJoinDialog::onRoomlistCreated(PURPLE::RoomList *rlist)
{
    RoomListDialog *dialog = new RoomListDialog(*this, rlist, m_cb);
    dialog->show();
    dialog->present();
    dialog->signalClosed.connect(sigc::mem_fun(this, &ChatJoinDialog::onRoomListClosed));
}

void ChatJoinDialog::onRoomListClosed(bool success, Glib::ustring selected)
{
    if(success) static_cast<Gtk::Entry*>(m_fields[1])->set_text(selected);
    m_chatButton->set_sensitive(true);
}


