/*
 *      choicerequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "choicerequestdialog.h"
#include "config.h"
#include "i18n.h"

ChoiceRequestDialog::ChoiceRequestDialog(Gtk::Window& parent, PURPLE::ChoiceRequest* request)
: Gtk::Dialog(request->getTitle().empty()?_("New request"):request->getTitle(), parent), m_request(request)
{
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    add_button(m_request->getOkText(), Gtk::RESPONSE_OK);
    get_vbox()->pack_start(*Gtk::manage(new Gtk::Label(m_request->getPrimary(), Gtk::ALIGN_START)));
    if(!m_request->getSecondary().empty()) get_vbox()->pack_start(*Gtk::manage(new Gtk::Label(m_request->getSecondary(), Gtk::ALIGN_START)));
    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_model = Gtk::ListStore::create(m_columns);
    m_list = new Gtk::TreeView();
    m_list->set_model(m_model);
    m_list->append_column("Choice", m_columns.m_label);
    m_list->set_headers_visible(false);
    m_list->get_selection()->set_mode(Gtk::SELECTION_SINGLE);
    std::map<std::string, int> choices = m_request->getChoices();
    std::map<std::string, int>::iterator it;
    for(it=choices.begin(); it != choices.end(); it++)
    {
        Gtk::TreeModel::Row row = *(m_model->append());
        row[m_columns.m_label] = (*it).first;
        row[m_columns.m_id] = (*it).second;
        if((*it).second == m_request->getDefaultValue())
            m_list->get_selection()->select(row);
    }
    swin->add(*m_list);
    get_vbox()->pack_start(*swin, Gtk::PACK_EXPAND_WIDGET, 3);
    show_all();
}

ChoiceRequestDialog::~ChoiceRequestDialog()
{
}

void ChoiceRequestDialog::on_response(int response)
{
    if(response == Gtk::RESPONSE_OK)
    {
        Gtk::TreeModel::iterator it = m_list->get_selection()->get_selected();
        if(it)
        {
            Gtk::TreeModel::Row row = *it;
            m_request->sendChoice(row[m_columns.m_id]);
        }
        else
            m_request->dennyRequest();
    }
    else
        m_request->dennyRequest();
}

