/*
 *      configdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CONFIGDIALOG_H
#define CONFIGDIALOG_H

#include <gtkmm.h>
#include "widgets/imagelabel.h"

class ConfigDialog : public Gtk::Dialog {
public:
    ConfigDialog(Gtk::Window& parent);
    virtual ~ConfigDialog();
private:
    bool m_showImportwarning;
    Gtk::Notebook m_tabbook;
    //GeneralTab widgets
    Gtk::CheckButton *m_oneInstance;
    Gtk::CheckButton *m_useAway;
    Gtk::CheckButton *m_minToTray;
    Gtk::CheckButton *m_logging;
    Gtk::CheckButton *m_logLoadCurrent;
    Gtk::SpinButton *m_awayTimeout;
    Gtk::FileChooserButton *m_logPath;
    Gtk::CheckButton *m_showRosterTree;
    Gtk::CheckButton *m_receiptsMark;
    //IconsTab widgets
    struct PathColumns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::ustring> m_path;

        PathColumns() { add(m_path); }
    };
    const PathColumns m_pathColumns;
    Glib::RefPtr<Gtk::ListStore> m_themesStatusModel;
    Glib::RefPtr<Gtk::ListStore> m_themesRoleModel;
    Gtk::TreeView *m_themesStatusList;
    Gtk::TreeView *m_themesRoleList;
    Gtk::Image *m_iconAway;
    Gtk::Image *m_iconIdle;
    Gtk::Image *m_iconDnd;
    Gtk::Image *m_iconInvisible;
    Gtk::Image *m_iconOffline;
    Gtk::Image *m_iconOnline;
    Gtk::Image *m_iconFounder;
    Gtk::Image *m_iconHalfop;
    Gtk::Image *m_iconNormal;
    Gtk::Image *m_iconOp;
    Gtk::Image *m_iconVoice;
    Gtk::Button *m_themeStatusAdd;
    Gtk::Button *m_themeStatusDelete;
    Gtk::Button *m_themeRoleAdd;
    Gtk::Button *m_themeRoleDelete;
    //LookTab widgets
    struct PositionColumns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::ustring> m_name;
        Gtk::TreeModelColumn<int> m_position;

        PositionColumns() { add(m_name); add(m_position); }
    };
    const PositionColumns m_tabColumns;
    Glib::RefPtr<Gtk::ListStore> m_tabModel;
    Gtk::ColorButton *m_unreadtabColor;
    Gtk::ColorButton *m_hltabColor;
    ImageLabel *m_unreadtabLabel;
    ImageLabel *m_hltabLabel;
    Gtk::ComboBox *m_tabPosition;
    Gtk::Notebook *m_lookTabs;
    //ImTextTab widgets
    Gtk::ColorButton *m_textColor;
    Gtk::ColorButton *m_backroundColor;
    Gtk::ColorButton *m_userColor;
    Gtk::ColorButton *m_actionColor;
    Gtk::ColorButton *m_errorColor;
    Gtk::ColorButton *m_hilightColor;
    Gtk::ColorButton *m_linkColor;
    Gtk::ColorButton *m_mymsgColor;
    Gtk::ColorButton *m_logColor;
    Gtk::TextView *m_imText;
    Gtk::Entry *m_nickCompletionChar;
    Gtk::CheckButton *m_coloredNick;
    Gtk::CheckButton *m_sameCmd;
    Gtk::CheckButton *m_sameList;
    Gtk::FontButton *m_imFont;
    //NotificationTab widgets
    Gtk::CheckButton *m_sounds;
    Gtk::CheckButton *m_soundConnect;
    Gtk::CheckButton *m_soundDisconnect;
    Gtk::CheckButton *m_soundMessage;
    Gtk::FileChooserButton *m_pathConnect;
    Gtk::FileChooserButton *m_pathDisconnect;
    Gtk::FileChooserButton *m_pathMessage;
    Gtk::Button *m_playConnect;
    Gtk::Button *m_playDisconnect;
    Gtk::Button *m_playMessage;
    Gtk::CheckButton *m_notify;
    Gtk::CheckButton *m_notifyAccountConnect;
    Gtk::CheckButton *m_notifyAccountDisconnect;
    Gtk::CheckButton *m_notifyBuddyConnect;
    Gtk::CheckButton *m_notifyBuddyDisconnect;
    Gtk::CheckButton *m_notifyMessage;
    Gtk::CheckButton *m_notifyFiletransfer;
    Gtk::CheckButton *m_raiseWindow;
    Gtk::CheckButton *m_notifyMessageTray;
    const PositionColumns m_notifyColumns;
    Glib::RefPtr<Gtk::ListStore> m_notifyModel;
    Gtk::ComboBox *m_notifyPosition;
    Gtk::Button *m_showNotify;
    //SmileysTab widgets
    struct SmileyColumns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_icon;
        Gtk::TreeModelColumn<Glib::ustring> m_smiley;
        Gtk::TreeModelColumn<Glib::ustring> m_path;

        SmileyColumns() { add(m_icon); add(m_smiley); add(m_path); }
    };
    const SmileyColumns m_smileysColumns;
    Glib::RefPtr<Gtk::ListStore> m_smileysModel;
    Gtk::CheckButton *m_useSmileys;
    Gtk::TreeView *m_smileysList;
    Gtk::Button *m_addSmiley;
    Gtk::Button *m_modifySmiley;
    Gtk::Button *m_deleteSmiley;
    Gtk::Button *m_importSmiley;
    Gtk::Button *m_exportSmiley;

    void createGeneralTab();
    void createIconsTab();
    void createLookTab();
    void createImtextTab();
    void createNotificationTab();
    void createSmileysTab();
    void createText();
    bool hasSmiley(const Glib::ustring &smiley);
    //slots start
    void onOK();
    void onUseAway();
    void onLogging();
    void onSetLogPath();
    void onAddStatusTheme();
    void onDeleteStatusTheme();
    void onAddRoleTheme();
    void onDeleteRoleTheme();
    void onStatusThemeChanged();
    void onRoleThemeChanged();
    void updateTabPosition();
    void onTabColor();
    void updateIMText();
    void onImFont();
    void onSounds();
    void onSoundsPath();
    void onPlay(int sound);
    void onNotify();
    void onShowNotify();
    void onSmileys();
    void onAddSmiley();
    void onModifySmiley();
    void onDeleteSmiley();
    void onImportSmiley();
    void onExportSmiley();
    void onSmileysDoubleClick(const Gtk::TreeModel::Path &path, Gtk::TreeViewColumn *column);
    void onRaiseWindow();
    //slots end
};

class SmileyDialog : public Gtk::Dialog {
public:
    SmileyDialog(const Glib::ustring &title, const Glib::ustring &smiley, const Glib::ustring &path, Gtk::Window& parent);

    Glib::ustring getSmiley() const { return m_smiley->get_text(); }
    Glib::ustring getPath() const { return m_path->get_filename(); }
private:
    Gtk::Entry *m_smiley;
    Gtk::FileChooserButton *m_path;
    Gtk::Image *m_icon;

    void onPath();
};

#endif  /* CONFIGDIALOG_H */

