/*
 *      imdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "imdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

IMDialog::IMDialog(Gtk::Window& parent, PURPLE::Purple* purple)
: Gtk::Dialog(_("New IM"), parent), m_purple(purple)
{
    createControls(purple->getConnectedAccountsList());
}

IMDialog::IMDialog(PURPLE::Account account, PURPLE::Purple* purple)
    : Gtk::Dialog(_("New IM")), m_purple(purple)
{
    std::vector<PURPLE::Account> accounts;
    accounts.push_back(account);
    createControls(accounts);
}

IMDialog::~IMDialog()
{
}

void IMDialog::onAccount()
{
    Gtk::TreeModel::iterator iter = m_ids.get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            PURPLE::Account account = row[m_idColumns.m_account];
            fillCompleter(account.getBuddies());
        }
    }
}

void IMDialog::on_response(int response)
{
    if(!m_with.get_text().empty() && response == Gtk::RESPONSE_OK)
    {
        Gtk::TreeModel::iterator iter = m_ids.get_active();
        if(iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row)
            {
                m_purple->createConversation(row[m_idColumns.m_account], m_with.get_text(), false);
            }
        }
    }
}

void IMDialog::fillCompleter(std::vector<PURPLE::Buddy> buddies)
{
    m_withModel->clear();
    FOREACH(std::vector<PURPLE::Buddy>, buddies, it)
    {
        PURPLE::Buddy buddy = *it;
        Gtk::TreeModel::Row row = *(m_withModel->append());
        row[m_withColumns.m_name] = buddy.getUsername();
        row[m_withColumns.m_hint] = buddy.getUsername()+" \""+buddy.getAlias()+"\"";
    }
}

void IMDialog::createControls(std::vector<PURPLE::Account> accounts)
{
    m_idModel = Gtk::TreeStore::create(m_idColumns);
    m_ids.set_model(m_idModel);
    for(guint i=0; i<accounts.size(); i++)
    {
        Gtk::TreeModel::Row row = *(m_idModel->append());
        row[m_idColumns.m_icon] = dxutils::getProtocolIcon(accounts[i].getProtocol().getPurpleID());
        row[m_idColumns.m_name] = accounts[i].getUsername();
        row[m_idColumns.m_account] = accounts[i];
    }
    m_ids.pack_start(m_idColumns.m_icon);
    m_ids.pack_start(m_idColumns.m_name);
    m_ids.set_active(0);
    m_ids.signal_changed().connect(sigc::mem_fun(this, &IMDialog::onAccount));
    m_table = Gtk::manage(new Gtk::Table());
    m_table->attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_table->attach(m_ids, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_table->attach(*Gtk::manage(new Gtk::Label(_("With:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    Glib::RefPtr<Gtk::EntryCompletion> completion = Gtk::EntryCompletion::create();
    m_with.set_completion(completion);
    m_withModel = Gtk::ListStore::create(m_withColumns);
    completion->set_model(m_withModel);
    completion->set_text_column(1);
    completion->signal_match_selected().connect(sigc::mem_fun(this, &IMDialog::onCompleter), false);
    m_table->attach(m_with, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    get_vbox()->pack_start(*m_table, Gtk::PACK_SHRINK);
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    add_button(_("_IM"), Gtk::RESPONSE_OK);
    set_default_response(Gtk::RESPONSE_OK);
    fillCompleter(accounts[0].getBuddies());
    show_all();
}

bool IMDialog::onCompleter(const Gtk::TreeModel::iterator& it)
{
    Gtk::TreeModel::Row row = *it;
    m_with.set_text(row[m_withColumns.m_name]);
    return true;
}


