/*
 *      logdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <glib.h>
#include <glib/gstdio.h>
#include <fstream>
#include <cstdio>
#include "logdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "preferences.h"
#include "icons.h"
#include "utilsmm.h"
#include "widgets/imagelabel.h"

template<typename T>
/** Someone from string */
T s2t(const std::string & Str)
{
    T Dest;
    std::istringstream iss( Str );
    iss >> Dest;
    return Dest;
}

LogDialog::LogDialog(Gtk::Window& parent)
    : Gtk::Dialog(_("dxim - log viewer"), parent)
{
    m_fullLog = true;
    createWidgets();
    loadTree();
}

LogDialog::LogDialog(PURPLE::Account account)
    : Gtk::Dialog(_("dxim - log viewer"))
{
    m_fullLog = false;
    Glib::ustring username = account.getUsername();
    username = username.substr(0, username.find_last_of('/'));
    ImageLabel *m_id = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(account.getProtocol().getPurpleID()), username));
    get_vbox()->pack_start(*m_id, false, true, 3);
    createWidgets();
    show_all();
    Gtk::TreeModel::Row row = *(m_logModel->append());
    row[m_logColumns.m_name] = _("Logs");
    row[m_logColumns.m_sortID] = _("Logs");
    row[m_logColumns.m_type] = Root;
    row[m_logColumns.m_path] = DXPREFERENCES.m_logPath;
    row[m_logColumns.m_time] = 0;
    row[m_logColumns.m_hidden] = false;
    Gtk::TreeModel::Row item = *(m_logModel->append(row.children()));
    item[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
    item[m_logColumns.m_type] = Account;
    item[m_logColumns.m_name] = username;
    item[m_logColumns.m_sortID] = username;
    item[m_logColumns.m_path] = Glib::build_filename(DXPREFERENCES.m_logPath, username);
    item[m_logColumns.m_time] = 0;
    item[m_logColumns.m_hidden] = false;
    listDirsChildItems(item);
    m_treeLoaded = true;
    m_logs->expand_row(m_logModel->get_path(row), true);
}

LogDialog::LogDialog(PURPLE::Buddy buddy)
    : Gtk::Dialog(_("dxim - log viewer"))
{
    m_fullLog = false;
    Glib::ustring aname = buddy.getAccount().getUsername();
    aname = aname.substr(0, aname.find_last_of('/'));
    ImageLabel *m_id = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(buddy.getAccount().getProtocol().getPurpleID()), buddy.getUsername()+"("+aname+")"));
    get_vbox()->pack_start(*m_id, false, true, 3);
    createWidgets();
    show_all();
    Gtk::TreeModel::Row row = *(m_logModel->append());
    row[m_logColumns.m_name] = _("Logs");
    row[m_logColumns.m_sortID] = _("Logs");
    row[m_logColumns.m_type] = Root;
    row[m_logColumns.m_path] = DXPREFERENCES.m_logPath;
    row[m_logColumns.m_time] = 0;
    row[m_logColumns.m_hidden] = false;
    Gtk::TreeModel::Row aitem = *(m_logModel->append(row.children()));
    aitem[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
    aitem[m_logColumns.m_type] = Account;
    aitem[m_logColumns.m_name] = aname;
    aitem[m_logColumns.m_sortID] = aname;
    aitem[m_logColumns.m_path] = Glib::build_filename(DXPREFERENCES.m_logPath, aname);
    aitem[m_logColumns.m_time] = 0;
    aitem[m_logColumns.m_hidden] = false;
    Gtk::TreeModel::Row bitem = *(m_logModel->append(aitem.children()));
    bitem[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
    bitem[m_logColumns.m_type] = Conversation;
    bitem[m_logColumns.m_name] = buddy.getUsername();
    bitem[m_logColumns.m_sortID] = buddy.getUsername();
    bitem[m_logColumns.m_path] = Glib::build_filename(DXPREFERENCES.m_logPath, aname, buddy.getUsername());
    bitem[m_logColumns.m_time] = 0;
    bitem[m_logColumns.m_hidden] = false;
    listFilesChildItems(bitem);
    m_treeLoaded = true;
    m_logs->expand_row(m_logModel->get_path(row), true);
}

LogDialog::LogDialog(PURPLE::Chat chat)
    : Gtk::Dialog(_("dxim - log viewer"))
{
    m_fullLog = false;
    Glib::ustring aname = chat.getAccount().getUsername();
    aname = aname.substr(0, aname.find_last_of('/'));
    ImageLabel *m_id = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(chat.getAccount().getProtocol().getPurpleID()), chat.getChatName()+"("+aname+")"));
    get_vbox()->pack_start(*m_id, false, true, 3);
    createWidgets();
    show_all();
    Gtk::TreeModel::Row row = *(m_logModel->append());
    row[m_logColumns.m_name] = _("Logs");
    row[m_logColumns.m_sortID] = _("Logs");
    row[m_logColumns.m_type] = Root;
    row[m_logColumns.m_path] = DXPREFERENCES.m_logPath;
    row[m_logColumns.m_time] = 0;
    row[m_logColumns.m_hidden] = false;
    Gtk::TreeModel::Row aitem = *(m_logModel->append(row.children()));
    aitem[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
    aitem[m_logColumns.m_type] = Account;
    aitem[m_logColumns.m_name] = aname;
    aitem[m_logColumns.m_sortID] = aname;
    aitem[m_logColumns.m_path] = Glib::build_filename(DXPREFERENCES.m_logPath, aname);
    aitem[m_logColumns.m_time] = 0;
    aitem[m_logColumns.m_hidden] = false;
    Gtk::TreeModel::Row chitem = *(m_logModel->append(aitem.children()));
    chitem[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
    chitem[m_logColumns.m_type] = Conversation;
    chitem[m_logColumns.m_name] = chat.getChatName();
    chitem[m_logColumns.m_sortID] = chat.getChatName();
    chitem[m_logColumns.m_path] = Glib::build_filename(DXPREFERENCES.m_logPath, aname, chat.getChatName());
    chitem[m_logColumns.m_time] = 0;
    chitem[m_logColumns.m_hidden] = false;
    listFilesChildItems(chitem);
    m_treeLoaded = true;
    m_logs->expand_row(m_logModel->get_path(row), true);
}

LogDialog::~LogDialog()
{
}

void LogDialog::setLogPath(const Glib::ustring &path)
{
    Gtk::TreeModel::Children roots = m_logModel->children();
    for(Gtk::TreeModel::Children::iterator rootIt = roots.begin(); rootIt != roots.end(); ++rootIt)
    {
        if((*rootIt)[m_logColumns.m_path] != path)
        {
            (*rootIt)[m_logColumns.m_path] = path;
            listDirsChildItems((*rootIt));
        }
    }
}

bool LogDialog::on_key_press_event(GdkEventKey *event)
{
    if(event->keyval == GDK_KEY_F3)
    {
        searchInFile(true);
        return true;
    }
    return Gtk::Window::on_key_press_event(event);
}

void LogDialog::onOK()
{
    onReset();
    hide();
}

void LogDialog::onSearch()
{
    LogItemType type = Root;
    Gtk::TreeModel::iterator it = m_logs->get_selection()->get_selected();
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        type = row[m_logColumns.m_type];
    }
    if(type != File) clearText();
    enableAllItems();
    if(m_searchLine->get_text().empty())
        return;
    m_searchString = m_searchLine->get_text();
    if(m_buttonFile->get_active())
        searchInFile();
    else if(m_buttonConv->get_active())
        searchInConversation();
    else if(m_buttonAccount->get_active())
        searchInAccount();
    else if(m_buttonAll->get_active())
        searchInAll();
}

void LogDialog::onReset()
{
    m_searchLine->set_text("");
    m_searchString = "";
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    buffer->remove_tag_by_name("selected", buffer->begin(), buffer->end());
    enableAllItems();
}

void LogDialog::enableAllItems()
{
    Gtk::TreeModel::Children roots = m_logModel->children();
    for(Gtk::TreeModel::Children::iterator rootIt = roots.begin(); rootIt != roots.end(); ++rootIt)
    {
        Gtk::TreeModel::Children accounts = (*rootIt)->children();
        for(Gtk::TreeModel::Children::iterator accountIt = accounts.begin(); accountIt != accounts.end(); ++accountIt)
        {
            (*accountIt)[m_logColumns.m_hidden] = false;
            Gtk::TreeModel::Children convs = (*accountIt)->children();
            for(Gtk::TreeModel::Children::iterator convIt = convs.begin(); convIt != convs.end(); ++convIt)
            {
                (*convIt)[m_logColumns.m_hidden] = false;
                Gtk::TreeModel::Children years = (*convIt)->children();
                for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
                {
                    (*yearIt)[m_logColumns.m_hidden] = false;
                    Gtk::TreeModel::Children months = (*yearIt)->children();
                    for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
                    {
                        (*monthIt)[m_logColumns.m_hidden] = false;
                        Gtk::TreeModel::Children files = (*monthIt)->children();
                        for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
                        {
                            (*fileIt)[m_logColumns.m_hidden] = false;
                        }
                    }
                }
            }
        }
        m_logs->expand_row(m_logModel->get_path(rootIt), false);
    }
}

void LogDialog::loadTree()
{
    Gtk::TreeModel::Row row = *(m_logModel->append());
    row[m_logColumns.m_name] = _("Logs");
    row[m_logColumns.m_sortID] = _("Logs");
    row[m_logColumns.m_type] = Root;
    row[m_logColumns.m_path] = DXPREFERENCES.m_logPath;
    row[m_logColumns.m_time] = 0;
    row[m_logColumns.m_hidden] = false;
    listDirsChildItems(row);
}

void LogDialog::loadFullTree()
{
    Gtk::TreeModel::Children roots = m_logModel->children();
    for(Gtk::TreeModel::Children::iterator rootIt = roots.begin(); rootIt != roots.end(); ++rootIt)
    {
        listDirsChildItems(*rootIt);
        Gtk::TreeModel::Children accs = (*rootIt)->children();
        for(Gtk::TreeModel::Children::iterator accIt = accs.begin(); accIt != accs.end(); ++accIt)
        {
            listDirsChildItems(*accIt);
            Gtk::TreeModel::Children convs = (*accIt)->children();
            for(Gtk::TreeModel::Children::iterator convIt = convs.begin(); convIt != convs.end(); ++convIt)
            {
                listFilesChildItems(*convIt);
            }
        }
    }
    m_treeLoaded = true;
}

void LogDialog::listDirsChildItems(Gtk::TreeModel::Row parent)
{
    Glib::ustring path = parent[m_logColumns.m_path];
    if(parent[m_logColumns.m_type]==Root && !Glib::file_test(path, Glib::FILE_TEST_EXISTS))
    {
        appendText(_("Any logs"));
        return;
    }
    GStatBuf info;
    g_stat(path.c_str(), &info);
    if(parent[m_logColumns.m_time]!=info.st_mtime)
    {
        std::vector<Glib::ustring> olditems;
        Gtk::TreeModel::Children children = parent->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            olditems.push_back((*iter)[m_logColumns.m_name]);
        }
        Glib::Dir dir(path);
        std::string name = dir.read_name();
        while(!name.empty())
        {
            if(name[0] != '.')
            {
                std::vector<Glib::ustring>::iterator it = std::find(olditems.begin(), olditems.end(), name);
                if(it == olditems.end())
                {
                    Glib::ustring childPath = Glib::build_filename(path, name);
                    if(Glib::file_test(childPath, Glib::FILE_TEST_IS_DIR))
                    {
                        Gtk::TreeModel::Row item = *(m_logModel->append(parent.children()));
                        item[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
                        item[m_logColumns.m_type] = parent[m_logColumns.m_type]==Root?Account:Conversation;
                        item[m_logColumns.m_name] = name;
                        item[m_logColumns.m_sortID] = name;
                        item[m_logColumns.m_path] = childPath;
                        item[m_logColumns.m_time] = 0;
                        item[m_logColumns.m_hidden] = false;
                    }
                }
                else
                {
                    olditems.erase(it);
                }
            }
            name = dir.read_name();
        }
        // Wipe items remaining in oldlist
        children = parent->children();
        FOREACH(std::vector<Glib::ustring>, olditems, it)
        {
            for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
            {
                if((*iter)[m_logColumns.m_name] == *it)
                {
                    m_logModel->erase(iter);
                    break;
                }
            }
        }
        parent[m_logColumns.m_time] = info.st_mtime;
    }
}

void LogDialog::onItemChanged()
{
    Gtk::TreeModel::iterator it = m_logModelFilter->convert_iter_to_child_iter(m_logs->get_selection()->get_selected());
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        LogItemType type = row[m_logColumns.m_type];
        if(type==Root || type==Account)
        {
            clearText();
            if(type==Root)
            {
                m_buttonFile->set_sensitive(false);
                m_buttonConv->set_sensitive(false);
                m_buttonAccount->set_sensitive(false);
                m_buttonAll->set_sensitive(true);
                m_buttonAll->set_active(true);
            }
            else
            {
                m_buttonFile->set_sensitive(false);
                m_buttonConv->set_sensitive(false);
                m_buttonAccount->set_sensitive(true);
                m_buttonAll->set_sensitive(true);
                m_buttonAccount->set_active(true);
            }
            if(m_fullLog) listDirsChildItems(row);
        }
        else if(type==Conversation)
        {
            clearText();
            m_buttonFile->set_sensitive(false);
            m_buttonConv->set_sensitive(true);
            m_buttonAccount->set_sensitive(true);
            m_buttonAll->set_sensitive(true);
            m_buttonConv->set_active(true);
            listFilesChildItems(row);
        }
        else if(type==Year || type==Month)
        {
            clearText();
            m_buttonFile->set_sensitive(false);
            m_buttonConv->set_sensitive(true);
            m_buttonAccount->set_sensitive(true);
            m_buttonAll->set_sensitive(true);
            m_buttonConv->set_active(true);
        }
        else if(type==File)
        {
            m_buttonFile->set_sensitive(true);
            m_buttonConv->set_sensitive(true);
            m_buttonAccount->set_sensitive(true);
            m_buttonAll->set_sensitive(true);
            m_buttonFile->set_active(true);
            if(!loadFile(row[m_logColumns.m_path]))
            {
                Gtk::TreeModel::iterator conv = row->parent()->parent()->parent(); //Conversation
                listFilesChildItems(*conv);
                m_logs->get_selection()->select(conv);
            }
            else
            {
                searchInFile(true);
            }
        }
    }
}

void LogDialog::listFilesChildItems(Gtk::TreeModel::Row parent)
{
    if(parent[m_logColumns.m_type]==Conversation)
    {
        std::vector<Glib::ustring> olditems = getLogFiles(parent);
        std::vector<Glib::ustring> files;
        Glib::ustring path = parent[m_logColumns.m_path];
        GStatBuf info;
        g_stat(path.c_str(), &info);
        if(parent[m_logColumns.m_time]!=info.st_mtime)
        {
            Glib::Dir dir(path);
            std::string name = dir.read_name();
            while(!name.empty())
            {
                if(name[0] != '.')
                {
                    std::vector<Glib::ustring>::iterator it = std::find(olditems.begin(), olditems.end(), name);
                    if(it == olditems.end())
                    {
                        Glib::ustring childPath = Glib::build_filename(path, name);
                        if(Glib::file_test(childPath, Glib::FILE_TEST_EXISTS)
                            && !Glib::file_test(childPath, Glib::FILE_TEST_IS_DIR)
                            && dxutils::isRightLogFile(childPath, name))
                        {
                            files.push_back(name);
                        }
                    }
                    else
                    {
                        olditems.erase(it);
                    }
                }
                name = dir.read_name();
            }
            // Wipe items remaining in oldlist
            FOREACH(std::vector<Glib::ustring>, olditems, it)
            {
                Gtk::TreeModel::Children years = parent->children();
                for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
                {
                    Gtk::TreeModel::Children months = (*yearIt)->children();
                    for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
                    {
                        Gtk::TreeModel::Children files = (*monthIt)->children();
                        for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
                        {
                            if(*it == getLogFileName(*fileIt))
                            {
                                m_logModel->erase(fileIt);
                                break;
                            }
                        }
                        if(!(*monthIt)->children().size())
                        {
                            m_logModel->erase(monthIt);
                            break;
                        }
                    }
                    if(!(*yearIt)->children().size())
                    {
                        m_logModel->erase(yearIt);
                        break;
                    }
                }
            }
            if(files.size())
            {
                Glib::ustring lastYear = dxutils::stringSection(files[0], '-', 0);
                int lastMonth = s2t<gint>(dxutils::stringSection(files[0], '-', 1));
                Gtk::TreeModel::Row year = getYearItem(parent, files[0]);
                if(!bool(year))
                {
                    year = *(m_logModel->append(parent.children()));
                    year[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
                    year[m_logColumns.m_type] = Year;
                    year[m_logColumns.m_name] = lastYear;
                    year[m_logColumns.m_sortID] = lastYear;
                    year[m_logColumns.m_path] = path;
                    year[m_logColumns.m_time] = 0;
                    year[m_logColumns.m_hidden] = false;
                }
                Gtk::TreeModel::Row month = getMonthItem(parent, files[0]);
                if(!bool(month))
                {
                    month = *(m_logModel->append(year.children()));
                    month[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
                    month[m_logColumns.m_type] = Month;
                    month[m_logColumns.m_name] = dxutils::getMonth(lastMonth);
                    month[m_logColumns.m_sortID] = dxutils::stringSection(files[0], '-', 1);
                    month[m_logColumns.m_path] = path;
                    month[m_logColumns.m_time] = 0;
                    month[m_logColumns.m_hidden] = false;
                }
                for(size_t i=0; i<files.size(); i++)
                {
                    if(lastYear!=dxutils::stringSection(files[i], '-', 0))
                    {
                        lastYear = dxutils::stringSection(files[i], '-', 0);
                        lastMonth = s2t<gint>(dxutils::stringSection(files[i], '-', 1));
                        year = getYearItem(parent, files[i]);
                        if(!bool(year))
                        {
                            year = *(m_logModel->append(parent.children()));
                            year[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
                            year[m_logColumns.m_type] = Year;
                            year[m_logColumns.m_name] = lastYear;
                            year[m_logColumns.m_sortID] = lastYear;
                            year[m_logColumns.m_path] = path;
                            year[m_logColumns.m_time] = 0;
                            year[m_logColumns.m_hidden] = false;
                        }
                        month = getMonthItem(parent, files[i]);
                        if(!bool(month))
                        {
                            month = *(m_logModel->append(year.children()));
                            month[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
                            month[m_logColumns.m_type] = Month;
                            month[m_logColumns.m_name] = dxutils::getMonth(lastMonth);
                            month[m_logColumns.m_sortID] = dxutils::stringSection(files[i], '-', 1);
                            month[m_logColumns.m_path] = path;
                            month[m_logColumns.m_time] = 0;
                            month[m_logColumns.m_hidden] = false;
                        }
                        Gtk::TreeModel::Row day = *(m_logModel->append(month.children()));
                        day[m_logColumns.m_icon] = ICO_FILE->get_pixbuf();
                        day[m_logColumns.m_type] = File;
                        day[m_logColumns.m_name] = dxutils::stringSection(files[i], '-', 2);
                        day[m_logColumns.m_sortID] = dxutils::stringSection(files[i], '-', 2);
                        day[m_logColumns.m_path] = Glib::build_filename(path, files[i]);
                        day[m_logColumns.m_time] = 0;
                        day[m_logColumns.m_hidden] = false;
                    }
                    else if(lastMonth!=s2t<gint>(dxutils::stringSection(files[i], '-', 1)))
                    {
                        lastMonth = s2t<gint>(dxutils::stringSection(files[i], '-', 1));
                        month = getMonthItem(parent, files[i]);
                        if(!bool(month))
                        {
                            month = *(m_logModel->append(year.children()));
                            month[m_logColumns.m_icon] = ICO_FOLDER->get_pixbuf();
                            month[m_logColumns.m_type] = Month;
                            month[m_logColumns.m_name] = dxutils::getMonth(lastMonth);
                            month[m_logColumns.m_sortID] = dxutils::stringSection(files[i], '-', 1);
                            month[m_logColumns.m_path] = path;
                            month[m_logColumns.m_time] = 0;
                            month[m_logColumns.m_hidden] = false;
                        }
                        Gtk::TreeModel::Row day = *(m_logModel->append(month.children()));
                        day[m_logColumns.m_icon] = ICO_FILE->get_pixbuf();
                        day[m_logColumns.m_type] = File;
                        day[m_logColumns.m_name] = dxutils::stringSection(files[i], '-', 2);
                        day[m_logColumns.m_sortID] = dxutils::stringSection(files[i], '-', 2);
                        day[m_logColumns.m_path] = Glib::build_filename(path, files[i]);
                        day[m_logColumns.m_time] = 0;
                        day[m_logColumns.m_hidden] = false;
                    }
                    else
                    {
                        Gtk::TreeModel::Row day = *(m_logModel->append(month.children()));
                        day[m_logColumns.m_icon] = ICO_FILE->get_pixbuf();
                        day[m_logColumns.m_type] = File;
                        day[m_logColumns.m_name] = dxutils::stringSection(files[i], '-', 2);
                        day[m_logColumns.m_sortID] = dxutils::stringSection(files[i], '-', 2);
                        day[m_logColumns.m_path] = Glib::build_filename(path, files[i]);
                        day[m_logColumns.m_time] = 0;
                        day[m_logColumns.m_hidden] = false;
                    }
                }
            }
            parent[m_logColumns.m_time] = info.st_mtime;
        }
    }
}


//Load file
bool LogDialog::loadFile(const Glib::ustring& path)
{
    clearText();
    std::ifstream file(path.c_str());
    if(!file.is_open())
    {
        Gtk::MessageDialog msg(*this, _("Error Loading File"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK_CANCEL);
        msg.set_secondary_text(Glib::ustring::compose(_("Unable to open file: %1"), path));
        msg.run();
        return false;
    }
    std::string buf;
    while(getline(file, buf))
    {
        appendLogLine(buf);
    }
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    m_textMark = buffer->create_mark(buffer->begin());
    file.close();
    return true;
}

void LogDialog::appendLogLine(const Glib::ustring &line)
{
    Glib::ustring::size_type pos = line.find(']')+2;
    if(pos<2) pos = 0;
    if(line.length()>pos+1 && line[pos]=='\014') //log style mark
    {
        Glib::ustring style = dxutils::stringSection(line, '\014', 1);
        if(dxutils::stringSection(line, '\014', 2, true)[0]=='<') //has nick
        {
            Glib::ustring nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
            Glib::ustring msg = dxutils::stringSection(line, '>',1, true);
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
            buffer->insert(buffer->end(), dxutils::stringSection(line, '\014', 0));
            buffer->insert_with_tag(buffer->end(), Glib::ustring::compose("%1%2", nick, DXPREFERENCES.m_nickCompletionChar), DXPREFERENCES.m_coloredNick?dxutils::getNickColor(nick):"bold");
            parseHtml(msg, style.c_str());
        }
        else
        {
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
            buffer->insert(buffer->end(), dxutils::stringSection(line, '\014', 0));
            parseHtml(dxutils::stringSection(line, '\014', 2, true), style.c_str());
        }
    }
    else
    {
        if(line.length()>pos+1 && line[pos]=='<') //line with nick
        {
            Glib::ustring nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
            Glib::ustring msg = dxutils::stringSection(line, '>',1, true);
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
            buffer->insert(buffer->end(), dxutils::stringSection(line, '<', 0));
            buffer->insert_with_tag(buffer->end(), Glib::ustring::compose("%1%2", nick, DXPREFERENCES.m_nickCompletionChar), DXPREFERENCES.m_coloredNick?dxutils::getNickColor(nick):"bold");
            parseHtml(msg, "");
        }
        else
        {
            parseHtml(line, "");
        }
    }
}

std::vector<Glib::ustring> LogDialog::getLogFiles(Gtk::TreeModel::Row parent)
{
    std::vector<Glib::ustring> v;
    LogItemType type = parent[m_logColumns.m_type];
    if(type == Conversation)
    {
        Gtk::TreeModel::Children years = parent->children();
        for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
        {
            Gtk::TreeModel::Children months = (*yearIt)->children();
            for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
            {
                Gtk::TreeModel::Children files = (*monthIt)->children();
                for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
                {
                    v.push_back(getLogFileName(*fileIt));
                }
            }
        }
    }
    else if(type == Year)
    {
        Gtk::TreeModel::Children months = parent->children();
        for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
        {
            Gtk::TreeModel::Children files = (*monthIt)->children();
            for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
            {
                v.push_back(getLogFileName(*fileIt));
            }
        }
    }
    else if(type == Month)
    {
        Gtk::TreeModel::Children files = parent->children();
        for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
        {
            v.push_back(getLogFileName(*fileIt));
        }
    }
    else
    {
        v.push_back(getLogFileName(parent));
    }
    return v;
}

Glib::ustring LogDialog::getLogFileName(Gtk::TreeModel::Row file)
{
    if(file[m_logColumns.m_type]==File)
    {
        Glib::ustring name = (*file->parent()->parent())[m_logColumns.m_name]; //Year item
        name.append("-");
        name.append(dxutils::getMonthNumber((*file->parent())[m_logColumns.m_name])); //month item
        name.append("-");
        name.append(file[m_logColumns.m_name]);
        return name;
    }
    return "";
}

Gtk::TreeModel::Row LogDialog::getYearItem(Gtk::TreeModel::Row conversation, const Glib::ustring& filename)
{
    Glib::ustring yearName = filename.substr(0, filename.find_first_of('-'));
    Gtk::TreeModel::Children years = conversation->children();
    for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
    {
        if((*yearIt)[m_logColumns.m_name] == yearName)
            return *yearIt;
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row LogDialog::getMonthItem(Gtk::TreeModel::Row conversation, const Glib::ustring& filename)
{
    Glib::ustring yearName = filename.substr(0, filename.find_first_of('-'));
    Glib::ustring monthName = dxutils::getMonth(s2t<gint>(filename.substr(filename.find_first_of('-')+1, filename.find_first_of('-', filename.find_first_of('-')))));
    Gtk::TreeModel::Children years = conversation->children();
    for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
    {
        if((*yearIt)[m_logColumns.m_name] == yearName)
        {
            Gtk::TreeModel::Children months = (*yearIt)->children();
            for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
            {
                if((*monthIt)[m_logColumns.m_name] == monthName)
                    return *monthIt;
            }
        }
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row LogDialog::getConversationItem()
{
    Gtk::TreeModel::iterator it = m_logModelFilter->convert_iter_to_child_iter(m_logs->get_selection()->get_selected());
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        LogItemType type = row[m_logColumns.m_type];
        if(type == Conversation)
            return row;
        if(type == Year)
            return *row->parent();
        if(type == Month)
            return *row->parent()->parent();
        if(type == File)
            return *row->parent()->parent()->parent();
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row LogDialog::getAccountItem()
{
    Gtk::TreeModel::iterator it = m_logModelFilter->convert_iter_to_child_iter(m_logs->get_selection()->get_selected());
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        LogItemType type = row[m_logColumns.m_type];
        if(type == Account)
            return row;
        if(type == Conversation)
            return *row->parent();
        if(type == Year)
            return *row->parent()->parent();
        if(type == Month)
            return *row->parent()->parent()->parent();
        if(type == File)
            return *row->parent()->parent()->parent()->parent();
    }
    return Gtk::TreeModel::Row();
}

void LogDialog::searchInFile(bool next)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    if(m_searchString.empty() || !buffer->get_line_count())
        return;
    bool founded = false;
    if(!next)
    {
        m_textMark = buffer->create_mark(buffer->begin());
    }
    Gtk::TextIter it = buffer->get_iter_at_mark(m_textMark);
    Gtk::TextIter start, end;
    founded = it.forward_search(m_searchString, Gtk::TEXT_SEARCH_VISIBLE_ONLY, start, end);
    if(founded)
    {
        buffer->remove_tag_by_name("selected", buffer->begin(), buffer->end());
        buffer->apply_tag_by_name("selected", start, end);
        m_textMark = buffer->create_mark(end);
        m_text->scroll_to(m_textMark);
    }
    else
    {
        if(!next)
        {
            Gtk::MessageDialog msg(*this, _("Information"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
            msg.set_secondary_text(Glib::ustring::compose(_("'%1' wasn't found"), m_searchString));
            msg.run();
        }
        else
        {
            m_textMark = buffer->create_mark(buffer->begin());
            searchInFile(false);
        }
    }
}

void LogDialog::searchInConversation()
{
    clearText();
    appendText(_("Search started..."));
    Gtk::TreeModel::Row conv = getConversationItem();
    if(bool(conv))
    {
        int count = 0;
        Gtk::TreeModel::Children years = conv->children();
        for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
        {
            Gtk::TreeModel::Children months = (*yearIt)->children();
            for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
            {
                Gtk::TreeModel::Children files = (*monthIt)->children();
                for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
                {
                    Glib::ustring path = (*fileIt)[m_logColumns.m_path];
                    std::ifstream stream(path.c_str());
                    std::string temp;
                    bool contain = false;
                    while(std::getline(stream, temp))
                    {
                        if(temp.find(m_searchString) != std::string::npos)
                        {
                            contain = true;
                            count++;
                            break;
                        }
                    }
                    if(!contain) (*fileIt)[m_logColumns.m_hidden] = true;
                    stream.close();
                }
                if(hideRow(*monthIt)) (*monthIt)[m_logColumns.m_hidden] = true;
            }
            if(hideRow(*yearIt)) (*yearIt)[m_logColumns.m_hidden] = true;
        }
        if(!count) onReset();
        appendText(Glib::ustring::compose(_n("Founded %1 file", "Founded %1 files", count), count));
    }
    appendText(_("Search finished"));
}

void LogDialog::searchInAccount()
{
    clearText();
    appendText(_("Search started..."));
    Gtk::TreeModel::Row acc = getAccountItem();
    if(bool(acc))
    {
        int count = 0;
        Gtk::TreeModel::Children convs = acc->children();
        for(Gtk::TreeModel::Children::iterator convIt = convs.begin(); convIt != convs.end(); ++convIt)
        {
            Gtk::TreeModel::Children years = convIt->children();
            if(!years.size())
            {
                listFilesChildItems((*convIt));
                years = convIt->children();
            }
            for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
            {
                Gtk::TreeModel::Children months = (*yearIt)->children();
                for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
                {
                    Gtk::TreeModel::Children files = (*monthIt)->children();
                    for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
                    {
                        Glib::ustring path = (*fileIt)[m_logColumns.m_path];
                        std::ifstream stream(path.c_str());
                        std::string temp;
                        bool contain = false;
                        while(std::getline(stream, temp))
                        {
                            if(temp.find(m_searchString) != std::string::npos)
                            {
                                contain = true;
                                count++;
                                break;
                            }
                        }
                        if(!contain) (*fileIt)[m_logColumns.m_hidden] = true;
                        stream.close();
                    }
                    if(hideRow(*monthIt)) (*monthIt)[m_logColumns.m_hidden] = true;
                }
                if(hideRow(*yearIt)) (*yearIt)[m_logColumns.m_hidden] = true;
            }
            if(hideRow(*convIt)) (*convIt)[m_logColumns.m_hidden] = true;
        }
        if(!count) onReset();
        appendText(Glib::ustring::compose(_n("Founded %1 file", "Founded %1 files", count), count));
    }
    appendText(_("Search finished"));
}

void LogDialog::searchInAll()
{
    clearText();
    appendText(_("Search started..."));
    Gtk::TreeModel::Children roots = m_logModel->children();
    if(!m_treeLoaded) loadFullTree();
    int count = 0;
    for(Gtk::TreeModel::Children::iterator rootIt = roots.begin(); rootIt != roots.end(); ++rootIt)
    {
        Gtk::TreeModel::Children accs = (*rootIt)->children();
        for(Gtk::TreeModel::Children::iterator accIt = accs.begin(); accIt != accs.end(); ++accIt)
        {
            Gtk::TreeModel::Children convs = (*accIt)->children();
            for(Gtk::TreeModel::Children::iterator convIt = convs.begin(); convIt != convs.end(); ++convIt)
            {
                Gtk::TreeModel::Children years = convIt->children();
                for(Gtk::TreeModel::Children::iterator yearIt = years.begin(); yearIt != years.end(); ++yearIt)
                {
                    Gtk::TreeModel::Children months = (*yearIt)->children();
                    for(Gtk::TreeModel::Children::iterator monthIt = months.begin(); monthIt != months.end(); ++monthIt)
                    {
                        Gtk::TreeModel::Children files = (*monthIt)->children();
                        for(Gtk::TreeModel::Children::iterator fileIt = files.begin(); fileIt != files.end(); ++fileIt)
                        {
                            Glib::ustring path = (*fileIt)[m_logColumns.m_path];
                            std::ifstream stream(path.c_str());
                            std::string temp;
                            bool contain = false;
                            while(std::getline(stream, temp))
                            {
                                if(temp.find(m_searchString) != std::string::npos)
                                {
                                    contain = true;
                                    count++;
                                    break;
                                }
                            }
                            if(!contain) (*fileIt)[m_logColumns.m_hidden] = true;
                            stream.close();
                        }
                        if(hideRow(*monthIt)) (*monthIt)[m_logColumns.m_hidden] = true;
                    }
                    if(hideRow(*yearIt)) (*yearIt)[m_logColumns.m_hidden] = true;
                }
                if(hideRow(*convIt)) (*convIt)[m_logColumns.m_hidden] = true;
            }
            if(hideRow(*accIt)) (*accIt)[m_logColumns.m_hidden] = true;
        }
    }
    if(!count) onReset();
    appendText(Glib::ustring::compose(_n("Founded %1 file", "Founded %1 files", count), count));
    appendText(_("Search finished"));
}

void LogDialog::clearText()
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    buffer->erase(buffer->begin(), buffer->end());
    updatePreferences();
}

void LogDialog::updatePreferences()
{
#if GTKMM_MAJOR_VERSION == 2
    m_text->modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_text->modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
#else
    m_text->override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text->override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
#endif
    Glib::RefPtr<Gtk::TextTag> tag = m_text->get_buffer()->get_tag_table()->lookup("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = m_text->get_buffer()->get_tag_table()->lookup("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = m_text->get_buffer()->get_tag_table()->lookup("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = m_text->get_buffer()->get_tag_table()->lookup("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = m_text->get_buffer()->get_tag_table()->lookup("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
}

void LogDialog::createWidgets()
{
    m_treeLoaded = false;
    m_searchString = "";

    Gtk::ScrolledWindow *textwin = Gtk::manage(new Gtk::ScrolledWindow());
    m_text = new Gtk::TextView();
    m_text->set_editable(false);
    m_text->set_can_focus(false);
    m_text->set_wrap_mode(Gtk::WRAP_WORD_CHAR);
#if GTKMM_MAJOR_VERSION == 2
    m_text->modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_text->modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_text->modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_text->override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text->override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text->override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    m_text->set_size_request(500, -1);
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    Glib::RefPtr<Gtk::TextTag> tag;
    tag = buffer->create_tag("nick0");
    tag->property_foreground().set_value("#C4A000");
    tag = buffer->create_tag("nick1");
    tag->property_foreground().set_value("#CE5C00");
    tag = buffer->create_tag("nick2");
    tag->property_foreground().set_value("#8F5902");
    tag = buffer->create_tag("nick3");
    tag->property_foreground().set_value("#4E9A06");
    tag = buffer->create_tag("nick4");
    tag->property_foreground().set_value("#204A87");
    tag = buffer->create_tag("nick5");
    tag->property_foreground().set_value("#75507B");
    tag = buffer->create_tag("nick6");
    tag->property_foreground().set_value("#A40000");
    tag = buffer->create_tag("nick7");
    tag->property_foreground().set_value("#555753");
    tag = buffer->create_tag("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
    tag = buffer->create_tag("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = buffer->create_tag("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = buffer->create_tag("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = buffer->create_tag("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = buffer->create_tag("selected");
    tag->property_background().set_value(createSelectedBackroundColor().to_string());
    tag = buffer->create_tag("bold");
    tag->property_weight().set_value(Pango::WEIGHT_BOLD);
    textwin->add(*m_text);
    textwin->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    m_splitter.pack1(*textwin, true, true);
    Gtk::VBox *searchBox = Gtk::manage(new Gtk::VBox());
    Gtk::HBox *searchBtnBox = Gtk::manage(new Gtk::HBox());
    m_searchLine = new Gtk::Entry();
    searchBtnBox->pack_start(*m_searchLine, Gtk::PACK_EXPAND_WIDGET, 3);
    m_buttonSearch = new Gtk::Button(_("_Search"), true);
    m_buttonSearch->signal_clicked().connect(sigc::mem_fun(this, &LogDialog::onSearch));
    searchBtnBox->pack_start(*m_buttonSearch, Gtk::PACK_SHRINK, 3);
    m_buttonReset = new Gtk::Button(_("_Reset"), true);
    m_buttonReset->signal_clicked().connect(sigc::mem_fun(this, &LogDialog::onReset));
    searchBtnBox->pack_start(*m_buttonReset, Gtk::PACK_SHRINK, 3);
    searchBox->pack_start(*searchBtnBox, Gtk::PACK_SHRINK);
    Gtk::Frame *group = Gtk::manage(new Gtk::Frame(_("Search options")));
    Gtk::VBox *searchOptions = Gtk::manage(new Gtk::VBox());
    Gtk::RadioButton::Group radioGroup;
    m_buttonFile = new Gtk::RadioButton(radioGroup, _("Search file"));
    m_buttonFile->set_sensitive(false);
    searchOptions->pack_start(*m_buttonFile, Gtk::PACK_SHRINK, 2);
    m_buttonConv = new Gtk::RadioButton(radioGroup, _("Search im/chat"));
    m_buttonConv->set_sensitive(false);
    searchOptions->pack_start(*m_buttonConv, Gtk::PACK_SHRINK, 2);
    m_buttonAccount = new Gtk::RadioButton(radioGroup, _("Search account"));
    m_buttonAccount->set_sensitive(false);
    searchOptions->pack_start(*m_buttonAccount, Gtk::PACK_SHRINK, 2);
    m_buttonAll = new Gtk::RadioButton(radioGroup, _("Search all"));
    m_buttonAll->set_active(true);
    searchOptions->pack_start(*m_buttonAll, Gtk::PACK_SHRINK, 2);
    group->add(*searchOptions);
    searchBox->pack_start(*group, Gtk::PACK_SHRINK);
    Gtk::ScrolledWindow *treewin = Gtk::manage(new Gtk::ScrolledWindow());
    treewin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    treewin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_logModel = Gtk::TreeStore::create(m_logColumns);
    m_logModel->set_sort_column(m_logColumns.m_sortID, Gtk::SORT_ASCENDING);
    m_logModelFilter = Gtk::TreeModelFilter::create(m_logModel);
    m_logModelFilter->set_visible_func(sigc::mem_fun(this, &LogDialog::filterLog));
    m_logs = new Gtk::TreeView();
    m_logs->set_model(m_logModelFilter);
    m_logs->append_column("", m_logColumns.m_icon);
    m_logs->append_column("name", m_logColumns.m_name);
    m_logs->set_headers_visible(false);
    m_logs->set_size_request(-1, 200);
    m_logs->get_selection()->signal_changed().connect(sigc::mem_fun(this, &LogDialog::onItemChanged));
    m_logs->signal_button_press_event().connect(sigc::mem_fun(this, &LogDialog::onButtonPressed), false);
    treewin->add(*m_logs);
    searchBox->pack_start(*treewin, Gtk::PACK_EXPAND_WIDGET, 3);
    m_splitter.pack2(*searchBox, true, true);
    get_vbox()->pack_start(m_splitter);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    Gtk::Button *ok = new Gtk::Button(Gtk::Stock::OK);
    ok->signal_clicked().connect(sigc::mem_fun(this, &LogDialog::onOK));
    btnbox->pack_end(*ok, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
}

void LogDialog::appendText(const Glib::ustring &text)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    buffer->insert(buffer->end(), text);
    buffer->insert(buffer->end(), "\n");
}

bool LogDialog::filterLog(const Gtk::TreeModel::const_iterator &iter)
{
    if(iter)
    {
        bool hidden;
        iter->get_value(6, hidden);
        if(hidden) return false;
    }
    return true;
}

bool LogDialog::hideRow(Gtk::TreeModel::Row row)
{
    size_t hidden = 0;
    Gtk::TreeModel::Children children = row->children();
    for(Gtk::TreeModel::Children::iterator it = children.begin(); it != children.end(); ++it)
    {
        if((*it)[m_logColumns.m_hidden]) hidden++;
    }
    if(hidden == children.size()) return true;
    return false;
}

dxColor LogDialog::createSelectedBackroundColor()
{
    dxColor back("#BFBFBF");
    if(back == DXPREFERENCES.m_backroundColor)
    {
        back.set("#00F");
    }
    return back;
}

void LogDialog::appendText(const Glib::ustring &text, bool bold, bool underline, bool italic, dxColor fore, dxColor back)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    std::vector< Glib::RefPtr<Gtk::TextTag> > tags;
    Glib::RefPtr<Gtk::TextTag> tag;
    if(bold)
    {
        tag = buffer->get_tag_table()->lookup("bold");
        if(!tag)
        {
            tag = buffer->create_tag("bold");
            tag->property_weight().set_value(Pango::WEIGHT_BOLD);
        }
        tags.push_back(tag);
    }
    if(underline)
    {
        tag = buffer->get_tag_table()->lookup("underline");
        if(!tag)
        {
            tag = buffer->create_tag("underline");
            tag->property_underline().set_value(Pango::UNDERLINE_SINGLE);
        }
        tags.push_back(tag);
    }
    if(italic)
    {
        tag = buffer->get_tag_table()->lookup("italic");
        if(!tag)
        {
            tag = buffer->create_tag("italic");
            tag->property_style().set_value(Pango::STYLE_ITALIC);
        }
        tags.push_back(tag);
    }
    if(fore != DXPREFERENCES.m_textColor)
    {
        tag = buffer->get_tag_table()->lookup(Glib::ustring::compose("fore%1", fore.to_string()));
        if(!tag)
        {
            tag = buffer->create_tag(Glib::ustring::compose("fore%1", fore.to_string()));
            tag->property_foreground().set_value(fore.to_string());
        }
        tags.push_back(tag);
    }
    if(back != DXPREFERENCES.m_backroundColor)
    {
        tag = buffer->get_tag_table()->lookup(Glib::ustring::compose("back%1", back.to_string()));
        if(!tag)
        {
            tag = buffer->create_tag(Glib::ustring::compose("back%1", back.to_string()));
            tag->property_foreground().set_value(back.to_string());
        }
        tags.push_back(tag);
    }
    buffer->insert_with_tags(buffer->end(), text, tags);
}

void LogDialog::parseHtml(Glib::ustring text, const Glib::ustring &style)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
    size_t i = 0;
    size_t tagLength = 0;
    bool bold = false;
    bool under = false;
    bool italic = false;
    Glib::RefPtr<Gtk::TextTag> tagStyle = buffer->get_tag_table()->lookup(style);
    dxColor foreColor, backColor;
    if(!style.empty() && tagStyle)
    {
#if GTKMM_MAJOR_VERSION == 2
        foreColor = dxColor(tagStyle->property_foreground_gdk());
#else
        foreColor = dxColor(tagStyle->property_foreground_rgba());
#endif
    }
    else
    {
        foreColor = DXPREFERENCES.m_textColor;
    }
    backColor = DXPREFERENCES.m_backroundColor;
    Glib::ustring normalText = "";
    Glib::ustring tag = "";
    std::list<guchar> span; //0 nothing, 1 bold, 2 under, 3 fore, 4 back
    size_t length = text.length();
    while(i<length)
    {
        if(text[i]=='<')
        {
            tagLength = 1;
            for(size_t j=i+1; j<length; j++)
            {
                if(text[j]=='>')
                {
                    tagLength++;
                    break;
                }
                else if(text[j]=='<')
                {
                    break;
                }
                tagLength++;
            }
            tag = text.substr(i, tagLength);
            dxutils::debugLine(text);
            dxutils::debugLine(tag);
            if(dxutils::stringContains(tag, "<br")) //tag which will replaced by PURPLE::stripHtml
            {
                normalText.append(tag);
                i += tagLength;
            }
            else
            {
                if(dxutils::stringContains(tag, "<b>") || dxutils::stringContains(tag, "</b>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    bold = !bold;
                }
                else if(dxutils::stringContains(tag, "<u>") || dxutils::stringContains(tag, "</u>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    under = !under;
                }
                else if(dxutils::stringContains(tag, "<i>") || dxutils::stringContains(tag, "</i>")
                    || dxutils::stringContains(tag, "<em>") || dxutils::stringContains(tag, "</em>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    italic = !italic;
                }
                else if(dxutils::stringContains(tag, "<img"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text->get_buffer();
                    buffer->insert_pixbuf(buffer->end(), createPixbuf(dxutils::stringSection(tag,'\'',1,false)));
                }
                else if(dxutils::stringContains(tag, "<span"))
                {
                    std::vector<Glib::ustring> items = dxutils::stringSplit(tag, '=');
                    FOREACH(std::vector<Glib::ustring>, items, it)
                    {
                        if(dxutils::stringContains((*it), "'font-weight: bold"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            bold = !bold;
                            span.push_back(1);
                        }
                        else if(dxutils::stringContains((*it), "'text-decoration: underline"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            under = !under;
                            span.push_back(2);
                        }
                        else if(dxutils::stringContains((*it), "'color"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection(dxutils::stringSection((*it), ' ', 1, false), ';', 0, false);
                            foreColor = dxColor(color);
                            span.push_back(3);
                        }
                        else if(dxutils::stringContains((*it), "'background"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection(dxutils::stringSection((*it), ' ', 1, false), ';', 0, false);
                            backColor = dxColor(color);
                            span.push_back(4);
                        }
                    }
                }
                else if(dxutils::stringContains(tag, "</span>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    switch(span.back()) {
                        case 1: bold = !bold; break;
                        case 2: under = !under; break;
                        case 3: foreColor = DXPREFERENCES.m_textColor; break;
                        case 4: backColor = DXPREFERENCES.m_backroundColor; break;
                        default: break;
                    }
                    span.pop_back();
                }
                else if(dxutils::stringContains(tag, "<font"))
                {
                    std::vector<Glib::ustring> items = dxutils::stringSplit(tag, '=');
                    FOREACH(std::vector<Glib::ustring>, items, it)
                    {
                        dxutils::debugLine(*it);
                        if(dxutils::stringContains((*it), "color"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection((*++it), '\"', 0, false);
                            foreColor = dxColor(color);
                        }
                        if(dxutils::stringContains((*it), "back"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection((*++it), '\"', 0, false);
                            backColor = dxColor(color);
                        }
                    }
                }
                else if(dxutils::stringContains(tag, "</font>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    foreColor = DXPREFERENCES.m_textColor;
                    backColor = DXPREFERENCES.m_backroundColor;
                }
                text.erase(i, tagLength);
                length = length-tagLength;
            }
        }
        else
        {
            normalText.append(1, text[i]);
            i++;
        }
    }
    if(!normalText.empty())
    {
        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
    }
    buffer->insert(buffer->end(), "\n");
}

bool LogDialog::onButtonPressed(GdkEventButton *event)
{
    bool value = false;
    if(event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
        Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_logs->get_selection();
        if(refSelection)
        {
            Gtk::TreeModel::Path path;
            m_logs->get_path_at_pos(event->x, event->y, path);
            refSelection->select(path);
            Gtk::TreeModel::iterator iter = refSelection->get_selected();
            if(iter)
            {
                LogItemType type = (*iter)[m_logColumns.m_type];
                if(type != Root)
                {
                    if(m_fullLog)
                    {
                        Gtk::Menu *popup = new Gtk::Menu();
                        Gtk::MenuItem* item = new Gtk::MenuItem(_("Delete"));
                        item->signal_activate().connect(sigc::mem_fun(this, &LogDialog::onPopupDelete));
                        popup->append(*item);
                        popup->accelerate(*this);
                        popup->show_all();
                        popup->popup(event->button, event->time);
                        value = true;
                    }
                }
            }
        }
    }
    return value;
}

void LogDialog::onPopupDelete()
{
    Gtk::TreeModel::iterator it = m_logModelFilter->convert_iter_to_child_iter(m_logs->get_selection()->get_selected());
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        LogItemType type = row[m_logColumns.m_type];
        Glib::ustring message;
        if(type == File)
        {
            message = Glib::ustring::compose(_("Delete file %1?\nThis cann't be UNDONE!"), getLogFileName(row));
        }
        else
        {
            message = Glib::ustring::compose(_("Delete %1 with all child items?\nThis cann't be UNDONE!"), row[m_logColumns.m_name]);
        }
        Gtk::MessageDialog msg(*this, _("Question"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
        msg.set_secondary_text(message);
        int result = msg.run();
        if(result == Gtk::RESPONSE_YES)
        {
            Glib::ustring path = row[m_logColumns.m_path];
            if(type == File)
            {
                std::remove(path.c_str());
                listFilesChildItems(*row->parent()->parent()->parent());
            }
            else if(type == Month || type == Year)
            {
                std::vector<Glib::ustring> files = getLogFiles(row);
                FOREACH(std::vector<Glib::ustring>, files, file)
                    std::remove(Glib::build_filename(path,*file).c_str());
                if(type == Month)
                    listFilesChildItems(*row->parent()->parent());
                else
                    listFilesChildItems(*row->parent());
            }
            else
            {
                dxutils::removeDir(path);
                listDirsChildItems(*row->parent());
            }
        }
    }
}
