/*
 *      logdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef LOGDIALOG_H
#define LOGDIALOG_H

#include <gtkmm.h>
#include "widgets/dxcolor.h"
#include <purplemm.h>

class LogDialog : public Gtk::Dialog {
public:
    LogDialog(Gtk::Window& parent);
    LogDialog(PURPLE::Account account);
    LogDialog(PURPLE::Buddy buddy);
    LogDialog(PURPLE::Chat chat);
    virtual ~LogDialog();

    void setLogPath(const Glib::ustring &path);
    void updatePreferences();
private:
    enum LogItemType {
        Root = 0,
        Account,
        Conversation,
        Year,
        Month,
        File
    };
    Gtk::HPaned m_splitter;
    struct Columns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_icon;
        Gtk::TreeModelColumn<Glib::ustring> m_name;
        Gtk::TreeModelColumn<Glib::ustring> m_sortID;
        Gtk::TreeModelColumn<LogItemType> m_type;
        Gtk::TreeModelColumn<Glib::ustring> m_path;
        Gtk::TreeModelColumn<time_t> m_time;
        Gtk::TreeModelColumn<bool> m_hidden;

        Columns() { add(m_icon); add(m_name); add(m_sortID); add(m_type), add(m_path); add(m_time); add(m_hidden); }
    };
    const Columns m_logColumns;
    Glib::RefPtr<Gtk::TreeStore> m_logModel;
    Glib::RefPtr<Gtk::TreeModelFilter> m_logModelFilter;
    Gtk::TreeView *m_logs;
    Gtk::TextView *m_text;
    Gtk::Button *m_buttonSearch, *m_buttonReset;
    Gtk::Entry *m_searchLine;
    Gtk::RadioButton *m_buttonAll, *m_buttonAccount, *m_buttonConv, *m_buttonFile;
    bool m_treeLoaded;
    Glib::ustring m_searchString;
    Glib::RefPtr<Gtk::TextBuffer::Mark> m_textMark;
    bool m_fullLog;

    void createWidgets();
    //slots start
    void onOK();
    void onSearch();
    void onReset();
    void onItemChanged();
    virtual bool on_key_press_event(GdkEventKey *event);
    bool filterLog(const Gtk::TreeModel::const_iterator &iter);
    bool onButtonPressed(GdkEventButton *event);
    void onPopupDelete();
    //slots end
    void loadTree();
    void loadFullTree();
    void listDirsChildItems(Gtk::TreeModel::Row parent);
    void listFilesChildItems(Gtk::TreeModel::Row parent);
    bool loadFile(const Glib::ustring &path);
    std::vector<Glib::ustring> getLogFiles(Gtk::TreeModel::Row parent);
    Glib::ustring getLogFileName(Gtk::TreeModel::Row file);
    Gtk::TreeModel::Row getYearItem(Gtk::TreeModel::Row conversation, const Glib::ustring& filename);
    Gtk::TreeModel::Row getMonthItem(Gtk::TreeModel::Row conversation, const Glib::ustring& filename);
    Gtk::TreeModel::Row getConversationItem();
    Gtk::TreeModel::Row getAccountItem();
    void appendLogLine(const Glib::ustring &line);
    void searchInFile(bool next=false);
    void searchInConversation();
    void searchInAccount();
    void searchInAll();
    void clearText();
    void appendText(const Glib::ustring &text);
    void appendText(const Glib::ustring &text, bool bold, bool underline, bool italic, dxColor fore, dxColor back);
    void enableAllItems();
    bool hideRow(Gtk::TreeModel::Row row);
    dxColor createSelectedBackroundColor();
    void parseHtml(Glib::ustring text, const Glib::ustring &style);
};

#endif  /* LOGDIALOG_H */

