/*
 *      plugindialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "plugindialog.h"
#include "plugineditor.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "preferences.h"

PluginDialog::PluginDialog(Gtk::Window& parent, PURPLE::Purple* purple)
: Gtk::Dialog(_("Plugins list"), parent)
{
    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_pluginModel = Gtk::ListStore::create(m_pluginColumns);
    m_pluginsList.set_model(m_pluginModel);
    m_pluginsList.append_column(_("Plugin"), m_pluginColumns.m_pluginName);
    Gtk::CellRendererToggle *check = Gtk::manage(new Gtk::CellRendererToggle());
    check->signal_toggled().connect(sigc::mem_fun(this, &PluginDialog::onEnabledChanged));
    m_pluginsList.append_column(_("Enabled"), *check);
    Gtk::TreeViewColumn *eColumn = m_pluginsList.get_column(1);
    if(eColumn)
    {
        eColumn->add_attribute(check->property_active(), m_pluginColumns.m_enabled);
    }
    m_pluginSelectionModel = m_pluginsList.get_selection();
    m_pluginSelectionModel->set_mode(Gtk::SELECTION_SINGLE);
    m_plugins = purple->getPluginsList();
    if(m_plugins.size())
    {
        for(guint i=0; i<m_plugins.size(); i++)
        {
            createPlugin(m_plugins[i]);
        }
    }
    m_pluginSelectionModel->signal_changed().connect(sigc::mem_fun(this, &PluginDialog::onSelectionChanged));
    m_pluginsList.set_size_request(-1, 155);
    swin->add(m_pluginsList);
    get_vbox()->pack_start(*swin, Gtk::PACK_EXPAND_WIDGET);
    m_details = new Gtk::Expander(_("Details"));
    Gtk::Table *table = Gtk::manage(new Gtk::Table());
    table->attach(*Gtk::manage(new Gtk::Label(_("Summary:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_summary = new Gtk::TextView();
    m_summary->set_editable(false);
    m_summary->set_can_focus(false);
    m_summary->set_wrap_mode(Gtk::WRAP_WORD_CHAR);
    table->attach(*m_summary, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Version:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_version = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_version, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Description:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_description = new Gtk::TextView();
    m_description->set_editable(false);
    m_description->set_can_focus(false);
    m_description->set_wrap_mode(Gtk::WRAP_WORD_CHAR);
    table->attach(*m_description, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Author:"), Gtk::ALIGN_START)), 0, 1, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_author = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_author, 1, 2, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Path:"), Gtk::ALIGN_START)), 0, 1, 4, 5, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_path = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_path, 1, 2, 4, 5, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Homepage:"), Gtk::ALIGN_START)), 0, 1, 5, 6, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_homepage = new LinkLabel("");
    m_homepage->setColor(DXPREFERENCES.m_linkColor);
    table->attach(*m_homepage, 1, 2, 5, 6, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_details->add(*table);
    get_vbox()->pack_start(*m_details, Gtk::PACK_SHRINK);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    m_modify = Gtk::manage(new Gtk::Button(_("_Modify"), true));
    m_modify->signal_clicked().connect(sigc::mem_fun(this, &PluginDialog::modifyPlugin));
    m_modify->set_sensitive(false);
    Gtk::Button *okbutton = Gtk::manage(new Gtk::Button(Gtk::Stock::OK));
    okbutton->signal_clicked().connect(sigc::mem_fun(this, &PluginDialog::onOK));
    Gtk::Button *closebutton = Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
    closebutton->signal_clicked().connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*m_modify, Gtk::PACK_SHRINK);
    btnbox->pack_end(*okbutton, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebutton, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    m_pluginSelectionModel->select(Gtk::TreePath(true));
    show_all();
}

PluginDialog::~PluginDialog()
{
}

void PluginDialog::onOK()
{
    response(Gtk::RESPONSE_OK);
    hide();
}

void PluginDialog::onSelectionChanged()
{
    Gtk::TreeModel::iterator it = m_pluginsList.get_selection()->get_selected();
    if(it)
    {
        PURPLE::Plugin plugin = (*it).get_value(m_pluginColumns.m_plugin);
        if(plugin.isLoaded())
        {
            if(plugin.hasOptions())
                m_modify->set_sensitive(true);
            else
                m_modify->set_sensitive(false);
        }
        else
        {
            m_modify->set_sensitive(false);
        }
        Glib::RefPtr<Gtk::TextBuffer> buffer = m_summary->get_buffer();
        buffer->set_text(plugin.getSummary());
        m_version->set_text(plugin.getVersion());
        buffer = m_description->get_buffer();
        buffer->set_text(plugin.getDescription());
        m_author->set_text(plugin.getAuthor());
        m_path->set_text(plugin.getPath());
        m_homepage->setText(plugin.getHomepage());
    }
}

void PluginDialog::onEnabledChanged(const Glib::ustring &path_string)
{
    Gtk::TreeModel::iterator it = m_pluginModel->get_iter(path_string);
    Gtk::TreeModel::Row row = *it;
    PURPLE::Plugin plugin = row[m_pluginColumns.m_plugin];
    if(plugin.isLoadable())
    {
        if(row[m_pluginColumns.m_enabled] == true)
        {
            row[m_pluginColumns.m_enabled] = false;
            plugin.unload();
        }
        else
        {
            row[m_pluginColumns.m_enabled] = true;
            plugin.load();
        }
    }
    if(plugin.isLoaded())
    {
        if(plugin.hasOptions())
            m_modify->set_sensitive(true);
        else
            m_modify->set_sensitive(false);
    }
    else
    {
        m_modify->set_sensitive(false);
    }
}

void PluginDialog::modifyPlugin()
{
    Gtk::TreeModel::iterator it = m_pluginSelectionModel->get_selected();
    if(it)
    {
        PURPLE::Plugin plugin = (*it)[m_pluginColumns.m_plugin];
        if(plugin.hasOptions())
        {
            PluginEditor editor(*this, plugin);
            editor.run();
        }
    }
}

void PluginDialog::createPlugin(PURPLE::Plugin plugin)
{
    if(plugin.isValid())
    {
        Gtk::TreeModel::Row row = *(m_pluginModel->append());
        row[m_pluginColumns.m_pluginName] = plugin.getName();
        row[m_pluginColumns.m_enabled] = plugin.isLoaded();
        row[m_pluginColumns.m_plugin] = plugin;
    }
}


