/*
 *      roomlistdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ROOMLISTDIALOG_H
#define ROOMLISTDIALOG_H

#include <gtkmm.h>
#include "callback/callback.h"

class RoomListDialog : public Gtk::Dialog {
public:
    RoomListDialog(Gtk::Window& parent, PURPLE::RoomList *rlist, dxgCallbacks *cb);
    virtual ~RoomListDialog();

    sigc::signal<void, bool, Glib::ustring> signalClosed;
private:
    dxgCallbacks *m_cb;
    PURPLE::RoomList *m_rlist;
    Gtk::Button *m_select;
    class RoomColumns : public Gtk::TreeModel::ColumnRecord
    {
    public:
        Gtk::TreeModelColumn<Glib::ustring> m_chat;

        RoomColumns() { add(m_chat); }
    };
    RoomColumns m_roomColumns;
    Glib::RefPtr<Gtk::ListStore> m_roomModel;
    Glib::RefPtr<Gtk::TreeSelection> m_roomSelectionModel;
    Glib::RefPtr<Gtk::TreeModelFilter> m_roomModelFilter;
    Gtk::TreeView m_rooms;
    bool m_finished;
    Gtk::Entry *m_search;

    //slot start
    void onSelect();
    void onClose();
    void onColumns(PURPLE::RoomList *list, std::vector<std::string> columns);
    void onAddRoom(PURPLE::RoomList *list, std::vector<std::string> fields);
    void onFinished(PURPLE::RoomList *list);
    void onSearch();
    //slot end
    bool filterRooms(const Gtk::TreeModel::const_iterator &iter);
};

#endif  /* ROOMLISTDIALOG_H */

