/*
 *      selectaccountdialog.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "selectaccountdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

SelectAccountDialog::SelectAccountDialog(Gtk::Window& parent, PURPLE::Purple* purple)
: Gtk::Dialog(_("Select account"), parent)
{
    std::vector<PURPLE::Account> accounts = purple->getConnectedAccountsList();
    m_idModel = Gtk::TreeStore::create(m_idColumns);
    m_ids.set_model(m_idModel);
    for(guint i=0; i<accounts.size(); i++)
    {
        Gtk::TreeModel::Row row = *(m_idModel->append());
        row[m_idColumns.m_icon] = dxutils::getProtocolIcon(accounts[i].getProtocol().getPurpleID());
        row[m_idColumns.m_name] = accounts[i].getUsername();
        row[m_idColumns.m_account] = accounts[i];
    }
    m_ids.pack_start(m_idColumns.m_icon);
    m_ids.pack_start(m_idColumns.m_name);
    m_ids.set_active(0);
    m_table = Gtk::manage(new Gtk::Table());
    m_table->attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_table->attach(m_ids, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    get_vbox()->pack_start(*m_table, Gtk::PACK_SHRINK);
    add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    set_default_response(Gtk::RESPONSE_OK);
    show_all();
}

SelectAccountDialog::~SelectAccountDialog()
{
}

PURPLE::Account SelectAccountDialog::getAccount()
{
    Gtk::TreeModel::iterator iter = m_ids.get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            return row[m_idColumns.m_account];
        }
    }
    return PURPLE::Account();
}

