/*
 *      dxim.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXIM_H
#define DXIM_H

#include <gtkmm.h>
#include "callback/callback.h"
#include "dialogs/logdialog.h"
#include "tabitems/convtabitem.h"
#include "tabitems/fttabitem.h"
#include "tabitems/msgtabitem.h"

class Tabbook;

class dxim : public Gtk::Window
{
public:
    dxim(bool offline=false);
    virtual ~dxim();

    bool onCmdNextUnread();
    bool onCmdSelectTab(guint index);
protected:
    void onCmdQuit();
    void onCmdAccounts();
    void onCmdAddBuddy();
    void onCmdAddChat();
    void onCmdAddGroup();
    void onCmdAbout();
    void onCmdNewIM();
    void onCmdJoinChat();
    void onTrayActivate();
    void onTrayPopup(guint button, guint32 activate_time);
    void onCmdReconnect();
    void onCmdAvailable();
    void onCmdAway();
    void onCmdDnd();
    void onCmdInvisible();
    void onCmdOffline();
    void onCmdStatusMsg();
    void onCmdCloseTab();
    void onCmdClear();
    void onCmdClearAll();
    void onCmdFileTransfers();
    void onCmdLogViewer();
    void onCmdFriendsList();
    void onCmdFriendsListChat();
    void onCmdShowAvatars();
    void onCmdHideOffline();
    void onCmdShowStatusmsg();
    void onCmdPlugins();
    void onCmdPreferences();
#if GTKMM_MAJOR_VERSION == 2
    void onSwitchTab(GtkNotebookPage *p, unsigned int n);
#else
    void onSwitchTab(Widget *p, unsigned int n);
#endif
    void onTabHighlight();
    void onShowLog(PURPLE::Account account, Glib::ustring conversation, Glib::ustring path);
    void onCloseMsgTab(int tab);
    bool onAwayTimeout();
private:
    Gtk::VBox m_vbox;
    Gtk::HPaned m_splitter;
    Tabbook m_tabbook;
    Roster *m_roster;
    Glib::RefPtr<Gtk::UIManager> m_uiManager;
    Glib::RefPtr<Gtk::StatusIcon> m_statusIcon;
    Glib::RefPtr<Gtk::Action> m_logAction, m_closetabAction, m_clearAction, m_clearallAction;
    dxgCallbacks *m_cb;
    LogDialog *m_log;
    FTTabItem *m_fttab;
    PurpleStatusPrimitive m_presence;
    bool m_newMsg;

    //purplemm slot start
    void onPurpleError(Glib::ustring error);
    void onAccountConnected(PURPLE::Account account);
    void onAccountDisconnected(PURPLE::Account account, Glib::ustring reason);
    void onNewIMMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring sender, Glib::ustring msg, bool highlight, bool action);
    void onNewChatMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring sender, Glib::ustring msg, bool delayed, bool highlight, bool action);
    void onNewIM(PURPLE::Conversation *conv);
    void onNewChat(PURPLE::Conversation *conv);
    void onNewActionRequest(PURPLE::ActionRequest *request);
    void onNewFieldsRequest(PURPLE::FieldsRequest *request);
    void onNewInputRequest(PURPLE::InputRequest *request);
    void onNewFileRequest(PURPLE::FileRequest *request);
    void onNewChoiceRequest(PURPLE::ChoiceRequest *request);
    void onNotifyMessage(PurpleNotifyMsgType type, Glib::ustring title, Glib::ustring text);
    void onNotifyEmail(Glib::ustring text, Glib::ustring url);
    void onNotifyUserinfo(PURPLE::Account account, Glib::ustring buddy, Glib::ustring text);
    void onNotifyUri(Glib::ustring uri);
    void onNewFileTransfer(PURPLE::FileTransfer *ft);
    void onCloseConvTab(PURPLE::Conversation *conv);
    //purplemm slot end

    void createMenus();
    void createAccel();
    void createTrayIcon();
    void updatePresence(PurpleStatusPrimitive presence);
    void updateTrayIconPresence(PurpleStatusPrimitive presence=PURPLE_STATUS_UNSET);
    void updateTrayIconNewmsg();
    void closeAccount(PURPLE::Account account);
    void updateMenu();
};

#endif  /* DXIM_H */

