/*
 *      dxpipe.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "dxpipe.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

dxPipe::dxPipe()
{
    m_running = false;
}

dxPipe::~dxPipe()
{
}

int dxPipe::execCmd(Glib::ustring cmd)
{
    m_command = cmd;
#ifndef _WIN32
    if (pipe(m_in) == -1 || pipe(m_out) == -1)
        return -1;

    m_pid = fork();
    if (m_pid == -1)
        return -1;

    m_running = true;
    Glib::signal_io().connect(sigc::mem_fun(this, &dxPipe::readData), m_out[0], Glib::IO_IN|Glib::IO_PRI);
    m_iochannel = Glib::IOChannel::create_from_fd(m_out[0]);

    if (m_pid == 0)
    {
        setsid();
        char *args[4];
        ::close(m_out[0]);
        ::dup2(m_out[1],STDOUT_FILENO);
        ::dup2(m_out[1],STDERR_FILENO);
        ::close(m_in[1]);
        ::dup2(m_in[0],STDIN_FILENO);
        args[0] = (char*) "sh";
        args[1] = (char*) "-c";
        args[2] = (char*) m_command.c_str();
        args[3] = NULL;
        execvp(args[0], args);
        _exit(1);
    }
    else
    {
        ::close(m_out[1]);
        ::close(m_in[0]);
    }
#endif
    return 1;
}

void dxPipe::stopCmd()
{
#ifndef _WIN32
    if(m_running)
    {
        killpg(m_pid, SIGKILL);
        m_iochannel->close();
    }
    m_running = false;
#endif
}

void dxPipe::closePipe()
{
#ifndef _WIN32
    if(m_running)
    {
        killpg(m_pid, SIGKILL);
        m_iochannel->close();
    }
    m_running = false;
#endif
}

bool dxPipe::readData(Glib::IOCondition io_condition)
{
#ifndef _WIN32
    if((io_condition & Glib::IO_IN) == 0)
    {
        dxutils::debugLine("Invalid fifo response");
        m_running = false;
    }
    else
    {
        Glib::ustring buf;
        while(Glib::IO_STATUS_EOF!=m_iochannel->read_line(buf))
        {
            if(buf.find_last_of('\n')==buf.bytes()-1)
                buf = buf.substr(0, buf.bytes()-1);
            if(buf.find_last_of('\r')==buf.bytes()-1)
                buf = buf.substr(0, buf.bytes()-1);
            if(!buf.empty())
                signalText(buf);
        }
    }
    return true;
#else
    return false;
#endif
}

Glib::ustring dxPipe::getCommand() const
{
    return m_command;
}
