/*
 *      dxutils.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXUTILS_H
#define DXUTILS_H

#include <gtkmm.h>
#include <purplemm.h>

#define FOREACH(t, v, it) \
    for(t::iterator it = v.begin(); it < v.end(); it++)

class dxutils {
public:
    dxutils();
    virtual ~dxutils();

    static dxutils &instance();
    static Glib::RefPtr<Gdk::Pixbuf> getProtocolIcon(const Glib::ustring& protocol);
    static Glib::RefPtr<Gdk::Pixbuf> getPresenceIcon(PurpleStatusPrimitive presence);
    static Glib::RefPtr<Gdk::Pixbuf> getChatRoleIcon(PurpleConvChatBuddyFlags role);
    static Glib::ustring getChatRoleString(PurpleConvChatBuddyFlags role);
    static Glib::RefPtr<Gdk::Pixbuf> createIcon(const Glib::ustring &path, int maxSize=-1);
    static guint hash(const Glib::ustring &str);
    static void debugLine(const Glib::ustring &line);
    static time_t now();
    static Glib::ustring time(const char *format, time_t time=0);
    static std::vector<Glib::ustring> stringSplit(const Glib::ustring &text, char splitter);
    static Glib::ustring stringSection(const Glib::ustring &text, char splitter, int section, bool toEnd=false);
    static bool stringContains(Glib::ustring text, Glib::ustring sub, bool ignoreCase=true);
    static Glib::ustring replaceLessGreater(Glib::ustring text);
    static Gtk::PositionType getTabPosition(int pos=-1);
    static void playFile(const Glib::ustring &file);
    static Glib::ustring absolutePath(const Glib::ustring &file);
    static Glib::ustring absolutePath(const Glib::ustring &base, const Glib::ustring &file);
    static Glib::ustring simplifyPath(const Glib::ustring &file);
    static Glib::ustring relativePath(const Glib::ustring &base, const Glib::ustring &file);
    static Glib::ustring fileExtension(const Glib::ustring &file);
    static Glib::ustring fileName(const Glib::ustring &path);
    static Glib::ustring getMonth(gint month);
    static Glib::ustring getMonthNumber(const Glib::ustring& month);
    static bool isRightLogFile(const Glib::ustring& path, const Glib::ustring& name);
    static Glib::ustring getNickColor(const Glib::ustring &nick);
    static int launchLink(const Glib::ustring& link);
    /**
     *
     * @param label         string to normalize
     * @param normalize     char for remove, usually '_'
     * @return              Normalized Glib::ustring
     */
    static Glib::ustring normalizeLabelString(std::string label, char normalize);
    static Glib::ustring getFileSize(size_t size);
    static Glib::ustring getRemaining(size_t size, size_t speed);
    void showNotify(Glib::ustring text, int position=-1);
    void removeNotify();
    static Glib::ustring typingToString(Glib::ustring nick, bool typing);
    static Glib::ustring checkThemeStatusPath(const Glib::ustring &path);
    static Glib::ustring checkThemesStatusList(const Glib::ustring &list);
    static Glib::ustring checkThemeRolePath(const Glib::ustring &path);
    static Glib::ustring checkThemesRoleList(const Glib::ustring &list);
    static bool removeDir(const Glib::ustring &dirPath);
    bool haveNotify();
    static bool hasLog(const Glib::ustring &accountName, const Glib::ustring &conversationName, bool account);
private:
    int m_notifiesHeight, m_notifyPosition;
    bool m_firstNotify, m_haveNotify;
};

#endif  /* DXUTILS_H */

