/*
 *      notifywindow.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "notifywindow.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "preferences.h"

NotifyWindow::NotifyWindow(const Glib::ustring &body, int previous, int delay)
: Gtk::Window(Gtk::WINDOW_POPUP), m_delay(delay), m_previous(previous)
{
    Gtk::Table *table = Gtk::manage(new Gtk::Table());
    table->attach(*Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "dxim.png")), 0, 1, 0, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    add(*table);
    Gtk::Label *title = Gtk::manage(new Gtk::Label("", Gtk::ALIGN_START));
    title->set_markup("<b>dxim</b>");
    table->attach(*title, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(body, Gtk::ALIGN_START)), 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);

    property_resizable().set_value(false);
    property_skip_taskbar_hint().set_value(true);
    property_skip_pager_hint().set_value(true);
    property_decorated().set_value(false);

    Glib::signal_timeout().connect(sigc::mem_fun(this, &NotifyWindow::onTimeout), m_delay*1000);

    show_all();
}

NotifyWindow::~NotifyWindow()
{}

void NotifyWindow::showNotify(int position)
{
    int moveToPointX = 0;
    int moveToPointY = 0;
    if(position<0) position = DXPREFERENCES.m_notifyPosition;
    switch(position)
    {
    case 0: //left top
        moveToPointX = 3;
        moveToPointY = 3 + m_previous;
        break;
    case 1: //right top
        moveToPointX = Gdk::screen_width() - get_width() - 3;
        moveToPointY = 3 + m_previous;
        break;
    case 2: //left bottom
        moveToPointX = 3;
        moveToPointY = Gdk::screen_height() - m_previous - get_height() - 3;
        break;
    default: //right bottom
        moveToPointX = Gdk::screen_width() - get_width() - 3;
        moveToPointY = Gdk::screen_height() - m_previous - get_height() - 3;
    }
    move(moveToPointX, moveToPointY);
}

bool NotifyWindow::onTimeout()
{
    dxutils::instance().removeNotify();
    destroy_();
    return false;
}


