/*
 *      preferences.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "preferences.h"

Preferences::Preferences()
{
    //General settings pane in Configdialog
    m_oneInstance = false;
    m_useAway = true;
    m_awayTimeout = 5;
    m_minToTray = false;
    m_logging = true;
#ifdef _WIN32
    m_logPath = Glib::getenv("AppData")+G_DIR_SEPARATOR_S+"dxim"+G_DIR_SEPARATOR_S+"logs";
#else
    m_logPath = Glib::get_home_dir()+G_DIR_SEPARATOR_S+".dxim"+G_DIR_SEPARATOR_S+"logs";
#endif
    m_logLoadCurrent = false;
    m_showRosterTree = true;
    m_receiptsMark = true;
    //Icons settings pane in Configdialog
    m_themeStatusPath = DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "statuses";
    m_themesStatusList = m_themeStatusPath+";";
    m_themeRolePath = DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "roles";
    m_themesRoleList = m_themeRolePath+";";
    //Look pane in ConfigDialog
    m_unreadtabColor = dxColor("blue");
    m_hltabColor = dxColor("red");
    m_tabPosition = 0;
    //Im Text settings pane in Configdialog
    m_textColor = dxColor("#000");
    m_backroundColor = dxColor("#fff");
    m_userColor = dxColor("#2B3923");
    m_actionColor = dxColor("#FFA500");
    m_errorColor = dxColor("#FF0000");
    m_hilightColor = dxColor("#0B9A0B");
    m_linkColor = dxColor("#8EC4D2");
    m_mymsgColor = dxColor("#8296FD");
    m_logColor = dxColor("#463131");
    m_imFont = "Sans 10";
    m_sameCmd = false;
    m_sameList = false;
    m_nickCompletionChar = ":";
    m_coloredNick = true;
    //Notification settings pane in Configdialog
    m_sounds = false;
    m_soundConnect = false;
    m_pathConnect = DXIM_DATADIR G_DIR_SEPARATOR_S "sounds" G_DIR_SEPARATOR_S "connected.wav";
    m_soundDisconnect = false;
    m_pathDisconnect = DXIM_DATADIR G_DIR_SEPARATOR_S "sounds" G_DIR_SEPARATOR_S "disconnected.wav";
    m_soundMessage = false;
    m_pathMessage = DXIM_DATADIR G_DIR_SEPARATOR_S "sounds" G_DIR_SEPARATOR_S "message.wav";
    m_notify = false;
    m_notifyPosition = 3;
    m_notifyAccountConnect = false;
    m_notifyAccountDisconnect = false;
    m_notifyBuddyConnect = false;
    m_notifyBuddyDisconnect = false;
    m_notifyMessage = false;
    m_notifyFiletransfer = false;
    m_raiseWindow = false;
    m_notifyMessageTray = false;
    //Other settings value
    m_hideOffline = true;
    m_rosterShown = true;
    m_chatRosterShown = true;
    m_showAvatars = true;
    m_showStatusmsg = true;
    m_x = 50;
    m_y = 50;
    m_w = 888;
    m_h = 666;
    m_maximized = false;
    m_rosterWidth = 220;
    setIniPath("");
}

Preferences &Preferences::instance()
{
    static Preferences p;
    return p;
}

void Preferences::setIniPath(const Glib::ustring& path)
{
    if(path.empty())
    {
        m_iniPath = Glib::build_filename(Glib::get_user_config_dir(), "dxim");
        if(!(Glib::file_test(m_iniPath, Glib::FILE_TEST_IS_DIR)))
        {
            g_mkdir_with_parents(m_iniPath.c_str(), 0755);
        }
        return;
    }
    m_iniPath = path;
    if(!Glib::file_test(m_iniPath, Glib::FILE_TEST_IS_DIR))
    {
        g_mkdir_with_parents(m_iniPath.c_str(), 0755);
    }
}

Glib::ustring Preferences::getIniPath()
{
    if(m_iniPath.empty())
        m_iniPath = Glib::build_filename(Glib::get_user_config_dir(), "dxim");
    if(!Glib::file_test(m_iniPath, Glib::FILE_TEST_IS_DIR))
    {
        g_mkdir_with_parents(m_iniPath.c_str(), 0755);
    }
    return m_iniPath;
}

void Preferences::load()
{
    Settings set(m_iniPath+G_DIR_SEPARATOR_S+"dxim-gtkmm");
    set.load();
    //General settings pane in Configdialog
    m_oneInstance = set.readBoolEntry("GENERAL", "oneInstance", m_oneInstance);
    m_useAway = set.readBoolEntry("GENERAL", "useAway", m_useAway);
    m_awayTimeout = set.readIntEntry("GENERAL", "awayTimeout", m_awayTimeout);
    m_minToTray = set.readBoolEntry("GENERAL", "minToTray", m_minToTray);
    m_logging = set.readBoolEntry("GENERAL", "logging", m_logging);
    m_logPath = set.readStringEntry("GENERAL", "logPath", m_logPath);
    m_logLoadCurrent = set.readBoolEntry("GENERAL", "logLoadCurrent", m_logLoadCurrent);
    if(!m_logging) m_logLoadCurrent = false;
    m_showRosterTree = set.readBoolEntry("GENERAL", "showRosterTree", m_showRosterTree);
    m_receiptsMark = set.readBoolEntry("GENERAL", "receiptsMark", m_receiptsMark);
    //Icons settings pane in Configdialog
    m_themeStatusPath = dxutils::checkThemeStatusPath(set.readStringEntry("THEMES", "themeStatusPath", m_themeStatusPath));
    m_themesStatusList = dxutils::checkThemesStatusList(set.readStringEntry("THEMES", "themesStatusList", m_themesStatusList));
    if(!dxutils::stringContains(m_themesStatusList, m_themeStatusPath))
        m_themesStatusList.append(m_themeStatusPath+";");
    m_themeRolePath = dxutils::checkThemeRolePath(set.readStringEntry("THEMES", "themesRolePath", m_themeRolePath));
    m_themesRoleList = dxutils::checkThemesRoleList(set.readStringEntry("THEMES", "themesRoleList", m_themesRoleList));
    if(!dxutils::stringContains(m_themesRoleList, m_themeRolePath))
        m_themesRoleList.append(m_themeRolePath+";");
    //Look pane in ConfigDialog
    m_unreadtabColor = dxColor(set.readStringEntry("LOOK", "unreadtabColor", m_unreadtabColor.to_string()));
    m_hltabColor = dxColor(set.readStringEntry("LOOK", "hltabColor", m_hltabColor.to_string()));
    m_tabPosition = set.readIntEntry("LOOK", "tabPosition", m_tabPosition);
    //Im Text settings pane in Configdialog
    m_textColor = dxColor(set.readStringEntry("IMTEXT", "textColor", m_textColor.to_string()));
    m_backroundColor = dxColor(set.readStringEntry("IMTEXT", "backroundColor", m_backroundColor.to_string()));
    m_userColor = dxColor(set.readStringEntry("IMTEXT", "userColor", m_userColor.to_string()));
    m_actionColor = dxColor(set.readStringEntry("IMTEXT", "actionColor", m_actionColor.to_string()));
    m_errorColor = dxColor(set.readStringEntry("IMTEXT", "errorColor", m_errorColor.to_string()));
    m_hilightColor = dxColor(set.readStringEntry("IMTEXT", "hilightColor", m_hilightColor.to_string()));
    m_linkColor = dxColor(set.readStringEntry("IMTEXT", "linkColor", m_linkColor.to_string()));
    m_mymsgColor = dxColor(set.readStringEntry("IMTEXT", "mymsgColor", m_mymsgColor.to_string()));
    m_logColor = dxColor(set.readStringEntry("IMTEXT", "logColor", m_logColor.to_string()));
    m_imFont = set.readStringEntry("IMTEXT", "imFont", m_imFont);
    m_sameCmd = set.readBoolEntry("IMTEXT", "sameCmd", m_sameCmd);
    m_sameList = set.readBoolEntry("IMTEXT", "sameList", m_sameList);
    m_nickCompletionChar = Glib::ustring(set.readStringEntry("IMTEXT", "nickCompletionChar", m_nickCompletionChar)).substr(0,1);
    m_coloredNick = set.readBoolEntry("IMTEXT", "coloredNick", m_coloredNick);
    //Notification settings pane in Configdialog
    m_sounds = set.readBoolEntry("NOTIFICATION", "sounds", m_sounds);
    m_soundConnect = set.readBoolEntry("NOTIFICATION", "soundConnect", m_soundConnect);
    m_pathConnect = set.readStringEntry("NOTIFICATION", "pathConnect", m_pathConnect);
    m_soundDisconnect = set.readBoolEntry("NOTIFICATION", "soundDisconnect", m_soundDisconnect);
    m_pathDisconnect = set.readStringEntry("NOTIFICATION", "pathDisconnect", m_pathDisconnect);
    m_soundMessage = set.readBoolEntry("NOTIFICATION", "soundMessage", m_soundMessage);
    m_pathMessage = set.readStringEntry("NOTIFICATION", "pathMessage", m_pathMessage);
    m_notify = set.readBoolEntry("NOTIFICATION", "notify", m_notify);
    m_notifyPosition = set.readIntEntry("NOTIFICATION", "notifyPosition", m_notifyPosition);
    m_notifyAccountConnect = set.readBoolEntry("NOTIFICATION", "notifyAccountConnect", m_notifyAccountConnect);
    m_notifyAccountDisconnect = set.readBoolEntry("NOTIFICATION", "notifyAccountDisconnect", m_notifyAccountDisconnect);
    m_notifyBuddyConnect = set.readBoolEntry("NOTIFICATION", "notifyBuddyConnect", m_notifyBuddyConnect);
    m_notifyBuddyDisconnect = set.readBoolEntry("NOTIFICATION", "notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    m_notifyMessage = set.readBoolEntry("NOTIFICATION", "notifyMessage", m_notifyMessage);
    m_notifyFiletransfer = set.readBoolEntry("NOTIFICATION", "notifyFiletransfer", m_notifyFiletransfer);
    m_raiseWindow = set.readBoolEntry("NOTIFICATION", "raiseWindow", m_raiseWindow);
    m_notifyMessageTray = set.readBoolEntry("NOTIFICATION", "notifyMessageTray", m_notifyMessageTray);
    if(m_raiseWindow) m_notifyMessageTray = false;
    //Smileys settings tab in Configdialog
    m_useSmileys = set.readBoolEntry("SMILEYS", "useSmileys", m_useSmileys);
    int smileysNum = set.readIntEntry("SMILEYS", "number", 0);
    m_smileysMap.clear();
    if(smileysNum)
    {
        for(int i=0; i<smileysNum; i++)
        {
            Glib::ustring key, value;
            key = set.readStringEntry("SMILEYS", Glib::ustring::compose("smiley%1", i), "");
            value = set.readStringEntry("SMILEYS", Glib::ustring::compose("path%1", i), "");
            if(!key.empty())
                m_smileysMap.insert(dxStringPair(key, value));
        }
    }
    //Other settings value
    m_hideOffline = set.readBoolEntry("OTHER", "hideOffline", m_hideOffline);
    m_rosterShown = set.readBoolEntry("OTHER", "rosterShown", m_rosterShown);
    m_chatRosterShown = set.readBoolEntry("OTHER", "chatRosterShown", m_chatRosterShown);
    m_showAvatars = set.readBoolEntry("OTHER", "showAvatars", m_showAvatars);
    m_showStatusmsg = set.readBoolEntry("OTHER", "showStatusmsg", m_showStatusmsg);
    m_x = set.readIntEntry("OTHER", "x", m_x);
    m_y = set.readIntEntry("OTHER", "y", m_y);
    m_w = set.readIntEntry("OTHER", "w", m_w);
    m_h = set.readIntEntry("OTHER", "h", m_h);
    m_maximized = set.readBoolEntry("OTHER", "maximized", m_maximized);
    m_rosterWidth = set.readIntEntry("OTHER", "rosterWidth", m_rosterWidth);
}

void Preferences::save()
{
    Settings set(m_iniPath+G_DIR_SEPARATOR_S+"dxim-gtkmm");
    //General settings pane in Configdialog
    set.writeBoolEntry("GENERAL", "oneInstance", m_oneInstance);
    set.writeBoolEntry("GENERAL", "useAway", m_useAway);
    set.writeIntEntry("GENERAL", "awayTimeout", m_awayTimeout);
    set.writeBoolEntry("GENERAL", "minToTray", m_minToTray);
    set.writeBoolEntry("GENERAL", "logging", m_logging);
    set.writeStringEntry("GENERAL", "logPath", m_logPath);
    set.writeBoolEntry("GENERAL", "logLoadCurrent", m_logLoadCurrent);
    set.writeBoolEntry("GENERAL", "showRosterTree", m_showRosterTree);
    set.writeBoolEntry("GENERAL", "receiptsMark", m_receiptsMark);
    //Icons settings pane in Configdialog
    set.writeStringEntry("THEMES", "themeStatusPath", m_themeStatusPath);
    set.writeStringEntry("THEMES", "themesStatusList", m_themesStatusList);
    set.writeStringEntry("THEMES", "themesRolePath", m_themeRolePath);
    set.writeStringEntry("THEMES", "themesRoleList", m_themesRoleList);
    //Look pane in ConfigDialog
    set.writeStringEntry("LOOK", "unreadtabColor", m_unreadtabColor.to_string());
    set.writeStringEntry("LOOK", "hltabColor", m_hltabColor.to_string());
    set.writeIntEntry("LOOK", "tabPosition", m_tabPosition);
    //Im Text settings pane in Configdialog
    set.writeStringEntry("IMTEXT", "textColor", m_textColor.to_string());
    set.writeStringEntry("IMTEXT", "backroundColor", m_backroundColor.to_string());
    set.writeStringEntry("IMTEXT", "userColor", m_userColor.to_string());
    set.writeStringEntry("IMTEXT", "actionColor", m_actionColor.to_string());
    set.writeStringEntry("IMTEXT", "errorColor", m_errorColor.to_string());
    set.writeStringEntry("IMTEXT", "hilightColor", m_hilightColor.to_string());
    set.writeStringEntry("IMTEXT", "linkColor", m_linkColor.to_string());
    set.writeStringEntry("IMTEXT", "mymsgColor", m_mymsgColor.to_string());
    set.writeStringEntry("IMTEXT", "logColor", m_logColor.to_string());
    set.writeStringEntry("IMTEXT", "imFont", m_imFont);
    set.writeBoolEntry("IMTEXT", "sameCmd", m_sameCmd);
    set.writeBoolEntry("IMTEXT", "sameList", m_sameList);
    set.writeStringEntry("IMTEXT", "nickCompletionChar", m_nickCompletionChar.substr(0,1));
    set.writeBoolEntry("IMTEXT", "coloredNick", m_coloredNick);
    //Notification settings pane in Configdialog
    set.writeBoolEntry("NOTIFICATION", "sounds", m_sounds);
    set.writeBoolEntry("NOTIFICATION", "soundConnect", m_soundConnect);
    set.writeStringEntry("NOTIFICATION", "pathConnect", m_pathConnect);
    set.writeBoolEntry("NOTIFICATION", "soundDisconnect", m_soundDisconnect);
    set.writeStringEntry("NOTIFICATION", "pathDisconnect", m_pathDisconnect);
    set.writeBoolEntry("NOTIFICATION", "soundMessage", m_soundMessage);
    set.writeStringEntry("NOTIFICATION", "pathMessage", m_pathMessage);
    set.writeBoolEntry("NOTIFICATION", "notify", m_notify);
    set.writeIntEntry("NOTIFICATION", "notifyPosition", m_notifyPosition);
    set.writeBoolEntry("NOTIFICATION", "notifyAccountConnect", m_notifyAccountConnect);
    set.writeBoolEntry("NOTIFICATION", "notifyAccountDisconnect", m_notifyAccountDisconnect);
    set.writeBoolEntry("NOTIFICATION", "notifyBuddyConnect", m_notifyBuddyConnect);
    set.writeBoolEntry("NOTIFICATION", "notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    set.writeBoolEntry("NOTIFICATION", "notifyMessage", m_notifyMessage);
    set.writeBoolEntry("NOTIFICATION", "notifyFiletransfer", m_notifyFiletransfer);
    set.writeBoolEntry("NOTIFICATION", "raiseWindow", m_raiseWindow);
    set.writeBoolEntry("NOTIFICATION", "notifyMessageTray", m_notifyMessageTray);
    //Smileys settings tab in Configdialog
    set.writeBoolEntry("SMILEYS", "useSmileys", m_useSmileys);
    set.writeIntEntry("SMILEYS", "number", (int)m_smileysMap.size());
    if((int)m_smileysMap.size())
    {
        dxStringIt it;
        int i;
        for(i=0, it=m_smileysMap.begin(); it!=m_smileysMap.end(); it++,i++)
        {
            set.writeStringEntry("SMILEYS", Glib::ustring::compose("smiley%1", i), (*it).first);
            set.writeStringEntry("SMILEYS", Glib::ustring::compose("path%1", i), (*it).second);
        }
    }
    //Other settings value
    set.writeBoolEntry("OTHER", "hideOffline", m_hideOffline);
    set.writeBoolEntry("OTHER", "rosterShown", m_rosterShown);
    set.writeBoolEntry("OTHER", "chatRosterShown", m_chatRosterShown);
    set.writeBoolEntry("OTHER", "showAvatars", m_showAvatars);
    set.writeBoolEntry("OTHER", "showStatusmsg", m_showStatusmsg);
    set.writeIntEntry("OTHER", "x", m_x);
    set.writeIntEntry("OTHER", "y", m_y);
    set.writeIntEntry("OTHER", "w", m_w);
    set.writeIntEntry("OTHER", "h", m_h);
    set.writeBoolEntry("OTHER", "maximized", m_maximized);
    set.writeIntEntry("OTHER", "rosterWidth", m_rosterWidth);
    set.save();
}
