/*
 *      roster.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ROSTER_H
#define ROSTER_H

#include <gtkmm.h>
#include "callback/callback.h"

class Roster : public Gtk::Frame {
    enum itemType {
        ACCOUNT = 0,
        GROUP,
        BUDDY,
        CHAT,
    };
public:
    Roster(dxgCallbacks *cb);

    void filterChange();
    void showTreeChange();
    void updateShowAvatars();
    void updateShowStatusmsg();
    void updateTheme();
    void updateAccountsTooltip();

    sigc::signal<void, PURPLE::Account> signalAccountDisconnected;
private:
    dxgCallbacks *m_cb;
    Glib::RefPtr<Gtk::TreeStore> m_rosterModel;
    Glib::RefPtr<Gtk::TreeModelFilter> m_rosterModelFilter;
    struct ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::ustring> m_ID;
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_icon;
        Gtk::TreeModelColumn<Glib::ustring> m_name;
        Gtk::TreeModelColumn<Glib::ustring> m_markup;
        Gtk::TreeModelColumn<itemType> m_type;
        Gtk::TreeModelColumn<PURPLE::Account> m_account;
        Gtk::TreeModelColumn<PURPLE::Buddy> m_buddy;
        Gtk::TreeModelColumn<PURPLE::Chat> m_chat;
        Gtk::TreeModelColumn<bool> m_hidden;
        Gtk::TreeModelColumn<Glib::ustring> m_tooltip;
        Gtk::TreeModelColumn<time_t> m_lastChange;
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_avatar;
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_protocol;

        ModelColumns() { add(m_ID); add(m_icon); add(m_name); add(m_markup); add(m_type);
            add(m_account); add(m_buddy); add(m_chat); add(m_hidden);
            add(m_tooltip); add(m_lastChange); add(m_avatar); add(m_protocol); }
    };
    const ModelColumns m_rosterColumns;
    Gtk::TreeView m_roster;
    Gtk::CellRendererText m_usernameRenderer;
    Gtk::TreeView::Column *m_avatarColumn;
    Gtk::TreeView::Column *m_protocolColumn;

    //slot start
    bool onButtonPressed(GdkEventButton *event);
    void onDoubleClick(const Gtk::TreeModel::Path &path, Gtk::TreeViewColumn *column);
    void onAccountCb(Glib::ustring cmd);
    void onPopupAccountIm();
    void onPopupAccountChat();
    void onPopupAccountStatus();
    void onBuddyCb(PURPLE::Buddy::BuddyMenuItem item);
    void onPopupJoin();
    void onPopupIm();
    void onPopupInfo();
    void onPopupSendFile();
    void onPopupAddBuddy();
    void onPopupAddChat();
    void onPopupAddGroup();
    void onPopupModifyChat();
    void onPopupRemoveChat();
    void onPopupRenameBuddy();
    void onPopupMoveBuddy();
    void onPopupRemoveBuddy();
    void onPopupReconnectAccount();
    void onPopupEnableAccount();
    void onPopupAliasAccount();
    void onPopupConfigAccount();
    void onPopupChatLogs();
    void onPopupBuddyLogs();
    void onPopupAccountLogs();
    //purplemm slots
    void onBuddylistLoaded();
    void onAccountAdded(PURPLE::Account account);
    void onAccountRemoved(Glib::ustring account, Glib::ustring protocolID);
    void onAccountConnecting(PURPLE::Account account, Glib::ustring msg);
    void onAccountConnected(PURPLE::Account account);
    void onAccountDisconnected(PURPLE::Account account, Glib::ustring reason);
    void onAccountStatusChanged(PURPLE::Account account);
    void onAccountSignedOn(PURPLE::Account account);
    void onBuddySignedOn(PURPLE::Buddy buddy);
    void onBuddySignedOff(PURPLE::Buddy buddy);
    void onBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive status);
    void onBuddyIconChanged(PURPLE::Buddy buddy);
    void onBuddyAdded(PURPLE::Buddy buddy);
    void onBuddyRemoved(PURPLE::Account account, Glib::ustring buddy);
    void onChatAdded(PURPLE::Chat chat);
    void onChatRemoved(PURPLE::Account account, Glib::ustring chat);
    //slot end

    void fillRoster();
    Gtk::TreeModel::Row addAccount(PURPLE::Account account);
    Gtk::TreeModel::Row addGroup(PURPLE::Account account, Glib::ustring name);
    Gtk::TreeModel::Row addBuddy(PURPLE::Buddy buddy);
    Gtk::TreeModel::Row addChat(PURPLE::Chat chat);
    Gtk::TreeModel::Row findAccountItem(Glib::ustring ID, Glib::ustring protocol);
    Gtk::TreeModel::Row findGroupItem(Gtk::TreeModel::Row parent, Glib::ustring text);
    Gtk::TreeModel::Row findBuddyItem(PURPLE::Account account, Glib::ustring buddy);
    Gtk::TreeModel::Row findChatItem(PURPLE::Account account, Glib::ustring chat);
    bool filterRoster(const Gtk::TreeModel::const_iterator &iter);
    void updateTooltip(Gtk::TreeModel::Row row);
};

#endif  /* ROSTER_H */

