/*
 *      settings.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <fstream>
#include "settings.h"

Settings::Settings(std::string iniFile) : m_iniFile(iniFile)
{}

void Settings::load()
{
    try
    {
        m_keyFile.load_from_file(m_iniFile);
    }
    catch(...)
    {}
}

void Settings::save()
{
    std::ofstream out(Glib::locale_from_utf8(m_iniFile).c_str());
    if(!out)
        return;
    out << m_keyFile.to_data();
}

void Settings::writeBoolEntry(std::string section, std::string key, bool val)
{
    m_keyFile.set_boolean(section, key, val);
}

void Settings::writeIntEntry(std::string section, std::string key, int val)
{
    m_keyFile.set_integer(section, key, val);
}

void Settings::writeRealEntry(std::string section, std::string key, double val)
{
    m_keyFile.set_double(section, key, val);
}

void Settings::writeStringEntry(std::string section, std::string key, std::string val)
{
    m_keyFile.set_value(section, key, val);
}

bool Settings::readBoolEntry(std::string section, std::string key, bool def)
{
    try {
        if(m_keyFile.has_key(section, key))
        {
            return m_keyFile.get_boolean(section, key);
        }
        return def;
    }
    catch(...) {
        return def;
    }
}

int Settings::readIntEntry(std::string section, std::string key, int def)
{
    try {
        if(m_keyFile.has_key(section, key))
        {
            return m_keyFile.get_integer(section, key);
        }
        return def;
    }
    catch(...) {
        return def;
    }
}

double Settings::readRealEntry(std::string section, std::string key, double def)
{
    try {
        if(m_keyFile.has_key(section, key))
        {
            return m_keyFile.get_double(section, key);
        }
        return def;
    }
    catch(...) {
        return def;
    }
}

std::string Settings::readStringEntry(std::string section, std::string key, std::string def)
{
    try {
        if(m_keyFile.has_key(section, key))
        {
            return m_keyFile.get_value(section, key);
        }
        return def;
    }
    catch(...) {
        return def;
    }
}

