/*
 *      settings.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef SETTINGS_H
#define SETTINGS_H

#include <string>
#include <map>
#include "dxutils.h"

typedef std::map<std::string,std::string> dxStringMap;
typedef std::pair<std::string,std::string> dxStringPair;
typedef std::map<std::string,std::string>::iterator dxStringIt;

class Settings {
public:
    Settings(std::string iniFile);
    Settings(const Settings& orig);

    //Load ini file
    void load();
    //Save ini file
    void save();
    // Clear all sections
    void clear();
    // Read a string registry entry; if no value is found, the default value def is returned
    std::string readStringEntry(std::string section, std::string key, std::string def="");
    // Read a integer registry entry; if no value is found, the default value def is returned
    int readIntEntry(std::string section, std::string key, int def=0);
    // Read a double-precision floating point registry entry; if no value is found, the default value def is returned
    double readRealEntry(std::string section, std::string key, double def=0.0);
    // Read a boolean registry entry; if no value is found, the default value def is returned
    bool readBoolEntry(std::string section, std::string key, bool def=false);
    // Write a string registry entry
    void writeStringEntry(std::string section, std::string key, std::string val);
    // Write a integer registry entry
    void writeIntEntry(std::string section, std::string key, int val);
    // Write a double-precision floating point registry entry
    void writeRealEntry(std::string section, std::string key, double val);
    // Write a boolean value entry
    void writeBoolEntry(std::string section, std::string key, bool val);
private:
    std::string m_iniFile;
    Glib::KeyFile m_keyFile;
};

#endif  /* SETTINGS_H */

