/*
 *      chattabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chattabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "preferences.h"

ChatTabItem::ChatTabItem(PURPLE::Conversation *conv, dxgCallbacks *cb, Gtk::Notebook *parent,
        ImageLabel *label) :
ConvTabItem(conv, parent, label, TABCHAT), m_cb(cb)
{
    m_id = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(m_conv->getAccount().getProtocol().getPurpleID()), m_conv->getName()+" ("+m_conv->getAccount().getUsername()+")"));
    pack_start(*m_id, false, true, 3);
    m_topicline.signal_activate().connect(sigc::mem_fun(this, &ChatTabItem::onTopicline));
#if GTKMM_MAJOR_VERSION == 2
    m_topicline.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_topicline.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_topicline.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_topicline.get_style_context()->add_provider(m_styleProvider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
    m_styleProvider->load_from_data(Glib::ustring::compose("GtkEntry { color: %1; background: %2; }", DXPREFERENCES.m_textColor.get_color().to_string(), DXPREFERENCES.m_backroundColor.get_color().to_string()));
//    m_topicline.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
//    m_topicline.override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_topicline.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    pack_start(m_topicline, false, true, 0);
    Gtk::ScrolledWindow *textwin = Gtk::manage(new Gtk::ScrolledWindow());
    textwin->add(m_text);
    textwin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    textwin->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    Gtk::ScrolledWindow *nickwin = Gtk::manage(new Gtk::ScrolledWindow());
    nickwin->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    m_nickModel = Gtk::ListStore::create(m_nickColumns);
    m_nickSort = Gtk::TreeModelSort::create(m_nickModel);
    m_nickSort->set_sort_column(m_nickColumns.m_name, Gtk::SORT_ASCENDING);
    m_nicks.set_model(m_nickSort);
    m_nicks.append_column(" ",m_nickColumns.m_icon);
    m_nicks.append_column("username", m_nickColumns.m_name);
    m_nicks.set_headers_visible(false);
    m_nicks.set_tooltip_column(3);
#if GTKMM_MAJOR_VERSION == 2
    if(DXPREFERENCES.m_sameList) m_nicks.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_nicks.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
#else
    if(DXPREFERENCES.m_sameList) m_nicks.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_nicks.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
#endif
    dynamic_cast<Gtk::CellRendererText*>(m_nicks.get_column_cell_renderer(1))->property_foreground().set_value(DXPREFERENCES.m_textColor.to_string());
    nickwin->add(m_nicks);
    m_splitter.pack1(*textwin, true, true);
    m_splitter.pack2(*nickwin, false, true);
    pack_start(m_splitter);
    pack_start(m_commandline, false, true, 0);
    show_all();
    if(!m_conv->hasTopic()) m_topicline.hide();
    if(DXPREFERENCES.m_chatRosterShown) m_splitter.get_child2()->show();
    else m_splitter.get_child2()->hide();

    m_cb->signalNewMyMessage.connect(sigc::mem_fun(this, &ChatTabItem::onNewMyMsg));
    m_cb->signalNewChatMessage.connect(sigc::mem_fun(this, &ChatTabItem::onNewChatMsg));
    m_cb->signalChatBuddyJoined.connect(sigc::mem_fun(this, &ChatTabItem::onChatBuddyJoined));
    m_cb->signalChatBuddyLeft.connect(sigc::mem_fun(this, &ChatTabItem::onChatBuddyLeft));
    m_cb->signalChatBuddyRenamed.connect(sigc::mem_fun(this, &ChatTabItem::onChatBuddyRenamed));
    m_cb->signalChatBuddyUpdated.connect(sigc::mem_fun(this, &ChatTabItem::onChatBuddyUpdated));
    m_cb->signalChatTopic.connect(sigc::mem_fun(this, &ChatTabItem::onChatTopic));
    m_cb->signalNewConvSystemMessage.connect(sigc::mem_fun(this, &ChatTabItem::onNewConvSystemMessage));
    m_cb->signalNewConvErrorMessage.connect(sigc::mem_fun(this, &ChatTabItem::onNewConvErrorMessage));

    m_nicks.signal_button_press_event().connect(sigc::mem_fun(this, &ChatTabItem::onButtonPressed), false);
}

bool ChatTabItem::onButtonPressed(GdkEventButton* event)
{
    bool value = false;
    if(event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
        Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_nicks.get_selection();
        if(refSelection)
        {
            Gtk::TreeModel::Path path;
            m_nicks.get_path_at_pos(event->x, event->y, path);
            refSelection->select(path);
            Gtk::TreeModel::iterator iter = refSelection->get_selected();
            if(iter)
            {
                if(m_conv->getMyNick() != (*iter)[m_nickColumns.m_name])
                {
                    Gtk::Menu *popup = new Gtk::Menu();
                    Gtk::MenuItem* item = new Gtk::MenuItem(_("IM"));
                    item->signal_activate().connect(sigc::mem_fun(this, &ChatTabItem::onPopupIm));
                    popup->append(*item);
                    item = new Gtk::MenuItem(_("Info"));
                    item->signal_activate().connect(sigc::mem_fun(this, &ChatTabItem::onPopupInfo));
                    popup->append(*item);
                    popup->append(*(new Gtk::SeparatorMenuItem()));
                    PURPLE::ChatBuddy buddy = (*iter)[m_nickColumns.m_buddy];
                    item = new Gtk::MenuItem(m_conv->isChatBuddyIgnored(buddy.getName())?_("Unignore"):_("Ignore"));
                    item->signal_activate().connect(sigc::mem_fun(this, &ChatTabItem::onPopupIgnore));
                    popup->append(*item);
                    popup->accelerate(*this);
                    popup->show_all();
                    popup->popup(event->button, event->time);
                }
                value = true;
            }
        }
    }
    return value;
}

void ChatTabItem::onPopupIm()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_nicks.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            PURPLE::ChatBuddy buddy = (*iter)[m_nickColumns.m_buddy];
            m_conv->createIM(buddy);
        }
    }
}

void ChatTabItem::onPopupInfo()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_nicks.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            PURPLE::ChatBuddy buddy = (*iter)[m_nickColumns.m_buddy];
            m_conv->retrieveChatBuddyInfo(buddy.getName());
        }
    }
}

void ChatTabItem::onPopupIgnore()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_nicks.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            PURPLE::ChatBuddy buddy = (*iter)[m_nickColumns.m_buddy];
            if(m_conv->isChatBuddyIgnored(buddy.getName()))
                m_conv->unignore(buddy.getName());
            else
                m_conv->ignore(buddy.getName());
        }
    }
}

void ChatTabItem::updatePreferences()
{
#if GTKMM_MAJOR_VERSION == 2
    m_text.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_text.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_text.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_text.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    Glib::RefPtr<Gtk::TextTag> tag = m_text.get_buffer()->get_tag_table()->lookup("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("logs");
    tag->property_foreground().set_value(DXPREFERENCES.m_logColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("link");
    tag->property_foreground().set_value(DXPREFERENCES.m_linkColor.to_string());
#if GTKMM_MAJOR_VERSION == 2
    m_topicline.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_topicline.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_topicline.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_commandline.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_commandline.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    if(DXPREFERENCES.m_sameCmd) m_commandline.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_commandline.modify_font(get_pango_context()->get_font_description());
    m_nicks.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    if(DXPREFERENCES.m_sameList) m_nicks.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_nicks.modify_font(get_pango_context()->get_font_description());
#else
    m_styleProvider->load_from_data(Glib::ustring::compose("GtkEntry { color: %1; background: %2; }", DXPREFERENCES.m_textColor.get_color().to_string(), DXPREFERENCES.m_backroundColor.get_color().to_string()));
//    m_topicline.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
//    m_topicline.override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_topicline.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    if(DXPREFERENCES.m_sameCmd) m_commandline.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_commandline.override_font(get_pango_context()->get_font_description());
    m_nicks.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    if(DXPREFERENCES.m_sameList) m_nicks.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_nicks.override_font(get_pango_context()->get_font_description());
#endif
    dynamic_cast<Gtk::CellRendererText*>(m_nicks.get_column_cell_renderer(1))->property_foreground().set_value(DXPREFERENCES.m_textColor.to_string());
    Gtk::TreeModel::Children children = m_nickModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        PURPLE::ChatBuddy buddy = (*iter)[m_nickColumns.m_buddy];
        (*iter)[m_nickColumns.m_icon] = dxutils::getChatRoleIcon(buddy.getChatRole());
    }
}

void ChatTabItem::onTopicline()
{
    m_conv->setTopic(m_topicline.get_text());
}

void ChatTabItem::onNewMyMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg, bool action, Glib::ustring /*messageID*/)
{
    if(m_conv == conv)
    {
        if(action) appendStyledText(m_conv->getMyNick()+" "+msg, "action", time);
        else appendMyMsg(time, msg);
    }
}

void ChatTabItem::onNewChatMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring sender, Glib::ustring msg, bool delayed, bool highlight, bool action)
{
    if(m_conv == conv)
    {
        if(delayed)
        {
            appendHistoryMsg(sender, msg, time);
            return;
        }
        if(highlight)
        {
            appendStyledText(sender+(action?" ":": ")+msg, "highlight", time);
            signalHighlight();
        }
        else
        {
            if(action) appendStyledText(sender+" "+msg, "action", time);
            else appendNickText(time, sender, msg, true);
        }
        if(!isCurrent())
        {
            if(highlight)
            {
                m_label->setColor(DXPREFERENCES.m_hltabColor);
                m_label->setIcon(ICO_CHATNEWMSG->get_pixbuf());
                m_status = HILIGHT;
                if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(Glib::ustring::compose(_("New highlighted message on %1"), m_label->getText()));
                if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
            }
            if(m_status!=HILIGHT)
            {
                m_label->setColor(DXPREFERENCES.m_unreadtabColor);
                m_status = UNREAD;
            }
        }
    }
}

void ChatTabItem::onNewConvSystemMessage(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg)
{
    if(m_conv==conv) appendStyledText(msg, "user", time);
}

void ChatTabItem::onNewConvErrorMessage(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg)
{
    if(m_conv==conv) appendStyledText(msg, "error", time);
}

void ChatTabItem::showHideUsers()
{
    if(DXPREFERENCES.m_chatRosterShown) m_splitter.get_child2()->show();
    else m_splitter.get_child2()->hide();
}

void ChatTabItem::removeHighligth()
{
    m_label->clearColor();
    m_label->setIcon(ICO_CHAT->get_pixbuf());
    m_status = STANDARD;
}

void ChatTabItem::onChatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
{
    if(m_conv == conv)
    {
        if(!hasBuddy(buddy.getName()))
        {
            Gtk::TreeModel::Row row = *(m_nickModel->append());
            row[m_nickColumns.m_icon] = dxutils::getChatRoleIcon(buddy.getChatRole());
            row[m_nickColumns.m_name] = buddy.getName();
            row[m_nickColumns.m_buddy] = buddy;
            row[m_nickColumns.m_tooltip] = createBuddyTooltip(buddy);
        }
    }
}

void ChatTabItem::onChatBuddyLeft(PURPLE::Conversation *conv, Glib::ustring buddy)
{
    if(m_conv == conv)
    {
        Gtk::TreeModel::Children children = m_nickModel->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row[m_nickColumns.m_name] == buddy)
            {
                m_nickModel->erase(iter);
                break;
            }
        }
    }
}

void ChatTabItem::onChatBuddyRenamed(PURPLE::Conversation *conv, Glib::ustring buddy, PURPLE::ChatBuddy nbuddy)
{
    if(m_conv == conv)
    {
        Gtk::TreeModel::Children children = m_nickModel->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row[m_nickColumns.m_name] == buddy)
            {
                row[m_nickColumns.m_name] = nbuddy.getName();
                row[m_nickColumns.m_buddy] = nbuddy;
                row[m_nickColumns.m_tooltip] = createBuddyTooltip(nbuddy);
            }
        }
    }
}

void ChatTabItem::onChatBuddyUpdated(PURPLE::Conversation *conv, Glib::ustring buddy)
{
    if(m_conv == conv)
    {
        Gtk::TreeModel::Children children = m_nickModel->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row[m_nickColumns.m_name] == buddy)
            {
                PURPLE::ChatBuddy cb = m_conv->findChatBuddy(buddy);
                if(cb.isValid())
                {
                    row[m_nickColumns.m_icon] = dxutils::getChatRoleIcon(cb.getChatRole());
                    row[m_nickColumns.m_buddy] = cb;
                    row[m_nickColumns.m_tooltip] = createBuddyTooltip(cb);
                }
            }
        }
    }
}

Glib::ustring ChatTabItem::createBuddyTooltip(PURPLE::ChatBuddy buddy)
{
    Glib::ustring tip = buddy.getName();
    tip.append(1, '\n');
    if(!buddy.getRealName().empty()) tip.append(Glib::ustring::compose(_("ID: %1\n"), buddy.getRealName()));
    tip.append(Glib::ustring::compose(_("Role: %1"), dxutils::getChatRoleString(buddy.getChatRole())));
    return tip;
}

void ChatTabItem::onChatTopic(PURPLE::Conversation *conv, Glib::ustring /*who*/, Glib::ustring topic)
{
    if(m_conv == conv)
    {
        m_topicline.set_text(topic);
        m_topicline.set_tooltip_text(topic);
    }
}

Glib::ustring ChatTabItem::getCompletion(Glib::ustring line)
{
    if(line.empty())
    {
        return (*m_nickModel->children().begin())[m_nickColumns.m_name]+DXPREFERENCES.m_nickCompletionChar+" ";
    }
    Glib::ustring toCompletion = line.lowercase();
    Glib::ustring nick = "";
    Gtk::TreeModel::Children children = m_nickModel->children();
    size_t j = 0;
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter, j++)
    {
        nick = (*iter)[m_nickColumns.m_name];
        nick = nick.lowercase();
        if(toCompletion.compare(nick.substr(0, toCompletion.bytes())) == 0)
        {
            return (*iter)[m_nickColumns.m_name]+DXPREFERENCES.m_nickCompletionChar+" ";
        }
        else if(toCompletion.substr(0, toCompletion.find(DXPREFERENCES.m_nickCompletionChar)).compare(nick) == 0)
        {
            if((j+1) < children.size())
            {
                iter++;
                return (*iter)[m_nickColumns.m_name]+DXPREFERENCES.m_nickCompletionChar+" ";
            }
            else
            {
                iter = children.begin();
                return (*iter)[m_nickColumns.m_name]+DXPREFERENCES.m_nickCompletionChar+" ";
            }
        }
    }
    return line;
}

Glib::ustring ChatTabItem::getCompletion(Glib::ustring line, int& cursorPos)
{
    if(line[cursorPos] == ' ') cursorPos = cursorPos-1;
    int pos = line.substr(0, cursorPos).find_last_of(' ')+1;
    int n = line.find(' ', cursorPos)!=Glib::ustring::npos ? line.find(' ', cursorPos)-pos : line.length()-pos;
    Glib::ustring toCompletion = line.substr(pos, n).lowercase();
    Glib::ustring nick = "";
    Gtk::TreeModel::Children children = m_nickModel->children();
    size_t j = 0;
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter, j++)
    {
        nick = (*iter)[m_nickColumns.m_name];
        nick = nick.lowercase();
        if(toCompletion.compare(nick) == 0)
        {
            if((j+1) < children.size())
            {
                iter++;
            }
            else
            {
                iter = children.begin();
            }
            nick = (*iter)[m_nickColumns.m_name];
            cursorPos = pos+nick.length();
            return line.replace(pos, n, nick);
        }
        else if(toCompletion.compare(nick.substr(0, toCompletion.bytes())) == 0)
        {
            nick = (*iter)[m_nickColumns.m_name];
            cursorPos = pos+nick.length();
            return line.replace(pos, n, nick);
        }

    }
    return line;
}

bool ChatTabItem::hasBuddy(const Glib::ustring& buddy)
{
    Gtk::TreeModel::Children children = m_nickModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row[m_nickColumns.m_name] == buddy)
        {
            return true;
        }
    }
    return false;
}


