/*
 *      chattabitem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CHATTABITEM_H
#define CHATTABITEM_H

#include "convtabitem.h"

class ChatTabItem : public ConvTabItem {
public:
    ChatTabItem(PURPLE::Conversation *conv, dxgCallbacks *cb, Gtk::Notebook *parent, ImageLabel *label);

    void showHideUsers();
    void removeHighligth();
    void updatePreferences();
private:
    Gtk::HPaned m_splitter;
    dxgCallbacks *m_cb;
    Glib::RefPtr<Gtk::ListStore> m_nickModel;
    Glib::RefPtr<Gtk::TreeModelSort> m_nickSort;
    class ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
    public:
        ModelColumns() { add(m_icon); add(m_name); add(m_buddy); add(m_tooltip); }
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_icon;
        Gtk::TreeModelColumn<Glib::ustring> m_name;
        Gtk::TreeModelColumn<PURPLE::ChatBuddy> m_buddy;
        Gtk::TreeModelColumn<Glib::ustring> m_tooltip;
    };
    ModelColumns m_nickColumns;
    Gtk::TreeView m_nicks;
    Gtk::Entry m_topicline;
    ImageLabel *m_id;

    //slot start
    void onTopicline();
    bool onButtonPressed(GdkEventButton *event);
    void onPopupIm();
    void onPopupInfo();
    void onPopupIgnore();
    //slot end
    //purplemm slot start
    void onNewMyMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg, bool action, Glib::ustring messageID);
    void onNewChatMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring sender, Glib::ustring msg, bool delayed, bool highlight, bool action);
    void onNewConvSystemMessage(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg);
    void onNewConvErrorMessage(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg);
    void onChatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy);
    void onChatBuddyUpdated(PURPLE::Conversation *conv, Glib::ustring buddy);
    void onChatBuddyLeft(PURPLE::Conversation *conv, Glib::ustring buddy);
    void onChatBuddyRenamed(PURPLE::Conversation *conv, Glib::ustring buddy, PURPLE::ChatBuddy nbuddy);
    void onChatTopic(PURPLE::Conversation *conv, Glib::ustring who, Glib::ustring topic);
    //purplemm slot end
    /** Completion on start of line*/
    Glib::ustring getCompletion(Glib::ustring line);
    /** Completion in cursorPos of line*/
    Glib::ustring getCompletion(Glib::ustring line, int &cursorPos);
    Glib::ustring createBuddyTooltip(PURPLE::ChatBuddy buddy);
    bool hasBuddy(const Glib::ustring& buddy);
};

#endif  /* CHATTABITEM_H */

