/*
 *      convtabitem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CONVTABITEM_H
#define CONVTABITEM_H

#include <gtkmm.h>
#include "tabitem.h"
#include "dxpipe.h"
#include "callback/callback.h"
#include "../logger/logger.h"

class dxim;

enum IMStateType {
    STARTED=0, //started typing
    PAUSED, //paused typing
    STOPPED, //stopped typing
    ACTIVE, //imtabitem is opended or text sent
    LEFTIM, //buddy left IM
};

class ConvTabItem : public TabItem {
    friend class dxim;
public:
    ConvTabItem(PURPLE::Conversation *conv, Gtk::Notebook *parent, ImageLabel *label, TabType type);

    PURPLE::Conversation *getConversation() { return m_conv; }
    void updatePreferences();
    void focusCommandline();

    sigc::signal<void> signalHighlight;
    sigc::signal<void, PURPLE::Account, Glib::ustring, Glib::ustring> signalShowLog;
protected:
    Gtk::Entry m_commandline;
    PURPLE::Conversation *m_conv;
    Logger m_logger;
    std::vector<Glib::ustring> m_commands;
    std::vector<Glib::ustring> m_history;
    std::vector<Glib::ustring>::iterator m_it;
    time_t m_lastKeyPress;
    bool m_removeLastLine;
    IMStateType m_imState;
    dxPipe *m_pipe;
    bool m_sendPipe;
    std::vector<Glib::ustring> m_pipeStrings;
    sigc::connection m_conn;
#if GTKMM_MAJOR_VERSION == 3
    Glib::RefPtr<Gtk::CssProvider> m_styleProvider;
#endif

    //slot start
    void onCommandline();
    bool onKeyPress(GdkEventKey *event);
    void onInsertText(Glib::ustring const &text, int *position);
    void onPipe(Glib::ustring text);
    bool onPipeTimeout();
    //slot end

    void createTextTags();
    void appendNickText(const Glib::ustring &time, const Glib::ustring &nick, const Glib::ustring &msg, bool log=true);
    void appendMyMsg(const Glib::ustring &time, const Glib::ustring &msg);
    void appendStyledText(const Glib::ustring& msg, const Glib::ustring& style, Glib::ustring time="", bool log=true);
    void appendHistoryMsg(const Glib::ustring& nick, const Glib::ustring& msg, const Glib::ustring& time);
    void makeLastRowVisible(bool force=false);
    void processCommandReturn(int ret, const Glib::ustring &command);
    void startLogging();
    void showInternalHelp(const Glib::ustring &command);
    /** Completion on start of line*/
    virtual Glib::ustring getCompletion(Glib::ustring line) { return ""; }
    /** Completion in cursorPos of line*/
    virtual Glib::ustring getCompletion(Glib::ustring line, int &cursorPos) { return ""; }
    void removeLastLine();
    Glib::ustring getLastLine();
    void completeHelpCommand();
    Glib::ustring escapeText(Glib::ustring text);
    void listLogFiles();
};

#endif  /* CONVTABITEM_H */

