/*
 *      fttabitem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef FTTABITEM_H
#define FTTABITEM_H

#include <gtkmm.h>
#include "tabitem.h"
#include "callback/callback.h"

class FTTabItem : public TabItem {
public:
    FTTabItem(dxgCallbacks *cb, Gtk::Notebook *parent, ImageLabel *label);

    void newFileTransfer(PURPLE::FileTransfer *ft);

    sigc::signal<void> signalFTTabItemClosed;
private:
    dxgCallbacks *m_cb;
    struct FTModelColumns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_icon;
        Gtk::TreeModelColumn<Glib::ustring> m_fileName;
        Gtk::TreeModelColumn<guint> m_progress;
        Gtk::TreeModelColumn<Glib::ustring> m_progressText;
        Gtk::TreeModelColumn<Glib::ustring> m_remainingTime;
        Gtk::TreeModelColumn<PURPLE::FileTransfer*> m_ft;

        FTModelColumns() { add(m_icon); add(m_fileName); add(m_progress);
            add(m_progressText), add(m_remainingTime); add(m_ft); }
    };
    const FTModelColumns m_ftColumns;
    Glib::RefPtr<Gtk::ListStore> m_ftModel;
    Glib::RefPtr<Gtk::TreeSelection> m_ftSelectionModel;
    Gtk::TreeView m_fts;
    Gtk::CellRendererProgress m_ftProgress;
    Gtk::Button *m_buttonClose, *m_buttonClearFinished, *m_buttonClearCanceled;
    Gtk::Expander *m_details;
    Gtk::Label *m_path, *m_account, *m_buddy, *m_direction;
    ImageLabel *m_protocol;

    //slot start
    void onClose();
    void onClearFinished();
    void onClearCanceled();
    void onFTChanged();
    bool onButtonPressed(GdkEventButton *event);
    void onPopupCancel();
    //purplemm
    void onFileTransferFinished(PURPLE::FileTransfer *ft);
    void onFileTransferStarted(PURPLE::FileTransfer *ft);
    void onFileTransferProgress(PURPLE::FileTransfer *ft);
    void onFileTransferCanceled(PURPLE::FileTransfer *ft);
    //slot end
};

#endif  /* FTTABITEM_H */

