/*
 *      imtabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "imtabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "preferences.h"

ImTabItem::ImTabItem(PURPLE::Conversation *conv, dxgCallbacks *cb, Gtk::Notebook *parent,
        ImageLabel *label) :
ConvTabItem(conv, parent, label, TABIM), m_cb(cb)
{
    PURPLE::Buddy buddy(m_conv->getAccount().findBuddy(m_conv->getName()));
    m_id = Gtk::manage(new ImageLabel(dxutils::getPresenceIcon(buddy.isValid()?buddy.getStatusPrimitive():PURPLE_STATUS_OFFLINE), m_conv->getName()+" ("+m_conv->getAccount().getUsername()+")"));
    Gtk::HBox *box = Gtk::manage(new Gtk::HBox());
    box->pack_start(*m_id, Gtk::PACK_SHRINK);
    box->pack_start(*Gtk::manage(new Gtk::HBox()), Gtk::PACK_EXPAND_WIDGET);
    box->pack_start(*Gtk::manage(new Gtk::Image(dxutils::getProtocolIcon(m_conv->getAccount().getProtocol().getPurpleID()))), Gtk::PACK_SHRINK);
    pack_start(*box, false, true, 3);
    m_textscroll.set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    m_textscroll.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    m_textscroll.add(m_text);
    add(m_textscroll);
    pack_end(m_commandline, false, true, 0);
    show_all();

    m_cb->signalNewMyMessage.connect(sigc::mem_fun(this, &ImTabItem::onNewMyMsg));
    m_cb->signalMessageRecived.connect(sigc::mem_fun(this, &ImTabItem::onMessageRecived));
    m_cb->signalNewIMMessage.connect(sigc::mem_fun(this, &ImTabItem::onNewIMMsg));
    m_cb->signalNewConvSystemMessage.connect(sigc::mem_fun(this, &ImTabItem::onNewConvSystemMessage));
    m_cb->signalNewConvErrorMessage.connect(sigc::mem_fun(this, &ImTabItem::onNewConvErrorMessage));
    m_cb->signalBuddySignedOn.connect(sigc::mem_fun(this, &ImTabItem::onBuddySignedOn));
    m_cb->signalBuddySignedOff.connect(sigc::mem_fun(this, &ImTabItem::onBuddySignedOff));
    m_cb->signalBuddyStatusChanged.connect(sigc::mem_fun(this, &ImTabItem::onBuddyStatusChanged));
    m_cb->signalBuddyTyping.connect(sigc::mem_fun(this, &ImTabItem::onBuddyTyping));

    Glib::signal_timeout().connect(sigc::mem_fun(this, &ImTabItem::onTimeout), 2000);

    m_removeLastLine = false;

    if(DXPREFERENCES.m_logLoadCurrent) loadCurrentLog();
}

void ImTabItem::onNewMyMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg, bool action, Glib::ustring messageID)
{
    if(m_conv == conv)
    {
        if(m_removeLastLine) removeLastLine();
        m_removeLastLine = false;
        if(action) appendStyledText(m_conv->getMyNick()+" "+msg, "action", time);
        else appendMyMsg(time, msg);
        if(!messageID.empty())
        {
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
            Gtk::TextIter it = buffer->end();
            it.backward_char();
            buffer->create_mark(messageID, it);
        }
    }
}

void ImTabItem::onMessageRecived(PURPLE::Conversation *conv, Glib::ustring messageID)
{
    if(m_conv == conv)
    {
        Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
        Glib::RefPtr<Gtk::TextBuffer::Mark> mark = buffer->get_mark(messageID);
        buffer->insert(buffer->get_iter_at_mark(mark), " \u2713");
    }
}

void ImTabItem::onNewIMMsg(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring sender, Glib::ustring msg, bool highlight, bool action)
{
    if(m_conv == conv)
    {
        if(m_removeLastLine) removeLastLine();
        m_removeLastLine = false;
        if(highlight)
        {
            appendStyledText(sender+(action?" ":": ")+msg, "highlight", time);
        }
        else
        {
            if(action) appendStyledText(sender+" "+msg, "action", time);
            else appendNickText(time, sender, msg, true);
        }
        if(!isCurrent())
        {
            if(highlight)
            {
                m_label->setColor(DXPREFERENCES.m_hltabColor);
                m_status = HILIGHT;
            }
            if(m_status!=HILIGHT)
            {
                m_label->setColor(DXPREFERENCES.m_unreadtabColor);
                m_status = UNREAD;
            }
            m_label->setIcon(ICO_IMNEWMSG->get_pixbuf());
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(Glib::ustring::compose(_("New message on IM with %1"), m_label->getText()));
            if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
        }
        signalHighlight();
    }
}

void ImTabItem::onNewConvSystemMessage(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg)
{
    if(m_conv == conv)
    {
        if(m_removeLastLine) removeLastLine();
        m_removeLastLine = false;
        appendStyledText(msg, "user", time);
    }
}

void ImTabItem::onNewConvErrorMessage(PURPLE::Conversation *conv, Glib::ustring time, Glib::ustring msg)
{
    if(m_conv == conv)
    {
        if(m_removeLastLine) removeLastLine();
        m_removeLastLine = false;
        appendStyledText(msg, "error", time);
    }
}

void ImTabItem::removeHighligth()
{
    m_label->clearColor();
    m_label->setIcon(m_conv->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?ICO_IMOFF->get_pixbuf():ICO_IM->get_pixbuf());
    m_status = STANDARD;
}

void ImTabItem::onBuddySignedOn(PURPLE::Buddy buddy)
{
    if(m_conv->getAccount().findBuddy(m_conv->getName()) == buddy)
    {
        m_id->setIcon(dxutils::getPresenceIcon(buddy.getStatusPrimitive()));
        m_label->setIcon(ICO_IM->get_pixbuf());
    }
}

void ImTabItem::onBuddySignedOff(PURPLE::Buddy buddy)
{
    if(m_conv->getAccount().findBuddy(m_conv->getName()) == buddy)
    {
        m_id->setIcon(dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE));
        m_label->setIcon(ICO_IMOFF->get_pixbuf());
    }
}

void ImTabItem::onBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive status)
{
    if(m_conv->getAccount().findBuddy(m_conv->getName()) == buddy)
        m_id->setIcon(dxutils::getPresenceIcon(status));
}

void ImTabItem::onBuddyTyping(PURPLE::Conversation *conv, bool typing)
{
    if(m_conv == conv)
    {
        if(m_removeLastLine) removeLastLine();
        m_removeLastLine = true;
        appendStyledText(dxutils::typingToString(m_label->getText(), typing), "user", "", false);
        if(m_status!=HILIGHT)
        {
            if(typing) m_label->setIcon(ICO_IMCOMPOSE->get_pixbuf());
            else m_label->setIcon(ICO_IM->get_pixbuf());
        }
    }
}

bool ImTabItem::onTimeout()
{
    if(!m_lastKeyPress) return true;
    if(dxutils::now()-m_lastKeyPress>5)
    {
        if(m_imState==STARTED)
        {
            m_imState = PAUSED;
            m_conv->sendTyping(false);
        }
        return true;
    }
    else
    {
        if(m_imState==ACTIVE || m_imState==PAUSED)
        {
            m_imState = STARTED;
            m_conv->sendTyping(true);
        }
        return true;
    }
    return true;
}

void ImTabItem::loadCurrentLog()
{
    clearText();
    Glib::ustring folder = m_conv->getName();
    folder = folder.substr(0, folder.find_last_of('/'));
    Glib::ustring username = m_conv->getAccount().getUsername();
    username = username.substr(0, username.find_last_of('/'));
    Glib::ustring path = DXPREFERENCES.m_logPath+G_DIR_SEPARATOR_S+username+G_DIR_SEPARATOR_S+folder+G_DIR_SEPARATOR_S+purple_utf8_strftime("%Y-%m-%d", NULL);
    if(!dxutils::isRightLogFile(path, purple_utf8_strftime("%Y-%m-%d", NULL)))
    {
        return;
    }
    std::ifstream file(path.c_str());
    if(!file.is_open())
    {
        return;
    }
    std::string line;
    while(getline(file, line))
    {
        Glib::ustring::size_type pos = line.find(']')+2;
        if(pos<2) pos = 0;
        if(line.length()>pos+1 && line[pos]=='\014') //log style mark
        {
            Glib::ustring style = dxutils::stringSection(line, '\014', 1);
            if(dxutils::stringSection(line, '\014', 2, true)[0]=='<') //has nick
            {
                Glib::ustring nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
                Glib::ustring msg = dxutils::stringSection(line, '>',1, true);
                Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
                buffer->insert(buffer->end(), dxutils::stringSection(line, '\014', 0));
                buffer->insert_with_tag(buffer->end(), Glib::ustring::compose("%1%2", nick, DXPREFERENCES.m_nickCompletionChar), DXPREFERENCES.m_coloredNick?dxutils::getNickColor(nick):"bold");
                parseHtml(msg, style.c_str());
            }
            else
            {
                Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
                buffer->insert(buffer->end(), dxutils::stringSection(line, '\014', 0));
                parseHtml(dxutils::stringSection(line, '\014', 2, true), style.c_str());
            }
        }
        else
        {
            if(line.length()>pos+1 && line[pos]=='<') //line with nick
            {
                Glib::ustring nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
                Glib::ustring msg = dxutils::stringSection(line, '>',1, true);
                Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
                buffer->insert(buffer->end(), dxutils::stringSection(line, '<', 0));
                buffer->insert_with_tag(buffer->end(), Glib::ustring::compose("%1%2", nick, DXPREFERENCES.m_nickCompletionChar), DXPREFERENCES.m_coloredNick?dxutils::getNickColor(nick):"bold");
                parseHtml(msg, "");
            }
            else
            {
                parseHtml(line, "");
            }
        }
    }
    makeLastRowVisible(true);
    file.close();
}

