/*
 *      msgtabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <fstream>
#include "msgtabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "utilsmm.h"
#include "preferences.h"

MsgTabItem::MsgTabItem(PurpleNotifyMsgType type, Glib::ustring id, Glib::ustring text, Glib::ustring okText, Glib::ustring url, Gtk::Notebook *parent, ImageLabel *label) :
TabItem(parent, label, TABMSG), m_url(url), m_id(id)
{
    Gtk::HBox *icobox = Gtk::manage(new Gtk::HBox);
    Gtk::Image *icon = NULL;
    switch(type){
        case PURPLE_NOTIFY_MSG_ERROR:
            icon = Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "errorbig.png"));
            break;
        case PURPLE_NOTIFY_MSG_INFO:
            icon = Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "infobig.png"));
            break;
        case PURPLE_NOTIFY_MSG_WARNING:
            icon = Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "warningbig.png"));
            break;
    };
    icobox->pack_start(*icon, Gtk::PACK_SHRINK, 5);
    icobox->pack_start(*Gtk::manage(new Gtk::HBox()), Gtk::PACK_EXPAND_WIDGET);
    pack_start(*icobox, Gtk::PACK_SHRINK, 5);
    m_textscroll.set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    m_textscroll.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    m_textscroll.add(m_text);
    pack_start(m_textscroll);
    appendText(text);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    m_okButton = new Gtk::Button(okText, true);
    m_okButton->signal_clicked().connect(sigc::mem_fun(this, &MsgTabItem::onOk));
    btnbox->pack_end(*m_okButton, Gtk::PACK_SHRINK);
    pack_start(*btnbox, false, false, 3);
    show_all();
}

//For show log files
MsgTabItem::MsgTabItem(Glib::ustring id, Glib::ustring okText, Gtk::Notebook *parent, ImageLabel *label) :
TabItem(parent, label, TABMSG), m_url(""), m_id(id)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    Glib::RefPtr<Gtk::TextTag> tag;
    tag = buffer->create_tag("nick0");
    tag->property_foreground().set_value("#C4A000");
    tag = buffer->create_tag("nick1");
    tag->property_foreground().set_value("#CE5C00");
    tag = buffer->create_tag("nick2");
    tag->property_foreground().set_value("#8F5902");
    tag = buffer->create_tag("nick3");
    tag->property_foreground().set_value("#4E9A06");
    tag = buffer->create_tag("nick4");
    tag->property_foreground().set_value("#204A87");
    tag = buffer->create_tag("nick5");
    tag->property_foreground().set_value("#75507B");
    tag = buffer->create_tag("nick6");
    tag->property_foreground().set_value("#A40000");
    tag = buffer->create_tag("nick7");
    tag->property_foreground().set_value("#555753");
    tag = buffer->create_tag("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = buffer->create_tag("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = buffer->create_tag("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = buffer->create_tag("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = buffer->create_tag("logs");
    tag->property_foreground().set_value(DXPREFERENCES.m_logColor.to_string());
    tag = buffer->create_tag("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
    m_textscroll.set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    m_textscroll.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    m_textscroll.add(m_text);
    pack_start(m_textscroll);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    m_okButton = new Gtk::Button(okText, true);
    m_okButton->signal_clicked().connect(sigc::mem_fun(this, &MsgTabItem::onOk));
    btnbox->pack_end(*m_okButton, Gtk::PACK_SHRINK);
    pack_start(*btnbox, false, false, 3);
    show_all();
}

void MsgTabItem::appendText(const Glib::ustring& text)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    buffer->insert_with_tag(buffer->end(), dxutils::time("%H:%M:%S"), "bold");
    buffer->insert(buffer->end(), "\n\n");
    parseHtml(text);
}

void MsgTabItem::loadLog(const Glib::ustring &path)
{
    clearMsgTab();
    std::ifstream file(path.c_str());
    if(!file.is_open())
    {
        Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
        buffer->insert_with_tag(buffer->end(), Glib::ustring::compose(_("Unable to open file: %1"), path), "error");
        return;
    }
    parseHtml(Glib::ustring("<b><u>")+Glib::ustring::compose((_("Log from %1:")), dxutils::fileName(path))+Glib::ustring("</u></b>"));
    std::string buf;
    while(getline(file, buf))
    {
        appendLogLine(buf);
    }
    file.close();
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    m_text.scroll_to(buffer->create_mark("end", buffer->end(), false), 0);
}

void MsgTabItem::clearMsgTab()
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    buffer->erase(buffer->begin(), buffer->end());
}

void MsgTabItem::onOk()
{
    if(!m_url.empty()) dxutils::launchLink(m_url);
    int i = 0;
    for(; i<m_parent->get_n_pages(); i++)
    {
        if(this == m_parent->get_nth_page(i))
            break;
    }
    signalMsgTabItemClosed(i);
}

void MsgTabItem::appendLogLine(const Glib::ustring &line)
{
    Glib::ustring::size_type pos = line.find(']')+2;
    if(pos<2) pos = 0;
    if(line.length()>pos+1 && line[pos]=='\014') //log style mark
    {
        Glib::ustring style = dxutils::stringSection(line, '\014', 1);
        if(dxutils::stringSection(line, '\014', 2, true)[0]=='<') //has nick
        {
            Glib::ustring nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
            Glib::ustring msg = dxutils::stringSection(line, '>',1, true);
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
            buffer->insert(buffer->end(), dxutils::stringSection(line, '\014', 0));
            buffer->insert_with_tag(buffer->end(), Glib::ustring::compose("%1:", nick), DXPREFERENCES.m_coloredNick?dxutils::getNickColor(nick).c_str():"bold");
            parseHtml(msg, style.c_str());
        }
        else
        {
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
            buffer->insert(buffer->end(), dxutils::stringSection(line, '\014', 0));
            parseHtml(dxutils::stringSection(line, '\014', 2, true), style.c_str());
        }
    }
    else
    {
        if(line.length()>pos+1 && line[pos]=='<') //line with nick
        {
            Glib::ustring nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
            Glib::ustring msg = dxutils::stringSection(line, '>',1, true);
            Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
            buffer->insert(buffer->end(), dxutils::stringSection(line, '<', 0));
            buffer->insert_with_tag(buffer->end(), Glib::ustring::compose("%1:", nick), DXPREFERENCES.m_coloredNick?dxutils::getNickColor(nick).c_str():"bold");
            parseHtml(msg, "");
        }
        else
        {
            parseHtml(line, "");
        }
    }
}


