/*
 *      tabbook.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "tabbook.h"
#include "imtabitem.h"
#include "chattabitem.h"
#include "msgtabitem.h"
#include "icons.h"
#include "dxutils.h"


Tabbook::Tabbook()
{
    set_tab_pos(dxutils::getTabPosition());
    set_scrollable(true);
}

ImTabItem* Tabbook::createImTab(PURPLE::Conversation *conv, dxgCallbacks* cb)
{
    ImageLabel *label = Gtk::manage(new ImageLabel(conv->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?ICO_IMOFF->get_pixbuf():ICO_IM->get_pixbuf(), conv->getTitle()));
    ImTabItem *tab = Gtk::manage(new ImTabItem(conv, cb, this, label));
    append_page(*tab, *label);
//    set_current_page(-1);
    return tab;
}

ChatTabItem* Tabbook::createChatTab(PURPLE::Conversation *conv, dxgCallbacks* cb)
{
    ImageLabel *label = Gtk::manage(new ImageLabel(ICO_CHAT->get_pixbuf(), conv->getTitle()));
    ChatTabItem *tab = Gtk::manage(new ChatTabItem(conv, cb, this, label));
    append_page(*tab, *label);
//    set_current_page(-1);
    return tab;
}

MsgTabItem* Tabbook::createMsgTab(Gtk::Image *icon, PurpleNotifyMsgType type, Glib::ustring id, Glib::ustring title, Glib::ustring text, Glib::ustring okText, Glib::ustring url)
{
    ImageLabel *label = Gtk::manage(new ImageLabel(icon->get_pixbuf(), title));
    MsgTabItem *tab = Gtk::manage(new MsgTabItem(type, id, text, okText, url, this, label));
    append_page(*tab, *label);
    set_current_page(-1);
    return tab;
}

MsgTabItem *Tabbook::createMsgTab(const Gtk::StockID &icon, Glib::ustring id, Glib::ustring title, Glib::ustring okText)
{
    ImageLabel *label = Gtk::manage(new ImageLabel(icon, title));
    MsgTabItem *tab = Gtk::manage(new MsgTabItem(id, okText, this, label));
    append_page(*tab, *label);
    set_current_page(-1);
    return tab;
}
