/*
 *      tabitem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef TABITEM_H
#define TABITEM_H

#include <gtkmm.h>
#include "tabbook.h"
#include "widgets/imagelabel.h"

enum TabType {
    TABIM,
    TABCHAT,
    TABMSG, //for warning, error, userinfo etc.
    TABFILETRANSFER
};

class TabItem : public Gtk::VBox {
public:
    TabItem(Gtk::Notebook *parent, ImageLabel *label, TabType type);

    enum TabStatus {
        STANDARD,
        UNREAD,
        HILIGHT
    };

    TabStatus getStatus() const { return m_status; }
    TabType getType() const { return m_type; }
    void clearText();
    virtual void showHideUsers() {}
    virtual void removeHighligth() {}
    virtual void updatePreferences();
    virtual void focusCommandline() {}
    bool isHighlighted() { return m_label->isColored(); }
protected:
    Gtk::TextView m_text;
    Gtk::Notebook *m_parent;
    ImageLabel *m_label;
    TabStatus m_status;
    TabType m_type;
    Glib::RefPtr<Gtk::TextTag> m_link;
    bool m_hovering;
#if GTKMM_MAJOR_VERSION == 2
    Gdk::Cursor m_default;
    Gdk::Cursor m_hand;
#else
    Glib::RefPtr<Gdk::Cursor> m_default;
    Glib::RefPtr<Gdk::Cursor> m_hand;
#endif

    //slots start
    bool onMotionEvent(GdkEventMotion *event);
    void onEventAfter(GdkEvent* event);
    //slots end
    bool isCurrent();
    void parseHtml(Glib::ustring text, const Glib::ustring &style="");
    void appendText(const Glib::ustring &text, bool bold, bool underline, bool italic, dxColor fore, dxColor back);
    Glib::ustring createSmileys(Glib::ustring text);
};

#endif  /* TABITEM_H */

