/*
 *      dxcolor.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "dxcolor.h"

dxColor::dxColor()
    : m_color()
{}

dxColor::dxColor(const Glib::ustring &value)
    : m_color(value)
{}

#if GTKMM_MAJOR_VERSION == 2
    dxColor::dxColor(Gdk::Color color)
    : m_color(color)
    {}
#else
    dxColor::dxColor(Gdk::RGBA color)
    : m_color(color)
    {}
#endif

Glib::ustring dxColor::to_string() const
{
#if GTKMM_MAJOR_VERSION == 2
    return m_color.to_string();
#else
    gchar *s = g_strdup_printf("#%04X%04X%04X", m_color.get_red_u(),
            m_color.get_green_u(),
            m_color.get_blue_u());
    Glib::ustring retStr = s;
    g_free(s);
    return retStr;
#endif
}

bool dxColor::set(const Glib::ustring& spec)
{
    return m_color.set(spec);
}

bool dxColor::operator==(const dxColor& color) const
{
    return this->m_color == color.m_color;
}

bool dxColor::operator!=(const dxColor& color) const
{
    return this->m_color != color.m_color;
}

gushort dxColor::get_red() const
{
#if GTKMM_MAJOR_VERSION == 2
    return m_color.get_red();
#else
    return m_color.get_red_u();
#endif
}

gushort dxColor::get_green() const
{
#if GTKMM_MAJOR_VERSION == 2
    return m_color.get_green();
#else
    return m_color.get_green_u();
#endif    
}

gushort dxColor::get_blue() const
{
#if GTKMM_MAJOR_VERSION == 2
    return m_color.get_blue();
#else
    return m_color.get_blue_u();
#endif    
}

void dxColor::set_red(gushort value)
{
#if GTKMM_MAJOR_VERSION == 2
    m_color.set_red(value);
#else
    m_color.set_red_u(value);
#endif
}

void dxColor::set_green(gushort value)
{
#if GTKMM_MAJOR_VERSION == 2
    m_color.set_green(value);
#else
    m_color.set_green_u(value);
#endif
}

void dxColor::set_blue(gushort value)
{
#if GTKMM_MAJOR_VERSION == 2
    m_color.set_blue(value);
#else
    m_color.set_blue_u(value);
#endif
}

