/*
 *      imagelabel.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "imagelabel.h"
#include "dxutils.h"

ImageLabel::ImageLabel()
 : m_text(""), m_image(Glib::RefPtr<Gdk::Pixbuf>()), m_label("")
{
    pack_start(m_image, Gtk::PACK_SHRINK);
    pack_start(m_label, Gtk::PACK_SHRINK);
    show_all();
}

ImageLabel::ImageLabel(const Glib::ustring &text)
 : m_text(text), m_image(Glib::RefPtr<Gdk::Pixbuf>()), m_label(text)
{
    pack_start(m_image, Gtk::PACK_SHRINK);
    pack_start(m_label, Gtk::PACK_SHRINK);
    show_all();
}

ImageLabel::ImageLabel(const Glib::RefPtr<Gdk::Pixbuf> &icon, const Glib::ustring &text)
 : m_text(text), m_image(icon), m_label(text)
{
    pack_start(m_image, Gtk::PACK_SHRINK);
    pack_start(m_label, Gtk::PACK_SHRINK);
    show_all();
}

ImageLabel::ImageLabel(const Gtk::StockID &icon, const Glib::ustring &text)
 : m_text(text), m_image(icon, Gtk::ICON_SIZE_MENU), m_label(text)
{
    pack_start(m_image, Gtk::PACK_SHRINK);
    pack_start(m_label, Gtk::PACK_SHRINK);
    show_all();
}

void ImageLabel::setIcon(const Glib::RefPtr<Gdk::Pixbuf> &icon)
{
    m_image.set(icon);
}

void ImageLabel::setText(const Glib::ustring &text)
{
    m_text = text;
    m_label.set_text(m_text);
}

void ImageLabel::setColor(const dxColor &color)
{
    m_label.set_markup("<span foreground=\"" + color.to_string() + "\">" + m_text + "</span>");
}

void ImageLabel::clearColor()
{
    m_label.set_text(m_text);
}

bool ImageLabel::isColored()
{
    return dxutils::stringContains(m_label.get_label(), "span");
}
