/*
 *      linklabel.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "linklabel.h"
#include "dxutils.h"

#if GTKMM_MAJOR_VERSION == 2
LinkLabel::LinkLabel()
 : m_text(""), m_label(""), m_hand(Gdk::HAND2)
{
 	create();
}

LinkLabel::LinkLabel(const Glib::ustring &text)
 : m_text(text), m_label(text), m_hand(Gdk::HAND2)
{
 	create();
}
#else
LinkLabel::LinkLabel()
 : m_text(""), m_label("")
{
    m_hand = Gdk::Cursor::create(Gdk::HAND2);
    create();
}

LinkLabel::LinkLabel(const Glib::ustring &text)
 : m_text(text), m_label(text)
{
    m_hand = Gdk::Cursor::create(Gdk::HAND2);
    create();
}
#endif

void LinkLabel::create()
{
    signal_button_release_event().connect(sigc::mem_fun(this, &LinkLabel::onClick));
    signal_enter_notify_event().connect(sigc::mem_fun(this, &LinkLabel::onEnterNotify));
    add(m_label);
    m_label.set_ellipsize(Pango::ELLIPSIZE_MIDDLE);
    show_all();
}

void LinkLabel::setText(const Glib::ustring &text)
{
    m_text = text;
    m_label.set_markup("<span foreground=\"" + m_color.to_string() + "\">" + m_text + "</span>");
}

void LinkLabel::setColor(const dxColor &color)
{
    m_color = color;
    m_label.set_markup("<span foreground=\"" + m_color.to_string() + "\">" + m_text + "</span>");
}

bool LinkLabel::onEnterNotify(GdkEventCrossing* /*event*/)
{
    m_label.get_window()->set_cursor(m_hand);
    return true;
}

bool LinkLabel::onClick(GdkEventButton* /*event*/)
{
    dxutils::launchLink(m_text);
    return true;
}
