/*
 *      logger.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <ctime>
#include <glib.h>
#include <utilsmm.h>
#include "logger.h"

std::string getName()
{
    time_t time = ::time(NULL);
    char buffer[512];
    int len=strftime(buffer, sizeof(buffer), "%Y-%m-%d", localtime(&time));
    return std::string(buffer,len);
}

Logger::Logger()
: m_path(""), m_logstream(NULL)
{}

Logger::Logger(const std::string &path)
: m_path(path), m_logstream(NULL)
{}

Logger::~Logger()
{
    stopLogging();
}

//set path
void Logger::setPath(const std::string &path)
{
    if(m_path != path)
    {
        stopLogging();
        m_path = path;
        startLogging();
    }
}

//start logging
void Logger::startLogging()
{
    std::string name = getName();
    std::string fullPath = m_path+G_DIR_SEPARATOR_S+name;
    if(m_logstream && g_file_test(fullPath.c_str(), G_FILE_TEST_EXISTS))
        return;
    if(g_file_test(fullPath.c_str(), G_FILE_TEST_EXISTS))
    {
        m_logstream = new std::ofstream(fullPath.c_str(), std::ios::out|std::ios::app);
        return;
    }
    if(g_file_test(m_path.c_str(), (GFileTest)(G_FILE_TEST_IS_DIR | G_FILE_TEST_EXISTS)))
    {
        m_logstream = new std::ofstream(fullPath.c_str(), std::ios::out|std::ios::app);
        return;
    }
    if(g_mkdir_with_parents(m_path.c_str(), 0755) != -1)
    {
        m_logstream = new std::ofstream(fullPath.c_str(), std::ios::out|std::ios::app);
    }
}

//stop logging
void Logger::stopLogging()
{
    if(m_logstream)
    {
        m_logstream->close();
        delete m_logstream;
        m_logstream = NULL;
    }
}

void Logger::logLine(const std::string &time, const std::string &msg)
{
    startLogging();
    if(isReady()) *m_logstream << time << " " << msg << std::endl;
}

void Logger::logNickLine(const std::string& time, const std::string& nick, const std::string& msg)
{
    startLogging();
    if(isReady()) *m_logstream << time << "<" << nick << "> " << msg << std::endl;
}

void Logger::logMyMsgLine(const std::string& time, const std::string& nick, const std::string& msg)
{
    startLogging();
    if(isReady()) *m_logstream << time << "\014mymsg\014<" << nick << "> " << msg << std::endl;
}

void Logger::logStyledLine(const std::string& time, const std::string& style, const std::string& msg)
{
    startLogging();
    if(isReady()) *m_logstream << time << "\014" << style << "\014" << msg << std::endl;
}

bool Logger::isReady()
{
    return m_logstream;
}
