/*
 *      logger.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef LOGGER_H
#define LOGGER_H

#include <string>
#include <fstream>

class Logger
{
public:
    Logger();
    Logger(const std::string &path);
    virtual ~Logger();

    void setPath(const std::string &path);
    bool isReady();
    void startLogging();
    void stopLogging();
    /** Log line
     * @param time      Message time, format "[%time] "
     * @param nick      Who sent the message
     * @param msg       Message
     */
    void logNickLine(const std::string &time, const std::string &nick, const std::string &msg);
    /** Log line
     * @param time      Message time, format "[%time] "
     * @param nick      My nick
     * @param msg       Message
     */
    void logMyMsgLine(const std::string &time, const std::string &nick, const std::string &msg);
    /** Log line
     * @param time      Message time, format "[%time] "
     * @param style     Style, e.g "user", "action", "error"
     * @param msg       Message
     */
    void logStyledLine(const std::string &time, const std::string &style, const std::string &msg);
    /** Log line
     * @param time      Message time, format "[%time] "
     * @param msg       Message
     */
    void logLine(const std::string &time, const std::string &msg);
private:
    std::string m_path;
    std::ofstream *m_logstream;
};

#endif  /* LOGGER_H */

