/*
 * buddymm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef BUDDYMM_H
#define BUDDYMM_H

#include <purple.h>
#include <string>
#include <vector>
#include "exportmm.h"

namespace PURPLE
{
class Account;

/** This class represents a buddy. */
class PURPLEMM_EXPORT Buddy
{
    PurpleBuddy* m_buddy;

    static void* getHandler();
    static PurpleBlistUiOps blistOps;
    static void buddySignonOn(PurpleBuddy* buddy);
    static void buddySignonOff(PurpleBuddy* buddy);
    static void buddyStatusChanged(PurpleBuddy *buddy, PurpleStatus *old_status, PurpleStatus *status);
    static void buddyIconChanged(PurpleBuddy* buddy);
    static void removedNode(PurpleBuddyList *list, PurpleBlistNode *node);
public:
    /** This class represents a buddy menu item. */
    class PURPLEMM_EXPORT BuddyMenuItem {
    private:
        std::string m_label;
        PurpleCallback m_cb;
        void *m_userData;
        void *m_node;
    public:
        /** Empty constructor */
        BuddyMenuItem() {}
        /** Create menu item instance */
        BuddyMenuItem(PurpleMenuAction *action, void *node);

        /** Get menu item label */
        std::string getLabel() const { return m_label; }
        /** Run menu item callback */
        void runCallback();
    };

    /** Initialization of libpurple buddies' stuffs. */
    static void init();
    /** Uninitialization of libpurple buddies' stuffs. */
    static void uninit();

    /** Empty constructor */
    Buddy();
    /** Create a Buddy instance
     *
     * @param buddy  the libpurple's buddy instance.
     */
    Buddy(PurpleBuddy* buddy);

    /** Comparaisons */
    bool operator==(const Buddy& buddy) const;
    /** Comparaisons */
    bool operator!=(const Buddy& buddy) const;
    /** Get true if Buddy is valid */
    bool isValid() const { return m_buddy != NULL; }
    /** Get buddy username */
    std::string getUsername() const;
    /** Get buddy alias */
    std::string getAlias() const;
    /** Get server-specified buddy alias (i.e. MSN "Friendly Names") */
    std::string getServerAlias() const;
    /** Set buddy alias */
    void setAlias(std::string alias, bool serverSide = true) const;
    /** Get buddy real name */
    std::string getRealName() const;
    /** Get buddy group */
    std::string getGroupName() const;
    /** Ask server to send server info.
     *
     * It'll be showed with notifyUserinfo callback.
     */
    void retrieveInfo() const;
    /** Send a file to this buddy.
     * filename will be requested by Callbacks
    */
    void sendFile();
    /** Get buddy is online */
    bool isOnline() const;
    /** Get buddy is available */
    bool isAvailable() const;
    /** Get buddy status as PurpleStatusPrimitive */
    PurpleStatusPrimitive getStatusPrimitive() const;
    /** Get buddy status name, ex. Online */
    std::string getStatusName() const;
    /** Get buddy status msg */
    std::string getStatusMsg() const;
    /** Get buddy idle time */
    time_t getIdleTime() const;
    /** Get path to the icon. */
    std::string getIconPath() const;
    /** Get buddy PurpleGroup */
    PurpleGroup* getPurpleGroup() const;
    /** Get buddy PurpleBuddy */
    PurpleBuddy* getPurpleBuddy() const { return m_buddy; }
    /** Get account this buddy is from. */
    Account getAccount() const;
    /** Get buddy menu items */
    std::vector<BuddyMenuItem> getMenuItems() const;
};

} /* namespace PURPLE */

#endif /* BUDDYMM_H */
