/*
 * chatmm.h
 *
 * Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CHATMM_H
#define CHATMM_H

#include <purple.h>
#include <string>
#include "exportmm.h"
#include "accountmm.h"

namespace PURPLE
{
class Account;

/** This class represents a chat. */
class PURPLEMM_EXPORT Chat
{
    PurpleChat* m_chat;
    bool m_autojoin;
public:
    /** Empty constructor */
    Chat();
    /** Create a Chat instance
     *
     * @param chat  the libpurple's chat instance.
     */
    Chat(PurpleChat* chat);

    /** Comparaisons */
    bool operator==(const Chat& chat) const;
    /** Comparaisons */
    bool operator!=(const Chat& chat) const;
    /** Get true if Chat is valid */
    bool isValid() const { return m_chat != NULL; }
    /** Get chat alias */
    std::string getAlias() const;
    /** Set chat alias */
    void setAlias(std::string alias);
    /** Get chat autojoin */
    bool getAutojoin() const { return m_autojoin; }
    /** Set chat autojoin */
    void setAutojoin(bool autojoin);
    /** Get chat group */
    std::string getGroupName() const;
    /** Get parameters list */
    std::vector<Account::ChatEntry> getChatParameters() const;
    /** Set parameters list */
    void setChatParameters(std::vector<Account::ChatEntry> parameters);
    /** Get chat PurpleGroup */
    PurpleGroup* getPurpleGroup() const;
    /** Get chat PurpleChat */
    PurpleChat* getPurpleChat() const { return m_chat; }
    /** Get account this chat is from. */
    Account getAccount() const;
    /** Get chat name */
    std::string getChatName() const;
    /** Join to chat */
    void join() const;
};

} /* namespace PURPLE */

#endif /* CHATMM_H */
