/*
 * filetransfermm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FILETRANSFERMM_H
#define FILETRANSFERMM_H

#include <purple.h>
#include <string>
#include "exportmm.h"

namespace PURPLE
{

/** This class represents a file transfer. */
class PURPLEMM_EXPORT FileTransfer
{
    PurpleXfer* m_xfer;

    static PurpleXferUiOps ftOps;
    static void destroy(PurpleXfer* xfer);
    static void addXfer(PurpleXfer* xfer);
    static void updateProgress(PurpleXfer* xfer, double percent);
    static void cancelLocal(PurpleXfer* xfer);
    static void cancelRemote(PurpleXfer* xfer);
public:
    /** Initialization of libpurple filetransfer' stuffs. */
    static void init();
    /** Uninitialization of libpurple filetransfer' stuffs. */
    static void uninit();

    /** Empty constructor */
    FileTransfer();
    /** Create a FileTransfer instance
     *
     * @param xfer  the libpurple's xfer instance.
     */
    FileTransfer(PurpleXfer* xfer);

    /** Comparaisons */
    bool operator==(const FileTransfer& ft) const;
    /** Get true if FileTransfer is valid */
    bool isValid() const { return m_xfer != NULL; }
    /** Cancel file transfer */
    void cancelFT();
    /** Get file transfer remote user */
    std::string getRemoteUser() const;
    /** Get file transfer Account */
    Account getAccount() const;
    /** Get name sent over the network */
    std::string getFileName() const;
    /** Get name on the local hard drive */
    std::string getLocalFileName() const;
    /** Get size of the file */
    size_t getFileSize() const;
    /** Get number of bytes sent */
    size_t getBytesSent() const;
    /** Get speed b/s */
    size_t getSpeed() const;
    /** File sending */
    bool isSending() const;
    /** File receiving */
    bool isReceiving() const;
    /** File transfer started */
    bool isStarted() const;
    /** File transfer finished */
    bool isFinished() const;
    /** File transfer canceled */
    bool isCanceled() const;
private:
    std::string m_name;
    std::string m_path;
    Account m_account;
    std::string m_buddy;
    bool m_started;
    bool m_canceled;
    bool m_finished;
    bool m_download;
    time_t m_lastChange; //last time for m_bytesSent change
    size_t m_fileSize;
    size_t m_bytesSent;
    size_t m_speed; //b/s

    void updateStarted(bool started) { m_started = started; }
    void updateCanceled(bool canceled) { m_canceled = canceled; }
    void updateFinished(bool finished) { m_finished = finished; }
    void updateProgress(size_t bytesSent, double percent);
};

} /* namespace PURPLE */

#endif // FILETRANSFERMM_H
