/*
 * notifymm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "utilsmm.h"
#include "purplemm.h"
#include "notifymm.h"
#include "buddymm.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()

Notify::UserInfo::UserInfo()
: m_label(""), m_value("")
{}

Notify::UserInfo::UserInfo(std::string label, std::string value)
: m_label(label), m_value(value)
{}

void Notify::init()
{
    purple_notify_set_ui_ops(&notifyOps);
}

void Notify::uninit()
{
    purple_notify_set_ui_ops(NULL);
}

/* STATIC */

PurpleNotifyUiOps Notify::notifyOps =
{
    Notify::notifyMessage,
    Notify::notifyEmail,
    Notify::notifyEmails,
    Notify::notifyFormatted,
    Notify::notifySearchResults,
    Notify::notifySearchResultsNewRows,
    Notify::notifyUserinfo,
    Notify::notifyUri,
    NULL, /* close notify */
    NULL, /* reserved */
    NULL, /* reserved */
    NULL, /* reserved */
    NULL  /* reserved */
};

void* Notify::notifyMessage(PurpleNotifyMsgType type, const char *title, const char *primary, const char *secondary)
{
    if(CALLBACKS) CALLBACKS->notifyMessage(type, p2s(title), p2s(primary), p2s(secondary));
    return NULL;
}

void* Notify::notifyEmail(PurpleConnection */*gc*/, const char *subject, const char *from, const char *to, const char *url)
{
    if(CALLBACKS) CALLBACKS->notifyEmail(p2s(subject), p2s(from), p2s(to), p2s(url));
    return NULL;
}

void* Notify::notifyEmails(PurpleConnection */*gc*/, size_t count, gboolean /*detailed*/, const char **subjects, const char **froms, const char **tos, const char **urls)
{
    if(!count)
        return NULL;
    for(size_t i=0; i<count; i++)
    {
        if(CALLBACKS) CALLBACKS->notifyEmail(p2s(subjects[i]), p2s(froms[i]), p2s(tos[i]), p2s(urls[i]));
    }
    return NULL;
}

void* Notify::notifyFormatted(const char *title, const char *primary, const char *secondary, const char *text)
{
    if(CALLBACKS) CALLBACKS->notifyFormatted(p2s(title), p2s(primary), p2s(secondary), p2s(text));
    return NULL;
}

void* Notify::notifySearchResults(PurpleConnection */*gc*/, const char */*title*/, const char */*primary*/, const char */*secondary*/, PurpleNotifySearchResults */*results*/, gpointer /*user_data*/)
{
    return NULL;
}

void Notify::notifySearchResultsNewRows(PurpleConnection */*gc*/, PurpleNotifySearchResults */*results*/, void */*data*/)
{
}

void* Notify::notifyUserinfo(PurpleConnection *gc, const char *who, PurpleNotifyUserInfo *user_info)
{
    Account acc(purple_connection_get_account(gc));
    GList* l = purple_notify_user_info_get_entries(user_info);
    std::vector<UserInfo> info;
    for(; l != NULL; l = l->next)
    {
        PurpleNotifyUserInfoEntry *user_info_entry = (PurpleNotifyUserInfoEntry*)l->data;
        const char *label = purple_notify_user_info_entry_get_label(user_info_entry);
        const char *value = purple_notify_user_info_entry_get_value(user_info_entry);
        if(!label)
            continue;
        info.push_back(UserInfo(p2s(label), p2s(value)));
    }
    if(CALLBACKS && acc.isValid()) CALLBACKS->notifyUserinfo(acc, p2s(who), info);
    return NULL;
}

void* Notify::notifyUri(const char *uri)
{
    if(CALLBACKS) CALLBACKS->notifyURI(p2s(uri));
    return NULL;
}

} /* namespace PURPLE */

