/*
 * notifymm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef NOTIFYMM_H
#define NOTIFYMM_H

#include <purple.h>
#include "exportmm.h"

namespace PURPLE
{

/** This class represents a notification */
class PURPLEMM_EXPORT Notify
{
    static PurpleNotifyUiOps notifyOps;
    static void* notifyMessage(PurpleNotifyMsgType type, const char *title, const char *primary, const char *secondary);
    static void* notifyEmail(PurpleConnection *gc, const char *subject, const char *from, const char *to, const char *url);
    static void* notifyEmails(PurpleConnection *gc, size_t count, gboolean detailed, const char **subjects, const char **froms, const char **tos, const char **urls);
    static void* notifyFormatted(const char *title, const char *primary, const char *secondary, const char *text);
    static void* notifySearchResults(PurpleConnection *gc, const char *title, const char *primary, const char *secondary, PurpleNotifySearchResults *results, gpointer user_data);
    static void notifySearchResultsNewRows(PurpleConnection *gc, PurpleNotifySearchResults *results, void *data);
    static void* notifyUserinfo(PurpleConnection *gc, const char *who, PurpleNotifyUserInfo *user_info);
    static void* notifyUri(const char *uri);
public:
    /** This class represents an notification user information. */
    class PURPLEMM_EXPORT UserInfo {
    private:
        std::string m_label;
        std::string m_value;
    public:
        /** Empty constructor */
        UserInfo();
        /** Create a UserInfo instance */
        UserInfo(std::string label, std::string value);
        /** Get label */
        std::string getLabel() const { return m_label; }
        /** Get value */
        std::string getValue() const { return m_value; }
    };

    /** Initialization of libpurple notify' stuffs. */
    static void init();
    /** Uninitialization of libpurple notify' stuffs. */
    static void uninit();
};

} /* namespace PURPLE */

#endif // NOTIFYMM_H
