/*
 * protocolmm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PROTOCOLMM_H
#define PROTOCOLMM_H

#include <purple.h>
#include <string>
#include <vector>
#include <map>
#include "exportmm.h"

namespace PURPLE
{

/** This class represents a protocol. */
class PURPLEMM_EXPORT Protocol
{
PurplePlugin* m_plugin;
public:
/** This class represents a protocol option. */
class PURPLEMM_EXPORT Option
{
public:
    enum type_t {
        NONE,
        BOOL,
        STR,
        STRLIST,
        INT
    };
private:
    enum type_t m_type;
    std::string m_name;
    std::string m_value;
    std::string m_text;
    std::map<std::string, std::string> m_list;
public:
    Option(enum type_t type, std::string name, std::string text, std::string value = "");
    Option(PurplePrefType type, std::string name, std::string text, std::string value = "");
    Option();

    bool operator==(std::string s) const;

    enum type_t getType() const { return m_type; }
    std::string getName() const { return m_name; }
    std::string getText() const { return m_text; }

    void setValue(std::string v) { m_value = v; }
    void setStringList(std::map<std::string, std::string> list) { m_list = list; }
    std::string getStringValue() const { return m_value; }
    int getIntValue() const;
    bool getBoolValue() const;
    /** first  key usefull text in GUI
     * second  value of option
     */
    std::map<std::string, std::string> getStringList() const;
};

/** A username split.
 * This is used by some protocols to separate the fields of the username
 * into more human-readable components.
*/
class PURPLEMM_EXPORT UsernameSplit
{
    private:
        std::string m_text;
        std::string m_defaultValue;
        char m_separator;
        bool m_reverse;
    public:
        UsernameSplit();
        UsernameSplit(std::string text, std::string defaultValue, char separator, bool reverse);

        /** Text that will appear to the user */
        std::string getText() const;
        /** Default value */
        std::string getDefaultValue() const;
        /** Separator */
        char getSeparator() const;
        /** TRUE if the separator should be found starting at the end of the string */
        bool getReverse() const;
};

typedef std::vector<Option> Options;

Protocol(PurplePlugin* plugin);
Protocol();

/** Comparaison */
bool operator==(const Protocol& proto) const;
/** Comparaison */
bool operator!=(const Protocol& proto) const;
/** Get true if Protocol is valid */
bool isValid() const { return m_plugin != NULL; }
/** Get protocol name */
std::string getName() const;
/** Get protocol ID */
std::string getPurpleID() const;
/** Get protocol options */
Options getOptions() const;
/** Get username split */
std::vector<UsernameSplit> getUsernameSplit() const;
/** Get protocol plugin information structure */
PurplePluginProtocolInfo* getPurpleProtocol() const;
/** Get using check new mail */
bool hasCheckMail() const;
/** Get using buddy icon */
bool hasBuddyIcon() const;
};

} /* namespace PURPLE */

#endif /* PROTOCOLMM_H */
