/*
 *      purplemm.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <iostream>
#include "purplemm.h"
#include "utilsmm.h"
#include "eventloopmm.h"
#include "notifymm.h"
#include "soundmm.h"
#include "requestmm.h"
#include "pi18n.h"

namespace PURPLE
{

Purple* Purple::m_pInstance = NULL;
GHashTable *Purple::uiInfo = NULL;
PurpleEventLoopUiOps *Purple::eventOps = NULL;

Purple::Purple(Callbacks *callbacks)
: m_callbacks(callbacks), m_name(PNAME), m_version(PVERSION), m_website(PWEBSITE)
{
    eventOps = getEventLoopUiOps();
    m_configPath = getDefaultConfigPath();
    m_idle = true;
    m_idleTime = 5;
    purple_util_set_user_dir(m_configPath.c_str());
}

Purple::Purple(Callbacks *callbacks, PurpleEventLoopUiOps *events)
: m_callbacks(callbacks), m_name(PNAME), m_version(PVERSION), m_website(PWEBSITE)
{
    eventOps = events;
    m_configPath = getDefaultConfigPath();
    m_idle = true;
    m_idleTime = 5;
    purple_util_set_user_dir(m_configPath.c_str());
}

Purple::Purple(Callbacks *callbacks, std::string name, std::string version, std::string website, std::string configPath)
: m_callbacks(callbacks), m_name(name), m_version(version), m_website(website), m_configPath(configPath)
{
    eventOps = getEventLoopUiOps();
    m_idle = true;
    m_idleTime = 5;
    purple_util_set_user_dir(m_configPath.c_str());
}

Purple::Purple(Callbacks *callbacks, PurpleEventLoopUiOps *events, std::string name, std::string version, std::string website, std::string configPath)
: m_callbacks(callbacks), m_name(name), m_version(version), m_website(website), m_configPath(configPath)
{
    eventOps = events;
    m_idle = true;
    m_idleTime = 5;
    purple_util_set_user_dir(m_configPath.c_str());
}

Purple::~Purple()
{
    m_pInstance = NULL;
}

PurpleCoreUiOps Purple::coreOps =
{
    NULL,
    NULL,
    Purple::init,
    Purple::uninit,
    Purple::getUiInfo,
    NULL,
    NULL,
    NULL
};

PurpleDebugUiOps Purple::debugOps =
{
    Purple::printDebug,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL
};

Purple* Purple::instance()
{
    return m_pInstance;
}

std::string Purple::getPurplemmVersion()
{
    return PVERSION;
}

std::string Purple::getLibpurpleVersion()
{
    return p2s(purple_core_get_version());
}

Callbacks* Purple::getCallbacks()
{
    return m_callbacks;
}

bool Purple::initPurple(bool offline)
{
    if(m_pInstance)
        return false;
#ifdef ENABLE_NLS
    bindtextdomain("pidgin", LOCALEDIR);
#endif //ENABLE_NLS
    m_pInstance = this;
    m_offline = offline;
    bool ret;
    purple_debug_set_ui_ops(&debugOps);
    purple_core_set_ui_ops(&coreOps);
    purple_eventloop_set_ui_ops(eventOps);
    ret = purple_core_init(m_name.c_str());
    if(ret)
    {
        purple_set_blist(purple_blist_new());
        purple_blist_load();
        purple_prefs_load();
        purple_prefs_set_bool("/purple/away/away_when_idle", m_idle);
        purple_prefs_set_int("/purple/away/mins_before_away", m_idleTime);
        purple_prefs_set_string("/purple/away/idle_reporting", m_idle?"system":"never");
        /* Disable all logging */
        purple_prefs_set_bool("/purple/logging/log_ims", false);
        purple_prefs_set_bool("/purple/logging/log_chats", false);
        purple_prefs_set_bool("/purple/logging/log_system", false);
        if(getCallbacks())
        {
            getCallbacks()->buddylistLoaded();
        }
    }
    return ret;
}

bool Purple::isOffline() const
{
    return m_offline;
}

void Purple::setIdle(bool idle, int idleTime)
{
    m_idle = idle;
    m_idleTime = idleTime;
    purple_prefs_set_bool("/purple/away/away_when_idle", m_idle);
    purple_prefs_set_int("/purple/away/mins_before_away", m_idleTime);
    purple_prefs_set_string("/purple/away/idle_reporting", m_idle?"system":"never");
}

bool Purple::isIdle() const
{
    return m_idle;
}

int Purple::getIdleTime() const
{
    return m_idleTime;
}

void Purple::idleTouch()
{
    purple_idle_touch();
}

void Purple::idleSet(time_t time)
{
    purple_idle_set(time);
}

void Purple::printDebug(PurpleDebugLevel level, const char *category, const char *arg_s)
{
    std::string debug = "";
    switch(level) {
        case PURPLE_DEBUG_ALL: debug = "ALL "; break;
        case PURPLE_DEBUG_MISC: debug = "MISC "; break;
        case PURPLE_DEBUG_INFO: debug = "DEBUG "; break;
        case PURPLE_DEBUG_WARNING: debug = "WARNING "; break;
        case PURPLE_DEBUG_ERROR: debug = "ERROR "; break;
        case PURPLE_DEBUG_FATAL: debug = "FATAL ";
    }
    debug += "[";
    debug += category;
    debug += "] ";
    debug += arg_s;
    if(instance()->getCallbacks())
        instance()->getCallbacks()->showDebug(debug);
}

GHashTable *Purple::getUiInfo()
{
    if(uiInfo == NULL)
    {
        uiInfo = g_hash_table_new(g_str_hash, g_str_equal);
        g_hash_table_insert(uiInfo, g_strdup("name"),           g_strdup(instance()->getName().c_str()));
        g_hash_table_insert(uiInfo, g_strdup("version"),        g_strdup(instance()->getVersion().c_str()));
        g_hash_table_insert(uiInfo, g_strdup("website"),        g_strdup(instance()->getWebsite().c_str()));
        g_hash_table_insert(uiInfo, g_strdup("dev_website"),    g_strdup(instance()->getWebsite().c_str()));
        g_hash_table_insert(uiInfo, g_strdup("client_type"),    g_strdup("pc"));
    }
    return uiInfo;
}

void Purple::init()
{
    Account::init();
    Buddy::init();
    Conversation::init();
    FileTransfer::init();
    RoomList::init();
    Notify::init();
    Sound::init();
    Request::init();
    if(!instance()->isOffline()) connectAccounts();
}

void Purple::uninit()
{
    if(uiInfo)
        g_hash_table_destroy(uiInfo);
    Account::uninit();
    Buddy::uninit();
    Conversation::uninit();
    FileTransfer::uninit();
    RoomList::uninit();
    Notify::uninit();
    Sound::uninit();
    Request::uninit();
}

void Purple::connectAccounts()
{
    GList *l = purple_accounts_get_all_active();
    while(l)
    {
        PurpleAccount *account = (PurpleAccount*)l->data;
        purple_account_set_enabled(account, instance()->getName().c_str(), true);
        l = l->next;
    }
    g_list_free(l);
}

std::vector<Plugin> Purple::getPluginsList()
{
    std::vector<Plugin> v;
    GList* list;
    purple_plugins_probe(G_MODULE_SUFFIX);
    for(list = purple_plugins_get_all(); list; list = list->next)
    {
        PurplePlugin* plugin = (PurplePlugin*)list->data;
        if(plugin->info->type == PURPLE_PLUGIN_LOADER)
        {
            GList *cur;
            for (cur = PURPLE_PLUGIN_LOADER_INFO(plugin)->exts; cur != NULL; cur = cur->next)
                purple_plugins_probe((const char*)cur->data);
            continue;
        }
        if(plugin->info->type != PURPLE_PLUGIN_STANDARD ||
            plugin->info->flags & PURPLE_PLUGIN_FLAG_INVISIBLE)
            continue;
        v.push_back(Plugin(plugin));
    }
    return v;
}

std::vector<Protocol> Purple::getProtocolsList()
{
    std::vector<Protocol> v;
    GList* list = purple_plugins_get_protocols();
    for(; list; list = list->next)
    {
        v.push_back(Protocol((PurplePlugin*)list->data));
    }
    return v;
}

Protocol Purple::getProtocolById(std::string id)
{
    PurplePlugin *plugin = purple_plugins_find_with_id(id.c_str());
    if(plugin != NULL)
    {
        return Protocol(plugin);
    }
    return Protocol();
}

bool Purple::hasProtocol(std::string id)
{
    PurplePlugin *plugin = purple_plugins_find_with_id(id.c_str());
    if(plugin != NULL)
    {
        return true;
    }
    return false;
}

std::vector<Account> Purple::getAccountsList()
{
    std::vector<Account> m;
    GList* list = purple_accounts_get_all();
    PurplePlugin *plugin;
    for(; list; list = list->next)
    {
        plugin = purple_find_prpl(purple_account_get_protocol_id((PurpleAccount*)list->data));
        if(!plugin)
            continue;
        m.push_back(Account((PurpleAccount*)list->data));
    }
    return m;
}

std::vector<Account> Purple::getEnabledAccountsList()
{
    std::vector<Account> m;
    GList *l = purple_accounts_get_all_active();
    PurplePlugin *plugin;
    while(l)
    {
        plugin = purple_find_prpl(purple_account_get_protocol_id((PurpleAccount*)l->data));
        if(!plugin)
            continue;
        m.push_back(Account((PurpleAccount*)l->data));
        l = l->next;
    }
    g_list_free(l);
    return m;
}

std::vector<Account> Purple::getConnectedAccountsList()
{
    std::vector<Account> m;
    GList *l = purple_accounts_get_all_active();
    PurplePlugin *plugin;
    while(l)
    {
        if(purple_account_is_connected((PurpleAccount*)l->data))
        {
            plugin = purple_find_prpl(purple_account_get_protocol_id((PurpleAccount*)l->data));
            if(!plugin)
                continue;
            m.push_back(Account((PurpleAccount*)l->data));
        }
        l = l->next;
    }
    g_list_free(l);
    return m;
}

bool Purple::isSomeAccountConnected()
{
    bool response = false;
    GList *l = purple_accounts_get_all_active();
    PurplePlugin *plugin;
    while(l)
    {
        if(purple_account_is_connected((PurpleAccount*)l->data))
        {
            plugin = purple_find_prpl(purple_account_get_protocol_id((PurpleAccount*)l->data));
            if(!plugin)
                continue;
            response = true;
            break;
        }
        l = l->next;
    }
    g_list_free(l);
    return response;
}

Account Purple::addAccount(const Protocol& proto, const std::string& username, const std::string& password, bool rememberPassword, const Protocol::Options& options, bool registerAccount)
{
    PurpleAccount *account = purple_account_new(username.c_str(), proto.getPurpleID().c_str());
    Account a(account);
    try {
        a.setOptions(options);
    } catch(...) {
        purple_account_destroy(account);
        throw;
    }
    a.setPassword(password, rememberPassword);
    if(registerAccount)
        a.registerAccount();
    purple_accounts_add(account);
    const PurpleSavedStatus *saved_status;
    saved_status = purple_savedstatus_get_current();
    if(saved_status != NULL) {
        purple_savedstatus_activate_for_account(saved_status, account);
        purple_account_set_enabled(account, m_name.c_str(), TRUE);
    }
    return a;
}

void Purple::deleteAccount(Account account)
{
    if(account.isValid())
    {
        purple_request_close_with_handle(account.getPurpleAccount());
        purple_accounts_delete(account.getPurpleAccount());
    }
}

Account Purple::findAccount(const std::string& username, const std::string& protocol) const
{
    GList* list = purple_accounts_get_all();
    for(; list; list = list->next)
    {
        if(!protocol.compare(((PurpleAccount*)list->data)->protocol_id)
                && !username.compare(((PurpleAccount*)list->data)->username))
                return Account((PurpleAccount*)list->data);
    }
    return Account();
}

bool Purple::getBoolPref(const std::string& name, bool value) const
{
    if(purple_prefs_exists(name.c_str()))
        return purple_prefs_get_bool(name.c_str());
    return value;
}

int Purple::getIntPref(const std::string& name, int value) const
{
    if(purple_prefs_exists(name.c_str()))
        return purple_prefs_get_int(name.c_str());
    return value;
}

std::string Purple::getStringPref(const std::string& name, const std::string& value) const
{
    if(purple_prefs_exists(name.c_str()))
        return purple_prefs_get_string(name.c_str());
    return value;
}

void Purple::addNonePref(const std::string& name)
{
    purple_prefs_add_none(name.c_str());
}

void Purple::addBoolPref(const std::string& name, bool value)
{
    purple_prefs_add_bool(name.c_str(), value);
}

void Purple::addIntPref(const std::string& name, int value)
{
    purple_prefs_add_int(name.c_str(), value);
}

void Purple::addStringPref(const std::string& name, const std::string& value)
{
    purple_prefs_add_string(name.c_str(), value.c_str());
}

void Purple::setBoolPref(const std::string& name, bool value)
{
    purple_prefs_set_bool(name.c_str(), value);
}

void Purple::setIntPref(const std::string& name, int value)
{
    purple_prefs_set_int(name.c_str(), value);
}

void Purple::setStringPref(const std::string& name, const std::string& value)
{
    purple_prefs_set_string(name.c_str(), value.c_str());
}

void Purple::saveLoadedPlugins(const std::string& key)
{
    if(!purple_prefs_exists(key.c_str()))
    {
        std::vector<std::string> v = stringSplit(key, '/');
        std::string path;
        if(!v.empty())
        {
            for(size_t i=0; i<v.size()-1; i++)
            {
                path += "/";
                path += v[i];
                addNonePref(path);
            }
            path += "/";
            path += v[v.size()-1];
            purple_prefs_add_path_list(path.c_str(), NULL);
        }
    }
    purple_plugins_save_loaded(key.c_str());
}

void Purple::loadSavedPlugins(const std::string& key)
{
    purple_plugins_load_saved(key.c_str());
}

void Purple::createConversation(const Account &account, const Buddy &buddy)
{
    purple_conversation_new(PURPLE_CONV_TYPE_IM, account.getPurpleAccount(), buddy.getUsername().c_str());
}

void Purple::createConversation(const Account &account, const std::string &name, bool chat)
{
    if(chat) purple_conversation_new(PURPLE_CONV_TYPE_CHAT, account.getPurpleAccount(), name.c_str());
    else purple_conversation_new(PURPLE_CONV_TYPE_IM, account.getPurpleAccount(), name.c_str());
}

} /* namespace PURPLE */

