/*
 * filetransfermm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef ROOMLISTMM_H
#define ROOMLISTMM_H

#include <purple.h>
#include <string>
#include <vector>
#include "exportmm.h"

namespace PURPLE
{
class Account;

/** This class represents a list of rooms for a given connection on a given protocol */
class PURPLEMM_EXPORT RoomList
{
    PurpleRoomlist* m_rlist;

    static PurpleRoomlistUiOps rlistOps;
    static void create(PurpleRoomlist* list);
    static void setFields(PurpleRoomlist* list, GList* fields);
    static void addRoom(PurpleRoomlist*, PurpleRoomlistRoom* room);
    static void inProgress(PurpleRoomlist* list, gboolean flag);
    static void destroy(PurpleRoomlist* list);
public:
    /** Initialization of libpurple roomlist' stuffs. */
    static void init();
    /** Uninitialization of libpurple roomlist' stuffs. */
    static void uninit();

    /** Empty constructor */
    RoomList();
    /** Create a RoomList instance
     *
     * @param rlist  the libpurple's roomlist instance.
     */
    RoomList(PurpleRoomlist* rlist);

    /** Comparaisons */
    bool operator==(const RoomList& list) const;
    /** Get true if RoomList is valid */
    bool isValid() const { return m_rlist != NULL; }
    /** Get account this room list is from. */
    Account getAccount() const;
    /** Get PurpleRoomlist */
    PurpleRoomlist* getPurpleRoomlist() const { return m_rlist; }
    /** Cancel rooms listing */
    void cancelListing();
    /** Listing is in progress */
    bool isInProgress() const;
};

} /* namespace PURPLE */

#endif // ROOMLISTMM_H
