/*
 *      utilsmm.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fstream>
#include <ctime>
#include <glib.h>
#include <purple.h>
#ifndef __WIN32
#include <unistd.h>
#include <pwd.h>
#else
#ifndef _WIN32_LEAN_AND_MEAN
#define _WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
#endif
#include "utilsmm.h"

namespace PURPLE
{

std::string getDefaultConfigPath()
{
    return g_build_filename(g_get_user_config_dir(), "purplemm", NULL);
}

std::string stringtok(std::string &in, const char * const delimiters)
{
    std::string::size_type i = 0;
    std::string s;
    // eat leading whitespace
    i = in.find_first_not_of (delimiters, i);
    // find the end of the token
    std::string::size_type j = in.find_first_of (delimiters, i);
    if(j == std::string::npos)
    {
        if(i == std::string::npos)
            s = "";
        else
            s = in.substr(i);
        in = "";
        return s;  // nothing left but white space
    }
    // push token
    s = in.substr(i, j-i);
    in = in.substr(j+1);
    return s;
}

std::vector<std::string> stringSplit(const std::string &text, char splitter)
{
    std::vector<std::string> v;
    std::size_t prev = 0, pos;
    while((pos = text.find_first_of(splitter, prev)) != std::string::npos)
    {
        if(pos > prev)
            v.push_back(text.substr(prev, pos-prev));
        prev = pos+1;
    }
    if(prev < text.length())
        v.push_back(text.substr(prev, std::string::npos));
    return v;
}

std::string stripHtml(std::string str)
{
    const char* temp = str.c_str();
    char* value = purple_markup_strip_html(temp);
    std::string final = std::string(value);
    g_free(value);
    return final;
}

std::string markupHtml(std::string str)
{
    const char* temp = str.c_str();
    char* value = purple_markup_linkify(temp);
    std::string final = std::string(value);
    g_free(value);
    return final;
}

time_t now()
{
    return ::time(NULL);
}

std::string time2s(const char *format, time_t time)
{
    char buffer[512];
    int len=strftime(buffer, sizeof(buffer), format, localtime(&time));
    return std::string(buffer,len);
}

std::string imagePathByID(int id)
{
    std::string path = "";
    PurpleStoredImage *img = purple_imgstore_find_by_id(id);
    if(img)
    {
        std::string name = t2s(id);
        name += time2s("%H%M%S", now());
        name += ".";
        name += purple_imgstore_get_extension(img);
        path = g_build_filename(g_get_tmp_dir(), name.c_str(), NULL);
        std::ofstream image(path.c_str(), std::ios_base::binary);
        image.write((const char *)purple_imgstore_get_data(img), purple_imgstore_get_size(img));
        image.close();
    }
    return path;
}

std::string imagePathByID(std::string id)
{
    std::string path = "";
    PurpleStoredImage *img = purple_imgstore_find_by_id(s2t<int>(id));
    if(img)
    {
        std::string name = id;
        name += time2s("%H%M%S", now());
        name += ".";
        name += purple_imgstore_get_extension(img);
        path = g_build_filename(g_get_tmp_dir(), name.c_str(), NULL);
        std::ofstream image(path.c_str(), std::ios_base::binary);
        image.write((const char *)purple_imgstore_get_data(img), purple_imgstore_get_size(img));
        image.close();
    }
    return path;
}

} /* namespace PURPLE */
