/*
 *      utilsmm.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef UTILSMM_H
#define UTILSMM_H

#include <string>
#include <vector>
#include <sstream>
#include "exportmm.h"

namespace PURPLE
{
std::string stringtok(std::string &in, const char * const delimiters);

std::vector<std::string> stringSplit(const std::string &text, char splitter);

/** Get default config path */
std::string getDefaultConfigPath();

template<typename T>
/** Someone from string */
T s2t(const std::string & Str)
{
    T Dest;
    std::istringstream iss( Str );
    iss >> Dest;
    return Dest;
}

template<typename T>
/** String from someone*/
std::string t2s( const T & Value )
{
    std::ostringstream oss;
    oss << Value;
    return oss.str();
}

/** Return a valid std::string from a pointer. */
static inline std::string p2s(const char* s) { return s ? s : ""; }

time_t now();

std::string time2s(const char *format, time_t time);

#define FOREACH(t, v, it) \
    for(t::iterator it = v.begin(); it < v.end(); it++)

/**
 * Strips HTML tags from a string.
 * @param str The string to strip HTML from.
 * @return The new string without HTML.
 */
PURPLEMM_EXPORT std::string stripHtml(std::string str);

/**
 * Adds the necessary HTML code in a string.
 * @param str The string to add HTML in.
 * @return The new string with HTML.
 */
PURPLEMM_EXPORT std::string markupHtml(std::string str);

/**
 * @param id ID of image file
 * @return path of image file, if id doesn't valid path is empty
 */
PURPLEMM_EXPORT std::string imagePathByID(int id);

/**
 * @param id ID of image file
 * @return path of image file, if id doesn't valid path is empty
 */
PURPLEMM_EXPORT std::string imagePathByID(std::string id);

} /* namespace PURPLE */

#endif // UTILSMM_H
