/*
 * purplemmtest.cpp
 * libpurplemm
 *
 * Created by David Vachulka.
 * Copyright (c) 2012 David Vachulka. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <src/purplemm.h>
#include <iostream>

#ifdef _WIN32
#define CUSTOM_USER_DIRECTORY  "c:/temp"
#define CUSTOM_PLUGIN_PATH     ""
#define PLUGIN_SAVE_PREF       "c:/temp/plugins/saved"
#else
#define CUSTOM_USER_DIRECTORY  "/tmp/purplemmtest"
#define CUSTOM_PLUGIN_PATH     ""
#define PLUGIN_SAVE_PREF       "/tmp/purplemmtest/plugins/saved"
#endif
#define UI_ID                  "purplemmtest"

#define FOREACH(t, v, it) \
    for(t::iterator it = v.begin(); it != v.end(); ++it)

class Callbacks: public PURPLE::Callbacks
{
public:
    //callbacks start
    /** Allow your application to be notified about errors on library layer */
    void showError(std::string msg)
    {}
    /** Allow your application to be notified about libpurple debug messages */
    void showDebug(std::string debug)
    {}
    /** Buddy list loaded */
    void buddylistLoaded()
    {
        std::cout << "Purplemm " << m_purple->getPurplemmVersion() << std::endl;
        std::cout << "LibPurple " << m_purple->getLibpurpleVersion() << std::endl;
    }
    /** New account added */
    void accountAdded(PURPLE::Account account)
    {}
    /** Account removed */
    void accountRemoved(std::string account, std::string protocolID)
    {}
    /** Account connecting */
    void accountConnecting(PURPLE::Account account, std::string msg)
    {
        std::cout << "Account " << account.getUsername() << " (" << account.getProtocol().getName() << ") " << msg << std::endl;
    }
    /** Acount connected*/
    void accountConnected(PURPLE::Account account)
    {
        std::cout << "Account " << account.getUsername() << " (" << account.getProtocol().getName() << ") connected" << std::endl;
    }
    /** Acount reason of disconnect */
    void accountDisconnected(PURPLE::Account account, std::string reason)
    {
        std::cout << "Account " << account.getUsername() << " (" << account.getProtocol().getName() << ") disconnected - " << reason << std::endl;
    }
    /** Account status changed*/
    void accountStatusChanged(PURPLE::Account account)
    {}
    /** Account signed on, so show buddy list */
    void accountSignedOn(PURPLE::Account account)
    {}
    /** Buddy signed on */
    void buddySignedOn(PURPLE::Buddy buddy)
    {}
    /** Buddy signed off */
    void buddySignedOff(PURPLE::Buddy buddy)
    {}
    /** Buddy idle changed
     * @param time  Time of idle
     */
    void buddyIdleChanged(PURPLE::Buddy buddy, time_t time)
    {}
    /** Buddy status changed */
    void buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus)
    {}
    /** Buddy icon changed */
    void buddyIconChanged(PURPLE::Buddy buddy)
    {}
    /** Buddy added */
    void buddyAdded(PURPLE::Buddy buddy)
    {}
    /** Buddy removed */
    void buddyRemoved(PURPLE::Account account, std::string buddy)
    {}
    /** Chat added */
    void chatAdded(PURPLE::Chat chat)
    {}
    /** Chat removed */
    void chatRemoved(PURPLE::Account account, std::string chat)
    {}
    /** New IM created */
    void newIM(PURPLE::Conversation *conv)
    {}
    /** New Chat created */
    void newChat(PURPLE::Conversation *conv)
    {}
    /** IM closed */
    void closedIM(PURPLE::Conversation *conv)
    {}
    /** Chat closed */
    void closedChat(PURPLE::Conversation *conv)
    {}
    /** New my message
     * @param conv          The conversation.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param action        Is message in 3rd person (/me)?
     * @param messageID     message ID, usefull for XEP-0184
     */
    void newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID)
    {}
    /** Message recived, usefull for XEP-0184
     * @param conv          The conversation.
     * @param messageID     message ID, usefull for XEP-0184
     */
    void messageRecived(PURPLE::Conversation *conv, std::string messageID)
    {}
    /** New IM message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
    {}
    /** New Chat message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
    {}
    /** New system mesage for conversation
     *
     * @param conv  The conversation
     * @param msg  The message text
     * @param time  The time the message was sent
     */
    void newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
    {}
    /** New error mesage for conversation
     *
     * @param conv      conversation
     * @param msg       message text
     * @param time      time the message was sent
     */
    void newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
    {}
    /** ChatBuddy joined chat */
    void chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
    {}
    /** ChatBuddy updated */
    void chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy)
    {}
    /** ChatBuddy left chat */
    void chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy)
    {}
    /** ChatBuddy renamed
     * @param obuddy  old buddy which was renamed
     * @param nbuddy  new buddy established by rename
     */
    void chatBuddyRenamed(PURPLE::Conversation *conv, std::string who, PURPLE::ChatBuddy nbuddy)
    {}
    /** Chat topic changed
     * @param who  topic changer
     * @param topic  new topic
     */
    void chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic)
    {}
    /** Buddy typing on IM
     * @param typing =true typing, =false stoped typing
     */
    void buddyTyping(PURPLE::Conversation *conv, bool typing)
    {}
    /** New FileTransfer */
    void newFileTransfer(PURPLE::FileTransfer *ft)
    {}
    /** FileTransfer finished */
    void fileTransferFinished(PURPLE::FileTransfer *ft)
    {}
    /** FileTransfer started */
    void fileTransferStarted(PURPLE::FileTransfer *ft)
    {}
    /** FileTransfer progress */
    void fileTransferProgress(PURPLE::FileTransfer *ft, double percent)
    {}
    /** FileTransfer canceled */
    void fileTransferCanceled(PURPLE::FileTransfer *ft)
    {}
    /** A new list was created */
    void roomlistCreated(PURPLE::RoomList *rlist)
    {}
    /** Sets the columns of RoomList */
    void roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns)
    {}
    /** Add a room to the list */
    void roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields)
    {}
    /** Listing RoomList finished */
    void roomlistFinished(PURPLE::RoomList *rlist)
    {}
    /** Notification message to the user
     * @param type  notification type
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
    */
    void notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary)
    {}
    /** Email notification to the user
     * @param subject  subject of the email
     * @param from  from address
     * @param to  estination address
     * @param url  URL where the message can be read
    */
    void notifyEmail(std::string subject, std::string from, std::string to, std::string url)
    {}
    /** Notification with formatted text
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
     * @param text  formatted text
    */
    void notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated)
    {}
    /** Notification information about buddy
     * @param account   Account requsted
     * @param buddy     Buddy which about
     * @param info      vector with UserInfo values
    */
    void notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info)
    {}
    /** Opens a URI or somehow presents it to the user
     * @param uri    The URI to display or go to.
    */
    void notifyURI(std::string uri)
    {}
    /** Play sound for given event */
    void playSound(PurpleSoundEventID event)
    {}
    /** New InputRequest */
    void newInputRequest(PURPLE::InputRequest *request)
    {}
    /** New ChoiceRequest */
    void newChoiceRequest(PURPLE::ChoiceRequest *request)
    {}
    /** New ActionRequest */
    void newActionRequest(PURPLE::ActionRequest *request)
    {}
    /** New FileRequest */
    void newFileRequest(PURPLE::FileRequest *request)
    {}
    /** New FieldsRequest */
    void newFieldsRequest(PURPLE::FieldsRequest *request)
    {}
    /** Close given request (ex. dialog) */
    void closeRequest(PURPLE::BasicRequest *request)
    {}
    // callbacks end

    void run()
    {
        m_purple = new PURPLE::Purple(this, UI_ID, "1.0.0", "http://dxim.dxsolutions.org", CUSTOM_USER_DIRECTORY);
        m_purple->initPurple();
        std::vector<PURPLE::Account> accs = m_purple->getAccountsList();
        if(!accs.size())
        {
            std::vector<PURPLE::Protocol> prpls = m_purple->getProtocolsList();
            FOREACH(std::vector<PURPLE::Protocol>, prpls, it)
            {
                if(!it->getName().compare("IRC"))
                {
                    char username[26];
                    sprintf(username, "mmtest%02d@irc.freenode.net", rand()%33);
                    m_purple->addAccount(*it, username, "", false, it->getOptions(), false);
                }
            }
        }
    }

    void handleCommnad(std::string cmd)
    {
        //TODO
        {
            fprintf(stderr, "You enter command \"%s\",\nbut we handle only 'quit'\n", cmd.c_str());
        }
    }

    PURPLE::Purple* getPurplemm() { return m_purple; }
private:
    PURPLE::Purple *m_purple;
};

static Callbacks *_cb = NULL;
static GIOChannel* g_stdin_channel = NULL;
static GMainLoop* g_loop = NULL;

static gboolean onStdinData(GIOChannel* source, G_GNUC_UNUSED GIOCondition condition, G_GNUC_UNUSED gpointer data)
{
      gchar* str = NULL;
      gsize len = 0;
      gsize term = 0;
      GError* err = NULL;
      g_io_channel_read_line (source, &str, &len, &term, &err);
      if(len > 0)
      {
            if(term) str[term] = 0;
            if(strcmp(str, "quit") == 0)
            {
                  g_main_loop_quit(g_loop);
            }
            else
            {
                _cb->handleCommnad(str);
            }
      }
      fflush(stdout);
      g_free(str);
      return true;
}

int main()
{
    g_loop = g_main_loop_new(NULL, FALSE);
    Callbacks cb;
    _cb = &cb;
    cb.run();
    g_stdin_channel = g_io_channel_unix_new(fileno(stdin));
    int tmp = 0;
    tmp |= G_IO_IN | G_IO_PRI;
    g_io_add_watch(g_stdin_channel, (GIOCondition)tmp, onStdinData, NULL);
    g_main_loop_run(g_loop);
    return 0;
}
