/*
 *      callback.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXQCALLBACKS_H
#define DXQCALLBACKS_H

#include <QObject>
#include <QVector>
#include <QPixmap>
#include <callbacksmm.h>
#include "defs.h"

class dxqCallbacks : public QObject, public PURPLE::Callbacks
{
    Q_OBJECT
    friend gpointer purpleMainEventLoop(gpointer data);
public:
    explicit dxqCallbacks(QObject *parent = 0);
    virtual ~dxqCallbacks();

    /** Allow your application to be notified about errors on library layer */
    void showError(std::string msg);
    /** Allow your application to be notified about libpurple debug messages */
    void showDebug(std::string debug);
    /** Buddy list loaded */
    void buddylistLoaded();
    /** New account added */
    void accountAdded(PURPLE::Account account);
    /** Account removed */
    void accountRemoved(std::string account, std::string protocolID);
    /** Account connecting */
    void accountConnecting(PURPLE::Account account, std::string msg);
    /** Acount connected*/
    void accountConnected(PURPLE::Account account);
    /** Acount disconnected*/
    void accountDisconnected(PURPLE::Account account, std::string reason);
    /** Account status changed*/
    void accountStatusChanged(PURPLE::Account account);
    /** Account signed on, so show buddy list */
    void accountSignedOn(PURPLE::Account account);
    /** Buddy signed on */
    void buddySignedOn(PURPLE::Buddy buddy);
    /** Buddy signed off */
    void buddySignedOff(PURPLE::Buddy buddy);
    /** Buddy status changed */
    void buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus);
    /** Buddy icon changed */
    void buddyIconChanged(PURPLE::Buddy buddy);
    /** Buddy added */
    void buddyAdded(PURPLE::Buddy buddy);
    /** Buddy removed */
    void buddyRemoved(PURPLE::Account account, std::string buddy);
    /** Chat added */
    void chatAdded(PURPLE::Chat chat);
    /** Chat removed */
    void chatRemoved(PURPLE::Account account, std::string chat);
    /** New IM created */
    void newIM(PURPLE::Conversation *conv);
    /** New Chat created */
    void newChat(PURPLE::Conversation *conv);
    /** IM closed */
    void closedIM(PURPLE::Conversation *conv);
    /** Chat closed */
    void closedChat(PURPLE::Conversation *conv);
    /** New my message
     * @param conv          The conversation.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param action        Is message in 3rd person (/me)?
     * @param messageID     message ID, usefull for XEP-0184
     */
    void newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID);
    /** Message recived, usefull for XEP-0184
     * @param conv          The conversation.
     * @param messageID     message ID, usefull for XEP-0184
     */
    void messageRecived(PURPLE::Conversation *conv, std::string messageID);
    /** New IM message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action);
    /** New Chat message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    void newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action);
    /** New system mesage for conversation
     *
     * @param conv  The conversation
     * @param msg  The message text
     * @param time  The time the message was sent
     */
    void newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time);
    /** New error mesage for conversation
     *
     * @param conv      conversation
     * @param msg       error text
     * @param time      time the error was sent
     */
    void newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time);
    /** ChatBuddy joined chat */
    void chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy);
    /** ChatBuddy updated */
    void chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy);
    /** ChatBuddy left chat */
    void chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy);
    /** ChatBuddy renamed
     * @param obuddy  old buddy which was renamed
     * @param nbuddy  new buddy established by rename
     */
    void chatBuddyRenamed(PURPLE::Conversation *conv, std::string buddy, PURPLE::ChatBuddy nbuddy);
    /** Chat topic changed
     * @param who  topic changer
     * @param topic  new topic
     */
    void chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic);
    /** Buddy typing on IM
     * @param typing =true typing, =false stoped typing
     */
    void buddyTyping(PURPLE::Conversation *conv, bool typing);
    /** New FileTransfer */
    void newFileTransfer(PURPLE::FileTransfer *ft);
    /** FileTransfer finished */
    void fileTransferFinished(PURPLE::FileTransfer *ft);
    /** FileTransfer started */
    void fileTransferStarted(PURPLE::FileTransfer *ft);
    /** FileTransfer progress */
    void fileTransferProgress(PURPLE::FileTransfer *ft, double percent);
    /** FileTransfer canceled */
    void fileTransferCanceled(PURPLE::FileTransfer *ft);
    /** A new list was created */
    void roomlistCreated(PURPLE::RoomList *rlist);
    /** Sets the columns of RoomList */
    void roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns);
    /** Add a room to the list */
    void roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields);
    /** Listing RoomList finished */
    void roomlistFinished(PURPLE::RoomList *rlist);
    /** Notification message to the user
     * @param type  notification type
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
    */
    void notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary);
    /** Email notification to the user
     * @param subject  subject of the email
     * @param from  from address
     * @param to  estination address
     * @param url  URL where the message can be read
    */
    void notifyEmail(std::string subject, std::string from, std::string to, std::string url);
    /** Notification with formatted text
     * @param title  title of the message
     * @param primary  main point of the message
     * @param secondary  secondary information
     * @param text  formatted text
    */
    void notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated);
    /** Notification information about buddy
     * @param account   requested account
     * @param buddy     Buddy which about
     * @param info      map<label,value>
    */
    void notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info);
    /** Opens a URI or somehow presents it to the user
     * @param uri    The URI to display or go to.
    */
    void notifyURI(std::string uri);
    /** Play sound for given event */
    void playSound(PurpleSoundEventID event);
    /** New InputRequest */
    void newInputRequest(PURPLE::InputRequest *request);
    /** New ChoiceRequest */
    void newChoiceRequest(PURPLE::ChoiceRequest *request);
    /** New ActionRequest */
    void newActionRequest(PURPLE::ActionRequest *request);
    /** New FileRequest */
    void newFileRequest(PURPLE::FileRequest *request);
    /** New FieldsRequest */
    void newFieldsRequest(PURPLE::FieldsRequest *request);
    /** Close given request (ex. dialog) */
    void closeRequest(PURPLE::BasicRequest *request);
signals:
    void signalPurpleError(QString error);
    void signalBuddylistLoaded();
    void signalAccountAdded(PURPLE::Account account);
    void signalAccountRemoved(QString account, QString protocolID);
    void signalAccountConnecting(PURPLE::Account account, QString msg);
    void signalAccountConnected(PURPLE::Account account);
    void signalAccountDisconnected(PURPLE::Account account, QString reason);
    void signalAccountStatusChanged(PURPLE::Account);
    void signalAccountSignedOn(PURPLE::Account);
    void signalNewMyMessage(PURPLE::Conversation *conv, QString msg, QString time, bool action, QString messageID);
    void signalMessageRecived(PURPLE::Conversation *conv, QString messageID);
    void signalNewMessage(PURPLE::Conversation *conv, QString sender, QString msg, QString time, bool highlight, bool history, bool action);
    void signalNewIM(PURPLE::Conversation *conv);
    void signalNewConvSystemMessage(PURPLE::Conversation *conv, QString msg, QString time);
    void signalNewConvErrorMessage(PURPLE::Conversation *conv, QString msg, QString time);
    void signalNewChat(PURPLE::Conversation *conv);
    void signalCloseConvTab(PURPLE::Conversation *conv);
    void signalChatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy);
    void signalChatBuddyUpdated(PURPLE::Conversation *conv, QString buddy);
    void signalChatBuddyLeft(PURPLE::Conversation *conv, QString buddy);
    void signalChatBuddyRenamed(PURPLE::Conversation *conv, QString buddy, PURPLE::ChatBuddy nbuddy);
    void signalChatTopicChanged(PURPLE::Conversation *conv, QString who, QString topic);
    void signalBuddyTyping(PURPLE::Conversation *conv, IMStateType typing);
    void signalBuddySignedOn(PURPLE::Buddy buddy);
    void signalBuddySignedOff(PURPLE::Buddy buddy);
    void signalBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus);
    void signalBuddyIconChanged(PURPLE::Buddy buddy);
    void signalBuddyAdded(PURPLE::Buddy buddy);
    void signalBuddyRemoved(PURPLE::Account account, QString buddy);
    void signalChatAdded(PURPLE::Chat chat);
    void signalChatRemoved(PURPLE::Account account, QString chat);
    void signalRoomlistCreated(PURPLE::RoomList *rlist);
    void signalRoomlistColumns(PURPLE::RoomList *rlist, QVector<QString> columns);
    void signalRoomlistAddRoom(PURPLE::RoomList *rlist, QVector<QString> fields);
    void signalRoomlistFinished(PURPLE::RoomList *rlist);
    void signalNotifyMessage(QPixmap icon, QString title, QString text);
    void signalNotifyEmail(QString text, QString url);
    void signalNotifyUserinfo(PURPLE::Account account, QString buddy, QString info);
    void signalNotifyURI(QString uri);
    void signalPlaySound(PurpleSoundEventID event);
    void signalNewInputRequest(PURPLE::InputRequest *request);
    void signalNewFileRequest(PURPLE::FileRequest *request);
    void signalNewActionRequest(PURPLE::ActionRequest *request);
    void signalNewChoiceRequest(PURPLE::ChoiceRequest *request);
    void signalNewFieldsRequest(PURPLE::FieldsRequest *request);
    void signalNewFT(PURPLE::FileTransfer *ft);
    void signalFinishedFT(PURPLE::FileTransfer *ft);
    void signalStartedFT(PURPLE::FileTransfer *ft);
    void signalProgressFT(PURPLE::FileTransfer *ft);
    void signalCanceledFT(PURPLE::FileTransfer *ft);
};

#endif // DXQCALLBACKS_H
