/*
 *      accountdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QStringList>
#include <QLabel>
#include <QHeaderView>
#include <QDialogButtonBox>
#include <QBoxLayout>
#include <QComboBox>
#include <QMessageBox>
#include "accountdialog.h"
#include "accounteditordialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

AccountDialog::AccountDialog(PURPLE::Purple *purple, QWidget *parent) :
    QDialog(parent), m_purple(purple)
{
    QStringList headers;
    headers << _("User name") << _("Protocol") << _("Enabled");
    m_accountView = new QTreeWidget(this);
    m_accountView->setHeaderLabels(headers);
    m_accountView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_accountView->setAlternatingRowColors(true);
    m_accountView->setRootIsDecorated(false);
    m_accountView->setSelectionMode(QAbstractItemView::SingleSelection);
    QFontMetrics fm = fontMetrics();
    QHeaderView *header = m_accountView->header();
#if QT_VERSION < QT_VERSION_CHECK(5,11,0)
    header->resizeSection(0, fm.width("ropotamo@jabber.cz/dxim")+20);
    header->resizeSection(1, fm.width(headers.at(1)+"  ")+20);
#else
    header->resizeSection(0, fm.horizontalAdvance("ropotamo@jabber.cz/dxim")+20);
    header->resizeSection(1, fm.horizontalAdvance(headers.at(1)+"  ")+20);
#endif
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Close);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    m_buttonEnable = new QPushButton(_("&Enable"));
    m_buttonEnable->setEnabled(false);
    buttons->addButton(m_buttonEnable, QDialogButtonBox::ActionRole);
    connect(m_buttonEnable, SIGNAL(clicked()), this, SLOT(slotEnableAccount()));
    QPushButton *buttonAdd = new QPushButton(_("&Add"));
    buttons->addButton(buttonAdd, QDialogButtonBox::ActionRole);
    connect(buttonAdd, SIGNAL(clicked()), this, SLOT(slotAddAccount()));
    m_buttonModify = new QPushButton(_("&Modify"));
    m_buttonModify->setEnabled(false);
    buttons->addButton(m_buttonModify, QDialogButtonBox::ActionRole);
    connect(m_buttonModify, SIGNAL(clicked()), this, SLOT(slotModifyAccount()));
    m_buttonDelete = new QPushButton(_("&Delete"));
    m_buttonDelete->setEnabled(false);
    buttons->addButton(m_buttonDelete, QDialogButtonBox::ActionRole);
    connect(m_buttonDelete, SIGNAL(clicked()), this, SLOT(slotDeleteAccount()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addWidget(m_accountView);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(_("Accounts list"));
    updateTable();

    connect(m_accountView, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(slotCurrentChanged(QTreeWidgetItem*)));
}

void AccountDialog::slotEnableAccount()
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    for(size_t i=0; i<accounts.size(); i++)
    {
        if(i == (size_t)m_accountView->currentIndex().row())
        {
            if(m_accountView->currentItem()->text(2) == _("Yes"))
            {
                accounts[i].disconnect();
                m_buttonEnable->setText(_("&Enable"));
            }
            else
            {
                accounts[i].connect();
                m_buttonEnable->setText(_("Di&sable"));
            }
        }
    }
    updateTable();
}

void AccountDialog::slotAddAccount()
{
    std::vector<PURPLE::Protocol> v = m_purple->getProtocolsList();
    QDialog protocols(this);
    QHBoxLayout *comboLayout = new QHBoxLayout;
    comboLayout->addWidget(new QLabel(_("Protocol:")));
    QComboBox *combo = new QComboBox;
    FOREACH(std::vector<PURPLE::Protocol>, v, it)
    {
        combo->addItem(QIcon(dxutils::protocolToIcon(dxutils::fromStdString(it->getPurpleID()))), dxutils::fromStdString(it->getName()));
    }
    comboLayout->addWidget(combo);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), &protocols, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), &protocols, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(&protocols);
    mainFrame->addLayout(comboLayout);
    mainFrame->addWidget(buttons);
    protocols.setLayout(mainFrame);
    protocols.setWindowTitle(_("Select protocol"));
    protocols.setModal(true);
    if(protocols.exec()==QDialog::Accepted)
    {
        FOREACH(std::vector<PURPLE::Protocol>, v, it)
        {
            if(!it->getName().compare(dxutils::toStdString(combo->currentText())))
            {
                AccountEditorDialog dialog(*it, false, this);
                dialog.setModal(true);
                if(dialog.exec()==QDialog::Accepted)
                {
                    NewAccount *account = new NewAccount;
                    dialog.fillAccount(account);
                    PURPLE::Account acc = m_purple->addAccount(account->protocol, account->username, account->password, account->rememberPassword, account->options, account->registerAccount);
                    acc.setCheckMail(account->checkEmail);
                    acc.setAlias(account->alias);
                    if(!account->iconPath.empty()) acc.setBuddyIcon(account->iconPath);
                    updateTable();
                }
                return;
            }
        }
        if(combo->currentText() == "Google Talk")
        {
            AccountEditorDialog dialog(m_purple->getProtocolById("prpl-jabber"), true, this);
            dialog.setModal(true);
            if(dialog.exec()==QDialog::Accepted)
            {
                NewAccount *account = new NewAccount;
                dialog.fillAccount(account);
                PURPLE::Account acc = m_purple->addAccount(account->protocol, account->username, account->password, account->rememberPassword, account->options, account->registerAccount);
                acc.setCheckMail(account->checkEmail);
                acc.setAlias(account->alias);
                if(!account->iconPath.empty()) acc.setBuddyIcon(account->iconPath);
                updateTable();
            }
            return;
        }
    }
}

void AccountDialog::slotModifyAccount()
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    for(size_t i=0; i<accounts.size(); i++)
    {
        if(i == (size_t)m_accountView->currentIndex().row())
        {
            AccountEditorDialog editor(accounts[i], this);
            editor.setModal(true);
            if(editor.exec()==QDialog::Accepted)
            {
                NewAccount *account = new NewAccount;
                editor.fillAccount(account);
                accounts[i].setOptions(account->options);
                accounts[i].setPassword(account->password, account->rememberPassword);
                accounts[i].setCheckMail(account->checkEmail);
                accounts[i].setAlias(account->alias);
                account->iconPath.empty()?accounts[i].removeBuddyIcon():accounts[i].setBuddyIcon(account->iconPath);
                updateTable();
            }
        }
    }
}

void AccountDialog::slotDeleteAccount()
{
    if(QMessageBox::question(this, _("Remove account"), _("Do you want really remove %1?").arg(m_accountView->currentItem()->text(0)), QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
    {
        std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
        for(size_t i=0; i<accounts.size(); i++)
        {
            if(i == (size_t)m_accountView->currentIndex().row())
            {
                m_purple->deleteAccount(accounts[i]);
            }
        }
        updateTable();
    }
}

void AccountDialog::slotCurrentChanged(QTreeWidgetItem *current)
{
    if(current)
    {
        if(current->text(2) == _("Yes"))
            m_buttonEnable->setText(_("Di&sable"));
        else
            m_buttonEnable->setText(_("&Enable"));
    }
}

void AccountDialog::updateTable()
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    if(accounts.size())
    {
        QTreeWidgetItem *current = NULL;
        m_accountView->clear();
        for(size_t i=0; i<accounts.size(); i++)
        {
            QTreeWidgetItem *item = new QTreeWidgetItem(m_accountView);
            item->setIcon(0, QIcon(dxutils::fromStdString(accounts[i].getBuddyIcon())));
            item->setText(0, dxutils::fromStdString(accounts[i].getUsername()));
            item->setIcon(1, QIcon(dxutils::protocolToIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID()))));
            item->setText(1, dxutils::fromStdString(accounts[i].getProtocol().getName()));
            item->setText(2, accounts[i].isEnabled()? _("Yes") : _("No"));
            item->setFlags(item->flags() & ~Qt::ItemIsEditable);
            if(i==0) current = item;
        }
        m_buttonDelete->setEnabled(true);
        m_buttonModify->setEnabled(true);
        m_buttonEnable->setEnabled(true);
        if(accounts[0].isConnected()) m_buttonEnable->setText(_("Di&sable"));
        else m_buttonEnable->setText(_("&Enable"));
        if(current) m_accountView->setCurrentItem(current);
    }
    else
    {
        m_buttonDelete->setEnabled(false);
        m_buttonModify->setEnabled(false);
        m_buttonEnable->setEnabled(false);
    }
}

