/*
 *      accounteditordialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ACCOUNTEDITORDIALOG_H
#define ACCOUNTEDITORDIALOG_H

#include <QDialog>
#include <QLabel>
#include <QLineEdit>
#include <QTabWidget>
#include <QCheckBox>
#include <QBoxLayout>
#include <QPushButton>
#include <purplemm.h>

struct NewAccount {
    PURPLE::Protocol protocol;
    PURPLE::Protocol::Options options;
    std::string username;
    std::string password;
    std::string iconPath;
    bool registerAccount;
    bool checkEmail;
    bool rememberPassword;
    std::string alias;
};

class AccountEditorDialog : public QDialog
{
    Q_OBJECT
public:
    /* Constructor for modifying account */
    AccountEditorDialog(PURPLE::Account account, QWidget *parent = 0);
    /* Constructor for creating new account */
    AccountEditorDialog(PURPLE::Protocol protocol, bool gtalk = false, QWidget *parent = 0);

    /* Fills new Account */
    void fillAccount(NewAccount *account);
private:
    PURPLE::Account m_account;  //use for modifying
    PURPLE::Protocol m_protocol; //use for creating
    PURPLE::Protocol::Options m_options;
    QLineEdit *m_password, *m_alias;
    QCheckBox *m_registerAccount, *m_checkEmail, *m_rememberPassword;
    bool m_editing;
    bool m_gtalk;
    QVector<QLineEdit*> m_usernameFields;
    QVector<QWidget*> m_optionsFields;
    QString m_iconPath;
    QHBoxLayout *m_iconBox;
    QPushButton *m_iconBtn, *m_removeIcon;
    QLabel *m_iconLabel;

    void initializeEditor();
    void createAccountTab(QTabWidget *tabs);
    void createProtocolTab(QTabWidget *tabs);
    std::string getStrlistValue(std::map<std::string, std::string> map, std::string key);
    void fillProtocolOptions();
    std::string joinUsername();
private slots:
    void slotIconPath();
    void slotRemoveIcon();
};

#endif // ACCOUNTEDITORDIALOG_H
