/*
 *      actionrequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QButtonGroup>
#include <QDialogButtonBox>
#include "dxutils.h"
#include "actionrequestdialog.h"
#include "config.h"
#include "i18n.h"

ActionRequestDialog::ActionRequestDialog(PURPLE::ActionRequest *request, QWidget *parent) :
    QDialog(parent), m_request(request)
{
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    if(!m_request->getPrimary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getPrimary())));
    if(!m_request->getSecondary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getSecondary())));
    QDialogButtonBox *buttons = new QDialogButtonBox(this);
    QButtonGroup *group = new QButtonGroup(this);
    connect(group, SIGNAL(buttonClicked(int)), this, SLOT(onButtonClick(int)));
    std::map<std::string, PurpleRequestActionCb> callbacks = m_request->getCallbacks();
    std::map<std::string,PurpleRequestActionCb>::iterator it;
    int i;
    for(it=callbacks.begin(),i=0; it != callbacks.end(); it++,i++)
    {
        QPushButton *button = buttons->addButton(dxutils::fromStdString((*it).first).replace('_', '&'), QDialogButtonBox::ActionRole);
        group->addButton(button, i);
    }
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(m_request->getTitle().empty()? _("Request") : dxutils::fromStdString(m_request->getTitle()));
}

void ActionRequestDialog::onButtonClick(int id)
{
    std::map<std::string, PurpleRequestActionCb> callbacks = m_request->getCallbacks();
    std::map<std::string,PurpleRequestActionCb>::iterator it;
    int i;
    for(it=callbacks.begin(),i=0; it != callbacks.end(); it++,i++)
    {
        if(id == i)
            m_request->runCallback((*it).second);
    }
    QDialog::accept();
}
