/*
 *      chatjoindialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QDialogButtonBox>
#include "defs.h"
#include "dxutils.h"
#include "chatjoindialog.h"
#include "roomlistdialog.h"
#include "config.h"
#include "i18n.h"

ChatJoinDialog::ChatJoinDialog(std::vector<PURPLE::Account> accounts, dxqCallbacks *cb, QWidget *parent) :
    QDialog(parent), m_accounts(accounts)
{
    //remove accounts !supportChats
    for(uint i=0; i<m_accounts.size(); i++)
    {
        if(!m_accounts[i].supportsChats())
        {
            m_accounts.erase(m_accounts.begin()+i);
        }
    }
    createControls(cb);
}

ChatJoinDialog::ChatJoinDialog(PURPLE::Account account, dxqCallbacks *cb, QWidget *parent) :
    QDialog(parent)
{
    m_accounts.push_back(account);
    createControls(cb);
}

void ChatJoinDialog::createControls(dxqCallbacks *cb)
{
    m_grid = new QGridLayout;
    m_IDs = new QComboBox;
    m_IDs->setInsertPolicy(QComboBox::NoInsert);
    for(uint i=0; i<m_accounts.size(); i++)
    {
        m_IDs->addItem(QIcon(dxutils::protocolToIcon(dxutils::fromStdString(m_accounts[i].getProtocol().getPurpleID()))), dxutils::fromStdString(m_accounts[i].getUsername()));
    }
    m_grid->addWidget(new QLabel(_("ID:")), 0, 0);
    m_grid->addWidget(m_IDs, 0, 1);
    createChatEntryFields(m_accounts[0].getChatParameters());
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    m_buttonList = new QPushButton(_("&Get chats list"));
    if(!m_accounts[0].supportRoomlist()) m_buttonList->setEnabled(false);
    buttons->addButton(m_buttonList, QDialogButtonBox::ActionRole);
    QPushButton *buttonChat = new QPushButton(_("&Join"));
    buttons->addButton(buttonChat, QDialogButtonBox::AcceptRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(m_buttonList, SIGNAL(clicked()), this, SLOT(slotGetChatList()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addLayout(m_grid);
    mainFrame->addStretch(1);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(_("Connect chat"));

    connect(cb, SIGNAL(signalRoomlistCreated(PURPLE::RoomList*)), this, SLOT(slotRoomlistCreated(PURPLE::RoomList*)));
    connect(m_IDs, SIGNAL(currentIndexChanged(int)), this, SLOT(slotIDs(int)));
}

void ChatJoinDialog::accept()
{
    m_accounts[m_IDs->currentIndex()].joinChat(fillChatEntryFields());
    QDialog::accept();
}

void ChatJoinDialog::slotIDs(int index)
{
    createChatEntryFields(m_accounts[index].getChatParameters());
    if(m_accounts[index].supportRoomlist()) m_buttonList->setEnabled(true);
    else m_buttonList->setEnabled(false);
}

void ChatJoinDialog::slotGetChatList()
{
    m_accounts[m_IDs->currentIndex()].displayRoomList();
    m_buttonList->setEnabled(false);
}

void ChatJoinDialog::slotRoomlistCreated(PURPLE::RoomList *rlist)
{
    if(m_accounts[m_IDs->currentIndex()].getUsername() == rlist->getAccount().getUsername())
        connect(new RoomListDialog(rlist, static_cast<dxqCallbacks*>(sender()), this), SIGNAL(signalClosed(bool,QString)), this, SLOT(slotRoomlistClosed(bool,QString)));
}

void ChatJoinDialog::slotRoomlistClosed(bool join, QString room)
{
    //Terrible hack :)
    if(join && m_fields.size())
        ((QLineEdit*)m_fields[0])->setText(room);
    m_buttonList->setEnabled(true);
}

void ChatJoinDialog::createChatEntryFields(std::vector<PURPLE::Account::ChatEntry> fields)
{
    QLabel *label;
    QLineEdit *text;
    QSpinBox *spinner;
    for(int i=0; i<m_labels.size(); i++)
    {
        delete m_labels[i];
    }
    m_labels.clear();
    for(int i=0; i<m_fields.size(); i++)
    {
        delete m_fields[i];
    }
    m_fields.clear();
    int i = 1;
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        label = new QLabel(dxutils::normalizeLabelString(entry.getLabel(), '_'));
        m_grid->addWidget(label, i, 0);
        m_labels.append(label);
        if(entry.getIsInt())
        {
            spinner = new QSpinBox();
            spinner->setMinimum(entry.getMin());
            spinner->setMaximum(entry.getMax());
            m_grid->addWidget(spinner, i, 1);
            m_fields.append(spinner);
        }
        else
        {
            text = new QLineEdit(dxutils::fromStdString(entry.getValue()));
            if(entry.getSecret()) text->setEchoMode(QLineEdit::PasswordEchoOnEdit);
            m_grid->addWidget(text, i, 1);
            m_fields.append(text);
        }
        i++;
    }
    resize(0, 0);
}

std::vector<PURPLE::Account::ChatEntry> ChatJoinDialog::fillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields = m_accounts[m_IDs->currentIndex()].getChatParameters();
    std::vector<PURPLE::Account::ChatEntry>::iterator it;
    uint i;
    for(it=fields.begin(), i=0; it!=fields.end(); it++,i++)
    {
        PURPLE::Account::ChatEntry& entry = *it;
        if(entry.getIsInt())
        {
            QString str;
            str.setNum(static_cast<QSpinBox*>(m_fields[i])->value());
            entry.setValue(dxutils::toStdString(str));
        }
        else
        {
            entry.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_fields[i])->text()));
        }
    }
    return fields;
}

