/*
 *      choicerequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QDialogButtonBox>
#include "dxutils.h"
#include "choicerequestdialog.h"
#include "config.h"
#include "i18n.h"

class ChoiceListWidgetItem : public QListWidgetItem
{
public:
    ChoiceListWidgetItem(const QString &text, int id) : QListWidgetItem(text), m_id(id) {}
    int getID() const { return m_id; }
private:
    int m_id;
};

ChoiceRequestDialog::ChoiceRequestDialog(PURPLE::ChoiceRequest *request, QWidget *parent) :
    QDialog(parent), m_request(request)
{
    m_choices = new QListWidget;
    std::map<std::string, int> choices = m_request->getChoices();
    std::map<std::string, int>::iterator it;
    for(it=choices.begin(); it != choices.end(); it++)
    {
        ChoiceListWidgetItem *item = new ChoiceListWidgetItem(dxutils::normalizeLabelString((*it).first, '_'), (*it).second);
        m_choices->addItem(item);
        if((*it).second == m_request->getDefaultValue())
            m_choices->setCurrentItem(item);
    }
    QDialogButtonBox *buttons = new QDialogButtonBox();
    QPushButton *buttonOk = new QPushButton(dxutils::fromStdString(request->getOkText()));
    buttons->addButton(buttonOk, QDialogButtonBox::AcceptRole);
    QPushButton *buttonCancel = new QPushButton(dxutils::fromStdString(request->getCancelText()));
    buttons->addButton(buttonCancel, QDialogButtonBox::RejectRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    if(!m_request->getPrimary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getPrimary())));
    if(!m_request->getSecondary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getSecondary())));
    mainFrame->addWidget(m_choices);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(m_request->getTitle().empty()? _("Request") : dxutils::fromStdString(m_request->getTitle()));
}

void ChoiceRequestDialog::accept()
{
    m_request->sendChoice(((ChoiceListWidgetItem*)m_choices->currentItem())->getID());
    QDialog::accept();
}

void ChoiceRequestDialog::reject()
{
    m_request->dennyRequest();
    QDialog::reject();
}
