/*
 *      configdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CONFIGDIALOG_H
#define CONFIGDIALOG_H

#include <QDialog>
#include <QBoxLayout>
#include <QTabWidget>
#include <QCheckBox>
#include <QSpinBox>
#include <QLineEdit>
#include <QPushButton>
#include <QListWidget>
#include <QLabel>
#include <QComboBox>
#include <QTextEdit>
#include "preferences.h"
#include "tabitems/tabwidget.h"
#include "widgets/colorbutton.h"

class ConfigDialog : public QDialog
{
    Q_OBJECT
public:
    ConfigDialog(QWidget *parent = 0);
private slots:
    void accept();
    void slotUseAway();
    void slotLogging();
    void slotLogPath();
    void slotThemeStatus(QListWidgetItem *current);
    void slotAddStatusTheme();
    void slotDeleteStatusTheme();
    void slotThemeRoles(QListWidgetItem *current);
    void slotAddRoleTheme();
    void slotDeleteRoleTheme();
    void slotTabColor();
    void slotTabPosition();
    void slotImTextColor();
    void slotSound();
    void slotSoundConnect();
    void slotSoundDisconnect();
    void slotSoundMessage();
    void slotPath();
    void slotPlay();
    void slotNotify();
    void slotShowNotify();
    void slotUseSmileys();
    void slotAddSmiley();
    void slotModifySmiley();
    void slotDeleteSmiley();
    void slotImportSmileys();
    void slotExportSmileys();
    void slotModifyIMFont();
    void slotRaiseWindow();
private:
    bool m_showImportwarning;
    //GeneralTab widgets
    QCheckBox *m_oneInstance;
    QCheckBox *m_useAway;
    QCheckBox *m_minToTray;
    QCheckBox *m_logging;
    QSpinBox *m_awayTimeout;
    QLabel *m_logPath;
    QPushButton *m_setLogPath;
    QCheckBox *m_logLoadCurrent;
    QCheckBox *m_showRosterTree;
    QCheckBox *m_receiptsMark;
    //IconsTab widgets
    QListWidget *m_themesStatusList;
    QListWidget *m_themesRoleList;
    QLabel *m_iconAway;
    QLabel *m_iconIdle;
    QLabel *m_iconDnd;
    QLabel *m_iconInvisible;
    QLabel *m_iconOffline;
    QLabel *m_iconOnline;
    QLabel *m_iconFounder;
    QLabel *m_iconHalfop;
    QLabel *m_iconNormal;
    QLabel *m_iconOp;
    QLabel *m_iconVoice;
    QPushButton *m_themeStatusAdd;
    QPushButton *m_themeStatusDelete;
    QPushButton *m_themeRoleAdd;
    QPushButton *m_themeRoleDelete;
    //LookTab widgets
    ColorButton *m_unreadtabColor;
    ColorButton *m_hltabColor;
    QComboBox *m_tabPosition;
    TabWidget *m_lookTabs;
    //ImTextTab widgets
    ColorButton *m_textColor;
    ColorButton *m_backroundColor;
    ColorButton *m_userColor;
    ColorButton *m_actionColor;
    ColorButton *m_errorColor;
    ColorButton *m_hilightColor;
    ColorButton *m_linkColor;
    ColorButton *m_mymsgColor;
    ColorButton *m_logColor;
    QTextEdit *m_imText;
    QLabel *m_imFont;
    QPushButton *m_imFontModify;
    QLineEdit *m_nickCompletionChar;
    QCheckBox *m_sameCmd;
    QCheckBox *m_sameList;
    QCheckBox *m_coloredNick;
    //NotificationTab widgets
    QCheckBox *m_sounds;
    QCheckBox *m_soundConnect;
    QCheckBox *m_soundDisconnect;
    QCheckBox *m_soundMessage;
    QLabel *m_pathConnect;
    QLabel *m_pathDisconnect;
    QLabel *m_pathMessage;
    QPushButton *m_setpathConnect;
    QPushButton *m_setpathDisconnect;
    QPushButton *m_setpathMessage;
    QPushButton *m_playConnect;
    QPushButton *m_playDisconnect;
    QPushButton *m_playMessage;
    QCheckBox *m_notify;
    QCheckBox *m_notifyBuddyConnect;
    QCheckBox *m_notifyBuddyDisconnect;
    QCheckBox *m_notifyMessage;
    QCheckBox *m_raiseWindow;
    QCheckBox *m_notifyAccountConnect;
    QCheckBox *m_notifyAccountDisconnect;
    QCheckBox *m_notifyFiletransfer;
    QCheckBox *m_notifyMessageTray;
    QComboBox *m_notifyPosition;
    QPushButton *m_showNotify;
    //SmileysTab widgets
    QCheckBox *m_useSmileys;
    QMap<QString,QString> m_smileysMap;
    QListWidget *m_smileysList;
    QPushButton *m_addSmiley;
    QPushButton *m_modifySmiley;
    QPushButton *m_deleteSmiley;
    QPushButton *m_importSmiley;
    QPushButton *m_exportSmiley;

    void createGeneralTab(QTabWidget *tabbook);
    void createIconsTab(QTabWidget *tabbook);
    void createLookTab(QTabWidget *tabbook);
    void createImtextTab(QTabWidget *tabbook);
    void createNotificationTab(QTabWidget *tabbook);
    void createSmileysTab(QTabWidget *tabbook);
    void updateTabPosition();
    void updateImText();
    void fillSmileys();
};

class SmileyDialog : public QDialog
{
    Q_OBJECT
public:
    SmileyDialog(const QString &title, const QString &smiley, const QString &path, QWidget *parent = 0);

    QString getSmiley() { return m_smiley->text(); }
    QString getPath() { return m_path->text(); }
private slots:
    void slotPath();
private:
    QLabel *m_path, *m_preview;
    QPushButton *m_selectPath;
    QLineEdit *m_smiley;
};

#endif // CONFIGDIALOG_H
