/*
 *      imdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include <QDialogButtonBox>
#include "account.h"
#include "dxutils.h"
#include "imdialog.h"
#include "config.h"
#include "i18n.h"

ImDialog::ImDialog(std::vector<PURPLE::Account> accounts, QWidget *parent) :
    QDialog(parent), m_completer(0), m_accounts(accounts)
{
    createControls();
}

ImDialog::ImDialog(PURPLE::Account account, QWidget *parent) :
    QDialog(parent), m_completer(0)
{
    m_accounts.push_back(account);
    createControls();
}

void ImDialog::slotAccountChanged(int index)
{
    QStringList buddies;
    std::vector<PURPLE::Buddy> v = m_accounts[index].getBuddies();
    for(uint i=0; i<v.size(); i++)
        buddies << QString("%1 (%2)").arg(dxutils::fromStdString(v[i].getAlias())).arg(dxutils::fromStdString(v[i].getUsername()));
    delete m_completer;
    m_completer = new QCompleter(buddies, this);
    m_completer->setCompletionMode(QCompleter::PopupCompletion);
    m_completer->setCaseSensitivity(Qt::CaseInsensitive);
    m_with->setCompleter(m_completer);
}

void ImDialog::createControls()
{
    QGridLayout *grid = new QGridLayout;
    m_accountsList = new QComboBox;
    m_accountsList->setInsertPolicy(QComboBox::NoInsert);
    for(uint i=0; i<m_accounts.size(); i++)
    {
        m_accountsList->addItem(QIcon(dxutils::protocolToIcon(dxutils::fromStdString(m_accounts[i].getProtocol().getPurpleID()))), dxutils::fromStdString(m_accounts[i].getUsername()));
    }
    connect(m_accountsList, SIGNAL(currentIndexChanged(int)), this, SLOT(slotAccountChanged(int)));
    grid->addWidget(new QLabel(_("Account:")), 0, 0);
    grid->addWidget(m_accountsList, 0, 1);
    m_with = new QLineEdit;
    slotAccountChanged(0);
    grid->addWidget(new QLabel(_("With:")), 1, 0);
    grid->addWidget(m_with, 1, 1);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    QPushButton *buttonIM = new QPushButton(_("&IM"));
    buttons->addButton(buttonIM, QDialogButtonBox::AcceptRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addLayout(grid);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(_("New IM"));
}

void ImDialog::fillIMInfo(QString &ID, QString &protocol, QString &IMID, QString &IMnick)
{
    ID = dxutils::fromStdString(m_accounts[m_accountsList->currentIndex()].getUsername());
    protocol = dxutils::fromStdString(m_accounts[m_accountsList->currentIndex()].getProtocol().getPurpleID());
    QString line = m_with->text();
    if(line.contains('(')&&line.contains(')'))
    {
        IMnick = line.section('(',0,0).trimmed();
        IMID = line.section('(',1).section(')',0,0);
    }
    else
    {
        IMnick = "";
        IMID = line;
    }
}

void ImDialog::fillIMInfo(QString &IMID, QString &IMnick)
{
    QString line = m_with->text();
    if(line.contains('(')&&line.contains(')'))
    {
        IMnick = line.section('(',0,0).trimmed();
        IMID = line.section('(',1).section(')',0,0);
    }
    else
    {
        IMnick = "";
        IMID = line;
    }
}
