/*
 *      inputrequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QDialogButtonBox>
#include "dxutils.h"
#include "inputrequestdialog.h"
#include "config.h"
#include "i18n.h"

InputRequestDialog::InputRequestDialog(PURPLE::InputRequest *request, QWidget *parent) :
    QDialog(parent), m_request(request)
{
    m_input = new QLineEdit;
    m_input->setText(dxutils::fromStdString(request->getDefaultValue()));
    if(m_request->getMasked()) m_input->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    QDialogButtonBox *buttons = new QDialogButtonBox();
    QPushButton *buttonOk = new QPushButton(dxutils::fromStdString(request->getOkText()));
    buttons->addButton(buttonOk, QDialogButtonBox::AcceptRole);
    QPushButton *buttonCancel = new QPushButton(dxutils::fromStdString(request->getCancelText()));
    buttons->addButton(buttonCancel, QDialogButtonBox::RejectRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    if(!m_request->getPrimary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getPrimary())));
    if(!m_request->getSecondary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getSecondary())));
    mainFrame->addWidget(m_input);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(m_request->getTitle().empty()? _("Request") : dxutils::fromStdString(m_request->getTitle()));
}

void InputRequestDialog::accept()
{
    m_request->sendInput(dxutils::toStdString(m_input->text()));
    QDialog::accept();
}

void InputRequestDialog::reject()
{
    m_request->dennyRequest();
    QDialog::reject();
}
