/*
 *      roomlistdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QDialogButtonBox>
#include <QLabel>
#include "roomlistdialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

RoomListDialog::RoomListDialog(PURPLE::RoomList *rlist, dxqCallbacks *cb, QWidget *parent) :
    QDialog(parent), m_rlist(rlist), m_finished(false)
{
    connect(cb, SIGNAL(signalRoomlistColumns(PURPLE::RoomList*,QVector<QString>)), this, SLOT(slotRoomlistColumns(PURPLE::RoomList*,QVector<QString>)));
    connect(cb, SIGNAL(signalRoomlistAddRoom(PURPLE::RoomList*,QVector<QString>)), this, SLOT(slotRoomlistAddRoom(PURPLE::RoomList*,QVector<QString>)));
    connect(cb, SIGNAL(signalRoomlistFinished(PURPLE::RoomList*)), this, SLOT(slotRoomlistFinished(PURPLE::RoomList*)));
}

void RoomListDialog::accept()
{
    emit signalClosed(true, m_roomView->currentItem()?m_roomView->currentItem()->text(0):"");
    QDialog::accept();
}

void RoomListDialog::reject()
{
    if(!m_finished) m_rlist->cancelListing();
    emit signalClosed(false, "");
    QDialog::reject();
}

void RoomListDialog::slotRoomlistColumns(PURPLE::RoomList *rlist, QVector<QString> columns)
{
    if(m_rlist==rlist)
    {
        QGridLayout *matrix = new QGridLayout;
        matrix->addWidget(new QLabel(_("Search")), 0, 0);
        m_search = new QLineEdit;
        m_search->setEnabled(false);
        connect(m_search, SIGNAL(textChanged(QString)), this, SLOT(slotSearch()));
        matrix->addWidget(m_search, 0, 1);
        QStringList headers;
        headers << _("Chat");
        for(int i=0; i<columns.size(); i++)
        {
            headers << columns[i];
        }
        m_columns = columns.size()+1;
        m_roomView = new QTreeWidget(this);
        m_roomView->setHeaderLabels(headers);
        m_roomView->setSelectionBehavior(QAbstractItemView::SelectRows);
        m_roomView->setAlternatingRowColors(true);
        m_roomView->setRootIsDecorated(false);
        QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
        m_selectButton = new QPushButton(_("&Select"));
        m_selectButton->setEnabled(false);
        buttons->addButton(m_selectButton, QDialogButtonBox::AcceptRole);
        QVBoxLayout *mainFrame = new QVBoxLayout(this);
        mainFrame->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(m_rlist->getAccount().getProtocol().getPurpleID())))
                                        .arg(dxutils::fromStdString(m_rlist->getAccount().getUsername()))));
        mainFrame->addLayout(matrix);
        mainFrame->addWidget(m_roomView);
        mainFrame->addWidget(buttons);
        setLayout(mainFrame);
        setWindowTitle(_("Chats list"));

        connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
        connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
        show();
    }
}

void RoomListDialog::slotRoomlistAddRoom(PURPLE::RoomList *rlist, QVector<QString> fields)
{
    if(m_rlist==rlist)
    {
        QString row = "";
        QTreeWidgetItem *item = new QTreeWidgetItem(m_roomView);
        for(int i=0; i<fields.size(); i++)
        {
            item->setText(i, fields[i]);
            row.append(fields[i]);
            row.append('\n');
        }
        m_rooms.append(row);
        item->setFlags(item->flags() & ~Qt::ItemIsEditable);
        item->setTextAlignment(1, Qt::AlignHCenter);
        if(m_roomView->topLevelItemCount()==1) m_roomView->setCurrentItem(item);
    }
}

void RoomListDialog::slotRoomlistFinished(PURPLE::RoomList *rlist)
{
    if(m_rlist==rlist)
    {
        m_selectButton->setEnabled(true);
        m_search->setEnabled(true);
        m_roomView->sortByColumn(0, Qt::AscendingOrder);
        m_finished = true;
    }
}

void RoomListDialog::slotSearch()
{
    m_roomView->clear();
    QTreeWidgetItem *current = NULL;
    if(m_search->text().isEmpty())
    {
        for(int i=0; i < m_rooms.size(); ++i)
        {
            QTreeWidgetItem *item = new QTreeWidgetItem(m_roomView);
            for(int j=0; j < m_columns; ++j)
            {
                item->setText(j, m_rooms[i].section('\n',j,j));
            }
            if(i == 0) current = item;
        }
    }
    else
    {
        for(int i=0; i < m_rooms.size(); ++i)
        {
            if(m_rooms[i].section('\n',0,0).contains(m_search->text()))
            {
                QTreeWidgetItem *item = new QTreeWidgetItem(m_roomView);
                for(int j=0; j < m_columns; ++j)
                {
                    item->setText(j, m_rooms[i].section('\n',j,j));
                }
                if(!current) current = item;
            }
        }
    }
    if(current) m_roomView->setCurrentItem(current);
    m_selectButton->setEnabled(current);
}
