/*
 *      dxim.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXIM_H
#define DXIM_H

#include <QMainWindow>
#include <QMenu>
#include <QAction>
#include <QSystemTrayIcon>
#include <QSplitter>
#include <QTreeView>
#include <QItemSelectionModel>
#include "tabitems/tabwidget.h"
#include "roster/rosterview.h"
#include "roster/rosteritemmodel.h"
#include "roster/rosteritemsortmodel.h"
#include "tabitems/fttabitem.h"
#include "tabitems/imtabitem.h"
#include "tabitems/chattabitem.h"
#include "idle/idle.h"
#include "callback/callback.h"
#include "logviewer/logdialog.h"

class dxim : public QMainWindow
{
    friend class RosterView;
    Q_OBJECT
public:
    explicit dxim(bool offline = false, QWidget *parent = 0);
    ~dxim();
private:
    QMenu *m_fileMenu, *m_statusMenu, *m_editMenu, *m_helpMenu, *m_trayMenu;
    QAction *m_quitAction, *m_logsAction, *m_closetabAction, *m_clearAction, *m_clearAllAction;
    QSystemTrayIcon *m_trayIcon;
    bool m_newMsg;
    PurpleStatusPrimitive m_presence;
    TabWidget *m_tabbook;
    RosterView *m_roster;
    QSplitter *m_splitter;
    RosterItemModel m_rosterModel;
    RosterItemSortModel m_rosterSortModel;
    FtTabItem *m_ftTab;
    Idle m_idle;
    dxqCallbacks *m_cb;
    PURPLE::Purple *m_purple;
    LogDialog *m_logDialog;

    void createActions();
    void createMenus();
    void createTrayIcon();
    RosterItem* addAccount(PURPLE::Account account);
    void updateTrayIconPresence(PurpleStatusPrimitive presence=PURPLE_STATUS_UNSET);
    void updateTrayIconNewmsg();
    void updatePresence(PurpleStatusPrimitive presence);
    ImTabItem* createIMTab(PURPLE::Conversation *conv);
    ChatTabItem* createChatTab(PURPLE::Conversation *conv);
    void updateTabPosition();
    void fillRoster();
private slots:
    void slotCmdManageAccounts();
    void slotCmdAddBuddy();
    void slotCmdAddChat();
    void slotCmdAddGroup();
    void slotCmdNewIM();
    void slotCmdJoinChat();
    void slotCmdFileTransfer();
    void slotCmdLogs();
    void slotCmdQuit();
    void slotIconActivated(QSystemTrayIcon::ActivationReason);
    void slotCmdReconnect();
    void slotCmdOnline();
    void slotCmdAway();
    void slotCmdDnd();
    void slotCmdInvisible();
    void slotCmdOffline();
    void slotCmdStatusMsg();
    void slotCmdAbout();
    void slotCmdCloseTab();
    void slotCmdClear();
    void slotCmdClearAll();
    void slotCmdRoster();
    void slotCmdChatRoster();
    void slotCmdAvatars();
    void slotCmdHideOffline();
    void slotCmdShowStatusmsg();
    void slotCmdPlugins();
    void slotCmdPreferences();
    void slotSecondsIdle(int sec);
    void slotImTabHighlight();
    void slotImTabNewmsg();
    void slotImTabComposing(bool compose);
    void slotImTabOffline(bool offline);
    void slotImTabNewName(const QString &name);
    void slotChatTabHighlight();
    void slotChatTabNewmsg();
    void slotTabChanged(int index);
    void slotCloseFtTabItem();
    void slotCloseAccount(PURPLE::Account account);
    void slotUpdateMenu();
    void slotCloseMsgTabItem();
    void slotShowLog(PURPLE::Account account, const QString &name, const QString &path);
    //purplemm slots
    void slotPurpleError(QString error);
    void slotBuddylistLoaded();
    void slotAccountAdded(PURPLE::Account account);
    void slotAccountRemoved(QString account, QString protocolID);
    void slotConnecting(PURPLE::Account account, QString msg);
    void slotConnected(PURPLE::Account account);
    void slotDisconnected(PURPLE::Account account, QString reason);
    void slotAccountStatusChanged(PURPLE::Account account);
    void slotAccountSignedOn(PURPLE::Account account);
    void slotBuddySignedOn(PURPLE::Buddy buddy);
    void slotBuddySignedOff(PURPLE::Buddy buddy);
    void slotBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus);
    void slotBuddyIconChanged(PURPLE::Buddy buddy);
    void slotNewIM(PURPLE::Conversation *conv);
    void slotNewChat(PURPLE::Conversation *conv);
    void slotCloseConvTab(PURPLE::Conversation *conv);
    void slotNewMsg(PURPLE::Conversation *conv, QString sender, QString msg, QString time, bool highlight, bool history, bool action);
    void slotNewMyMsg(PURPLE::Conversation *conv, QString msg, QString time, bool action, QString messageID);
    void slotMessageRecived(PURPLE::Conversation *conv, QString messageID);
    void slotNewConvSystemMessage(PURPLE::Conversation *conv, QString msg, QString time);
    void slotNewConvErrorMessage(PURPLE::Conversation *conv, QString msg, QString time);
    void slotBuddyTyping(PURPLE::Conversation *conv, IMStateType typing);
    void slotChatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy);
    void slotChatBuddyUpdated(PURPLE::Conversation *conv, QString buddy);
    void slotChatBuddyLeft(PURPLE::Conversation *conv, QString buddy);
    void slotChatBuddyRenamed(PURPLE::Conversation *conv, QString who, PURPLE::ChatBuddy nbuddy);
    void slotChatTopicChanged(PURPLE::Conversation *conv, QString who, QString topic);
    void slotNewInputRequest(PURPLE::InputRequest *request);
    void slotNewActionRequest(PURPLE::ActionRequest *request);
    void slotNewChoiceRequest(PURPLE::ChoiceRequest *request);
    void slotNewFieldsRequest(PURPLE::FieldsRequest *request);
    void slotNewFileRequest(PURPLE::FileRequest *request);
    void slotNotifyMessage(QPixmap icon, QString title, QString text);
    void slotNotifyEmail(QString text, QString url);
    void slotNotifyURI(QString uri);
    void slotNotifyUserinfo(PURPLE::Account account, QString buddy, QString info);
protected:
    void closeEvent(QCloseEvent *ev);
    bool eventFilter(QObject *obj, QEvent *ev);
    dxqCallbacks *getCallbacks();
protected slots:
    void slotBuddyAdded(PURPLE::Buddy buddy);
    void slotBuddyRemoved(PURPLE::Account account, QString buddy);
    void slotChatAdded(PURPLE::Chat chat);
    void slotChatRemoved(PURPLE::Account account, QString chat);
};

#endif // DXIM_H
