/*
 *      dxutils.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXUTILS_H
#define DXUTILS_H

#include <QString>
#include <QIcon>
#include "defs.h"

class dxutils
{
public:
    dxutils();

    static dxutils &instance();
    static QString getFileSize(quint64 size);
    static QString fromStdString(const std::string &str);
    static std::string toStdString(const QString &str);
    static QString presenceToIcon(PurpleStatusPrimitive presence);
    static QString protocolToIcon(const QString &protocol);
    static QString getNickColor(const QString &nick);
    static bool isDelimiter(char c);
    static QString imStateToString(const QString &nick, IMStateType type);
    static QString getSpeed(quint64 speed);
    static QString getRemaining(quint64 size, quint64 speed);
    static QImage getImageFromPath(const QString &path, int maxSize=-1);
    void showNotify(QString text, int position=-1);
    void removeNotify();
    static QString getChatRoleString(PurpleConvChatBuddyFlags role);
    static QString getChatRoleIcon(PurpleConvChatBuddyFlags role);
    static void debugLine(const QString &line);
    static std::string styleToString(QColor color);
    static QColor makeOfflineColor(QColor clr);
    /**
     *
     * @param label         string to normalize
     * @param normalize     char for remove, usually '_'
     * @return              Normalized QString
     */
    static QString normalizeLabelString(std::string label, char normalize);
    static bool isRightLogFile(const QString &path, const QString &name);
    static QString getMonth(int month);
    static QString getMonthNumber(const QString &month);
    static void playFile(const QString &file);
    static QString createSmileys(QString text);
    static QString parseHtml(QString text);
    static QString checkThemeStatusPath(const QString &path);
    static QString checkThemesStatusList(const QString &list);
    static QString checkThemeRolePath(const QString &path);
    static QString checkThemesRoleList(const QString &list);
    /**
     * @param dirName       path of directory to remove
     */
    static bool removeDir(const QString &dirName);
    bool haveNotify();
    static QString wrapText(QString text, int width);
    static bool hasLog(const QString &accountName, const QString &conversationName, bool account);
    static QString enquote(const QString &string);
private:
    int m_notifiesHeight, m_notifyPosition;
    bool m_firstNotify, m_haveNotify;
};

#endif // DXUTILS_H
