/*
 *      logdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QLabel>
#include <QDialogButtonBox>
#include <QHBoxLayout>
#include <QDir>
#include <QMessageBox>
#include <QGroupBox>
#include <QEvent>
#include <QKeyEvent>
#include <QProgressDialog>
#include <QTextStream>
#include <QApplication>
#include "logdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "preferences.h"

LogDialog::LogDialog(QWidget *parent) :
    QDialog(parent), m_logModel(this)
{
    m_fullLog = true;
    m_mainLayout = new QVBoxLayout;
    createWidgets();
    loadTree();
    installEventFilter(this);
}

LogDialog::LogDialog(PURPLE::Account account, QWidget *parent) :
    QDialog(parent), m_logModel(this)
{
    m_fullLog = false;
    QLabel *info = new QLabel;
    info->setText(dxutils::fromStdString(account.getUsername()).section('/',0,0));
    QLabel *icon = new QLabel;
    icon->setPixmap(QPixmap(dxutils::protocolToIcon(dxutils::fromStdString(account.getProtocol().getPurpleID()))));
    QHBoxLayout *idLayout = new QHBoxLayout;
    idLayout->addWidget(icon);
    idLayout->addWidget(info);
    idLayout->addSpacerItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    m_mainLayout = new QVBoxLayout;
    m_mainLayout->addLayout(idLayout);
    createWidgets();
    LogItem *item = new LogItem(_("Logs"), LogItem::Root, DXPREFERENCES.m_logPath);
    item->setFlags(item->flags() & ~Qt::ItemIsEditable);
    m_logModel.appendRow(item);
    LogItem *aitem = new LogItem(dxutils::fromStdString(account.getUsername()).section('/',0,0), LogItem::Account,
                                 DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(account.getUsername()).section('/',0,0));
    aitem->setFlags(aitem->flags() & ~Qt::ItemIsEditable);
    m_logModel.addLogItem(item, aitem);
    listDirsChildItems(aitem);
    m_treeLoaded = true;
    m_log->setExpanded(m_logModel.indexFromItem(item), true);
    m_log->setExpanded(m_logModel.indexFromItem(aitem), true);
    installEventFilter(this);
}

LogDialog::LogDialog(PURPLE::Buddy buddy, QWidget *parent) :
    QDialog(parent), m_logModel(this)
{
    m_fullLog = false;
    QLabel *info = new QLabel;
    info->setText(dxutils::fromStdString(buddy.getUsername()).section('/',0,0)+" ("+dxutils::fromStdString(buddy.getAccount().getUsername()).section('/',0,0)+")");
    QLabel *icon = new QLabel;
    icon->setPixmap(QPixmap(dxutils::protocolToIcon(dxutils::fromStdString(buddy.getAccount().getProtocol().getPurpleID()))));
    QHBoxLayout *idLayout = new QHBoxLayout;
    idLayout->addWidget(icon);
    idLayout->addWidget(info);
    idLayout->addSpacerItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    m_mainLayout = new QVBoxLayout;
    m_mainLayout->addLayout(idLayout);
    createWidgets();
    LogItem *item = new LogItem(_("Logs"), LogItem::Root, DXPREFERENCES.m_logPath);
    item->setFlags(item->flags() & ~Qt::ItemIsEditable);
    m_logModel.appendRow(item);
    LogItem *aitem = new LogItem(dxutils::fromStdString(buddy.getAccount().getUsername()).section('/',0,0), LogItem::Account,
                                 DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(buddy.getAccount().getUsername()).section('/',0,0));
    aitem->setFlags(aitem->flags() & ~Qt::ItemIsEditable);
    m_logModel.addLogItem(item, aitem);
    LogItem *bitem = new LogItem(dxutils::fromStdString(buddy.getUsername()).section('/',0,0), LogItem::Conversation,
                                 DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(buddy.getAccount().getUsername()).section('/',0,0)+"/"+dxutils::fromStdString(buddy.getUsername()).section('/',0,0));
    bitem->setFlags(bitem->flags() & ~Qt::ItemIsEditable);
    m_logModel.addLogItem(aitem, bitem);
    listFilesChildItems(bitem);
    m_treeLoaded = true;
    m_log->setExpanded(m_logModel.indexFromItem(item), true);
    m_log->setExpanded(m_logModel.indexFromItem(aitem), true);
    m_log->setExpanded(m_logModel.indexFromItem(bitem), true);
    installEventFilter(this);
}

LogDialog::LogDialog(PURPLE::Chat chat, QWidget *parent) :
    QDialog(parent), m_logModel(this)
{
    m_fullLog = false;
    QLabel *info = new QLabel;
    info->setText(dxutils::fromStdString(chat.getChatName())+" ("+dxutils::fromStdString(chat.getAccount().getUsername()).section('/',0,0)+")");
    QLabel *icon = new QLabel;
    icon->setPixmap(QPixmap(dxutils::protocolToIcon(dxutils::fromStdString(chat.getAccount().getProtocol().getPurpleID()))));
    QHBoxLayout *idLayout = new QHBoxLayout;
    idLayout->addWidget(icon);
    idLayout->addWidget(info);
    idLayout->addSpacerItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    m_mainLayout = new QVBoxLayout;
    m_mainLayout->addLayout(idLayout);
    createWidgets();
    LogItem *item = new LogItem(_("Logs"), LogItem::Root, DXPREFERENCES.m_logPath);
    item->setFlags(item->flags() & ~Qt::ItemIsEditable);
    m_logModel.appendRow(item);
    LogItem *aitem = new LogItem(dxutils::fromStdString(chat.getAccount().getUsername()).section('/',0,0), LogItem::Account,
                                 DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(chat.getAccount().getUsername()).section('/',0,0));
    aitem->setFlags(aitem->flags() & ~Qt::ItemIsEditable);
    m_logModel.addLogItem(item, aitem);
    LogItem *bitem = new LogItem(dxutils::fromStdString(chat.getChatName()), LogItem::Conversation,
                                 DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(chat.getAccount().getUsername()).section('/',0,0)+"/"+dxutils::fromStdString(chat.getChatName()));
    bitem->setFlags(bitem->flags() & ~Qt::ItemIsEditable);
    m_logModel.addLogItem(aitem, bitem);
    listFilesChildItems(bitem);
    m_treeLoaded = true;
    m_log->setExpanded(m_logModel.indexFromItem(item), true);
    m_log->setExpanded(m_logModel.indexFromItem(aitem), true);
    m_log->setExpanded(m_logModel.indexFromItem(bitem), true);
    installEventFilter(this);
}

void LogDialog::createWidgets()
{
    m_treeLoaded = false;
    m_searchString = "";

    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    m_splitter = new QSplitter;
    m_text = new QTextBrowser;
    m_text->setOpenExternalLinks(true);
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_splitter->addWidget(m_text);
    QWidget *rightSide = new QWidget;
    QVBoxLayout *rightLayout = new QVBoxLayout;
    rightSide->setLayout(rightLayout);
    QHBoxLayout *searchLayout = new QHBoxLayout;
    m_searchLine = new QLineEdit;
    m_searchLine->installEventFilter(this);
    searchLayout->addWidget(m_searchLine);
    m_buttonSearch = new QPushButton(_("&Search"));
    searchLayout->addWidget(m_buttonSearch);
    m_buttonReset = new QPushButton(_("&Reset"));
    searchLayout->addWidget(m_buttonReset);
    QGroupBox *group = new QGroupBox(_("Search options"));
    m_buttonFile = new QRadioButton(_("Search file"));
    m_buttonConv = new QRadioButton(_("Search im/chat"));
    m_buttonAccount = new QRadioButton(_("Search account"));
    m_buttonAll = new QRadioButton(_("Search all"));
    m_buttonFile->setEnabled(false);
    m_buttonConv->setEnabled(false);
    m_buttonAccount->setEnabled(false);
    m_buttonAll->setEnabled(true);
    m_buttonAll->setChecked(true);
    m_buttonIcase = new QCheckBox(_("&Ignore case"));
    QVBoxLayout *groupLayout = new QVBoxLayout;
    groupLayout->addWidget(m_buttonIcase);
    groupLayout->addWidget(m_buttonFile);
    groupLayout->addWidget(m_buttonConv);
    groupLayout->addWidget(m_buttonAccount);
    groupLayout->addWidget(m_buttonAll);
    group->setLayout(groupLayout);
    m_log = new LogView;
    m_log->setFullLogs(m_fullLog);
    m_log->setModel(&m_logModel);
    m_log->setHeaderHidden(true);
    m_log->setContextMenuPolicy(Qt::CustomContextMenu);
    m_log->setStyleSheet(QString("QTreeView::branch { \
                    border-image: none; \
                    } \
                    QTreeView::branch:open:has-children:!has-siblings, \
                    QTreeView::branch:open:has-children:has-siblings { \
                    image: url(%1); \
                    } \
                    QTreeView::branch:has-children:!has-siblings:closed, \
                    QTreeView::branch:closed:has-children:has-siblings { \
                    image: url(%2); \
                    } \
                    ")
                    .arg(DXIM_DATADIR "/icons/groupopen.png")
                    .arg(DXIM_DATADIR "/icons/groupclose.png"));
    QItemSelectionModel *selmod = m_log->selectionModel();
    rightLayout->addLayout(searchLayout);
    rightLayout->addWidget(group);
    rightLayout->addWidget(m_log);
    m_splitter->addWidget(rightSide);
    m_mainLayout->addWidget(m_splitter);
    m_mainLayout->addWidget(buttons);
    setLayout(m_mainLayout);
    setWindowTitle(_("dxim - log viewer"));

    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(selmod, SIGNAL(currentChanged(QModelIndex,QModelIndex)), this, SLOT(slotCurrentChanged(QModelIndex)));
    connect(m_buttonSearch, SIGNAL(clicked()), this, SLOT(slotSearch()));
    connect(m_buttonReset, SIGNAL(clicked()), this, SLOT(slotReset()));
    connect(m_log, SIGNAL(deleteItem(LogItem*)), this, SLOT(slotDeleteItem(LogItem*)));
}

void LogDialog::setLogPath(const QString &path)
{
    LogItem *item = static_cast<LogItem*>(m_logModel.item(0));
    if(item->getPath() != path)
    {
        item->setPath(path);
        listDirsChildItems(item);
    }
}

void LogDialog::loadTree()
{
    LogItem *item = new LogItem(_("Logs"), LogItem::Root, DXPREFERENCES.m_logPath);
    item->setFlags(item->flags() & ~Qt::ItemIsEditable);
    m_logModel.appendRow(item);
    listDirsChildItems(item);
    m_log->setExpanded(m_logModel.indexFromItem(item), true);
    if(!item->hasChildren())
    {
        m_text->append(_("Any logs"));
    }
}

void LogDialog::listDirsChildItems(LogItem *parent)
{
    QFileInfo info(parent->getPath());
    if(parent->getTime()!=info.lastModified())
    {
        QDir dir(parent->getPath());
        dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
        dir.setSorting(QDir::Name);
        QStringList olditems;
        for(int i=0; i<parent->rowCount(); i++)
            olditems << parent->child(i)->text();
        QFileInfoList list = dir.entryInfoList();
        for(int i=0; i<list.size(); ++i)
        {
            QFileInfo fileInfo = list.at(i);
            if(!olditems.contains(fileInfo.fileName()))
            {
                LogItem *item = new LogItem(fileInfo.fileName(), parent->getLogItemType()==LogItem::Root?LogItem::Account:LogItem::Conversation, fileInfo.absoluteFilePath());
                item->setFlags(item->flags() & ~Qt::ItemIsEditable);
                m_logModel.addLogItem(parent, item);
            }
            else
            {
                olditems.removeAll(fileInfo.fileName());
            }
        }
        parent->setTime(info.lastModified());
        // Wipe items remaining in oldlist
        for(int i=0; i<olditems.size(); i++)
        {
            for(int j=parent->rowCount()-1; j>-1; j--)
            {
                if(!olditems[i].compare(parent->child(j)->text()))
                {
                    parent->removeRow(j);
                }
            }
        }
    }
}

void LogDialog::listFilesChildItems(LogItem *parent)
{
    if(parent->getLogItemType()==LogItem::Conversation)
    {
        QFileInfo info(parent->getPath());
        if(parent->getTime()!=info.lastModified())
        {
            QDir dir(parent->getPath());
            dir.setFilter(QDir::Files | QDir::Readable);
            dir.setSorting(QDir::Name);
            QStringList oldfiles = getLogFiles(parent);
            QStringList files;
            QFileInfoList list = dir.entryInfoList();
            for(int i = 0; i < list.size(); ++i)
            {
                QFileInfo fileInfo = list.at(i);
                if(!oldfiles.contains(fileInfo.fileName()))
                {
                    if(dxutils::isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
                    {
                        files << fileInfo.fileName();
                    }
                }
                else
                {
                    oldfiles.removeAll(fileInfo.fileName());
                }
            }
            parent->setTime(info.lastModified());
            // Wipe items remaining in oldfiles
            for(int i=0; i<oldfiles.size(); i++)
            {
                for(int j=parent->rowCount()-1; j>-1; j--)
                {
                    QStandardItem *year = parent->child(j);
                    for(int k=year->rowCount()-1; k>-1; k--)
                    {
                        QStandardItem *month = year->child(k);
                        for(int m=month->rowCount()-1; m>-1; m--)
                        {
                            if(!oldfiles[i].compare(getLogFileName((LogItem*)month->child(m))))
                            {
                                month->removeRow(m);
                            }
                        }
                        if(!month->hasChildren())
                            year->removeRow(k);
                    }
                    if(!year->hasChildren())
                        parent->removeRow(j);
                }
            }
            if(files.size())
            {
                files.sort();
                QString lastYear = files[0].section('-',0,0);
                int lastMonth = files[0].section('-',1,1).toInt();
                LogItem *year = getYearItem(parent, files[0]);
                if(!year)
                {
                    year = new LogItem(lastYear, LogItem::Year, parent->getPath());
                    year->setFlags(year->flags() & ~Qt::ItemIsEditable);
                    m_logModel.addLogItem(parent, year);
                }
                LogItem *month = getMonthItem(parent, files[0]);
                if(!month)
                {
                    month = new LogItem(dxutils::getMonth(lastMonth), LogItem::Month, parent->getPath());
                    month->setFlags(month->flags() & ~Qt::ItemIsEditable);
                    m_logModel.addLogItem(year, month);
                }
                for(int i=0; i<files.size(); i++)
                {
                    if(lastYear!=files[i].section('-',0,0))
                    {
                        lastYear = files[i].section('-',0,0);
                        lastMonth = files[i].section('-',1,1).toInt();
                        year = getYearItem(parent, files[i]);
                        if(!year)
                        {
                            year = new LogItem(lastYear, LogItem::Year, parent->getPath());
                            year->setFlags(year->flags() & ~Qt::ItemIsEditable);
                            m_logModel.addLogItem(parent, year);
                        }
                        month = getMonthItem(parent, files[i]);
                        if(!month)
                        {
                            month = new LogItem(dxutils::getMonth(lastMonth), LogItem::Month, parent->getPath());
                            month->setFlags(month->flags() & ~Qt::ItemIsEditable);
                            m_logModel.addLogItem(year, month);
                        }
                        LogItem *day = new LogItem(files[i].section('-',2,2), LogItem::File, parent->getPath()+"/"+files[i]);
                        day->setFlags(year->flags() & ~Qt::ItemIsEditable);
                        m_logModel.addLogItem(month, day);
                    }
                    else if(lastMonth!=files[i].section('-',1,1).toInt())
                    {
                        lastMonth = files[i].section('-',1,1).toInt();
                        month = getMonthItem(parent, files[i]);
                        if(!month)
                        {
                            month = new LogItem(dxutils::getMonth(lastMonth), LogItem::Month, parent->getPath());
                            month->setFlags(month->flags() & ~Qt::ItemIsEditable);
                            m_logModel.addLogItem(year, month);
                        }
                        LogItem *day = new LogItem(files[i].section('-',2,2), LogItem::File, parent->getPath()+"/"+files[i]);
                        day->setFlags(year->flags() & ~Qt::ItemIsEditable);
                        m_logModel.addLogItem(month, day);
                    }
                    else
                    {
                        LogItem *day = new LogItem(files[i].section('-',2,2), LogItem::File, parent->getPath()+"/"+files[i]);
                        day->setFlags(year->flags() & ~Qt::ItemIsEditable);
                        m_logModel.addLogItem(month, day);
                    }
                }
            }
        }
    }
}

void LogDialog::slotCurrentChanged(const QModelIndex &current)
{
    m_text->clear();
    LogItem *item = static_cast<LogItem*>(m_logModel.itemFromIndex(current));
    if(item->getLogItemType()==LogItem::Root || item->getLogItemType()==LogItem::Account)
    {
        if(item->getLogItemType()==LogItem::Root)
        {
            m_buttonFile->setEnabled(false);
            m_buttonConv->setEnabled(false);
            m_buttonAccount->setEnabled(false);
            m_buttonAll->setEnabled(true);
            m_buttonAll->setChecked(true);
        }
        else
        {
            m_buttonFile->setEnabled(false);
            m_buttonConv->setEnabled(false);
            m_buttonAccount->setEnabled(true);
            m_buttonAll->setEnabled(true);
            m_buttonAccount->setChecked(true);
        }
        if(m_fullLog) listDirsChildItems(item);
    }
    else if(item->getLogItemType()==LogItem::Conversation)
    {
        m_buttonFile->setEnabled(false);
        m_buttonConv->setEnabled(true);
        m_buttonAccount->setEnabled(true);
        m_buttonAll->setEnabled(true);
        m_buttonConv->setChecked(true);
        listFilesChildItems(item);
    }
    else if(item->getLogItemType()==LogItem::Year || item->getLogItemType()==LogItem::Month)
    {
        m_buttonFile->setEnabled(false);
        m_buttonConv->setEnabled(true);
        m_buttonAccount->setEnabled(true);
        m_buttonAll->setEnabled(true);
        m_buttonConv->setChecked(true);
    }
    else if(item->getLogItemType()==LogItem::File)
    {
        m_buttonFile->setEnabled(true);
        m_buttonConv->setEnabled(true);
        m_buttonAccount->setEnabled(true);
        m_buttonAll->setEnabled(true);
        m_buttonFile->setChecked(true);
        if(!loadFile(item->getPath()))
        {
            LogItem *conv = (LogItem*)item->parent()->parent()->parent(); //Conversation
            listDirsChildItems(conv);
            m_log->setCurrentIndex(m_logModel.indexFromItem(conv));
        }
        else
        {
            searchInFile(true);
        }
    }
}

//for conversation,year and month
QStringList LogDialog::getLogFiles(LogItem* parent)
{
    QStringList files;
    if(parent->getLogItemType()==LogItem::Conversation)
    {
        for(int i=0; i<parent->rowCount(); i++)
        {
            QStandardItem *year = parent->child(i);
            for(int j=0; j<year->rowCount(); j++)
            {
                QStandardItem *month = year->child(j);
                for(int k=0; k<month->rowCount(); k++)
                {
                    files << getLogFileName((LogItem*)month->child(k));
                }
            }
        }
    }
    else if(parent->getLogItemType()==LogItem::Year)
    {
        for(int i=0; i<parent->rowCount(); i++)
        {
            QStandardItem *month = parent->child(i);
            for(int j=0; j<month->rowCount(); j++)
            {
                files << getLogFileName((LogItem*)month->child(j));
            }
        }
    }
    else if(parent->getLogItemType()==LogItem::Month)
    {
        for(int i=0; i<parent->rowCount(); i++)
        {
            files << getLogFileName((LogItem*)parent->child(i));
        }
    }
    else
    {
        files << getLogFileName(parent);
    }
    files.sort();
    return files;
}

QString LogDialog::getLogFileName(LogItem* file)
{
    if(file->getLogItemType()==LogItem::File)
    {
        QString name = file->parent()->parent()->text(); //Year item
        name.append('-');
        name.append(dxutils::getMonthNumber(file->parent()->text())); //month item
        name.append('-');
        name.append(file->text());
        return name;
    }
    return "";
}

LogItem *LogDialog::getYearItem(LogItem* conversation, const QString& filename)
{
    QString yearName = filename.section('-',0,0);
    for(int i=0; i<conversation->rowCount(); i++)
    {
        if(!conversation->child(i)->text().compare(yearName))
            return (LogItem*)conversation->child(i);
    }
    return NULL;
}

LogItem *LogDialog::getMonthItem(LogItem* conversation, const QString& filename)
{
    QString yearName = filename.section('-',0,0);
    QString monthName = dxutils::getMonth(filename.section('-',1,1).toInt());
    for(int i=0; i<conversation->rowCount(); i++)
    {
        if(!conversation->child(i)->text().compare(yearName))
        {
            QStandardItem *year = conversation->child(i);
            for(int j=0; j<year->rowCount(); j++)
            {
                if(!year->child(j)->text().compare(monthName))
                    return (LogItem*)year->child(j);
            }

        }
    }
    return NULL;
}

// Load file
bool LogDialog::loadFile(const QString& path)
{
    m_text->clear();
    QFile file(path);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::warning(this, _("Error Loading File"), _("Unable to open file: %1").arg(path));
        return false;
    }
    while(!file.atEnd())
    {
        QByteArray line = file.readLine();
        appendLogLine(QString::fromUtf8(line));
    }
    return true;
}

QColor LogDialog::stringToColor(QString style)
{
    if(style.compare("action", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_actionColor;
    if(style.compare("error", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_errorColor;
    if(style.compare("user", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_userColor;
    if(style.compare("mymsg", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_mymsgColor;
    if(style.compare("highlight", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_hilightColor;
    return DXPREFERENCES.m_textColor;
}

void LogDialog::appendLogLine(const QString &line)
{
    int pos = line.indexOf(']')+2;
    if(pos<2) pos = 0;
    if(line.length()>pos+1 && line[pos]=='\014') //log style mark
    {
        QColor style = stringToColor(line.section('\014',1,1));
        if(line.section('\014',2)[0]=='<') //has nick
        {
            QString nick = line.section('<',1,1).section('>',0,0);
            QString msg = line.section('>',1);
            if(DXPREFERENCES.m_coloredNick)
                m_text->append(line.section('\014',0,0)+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+DXPREFERENCES.m_nickCompletionChar+"</font>"
                           +QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(msg)+"</font>");
            else
                m_text->append(line.section('\014',0,0)+"<b>"+nick+DXPREFERENCES.m_nickCompletionChar+"</b>"
                           +QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(msg)+"</font>");
        }
        else m_text->append(line.section('\014',0,0)+QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(line.section('\014',2))+"</font>");
    }
    else
    {
        if(line.length()>pos+1 && line[pos]=='<') //line with nick
        {
            QString nick = line.section('<',1,1).section('>',0,0);
            QString msg = line.section('>',1);
            if(DXPREFERENCES.m_coloredNick)
                m_text->append(line.section('<',0,0)+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+DXPREFERENCES.m_nickCompletionChar+"</font>"+dxutils::parseHtml(msg));
            else
                m_text->append(line.section('<',0,0)+"<b>"+nick+DXPREFERENCES.m_nickCompletionChar+"</b>"+dxutils::parseHtml(msg));
        }
        else m_text->append(dxutils::parseHtml(line));
    }
}

void LogDialog::slotSearch()
{
    LogItem::LogItemType type = LogItem::Root;
    if(m_log->getCurrentItem())
    {
        type = m_log->getCurrentItem()->getLogItemType();
    }
    if(type != LogItem::File) m_text->clear();
    enableAllItems();
    if(m_searchLine->text().isEmpty())
        return;
    m_searchString = m_searchLine->text();
    m_text->moveCursor(QTextCursor::Start);
    if(m_buttonFile->isChecked())
        searchInFile();
    else if(m_buttonConv->isChecked())
        searchInConversation();
    else if(m_buttonAccount->isChecked())
        searchInAccount();
    else if(m_buttonAll->isChecked())
        searchInAll();
}

void LogDialog::slotReset()
{
    m_searchString = "";
    enableAllItems();
}

bool LogDialog::eventFilter(QObject *obj, QEvent *ev)
{
    if(obj==m_searchLine && ev->type()==QEvent::KeyPress)
    {
        QKeyEvent *e = (QKeyEvent *)ev;
        if(e->key()==Qt::Key_Return || e->key()==Qt::Key_Enter)
        {
            slotSearch();
            return true;
        }
    }
    else if(obj==this && ev->type()==QEvent::KeyPress)
    {
        QKeyEvent *e = (QKeyEvent *)ev;
        if(e->key()==Qt::Key_F3)
        {
            searchInFile(true);
            return true;
        }
    }
    return QWidget::eventFilter(obj, ev);
}

void LogDialog::enableAllItems()
{
    QStandardItem *root = m_logModel.item(0);
    for(int i=0; i<root->rowCount(); i++)
    {
        QStandardItem *account = root->child(i);
        account->setEnabled(true);
        for(int j=0; j<account->rowCount(); j++)
        {
            QStandardItem *conv = account->child(j);
            conv->setEnabled(true);
            for(int k=0; k<conv->rowCount(); k++)
            {
                QStandardItem *year = conv->child(k);
                year->setEnabled(true);
                for(int l=0; l<year->rowCount(); l++)
                {
                    QStandardItem *month = year->child(l);
                    month->setEnabled(true);
                    for(int m=0; m<month->rowCount(); m++)
                    {
                        month->child(m)->setEnabled(true);
                    }
                }
            }
        }
    }
}

void LogDialog::searchInFile(bool next)
{
    if(m_searchString.isEmpty() || m_text->toPlainText().isEmpty())
        return;
    bool founded = false;
    if(m_buttonIcase->isChecked()) founded = m_text->find(m_searchString);
    else founded = m_text->find(m_searchString, QTextDocument::FindCaseSensitively);
    if(founded)
        m_text->ensureCursorVisible();
    else
    {
        if(!next) QMessageBox::information(this, _("Information"), _("'%1' wasn't found").arg(m_searchString));
        else
        {
            m_text->moveCursor(QTextCursor::Start);
            searchInFile(false);
        }
    }
}

void LogDialog::searchInConversation()
{
    m_text->clear();
    m_text->append(_("Search started..."));
    LogItem *conv = m_log->getConversationItem();
    if(conv)
    {
        QMap<LogItem*,QStringList> files;
        files.insert(conv, getLogFiles(conv));
        QMap<LogItem*,QStringList> founded = findFiles(files);
        int count = 0;
        foreach(QStringList list, founded)
        {
            for(int i=0; i<conv->rowCount(); i++)
            {
                LogItem *year = (LogItem*)conv->child(i);
                for(int j=0; j<year->rowCount(); j++)
                {
                    LogItem *month = (LogItem*)year->child(j);
                    for(int l=0; l<month->rowCount(); l++)
                    {
                        LogItem *file = (LogItem*)month->child(l);
                        QString fileName = getLogFileName(file);
                        if(!list.contains(fileName))
                            file->setEnabled(false);
                    }
                    if(month->rowCount()==month->disabledCount())
                        month->setEnabled(false);
                }
                if(year->rowCount()==year->disabledCount())
                    year->setEnabled(false);
            }
            count += list.size();
        }
        if(!count) slotReset();
        m_text->append(_n("Founded <b>%1 file</b>", "Founded <b>%1 files</b>", count).arg(count));
    }
    m_text->append(_("Search finished"));
}

void LogDialog::searchInAccount()
{
    m_text->clear();
    m_text->append(_("Search started..."));
    LogItem *account = m_log->getAccountItem();
    if(account)
    {
        QMap<LogItem*,QStringList> files;
        for(int i=0; i<account->rowCount(); i++)
        {
            LogItem *conv = (LogItem*)account->child(i);
            listFilesChildItems(conv);
            files.insert(conv, getLogFiles(conv));
        }
        QMap<LogItem*,QStringList> founded = findFiles(files);
        int count = 0;
        QMapIterator<LogItem*,QStringList> it(founded);
        while(it.hasNext())
        {
            it.next();
            LogItem *conv = it.key();
            for(int i=0; i<conv->rowCount(); i++)
            {
                LogItem *year = (LogItem*)conv->child(i);
                for(int j=0; j<year->rowCount(); j++)
                {
                    LogItem *month = (LogItem*)year->child(j);
                    for(int l=0; l<month->rowCount(); l++)
                    {
                        LogItem *file = (LogItem*)month->child(l);
                        QString fileName = getLogFileName(file);
                        if(!it.value().contains(fileName))
                            file->setEnabled(false);
                    }
                    if(month->rowCount()==month->disabledCount())
                        month->setEnabled(false);
                }
                if(year->rowCount()==year->disabledCount())
                    year->setEnabled(false);
            }
            if(conv->rowCount()==conv->disabledCount())
                conv->setEnabled(false);
            count += it.value().size();
        }
        if(!count) slotReset();
        m_text->append(_n("Founded <b>%1 file</b>", "Founded <b>%1 files</b>", count).arg(count));
    }
    m_text->append(_("Search finished"));
}

void LogDialog::searchInAll()
{
    m_text->clear();
    m_text->append(_("Search started..."));
    LogItem *root = (LogItem*)m_logModel.item(0);
    //we need load full tree
    if(!m_treeLoaded)
    {
        listDirsChildItems(root);
        for(int i=0; i<root->rowCount(); i++)
        {
            LogItem *account = (LogItem*)root->child(i);
            listDirsChildItems(account);
            for(int j=0; j<account->rowCount(); j++)
            {
                LogItem *conv = (LogItem*)account->child(j);
                listFilesChildItems(conv);
            }
        }
        m_treeLoaded = true;
    }
    QMap<LogItem*,QStringList> files;
    for(int i=0; i<root->rowCount(); i++)
    {
        LogItem *account = (LogItem*)root->child(i);
        for(int j=0; j<account->rowCount(); j++)
        {
            LogItem *conv = (LogItem*)account->child(j);
            files.insert(conv, getLogFiles(conv));
        }
    }
    QMap<LogItem*,QStringList> founded = findFiles(files);
    int count = 0;
    QMapIterator<LogItem*,QStringList> it(founded);
    while(it.hasNext())
    {
        it.next();
        LogItem *conv = it.key();
        for(int i=0; i<conv->rowCount(); i++)
        {
            LogItem *year = (LogItem*)conv->child(i);
            for(int j=0; j<year->rowCount(); j++)
            {
                LogItem *month = (LogItem*)year->child(j);
                for(int l=0; l<month->rowCount(); l++)
                {
                    LogItem *file = (LogItem*)month->child(l);
                    QString fileName = getLogFileName(file);
                    if(!it.value().contains(fileName))
                        file->setEnabled(false);
                }
                if(month->rowCount()==month->disabledCount())
                    month->setEnabled(false);
            }
            if(year->rowCount()==year->disabledCount())
                year->setEnabled(false);
        }
        if(conv->rowCount()==conv->disabledCount())
            conv->setEnabled(false);
        count += it.value().size();
    }
    for(int i=0; i<root->rowCount(); i++)
    {
        LogItem *account = (LogItem*)root->child(i);
        if(account->rowCount()==account->disabledCount())
            account->setEnabled(false);
    }
    if(!count) slotReset();
    m_text->append(_n("Founded <b>%1 file</b>", "Founded <b>%1 files</b>", count).arg(count));
    m_text->append(_("Search finished"));
}

QMap<LogItem*,QStringList> LogDialog::findFiles(const QMap<LogItem*,QStringList> &files)
{
    QProgressDialog progressDialog(this);
    progressDialog.setCancelButtonText(_("&Cancel"));
    progressDialog.setRange(0, files.size());
    progressDialog.setWindowTitle(_("Find Files"));
    QMap<LogItem*,QStringList> founded;
    quint64 count = 0;
    foreach(QStringList list, files)
    {
        count += list.size();
    }
    QMapIterator<LogItem*,QStringList> it(files);
    quint64 i = 0;
    while(it.hasNext())
    {
        it.next();
        QStringList fileList = it.value();
        QStringList foundedList;
        for(int j=0; j<fileList.size(); ++j)
        {
            progressDialog.setValue(i);
            progressDialog.setLabelText(_("Searching file number %1 of %2...")
                                        .arg(i).arg(count));
            qApp->processEvents();
            if(progressDialog.wasCanceled())
                break;
            QFile file(it.key()->getPath()+"/"+(fileList[j]));
            if(file.open(QIODevice::ReadOnly))
            {
                QString line;
                QTextStream in(&file);
                while(!in.atEnd())
                {
                    if(progressDialog.wasCanceled())
                        break;
                    line = in.readLine();
                    if(line.contains(m_searchString, m_buttonIcase->isChecked()?Qt::CaseInsensitive:Qt::CaseSensitive))
                    {
                        foundedList << fileList[j];
                        break;
                    }
                }
            }
        }
        founded.insert(it.key(), foundedList);
        i++;
    }
    return founded;
}

void LogDialog::slotDeleteItem(LogItem *item)
{
    if(item->getLogItemType()==LogItem::File)
    {
        if(QMessageBox::question(this, _("Question"), _("Delete file %1?\nThis cann't be UNDONE!").arg(getLogFileName(item)),
                QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            QFile(item->getPath()).remove();
            listFilesChildItems((LogItem*)item->parent()->parent()->parent());
        }
    }
    else
    {
        if(QMessageBox::question(this, _("Question"), _("Delete %1 with all child items?\nThis cann't be UNDONE!").arg(item->text()),
                QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            if(item->getLogItemType()==LogItem::Month)
            {
                foreach(QString file, getLogFiles(item))
                    QFile::remove(item->getPath()+"/"+file);
                listFilesChildItems((LogItem*)item->parent()->parent());
            }
            else if(item->getLogItemType()==LogItem::Year)
            {
                foreach(QString file, getLogFiles(item))
                    QFile::remove(item->getPath()+"/"+file);
                listFilesChildItems((LogItem*)item->parent());
            }
            else
            {
                dxutils::removeDir(item->getPath());
                listDirsChildItems((LogItem*)item->parent());
            }
        }
    }
}



