/*
 *      logdialog.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef LOGDIALOG_H
#define LOGDIALOG_H

#include <QDialog>
#include <QSplitter>
#include <QTextBrowser>
#include <QPushButton>
#include <QLineEdit>
#include <QCheckBox>
#include <QRadioButton>
#include <QVBoxLayout>
#include "logitemmodel.h"
#include "logview.h"
#include "purplemm.h"

class LogDialog : public QDialog
{
    Q_OBJECT
public:
    LogDialog(QWidget *parent = 0);
    LogDialog(PURPLE::Account account, QWidget *parent = 0);
    LogDialog(PURPLE::Buddy buddy, QWidget *parent = 0);
    LogDialog(PURPLE::Chat chat, QWidget *parent = 0);

    void setLogPath(const QString &path);
private:
    QVBoxLayout *m_mainLayout;
    QTextBrowser *m_text;
    LogView *m_log;
    QSplitter *m_splitter;
    LogItemModel m_logModel;
    QPushButton *m_buttonSearch, *m_buttonReset;
    QLineEdit *m_searchLine;
    QCheckBox *m_buttonIcase;
    QRadioButton *m_buttonAll, *m_buttonAccount, *m_buttonConv, *m_buttonFile;
    bool m_treeLoaded;
    QString m_searchString;
    bool m_fullLog;

    void createWidgets();
    void loadTree();
    void listDirsChildItems(LogItem *parent);
    void listFilesChildItems(LogItem *parent);
    QStringList getLogFiles(LogItem *parent);
    QString getLogFileName(LogItem *file);
    LogItem *getYearItem(LogItem *conversation, const QString &filename);
    LogItem *getMonthItem(LogItem *conversation, const QString &filename);
    bool loadFile(const QString& path);
    QColor stringToColor(QString style);
    void appendLogLine(const QString &line);
    void enableAllItems();
    void searchInFile(bool next=false);
    void searchInConversation();
    void searchInAccount();
    void searchInAll();
    QMap<LogItem*,QStringList> findFiles(const QMap<LogItem*,QStringList> &files);
private slots:
    void slotCurrentChanged(const QModelIndex &current);
    void slotSearch();
    void slotReset();
    void slotDeleteItem(LogItem *item);
protected:
    bool eventFilter(QObject *obj, QEvent *ev);
};

#endif // LOGDIALOG_H
