/*
 *      logitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "logitem.h"

LogItem::LogItem(const QString &name, LogItemType type, const QString &path)
{
    setText(name);
    setData(static_cast<int>(type), LogItem::LType);
    setData(path, LogItem::LPath);
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    setData(QDateTime::fromSecsSinceEpoch(0), LogItem::LTime);
#else
    QDateTime time;
    time.setTime_t(0);
    setData(time, LogItem::LTime);
#endif
    if(type==LogItem::File)
        setIcon(QIcon(DXIM_DATADIR "/icons/page.png"));
    else if(type!=LogItem::Root)
        setIcon(QIcon(DXIM_DATADIR "/icons/folder.png"));
}

LogItem::LogItemType LogItem::getLogItemType()
{
    return static_cast<LogItem::LogItemType>(data(LogItem::LType).toInt());
}

QString LogItem::getPath()
{
    return data(LogItem::LPath).toString();
}

void LogItem::setPath(const QString &path)
{
    setData(path, LogItem::LPath);
}

void LogItem::setTime(QDateTime time)
{
    setData(time, LogItem::LTime);
}

QDateTime LogItem::getTime()
{
    return data(LogItem::LTime).toDateTime();
}

int LogItem::disabledCount()
{
    int count = 0;
    for(int i=0; i<rowCount(); i++)
        if(!child(i)->isEnabled())
            count++;
    return count;
}



