/*
 *      logview.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "logview.h"
#include "logitemmodel.h"
#include "defs.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include <QMenu>

LogView::LogView(QWidget *parent) :
    QTreeView(parent)
{
    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotLogPopup(QPoint)));
}

void LogView::slotLogPopup(const QPoint &pos)
{
    if(!m_fullLogs) return;
    LogItem *item = getCurrentItem();
    if(item && item->getLogItemType()!=LogItem::Root)
    {
        QPoint globalPos = mapToGlobal(pos);
        QMenu popup;
        QAction *del = new QAction(_("Delete"), this);
        connect(del, SIGNAL(triggered()), this, SLOT(slotDelete()));
        popup.addAction(del);
        popup.exec(globalPos);
    }
}

void LogView::slotDelete()
{
    LogItem *item = getCurrentItem();
    if(item && item->getLogItemType()!=LogItem::Root)
    {
        emit deleteItem(item);
    }
}

LogItem *LogView::getConversationItem()
{
    if(selectedIndexes().size() > 0)
    {
        LogItemModel *logmodel = (LogItemModel*)model();
        if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Conversation)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0));
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Year)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent();
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Month)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent()->parent();
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::File)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent()->parent()->parent();
        }
        else
            return NULL;
    }
    return NULL;
}

LogItem *LogView::getAccountItem()
{
    if(selectedIndexes().size() > 0)
    {
        LogItemModel *logmodel = (LogItemModel*)model();
        if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Account)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0));
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Conversation)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent();
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Year)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent()->parent();
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::Month)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent()->parent()->parent();
        }
        else if(selectedIndexes().at(0).data(LogItem::LType).toInt()==LogItem::File)
        {
            return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0))->parent()->parent()->parent()->parent();
        }
        else
            return NULL;
    }
    return NULL;
}

LogItem *LogView::getCurrentItem()
{
    if(selectedIndexes().size() > 0)
    {
        LogItemModel *logmodel = (LogItemModel*)model();
        return (LogItem*)logmodel->itemFromIndex(selectedIndexes().at(0));
    }
    return NULL;
}

void LogView::setFullLogs(bool fullLogs)
{
    m_fullLogs = fullLogs;
}

