/*
 *      preferences.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <QColor>
#include <QFont>
#include <QList>
#include <QMap>
#include "defs.h"

#define DXPREFERENCES Preferences::instance()

class Preferences
{
public:
    Preferences();
    Preferences(const Preferences &p);

    //Set path for -l argv
    void setIniPath(const QString &path);
    //Get ini file path, usefull e.g. PURPLE::Purple
    QString getIniPath();
    //Save to QSettings
    void save();
    //Load from QSettings
    void load();
    static Preferences &instance();

    //General settings tab in Configdialog
    bool m_oneInstance;
    bool m_useAway;
    int m_awayTimeout; //in minutes
    bool m_minToTray;
    bool m_logging;
    QString m_logPath;
    bool m_logLoadCurrent;
    bool m_showRosterTree;
    bool m_receiptsMark;
    //Icons settings tab in Configdialog
    QString m_themeStatusPath;
    QString m_themesStatusList;
    QString m_themeRolePath;
    QString m_themesRoleList;
    //Look tab in ConfigDialog
    QColor m_unreadtabColor;
    QColor m_hltabColor;
    int m_tabPosition;
    //Im Text settings tab in Configdialog
    QColor m_textColor;
    QColor m_backroundColor;
    QColor m_userColor;
    QColor m_actionColor;
    QColor m_errorColor;
    QColor m_hilightColor;
    QColor m_linkColor;
    QColor m_mymsgColor;
    QColor m_logColor;
    QFont m_imFont;
    bool m_sameCmd;
    bool m_sameList;
    QString m_nickCompletionChar;
    bool m_coloredNick;
    //Notification settings tab in Configdialog
    bool m_sounds;
    bool m_soundConnect;
    QString m_pathConnect;
    bool m_soundDisconnect;
    QString m_pathDisconnect;
    bool m_soundMessage;
    QString m_pathMessage;
    bool m_notify;
    int m_notifyPosition;
    bool m_notifyAccountConnect;
    bool m_notifyAccountDisconnect;
    bool m_notifyBuddyConnect;
    bool m_notifyBuddyDisconnect;
    bool m_notifyMessage;
    bool m_notifyFiletransfer;
    bool m_raiseWindow;
    bool m_notifyMessageTray; //true = all messages change tray icon
    //Smileyes settings tab in ConfigDialog
    bool m_useSmileys;
    QMap<QString,QString> m_smileysMap;
    //Other settings value
    bool m_hideOffline;
    bool m_rosterShown;
    bool m_chatRosterShown;
    bool m_showAvatars;
    bool m_showStatusmsg;
    int m_x;
    int m_y;
    int m_w;
    int m_h;
    bool m_maximized;
    int m_rosterWidth;
private:
    QString m_iniPath;
};

#endif // PREFERENCES_H
