/*
 *      rosteritem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ROSTERITEM_H
#define ROSTERITEM_H

#include <QStandardItem>
#include <QItemDelegate>
#include <QPainter>
#include <QTime>
#include "defs.h"

class RosterItem : public QStandardItem
{
public:
    enum RosterItemType {
        Account = 0,
        Group,
        Buddy,
        Chat
    };

    enum UserRoles {
        RAccount = Qt::UserRole+2,
        RBuddy,
        RChat,
        RType,
        RID
    };

    RosterItem(const QString &name, const QString &ID, RosterItemType type, PURPLE::Account account);

    void setName(const QString &name);
    QString getName();
    void setID(const QString &ID);
    QString getID();
    void setRosterItemType(RosterItemType type);
    RosterItemType getRosterItemType();
    void setAccount(PURPLE::Account account);
    PURPLE::Account getAccount();
    void setBuddy(PURPLE::Buddy buddy);
    PURPLE::Buddy getBuddy();
    void setChat(PURPLE::Chat chat);
    PURPLE::Chat getChat();
    void accountConnecting();
    void buddySignedOn();
    void buddySignedOff();
    void buddyStatusChanged();
    void updateTooltip();
    void updateStatusIconTheme();
private:
    QTime m_lastPresenceChange;
};

class RosterItemDelegate : public QItemDelegate
{
public:
    RosterItemDelegate();
    QSize sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const;
    void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;
};

#endif // ROSTERITEM_H
