/*
 *      rosteritemsortmodel.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "rosteritemsortmodel.h"
#include "rosteritem.h"
#include "dxutils.h"
#include "preferences.h"

RosterItemSortModel::RosterItemSortModel(QObject *parent) :
    QSortFilterProxyModel(parent), m_hideOfflineBuddy(DXPREFERENCES.m_hideOffline)
{
    setDynamicSortFilter(true);
    setFilterRole(Qt::DisplayRole);
    setFilterCaseSensitivity(Qt::CaseInsensitive);
}

void RosterItemSortModel::slotShowOfflineBuddy(bool hideOfflineBuddy)
{
    m_hideOfflineBuddy = hideOfflineBuddy;
    invalidateFilter();
}

bool RosterItemSortModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
{
    int compare = left.data().toString().compare(right.data().toString(), Qt::CaseInsensitive);
    if(compare < 0)
        return true;
    else
        return false;
}

bool RosterItemSortModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    if(filterRegularExpression().isValid())
        return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
#else
    if(!filterRegExp().isEmpty())
        return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
#endif
    QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
    int rosterType = sourceModel()->data(index, RosterItem::RType).toInt();
    bool online = sourceModel()->data(index, RosterItem::RAccount).value<PURPLE::Account>().isConnected();
    if(rosterType==RosterItem::Chat)
    {
        if(online) return true;
        else return false;
    }
    else if(rosterType==RosterItem::Account)
    {
        return true;
    }
    else if(rosterType==RosterItem::Group)
    {
        if(!online)
            return false;
        if(!m_hideOfflineBuddy)
            return true;
        else
        {
            PURPLE::Account account = sourceModel()->data(index, RosterItem::RAccount).value<PURPLE::Account>();
            std::string name = dxutils::toStdString(sourceModel()->data(index, RosterItem::RID).toString());
            return account.isGroupOnline(name);
        }
    }
    else
    {
        if(!online)
            return false;
        if(!m_hideOfflineBuddy)
            return true;
        else
        {
            PURPLE::Buddy buddy = sourceModel()->data(index, RosterItem::RBuddy).value<PURPLE::Buddy>();
            if(buddy.isValid() && buddy.isOnline())
                return true;
            else
                return false;
        }
    }
}
