/*
 *      rosterview.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "rosterview.h"
#include "rosteritem.h"
#include "defs.h"
#include "dxutils.h"
#include "preferences.h"
#include "config.h"
#include "i18n.h"
#include "dialogs/chatdialog.h"
#include "dialogs/chatjoindialog.h"
#include "dialogs/imdialog.h"
#include "dialogs/accounteditordialog.h"
#include "logviewer/logdialog.h"
#include "dxim.h"
#include <QMenu>
#include <QInputDialog>
#include <QLineEdit>
#include <QMessageBox>
#include <QDialog>
#include <QDialogButtonBox>
#include <QGridLayout>
#include <QLineEdit>
#include <QCheckBox>
#include <QStandardItemModel>

BuddyAction::BuddyAction(const QString& text, QObject* parent, PURPLE::Buddy::BuddyMenuItem item) :
QAction(text, parent), m_item(item)
{}

void BuddyAction::runCallback()
{
    m_item.runCallback();
}

RosterView::RosterView(PURPLE::Purple *purple, QWidget *parent) :
    QTreeView(parent), m_purple(purple), m_parent(parent)
{
    setStyleSheet(QString("QTreeView { \
                    background-color: %1; \
                    } \
                    QTreeView::branch { \
                    image: none; \
                    } \
                    QTreeView::branch:open:has-children:!has-siblings, \
                    QTreeView::branch:open:has-children:has-siblings { \
                    image: url(%2); \
                    } \
                    QTreeView::branch:has-children:!has-siblings:closed, \
                    QTreeView::branch:closed:has-children:has-siblings { \
                    image: url(%3); \
                    } \
                    ").arg(DXPREFERENCES.m_backroundColor.name())
                    .arg(DXIM_DATADIR "/icons/groupopen.png")
                    .arg(DXIM_DATADIR "/icons/groupclose.png"));
    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotRosterPopup(QPoint)));
    connect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleClicked(QModelIndex)));
}

void RosterView::updateTheme()
{
    setStyleSheet(QString("QTreeView { \
                    background-color: %1; \
                    } \
                    QTreeView::branch { \
                    image: none; \
                    } \
                    QTreeView::branch:open:has-children:!has-siblings, \
                    QTreeView::branch:open:has-children:has-siblings { \
                    image: url(%2); \
                    } \
                    QTreeView::branch:has-children:!has-siblings:closed, \
                    QTreeView::branch:closed:has-children:has-siblings { \
                    image: url(%3); \
                    } \
                    ").arg(DXPREFERENCES.m_backroundColor.name())
                    .arg(DXIM_DATADIR "/icons/groupopen.png")
                    .arg(DXIM_DATADIR "/icons/groupclose.png"));
}

void RosterView::slotRosterPopup(const QPoint &pos)
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
            if(buddy.isValid())
            {
                QPoint globalPos = mapToGlobal(pos);
                QMenu popup;
                bool online = buddy.isOnline();
                if(online)
                {
                    QAction *im = new QAction(_("IM"), this);
                    connect(im, SIGNAL(triggered()), this, SLOT(slotBuddyIM()));
                    popup.addAction(im);
                }
                QAction *info = new QAction(_("Info"), this);
                connect(info, SIGNAL(triggered()), this, SLOT(slotBuddyInfo()));
                popup.addAction(info);
                if(online && buddy.getAccount().supportsSendFile())
                {
                    QAction *file = new QAction(_("Send file"), this);
                    connect(file, SIGNAL(triggered()), this, SLOT(slotBuddySendFile()));
                    popup.addAction(file);
                }
                if(DXPREFERENCES.m_logging)
                {
                    QAction *log = new QAction(_("Show logs"), this);
                    connect(log, SIGNAL(triggered()), this, SLOT(slotBuddyLogs()));
                    popup.addAction(log);
                }
                popup.addSeparator();
                QAction *rename = new QAction(_("Rename"), this);
                connect(rename, SIGNAL(triggered()), this, SLOT(slotRenameBuddy()));
                popup.addAction(rename);
                QAction *move = new QAction(_("Move"), this);
                connect(move, SIGNAL(triggered()), this, SLOT(slotMoveBuddy()));
                popup.addAction(move);
                QAction *remove = new QAction(_("Remove"), this);
                connect(remove, SIGNAL(triggered()), this, SLOT(slotRemoveBuddy()));
                popup.addAction(remove);
                std::vector<PURPLE::Buddy::BuddyMenuItem> v = buddy.getMenuItems();
                if(v.size()) popup.addSeparator();
                FOREACH(std::vector<PURPLE::Buddy::BuddyMenuItem>, v, it)
                {
                    BuddyAction *action = new BuddyAction(dxutils::fromStdString((*it).getLabel()).replace('_','&'), this, (*it));
                    popup.addAction(action);
                    connect(action, SIGNAL(triggered()), this, SLOT(slotBuddyCb()));
                }
                popup.exec(globalPos);
            }
        }
        else if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Chat)
        {
            PURPLE::Chat chat = selectedIndexes().at(0).data(RosterItem::RChat).value<PURPLE::Chat>();
            if(chat.isValid())
            {
                QPoint globalPos = mapToGlobal(pos);
                QMenu popup;
                QAction *join = new QAction(_("Join"), this);
                connect(join, SIGNAL(triggered()), this, SLOT(slotJoinChat()));
                popup.addAction(join);
                if(DXPREFERENCES.m_logging)
                {
                    QAction *log = new QAction(_("Show logs"), this);
                    connect(log, SIGNAL(triggered()), this, SLOT(slotChatLogs()));
                    popup.addAction(log);
                }
                popup.addSeparator();
                QAction *modify = new QAction(_("Modify"), this);
                connect(modify, SIGNAL(triggered()), this, SLOT(slotModifyChat()));
                popup.addAction(modify);
                QAction *remove = new QAction(_("Remove"), this);
                connect(remove, SIGNAL(triggered()), this, SLOT(slotRemoveChat()));
                popup.addAction(remove);
                popup.exec(globalPos);
            }
        }
        else if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                QPoint globalPos = mapToGlobal(pos);
                QMenu popup;
                if(account.isEnabled())
                {
                    connect(popup.addAction(_("Status")), SIGNAL(triggered()), this, SLOT(slotAccountStatus()));
                    popup.addSeparator();
                    connect(popup.addAction(_("Reconnect")), SIGNAL(triggered()), this, SLOT(slotReconnectAccount()));
                    popup.addSeparator();
                    connect(popup.addAction(_("Disable")), SIGNAL(triggered()), this, SLOT(slotEnableAccount()));
                }
                else
                {
                    connect(popup.addAction(_("Enable")), SIGNAL(triggered()), this, SLOT(slotEnableAccount()));
                }
                popup.addSeparator();
                connect(popup.addAction(_("Account settings")), SIGNAL(triggered()), this, SLOT(slotConfigAccount()));
                connect(popup.addAction(_("Alias")), SIGNAL(triggered()), this, SLOT(slotAliasAccount()));
                if(DXPREFERENCES.m_logging)
                {
                    QAction *log = new QAction(_("Show logs"), this);
                    connect(log, SIGNAL(triggered()), this, SLOT(slotAccountLogs()));
                    popup.addAction(log);
                }
                if(account.isConnected())
                {
                    popup.addSeparator();
                    connect(popup.addAction(_("New IM")), SIGNAL(triggered()), this, SLOT(slotAccountIM()));
                    if(account.supportsChats()) connect(popup.addAction(_("Join chat")), SIGNAL(triggered()), this, SLOT(slotAccountChat()));
                    popup.addSeparator();
                    connect(popup.addAction(_("Add buddy")), SIGNAL(triggered()), this, SLOT(slotAccountAddBuddy()));
                    if(account.supportsChats()) connect(popup.addAction(_("Add chat")), SIGNAL(triggered()), this, SLOT(slotAccountAddChat()));
                    connect(popup.addAction(_("Add group")), SIGNAL(triggered()), this, SLOT(slotAccountAddGroup()));
                    std::vector<std::string> v = account.getCommandsList();
                    if(v.size())
                        popup.addSeparator();
                    FOREACH(std::vector<std::string>, v, it)
                    {
                        if((*it).compare("----"))
                        {
                            QAction *a = new QAction(dxutils::fromStdString((*it)), this);
                            popup.addAction(a);
                            connect(a, SIGNAL(triggered()), this, SLOT(slotAccountCb()));
                        }
                        else
                        {
                            popup.addSeparator();
                        }
                    }
                }
                popup.exec(globalPos);
            }
        }
    }
}

void RosterView::slotDoubleClicked(const QModelIndex &)
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
                if(buddy.isValid())
                {
                    m_purple->createConversation(account, buddy);
                }
            }
        }
        else if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Chat)
        {
            PURPLE::Chat chat = selectedIndexes().at(0).data(RosterItem::RChat).value<PURPLE::Chat>();
            if(chat.isValid())
            {
                chat.join();
            }
        }
    }
}

void RosterView::slotBuddyCb()
{
    BuddyAction *action = static_cast<BuddyAction*>(sender());
    action->runCallback();
}

void RosterView::slotBuddyIM()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
                if(buddy.isValid())
                {
                    m_purple->createConversation(account, buddy);
                }
            }
        }
    }
}

void RosterView::slotBuddyInfo()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
                if(buddy.isValid())
                {
                    buddy.retrieveInfo();
                }
            }
        }
    }
}

void RosterView::slotBuddySendFile()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
                if(buddy.isValid())
                {
                    buddy.sendFile();
                }
            }
        }
    }
}

void RosterView::slotAccountCb()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                QAction *action = static_cast<QAction*>(sender());
                account.callCommand(dxutils::toStdString(action->text()));
            }
        }
    }
}

void RosterView::slotAccountStatus()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                QDialog dialog(this);
                QGridLayout *matrix = new QGridLayout;
                matrix->addWidget(new QLabel(_("Account:")), 0, 0);
                matrix->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(account.getProtocol().getPurpleID()))).arg(dxutils::fromStdString(account.getUsername()))), 0, 1);
                matrix->addWidget(new QLabel(_("Status:")), 1, 0);
                QComboBox *statuses = new QComboBox;
                std::vector<PURPLE::Account::AccountStatusItem> v = account.getStatusMenuItems();
                FOREACH(std::vector<PURPLE::Account::AccountStatusItem>, v, it)
                {
                    statuses->addItem(dxutils::fromStdString((*it).getName()), dxutils::fromStdString((*it).getID()));
                }
                matrix->addWidget(statuses, 1, 1);
                matrix->addWidget(new QLabel(_("Status message:")), 2, 0);
                QLineEdit *msg = new QLineEdit;
                msg->setText(dxutils::fromStdString(account.getStatusMessage()));
                matrix->addWidget(msg, 2, 1);
                QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
                connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
                connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
                QVBoxLayout *mainFrame = new QVBoxLayout(&dialog);
                mainFrame->addLayout(matrix);
                mainFrame->addWidget(buttons);
                dialog.setLayout(mainFrame);
                dialog.setWindowTitle(_("Set status"));
                if(dialog.exec()==QDialog::Accepted)
                {
                    account.setStatus(dxutils::toStdString(statuses->itemData(statuses->currentIndex()).toString()));
                    if(account.isConnected()) account.setStatusMessage(dxutils::toStdString(msg->text()));
                }
            }
        }
    }
}

void RosterView::slotJoinChat()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Chat)
        {
            PURPLE::Chat chat = selectedIndexes().at(0).data(RosterItem::RChat).value<PURPLE::Chat>();
            if(chat.isValid())
            {
                chat.join();
            }
        }
    }
}

void RosterView::slotAccountAddBuddy()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                QDialog dialog(this);
                QGridLayout *matrix = new QGridLayout;
                matrix->addWidget(new QLabel(_("Account:")), 0, 0);
                matrix->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(account.getProtocol().getPurpleID()))).arg(dxutils::fromStdString(account.getUsername()))), 0, 1);
                matrix->addWidget(new QLabel(_("ID:")), 1, 0);
                QLineEdit *id = new QLineEdit;
                matrix->addWidget(id, 1, 1);
                matrix->addWidget(new QLabel(_("Nick:")), 2, 0);
                QLineEdit *nick = new QLineEdit;
                matrix->addWidget(nick, 2, 1);
                matrix->addWidget(new QLabel(_("Group:")), 3, 0);
                QComboBox *groups = new QComboBox;
                std::vector<std::string> v = account.getAllGroups();
                bool friends = false;
                FOREACH(std::vector<std::string>, v, it)
                {
                    if(dxutils::fromStdString(*it) == _("Friends"))
                        friends = true;
                    groups->addItem(dxutils::fromStdString(*it));
                }
                if(!friends) groups->insertItem(0, _("Friends"));
                groups->setEditable(true);
                groups->setCurrentIndex(0);
                matrix->addWidget(groups, 3, 1);
                QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
                connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
                connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
                QVBoxLayout *mainFrame = new QVBoxLayout(&dialog);
                mainFrame->addLayout(matrix);
                mainFrame->addWidget(buttons);
                dialog.setLayout(mainFrame);
                dialog.setWindowTitle(_("Add friend"));
                if(dialog.exec()==QDialog::Accepted
                    && !id->text().isEmpty())
                {
                    std::string group = dxutils::toStdString(groups->currentText());
                    if(group.empty()) group = dxutils::toStdString(_("Friends"));
                    account.addBuddy(dxutils::toStdString(id->text()),
                                     group,
                                     dxutils::toStdString(_("I want to add you in buddies")),
                                     dxutils::toStdString(nick->text()));
                }
            }
        }
    }
}

void RosterView::slotAccountAddChat()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                ChatDialog *dialog = new ChatDialog(account, this);
                dialog->setModal(true);
                dialog->exec();
            }
        }
    }
}

void RosterView::slotAccountAddGroup()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                bool ok;
                QString group = QInputDialog::getText(this, _("Add group"), _("New group:"), QLineEdit::Normal, "", &ok);
                if(!ok)
                    return;
                account.addGroup(dxutils::toStdString(group));
            }
        }
    }
}

void RosterView::slotModifyChat()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Chat)
        {
            PURPLE::Chat chat = selectedIndexes().at(0).data(RosterItem::RChat).value<PURPLE::Chat>();
            if(chat.isValid())
            {
                QString alias = dxutils::fromStdString(chat.getAlias());
                ChatDialog *dialog = new ChatDialog(chat, this);
                dialog->setModal(true);
                if(dialog->exec() == QDialog::Accepted)
                {
                    qobject_cast<dxim*>(m_parent)->slotChatRemoved(chat.getAccount(), alias);
                    qobject_cast<dxim*>(m_parent)->slotChatAdded(chat);
                }
            }
        }
    }
}

void RosterView::slotRemoveChat()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Chat)
        {
            PURPLE::Chat chat = selectedIndexes().at(0).data(RosterItem::RChat).value<PURPLE::Chat>();
            if(chat.isValid())
            {
                if(QMessageBox::question(this, _("Question"), _("Do you want really remove %1?").arg(dxutils::fromStdString(chat.getAlias())), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
                {
                    chat.getAccount().removeChat(chat);
                }
            }
        }
    }
}

void RosterView::slotRenameBuddy()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
            if(buddy.isValid())
            {
                QDialog rename(this);
                QGridLayout *matrix = new QGridLayout;
                matrix->addWidget(new QLabel(_("ID:")), 0, 0);
                matrix->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(buddy.getAccount().getProtocol().getPurpleID()))).arg(dxutils::fromStdString(buddy.getUsername()))), 0, 1);
                matrix->addWidget(new QLabel(_("Alias:")), 1, 0);
                QLineEdit *alias = new QLineEdit;
                alias->setText(dxutils::fromStdString(buddy.getAlias()));
                matrix->addWidget(alias, 1, 1);
                QCheckBox *autojoin = new QCheckBox(_("Set on server too"));
                if(buddy.getAccount().hasServerAliases())
                {
                    autojoin->setChecked(true);
                }
                else
                {
                    autojoin->setChecked(false);
                    autojoin->hide();
                }
                QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
                connect(buttons, SIGNAL(accepted()), &rename, SLOT(accept()));
                connect(buttons, SIGNAL(rejected()), &rename, SLOT(reject()));
                QVBoxLayout *mainFrame = new QVBoxLayout(&rename);
                mainFrame->addLayout(matrix);
                mainFrame->addWidget(autojoin);
                mainFrame->addWidget(buttons);
                rename.setLayout(mainFrame);
                rename.setWindowTitle(_("Rename friend"));
                if(rename.exec()==QDialog::Accepted)
                {
                    buddy.setAlias(dxutils::toStdString(alias->text()), autojoin->isChecked());
                    qobject_cast<dxim*>(m_parent)->slotBuddyRemoved(buddy.getAccount(), dxutils::fromStdString(buddy.getUsername()));
                    qobject_cast<dxim*>(m_parent)->slotBuddyAdded(buddy);
                }
            }
        }
    }
}

void RosterView::slotMoveBuddy()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
            if(buddy.isValid())
            {
                QDialog move(this);
                QGridLayout *matrix = new QGridLayout;
                matrix->addWidget(new QLabel(_("ID:")), 0, 0);
                matrix->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(buddy.getAccount().getProtocol().getPurpleID()))).arg(dxutils::fromStdString(buddy.getUsername()))), 0, 1);
                matrix->addWidget(new QLabel(_("Group:")), 1, 0);
                QComboBox *groups = new QComboBox;
                std::vector<std::string> v = buddy.getAccount().getAllGroups();
                bool friends = false;
                FOREACH(std::vector<std::string>, v, it)
                {
                    if(*it != buddy.getGroupName())
                        groups->addItem(dxutils::fromStdString(*it));
                    if(dxutils::fromStdString(*it) == _("Friends"))
                        friends = true;
                }
                if(dxutils::fromStdString(buddy.getGroupName()) != _("Friends") && !friends)
                    groups->insertItem(0, _("Friends"));
                groups->setEditable(true);
                groups->setCurrentIndex(0);
                matrix->addWidget(groups, 1, 1);
                QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
                connect(buttons, SIGNAL(accepted()), &move, SLOT(accept()));
                connect(buttons, SIGNAL(rejected()), &move, SLOT(reject()));
                QVBoxLayout *mainFrame = new QVBoxLayout(&move);
                mainFrame->addLayout(matrix);
                mainFrame->addWidget(buttons);
                move.setLayout(mainFrame);
                move.setWindowTitle(_("Move friend"));
                if(move.exec()==QDialog::Accepted)
                {
                    std::string oldgroup = buddy.getGroupName();
                    std::string newgroup = dxutils::toStdString(groups->currentText());
                    if(newgroup.empty()) newgroup = dxutils::toStdString(_("Friends"));
                    if(oldgroup != newgroup) buddy.getAccount().moveBuddy(buddy, newgroup);
                }
            }
        }
    }
}

void RosterView::slotRemoveBuddy()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
            if(buddy.isValid())
            {
                if(QMessageBox::question(this, _("Question"), _("Do you want really remove %1?").arg(dxutils::fromStdString(buddy.getUsername())), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
                {
                    buddy.getAccount().removeBuddy(buddy);
                }
            }
        }
    }
}

void RosterView::slotReconnectAccount()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid() && account.isEnabled())
            {
                if(account.isConnecting())
                    return;
                if(account.isConnected())
                    account.disconnect();
                account.connect();
            }
        }
    }
}

void RosterView::slotEnableAccount()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                if(account.isEnabled())
                    account.disconnect();
                else
                    account.connect();
            }
        }
    }
}

void RosterView::slotAliasAccount()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                bool ok;
                QString alias = QInputDialog::getText(this, _("Set alias"), _("Alias"), QLineEdit::Normal, dxutils::fromStdString(account.getAlias()), &ok);
                if(!ok)
                    return;
                account.setAlias(dxutils::toStdString(alias));
            }
        }
    }
}

void RosterView::slotAccountIM()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                ImDialog *dialog = new ImDialog(account, this);
                dialog->setModal(true);
                if(dialog->exec() == QDialog::Accepted)
                {
                    QString IMID, IMnick;
                    dialog->fillIMInfo(IMID, IMnick);
                    if(IMID.isEmpty()) return;
                    std::vector<PURPLE::Buddy> v = account.getBuddies();
                    PURPLE::Buddy buddy;
                    for(uint j=0; j<v.size(); j++)
                    {
                        if(dxutils::fromStdString(v[j].getUsername())==IMID)
                        {
                            buddy = v[j];
                            m_purple->createConversation(account, buddy);
                            return;
                        }
                    }
                    m_purple->createConversation(account, dxutils::toStdString(IMID), false);
                }
            }
        }
    }
}

void RosterView::slotAccountChat()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                ChatJoinDialog *dialog = new ChatJoinDialog(account, qobject_cast<dxim*>(m_parent)->getCallbacks(), this);
                dialog->show();
            }
        }
    }
}

void RosterView::slotConfigAccount()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                AccountEditorDialog *dialog = new AccountEditorDialog(account, this);
                dialog->setModal(true);
                if(dialog->exec() == QDialog::Accepted)
                {
                    NewAccount *nac = new NewAccount;
                    dialog->fillAccount(nac);
                    account.setOptions(nac->options);
                    account.setPassword(nac->password, nac->rememberPassword);
                    account.setCheckMail(nac->checkEmail);
                    account.setAlias(nac->alias);
                    nac->iconPath.empty()?account.removeBuddyIcon():account.setBuddyIcon(nac->iconPath);
                }
            }
        }
    }
}

void RosterView::slotBuddyLogs()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Buddy)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                PURPLE::Buddy buddy = selectedIndexes().at(0).data(RosterItem::RBuddy).value<PURPLE::Buddy>();
                if(buddy.isValid())
                {
                    if(dxutils::hasLog(dxutils::fromStdString(account.getUsername()).section('/',0,0), dxutils::fromStdString(buddy.getUsername()).section('/',0,0), false))
                    {
                        LogDialog *dialog = new LogDialog(buddy, this);
                        dialog->setModal(true);
                        dialog->exec();
                    }
                    else
                    {
                        QMessageBox::information(this, _("Information"), _("Any logs for buddy %1").arg(dxutils::fromStdString(buddy.getUsername())));
                    }
                }
            }
        }
    }
}

void RosterView::slotChatLogs()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Chat)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                PURPLE::Chat chat = selectedIndexes().at(0).data(RosterItem::RChat).value<PURPLE::Chat>();
                if(chat.isValid())
                {
                    if(dxutils::hasLog(dxutils::fromStdString(account.getUsername()).section('/',0,0), dxutils::fromStdString(chat.getChatName()), false))
                    {
                        LogDialog *dialog = new LogDialog(chat, this);
                        dialog->setModal(true);
                        dialog->exec();
                    }
                    else
                    {
                        QMessageBox::information(this, _("Information"), _("Any logs for chat %1").arg(dxutils::fromStdString(chat.getChatName())));
                    }
                }
            }
        }
    }
}

void RosterView::slotAccountLogs()
{
    if(selectedIndexes().size() > 0)
    {
        if(selectedIndexes().at(0).data(RosterItem::RType).toInt()==RosterItem::Account)
        {
            PURPLE::Account account = selectedIndexes().at(0).data(RosterItem::RAccount).value<PURPLE::Account>();
            if(account.isValid())
            {
                if(dxutils::hasLog(dxutils::fromStdString(account.getUsername()).section('/',0,0), "", true))
                {
                    LogDialog *dialog = new LogDialog(account, this);
                    dialog->setModal(true);
                    dialog->exec();
                }
                else
                {
                    QMessageBox::information(this, _("Information"), _("Any logs for account %1").arg(dxutils::fromStdString(account.getUsername()).section('/',0,0)));
                }
            }
        }
    }
}


