/*
 *      chatitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chatitem.h"
#include <QImage>
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

ChatItem::ChatItem(const QString &name, const QString &realName, PurpleConvChatBuddyFlags role)
{
    setText(name);
    setName(name);
    setRealName(realName);
    setChatRole(role);
}

void ChatItem::setName(const QString &name)
{
    setData(name, ChatItem::Name);
    updateTip();
}

QString ChatItem::getName()
{
    return data(ChatItem::Name).toString();
}

void ChatItem::setRealName(const QString &realName)
{
    setData(realName, ChatItem::RealName);
    updateTip();
}

QString ChatItem::getRealName()
{
    return data(ChatItem::RealName).toString();
}

void ChatItem::setChatRole(PurpleConvChatBuddyFlags role)
{
    setData(static_cast<int>(role), ChatItem::Role);
    setIcon(QIcon(dxutils::getChatRoleIcon(role)));
    updateTip();
}

void ChatItem::updateRoleIconTheme()
{
    setIcon(QIcon(dxutils::getChatRoleIcon(getChatRole())));
}

PurpleConvChatBuddyFlags ChatItem::getChatRole()
{
    return static_cast<PurpleConvChatBuddyFlags>(data(ChatItem::Role).toInt());
}

void ChatItem::updateTip()
{
    QString tip = text()+"\n";
    tip.append(_("ID: %1\n").arg(getRealName()));
    tip.append(_("Role: %1").arg(dxutils::getChatRoleString(getChatRole())));
    setToolTip(tip);
}

