/*
 *      chatitemmodel.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chatitemmodel.h"
#include "dxutils.h"
#include "preferences.h"

ChatItemModel::ChatItemModel(QObject* parent) :
    QStandardItemModel(parent)
{
}

ChatItem* ChatItemModel::getChatItem(const QString &name)
{
    for(int r = 0; r < rowCount(); ++r)
    {
        ChatItem *buddy = static_cast<ChatItem*>(item(r));
        if(buddy->getName() == name)
            return buddy;
    }
    return 0;
}

bool ChatItemModel::hasChatItem(const QString &name)
{
    for(int r = 0; r < rowCount(); ++r)
    {
        ChatItem *buddy = static_cast<ChatItem*>(item(r));
        if(buddy->getName() == name)
            return true;
    }
    return false;
}

void ChatItemModel::addChatItem(const QString &name, const QString &realName, PurpleConvChatBuddyFlags role)
{
    ChatItem *item = new ChatItem(name, realName, role);
    appendRow(item);
    sort(0);
}

void ChatItemModel::removeChatItem(const QString &name)
{
    ChatItem *item = getChatItem(name);
    if(item)
        removeRow(item->row());
}

void ChatItemModel::updateItem(const QString &name, PURPLE::ChatBuddy buddy)
{
    ChatItem *item = getChatItem(name);
    if(item && buddy.isValid())
    {
        item->setText(dxutils::fromStdString(buddy.getName()));
        item->setName(dxutils::fromStdString(buddy.getName()));
        item->setRealName(dxutils::fromStdString(buddy.getRealName()));
        item->setChatRole(buddy.getChatRole());
        sort(0);
    }
}

void ChatItemModel::clear()
{
    QStandardItemModel::clear();
}

void ChatItemModel::updateRoleIconTheme()
{
    for(int r = 0; r < rowCount(); ++r)
    {
        static_cast<ChatItem*>(item(r))->updateRoleIconTheme();
    }
}
