/*
 *      chatitemsortmodel.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chatitemsortmodel.h"
#include "dxutils.h"
#include "preferences.h"

ChatItemSortModel::ChatItemSortModel(QObject *parent) :
    QSortFilterProxyModel(parent)
{
    setDynamicSortFilter(true);
    setFilterRole(Qt::DisplayRole);
    setFilterCaseSensitivity(Qt::CaseInsensitive);
}

bool ChatItemSortModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
{
    int compare = left.data().toString().compare(right.data().toString(), Qt::CaseInsensitive);
    if(compare < 0)
        return true;
    else
        return false;
}

bool ChatItemSortModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    if(filterRegularExpression().isValid())
        return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
#else
    if(!filterRegExp().isEmpty())
        return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
#endif
    return true;
}

QString ChatItemSortModel::getCompletion(QString line)
{
    if(line.isEmpty())
    {
        return index(0,0).data().toString()+DXPREFERENCES.m_nickCompletionChar+" ";
    }
    for(int j = 0; j < rowCount(); ++j)
    {
        if(line.compare(index(j,0).data().toString().left(line.length()), Qt::CaseInsensitive) == 0)
        {
            return index(j,0).data().toString()+DXPREFERENCES.m_nickCompletionChar+" ";
        }
        else if(line.section(DXPREFERENCES.m_nickCompletionChar, 0, 0).compare(index(j,0).data().toString()) == 0)
        {
            if((j+1) < rowCount()) return index(++j,0).data().toString()+DXPREFERENCES.m_nickCompletionChar+" ";
            else return index(0,0).data().toString()+DXPREFERENCES.m_nickCompletionChar+" ";
        }
    }
    return line;
}

QString ChatItemSortModel::getCompletion(QString line, int &cursorPos)
{
    if(line[cursorPos] == ' ') cursorPos = cursorPos-1;
    int pos = line.left(cursorPos).lastIndexOf(' ')+1;
    int n = line.indexOf(' ', cursorPos)>0 ? line.indexOf(' ', cursorPos)-pos : line.length()-pos;
    QString toCompletion = line.mid(pos, n);
    for(int j = 0; j < rowCount(); ++j)
    {
        if(toCompletion.compare(index(j,0).data().toString(), Qt::CaseInsensitive) == 0)
        {
            if((j+1) < rowCount())
            {
                cursorPos = pos+index(j+1,0).data().toString().length();
                return line.replace(pos, n, index(j+1,0).data().toString());
            }
            else
            {
                cursorPos = pos+index(0,0).data().toString().length();
                return line.replace(pos, n, index(0,0).data().toString());
            }
        }
        else if(toCompletion.compare(index(j,0).data().toString().left(toCompletion.length()), Qt::CaseInsensitive) == 0)
        {
            cursorPos = pos+index(j,0).data().toString().length();
            return line.replace(pos, n, index(j,0).data().toString());
        }
    }
    return line;
}
