/*
 *      chattabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QMenu>
#include <QDir>
#include "chattabitem.h"
#include "preferences.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

#define HISTORY_MAX 25

ChatTabItem::ChatTabItem(PURPLE::Conversation *conv, QWidget *parent) :
    QWidget(parent), m_name(dxutils::fromStdString(conv->getTitle())), m_ID(dxutils::fromStdString(conv->getName())), m_conversation(conv), m_nickModel(this),
    m_nickSortModel(this), m_currentPosition(0)
{
    m_process = NULL;
    m_pipeTimer = new QTimer(this);
    m_info = new QLabel;
    m_info->setText(dxutils::fromStdString(m_conversation->getName())+" ("+dxutils::fromStdString(m_conversation->getAccount().getUsername())+")");
    m_icon = new QLabel;
    m_icon->setPixmap(QPixmap(dxutils::protocolToIcon(dxutils::fromStdString(m_conversation->getAccount().getProtocol().getPurpleID()))));
    QHBoxLayout *idLayout = new QHBoxLayout;
    idLayout->addWidget(m_icon);
    idLayout->addWidget(m_info);
    idLayout->addSpacerItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    m_topicline = new QLineEdit;
    m_topicline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_topicline->setFont(DXPREFERENCES.m_imFont);
    m_splitter = new QSplitter(this);
    m_splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_text = new QTextBrowser;
    m_text->setOpenExternalLinks(true);
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_text->setFont(DXPREFERENCES.m_imFont);
    m_splitter->addWidget(m_text);
    m_nickView = new QListView(this);
    m_nickSortModel.setSourceModel(&m_nickModel);
    m_nickView->setModel(&m_nickSortModel);
    m_nickView->setStyleSheet(QString("QListView { \
                    background-color: %1; \
                    color: %2; \
                    } \
                    ").arg(DXPREFERENCES.m_backroundColor.name())
                    .arg(DXPREFERENCES.m_textColor.name())
                    );
    if(DXPREFERENCES.m_sameList) m_nickView->setFont(DXPREFERENCES.m_imFont);
    m_nickView->setContextMenuPolicy(Qt::CustomContextMenu);
    m_nickView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_nickSortModel.sort(0);
    m_splitter->addWidget(m_nickView);
    m_commandline = new QLineEdit;
    m_commandline->installEventFilter(this);
    m_commandline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    if(DXPREFERENCES.m_sameCmd) m_commandline->setFont(DXPREFERENCES.m_imFont);
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addLayout(idLayout);
    mainLayout->addWidget(m_topicline);
    mainLayout->addWidget(m_splitter);
    mainLayout->addWidget(m_commandline);
    setLayout(mainLayout);
    QList<int> list;
    list << width()-120 << 120;
    m_splitter->setSizes(list);

    connect(m_commandline, SIGNAL(returnPressed()), this, SLOT(slotProcessCommandline()));
    connect(m_topicline, SIGNAL(returnPressed()), this, SLOT(slotProcessTopicline()));
    connect(m_nickView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    connect(m_pipeTimer, SIGNAL(timeout()), this, SLOT(slotPipeTimeout()));
}

ChatTabItem::~ChatTabItem()
{}

void ChatTabItem::updatePreferences()
{
    m_topicline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_nickView->setStyleSheet(QString("QListView { \
                    background-color: %1; \
                    color: %2; \
                    } \
                    ").arg(DXPREFERENCES.m_backroundColor.name())
                    .arg(DXPREFERENCES.m_textColor.name())
                    );
    m_commandline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_topicline->setFont(DXPREFERENCES.m_imFont);
    m_text->setFont(DXPREFERENCES.m_imFont);
    if(DXPREFERENCES.m_sameCmd) m_commandline->setFont(DXPREFERENCES.m_imFont);
    else m_commandline->setFont(QFont());
    if(DXPREFERENCES.m_sameList) m_nickView->setFont(DXPREFERENCES.m_imFont);
    else m_nickView->setFont(QFont());
    m_nickModel.updateRoleIconTheme();
    m_logger.stopLogging();
}

void ChatTabItem::focusCommandline()
{
    m_commandline->setFocus();
}

bool ChatTabItem::eventFilter(QObject *obj, QEvent *ev)
{
    if(obj == m_commandline && ev->type() == QEvent::KeyPress)
    {
        QKeyEvent *e = (QKeyEvent *)ev;
        if(e->key() == Qt::Key_Tab)
        {
            processCompletion();
            return true;
        }
        else if(e->key() == Qt::Key_Up)
        {
            QString line = m_commandline->text();
            if(m_currentPosition!=-1 && m_currentPosition<m_history.size())
            {
                if(!line.isEmpty() && line!=m_history[m_currentPosition])
                {
                    m_history.append(line);
                    if(m_history.size() > HISTORY_MAX)
                        m_history.removeFirst();
                    m_currentPosition = m_history.size()-1;
                }
                if(m_currentPosition > 0 && !line.isEmpty())
                    --m_currentPosition;
                m_commandline->setText(m_history[m_currentPosition]);
            }
        }
        else if(e->key() == Qt::Key_Down)
        {
            QString line = m_commandline->text();
            if(m_currentPosition!=-1 && m_currentPosition<m_history.size())
            {
                if(!line.isEmpty() && line!=m_history[m_currentPosition])
                {
                    m_history.append(line);
                    if(m_history.size() > HISTORY_MAX)
                        m_history.removeFirst();
                    m_currentPosition = m_history.size()-1;
                }
                if(m_currentPosition < m_history.size()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_history[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
        }
    }
    return QWidget::eventFilter(obj, ev);
}

void ChatTabItem::processCompletion()
{
    QString line = m_commandline->text();
    if(m_commands.size()==0)
    {
        m_commands << "exec" << "help" << "say" << "ignored" << "log";
        std::vector<std::string> cmds = m_conversation->getCommandsList();
        FOREACH(std::vector<std::string>, cmds, it)
            m_commands.append(dxutils::fromStdString((*it)));
        if(!m_commands.contains("me"))
            m_commands.append("me");
        m_commands.sort();
    }
    if(line.section(' ',0,0).compare("/help", Qt::CaseInsensitive) == 0 && !line.section(' ',1).isEmpty())
    {
        completeHelpCommand();
        return;
    }
    if(line[0] == '/' && line.section(' ', 1).isEmpty())
    {
        line = line.section(' ',0,0).section('/',1);
        for(int i = 0; i < m_commands.size(); i++)
        {
            if(line.compare(m_commands[i], Qt::CaseInsensitive) == 0)
            {
                if((i+1) < m_commands.size()) m_commandline->setText("/"+m_commands[++i]+" ");
                else m_commandline->setText("/"+m_commands[0]+" ");
                break;
            }
            else if(line.compare(m_commands[i].left(line.length()), Qt::CaseInsensitive) == 0)
            {
                m_commandline->setText("/"+m_commands[i]+" ");
                break;
            }
        }
        return;
    }
    if(line[0] != '/' && line.section(' ',1).isEmpty())
    {
        m_commandline->setText(m_nickSortModel.getCompletion(line));
        return;
    }
    if(line.contains(' '))
    {
        int curpos = m_commandline->cursorPosition();
        m_commandline->setText(m_nickSortModel.getCompletion(line, curpos));
        m_commandline->setCursorPosition(curpos);
        return;
    }
}

void ChatTabItem::onNewMyMsg(QString msg, QString time, bool action)
{
    if(action) appendStyledText(time, dxutils::fromStdString(m_conversation->getMyNick())+" "+msg, DXPREFERENCES.m_actionColor);
    else appendMyMsgText(time, msg);
}

void ChatTabItem::onNewMsg(QString sender, QString msg, QString time, bool mhighlight, bool history, bool action)
{
    if(history)
        appendHistoryMsg(time, sender, msg);
    else
    {
        if(mhighlight)
        {
            appendStyledText(time, sender+(action?" ":": ")+msg, DXPREFERENCES.m_hilightColor);
            emit highlight();
        }
        else
        {
            if(action) {appendStyledText(time, sender+" "+msg, DXPREFERENCES.m_actionColor);}
            else {appendNickText(time, sender, msg);}
            emit newmsg();
        }
    }
}

void ChatTabItem::slotProcessCommandline()
{
    if(!m_conversation->getAccount().isConnected())
        return;
    QString commandtext = m_commandline->text();
    if(commandtext.isEmpty())
        return;
    m_history.append(commandtext);
    if (m_history.size() > HISTORY_MAX)
        m_history.removeFirst();
    m_currentPosition = m_history.size()-1;
    m_commandline->clear();
    if(commandtext[0]=='/')
    {
        if(!commandtext.section(' ', 0, 0).compare("/exec", Qt::CaseInsensitive))
        {
            QString command = commandtext.section(' ', 1);
            if(command.isEmpty()) showInternalHelp("exec");
            else
            {
                if(!m_process)
                {
                    m_process = new QProcess;
                    connect(m_process, SIGNAL(readyReadStandardOutput()), this, SLOT(slotOnProcess()));
                }
                if(!command.section(' ', 0, 0).compare("-o", Qt::CaseInsensitive))
                {
                    m_sendPipe = true;
                    appendStyledText("", _("Command %1 started").arg(command.section(' ',1)), DXPREFERENCES.m_hilightColor, false);
                    m_process->setProgram(command.section(' ',1,1));
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
                    m_process->setArguments(command.section(' ',2).split(' ', QString::SkipEmptyParts));
#else
                    m_process->setArguments(command.section(' ',2).split(' ', Qt::SkipEmptyParts));
#endif
                    m_process->start();
                }
                else if(!command.section(' ', 0, 0).compare("-c", Qt::CaseInsensitive))
                {
                    m_sendPipe = false;
                    appendStyledText("", _("Command %1 stopped").arg(m_process->program()), DXPREFERENCES.m_hilightColor, false);
                    m_pipeStrings.clear();
                    m_process->close();
                }
                else
                {
                    m_sendPipe = false;
                    appendStyledText("", _("Command %1 started").arg(command), DXPREFERENCES.m_hilightColor, false);
                    m_process->setProgram(command.section(' ',0,0));
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
                    m_process->setArguments(command.section(' ',1).split(' ', QString::SkipEmptyParts));
#else
                    m_process->setArguments(command.section(' ',1).split(' ', Qt::SkipEmptyParts));
#endif
                    m_process->start();
                }
            }
        }
        else if(!commandtext.section(' ', 0, 0).compare("/help", Qt::CaseInsensitive))
        {
            QString command = commandtext.section(' ', 1);
            if(command.isEmpty())
            {
                QString commands = "";
                for(int i = 0; i < m_commands.size(); i++)
                {
                    if(i+1 == m_commands.size()) commands.append(m_commands[i] + ".");
                    else commands.append(m_commands[i] + ", ");
                }
                appendStyledText("", _("Available commands: %1").arg(commands), DXPREFERENCES.m_hilightColor, false);
                return;
            }
            if(command.compare("help", Qt::CaseInsensitive)==0 || command.compare("me", Qt::CaseInsensitive)==0 || command.compare("say", Qt::CaseInsensitive)==0
               || command.compare("ignored", Qt::CaseInsensitive)==0 || command.compare("log", Qt::CaseInsensitive)==0)
            {
                showInternalHelp(command);
                return;
            }
            QString response = dxutils::fromStdString(m_conversation->getCommandHelp(dxutils::toStdString(command)));
            if(response.isEmpty())
                appendStyledText("", _("Command '%1' doesn't exist or hasn't help").arg(command), DXPREFERENCES.m_errorColor, false);
            else
                appendStyledText("", _("Help for: %1").arg(response), DXPREFERENCES.m_hilightColor, false);
        }
        else if(!commandtext.section(' ', 0, 0).compare("/me", Qt::CaseInsensitive))
        {
            if(!commandtext.section(' ', 1).isEmpty()) m_conversation->sendActionMessage(dxutils::toStdString(commandtext.section(' ', 1).replace('<',"&lt;").replace('>',"&gt;")));
        }
        else if(!commandtext.section(' ', 0, 0).compare("/say", Qt::CaseInsensitive))
        {
            if(!commandtext.section(' ', 1).isEmpty()) m_conversation->sendMessage(dxutils::toStdString(commandtext.section(' ', 1).replace('<',"&lt;").replace('>',"&gt;")));
        }
        else if(!commandtext.section(' ', 0, 0).compare("/ignored", Qt::CaseInsensitive))
        {
            QString ignored = "";
            std::vector<std::string> v = m_conversation->getIgnored();
            for(size_t i=0; i<v.size(); i++)
            {
                if(i+1!=v.size())
                    ignored.append(dxutils::fromStdString(v[i])+", ");
                else
                    ignored.append(dxutils::fromStdString(v[i])+".");
            }
            if(v.size())
                appendStyledText("", _("Ignored buddies: %1").arg(ignored), DXPREFERENCES.m_hilightColor, false);
            else
                appendStyledText("", _("Nobody ignored"), DXPREFERENCES.m_hilightColor, false);
        }
        else if(!commandtext.section(' ', 0, 0).compare("/log", Qt::CaseInsensitive))
        {
            QString command = commandtext.section(' ', 1, 1);
            QString logfile = commandtext.section(' ', 2, 2);
            if(!command.compare("list", Qt::CaseInsensitive))
            {
                listLogFiles();
            }
            else if(!command.compare("show", Qt::CaseInsensitive))
            {
                if(logfile.isEmpty()) logfile = QDateTime::currentDateTime().toString("yyyy-MM-dd");
                QString path = DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0);
                if(!QFile().exists(path+"/"+logfile))
                {
                    appendStyledText("", _("Log file %1 doesn't exist").arg(logfile), DXPREFERENCES.m_errorColor, false);
                    return;
                }
                if(dxutils::isRightLogFile(path+"/"+logfile, logfile))
                {
                    emit showLog(m_conversation->getAccount(), dxutils::fromStdString(m_conversation->getName()), path+"/"+logfile);
                }
            }
            else
            {
                appendStyledText("", _("'%1' isn't <list|show>").arg(command), DXPREFERENCES.m_errorColor, false);
            }
        }
        else
        {
            int ret = m_conversation->sendCommand(dxutils::toStdString(commandtext.section('/', 1)));
            processCommandReturn(ret, commandtext.section('/', 1).section(' ', 0, 0));
        }
    }
    else
    {
        m_conversation->sendMessage(dxutils::toStdString(commandtext.replace('<',"&lt;").replace('>',"&gt;")));
    }
}

//process return m_conversation->sendCommand
void ChatTabItem::processCommandReturn(int ret, const QString &command)
{
    switch(ret) {
        case PURPLE_CMD_STATUS_OK:
            break;
        case PURPLE_CMD_STATUS_NOT_FOUND:
            appendStyledText("", _("Unknown command '%1'.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_WRONG_ARGS:
            appendStyledText("", _("Syntax Error:  You typed the wrong number of arguments "
                                "to that command '%1'.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_FAILED:
            appendStyledText("", _("Your command '%1' failed for an unknown reason.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_WRONG_TYPE:
            if(m_conversation->getType() == PURPLE_CONV_TYPE_IM)
                appendStyledText("", _("That command '%1' only works in chats, not IMs.").arg(command), DXPREFERENCES.m_errorColor, false);
            else
                appendStyledText("", _("That command '%1' only works in IMs, not chats.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_WRONG_PRPL:
            appendStyledText("", _("That command '%1' doesn't work on this protocol.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
    }
}

void ChatTabItem::showInternalHelp(const QString &command)
{
    if(command.compare("exec", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: exec [-o|-c] <command>, executes command, -o sends output to chat, -c closes running command"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("help", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: help [command]: lists available commands or show help for command"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("me", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: me <text>: sends text in 3rd person"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("say", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: say <text>: sends text"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("ignored", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: ignored: show list ignored buddies"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("log", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: log"), DXPREFERENCES.m_hilightColor, false);
        appendStyledText("", _("LOG list, shows list of log files for current im/chat"), DXPREFERENCES.m_hilightColor, false);
        appendStyledText("", _("LOG show [date], shows current log file for current im/chat, if date is empty, else file for given date"), DXPREFERENCES.m_hilightColor, false);
        appendStyledText("", _("date example: 2011-01-31"), DXPREFERENCES.m_hilightColor, false);
    }
}

void ChatTabItem::slotProcessTopicline()
{
    m_conversation->setTopic(dxutils::toStdString(m_topicline->text()));
}

void ChatTabItem::onChatBuddies(QVector<PURPLE::ChatBuddy> buddies)
{
    QString name;
    for(int i=0; i<buddies.size(); i++)
    {
        name = dxutils::fromStdString(buddies[i].getName());
        if(!m_nickModel.hasChatItem(name))
            m_nickModel.addChatItem(name, dxutils::fromStdString(buddies[i].getRealName()), buddies[i].getChatRole());
    }
}

void ChatTabItem::onChatBuddyJoined(PURPLE::ChatBuddy buddy)
{
    QString name = dxutils::fromStdString(buddy.getName());
    if(!m_nickModel.hasChatItem(name))
        m_nickModel.addChatItem(name, dxutils::fromStdString(buddy.getRealName()), buddy.getChatRole());
}

void ChatTabItem::onChatBuddyUpdated(PURPLE::ChatBuddy buddy)
{
    m_nickModel.updateItem(dxutils::fromStdString(buddy.getName()), buddy);
}

void ChatTabItem::onChatBuddyLeft(QString buddy)
{
    m_nickModel.removeChatItem(buddy);
}

void ChatTabItem::onChatBuddyRenamed(QString who, PURPLE::ChatBuddy nbuddy)
{
    m_nickModel.updateItem(who, nbuddy);
}

void ChatTabItem::onChatTopicChanged(QString topic)
{
    m_topicline->setText(topic);
    m_topicline->setToolTip(dxutils::wrapText(topic, 400));
    m_topicline->setCursorPosition(0);
}

void ChatTabItem::onNewConvSystemMessage(QString msg, QString time)
{
    appendStyledText(time, msg, DXPREFERENCES.m_userColor);
}

void ChatTabItem::onNewConvErrorMessage(QString msg, QString time)
{
    appendStyledText(time, msg, DXPREFERENCES.m_errorColor);
}

void ChatTabItem::appendStyledText(QString time, QString msg, QColor clr, bool log)
{
    if(time.isEmpty()) time = QTime::currentTime().toString("hh:mm:ss");
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    m_text->append(time+QString("<font color=\"%1\">").arg(clr.name())+dxutils::parseHtml(msg)+"</font>");
    if(log && startLogging())
    {
        m_logger.logStyledLine(dxutils::toStdString(time), dxutils::styleToString(clr), dxutils::toStdString(msg));
    }
}

void ChatTabItem::appendNickText(QString time, QString nick, QString msg, bool log)
{
    if(time.isEmpty()) time = QTime::currentTime().toString("hh:mm:ss");
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    if(DXPREFERENCES.m_coloredNick) m_text->append(time+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+": </font>"+dxutils::parseHtml(msg));
    else m_text->append(time+"<b>"+nick+": </b>"+dxutils::parseHtml(msg));
    if(log && startLogging())
    {
        m_logger.logNickLine(dxutils::toStdString(time), dxutils::toStdString(nick), dxutils::toStdString(msg));
    }
}

void ChatTabItem::appendMyMsgText(QString time, QString msg)
{
    if(time.isEmpty()) time = QTime::currentTime().toString("hh:mm:ss");
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    if(DXPREFERENCES.m_coloredNick) m_text->append(time+QString("<font color=\"%1\">").arg(dxutils::getNickColor(dxutils::fromStdString(m_conversation->getMyNick())))+dxutils::fromStdString(m_conversation->getMyNick())+DXPREFERENCES.m_nickCompletionChar+" </font>"+QString("<font color=\"%1\">").arg(DXPREFERENCES.m_mymsgColor.name())+dxutils::parseHtml(msg)+"</font>");
    else m_text->append(time+"<b>"+dxutils::fromStdString(m_conversation->getMyNick())+DXPREFERENCES.m_nickCompletionChar+" </b>"+QString("<font color=\"%1\">").arg(DXPREFERENCES.m_mymsgColor.name())+dxutils::parseHtml(msg)+"</font>");
    if(startLogging()) m_logger.logMyMsgLine(dxutils::toStdString(time), m_conversation->getMyNick(), dxutils::toStdString(msg));
}

void ChatTabItem::appendHistoryMsg(QString time, QString nick, QString msg)
{
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    if(DXPREFERENCES.m_coloredNick) m_text->append(time+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+DXPREFERENCES.m_nickCompletionChar+" </font>"+QString("<font color=\"%1\">").arg(DXPREFERENCES.m_logColor.name())+dxutils::parseHtml(msg)+"</font>");
    else m_text->append(time+"<b>"+nick+DXPREFERENCES.m_nickCompletionChar+" </b>"+QString("<font color=\"%1\">").arg(DXPREFERENCES.m_logColor.name())+dxutils::parseHtml(msg)+"</font>");
}

void ChatTabItem::clearText()
{
    m_text->clear();
}

void ChatTabItem::showHideRoster()
{
    if(DXPREFERENCES.m_chatRosterShown)
        m_nickView->show();
    else
        m_nickView->hide();
}

bool ChatTabItem::startLogging()
{
    if(!DXPREFERENCES.m_logging) return false;
    if(m_logger.isReady()) return true;
    m_logger.setPath(dxutils::toStdString(DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0)));
    return true;
}

void ChatTabItem::completeHelpCommand()
{
    QString line = m_commandline->text();
    for(int i = 0; i < m_commands.size(); i++)
    {
        if(line.section(' ',1).compare(m_commands[i], Qt::CaseInsensitive) == 0)
        {
            if((i+1) < m_commands.size()) m_commandline->setText("/help "+m_commands[++i]);
            else m_commandline->setText("/help "+m_commands[0]);
            break;
        }
        else if(line.section(' ',1).compare(m_commands[i].left(line.section(' ',1).length()), Qt::CaseInsensitive) == 0)
        {
            m_commandline->setText("/help "+m_commands[i]);
            break;
        }
    }
}

void ChatTabItem::slotPopup(const QPoint &pos)
{
    QModelIndex index = m_nickView->indexAt(pos);
    if(index.isValid() && index.data(ChatItem::Name) != dxutils::fromStdString(m_conversation->getMyNick()))
    {
        QPoint globalPos = m_nickView->mapToGlobal(pos);
        QMenu popup;
        QAction *im = new QAction(_("IM"), this);
        connect(im, SIGNAL(triggered()), this, SLOT(slotIM()));
        popup.addAction(im);
        QAction *info = new QAction(_("Info"), this);
        connect(info, SIGNAL(triggered()), this, SLOT(slotInfo()));
        popup.addAction(info);
        popup.addSeparator();
        PURPLE::ChatBuddy buddy = m_conversation->findChatBuddy(dxutils::toStdString(index.data(ChatItem::Name).toString()));
        if(buddy.isValid())
        {
            QAction *ignore = new QAction(m_conversation->isChatBuddyIgnored(buddy.getName())?_("Unignore"):_("Ignore"), this);
            connect(ignore, SIGNAL(triggered()), this, SLOT(slotIgnore()));
            popup.addAction(ignore);
        }
        popup.exec(globalPos);
    }
}

void ChatTabItem::slotIM()
{
    QModelIndex index = m_nickView->currentIndex();
    if(index.isValid())
    {
        PURPLE::ChatBuddy buddy = m_conversation->findChatBuddy(dxutils::toStdString(index.data(ChatItem::Name).toString()));
        if(buddy.isValid())
            m_conversation->createIM(buddy);
    }
}

void ChatTabItem::slotInfo()
{
    QModelIndex index = m_nickView->currentIndex();
    if(index.isValid())
    {
        PURPLE::ChatBuddy buddy = m_conversation->findChatBuddy(dxutils::toStdString(index.data(ChatItem::Name).toString()));
        if(buddy.isValid())
            m_conversation->retrieveChatBuddyInfo(buddy.getName());
    }
}

void ChatTabItem::slotIgnore()
{
    QModelIndex index = m_nickView->currentIndex();
    if(index.isValid())
    {
        PURPLE::ChatBuddy buddy = m_conversation->findChatBuddy(dxutils::toStdString(index.data(ChatItem::Name).toString()));
        if(buddy.isValid())
        {
            if(m_conversation->isChatBuddyIgnored(buddy.getName()))
                m_conversation->unignore(buddy.getName());
            else
                m_conversation->ignore(buddy.getName());
        }
    }
}

void ChatTabItem::slotOnProcess()
{
    QByteArray byteArray = m_process->readAllStandardOutput();
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
    QStringList lines = QString(byteArray).split('\n', QString::SkipEmptyParts);
#else
    QStringList lines = QString(byteArray).split('\n', Qt::SkipEmptyParts);
#endif
    foreach(QString line, lines)
    {
        if(m_sendPipe)
        {
            if(!m_pipeTimer->isActive()) m_pipeTimer->start(3000);
            m_pipeStrings.append(line);
        }
        else appendStyledText("", line, DXPREFERENCES.m_mymsgColor, false);
    }
}

void ChatTabItem::slotPipeTimeout()
{
    if(m_pipeStrings.size() > 3)
    {
        if(m_pipeStrings[0].isEmpty()) m_pipeStrings[0] = " ";
        if(m_conversation->isValid())
        {
            m_conversation->sendMessage(dxutils::toStdString(m_pipeStrings[0]));
        }
        m_pipeStrings.pop_front();
    }
    else
    {
        while(m_pipeStrings.size())
        {
            if(m_pipeStrings[0].isEmpty()) m_pipeStrings[0] = " ";
            if(m_conversation->isValid())
            {
                m_conversation->sendMessage(dxutils::toStdString(m_pipeStrings[0]));
            }
            m_pipeStrings.pop_front();
        }
    }
}

void ChatTabItem::listLogFiles()
{
    QDir dir(DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0));
    dir.setFilter(QDir::Files | QDir::Readable);
    dir.setSorting(QDir::Name);
    QStringList files;
    QFileInfoList list = dir.entryInfoList();
    for(int i = 0; i < list.size(); ++i)
    {
        QFileInfo fileInfo = list.at(i);
        if(dxutils::isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
        {
            files << fileInfo.fileName();
        }
    }
    if(files.size())
    {
        appendStyledText("", QString("<b><u>%1</u></b>").arg(_("Log files:")), DXPREFERENCES.m_textColor, false);
        files.sort();
        QString lastYear = files[0].section('-',0,0);
        int lastMonth = files[0].section('-',1,1).toInt();
        appendStyledText("", QString("<b>%1:</b>").arg(lastYear), DXPREFERENCES.m_textColor, false);
        appendStyledText("", QString("<b><i>%1</i>:</b>").arg(dxutils::getMonth(lastMonth)), DXPREFERENCES.m_textColor, false);
        QString list = "";
        for(int i=0; i<files.size(); i++)
        {
            if(lastYear!=files[i].section('-',0,0))
            {
                lastYear = files[i].section('-',0,0);
                lastMonth = files[i].section('-',1,1).toInt();
                appendStyledText("", list.left(list.length()-2), DXPREFERENCES.m_textColor, false);
                appendStyledText("", QString("<b>%1:</b>").arg(lastYear), DXPREFERENCES.m_textColor, false);
                appendStyledText("", QString("<b><i>%1</i>:</b>").arg(dxutils::getMonth(lastMonth)), DXPREFERENCES.m_textColor, false);
                list.clear();
                list.append(files[i].section('-',2,2)+", ");
            }
            else if(lastMonth!=files[i].section('-',1,1).toInt())
            {
                lastMonth = files[i].section('-',1,1).toInt();
                appendStyledText("", list.left(list.length()-2), DXPREFERENCES.m_textColor, false);
                appendStyledText("", QString("<b><i>%1</i>:</b>").arg(dxutils::getMonth(lastMonth)), DXPREFERENCES.m_textColor, false);
                list.clear();
                list.append(files[i].section('-',2,2)+", ");
            }
            else
            {
                list.append(files[i].section('-',2,2)+", ");
            }
        }
        if(!list.isEmpty()) appendStyledText("", list.left(list.length()-2), DXPREFERENCES.m_textColor, false);
    }
    else
    {
        appendStyledText("", _("No log files for %1-%2.").arg(dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)).arg(m_ID.section('/',0,0)), DXPREFERENCES.m_errorColor, false);
    }
}

