/*
 *      chattabitem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CHATTABITEM_H
#define CHATTABITEM_H

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QTextBrowser>
#include <QKeyEvent>
#include <QTime>
#include <QSplitter>
#include <QListView>
#include <QTimer>
#include <QProcess>
#include "defs.h"
#include "chatitemmodel.h"
#include "chatitemsortmodel.h"
#include "callback/callback.h"
#include "../logger/logger.h"

class ChatTabItem : public QWidget
{
    Q_OBJECT
public:
    explicit ChatTabItem(PURPLE::Conversation *conv, QWidget *parent = 0);
    virtual ~ChatTabItem();

    void clearText();
    void showHideRoster();
    PURPLE::Conversation *getConversation() { return m_conversation; }
    void updatePreferences();
    void focusCommandline();
    void appendStyledText(QString time, QString msg, QColor clr, bool log=true);

    void onNewMsg(QString sender, QString msg, QString time, bool highlight, bool history, bool action);
    void onNewMyMsg(QString msg, QString time, bool action);
    void onNewConvSystemMessage(QString msg, QString time);
    void onNewConvErrorMessage(QString msg, QString time);
    void onChatBuddies(QVector<PURPLE::ChatBuddy> buddies);
    void onChatBuddyJoined(PURPLE::ChatBuddy buddy);
    void onChatBuddyUpdated(PURPLE::ChatBuddy buddy);
    void onChatBuddyLeft(QString buddy);
    void onChatBuddyRenamed(QString who, PURPLE::ChatBuddy nbuddy);
    void onChatTopicChanged(QString topic);
signals:
    void highlight();
    void newmsg();
    void showLog(PURPLE::Account account, const QString &name, const QString &path);
private slots:
    void slotPopup(const QPoint &pos);
    void slotIM();
    void slotInfo();
    void slotIgnore();
    void slotProcessCommandline();
    void slotProcessTopicline();
    void slotOnProcess();
    void slotPipeTimeout();
private:
    QString m_name, m_ID;
    PURPLE::Conversation *m_conversation;
    QLabel *m_info;
    QLabel *m_icon;
    QLineEdit *m_commandline, *m_topicline;
    QTextBrowser *m_text;
    Logger m_logger;
    QListView *m_nickView;
    ChatItemModel m_nickModel;
    ChatItemSortModel m_nickSortModel;
    QSplitter *m_splitter;
    QStringList m_commands;
    QList<QString> m_history, m_pipeStrings;
    int m_currentPosition;
    QTimer *m_pipeTimer;
    QProcess *m_process;
    bool m_sendPipe;

    void processCompletion();
    void appendNickText(QString time, QString nick, QString msg, bool log=true);
    void appendMyMsgText(QString time, QString msg);
    void appendHistoryMsg(QString time, QString nick, QString msg);
    bool startLogging();
    void processCommandReturn(int ret, const QString &command);
    void showInternalHelp(const QString &command);
    void completeHelpCommand();
    void listLogFiles();
protected:
    bool eventFilter(QObject *obj, QEvent *ev);
};

#endif // CHATTABITEM_H
