/*
 *      imtabitem.h
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef IMTABITEM_H
#define IMTABITEM_H

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QTextBrowser>
#include <QKeyEvent>
#include <QDateTime>
#include <QTimer>
#include <QProcess>
#include "defs.h"
#include "../logger/logger.h"

class ImTabItem : public QWidget
{
    Q_OBJECT
public:
    explicit ImTabItem(PURPLE::Conversation *conv, QWidget *parent = 0);
    virtual ~ImTabItem();

    void clearText();
    void closeConversation();
    PURPLE::Conversation *getConversation() { return m_conversation; }
    void updatePreferences();
    void focusCommandline();
    void appendStyledText(QString time, QString msg, QColor clr, bool log=true);

    void onNewMsg(QString sender, QString msg, QString time, bool highlight, bool history, bool action);
    void onNewMyMsg(QString msg, QString time, bool action, QString messageID);
    void onMessageRecived(QString messageID);
    void onNewConvSystemMessage(QString msg, QString time);
    void onNewConvErrorMessage(QString msg, QString time);
    void onBuddyTyping(IMStateType typing);
    void onBuddySignedOn(PURPLE::Buddy buddy);
    void onBuddySignedOff(PURPLE::Buddy buddy);
    void onBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive newStatus);
signals:
    void highlight();
    void newmsg();
    void composing(bool);
    void offline(bool);
    void newIMName(const QString &ID);
    void showLog(PURPLE::Account account, const QString &name, const QString &path);
private slots:
    void slotProcessCommandline();
    void slotComposeTimeout();
    void slotOnProcess();
    void slotPipeTimeout();
private:
    QString m_name, m_ID;
    PURPLE::Conversation *m_conversation;
    QLabel *m_info;
    QLabel *m_presence;
    QLabel *m_protocol;
    QLineEdit *m_commandline;
    QTextBrowser *m_text;
    QDateTime m_lastKeyPress;
    IMStateType m_imState;
    QTimer *m_composeTimer, *m_pipeTimer;
    Logger m_logger;
    QStringList m_commands;
    QList<QString> m_history, m_pipeStrings;
    int m_currentPosition;
    QProcess *m_process;
    bool m_sendPipe;
    QStringList m_messageIDs;

    void processCompletion();
    void appendNickText(QString time, QString nick, QString msg, bool log=true);
    void appendMyMsgText(QString time, QString msg);
    bool startLogging();
    void processCommandReturn(int ret, const QString &command);
    void showInternalHelp(const QString &command);
    void completeHelpCommand();
    void loadCurrentLog();
    QColor stringToColor(QString style);
    void appendLogLine(const QString &line);
    void listLogFiles();
protected:
    bool eventFilter(QObject *obj, QEvent *ev);
};

#endif // IMTABITEM_H
